@@ -40,7 +40,6 @@
 #include "nsIDOMNSHTMLTextAreaElement.h"
 #include "nsITextControlElement.h"
 #include "nsIControllers.h"
-#include "nsIEditorController.h"
 #include "nsContentCID.h"
 #include "nsCOMPtr.h"
 #include "nsIComponentManager.h"
@@ -52,15 +51,12 @@
 #include "nsIHTMLContent.h"
 #include "nsGenericHTMLElement.h"
 #include "nsHTMLAtoms.h"
-#include "nsHTMLIIDs.h"
-#include "nsIStyleContext.h"
 #include "nsStyleConsts.h"
 #include "nsIPresContext.h"
-#include "nsIHTMLAttributes.h"
+#include "nsMappedAttributes.h"
 #include "nsIFormControlFrame.h"
-#include "nsIGfxTextControlFrame.h"
+#include "nsITextControlFrame.h"
 #include "nsIEventStateManager.h"
-#include "nsISizeOfHandler.h"
 #include "nsLinebreakConverter.h"
 #include "nsIDocument.h"
 #include "nsIPresShell.h"
@@ -77,26 +73,26 @@
 static NS_DEFINE_CID(kXULControllersCID,  NS_XULCONTROLLERS_CID);
 
 
-class nsHTMLTextAreaElement : public nsGenericHTMLContainerFormElement,
+class nsHTMLTextAreaElement : public nsGenericHTMLFormElement,
                               public nsIDOMHTMLTextAreaElement,
                               public nsIDOMNSHTMLTextAreaElement,
                               public nsITextControlElement
 {
 public:
-  nsHTMLTextAreaElement();
+  nsHTMLTextAreaElement(PRBool aFromParser = PR_FALSE);
   virtual ~nsHTMLTextAreaElement();
 
   // nsISupports
   NS_DECL_ISUPPORTS_INHERITED
 
   // nsIDOMNode
-  NS_FORWARD_NSIDOMNODE_NO_CLONENODE(nsGenericHTMLContainerElement::)
+  NS_FORWARD_NSIDOMNODE_NO_CLONENODE(nsGenericHTMLFormElement::)
 
   // nsIDOMElement
-  NS_FORWARD_NSIDOMELEMENT(nsGenericHTMLContainerElement::)
+  NS_FORWARD_NSIDOMELEMENT(nsGenericHTMLFormElement::)
 
   // nsIDOMHTMLElement
-  NS_FORWARD_NSIDOMHTMLELEMENT(nsGenericHTMLContainerElement::)
+  NS_FORWARD_NSIDOMHTMLELEMENT(nsGenericHTMLFormElement::)
 
   // nsIDOMHTMLTextAreaElement
   NS_DECL_NSIDOMHTMLTEXTAREAELEMENT
@@ -105,55 +101,80 @@ public:
   NS_DECL_NSIDOMNSHTMLTEXTAREAELEMENT
 
   // nsIFormControl
-  NS_IMETHOD GetType(PRInt32* aType);
+  NS_IMETHOD_(PRInt32) GetType() { return NS_FORM_TEXTAREA; }
   NS_IMETHOD Reset();
   NS_IMETHOD SubmitNamesValues(nsIFormSubmission* aFormSubmission,
                                nsIContent* aSubmitElement);
-  NS_IMETHOD SaveState(nsIPresContext* aPresContext, nsIPresState** aState);
-  NS_IMETHOD RestoreState(nsIPresContext* aPresContext, nsIPresState* aState);
+  NS_IMETHOD SaveState();
+  NS_IMETHOD RestoreState(nsIPresState* aState);
 
-  // nsITextControlElement
-  NS_IMETHOD SetValueGuaranteed(const nsAString& aValue, nsIGfxTextControlFrame2* aFrame);
+  // nsITextControlElemet
+  NS_IMETHOD TakeTextFrameValue(const nsAString& aValue);
   NS_IMETHOD SetValueChanged(PRBool aValueChanged);
 
   // nsIContent
-  NS_IMETHOD InsertChildAt(nsIContent* aKid, PRInt32 aIndex, PRBool aNotify,
-                           PRBool aDeepSetDocument);
-  NS_IMETHOD ReplaceChildAt(nsIContent* aKid, PRInt32 aIndex, PRBool aNotify,
-                            PRBool aDeepSetDocument);
-  NS_IMETHOD AppendChildTo(nsIContent* aKid, PRBool aNotify,
-                           PRBool aDeepSetDocument);
-  NS_IMETHOD RemoveChildAt(PRInt32 aIndex, PRBool aNotify);
-  NS_IMETHOD StringToAttribute(nsIAtom* aAttribute,
-                               const nsAReadableString& aValue,
-                               nsHTMLValue& aResult);
+  virtual nsresult InsertChildAt(nsIContent* aKid, PRUint32 aIndex,
+                                 PRBool aNotify, PRBool aDeepSetDocument);
+  virtual nsresult ReplaceChildAt(nsIContent* aKid, PRUint32 aIndex,
+                                  PRBool aNotify, PRBool aDeepSetDocument);
+  virtual nsresult AppendChildTo(nsIContent* aKid, PRBool aNotify,
+                                 PRBool aDeepSetDocument);
+  virtual nsresult RemoveChildAt(PRUint32 aIndex, PRBool aNotify);
+  virtual PRBool ParseAttribute(nsIAtom* aAttribute,
+                                const nsAString& aValue,
+                                nsAttrValue& aResult);
   NS_IMETHOD GetAttributeMappingFunction(nsMapRuleToAttributesFunc& aMapRuleFunc) const;
-  NS_IMETHOD GetMappedAttributeImpact(const nsIAtom* aAttribute, PRInt32 aModType,
-                                      PRInt32& aHint) const;
-  NS_IMETHOD HandleDOMEvent(nsIPresContext* aPresContext, nsEvent* aEvent,
-                            nsIDOMEvent** aDOMEvent, PRUint32 aFlags,
-                            nsEventStatus* aEventStatus);
-  NS_IMETHOD SetFocus(nsIPresContext* aPresContext);
-  NS_IMETHOD RemoveFocus(nsIPresContext* aPresContext);
-#ifdef DEBUG
-  NS_IMETHOD SizeOf(nsISizeOfHandler* aSizer, PRUint32* aResult) const;
-#endif
+  NS_IMETHOD GetAttributeChangeHint(const nsIAtom* aAttribute,
+                                    PRInt32 aModType,
+                                    nsChangeHint& aHint) const;
+  NS_IMETHOD_(PRBool) IsAttributeMapped(const nsIAtom* aAttribute) const;
+  virtual nsresult HandleDOMEvent(nsIPresContext* aPresContext,
+                                  nsEvent* aEvent, nsIDOMEvent** aDOMEvent,
+                                  PRUint32 aFlags,
+                                  nsEventStatus* aEventStatus);
+  virtual void SetFocus(nsIPresContext* aPresContext);
+  virtual void RemoveFocus(nsIPresContext* aPresContext);
+
+  virtual nsresult GetInnerHTML(nsAString& aInnerHTML);
+  virtual nsresult SetInnerHTML(const nsAString& aInnerHTML);
+
+  virtual void DoneAddingChildren();
+  virtual PRBool IsDoneAddingChildren();
 
 protected:
   nsCOMPtr<nsIControllers> mControllers;
+  /** The current value.  This is null if the frame owns the value. */
   char*                    mValue;
+  /** Whether or not the value has changed since its default value was given. */
   PRPackedBool             mValueChanged;
+  /** Whether or not we are already handling select event. */
+  PRPackedBool             mHandlingSelect;
+  /** Whether or not we are done adding children (always PR_TRUE if not
+      created by a parser */
+  PRPackedBool             mDoneAddingChildren;
 
   NS_IMETHOD SelectAll(nsIPresContext* aPresContext);
+  /**
+   * Get the value, whether it is from the content or the frame.
+   * @param aValue the value [out]
+   * @param aIgnoreWrap whether to ignore the wrap attribute when getting the
+   *        value.  If this is true, linebreaks will not be inserted even if
+   *        wrap=hard.
+   */
+  void GetValueInternal(nsAString& aValue, PRBool aIgnoreWrap);
+
+  nsresult SetValueInternal(const nsAString& aValue,
+                            nsITextControlFrame* aFrame);
+  nsresult GetSelectionRange(PRInt32* aSelectionStart, PRInt32* aSelectionEnd);
 };
 
 nsresult
 NS_NewHTMLTextAreaElement(nsIHTMLContent** aInstancePtrResult,
-                          nsINodeInfo *aNodeInfo)
+                          nsINodeInfo *aNodeInfo, PRBool aFromParser)
 {
   NS_ENSURE_ARG_POINTER(aInstancePtrResult);
 
-  nsHTMLTextAreaElement* it = new nsHTMLTextAreaElement();
+  nsHTMLTextAreaElement* it = new nsHTMLTextAreaElement(aFromParser);
 
   if (!it) {
     return NS_ERROR_OUT_OF_MEMORY;
@@ -174,16 +195,19 @@ NS_NewHTMLTextAreaElement(nsIHTMLContent** aInstancePtrResult,
 }
 
 
-nsHTMLTextAreaElement::nsHTMLTextAreaElement()
+nsHTMLTextAreaElement::nsHTMLTextAreaElement(PRBool aFromParser)
+  : mValue(nsnull),
+    mValueChanged(PR_FALSE),
+    mHandlingSelect(PR_FALSE),
+    mDoneAddingChildren(!aFromParser)
 {
-  mValue = 0;
-  mValueChanged = PR_FALSE;
 }
 
 nsHTMLTextAreaElement::~nsHTMLTextAreaElement()
 {
-  // Null out form's pointer to us - no ref counting here!
-  SetForm(nsnull);
+  if (mValue) {
+    nsMemory::Free(mValue);
+  }
 }
 
 
@@ -193,7 +217,7 @@ NS_IMPL_RELEASE_INHERITED(nsHTMLTextAreaElement, nsGenericElement)
 
 // QueryInterface implementation for nsHTMLTextAreaElement
 NS_HTML_CONTENT_INTERFACE_MAP_BEGIN(nsHTMLTextAreaElement,
-                                    nsGenericHTMLContainerFormElement)
+                                    nsGenericHTMLFormElement)
   NS_INTERFACE_MAP_ENTRY(nsIDOMHTMLTextAreaElement)
   NS_INTERFACE_MAP_ENTRY(nsIDOMNSHTMLTextAreaElement)
   NS_INTERFACE_MAP_ENTRY(nsITextControlElement)
@@ -222,7 +246,7 @@ nsHTMLTextAreaElement::CloneNode(PRBool aDeep, nsIDOMNode** aReturn)
   if (NS_FAILED(rv))
     return rv;
 
-  CopyInnerTo(this, it, aDeep);
+  CopyInnerTo(it, aDeep);
 
   *aReturn = NS_STATIC_CAST(nsIDOMNode *, it);
 
@@ -234,7 +258,7 @@ nsHTMLTextAreaElement::CloneNode(PRBool aDeep, nsIDOMNode** aReturn)
 NS_IMETHODIMP
 nsHTMLTextAreaElement::GetForm(nsIDOMHTMLFormElement** aForm)
 {
-  return nsGenericHTMLContainerFormElement::GetForm(aForm);
+  return nsGenericHTMLFormElement::GetForm(aForm);
 }
 
 
@@ -243,33 +267,38 @@ nsHTMLTextAreaElement::GetForm(nsIDOMHTMLFormElement** aForm)
 NS_IMETHODIMP
 nsHTMLTextAreaElement::Blur()
 {
-  return SetElementFocus(PR_FALSE);
+  SetElementFocus(PR_FALSE);
+
+  return NS_OK;
 }
 
 NS_IMETHODIMP
 nsHTMLTextAreaElement::Focus() 
 {
-  return SetElementFocus(PR_TRUE);
+  if (ShouldFocus(this)) {
+    SetElementFocus(PR_TRUE);
+  }
+
+  return NS_OK;
 }
 
-NS_IMETHODIMP
+void
 nsHTMLTextAreaElement::SetFocus(nsIPresContext* aPresContext)
 {
-  NS_ENSURE_ARG_POINTER(aPresContext);
+  if (!aPresContext)
+    return;
+
   // first see if we are disabled or not. If disabled then do nothing.
   nsAutoString disabled;
 
   if (NS_CONTENT_ATTR_HAS_VALUE ==
-      nsGenericHTMLContainerFormElement::GetAttr(kNameSpaceID_HTML,
-                                                 nsHTMLAtoms::disabled,
-                                                 disabled)) {
-    return NS_OK;
+      nsGenericHTMLFormElement::GetAttr(kNameSpaceID_None,
+                                        nsHTMLAtoms::disabled, disabled)) {
+    return;
   }
 
-  nsCOMPtr<nsIEventStateManager> esm;
-  if (NS_OK == aPresContext->GetEventStateManager(getter_AddRefs(esm))) {
-    esm->SetContentState(this, NS_EVENT_STATE_FOCUS);
-  }
+  aPresContext->EventStateManager()->SetContentState(this,
+                                                     NS_EVENT_STATE_FOCUS);
 
   nsIFormControlFrame* formControlFrame = GetFormControlFrame(PR_TRUE);
 
@@ -279,17 +308,16 @@ nsHTMLTextAreaElement::SetFocus(nsIPresContext* aPresContext)
     // Could call SelectAll(aPresContext) here to automatically
     // select text when we receive focus.
   }
-
-  return NS_OK;
 }
 
-NS_IMETHODIMP
+void
 nsHTMLTextAreaElement::RemoveFocus(nsIPresContext* aPresContext)
 {
-  NS_ENSURE_ARG_POINTER(aPresContext);
+  if (!aPresContext)
+    return;
+
   // If we are disabled, we probably shouldn't have focus in the
   // first place, so allow it to be removed.
-  nsresult rv = NS_OK;
 
   nsIFormControlFrame* formControlFrame = GetFormControlFrame(PR_FALSE);
 
@@ -297,24 +325,10 @@ nsHTMLTextAreaElement::RemoveFocus(nsIPresContext* aPresContext)
     formControlFrame->SetFocus(PR_FALSE, PR_FALSE);
   }
 
-  nsCOMPtr<nsIEventStateManager> esm;
-
-  aPresContext->GetEventStateManager(getter_AddRefs(esm));
-
-  if (esm) {
-    nsCOMPtr<nsIDocument> doc;
-
-    GetDocument(*getter_AddRefs(doc));
-    if (!doc)
-      return NS_ERROR_NULL_POINTER;
-
-    nsCOMPtr<nsIContent> rootContent;
-    doc->GetRootContent(getter_AddRefs(rootContent));
-
-    rv = esm->SetContentState(rootContent, NS_EVENT_STATE_FOCUS);
+  if (mDocument) {
+    aPresContext->EventStateManager()->SetContentState(nsnull,
+                                                       NS_EVENT_STATE_FOCUS);
   }
-
-  return rv;
 }
 
 NS_IMETHODIMP
@@ -325,9 +339,8 @@ nsHTMLTextAreaElement::Select()
   // first see if we are disabled or not. If disabled then do nothing.
   nsAutoString disabled;
   if (NS_CONTENT_ATTR_HAS_VALUE ==
-      nsGenericHTMLContainerFormElement::GetAttr(kNameSpaceID_HTML,
-                                                 nsHTMLAtoms::disabled,
-                                                 disabled)) {
+      nsGenericHTMLFormElement::GetAttr(kNameSpaceID_None,
+                                        nsHTMLAtoms::disabled, disabled)) {
     return rv;
   }
 
@@ -339,28 +352,25 @@ nsHTMLTextAreaElement::Select()
   GetPresContext(this, getter_AddRefs(presContext)); 
 
   nsEventStatus status = nsEventStatus_eIgnore;
-  nsGUIEvent event;
-  event.eventStructType = NS_GUI_EVENT;
-  event.message = NS_FORM_SELECTED;
-  event.flags = NS_EVENT_FLAG_NONE;
-  event.widget = nsnull;
+  nsGUIEvent event(NS_FORM_SELECTED);
   rv = HandleDOMEvent(presContext, &event, nsnull, NS_EVENT_FLAG_INIT, &status);
 
   // If the DOM event was not canceled (e.g. by a JS event handler
   // returning false)
   if (status == nsEventStatus_eIgnore) {
-    nsCOMPtr<nsIEventStateManager> esm;
+    PRBool shouldFocus = ShouldFocus(this);
 
-    presContext->GetEventStateManager(getter_AddRefs(esm));
-
-    if (esm) {
-      esm->SetContentState(this, NS_EVENT_STATE_FOCUS);
+    if (shouldFocus) {
+      presContext->EventStateManager()->SetContentState(this,
+                                                        NS_EVENT_STATE_FOCUS);
     }
 
     nsIFormControlFrame* formControlFrame = GetFormControlFrame(PR_TRUE);
 
     if (formControlFrame) {
-      formControlFrame->SetFocus(PR_TRUE, PR_TRUE);
+      if (shouldFocus) {
+        formControlFrame->SetFocus(PR_TRUE, PR_TRUE);
+      }
 
       // Now Select all the text!
       SelectAll(presContext);
@@ -393,7 +403,7 @@ NS_IMPL_INT_ATTR(nsHTMLTextAreaElement, TabIndex, tabindex)
   
 
 NS_IMETHODIMP 
-nsHTMLTextAreaElement::GetType(nsAWritableString& aType)
+nsHTMLTextAreaElement::GetType(nsAString& aType)
 {
   aType.Assign(NS_LITERAL_STRING("textarea"));
 
@@ -401,41 +411,58 @@ nsHTMLTextAreaElement::GetType(nsAWritableString& aType)
 }
 
 NS_IMETHODIMP 
-nsHTMLTextAreaElement::GetValue(nsAWritableString& aValue)
+nsHTMLTextAreaElement::GetValue(nsAString& aValue)
 {
+  GetValueInternal(aValue, PR_TRUE);
+  return NS_OK;
+}
+
+void
+nsHTMLTextAreaElement::GetValueInternal(nsAString& aValue, PRBool aIgnoreWrap)
+{
+  // Get the frame.
   // No need to flush here, if there is no frame yet for this textarea
   // there won't be a value in it we don't already have even if we
   // force the frame to be created.
-  nsIFormControlFrame* formControlFrame = GetFormControlFrame(PR_FALSE);
-
-  nsIGfxTextControlFrame2* textControlFrame = nsnull;
-  if (formControlFrame) {
-    CallQueryInterface(formControlFrame, &textControlFrame);
+  nsIFrame* primaryFrame = GetPrimaryFrame(PR_FALSE);
+  nsITextControlFrame* textControlFrame = nsnull;
+  if (primaryFrame) {
+    CallQueryInterface(primaryFrame, &textControlFrame);
   }
 
+  // If the frame exists and owns the value, get it from the frame.  Otherwise
+  // get it from content.
   PRBool frameOwnsValue = PR_FALSE;
   if (textControlFrame) {
     textControlFrame->OwnsValue(&frameOwnsValue);
   }
   if (frameOwnsValue) {
-    formControlFrame->GetProperty(nsHTMLAtoms::value, aValue);
-    return NS_OK;
+    textControlFrame->GetValue(aValue, aIgnoreWrap);
   } else {
     if (!mValueChanged || !mValue) {
       GetDefaultValue(aValue);
     } else {
-      aValue = NS_ConvertUTF8toUCS2(mValue);
+      CopyUTF8toUTF16(mValue, aValue);
     }
   }
+}
+
+NS_IMETHODIMP
+nsHTMLTextAreaElement::TakeTextFrameValue(const nsAString& aValue)
+{
+  if (mValue) {
+    nsMemory::Free(mValue);
+  }
+  mValue = ToNewUTF8String(aValue);
+  SetValueChanged(PR_TRUE);
   return NS_OK;
 }
 
-
-NS_IMETHODIMP
-nsHTMLTextAreaElement::SetValueGuaranteed(const nsAString& aValue,
-                                          nsIGfxTextControlFrame2* aFrame)
+nsresult
+nsHTMLTextAreaElement::SetValueInternal(const nsAString& aValue,
+                                        nsITextControlFrame* aFrame)
 {
-  nsIGfxTextControlFrame2* textControlFrame = aFrame;
+  nsITextControlFrame* textControlFrame = aFrame;
   nsIFormControlFrame* formControlFrame = textControlFrame;
   if (!textControlFrame) {
     // No need to flush here, if there is no frame for this yet forcing
@@ -471,9 +498,9 @@ nsHTMLTextAreaElement::SetValueGuaranteed(const nsAString& aValue,
 }
 
 NS_IMETHODIMP 
-nsHTMLTextAreaElement::SetValue(const nsAReadableString& aValue)
+nsHTMLTextAreaElement::SetValue(const nsAString& aValue)
 {
-  return SetValueGuaranteed(aValue, nsnull);
+  return SetValueInternal(aValue, nsnull);
 }
 
 
@@ -489,188 +516,121 @@ nsHTMLTextAreaElement::SetValueChanged(PRBool aValueChanged)
 }
 
 NS_IMETHODIMP
-nsHTMLTextAreaElement::GetDefaultValue(nsAWritableString& aDefaultValue)
+nsHTMLTextAreaElement::GetDefaultValue(nsAString& aDefaultValue)
 {
-  nsresult rv;
-  PRInt32 nChildren, i;
-
-  nsAutoString defVal;
-
-  ChildCount(nChildren);
-  for (i = 0; i < nChildren; i++) {
-    nsCOMPtr<nsIContent> child;
-    nsCOMPtr<nsIDOMText> textNode;
-
-    rv = ChildAt(i, *getter_AddRefs(child));
-    NS_ENSURE_SUCCESS(rv, rv);
-    textNode = do_QueryInterface(child);
-    if(textNode) {
-      nsAutoString tmp;
-      textNode->GetData(tmp);
-      defVal.Append(tmp);
-    }
-  }
-
-  aDefaultValue.Assign(defVal);
-  
-  return NS_OK;
+  return GetContentsAsText(aDefaultValue);
 }  
 
 NS_IMETHODIMP
-nsHTMLTextAreaElement::SetDefaultValue(const nsAReadableString& aDefaultValue)
+nsHTMLTextAreaElement::SetDefaultValue(const nsAString& aDefaultValue)
 {
-  nsresult rv;
-  PRInt32 nChildren, i;
-  PRBool firstChildUsed = PR_FALSE;
-
-  ChildCount(nChildren);
-  // If a child exist we try to reuse it
-  if (nChildren > 0) {
-    nsCOMPtr<nsIContent> child;
-    nsCOMPtr<nsIDOMText> textNode;
-
-    rv = ChildAt(0, *getter_AddRefs(child));
-    NS_ENSURE_SUCCESS(rv, rv);
-    textNode = do_QueryInterface(child);
-    if(textNode) {
-      rv = textNode->SetData(aDefaultValue);
-      NS_ENSURE_SUCCESS(rv, rv);
-      firstChildUsed = PR_TRUE;
-    }
-  }
-  
-  PRInt32 lastChild = firstChildUsed ? 1 : 0;
-  for (i = nChildren-1; i >= lastChild; i--) {
-    RemoveChildAt(i, PR_TRUE);
-  }
-  
-  if (!firstChildUsed) {
-    nsCOMPtr<nsIContent> textContent;
-    rv = NS_NewTextNode(getter_AddRefs(textContent));
-    NS_ENSURE_SUCCESS(rv, rv);
-      
-    nsCOMPtr<nsIDOMText> textNode;
-    textNode = do_QueryInterface(textContent);
-    rv = textNode->SetData(aDefaultValue);
-    NS_ENSURE_SUCCESS(rv, rv);
-    
-    AppendChildTo(textContent, PR_TRUE, PR_TRUE);
-  }
-  
-  return NS_OK;
+  return ReplaceContentsWithText(aDefaultValue, PR_TRUE);
 }
 
-NS_IMETHODIMP
-nsHTMLTextAreaElement::InsertChildAt(nsIContent* aKid, PRInt32 aIndex,
+nsresult
+nsHTMLTextAreaElement::InsertChildAt(nsIContent* aKid, PRUint32 aIndex,
                                      PRBool aNotify, PRBool aDeepSetDocument)
 {
   nsresult rv;
-  rv = nsGenericHTMLContainerFormElement::InsertChildAt(aKid, aIndex, aNotify,
-                                                        aDeepSetDocument);
+  rv = nsGenericHTMLFormElement::InsertChildAt(aKid, aIndex, aNotify,
+                                               aDeepSetDocument);
   if (!mValueChanged) {
     Reset();
   }
   return rv;
 }
 
-NS_IMETHODIMP
-nsHTMLTextAreaElement::ReplaceChildAt(nsIContent* aKid, PRInt32 aIndex,
+nsresult
+nsHTMLTextAreaElement::ReplaceChildAt(nsIContent* aKid, PRUint32 aIndex,
                                       PRBool aNotify, PRBool aDeepSetDocument)
 {
   nsresult rv;
-  rv = nsGenericHTMLContainerFormElement::ReplaceChildAt(aKid, aIndex, aNotify,
-                                                         aDeepSetDocument);
+  rv = nsGenericHTMLFormElement::ReplaceChildAt(aKid, aIndex, aNotify,
+                                                aDeepSetDocument);
   if (!mValueChanged) {
     Reset();
   }
   return rv;
 }
 
-NS_IMETHODIMP
+nsresult
 nsHTMLTextAreaElement::AppendChildTo(nsIContent* aKid, PRBool aNotify,
                                      PRBool aDeepSetDocument)
 {
   nsresult rv;
-  rv = nsGenericHTMLContainerFormElement::AppendChildTo(aKid, aNotify,
-                                                        aDeepSetDocument);
+  rv = nsGenericHTMLFormElement::AppendChildTo(aKid, aNotify,
+                                               aDeepSetDocument);
   if (!mValueChanged) {
     Reset();
   }
   return rv;
 }
 
-NS_IMETHODIMP
-nsHTMLTextAreaElement::RemoveChildAt(PRInt32 aIndex, PRBool aNotify)
+nsresult
+nsHTMLTextAreaElement::RemoveChildAt(PRUint32 aIndex, PRBool aNotify)
 {
   nsresult rv;
-  rv = nsGenericHTMLContainerFormElement::RemoveChildAt(aIndex, aNotify);
+  rv = nsGenericHTMLFormElement::RemoveChildAt(aIndex, aNotify);
   if (!mValueChanged) {
     Reset();
   }
   return rv;
 }
 
-NS_IMETHODIMP
-nsHTMLTextAreaElement::StringToAttribute(nsIAtom* aAttribute,
-                                         const nsAReadableString& aValue,
-                                         nsHTMLValue& aResult)
+PRBool
+nsHTMLTextAreaElement::ParseAttribute(nsIAtom* aAttribute,
+                                      const nsAString& aValue,
+                                      nsAttrValue& aResult)
 {
-  if (aAttribute == nsHTMLAtoms::disabled) {
-    aResult.SetEmptyValue();
-    return NS_CONTENT_ATTR_HAS_VALUE;
+  if (aAttribute == nsHTMLAtoms::cols) {
+    return aResult.ParseIntWithBounds(aValue, 0);
   }
-  else if (aAttribute == nsHTMLAtoms::cols) {
-    if (ParseValue(aValue, 0, aResult, eHTMLUnit_Integer)) {
-      return NS_CONTENT_ATTR_HAS_VALUE;
-    }
+  if (aAttribute == nsHTMLAtoms::rows) {
+    return aResult.ParseIntWithBounds(aValue, 0);
   }
-  else if (aAttribute == nsHTMLAtoms::readonly) {
-    aResult.SetEmptyValue();
-    return NS_CONTENT_ATTR_HAS_VALUE;
-  }
-  else if (aAttribute == nsHTMLAtoms::rows) {
-    if (ParseValue(aValue, 0, aResult, eHTMLUnit_Integer)) {
-      return NS_CONTENT_ATTR_HAS_VALUE;
-    }
-  }
-  else if (aAttribute == nsHTMLAtoms::tabindex) {
-    if (ParseValue(aValue, 0, aResult, eHTMLUnit_Integer)) {
-      return NS_CONTENT_ATTR_HAS_VALUE;
-    }
+  if (aAttribute == nsHTMLAtoms::tabindex) {
+    return aResult.ParseIntWithBounds(aValue, 0, 32767);
   }
 
-  return NS_CONTENT_ATTR_NOT_THERE;
+  return nsGenericHTMLElement::ParseAttribute(aAttribute, aValue, aResult);
 }
 
 static void
-MapAttributesIntoRule(const nsIHTMLMappedAttributes* aAttributes,
+MapAttributesIntoRule(const nsMappedAttributes* aAttributes,
                       nsRuleData* aData)
 {
-  if (!aAttributes || !aData)
-    return;
-
-  nsGenericHTMLElement::MapAlignAttributeInto(aAttributes, aData);
-  nsGenericHTMLElement::MapCommonAttributesInto(aAttributes, aData);
+  nsGenericHTMLFormElement::MapDivAlignAttributeInto(aAttributes, aData);
+  nsGenericHTMLFormElement::MapCommonAttributesInto(aAttributes, aData);
 }
 
 NS_IMETHODIMP
-nsHTMLTextAreaElement::GetMappedAttributeImpact(const nsIAtom* aAttribute, PRInt32 aModType,
-                                                PRInt32& aHint) const
+nsHTMLTextAreaElement::GetAttributeChangeHint(const nsIAtom* aAttribute,
+                                              PRInt32 aModType,
+                                              nsChangeHint& aHint) const
 {
   // XXX Bug 50280 - It is unclear why we need to do this here for 
   // rows and cols and why the AttributeChanged method in
-  // nsGfxTextControlFrame2 does take care of the entire problem, but
+  // nsTextControlFrame does take care of the entire problem, but
   // it doesn't and this makes things better
-  if (aAttribute == nsHTMLAtoms::align ||
-      aAttribute == nsHTMLAtoms::rows ||
+  nsresult rv =
+    nsGenericHTMLFormElement::GetAttributeChangeHint(aAttribute, aModType,
+                                                     aHint);
+  if (aAttribute == nsHTMLAtoms::rows ||
       aAttribute == nsHTMLAtoms::cols) {
-    aHint = NS_STYLE_HINT_REFLOW;
-  }
-  else if (!GetCommonMappedAttributesImpact(aAttribute, aHint)) {
-    aHint = NS_STYLE_HINT_CONTENT;
+    NS_UpdateHint(aHint, NS_STYLE_HINT_REFLOW);
   }
+  return rv;
+}
 
-  return NS_OK;
+NS_IMETHODIMP_(PRBool)
+nsHTMLTextAreaElement::IsAttributeMapped(const nsIAtom* aAttribute) const
+{
+  static const MappedAttributeEntry* const map[] = {
+    sDivAlignAttributeMap,
+    sCommonAttributeMap,
+  };
+
+  return FindAttributeDependence(aAttribute, map, NS_ARRAY_LENGTH(map));
 }
 
 NS_IMETHODIMP
@@ -681,7 +641,7 @@ nsHTMLTextAreaElement::GetAttributeMappingFunction(nsMapRuleToAttributesFunc& aM
 }
 
 
-NS_IMETHODIMP
+nsresult
 nsHTMLTextAreaElement::HandleDOMEvent(nsIPresContext* aPresContext,
                                       nsEvent* aEvent,
                                       nsIDOMEvent** aDOMEvent,
@@ -699,12 +659,9 @@ nsHTMLTextAreaElement::HandleDOMEvent(nsIPresContext* aPresContext,
   nsIFrame* formFrame = nsnull;
 
   if (formControlFrame &&
-      NS_SUCCEEDED(formControlFrame->QueryInterface(NS_GET_IID(nsIFrame),
-                                                    (void **)&formFrame)) &&
+      NS_SUCCEEDED(CallQueryInterface(formControlFrame, &formFrame)) &&
       formFrame) {
-    const nsStyleUserInterface* uiStyle;
-    formFrame->GetStyleData(eStyleStruct_UserInterface,
-                            (const nsStyleStruct *&)uiStyle);
+    const nsStyleUserInterface* uiStyle = formFrame->GetStyleUserInterface();
 
     if (uiStyle->mUserInput == NS_STYLE_USER_INPUT_NONE ||
         uiStyle->mUserInput == NS_STYLE_USER_INPUT_DISABLED) {
@@ -712,46 +669,11 @@ nsHTMLTextAreaElement::HandleDOMEvent(nsIPresContext* aPresContext,
     }
   }
 
-  // We have anonymous content underneath
-  // that we need to hide.  We need to set the event target now
-  // to ourselves
-
-  // If the event is starting here that's fine.  If it's not
-  // init'ing here it started beneath us and needs modification.
-  if (!(NS_EVENT_FLAG_INIT & aFlags)) {
-    if (!*aDOMEvent) {
-      // We haven't made a DOMEvent yet.  Force making one now.
-      nsCOMPtr<nsIEventListenerManager> listenerManager;
-
-      rv = GetListenerManager(getter_AddRefs(listenerManager));
-
-      if (NS_FAILED(rv)) {
-        return rv;
-      }
-
-      nsAutoString empty;
-
-      rv = listenerManager->CreateEvent(aPresContext, aEvent, empty,
-                                        aDOMEvent);
-
-      if (NS_FAILED(rv)) {
-        return rv;
-      }
-
-      if (!*aDOMEvent) {
-        return NS_ERROR_FAILURE;
-      }
-
-      nsCOMPtr<nsIPrivateDOMEvent> privateEvent(do_QueryInterface(*aDOMEvent));
-
-      if (!privateEvent) {
-        return NS_ERROR_FAILURE;
-      }
-
-      nsCOMPtr<nsIDOMEventTarget> target(do_QueryInterface(NS_STATIC_CAST(nsIDOMHTMLTextAreaElement *, this)));
-
-      privateEvent->SetTarget(target);
-    }
+  PRBool isSelectEvent = (aEvent->message == NS_FORM_SELECTED);
+  // Don't dispatch a second select event if we are already handling
+  // one.
+  if (isSelectEvent && mHandlingSelect) {
+    return NS_OK;
   }
 
   // If NS_EVENT_FLAG_NO_CONTENT_DISPATCH is set we will not allow content to handle
@@ -762,57 +684,48 @@ nsHTMLTextAreaElement::HandleDOMEvent(nsIPresContext* aPresContext,
     aEvent->flags &= ~NS_EVENT_FLAG_NO_CONTENT_DISPATCH;
   }
 
-  rv = nsGenericHTMLContainerFormElement::HandleDOMEvent(aPresContext, aEvent,
-                                                         aDOMEvent,
-                                                         aFlags, aEventStatus);
+  if (isSelectEvent) {
+    mHandlingSelect = PR_TRUE;
+  }
+
+  rv = nsGenericHTMLFormElement::HandleDOMEvent(aPresContext, aEvent,
+                                                aDOMEvent, aFlags,
+                                                aEventStatus);
+
+  if (isSelectEvent) {
+    mHandlingSelect = PR_FALSE;
+  }
 
   // Reset the flag for other content besides this text field
   aEvent->flags |= noContentDispatch ? NS_EVENT_FLAG_NO_CONTENT_DISPATCH : NS_EVENT_FLAG_NONE;
 
-  // Finish the special anonymous content processing...
-  // If the event is starting here that's fine.  If it's not
-  // init'ing here it started beneath us and needs modification.
-  if (!(NS_EVENT_FLAG_INIT & aFlags)) {
-    if (!*aDOMEvent) {
-      return NS_ERROR_FAILURE;
-    }
-
-    nsCOMPtr<nsIPrivateDOMEvent> privateEvent = do_QueryInterface(*aDOMEvent);
-    if (!privateEvent) {
-      return NS_ERROR_FAILURE;
-    }
-
-    // This will reset the target to its original value
-    privateEvent->SetTarget(nsnull);
-  }
-
   return rv;
 }
 
-// nsIFormControl
-
-NS_IMETHODIMP
-nsHTMLTextAreaElement::GetType(PRInt32* aType)
+void
+nsHTMLTextAreaElement::DoneAddingChildren()
 {
-  if (aType) {
-    *aType = NS_FORM_TEXTAREA;
-    return NS_OK;
-  } else {
-    return NS_FORM_NOTOK;
-  }
+  mDoneAddingChildren = PR_TRUE;
+  RestoreFormControlState(this, this);
 }
 
-#ifdef DEBUG
-NS_IMETHODIMP
-nsHTMLTextAreaElement::SizeOf(nsISizeOfHandler* aSizer,
-                              PRUint32* aResult) const
+PRBool
+nsHTMLTextAreaElement::IsDoneAddingChildren()
 {
-  *aResult = sizeof(*this) + BaseSizeOf(aSizer);
-
-  return NS_OK;
+  return mDoneAddingChildren;
 }
-#endif
 
+nsresult
+nsHTMLTextAreaElement::GetInnerHTML(nsAString& aInnerHTML)
+{
+  return GetContentsAsText(aInnerHTML);
+}
+
+nsresult
+nsHTMLTextAreaElement::SetInnerHTML(const nsAString& aInnerHTML)
+{
+  return ReplaceContentsWithText(aInnerHTML, PR_TRUE);
+}
 
 // Controllers Methods
 
@@ -823,27 +736,14 @@ nsHTMLTextAreaElement::GetControllers(nsIControllers** aResult)
 
   if (!mControllers)
   {
-    NS_ENSURE_SUCCESS (
-      nsComponentManager::CreateInstance(kXULControllersCID,
-                                         nsnull,
-                                         NS_GET_IID(nsIControllers),
-                                         getter_AddRefs(mControllers)),
-      NS_ERROR_FAILURE);
-    if (!mControllers) { return NS_ERROR_NULL_POINTER; }
-
     nsresult rv;
+    mControllers = do_CreateInstance(kXULControllersCID, &rv);
+    NS_ENSURE_SUCCESS(rv, rv);
+
     nsCOMPtr<nsIController> controller = do_CreateInstance("@mozilla.org/editor/editorcontroller;1", &rv);
     if (NS_FAILED(rv))
       return rv;
 
-    nsCOMPtr<nsIEditorController> editorController = do_QueryInterface(controller, &rv);
-    if (NS_FAILED(rv))
-      return rv;
-
-    rv = editorController->Init(nsnull);
-    if (NS_FAILED(rv))
-      return rv;
-
     mControllers->AppendController(controller);
   }
 
@@ -853,6 +753,103 @@ nsHTMLTextAreaElement::GetControllers(nsIControllers** aResult)
   return NS_OK;
 }
 
+NS_IMETHODIMP
+nsHTMLTextAreaElement::GetTextLength(PRInt32 *aTextLength)
+{
+  NS_ENSURE_ARG_POINTER(aTextLength);
+  nsAutoString val;
+  nsresult rv = GetValue(val);
+  *aTextLength = val.Length();
+
+  return rv;
+}
+
+NS_IMETHODIMP
+nsHTMLTextAreaElement::GetSelectionStart(PRInt32 *aSelectionStart)
+{
+  NS_ENSURE_ARG_POINTER(aSelectionStart);
+  
+  PRInt32 selEnd;
+  return GetSelectionRange(aSelectionStart, &selEnd);
+}
+
+NS_IMETHODIMP
+nsHTMLTextAreaElement::SetSelectionStart(PRInt32 aSelectionStart)
+{
+  nsresult rv = NS_ERROR_FAILURE;
+  nsIFormControlFrame* formControlFrame = GetFormControlFrame(PR_TRUE);
+
+  if (formControlFrame){
+    nsITextControlFrame* textControlFrame = nsnull;
+    CallQueryInterface(formControlFrame, &textControlFrame);
+
+    if (textControlFrame)
+      rv = textControlFrame->SetSelectionStart(aSelectionStart);
+  }
+
+  return rv;
+}
+
+NS_IMETHODIMP
+nsHTMLTextAreaElement::GetSelectionEnd(PRInt32 *aSelectionEnd)
+{
+  NS_ENSURE_ARG_POINTER(aSelectionEnd);
+  
+  PRInt32 selStart;
+  return GetSelectionRange(&selStart, aSelectionEnd);
+}
+
+NS_IMETHODIMP
+nsHTMLTextAreaElement::SetSelectionEnd(PRInt32 aSelectionEnd)
+{
+  nsresult rv = NS_ERROR_FAILURE;
+  nsIFormControlFrame* formControlFrame = GetFormControlFrame(PR_TRUE);
+
+  if (formControlFrame) {
+    nsITextControlFrame* textControlFrame = nsnull;
+    CallQueryInterface(formControlFrame, &textControlFrame);
+
+    if (textControlFrame)
+      rv = textControlFrame->SetSelectionEnd(aSelectionEnd);
+  }
+
+  return rv;
+}
+
+nsresult
+nsHTMLTextAreaElement::GetSelectionRange(PRInt32* aSelectionStart,
+                                      PRInt32* aSelectionEnd)
+{
+  nsresult rv = NS_ERROR_FAILURE;
+  nsIFormControlFrame* formControlFrame = GetFormControlFrame(PR_TRUE);
+
+  if (formControlFrame) {
+    nsITextControlFrame* textControlFrame = nsnull;
+    CallQueryInterface(formControlFrame, &textControlFrame);
+
+    if (textControlFrame)
+      rv = textControlFrame->GetSelectionRange(aSelectionStart, aSelectionEnd);
+  }
+
+  return rv;
+}
+
+NS_IMETHODIMP
+nsHTMLTextAreaElement::SetSelectionRange(PRInt32 aSelectionStart, PRInt32 aSelectionEnd)
+{ 
+  nsresult rv = NS_ERROR_FAILURE;
+  nsIFormControlFrame* formControlFrame = GetFormControlFrame(PR_TRUE);
+
+  if (formControlFrame) {
+    nsITextControlFrame* textControlFrame = nsnull;
+    CallQueryInterface(formControlFrame, &textControlFrame);
+
+    if (textControlFrame)
+      rv = textControlFrame->SetSelectionRange(aSelectionStart, aSelectionEnd);
+  }
+
+  return rv;
+} 
 
 nsresult
 nsHTMLTextAreaElement::Reset()
@@ -899,10 +896,7 @@ nsHTMLTextAreaElement::SubmitNamesValues(nsIFormSubmission* aFormSubmission,
   // Get the value
   //
   nsAutoString value;
-  rv = GetValue(value);
-  if (NS_FAILED(rv)) {
-    return rv;
-  }
+  GetValueInternal(value, PR_FALSE);
 
   //
   // Submit
@@ -914,30 +908,33 @@ nsHTMLTextAreaElement::SubmitNamesValues(nsIFormSubmission* aFormSubmission,
 
 
 NS_IMETHODIMP
-nsHTMLTextAreaElement::SaveState(nsIPresContext* aPresContext,
-                                 nsIPresState** aState)
+nsHTMLTextAreaElement::SaveState()
 {
-  nsresult rv = GetPrimaryPresState(this, aState);
-  if (*aState) {
-    nsString value;
-    GetValue(value);
-    // XXX Should use nsAutoString above but ConvertStringLineBreaks requires
-    // mOwnsBuffer!
-    rv = nsLinebreakConverter::ConvertStringLineBreaks(
-             value,
-             nsLinebreakConverter::eLinebreakPlatform,
-             nsLinebreakConverter::eLinebreakContent);
-    NS_ASSERTION(NS_SUCCEEDED(rv), "Converting linebreaks failed!");
-    rv = (*aState)->SetStateProperty(NS_LITERAL_STRING("value"), value);
-    NS_ASSERTION(NS_SUCCEEDED(rv), "value save failed!");
+  nsresult rv = NS_OK;
+
+  // Only save if value != defaultValue (bug 62713)
+  if (mValueChanged) {
+    nsCOMPtr<nsIPresState> state;
+    rv = GetPrimaryPresState(this, getter_AddRefs(state));
+    if (state) {
+      nsAutoString value;
+      GetValueInternal(value, PR_TRUE);
+
+      rv = nsLinebreakConverter::ConvertStringLineBreaks(
+               value,
+               nsLinebreakConverter::eLinebreakPlatform,
+               nsLinebreakConverter::eLinebreakContent);
+      NS_ASSERTION(NS_SUCCEEDED(rv), "Converting linebreaks failed!");
+      rv = state->SetStateProperty(NS_LITERAL_STRING("value"), value);
+      NS_ASSERTION(NS_SUCCEEDED(rv), "value save failed!");
+    }
   }
 
   return rv;
 }
 
 NS_IMETHODIMP
-nsHTMLTextAreaElement::RestoreState(nsIPresContext* aPresContext,
-                                    nsIPresState* aState)
+nsHTMLTextAreaElement::RestoreState(nsIPresState* aState)
 {
   nsresult rv = NS_OK;
 
