@@ -23,12 +23,11 @@
  *   Simon Montagu <smontagu@netscape.com>
  *   Roozbeh Pournader <roozbeh@sharif.edu>
  */
-#ifdef IBMBIDI
 #include "nsBidiUtils.h"
 
 #define FE_TO_06_OFFSET 0xfe70
 
-static PRUnichar FE_TO_06 [][2] = {
+static const PRUnichar FE_TO_06 [][2] = {
     {0x064b,0x0000},{0x064b,0x0640},{0x064c,0x0000},
     {0x0000,0x0000},{0x064d,0x0000},{0x0000,0x0000},
     {0x064e,0x0000},{0x064e,0x0640},{0x064f,0x0000},
@@ -78,7 +77,7 @@ static PRUnichar FE_TO_06 [][2] = {
     {0x0644,0x0625},{0x0644,0x0627},{0x0644,0x0627}
 };
 
-static PRUnichar FB_TO_06 [] = {
+static const PRUnichar FB_TO_06 [] = {
     0x0671,0x0671,0x067B,0x067B,0x067B,0x067B,0x067E,0x067E, //FB50-FB57
     0x067E,0x067E,0x0680,0x0680,0x0680,0x0680,0x067A,0x067A, //FB58-FB5F
     0x067A,0x067A,0x067F,0x067F,0x067F,0x067F,0x0679,0x0679, //FB60-FB67
@@ -111,19 +110,19 @@ static PRUnichar FB_TO_06 [] = {
 
 //============ Begin Arabic Basic to Presentation Form B Code ============
 // Note: the following code are moved from gfx/src/windows/nsRenderingContextWin.cpp
-static PRUint8 gArabicMap1[] = {
+static const PRUint8 gArabicMap1[] = {
             0x81, 0x83, 0x85, 0x87, 0x89, 0x8D, // 0622-0627
 0x8F, 0x93, 0x95, 0x99, 0x9D, 0xA1, 0xA5, 0xA9, // 0628-062F
 0xAB, 0xAD, 0xAF, 0xB1, 0xB5, 0xB9, 0xBD, 0xC1, // 0630-0637
 0xC5, 0xC9, 0xCD                                // 0638-063A
 };
 
-static PRUint8 gArabicMap2[] = {
+static const PRUint8 gArabicMap2[] = {
       0xD1, 0xD5, 0xD9, 0xDD, 0xE1, 0xE5, 0xE9, // 0641-0647
 0xED, 0xEF, 0xF1                                // 0648-064A
 };
 
-static PRUint8 gArabicMapEx[] = {
+static const PRUint8 gArabicMapEx[] = {
       0x50, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // 0671-0677
 0x00, 0x66, 0x5E, 0x52, 0x00, 0x00, 0x56, 0x62, // 0678-067F
 0x5A, 0x00, 0x00, 0x76, 0x72, 0x00, 0x7A, 0x7E, // 0680-0687
@@ -133,7 +132,7 @@ static PRUint8 gArabicMapEx[] = {
 0x00, 0x00, 0x00, 0x00, 0x6A, 0x00, 0x6E, 0x00, // 06A0-06A7
 0x00, 0x8E, 0x00, 0x00, 0x00, 0xD3, 0x00, 0x92, // 06A8-06AF
 0x00, 0x9A, 0x00, 0x96, 0x00, 0x00, 0x00, 0x00, // 06B0-06B7
-0x00, 0x00, 0x00, 0xA0, 0x00, 0x00, 0xAA, 0x00, // 06B8-06BF
+0x00, 0x00, 0x9E, 0xA0, 0x00, 0x00, 0xAA, 0x00, // 06B8-06BF
 0xA4, 0xA6, 0x00, 0x00, 0x00, 0xE0, 0xD9, 0xD7, // 06C0-06C7
 0xDB, 0xE2, 0x00, 0xDE, 0xFC, 0x00, 0x00, 0x00, // 06C8-06CF
 0xE4, 0x00, 0xAE, 0xB0                          // 06D0-06D3
@@ -146,13 +145,15 @@ static PRUint8 gArabicMapEx[] = {
         (0xFE00|(gArabicMap2[(c)-0x0641] + (form))) :                    \
          (((0x0671<=(c)) && ((c))<=0x06D3) && gArabicMapEx[(c)-0x0671]) ? \
           (0xFB00|(gArabicMapEx[(c)-0x0671] + (form))) : (c)))
-enum {
+
+typedef enum {
    eIsolated,  // or Char N
    eFinal,     // or Char R
    eInitial,   // or Char L
    eMedial     // or Char M
 } eArabicForm;
-enum {
+
+typedef enum {
    eTr = 0, // Transparent
    eRJ = 1, // Right-Joining
    eLJ = 2, // Left-Joining
@@ -180,15 +181,19 @@ enum {
 
 // All letters without an equivalen in the FB50 block are 'eNJ' here. This
 // should be fixed after finding some better mechanism for handling Arabic.
-static PRInt8 gJoiningClass[] = {
-          eNJ, eRJ, eRJ, eRJ, eRJ, eDJ, eRJ, // 0621-0627
+static const PRInt8 gJoiningClass[] = {
+eNJ, eNJ, eNJ, eNJ, eNJ, eNJ, eNJ, eNJ, // 0600-0607
+eNJ, eNJ, eNJ, eNJ, eNJ, eNJ, eNJ, eNJ, // 0608-060F
+eTr, eTr, eTr, eTr, eTr, eTr, eNJ, eNJ, // 0610-0617
+eNJ, eNJ, eNJ, eNJ, eNJ, eNJ, eNJ, eNJ, // 0618-061F
+eNJ, eNJ, eRJ, eRJ, eRJ, eRJ, eDJ, eRJ, // 0620-0627
 eDJ, eRJ, eDJ, eDJ, eDJ, eDJ, eDJ, eRJ, // 0628-062F
 eRJ, eRJ, eRJ, eDJ, eDJ, eDJ, eDJ, eDJ, // 0630-0637
 eDJ, eDJ, eDJ, eNJ, eNJ, eNJ, eNJ, eNJ, // 0638-063F
 eJC, eDJ, eDJ, eDJ, eDJ, eDJ, eDJ, eDJ, // 0640-0647
-eRJ, eRJ, eDJ, eTr, eTr, eTr, eTr, eTr, // 0648-064F
-eTr, eTr, eTr, eTr, eTr, eTr, eNJ, eNJ, // 0650-0657
-eNJ, eNJ, eNJ, eNJ, eNJ, eNJ, eNJ, eNJ, // 0658-065F
+eRJ, eDJ, eDJ, eTr, eTr, eTr, eTr, eTr, // 0648-064F
+eTr, eTr, eTr, eTr, eTr, eTr, eTr, eTr, // 0650-0657
+eTr, eNJ, eNJ, eNJ, eNJ, eNJ, eNJ, eNJ, // 0658-065F
 eNJ, eNJ, eNJ, eNJ, eNJ, eNJ, eNJ, eNJ, // 0660-0667
 eNJ, eNJ, eNJ, eNJ, eNJ, eNJ, eNJ, eNJ, // 0668-066F
 eTr, eRJ, eNJ, eNJ, eNJ, eNJ, eNJ, eNJ, // 0670-0677
@@ -200,11 +205,11 @@ eRJ, eNJ, eNJ, eNJ, eNJ, eNJ, eNJ, eNJ, // 0698-069F
 eNJ, eNJ, eNJ, eNJ, eDJ, eNJ, eDJ, eNJ, // 06A0-06A7
 eNJ, eDJ, eNJ, eNJ, eNJ, eDJ, eNJ, eDJ, // 06A8-06AF
 eNJ, eDJ, eNJ, eDJ, eNJ, eNJ, eNJ, eNJ, // 06B0-06B7
-eNJ, eNJ, eNJ, eDJ, eNJ, eNJ, eDJ, eNJ, // 06B8-06BF
+eNJ, eNJ, eDJ, eDJ, eNJ, eNJ, eDJ, eNJ, // 06B8-06BF
 eRJ, eDJ, eNJ, eNJ, eNJ, eRJ, eRJ, eRJ, // 06C0-06C7
 eRJ, eRJ, eNJ, eRJ, eDJ, eNJ, eNJ, eNJ, // 06C8-06CF
 eDJ, eNJ, eRJ, eRJ, eNJ, eNJ, eTr, eTr, // 06D0-06D7
-eTr, eTr, eTr, eTr, eTr, eTr, eTr, eTr, // 06D8-06DF
+eTr, eTr, eTr, eTr, eTr, eNJ, eNJ, eTr, // 06D8-06DF
 eTr, eTr, eTr, eTr, eTr, eNJ, eNJ, eTr, // 06E0-06E7
 eTr, eNJ, eTr, eTr, eTr, eTr, eNJ, eNJ, // 06E8-06EF
 eNJ, eNJ, eNJ, eNJ, eNJ, eNJ, eNJ, eNJ, // 06F0-06F7
@@ -212,11 +217,11 @@ eNJ, eNJ, eNJ, eNJ, eNJ, eNJ, eNJ, eNJ  // 06F8-06FF
 };
 
 #define GetJoiningClass(c)                   \
-  (((0x0621 <= (c)) && ((c) <= 0x06FF)) ?    \
-       (gJoiningClass[(c) - 0x0621]) :       \
+  ((IS_ARABIC_CHAR(c)) ?                     \
+      (gJoiningClass[(c) - 0x0600]) :        \
       ((0x200D == (c)) ? eJC : eTr))
 
-static PRUint16 gArabicLigatureMap[] = 
+static const PRUint16 gArabicLigatureMap[] = 
 {
 0x82DF, // 0xFE82 0xFEDF -> 0xFEF5
 0x82E0, // 0xFE82 0xFEE0 -> 0xFEF6
@@ -228,34 +233,63 @@ static PRUint16 gArabicLigatureMap[] =
 0x8EE0  // 0xFE8E 0xFEE0 -> 0xFEFC
 };
 
-nsresult ArabicShaping(const PRUnichar* aString, PRUint32 aLen,
-                       PRUnichar* aBuf, PRUint32 *aBufLen)
+#define ARABIC_TO_HINDI_DIGIT_INCREMENT (START_HINDI_DIGITS - START_ARABIC_DIGITS)
+#define NUM_TO_ARABIC(c) \
+  ((((c)>=START_HINDI_DIGITS) && ((c)<=END_HINDI_DIGITS)) ? \
+   ((c) - (PRUint16)ARABIC_TO_HINDI_DIGIT_INCREMENT) : \
+   (c))
+#define NUM_TO_HINDI(c) \
+  ((((c)>=START_ARABIC_DIGITS) && ((c)<=END_ARABIC_DIGITS)) ? \
+   ((c) + (PRUint16)ARABIC_TO_HINDI_DIGIT_INCREMENT): \
+   (c))
+
+// helper function to reverse a PRUnichar buffer
+static void ReverseString(PRUnichar* aBuffer, PRUint32 aLen)
 {
-  const PRUnichar* src = aString;
+  PRUnichar *start, *end;
+  PRUnichar swapChar;
+
+  for (start = aBuffer, end = aBuffer + aLen - 1; start < end; ++start, --end) {
+    swapChar = *start;
+    *start = *end;
+    *end = swapChar;
+  }
+}
+
+nsresult ArabicShaping(const PRUnichar* aString, PRUint32 aLen,
+                       PRUnichar* aBuf, PRUint32 *aBufLen, 
+                       PRBool aInputLogical, PRBool aOutputLogical)
+{
+  nsAutoString tempString(aString, aLen);
+  PRUnichar *tempBuf = tempString.BeginWriting();
+  if (aInputLogical) {
+    ReverseString(tempBuf, aLen);
+  }
+  const PRUnichar* src = tempBuf;
   const PRUnichar* p;
   PRUnichar* dest = aBuf;
   PRUnichar formB;
   PRInt8 leftJ, thisJ, rightJ;
   PRInt8 leftNoTrJ, rightNoTrJ;
-  thisJ = eNJ;
+  thisJ = leftNoTrJ = eNJ;
   rightJ = GetJoiningClass(*(src));
-  while(src<aString+aLen-1) {
+  while(src<tempBuf+aLen-1) {
     leftJ = thisJ;
 
     if ((eTr != leftJ) || ((leftJ == eTr) && 
-        ( ( (src-1) >= aString ) && !IS_ARABIC_CHAR(*(src-1)))))
+        ( ( (src-1) >= tempBuf) && !IS_ARABIC_CHAR(*(src-1)))))
       leftNoTrJ = thisJ;
 
-    if(src-2 >= (aString)){
-      for(p=src-2; (p >= (aString))&& (eTr == leftNoTrJ) && (IS_ARABIC_CHAR(*(p+1))) ; p--)  
+    if(src-2 >= (tempBuf)){
+      for(p=src-2; (p >= (tempBuf))&& (eTr == leftNoTrJ) && (IS_ARABIC_CHAR(*(p+1))) ; p--)  
         leftNoTrJ = GetJoiningClass(*(p)) ;
     }
 
     thisJ = rightJ;
     rightJ = rightNoTrJ = GetJoiningClass(*(src+1)) ;
 
-    if(src+2 <= (aString+aLen-1)){
-      for(p=src+2; (p <= (aString+aLen-1))&&(eTr == rightNoTrJ) && (IS_ARABIC_CHAR(*(src+1))); p++)
+    if(src+2 <= (tempBuf+aLen-1)){
+      for(p=src+2; (p <= (tempBuf+aLen-1))&&(eTr == rightNoTrJ) && (IS_ARABIC_CHAR(*(src+1))); p++)
         rightNoTrJ = GetJoiningClass(*(p)) ;
     }
 
@@ -265,11 +299,11 @@ nsresult ArabicShaping(const PRUnichar* aString, PRUint32 aLen,
 
   }
   if((eTr != thisJ) || 
-     ((thisJ == eTr) && (((src-1)>=aString) && (!IS_ARABIC_CHAR(*(src-1))))))
+     ((thisJ == eTr) && (((src-1)>=tempBuf) && (!IS_ARABIC_CHAR(*(src-1))))))
     leftNoTrJ = thisJ;
 
-  if(src-2 >= (aString)){
-    for(p=src-2; (src-2 >= (aString)) && (eTr == leftNoTrJ) && (IS_ARABIC_CHAR(*(p+1))); p--)
+  if(src-2 >= (tempBuf)){
+    for(p=src-2; (src-2 >= (tempBuf)) && (eTr == leftNoTrJ) && (IS_ARABIC_CHAR(*(p+1))); p--)
       leftNoTrJ = GetJoiningClass(*(p)) ;
   }
 
@@ -289,7 +323,8 @@ nsresult ArabicShaping(const PRUnichar* aString, PRUint32 aLen,
       for(i=0;i<8;i++) {
         if(key == gArabicLigatureMap[i]) {
           done = PR_TRUE;
-          *lDest++ = 0x200B;//ZERO WIDTH SPACE
+          // lam and alef in the source are mapped to a lam-alef ligature in the
+          // destination, so lSrc is incremented by 2 here
           *lDest++ = 0xFEF5 + i;
           lSrc+=2;
           break;
@@ -307,6 +342,9 @@ nsresult ArabicShaping(const PRUnichar* aString, PRUint32 aLen,
 
   *aBufLen = lDest - aBuf;
 
+  if (aOutputLogical) {
+    ReverseString(aBuf, *aBufLen);
+  }
   return NS_OK;
 }
 
@@ -316,7 +354,6 @@ nsresult Conv_FE_06(const nsString& aSrc, nsString& aDst)
   PRUint32 i, size = aSrc.Length();
   aDst.Truncate();
   for (i=0;i<size;i++) { // i : Source
-    aSrcUnichars[i];
     if (aSrcUnichars[i] == 0x0000) 
       break; // no need to convert char after the NULL
     if (IS_FE_CHAR(aSrcUnichars[i])) {
@@ -340,7 +377,9 @@ nsresult Conv_FE_06_WithReverse(const nsString& aSrc, nsString& aDst)
 {
   PRUnichar *aSrcUnichars = (PRUnichar *)aSrc.get();
   PRBool foundArabic = PR_FALSE;
-  PRUint32 i,endArabic, beginArabic, size = aSrc.Length();
+  PRUint32 i, endArabic, beginArabic, size;
+  beginArabic = 0;
+  size = aSrc.Length();
   aDst.Truncate();
   for (endArabic=0;endArabic<size;endArabic++) {
     if (aSrcUnichars[endArabic] == 0x0000) 
@@ -388,7 +427,9 @@ nsresult Conv_06_FE_WithReverse(const nsString& aSrc,
                                 PRUint32 aDir)
 {
   PRUnichar *aSrcUnichars = (PRUnichar *)aSrc.get();
-  PRUint32 i,beginArabic, endArabic, size = aSrc.Length();
+  PRUint32 i, beginArabic, endArabic, size;
+  beginArabic = 0;
+  size = aSrc.Length();
   aDst.Truncate();
   PRBool foundArabic = PR_FALSE;
   for (endArabic=0;endArabic<size;endArabic++) {
@@ -410,17 +451,12 @@ nsresult Conv_06_FE_WithReverse(const nsString& aSrc,
       endArabic--;
       PRUnichar buf[8192];
       PRUint32 len=8192;
-      //reverse the buffer for shaping
 
-      for(i=beginArabic; i<=endArabic; i++) {
-        buf[i-beginArabic]=aSrcUnichars[endArabic-i+beginArabic];
-      }
-      for(i=0; i<=endArabic-beginArabic; i++) {
-        aSrcUnichars[i+beginArabic]=buf[i];
-      }
-      ArabicShaping(&aSrcUnichars[beginArabic], endArabic-beginArabic+1, buf, &len);
+      ArabicShaping(&aSrcUnichars[beginArabic], endArabic-beginArabic+1,
+                    buf, &len, 
+                    PR_TRUE, PR_FALSE);
       // to reverse the numerals
-      PRUint32 endNumeral, beginNumeral;
+      PRUint32 endNumeral, beginNumeral = 0;
       for (endNumeral=0;endNumeral<=len-1;endNumeral++){
         PRBool foundNumeral = PR_FALSE;
         while((endNumeral < len) && (IS_ARABIC_DIGIT(buf[endNumeral]))  ) {
@@ -459,4 +495,45 @@ nsresult Conv_06_FE_WithReverse(const nsString& aSrc,
   }// for : loop the buffer
   return NS_OK;
 }
-#endif //IBMBIDI
+
+nsresult HandleNumbers(PRUnichar* aBuffer, PRUint32 aSize, PRUint32 aNumFlag)
+{
+  PRUint32 i;
+  // IBMBIDI_NUMERAL_NOMINAL *
+  // IBMBIDI_NUMERAL_REGULAR
+  // IBMBIDI_NUMERAL_HINDICONTEXT
+  // IBMBIDI_NUMERAL_ARABIC
+  // IBMBIDI_NUMERAL_HINDI
+
+  switch (aNumFlag) {
+    case IBMBIDI_NUMERAL_HINDI:
+      for (i=0;i<aSize;i++)
+        aBuffer[i] = NUM_TO_HINDI(aBuffer[i]);
+      break;
+    case IBMBIDI_NUMERAL_ARABIC:
+      for (i=0;i<aSize;i++)
+        aBuffer[i] = NUM_TO_ARABIC(aBuffer[i]);
+      break;
+    case IBMBIDI_NUMERAL_REGULAR:
+    case IBMBIDI_NUMERAL_HINDICONTEXT:
+        // for clipboard handling
+        //XXX do we really want to convert numerals when copying text?
+      for (i=1;i<aSize;i++) {
+        if (IS_ARABIC_CHAR(aBuffer[i-1])) 
+          aBuffer[i] = NUM_TO_HINDI(aBuffer[i]);
+        else 
+          aBuffer[i] = NUM_TO_ARABIC(aBuffer[i]);
+      }
+    case IBMBIDI_NUMERAL_NOMINAL:
+    default:
+      break;
+  }
+  return NS_OK;
+}
+
+nsresult HandleNumbers(const nsString& aSrc, nsString& aDst)
+{
+  aDst = aSrc;
+  return HandleNumbers((PRUnichar *)aDst.get(),aDst.Length(), IBMBIDI_NUMERAL_REGULAR);
+}
+
