@@ -39,38 +39,21 @@
 #ifndef nsXMLContentSink_h__
 #define nsXMLContentSink_h__
 
+#include "nsContentSink.h"
 #include "nsIXMLContentSink.h"
-#include "nsIViewManager.h"
-#include "nsIScrollableView.h"
-#include "nsIObserver.h"
-#include "nsWeakReference.h"
-#include "nsITransformMediator.h"
-#include "nsIUnicharInputStream.h"
-#include "nsIStreamLoader.h"
-#include "nsISupportsArray.h"
-#include "nsINodeInfo.h"
-#include "nsIDOMHTMLTextAreaElement.h"
-class nsICSSStyleSheet;
-#include "nsICSSLoaderObserver.h"
-#include "nsIHTMLContent.h"
-#include "nsIDOMHTMLScriptElement.h"
-#include "nsIScriptLoader.h"
-#include "nsIScriptLoaderObserver.h"
-#include "nsSupportsArray.h"
 #include "nsIExpatSink.h"
+#include "nsIDocumentTransformer.h"
+#include "nsCOMArray.h"
+#include "nsCOMPtr.h"
+
 
 class nsIDocument;
 class nsIURI;
-class nsIWebShell;
 class nsIContent;
-class nsAutoVoidArray;
-class nsIXMLDocument;
-class nsIUnicharInputStream;
+class nsINodeInfo;
 class nsIParser;
 class nsINameSpace;
-class nsICSSLoader;
-class nsINameSpaceManager;
-class nsIElementFactory;
+class nsIViewManager;
 
 typedef enum {
   eXMLContentSinkState_InProlog,
@@ -78,13 +61,9 @@ typedef enum {
   eXMLContentSinkState_InEpilog
 } XMLContentSinkState;
 
-// XXX Till the parser knows a little bit more about XML, 
-// this is a HTMLContentSink.
-class nsXMLContentSink : public nsIXMLContentSink,
-                         public nsIObserver,
-                         public nsSupportsWeakReference,
-                         public nsIScriptLoaderObserver,
-                         public nsICSSLoaderObserver,
+class nsXMLContentSink : public nsContentSink,
+                         public nsIXMLContentSink,
+                         public nsITransformObserver,
                          public nsIExpatSink
 {
 public:
@@ -93,50 +72,46 @@ public:
 
   nsresult Init(nsIDocument* aDoc,
                 nsIURI* aURL,
-                nsIWebShell* aContainer,
+                nsISupports* aContainer,
                 nsIChannel* aChannel);
 
   // nsISupports
-  NS_DECL_ISUPPORTS
-  NS_DECL_NSISCRIPTLOADEROBSERVER
+  NS_DECL_ISUPPORTS_INHERITED
 
   NS_DECL_NSIEXPATSINK
 
   // nsIContentSink
   NS_IMETHOD WillBuildModel(void);
-  NS_IMETHOD DidBuildModel(PRInt32 aQualityLevel);
+  NS_IMETHOD DidBuildModel(void);
   NS_IMETHOD WillInterrupt(void);
   NS_IMETHOD WillResume(void);
   NS_IMETHOD SetParser(nsIParser* aParser);  
   NS_IMETHOD FlushPendingNotifications() { return NS_OK; }
-  NS_IMETHOD SetDocumentCharset(nsAString& aCharset);
+  NS_IMETHOD SetDocumentCharset(nsACString& aCharset);
+  virtual nsISupports *GetTarget();
 
-  // nsICSSLoaderObserver
-  NS_IMETHOD StyleSheetLoaded(nsICSSStyleSheet*aSheet, PRBool aNotify);
-
-  // nsIObserver
-  NS_IMETHOD Observe(nsISupports *aSubject, 
-                     const char *aTopic, 
-                     const PRUnichar *someData);
+  // nsITransformObserver
+  NS_IMETHOD OnDocumentCreated(nsIDOMDocument *aResultDocument);
+  NS_IMETHOD OnTransformDone(nsresult aResult, nsIDOMDocument *aResultDocument);
 
 protected:
   void StartLayout();
 
   nsresult PushNameSpacesFrom(const PRUnichar** aAttributes);
-  nsresult AddAttributes(const PRUnichar** aNode, nsIContent* aContent,PRBool aIsHTML);
+  nsresult AddAttributes(const PRUnichar** aNode, nsIContent* aContent);
   nsresult AddText(const PRUnichar* aString, PRInt32 aLength);
-  nsresult ProcessStartSCRIPTTag(PRUint32 aLineNo);
-  nsresult ProcessEndSCRIPTTag();
+  nsresult ProcessEndSCRIPTTag(nsIContent* aContent);
 
   virtual PRBool OnOpenContainer(const PRUnichar **aAtts, 
                                  PRUint32 aAttsCount, 
                                  PRInt32 aNameSpaceID, 
-                                 nsIAtom* aTagName) { return PR_TRUE; }
-  virtual nsresult CreateElement(const PRUnichar** aAtts, 
-                                 PRUint32 aAttsCount, 
-                                 PRInt32 aNameSpaceID, 
-                                 nsINodeInfo* aNodeInfo, 
-                                 nsIContent** aResult);
+                                 nsIAtom* aTagName,
+                                 PRUint32 aLineNumber) { return PR_TRUE; }
+  virtual nsresult CreateElement(const PRUnichar** aAtts, PRUint32 aAttsCount,
+                                 nsINodeInfo* aNodeInfo, PRUint32 aLineNumber,
+                                 nsIContent** aResult, PRBool* aAppendContent);
+
+  virtual nsresult CloseElement(nsIContent* aContent, PRBool* aAppendContent);
 
   virtual nsresult FlushText(PRBool aCreateTextNode=PR_TRUE,
                              PRBool* aDidFlush=nsnull);
@@ -146,73 +121,49 @@ protected:
   static void SplitXMLName(const nsAString& aString, nsIAtom **aPrefix,
                            nsIAtom **aTagName);
   PRInt32 GetNameSpaceId(nsIAtom* aPrefix);
-  nsINameSpace*    PopNameSpaces();
-  PRBool  IsHTMLNameSpace(PRInt32 aId);
+  already_AddRefed<nsINameSpace> PopNameSpaces();
 
   nsIContent* GetCurrentContent();
   PRInt32 PushContent(nsIContent *aContent);
-  nsIContent* PopContent();
+  already_AddRefed<nsIContent> PopContent();
 
 
-  nsresult ProcessBASETag();
-  nsresult ProcessMETATag();
-  nsresult ProcessLINKTag();
-  nsresult ProcessHeaderData(nsIAtom* aHeader, const nsAString& aValue,
-                             nsIHTMLContent* aContent);
-  nsresult ProcessHTTPHeaders(nsIChannel* aChannel);
-  nsresult ProcessLink(nsIHTMLContent* aElement, const nsAString& aLinkData);
+  nsresult ProcessBASETag(nsIContent* aContent);
 
-  nsresult RefreshIfEnabled(nsIViewManager* vm);
   
-  NS_IMETHOD ProcessStyleLink(nsIContent* aElement,
-                              const nsString& aHref,
-                              PRBool aAlternate,
-                              const nsString& aTitle,
-                              const nsString& aType,
-                              const nsString& aMedia);
+  // nsContentSink override
+  virtual nsresult ProcessStyleLink(nsIContent* aElement,
+                                    const nsAString& aHref,
+                                    PRBool aAlternate,
+                                    const nsAString& aTitle,
+                                    const nsAString& aType,
+                                    const nsAString& aMedia);
 
   nsresult LoadXSLStyleSheet(nsIURI* aUrl);
-  nsresult SetupTransformMediator();
 
-  static void
-  GetElementFactory(PRInt32 aNameSpaceID, nsIElementFactory** aResult);
-
-  void ScrollToRef();
+  nsresult MaybePrettyPrint();
   
-  static nsINameSpaceManager* gNameSpaceManager;
-  static PRUint32 gRefCnt;
-
-  nsIDocument*     mDocument;
-  nsIURI*          mDocumentURL;
-  nsIURI*          mDocumentBaseURL; // can be set via HTTP headers
-  nsIWebShell*     mWebShell;
-  nsIParser*       mParser;
   nsIContent*      mDocElement;
-  nsAutoVoidArray* mNameSpaceStack;
   PRUnichar*       mText;
-  nsICSSLoader*    mCSSLoader;  
 
-  nsSupportsArray mScriptElements;
   XMLContentSinkState mState;
 
-  nsString mRef; // ScrollTo #ref
   nsString mTitleText; 
   
-  PRInt32 mStyleSheetCount;
   PRInt32 mTextLength;
   PRInt32 mTextSize;
   PRUint32 mScriptLineNo;
   
   PRPackedBool mConstrainSize;
   PRPackedBool mInTitle;
-  PRPackedBool mNeedToBlockParser;
+  PRPackedBool mPrettyPrintXML;
+  PRPackedBool mPrettyPrintHasSpecialRoot;
+  PRPackedBool mPrettyPrintHasFactoredElements;
+  PRPackedBool mHasProcessedBase;
 
-  nsCOMPtr<nsISupportsArray>          mContentStack;
-  nsCOMPtr<nsINodeInfoManager>        mNodeInfoManager;
-  nsCOMPtr<nsITransformMediator>      mXSLTransformMediator;
-  nsCOMPtr<nsIHTMLContent>            mBaseElement;
-  nsCOMPtr<nsIHTMLContent>            mMetaElement;
-  nsCOMPtr<nsIHTMLContent>            mLinkElement;
+  nsCOMArray<nsIContent>           mContentStack;
+  nsCOMArray<nsINameSpace>         mNameSpaceStack;
+  nsCOMPtr<nsIDocumentTransformer> mXSLTProcessor;
 };
 
 #endif // nsXMLContentSink_h__
