@@ -46,6 +46,7 @@
 // Local Includes
 // Helper Classes
 #include "nsCOMPtr.h"
+#include "nsAutoPtr.h"
 #include "nsWeakReference.h"
 #include "nsHashtable.h"
 
@@ -54,10 +55,12 @@
 #include "nsIBaseWindow.h"
 #include "nsIChromeEventHandler.h"
 #include "nsIControllers.h"
+#include "nsIObserver.h"
 #include "nsIDocShellTreeOwner.h"
 #include "nsIDocShellTreeItem.h"
 #include "nsIDOMViewCSS.h"
 #include "nsIDOMEventReceiver.h"
+#include "nsIDOM3EventTarget.h"
 #include "nsIDOMNavigator.h"
 #include "nsIDOMNSLocation.h"
 #include "nsIDOMWindowInternal.h"
@@ -70,11 +73,11 @@
 #include "nsITimer.h"
 #include "nsIWebBrowserChrome.h"
 #include "nsPIDOMWindow.h"
+#include "nsIScriptSecurityManager.h"
 #include "nsIEventListenerManager.h"
 #include "nsIDOMDocument.h"
 #include "nsIDOMCrypto.h"
 #include "nsIDOMPkcs11.h"
-#include "nsISidebar.h"
 #include "nsIPrincipal.h"
 #include "nsPluginArray.h"
 #include "nsMimeTypeArray.h"
@@ -116,6 +119,7 @@ class GlobalWindowImpl : public nsIScriptGlobalObject,
                          public nsIDOMJSWindow,
                          public nsIScriptObjectPrincipal,
                          public nsIDOMEventReceiver,
+                         public nsIDOM3EventTarget,
                          public nsPIDOMWindow,
                          public nsIDOMViewCSS,
                          public nsSupportsWeakReference,
@@ -126,28 +130,34 @@ public:
   NS_DECL_ISUPPORTS
 
   // nsIScriptGlobalObject
-  NS_IMETHOD SetContext(nsIScriptContext *aContext);
-  NS_IMETHOD GetContext(nsIScriptContext **aContext);
-  NS_IMETHOD SetNewDocument(nsIDOMDocument *aDocument,
-                            PRBool removeEventListeners);
-  NS_IMETHOD SetDocShell(nsIDocShell* aDocShell);
-  NS_IMETHOD GetDocShell(nsIDocShell** aDocShell);
-  NS_IMETHOD SetOpenerWindow(nsIDOMWindowInternal *aOpener);
-  NS_IMETHOD SetGlobalObjectOwner(nsIScriptGlobalObjectOwner* aOwner);
-  NS_IMETHOD GetGlobalObjectOwner(nsIScriptGlobalObjectOwner** aOwner);
-  NS_IMETHOD HandleDOMEvent(nsIPresContext* aPresContext, nsEvent* aEvent,
-                            nsIDOMEvent** aDOMEvent, PRUint32 aFlags,
-                            nsEventStatus* aEventStatus);
-  NS_IMETHOD_(JSObject *) GetGlobalJSObject();
-  NS_IMETHOD OnFinalize(JSObject *aJSObject);
-  NS_IMETHOD SetScriptsEnabled(PRBool aEnabled, PRBool aFireTimeouts);
+  virtual void SetContext(nsIScriptContext *aContext);
+  virtual nsIScriptContext *GetContext();
+  virtual nsresult SetNewDocument(nsIDOMDocument *aDocument,
+                                  PRBool aRemoveEventListeners,
+                                  PRBool aClearScopeHint);
+  virtual void SetDocShell(nsIDocShell* aDocShell);
+  virtual nsIDocShell *GetDocShell();
+  virtual void SetOpenerWindow(nsIDOMWindowInternal *aOpener);
+  virtual void SetGlobalObjectOwner(nsIScriptGlobalObjectOwner* aOwner);
+  virtual nsIScriptGlobalObjectOwner *GetGlobalObjectOwner();
+  virtual nsresult HandleDOMEvent(nsIPresContext* aPresContext,
+                                  nsEvent* aEvent, nsIDOMEvent** aDOMEvent,
+                                  PRUint32 aFlags,
+                                  nsEventStatus* aEventStatus);
+  virtual JSObject *GetGlobalJSObject();
+  virtual void OnFinalize(JSObject *aJSObject);
+  virtual void SetScriptsEnabled(PRBool aEnabled, PRBool aFireTimeouts);
 
   // nsIScriptObjectPrincipal
+  NS_IMETHOD GetPrincipalObsolete(nsIPrincipalObsolete **prin);
   NS_IMETHOD GetPrincipal(nsIPrincipal **prin);
 
   // nsIDOMWindow
   NS_DECL_NSIDOMWINDOW
 
+  // nsIDOMWindow2
+  NS_DECL_NSIDOMWINDOW2
+
   // nsIDOMWindowInternal
   NS_DECL_NSIDOMWINDOWINTERNAL
 
@@ -155,13 +165,10 @@ public:
   NS_DECL_NSIDOMJSWINDOW
 
   // nsIDOMEventTarget
-  NS_IMETHOD AddEventListener(const nsAReadableString& aType,
-                              nsIDOMEventListener* aListener,
-                              PRBool aUseCapture);
-  NS_IMETHOD RemoveEventListener(const nsAReadableString& aType,
-                                 nsIDOMEventListener* aListener,
-                                 PRBool aUseCapture);
-  NS_IMETHOD DispatchEvent(nsIDOMEvent* aEvent, PRBool *_retval);
+  NS_DECL_NSIDOMEVENTTARGET
+
+  // nsIDOM3EventTarget
+  NS_DECL_NSIDOM3EVENTTARGET
 
   // nsIDOMEventReceiver
   NS_IMETHOD AddEventListenerByIID(nsIDOMEventListener *aListener,
@@ -169,8 +176,8 @@ public:
   NS_IMETHOD RemoveEventListenerByIID(nsIDOMEventListener *aListener,
                                       const nsIID& aIID);
   NS_IMETHOD GetListenerManager(nsIEventListenerManager** aInstancePtrResult);
-  NS_IMETHOD GetNewListenerManager(nsIEventListenerManager **aInstancePtrResult);
   NS_IMETHOD HandleEvent(nsIDOMEvent *aEvent);
+  NS_IMETHOD GetSystemEventGroup(nsIDOMEventGroup** aGroup);
 
   // nsPIDOMWindow
   NS_IMETHOD GetPrivateParent(nsPIDOMWindow** aResult);
@@ -187,6 +194,18 @@ public:
   NS_IMETHOD GetExtantDocument(nsIDOMDocument** aDocument);
 
   NS_IMETHOD ReallyCloseWindow();
+  NS_IMETHOD IsLoadingOrRunningTimeout(PRBool* aResult);
+  NS_IMETHOD IsPopupSpamWindow(PRBool *aResult);
+  NS_IMETHOD SetPopupSpamWindow(PRBool aPopup);
+
+  NS_IMETHOD GetFrameElementInternal(nsIDOMElement** aFrameElement);
+  NS_IMETHOD SetFrameElementInternal(nsIDOMElement* aFrameElement);
+  NS_IMETHOD SetOpenerScriptURL(nsIURI* aURI);
+
+  virtual NS_HIDDEN_(PopupControlState) PushPopupControlState(PopupControlState state) const;
+  virtual NS_HIDDEN_(void) PopPopupControlState(PopupControlState state) const;
+  virtual NS_HIDDEN_(PopupControlState) GetPopupControlState() const;
+  virtual NS_HIDDEN_(PRBool) IsHandlingResizeEvent() const;
 
   // nsIDOMViewCSS
   NS_DECL_NSIDOMVIEWCSS
@@ -201,19 +220,21 @@ public:
   GlobalWindowImpl();
 
   static void ShutDown();
+  static PRBool IsCallerChrome();
 
 protected:
   // Object Management
   virtual ~GlobalWindowImpl();
   void CleanUp();
+  void ClearControllers();
 
   // Get the parent, returns null if this is a toplevel window
-  void GetParentInternal(nsIDOMWindowInternal **parent);
+  nsIDOMWindowInternal *GetParentInternal();
 
   // Window Control Functions
-  NS_IMETHOD OpenInternal(const nsAReadableString& aUrl,
-                          const nsAReadableString& aName,
-                          const nsAReadableString& aOptions,
+  NS_IMETHOD OpenInternal(const nsAString& aUrl,
+                          const nsAString& aName,
+                          const nsAString& aOptions,
                           PRBool aDialog, jsval *argv, PRUint32 argc,
                           nsISupports *aExtraArgument, nsIDOMWindow **aReturn);
   static void CloseWindow(nsISupports* aWindow);
@@ -221,9 +242,7 @@ protected:
   // Timeout Functions
   nsresult SetTimeoutOrInterval(PRBool aIsInterval, PRInt32* aReturn);
   void RunTimeout(nsTimeoutImpl *aTimeout);
-  void DropTimeout(nsTimeoutImpl *aTimeout, nsIScriptContext* aContext=nsnull);
-  void HoldTimeout(nsTimeoutImpl *aTimeout);
-  nsresult ClearTimeoutOrInterval(PRInt32 aTimerID);
+  nsresult ClearTimeoutOrInterval();
   void ClearAllTimeouts();
   void InsertTimeoutIntoList(nsTimeoutImpl **aInsertionPoint,
                              nsTimeoutImpl *aTimeout);
@@ -236,20 +255,44 @@ protected:
   nsresult GetScrollInfo(nsIScrollableView** aScrollableView, float* aP2T,
                          float* aT2P);
   nsresult SecurityCheckURL(const char *aURL);
-  PRBool   CheckForAbusePoint();
+  PopupControlState CheckForAbusePoint();
+  PRUint32 CheckOpenAllow(PopupControlState aAbuseLevel,
+                          const nsAString &aName);
+  void     FireAbuseEvents(PRBool aBlocked, PRBool aWindow,
+                           const nsAString &aPopupURL,
+                           const nsAString &aPopupWindowFeatures);
 
   void FlushPendingNotifications(PRBool aFlushReflows);
   void EnsureReflowFlushAndPaint();
   nsresult CheckSecurityWidthAndHeight(PRInt32* width, PRInt32* height);
   nsresult CheckSecurityLeftAndTop(PRInt32* left, PRInt32* top);
+  static nsresult CheckSecurityIsChromeCaller(PRBool *isChrome);
+  static PRBool CanSetProperty(const char *aPrefName);
+
+  void MakeScriptDialogTitle(const nsAString &aInTitle, nsAString &aOutTitle);
 
   // Helper for window.find()
-  nsresult FindInternal(nsAReadableString& aStr, PRBool caseSensitive,
+  nsresult FindInternal(const nsAString& aStr, PRBool caseSensitive,
                        PRBool backwards, PRBool wrapAround, PRBool wholeWord, 
                        PRBool searchInFrames, PRBool showDialog, 
                        PRBool *aReturn);
 
-protected:
+  nsresult ConvertCharset(const nsAString& aStr, char** aDest);
+
+  PRBool   GetBlurSuppression();
+
+  nsresult GetScrollXY(PRInt32* aScrollX, PRInt32* aScrollY);
+  nsresult GetScrollMaxXY(PRInt32* aScrollMaxX, PRInt32* aScrollMaxY);
+
+  PRBool IsFrame()
+  {
+    return GetParentInternal() != nsnull;
+  }
+
+  PRBool DispatchCustomEvent(const char *aEventName);
+
+  PRBool WindowExists(const nsAString& aName);
+
   // When adding new member variables, be careful not to create cycles
   // through JavaScript.  If there is any chance that a member variable
   // could own objects that are implemented in JavaScript, then those
@@ -261,47 +304,60 @@ protected:
   nsCOMPtr<nsIDOMWindowInternal> mOpener;
   nsCOMPtr<nsIControllers>      mControllers;
   nsCOMPtr<nsIEventListenerManager> mListenerManager;
-  nsCOMPtr<nsISidebar>          mSidebar;
   JSObject*                     mJSObject;
-  NavigatorImpl*                mNavigator;
-  ScreenImpl*                   mScreen;
-  HistoryImpl*                  mHistory;
-  nsDOMWindowList*              mFrames;
-  LocationImpl*                 mLocation;
-  BarPropImpl*                  mMenubar;
-  BarPropImpl*                  mToolbar;
-  BarPropImpl*                  mLocationbar;
-  BarPropImpl*                  mPersonalbar;
-  BarPropImpl*                  mStatusbar;
-  BarPropImpl*                  mScrollbars;
+  nsRefPtr<NavigatorImpl>       mNavigator;
+  nsRefPtr<ScreenImpl>          mScreen;
+  nsRefPtr<HistoryImpl>         mHistory;
+  nsRefPtr<nsDOMWindowList>     mFrames;
+  nsRefPtr<LocationImpl>        mLocation;
+  nsRefPtr<BarPropImpl>         mMenubar;
+  nsRefPtr<BarPropImpl>         mToolbar;
+  nsRefPtr<BarPropImpl>         mLocationbar;
+  nsRefPtr<BarPropImpl>         mPersonalbar;
+  nsRefPtr<BarPropImpl>         mStatusbar;
+  nsRefPtr<BarPropImpl>         mScrollbars;
+  nsCOMPtr<nsIWeakReference>    mWindowAccess;
   nsTimeoutImpl*                mTimeouts;
   nsTimeoutImpl**               mTimeoutInsertionPoint;
   nsTimeoutImpl*                mRunningTimeout;
   PRUint32                      mTimeoutPublicIdCounter;
   PRUint32                      mTimeoutFiringDepth;
-  PRPackedBool                  mTimeoutsWereCleared;
   PRPackedBool                  mFirstDocumentLoad;
   PRPackedBool                  mIsScopeClear;
   PRPackedBool                  mIsDocumentLoaded; // true between onload and onunload events
-  PRTime                        mLastMouseButtonAction;
   PRPackedBool                  mFullScreen;
+  PRPackedBool                  mIsClosed;
+  PRPackedBool                  mOpenerWasCleared;
+  PRPackedBool                  mIsPopupSpam;
+  PRPackedBool                  mIsHandlingResizeEvent;
   nsString                      mStatus;
   nsString                      mDefaultStatus;
 
-  // state preservation for full screen mode
-  nsPoint*                     mOriginalPos;
-  nsSize*                      mOriginalSize;
-
   nsIScriptGlobalObjectOwner*   mGlobalObjectOwner; // Weak Reference
   nsIDocShell*                  mDocShell;  // Weak Reference
+  nsEvent*                      mCurrentEvent;
   PRUint32                      mMutationBits;
   nsCOMPtr<nsIChromeEventHandler> mChromeEventHandler; // [Strong] We break it when we get torn down.
   nsCOMPtr<nsIDOMCrypto>        mCrypto;
   nsCOMPtr<nsIDOMPkcs11>        mPkcs11;
   nsCOMPtr<nsIPrincipal>        mDocumentPrincipal;
+  nsCOMPtr<nsIURI>              mOpenerScriptURL; // Used to determine whether to clear scope
+
+  // XXX We need mNavigatorHolder because we make two SetNewDocument()
+  // calls when transitioning from page to page. This keeps a reference
+  // to the JSObject holder for the navigator object in between
+  // SetNewDocument() calls so that the JSObject doesn't get garbage
+  // collected in between these calls.
+  // See bug 163645 for more on why we need this and bug 209607 for info
+  // on how we can remove the need for this.
+  nsCOMPtr<nsIXPConnectJSObjectHolder> mNavigatorHolder;
+
+  nsIDOMElement*                mFrameElement; // WEAK
 
   friend class nsDOMScriptableHelper;
   static nsIXPConnect *sXPConnect;
+  static nsIScriptSecurityManager *sSecMan;
+  static nsIFactory *sComputedDOMStyleFactory;
 };
 
 /*
@@ -328,35 +384,91 @@ protected:
  * Timeout struct that holds information about each JavaScript
  * timeout.
  */
-struct nsTimeoutImpl {
-  nsTimeoutImpl() {
+struct nsTimeoutImpl
+{
+  nsTimeoutImpl()
+  {
+#ifdef DEBUG_jst
+    {
+      extern PRInt32 gTimeoutCnt;
+
+      ++gTimeoutCnt;
+    }
+#endif
+
     memset(this, 0, sizeof(*this));
 
     MOZ_COUNT_CTOR(nsTimeoutImpl);
   }
 
-  ~nsTimeoutImpl() {
+  ~nsTimeoutImpl()
+  {
+#ifdef DEBUG_jst
+    {
+      extern PRInt32 gTimeoutCnt;
+
+      --gTimeoutCnt;
+    }
+#endif
+
     MOZ_COUNT_DTOR(nsTimeoutImpl);
   }
 
-  PRInt32             ref_count;      /* reference count to shared usage */
-  GlobalWindowImpl    *window;        /* window for which this timeout fires */
-  JSString            *expr;          /* the JS expression to evaluate */
-  JSObject            *funobj;        /* or function to call, if !expr */
-  nsCOMPtr<nsITimer>  timer;          /* The actual timer object */
-  jsval               *argv;          /* function actual arguments */
-  PRUint16            argc;           /* and argument count */
-  PRUint16            spare;          /* alignment padding */
-  PRUint32            public_id;      /* Returned as value of setTimeout() */
-  PRInt32             interval;       /* Non-zero if repetitive timeout */
-  PRInt64             when;           /* nominal time to run this timeout */
-  nsCOMPtr<nsIPrincipal> principal;   /* principals with which to execute */
-  char                *filename;      /* filename of setTimeout call */
-  PRUint32            lineno;         /* line number of setTimeout call */
-  const char          *version;       /* JS language version string constant */
-  PRUint32            firingDepth;    /* stack depth at which timeout is
-                                         firing */
-  nsTimeoutImpl       *next;
+  void Release(nsIScriptContext* aContext);
+  void AddRef();
+
+  // Window for which this timeout fires
+  GlobalWindowImpl *mWindow;
+
+  // The JS expression to evaluate or function to call, if !mExpr
+  JSString *mExpr;
+  JSObject *mFunObj;
+
+  // The actual timer object
+  nsCOMPtr<nsITimer> mTimer;
+
+  // Function actual arguments and argument count
+  jsval *mArgv;
+  PRUint16 mArgc;
+
+  // True if the timeout was cleared
+  PRPackedBool mCleared;
+
+  // True if this is one of the timeouts that are currently running
+  PRPackedBool mRunning;
+
+  // Returned as value of setTimeout()
+  PRUint32 mPublicId;
+
+  // Non-zero if repetitive timeout
+  PRInt32 mInterval;
+
+  // Nominal time to run this timeout
+  PRInt64 mWhen;
+
+  // Principal with which to execute
+  nsCOMPtr<nsIPrincipal> mPrincipal;
+
+  // filename, line number and JS language version string of the
+  // caller of setTimeout()
+  char *mFileName;
+  PRUint32 mLineNo;
+  const char *mVersion;
+
+  // stack depth at which timeout is firing
+  PRUint32 mFiringDepth;
+
+  // Pointer to the next timeout in the linked list of scheduled
+  // timeouts
+  nsTimeoutImpl *mNext;
+
+  // The popup state at timeout creation time if not created from
+  // another timeout
+  PopupControlState mPopupState;
+
+private:
+  // reference count for shared usage
+  PRInt32 mRefCnt;
 };
 
 //*****************************************************************************
@@ -375,11 +487,12 @@ public:
   NS_DECL_NSIDOMJSNAVIGATOR
   
   void SetDocShell(nsIDocShell *aDocShell);
+  void LoadingNewDocument();
   nsresult RefreshMIMEArray();
 
 protected:
-  MimeTypeArrayImpl* mMimeTypes;
-  PluginArrayImpl* mPlugins;
+  nsRefPtr<MimeTypeArrayImpl> mMimeTypes;
+  nsRefPtr<PluginArrayImpl> mPlugins;
   nsIDocShell* mDocShell; // weak reference
 
   static jsval       sPrefInternal_id;
@@ -400,7 +513,7 @@ public:
 
   NS_DECL_ISUPPORTS
 
-  NS_IMETHOD_(void)       SetDocShell(nsIDocShell *aDocShell);
+  void SetDocShell(nsIDocShell *aDocShell);
 
   // nsIDOMLocation
   NS_DECL_NSIDOMLOCATION
@@ -409,12 +522,15 @@ public:
   NS_DECL_NSIDOMNSLOCATION
 
 protected:
-  nsresult GetURI(nsIURI** aURL);
+  // In the case of jar: uris, we sometimes want the place the jar was
+  // fetched from as the URI instead of the jar: uri itself.  Pass in
+  // PR_TRUE for aGetInnermostURI when that's the case.
+  nsresult GetURI(nsIURI** aURL, PRBool aGetInnermostURI = PR_FALSE);
   nsresult GetWritableURI(nsIURI** aURL);
   nsresult SetURI(nsIURI* aURL);
-  nsresult SetHrefWithBase(const nsAReadableString& aHref, nsIURI* aBase,
+  nsresult SetHrefWithBase(const nsAString& aHref, nsIURI* aBase,
                            PRBool aReplace);
-  nsresult SetHrefWithContext(JSContext* cx, const nsAReadableString& aHref,
+  nsresult SetHrefWithContext(JSContext* cx, const nsAString& aHref,
                               PRBool aReplace);
 
   nsresult GetSourceURL(JSContext* cx, nsIURI** sourceURL);
@@ -422,34 +538,13 @@ protected:
   nsresult GetSourceDocument(JSContext* cx, nsIDocument** aDocument);
 
   nsresult CheckURL(nsIURI *url, nsIDocShellLoadInfo** aLoadInfo);
+  nsresult FindUsableBaseURI(nsIURI * aBaseURI, nsIDocShell * aParent, nsIURI ** aUsableURI);
 
   nsIDocShell *mDocShell; // Weak Reference
 };
 
-#define DOM_CONTROLLER
-#ifdef DOM_CONTROLLER
-class nsIContentViewerEdit;
-
-class nsISelectionController;
-
-class nsDOMWindowController : public nsIController
-{
-public:
-	nsDOMWindowController( nsIDOMWindowInternal* aWindow );
-  NS_DECL_ISUPPORTS
-  NS_DECL_NSICONTROLLER
-
-private:
-  nsresult GetPresShell(nsIPresShell **aPresShell);
-	nsresult GetEditInterface( nsIContentViewerEdit** aEditInterface);
-  nsresult GetSelectionController(nsISelectionController ** aSelCon);
-
-  nsresult DoCommandWithEditInterface(const nsCString& aCommandName);
-  nsresult DoCommandWithSelectionController(const nsCString& aCommandName);
-
-	nsIDOMWindowInternal *mWindow;
-  PRBool mBrowseWithCaret;
-};
-#endif // DOM_CONTROLLER
+/* factory function */
+nsresult NS_NewScriptGlobalObject(PRBool aIsChrome,
+                                  nsIScriptGlobalObject **aResult);
 
 #endif /* nsGlobalWindow_h___ */
