@@ -41,6 +41,12 @@
 #include "nscore.h"
 #include "nsHistory.h"
 #include "nsIDOMWindowInternal.h"
+#include "nsIDOMDocument.h"
+#include "nsIDocument.h"
+#include "nsIPresShell.h"
+#include "nsIPresContext.h"
+#include "nsJSUtils.h"
+#include "nsPIDOMWindow.h"
 #include "nsIScriptGlobalObject.h"
 #include "nsIWebShell.h"
 #include "nsIDocShell.h"
@@ -49,6 +55,7 @@
 #include "nsIHistoryEntry.h"
 #include "nsIURI.h"
 #include "nsIServiceManager.h"
+#include "nsIInterfaceRequestorUtils.h"
 #include "nsXPIDLString.h"
 #include "nsReadableUtils.h"
 #include "nsDOMClassInfo.h"
@@ -274,6 +281,37 @@ HistoryImpl::Go()
     delta = JSVAL_TO_INT(argv[0]);
   }
 
+  if (delta == 0) {
+    nsCOMPtr<nsPIDOMWindow> window(do_GetInterface(mDocShell));
+
+    if (window && window->IsHandlingResizeEvent()) {
+      // history.go(0) (aka location.reload()) was called on a window
+      // that is handling a resize event. Sites do this since Netscape
+      // 4.x needed it, but we don't, and it's a horrible experience
+      // for nothing.  In stead of reloading the page, just clear
+      // style data and reflow the page since some sites may use this
+      // trick to work around gecko reflow bugs, and this should have
+      // the same effect.
+
+      nsCOMPtr<nsIDOMDocument> domDoc;
+      window->GetExtantDocument(getter_AddRefs(domDoc));
+
+      nsCOMPtr<nsIDocument> doc(do_QueryInterface(domDoc));
+
+      nsIPresShell *shell;
+      if (doc && (shell = doc->GetShellAt(0))) {
+        nsCOMPtr<nsIPresContext> pcx;
+        shell->GetPresContext(getter_AddRefs(pcx));
+
+        if (pcx) {
+          pcx->ClearStyleDataAndReflow();
+        }
+      }
+
+      return NS_OK;
+    }
+  }
+
   return Go(delta);
 }
 
