@@ -45,7 +45,9 @@
 
 #include "nsIEditor.h"
 #include "nsIEditorIMESupport.h"
+#include "nsIPhonetic.h"
 
+#include "nsIAtom.h"
 #include "nsIDOMDocument.h"
 #include "nsISelection.h"
 #include "nsIDOMCharacterData.h"
@@ -92,7 +94,8 @@ class nsISelectionController;
  */
 class nsEditor : public nsIEditor,
                  public nsIEditorIMESupport,
-                 public nsSupportsWeakReference
+                 public nsSupportsWeakReference,
+                 public nsIPhonetic
 {
 public:
 
@@ -148,6 +151,10 @@ public:
   NS_IMETHOD EndComposition(void);
   NS_IMETHOD ForceCompositionEnd(void);
   NS_IMETHOD GetReconversionString(nsReconversionEventReply *aReply);
+  
+  // nsIPhonetic
+  NS_DECL_NSIPHONETIC
+
 
 public:
 
@@ -193,6 +200,8 @@ protected:
   NS_IMETHOD GetProperties(nsVoidArray *aPropList);
   */
   
+  nsCString mContentMIMEType;       // MIME type of the doc we are editing.
+
   /** create a transaction for setting aAttribute to aValue on aElement
     */
   NS_IMETHOD CreateTxnForSetAttribute(nsIDOMElement *aElement, 
@@ -297,12 +306,17 @@ protected:
   /* Helper for output routines -- we expect subclasses to override this */
   NS_IMETHOD GetWrapWidth(PRInt32* aWrapCol);
 
-// XXXX: Horrible hack! We are doing this because
-// of an error in Gecko which is not rendering the
-// document after a change via the DOM - gpk 2/13/99
-  void HACKForceRedraw(void);
-
-  NS_IMETHOD ScrollIntoView(PRBool aScrollToBegin);
+  /** helper method for scrolling the selection into view after
+   *  an edit operation. aScrollToAnchor should be PR_TRUE if you
+   *  want to scroll to the point where the selection was started.
+   *  If PR_FALSE, it attempts to scroll the end of the selection into view.
+   *
+   *  Editor methods *should* call this method instead of the versions
+   *  in the various selection interfaces, since this version makes sure
+   *  that the editor's sync/async settings for reflowing, painting, and
+   *  scrolling match.
+   */
+  NS_IMETHOD ScrollSelectionIntoView(PRBool aScrollToAnchor);
 
   // stub.  see comment in source.                     
   virtual PRBool IsBlockNode(nsIDOMNode *aNode);
@@ -438,11 +452,21 @@ public:
     * return nsnull if aCurrentNode has no children.
     */
   nsCOMPtr<nsIDOMNode> GetLeftmostChild(nsIDOMNode  *aCurrentNode, 
-                                        PRBool       bNoBlockCrossing = PR_FALSE);
+                                         PRBool      bNoBlockCrossing = PR_FALSE);
 
   /** returns PR_TRUE if aNode is of the type implied by aTag */
-  static PRBool NodeIsType(nsIDOMNode *aNode, nsIAtom *aTag);
-  static PRBool NodeIsType(nsIDOMNode *aNode, const nsAString &aTag);
+  static inline PRBool NodeIsType(nsIDOMNode *aNode, nsIAtom *aTag)
+  {
+    return GetTag(aNode) == aTag;
+  }
+
+  // we should get rid of this method if we can
+  static inline PRBool NodeIsTypeString(nsIDOMNode *aNode, const nsAString &aTag)
+  {
+    nsIAtom *nodeAtom = GetTag(aNode);
+    return nodeAtom && nodeAtom->Equals(aTag);
+  }
+
 
   /** returns PR_TRUE if aParent can contain a child of type aTag */
   PRBool CanContainTag(nsIDOMNode* aParent, const nsAString &aTag);
@@ -461,6 +485,8 @@ public:
   /** returns PR_TRUE if aNode is an editable node */
   PRBool IsEditable(nsIDOMNode *aNode);
 
+  virtual PRBool IsTextInDirtyFrameVisible(nsIDOMNode *aNode);
+
   /** returns PR_TRUE if aNode is a MozEditorBogus node */
   PRBool IsMozEditorBogusNode(nsIDOMNode *aNode);
 
@@ -476,10 +502,13 @@ public:
   nsresult GetLastEditableNode(nsIDOMNode *aRoot, nsCOMPtr<nsIDOMNode> *outLastNode);
 #endif
 
+  nsresult GetIMEBufferLength(PRInt32* length);
+  PRBool   IsIMEComposing();    /* test if IME is in composition state */
+  void     SetIsIMEComposing(); /* call this before |IsIMEComposing()| */
 
   /** from html rules code - migration in progress */
   static nsresult GetTagString(nsIDOMNode *aNode, nsAString& outString);
-  static nsCOMPtr<nsIAtom> GetTag(nsIDOMNode *aNode);
+  static nsIAtom *GetTag(nsIDOMNode *aNode);
   virtual PRBool NodesSameType(nsIDOMNode *aNode1, nsIDOMNode *aNode2);
   static PRBool IsTextOrElementNode(nsIDOMNode *aNode);
   static PRBool IsTextNode(nsIDOMNode *aNode);
@@ -489,6 +518,9 @@ public:
   
   static nsresult GetStartNodeAndOffset(nsISelection *aSelection, nsCOMPtr<nsIDOMNode> *outStartNode, PRInt32 *outStartOffset);
   static nsresult GetEndNodeAndOffset(nsISelection *aSelection, nsCOMPtr<nsIDOMNode> *outEndNode, PRInt32 *outEndOffset);
+#if DEBUG_JOE
+  static void DumpNode(nsIDOMNode *aNode, PRInt32 indent=0);
+#endif
 
   // Helpers to add a node to the selection. 
   // Used by table cell selection methods
@@ -515,7 +547,7 @@ public:
   nsresult GetString(const nsAString& name, nsAString& value);
 
   nsresult BeginUpdateViewBatch(void);
-  nsresult EndUpdateViewBatch(void);
+  virtual nsresult EndUpdateViewBatch(void);
 
   PRBool GetShouldTxnSetSelection();
 
@@ -536,7 +568,6 @@ protected:
   nsIViewManager *mViewManager;
   PRInt32         mUpdateCount;
   nsCOMPtr<nsITransactionManager> mTxnMgr;
-  nsCOMPtr<nsICSSStyleSheet> mLastStyleSheet;			// is owning this dangerous?
   nsWeakPtr         mPlaceHolderTxn;     // weak reference to placeholder for begin/end batch purposes
   nsIAtom          *mPlaceHolderName;    // name of placeholder transaction
   PRInt32           mPlaceHolderBatch;   // nesting count for batching
@@ -549,11 +580,13 @@ protected:
   EDirection        mDirection;          // the current direction of editor action
   
   // data necessary to build IME transactions
-  PRBool						mInIMEMode;          // are we inside an IME composition?
-  nsIPrivateTextRangeList*      mIMETextRangeList;   // IME special selection ranges
-  nsCOMPtr<nsIDOMCharacterData> mIMETextNode;        // current IME text node
-  PRUint32						mIMETextOffset;      // offset in text node where IME comp string begins
-  PRUint32						mIMEBufferLength;    // current length of IME comp string
+  PRBool                        mInIMEMode;        // are we inside an IME composition?
+  nsIPrivateTextRangeList*      mIMETextRangeList; // IME special selection ranges
+  nsCOMPtr<nsIDOMCharacterData> mIMETextNode;      // current IME text node
+  PRUint32                      mIMETextOffset;    // offset in text node where IME comp string begins
+  PRUint32                      mIMEBufferLength;  // current length of IME comp string
+  PRBool                        mIsIMEComposing;   // is IME in composition state?
+                                                   // This is different from mInIMEMode. see Bug 98434.
 
   // various listeners
   nsVoidArray*                  mActionListeners;  // listens to all low level actions on the doc
@@ -564,12 +597,15 @@ protected:
   nsWeakPtr        mDocWeak;  // weak reference to the nsIDOMDocument
   nsCOMPtr<nsIDTD> mDTD;
 
+  nsString* mPhonetic;
+
   static PRInt32 gInstanceCount;
 
   friend PRBool NSCanUnload(nsISupports* serviceMgr);
   friend class nsAutoTxnsConserveSelection;
   friend class nsAutoSelectionReset;
   friend class nsAutoRules;
+  friend class nsRangeUpdater;
 };
 
 
