@@ -48,36 +48,13 @@
 
 nsAOLCiter::nsAOLCiter()
 {
-  NS_INIT_REFCNT();
 }
 
 nsAOLCiter::~nsAOLCiter()
 {
 }
 
-NS_IMPL_ADDREF(nsAOLCiter)
-
-NS_IMPL_RELEASE(nsAOLCiter)
-
-NS_IMETHODIMP
-nsAOLCiter::QueryInterface(REFNSIID aIID, void** aInstancePtr)
-{
-  if (nsnull == aInstancePtr) {
-    return NS_ERROR_NULL_POINTER;
-  }
-  if (aIID.Equals(NS_GET_IID(nsISupports)))
-  {
-    *aInstancePtr = (void*)this;
-    NS_ADDREF_THIS();
-    return NS_OK;
-  }
-  if (aIID.Equals(NS_GET_IID(nsICiter))) {
-    *aInstancePtr = (void*)(nsICiter*)this;
-    NS_ADDREF_THIS();
-    return NS_OK;
-  }
-  return NS_NOINTERFACE;
-}
+NS_IMPL_ISUPPORTS1(nsAOLCiter, nsICiter)
 
 NS_IMETHODIMP
 nsAOLCiter::GetCiteString(const nsAString& aInString, nsAString& aOutString)
@@ -89,14 +66,11 @@ nsAOLCiter::GetCiteString(const nsAString& aInString, nsAString& aOutString)
   PRUnichar newline ('\n');
   if (aOutString.Last() == newline)
   {
-    aOutString.Append(PRUnichar(' '));
-    aOutString.Append(NS_LITERAL_STRING("<<\n"));
-  }
-  else
-  {
-    aOutString.Append(NS_LITERAL_STRING(" <<\n"));
+    aOutString.SetLength(aOutString.Length() - 1);
   }
 
+  aOutString.Append(NS_LITERAL_STRING(" <<\n"));
+
   return NS_OK;
 }
 
@@ -105,18 +79,18 @@ nsAOLCiter::StripCites(const nsAString& aInString, nsAString& aOutString)
 {
   // Remove the beginning cites, if any:
   nsAutoString tOutputString;
-  nsReadingIterator <PRUnichar> iter,enditer;
+  nsReadingIterator <PRUnichar> iter, enditer;
   aInString.BeginReading(iter);
   aInString.EndReading(enditer);
-  if (Substring(aInString,0,2).Equals(NS_LITERAL_STRING(">>")))
+  if (StringBeginsWith(aInString, NS_LITERAL_STRING(">>")))
   {
     iter.advance(2);
     while (nsCRT::IsAsciiSpace(*iter))
       ++iter;
-    AppendUnicodeTo(iter,enditer,tOutputString);
+    AppendUnicodeTo(iter, enditer, tOutputString);
   }
   else
-    CopyUnicodeTo(iter,enditer,tOutputString);
+    CopyUnicodeTo(iter, enditer, tOutputString);
 
   // Remove the end cites, if any:
   tOutputString.Trim("<", PR_FALSE, PR_TRUE, PR_FALSE);
