@@ -83,7 +83,7 @@ public:
   NS_IMETHOD SetColor(nscolor aColor);
   NS_IMETHOD GetColor(nscolor &aColor) const;
 
-  NS_IMETHOD SetFont(const nsFont& aFont);
+  NS_IMETHOD SetFont(const nsFont& aFont, nsIAtom* aLangGroup);
   NS_IMETHOD SetFont(nsIFontMetrics *aFontMetrics);
 
   NS_IMETHOD GetFontMetrics(nsIFontMetrics *&aFontMetrics);
@@ -92,7 +92,7 @@ public:
   NS_IMETHOD Scale(float aSx, float aSy);
   NS_IMETHOD GetCurrentTransform(nsTransform2D *&aTransform);
 
-  NS_IMETHOD CreateDrawingSurface(nsRect *aBounds, PRUint32 aSurfFlags, nsDrawingSurface &aSurface);
+  NS_IMETHOD CreateDrawingSurface(const nsRect& aBounds, PRUint32 aSurfFlags, nsDrawingSurface &aSurface);
   NS_IMETHOD DestroyDrawingSurface(nsDrawingSurface aDS);
 
   NS_IMETHOD DrawLine(nscoord aX0, nscoord aY0, nscoord aX1, nscoord aY1);
@@ -178,12 +178,6 @@ public:
                         PRInt32 aFontID,
                         const nscoord* aSpacing);
 
-  NS_IMETHOD DrawImage(nsIImage *aImage, nscoord aX, nscoord aY);
-  NS_IMETHOD DrawImage(nsIImage *aImage, nscoord aX, nscoord aY,
-                       nscoord aWidth, nscoord aHeight); 
-  NS_IMETHOD DrawImage(nsIImage *aImage, const nsRect& aRect);
-  NS_IMETHOD DrawImage(nsIImage *aImage, const nsRect& aSRect, const nsRect& aDRect);
-
   NS_IMETHOD CopyOffScreenBits(nsDrawingSurface aSrcSurf, PRInt32 aSrcX, PRInt32 aSrcY,
                                const nsRect &aDestBounds, PRUint32 aCopyFlags);
   //~~~
@@ -209,23 +203,13 @@ public:
                      PRInt32*           aFontID);
 #endif
 
-#ifdef IBMBIDI
   NS_IMETHOD SetRightToLeftText(PRBool aIsRTL);
-#endif // IBMBIDI
 
 protected:
   void SetupFontAndColor(void);
 
   ~nsRenderingContextWin();
 
-  /** ---------------------------------------------------
-   *  See documentation in nsIRenderingContextImpl.h
-   *	@update 4/01/00 dwc
-   */
-  virtual PRBool CanTile(nscoord aWidth,nscoord aHeight);
-
-
-
 private:
    // ConditionRect is used to fix a coordinate overflow problem under WIN95. 
   void ConditionRect(nsRect& aSrcRect, RECT& aDestRect);
@@ -238,15 +222,14 @@ private:
   HPEN SetupDashedPen(void);
   HPEN SetupDottedPen(void);
   void PushClipState(void);
-#ifdef IBMBIDI
   void InitBidiInfo(void);
-#endif // IBMBIDI
 
 friend class nsNativeThemeWin;
 
 protected:
-  nscolor					  mCurrentColor;
-  nsIFontMetrics	  *mFontMetrics;
+  nscolor            mCurrentColor;
+  // current font-metrics set in this RC
+  nsIFontMetrics    *mFontMetrics;
   HDC               mDC;
   HDC               mMainDC;
   nsDrawingSurfaceWin *mSurface;
@@ -267,7 +250,11 @@ protected:
   nsVoidArray       *mStateCache;
   nscolor           mCurrBrushColor;
   HBRUSH            mCurrBrush;
-  nsIFontMetrics    *mCurrFontMetrics;
+  // mFontMetrics owns mCurrFontWin which is a thin wrapper
+  // around mCurrFont (the actual GDI font handle). These variables
+  // allow us to quickly tell the current selected font and to
+  // avoid the extra cost of a redundant setup of the same font.
+  nsFontWin         *mCurrFontWin;
   HFONT             mCurrFont;
   nscolor           mCurrPenColor;
   HPEN              mCurrPen;
@@ -277,11 +264,9 @@ protected:
   nsLineStyle       mCurrLineStyle;
 
 #ifdef NS_DEBUG
-  PRBool            mInitialized;
+  PRPackedBool      mInitialized;
 #endif
-#ifdef IBMBIDI
-  PRBool            mRightToLeftText;
-#endif // IBMBIDI
+  PRPackedBool      mRightToLeftText;
 };
 
 #endif /* nsRenderingContextWin_h___ */
