@@ -66,29 +66,28 @@ XlibRgbHandle *nsImageXlib::mXlibRgbHandle = nsnull;
 Display       *nsImageXlib::mDisplay       = nsnull;
 
 nsImageXlib::nsImageXlib()
+: mImageBits(nsnull)
+, mAlphaBits(nsnull)
+, mWidth(0)
+, mHeight(0)
+, mDepth(0)
+, mRowBytes(0)
+, mSizeImage(0)
+, mNumBytesPixel(0)
+, mImagePixmap(nsnull)
+, mAlphaPixmap(nsnull)
+, mAlphaDepth(0)
+, mAlphaRowBytes(0)
+, mAlphaValid(PR_FALSE)
+, mIsSpacer(PR_TRUE)
+, mGC(nsnull)
+, mPendingUpdate(PR_FALSE)
+, mDecodedX1(PR_INT32_MAX)
+, mDecodedY1(PR_INT32_MAX)
+, mDecodedX2(0)
+, mDecodedY2(0)
 {
   PR_LOG(ImageXlibLM, PR_LOG_DEBUG, ("nsImageXlib::nsImageXlib()\n"));
-  NS_INIT_REFCNT();
-  mImageBits = nsnull;
-  mAlphaBits = nsnull;
-  mWidth = 0;
-  mHeight = 0;
-  mDepth = 0;
-  mRowBytes = 0;
-  mSizeImage = 0;
-  mNumBytesPixel = 0;
-  mImagePixmap = nsnull;
-  mAlphaPixmap = nsnull;
-  mAlphaDepth = 0;
-  mAlphaRowBytes = 0;
-  mAlphaWidth = 0;
-  mAlphaHeight = 0;
-  mAlphaValid = PR_FALSE;
-  mIsSpacer = PR_TRUE;
-  mGC = nsnull;
-  mNaturalWidth = 0;
-  mNaturalHeight = 0;
-  mPendingUpdate = PR_FALSE;
 
   if (!mXlibRgbHandle) {
     mXlibRgbHandle = xxlib_find_handle(XXLIBRGB_DEFAULT_HANDLE);
@@ -155,37 +154,15 @@ nsImageXlib::~nsImageXlib()
   
 }
 
-NS_IMPL_ISUPPORTS1(nsImageXlib, nsIImage);
+NS_IMPL_ISUPPORTS1(nsImageXlib, nsIImage)
 
 nsresult nsImageXlib::Init(PRInt32 aWidth, PRInt32 aHeight,
                            PRInt32 aDepth, nsMaskRequirements aMaskRequirements)
 {
+  // gfxImageFrame makes sure nsImageXlib::Init gets called only once
   if ((aWidth == 0) || (aHeight == 0))
     return NS_ERROR_FAILURE;
 
-  if (nsnull != mImageBits) {
-    delete[] mImageBits;
-    mImageBits = nsnull;
-  }
-  
-  if (nsnull != mAlphaBits) {
-    delete[] mAlphaBits;
-    mAlphaBits = nsnull;;
-  }
-  if (nsnull != mAlphaPixmap) {
-    XFreePixmap(mDisplay, mAlphaPixmap);
-    mAlphaPixmap = nsnull;
-  }
-
-  SetDecodedRect(0,0,0,0);
-  SetNaturalWidth(0);
-  SetNaturalHeight(0);
-
-  if (nsnull != mImagePixmap) {
-    XFreePixmap(mDisplay, mImagePixmap);
-    mImagePixmap = nsnull;
-  }
-
   if (24 == aDepth) {
     mNumBytesPixel = 3;
   } else {
@@ -196,7 +173,6 @@ nsresult nsImageXlib::Init(PRInt32 aWidth, PRInt32 aHeight,
   mWidth = aWidth;
   mHeight = aHeight;
   mDepth = aDepth;
-  mIsTopToBottom = PR_TRUE;
 
   // Create the memory for the image
   ComputeMetrics();
@@ -204,12 +180,6 @@ nsresult nsImageXlib::Init(PRInt32 aWidth, PRInt32 aHeight,
   mImageBits = (PRUint8*)new PRUint8[mSizeImage];
 
   switch(aMaskRequirements) {
-    case nsMaskRequirements_kNoMask:
-      mAlphaBits = nsnull;
-      mAlphaWidth = 0;
-      mAlphaHeight = 0;
-      break;
-
     case nsMaskRequirements_kNeeds1Bit:
       mAlphaRowBytes = (aWidth  + 7) / 8;
       mAlphaDepth = 1;
@@ -218,8 +188,6 @@ nsresult nsImageXlib::Init(PRInt32 aWidth, PRInt32 aHeight,
       mAlphaRowBytes = (mAlphaRowBytes + 3) & ~0x3;
 
       mAlphaBits = new unsigned char[mAlphaRowBytes * aHeight];
-      mAlphaWidth = aWidth;
-      mAlphaHeight = aHeight;
       break;
 
     case nsMaskRequirements_kNeeds8Bit:
@@ -229,9 +197,10 @@ nsresult nsImageXlib::Init(PRInt32 aWidth, PRInt32 aHeight,
       // 32-bit align each row
       mAlphaRowBytes = (mAlphaRowBytes + 3) & ~0x3;
       mAlphaBits = new unsigned char[mAlphaRowBytes * aHeight];
-      mAlphaWidth = aWidth;
-      mAlphaHeight = aHeight;
       break;
+
+    default:
+      break; // avoid compiler warning
   }
   return NS_OK;
 }
@@ -268,49 +237,16 @@ nsColorMap *nsImageXlib::GetColorMap()
   return nsnull;
 }
 
-PRBool nsImageXlib::IsOptimized()
-{
-  return PR_TRUE;
-}
-
 PRUint8 *nsImageXlib::GetAlphaBits()
 {
   return mAlphaBits;
 }
 
-PRInt32 nsImageXlib::GetAlphaWidth()
-{
-  return mAlphaWidth;
-}
-
-PRInt32 nsImageXlib::GetAlphaHeight()
-{
-  return mAlphaHeight;
-}
-
 PRInt32 nsImageXlib::GetAlphaLineStride()
 {
   return mAlphaRowBytes;
 }
 
-nsIImage *nsImageXlib::DuplicateImage()
-{
-  return nsnull;
-}
-
-void nsImageXlib::SetAlphaLevel(PRInt32 aAlphaLevel)
-{
-}
-
-PRInt32 nsImageXlib::GetAlphaLevel()
-{
-  return 0;
-}
-
-void nsImageXlib::MoveAlphaMask(PRInt32 aX, PRInt32 aY)
-{
-}
-
 //-----------------------------------------------------------------------
 
 // Set up the palette to the passed in color array, RGB only in this array
@@ -320,6 +256,14 @@ void nsImageXlib::ImageUpdated(nsIDeviceContext *aContext,
 {
   mPendingUpdate = PR_TRUE;
   mUpdateRegion.Or(mUpdateRegion, *aUpdateRect);
+
+  mDecodedX1 = PR_MIN(mDecodedX1, aUpdateRect->x);
+  mDecodedY1 = PR_MIN(mDecodedY1, aUpdateRect->y);
+
+  if (aUpdateRect->YMost() > mDecodedY2)
+    mDecodedY2 = aUpdateRect->YMost();
+  if (aUpdateRect->XMost() > mDecodedX2)
+    mDecodedX2 = aUpdateRect->XMost();
 }
 
 void nsImageXlib::UpdateCachedImage()
@@ -433,7 +377,7 @@ void nsImageXlib::UpdateCachedImage()
     }
   }
 
-  mUpdateRegion.Empty();
+  mUpdateRegion.SetEmpty();
   mPendingUpdate = PR_FALSE;
   mFlags = nsImageUpdateFlags_kBitsChanged; // this should be 0'd out by Draw()
 }
@@ -456,6 +400,9 @@ nsImageXlib::DrawScaled(nsIRenderingContext &aContext,
   if (0 == aSWidth || 0 == aDWidth || 0 == aSHeight || 0 == aDHeight)
     return NS_OK;
 
+  if (mDecodedX2 < mDecodedX1 || mDecodedY2 < mDecodedY1)
+    return NS_OK;
+
   // limit the size of the blit to the amount of the image read in
   if (aSX + aSWidth > mDecodedX2) {
     aDWidth -= ((aSX + aSWidth - mDecodedX2)*origDWidth)/origSWidth;
@@ -524,9 +471,9 @@ nsImageXlib::DrawScaled(nsIRenderingContext &aContext,
     // creation fails for some reason.  thus no easy-out "return"
     if (scaledAlpha) {
       memset(scaledAlpha, 0, aDHeight*scaledRowBytes);
-      RectStretch(aSX, aSY, aSX+aSWidth-1, aSY+aSHeight-1,
-          0, 0, aDWidth-1, aDHeight-1,
-          mAlphaBits, mAlphaRowBytes, scaledAlpha, scaledRowBytes, 1);
+      RectStretch(mWidth, mHeight, origDWidth, origDHeight,
+                  aDX, aDY, aDX + aDWidth - 1, aDY + aDHeight - 1,
+                  mAlphaBits, mAlphaRowBytes, scaledAlpha, scaledRowBytes, 1);
 
       pixmap = XCreatePixmap(mDisplay, DefaultRootWindow(mDisplay),
                              aDWidth, aDHeight, 1);
@@ -586,8 +533,8 @@ nsImageXlib::DrawScaled(nsIRenderingContext &aContext,
 
   PRUint8 *scaledRGB = (PRUint8 *)nsMemory::Alloc(3*aDWidth*aDHeight);
   if (scaledRGB && gc) {
-    RectStretch(aSX, aSY, aSX+aSWidth-1, aSY+aSHeight-1,
-                0, 0, aDWidth-1, aDHeight-1,
+    RectStretch(mWidth, mHeight, origDWidth, origDHeight,
+                aDX, aDY, aDX + aDWidth - 1, aDY + aDHeight - 1,
                 mImageBits, mRowBytes, scaledRGB, 3*aDWidth, 24);
 
     Drawable drawable; drawing->GetDrawable(drawable);
@@ -626,6 +573,9 @@ nsImageXlib::Draw(nsIRenderingContext &aContext, nsDrawingSurface aSurface,
   if ((mAlphaDepth == 1) && mIsSpacer)
     return NS_OK;
 
+  if (mDecodedX2 < mDecodedX1 || mDecodedY2 < mDecodedY1)
+    return NS_OK;
+
   if (aSWidth != aDWidth || aSHeight != aDHeight) {
     return DrawScaled(aContext, aSurface, aSX, aSY, aSWidth, aSHeight,
                       aDX, aDY, aDWidth, aDHeight);
@@ -1096,11 +1046,11 @@ nsImageXlib::DrawComposited(nsIRenderingContext &aContext,
         nsMemory::Free(scaledAlpha);
       return;
     }
-    RectStretch(x1, y1, x2-1, y2-1,
-                0, 0, readWidth-1, readHeight-1,
+    RectStretch(aSWidth, aSHeight, aDWidth, aDHeight,
+                0, 0, aDWidth-1, aDHeight-1,
                 mImageBits, mRowBytes, scaledImage, 3*readWidth, 24);
     RectStretch(x1, y1, x2-1, y2-1,
-                0, 0, readWidth-1, readHeight-1,
+                0, 0, aDWidth-1, aDHeight-1,
                 mAlphaBits, mAlphaRowBytes, scaledAlpha, readWidth, 8);
     imageOrigin = scaledImage;
     imageStride = 3*readWidth;
@@ -1270,6 +1220,9 @@ nsImageXlib::Draw(nsIRenderingContext &aContext,
   if (aSurface == nsnull)
     return NS_ERROR_FAILURE;
 
+  if (mDecodedX2 < mDecodedX1 || mDecodedY2 < mDecodedY1)
+    return NS_OK;
+
   if ((mAlphaDepth == 8) && mAlphaValid) {
     DrawComposited(aContext, aSurface,
         0, 0, aWidth, aHeight,
@@ -1379,6 +1332,7 @@ void nsImageXlib::TilePixmap(Pixmap src, Pixmap dest, PRInt32 aSXOffset,
 NS_IMETHODIMP nsImageXlib::DrawTile(nsIRenderingContext &aContext,
                                     nsDrawingSurface aSurface,
                                     PRInt32 aSXOffset, PRInt32 aSYOffset,
+                                    PRInt32 aPadX, PRInt32 aPadY,
                                     const nsRect &aTileRect)
 {
   if (mPendingUpdate)
@@ -1390,7 +1344,10 @@ NS_IMETHODIMP nsImageXlib::DrawTile(nsIRenderingContext &aContext,
   if (aTileRect.width <= 0 || aTileRect.height <= 0) {
     return NS_OK;
   }
-  
+
+  if (mDecodedX2 < mDecodedX1 || mDecodedY2 < mDecodedY1)
+    return NS_OK;
+
   nsIDrawingSurfaceXlib *drawing = NS_STATIC_CAST(nsIDrawingSurfaceXlib *, aSurface);
 
   PRBool partial = PR_FALSE;
@@ -1426,7 +1383,7 @@ NS_IMETHODIMP nsImageXlib::DrawTile(nsIRenderingContext &aContext,
     return NS_OK;
   }
    
-  if (partial || ((mAlphaDepth == 8) && mAlphaValid)) {
+  if (partial || ((mAlphaDepth == 8) && mAlphaValid) || (aPadX || aPadY)) {
     PRInt32 aY0 = aTileRect.y - aSYOffset,
             aX0 = aTileRect.x - aSXOffset,
             aY1 = aTileRect.y + aTileRect.height,
@@ -1435,11 +1392,11 @@ NS_IMETHODIMP nsImageXlib::DrawTile(nsIRenderingContext &aContext,
     // Set up clipping and call Draw().
     PRBool clipState;
     aContext.PushState();
-    aContext.SetClipRect(aTileRect, nsClipCombine_kIntersect,
-                         clipState);
-
-    for (PRInt32 y = aY0; y < aY1; y += mHeight)
-      for (PRInt32 x = aX0; x < aX1; x += mWidth)
+    ((nsRenderingContextXlib&)aContext).SetClipRectInPixels(
+      aTileRect, nsClipCombine_kIntersect, clipState);
+    ((nsRenderingContextXlib&)aContext).UpdateGC();
+    for (PRInt32 y = aY0; y < aY1; y += mHeight + aPadY)
+      for (PRInt32 x = aX0; x < aX1; x += mWidth + aPadX)
         Draw(aContext,aSurface, x, y,
              PR_MIN(validWidth, aX1 - x),
              PR_MIN(validHeight, aY1 - y));
@@ -1533,19 +1490,6 @@ nsImageXlib::UnlockImagePixels(PRBool aMaskPixels)
   return NS_OK;
 }
 
-//---------------------------------------------------------------------
-// Set the decoded dimens of the image
-NS_IMETHODIMP
-nsImageXlib::SetDecodedRect(PRInt32 x1, PRInt32 y1, PRInt32 x2, PRInt32 y2)
-{
-  mDecodedX1 = x1;
-  mDecodedY1 = y1;
-  mDecodedX2 = x2;
-  mDecodedY2 = y2;
-
-  return NS_OK;
-}
-
 NS_IMETHODIMP nsImageXlib::DrawToImage(nsIImage* aDstImage,
                                        nscoord aDX, nscoord aDY,
                                        nscoord aDWidth, nscoord aDHeight)
@@ -1554,6 +1498,9 @@ NS_IMETHODIMP nsImageXlib::DrawToImage(nsIImage* aDstImage,
   if (!dest)
     return NS_ERROR_FAILURE;
 
+  if (aDX >= dest->mWidth || aDY >= dest->mHeight)
+    return NS_OK;
+
   if (mPendingUpdate)
     UpdateCachedImage();
 
@@ -1590,7 +1537,8 @@ NS_IMETHODIMP nsImageXlib::DrawToImage(nsIImage* aDstImage,
     if (!scaledImage)
       return NS_ERROR_OUT_OF_MEMORY;
 
-    RectStretch(0, 0, mWidth-1, mHeight-1, 0, 0, aDWidth-1, aDHeight-1,
+    RectStretch(mWidth, mHeight, aDWidth, aDHeight,
+                0, 0, aDWidth-1, aDHeight-1,
                 mImageBits, mRowBytes, scaledImage, 3*aDWidth, 24);
 
     if (mAlphaDepth) {
@@ -1605,7 +1553,8 @@ NS_IMETHODIMP nsImageXlib::DrawToImage(nsIImage* aDstImage,
         return NS_ERROR_OUT_OF_MEMORY;
       }
 
-      RectStretch(0, 0, mWidth-1, mHeight-1, 0, 0, aDWidth-1, aDHeight-1,
+      RectStretch(mWidth, mHeight, aDWidth, aDHeight,
+                  0, 0, aDWidth-1, aDHeight-1,
                   mAlphaBits, mAlphaRowBytes, scaledAlpha, alphaStride,
                   mAlphaDepth);
     }
@@ -1655,9 +1604,9 @@ NS_IMETHODIMP nsImageXlib::DrawToImage(nsIImage* aDstImage,
           else {
             dstAlpha[(aDX+x)>>3]       |= alphaPixels >> offset;
             // avoid write if no 1's to write - also avoids going past end of array
-            // compiler should merge the common sub-expressions
-            if (alphaPixels << (8U - offset))
-              dstAlpha[((aDX+x)>>3) + 1] |= alphaPixels << (8U - offset);
+            PRUint8 alphaTemp = alphaPixels << (8U - offset);
+            if (alphaTemp & 0xff)
+              dstAlpha[((aDX+x)>>3) + 1] |= alphaTemp;
           }
 
           if (alphaPixels == 0xff) {
