@@ -41,6 +41,7 @@
 #include "nsReadableUtils.h"
 #include "plhash.h"
 #include "nsString.h"
+#include "nsStaticAtom.h"
 
 // C++ sucks! There's no way to do this with a macro, at least not
 // that I know, if you know how to do this with a macro then please do
@@ -145,8 +146,6 @@ static const PRUnichar sHTMLTagUnicodeName_i[] =
   {'i', '\0'};
 static const PRUnichar sHTMLTagUnicodeName_iframe[] =
   {'i', 'f', 'r', 'a', 'm', 'e', '\0'};
-static const PRUnichar sHTMLTagUnicodeName_ilayer[] =
-  {'i', 'l', 'a', 'y', 'e', 'r', '\0'};
 static const PRUnichar sHTMLTagUnicodeName_image[] =
   {'i', 'm', 'a', 'g', 'e', '\0'};
 static const PRUnichar sHTMLTagUnicodeName_img[] =
@@ -163,8 +162,6 @@ static const PRUnichar sHTMLTagUnicodeName_keygen[] =
   {'k', 'e', 'y', 'g', 'e', 'n', '\0'};
 static const PRUnichar sHTMLTagUnicodeName_label[] =
   {'l', 'a', 'b', 'e', 'l', '\0'};
-static const PRUnichar sHTMLTagUnicodeName_layer[] =
-  {'l', 'a', 'y', 'e', 'r', '\0'};
 static const PRUnichar sHTMLTagUnicodeName_legend[] =
   {'l', 'e', 'g', 'e', 'n', 'd', '\0'};
 static const PRUnichar sHTMLTagUnicodeName_li[] =
@@ -175,6 +172,8 @@ static const PRUnichar sHTMLTagUnicodeName_listing[] =
   {'l', 'i', 's', 't', 'i', 'n', 'g', '\0'};
 static const PRUnichar sHTMLTagUnicodeName_map[] =
   {'m', 'a', 'p', '\0'};
+static const PRUnichar sHTMLTagUnicodeName_marquee[] =
+  {'m', 'a', 'r', 'q', 'u', 'e', 'e', '\0'};
 static const PRUnichar sHTMLTagUnicodeName_menu[] =
   {'m', 'e', 'n', 'u', '\0'};
 static const PRUnichar sHTMLTagUnicodeName_meta[] =
@@ -187,8 +186,6 @@ static const PRUnichar sHTMLTagUnicodeName_noembed[] =
   {'n', 'o', 'e', 'm', 'b', 'e', 'd', '\0'};
 static const PRUnichar sHTMLTagUnicodeName_noframes[] =
   {'n', 'o', 'f', 'r', 'a', 'm', 'e', 's', '\0'};
-static const PRUnichar sHTMLTagUnicodeName_nolayer[] =
-  {'n', 'o', 'l', 'a', 'y', 'e', 'r', '\0'};
 static const PRUnichar sHTMLTagUnicodeName_noscript[] =
   {'n', 'o', 's', 'c', 'r', 'i', 'p', 't', '\0'};
 static const PRUnichar sHTMLTagUnicodeName_object[] =
@@ -272,23 +269,24 @@ static const PRUnichar sHTMLTagUnicodeName_wbr[] =
 static const PRUnichar sHTMLTagUnicodeName_xmp[] =
   {'x', 'm', 'p', '\0'};
 
-
 // static array of unicode tag names
-#define HTML_TAG(_tag) sHTMLTagUnicodeName_##_tag,
-static const PRUnichar* kTagUnicodeTable[] = {
+#define HTML_TAG(_tag, _classname) sHTMLTagUnicodeName_##_tag,
+#define HTML_OTHER(_tag, _classname)
+static const PRUnichar* const kTagUnicodeTable[] = {
 #include "nsHTMLTagList.h"
 };
 #undef HTML_TAG
 
+// static array of tag atoms
+static nsIAtom* kTagAtomTable[eHTMLTag_userdefined - 1];
 
-#ifdef DEBUG
-// static array of ASCII tag names for debugging purposes
-#define HTML_TAG(_tag) #_tag,
-static const char* kTagASCIIDebugTable[] = {
+// static array of tag StaticAtom structs
+#define HTML_TAG(_tag, _classname) { #_tag, &kTagAtomTable[eHTMLTag_##_tag - 1] },
+static const nsStaticAtom kTagAtoms_info[] = {
 #include "nsHTMLTagList.h"
 };
 #undef HTML_TAG
-#endif
+#undef HTML_OTHER
 
 static PRInt32 gTableRefCount;
 static PLHashTable* gTagTable;
@@ -346,12 +344,15 @@ nsHTMLTags::AddRefTable(void)
     NS_ASSERTION(sMaxTagNameLength == NS_HTMLTAG_NAME_MAX_LENGTH,
                  "NS_HTMLTAG_NAME_MAX_LENGTH not set correctly!");
 
+    // Fill in our static atom pointers
+    NS_RegisterStaticAtoms(kTagAtoms_info, NS_ARRAY_LENGTH(kTagAtoms_info));
+
 #ifdef DEBUG
     {
       // let's verify that all names in the the table are lowercase...
       for (i = 0; i < NS_HTML_TAG_MAX; ++i) {
-        nsCAutoString temp1(kTagASCIIDebugTable[i]);
-        nsCAutoString temp2(kTagASCIIDebugTable[i]);
+        nsCAutoString temp1(kTagAtoms_info[i].mString);
+        nsCAutoString temp2(kTagAtoms_info[i].mString);
         ToLowerCase(temp1);
         NS_ASSERTION(temp1.Equals(temp2), "upper case char in table");
       }
@@ -360,7 +361,7 @@ nsHTMLTags::AddRefTable(void)
       // correct.
       for (i = 0; i < NS_HTML_TAG_MAX; ++i) {
         nsAutoString temp1(kTagUnicodeTable[i]);
-        nsAutoString temp2; temp2.AssignWithConversion(kTagASCIIDebugTable[i]);
+        nsAutoString temp2; temp2.AssignWithConversion(kTagAtoms_info[i].mString);
         NS_ASSERTION(temp1.Equals(temp2), "Bad unicode tag name!");
       }
     }
@@ -445,11 +446,11 @@ nsHTMLTags::LookupTag(const nsAString& aTagName)
   if (tag == eHTMLTag_unknown) {
     // "__moz_text"
     static const PRUnichar moz_text[] =
-      {'_', '_', 'm', 'o', 'z', '_', 't', 'e', 'x', 't'};
+      {'_', '_', 'm', 'o', 'z', '_', 't', 'e', 'x', 't', PRUnichar(0) };
 
     // "#text"
     static const PRUnichar text[] =
-      {'#', 't', 'e', 'x', 't'};
+      {'#', 't', 'e', 'x', 't', PRUnichar(0) };
 
     if (nsCRT::strcmp(buf, moz_text) == 0) {
       tag = eHTMLTag_text;
@@ -474,6 +475,17 @@ nsHTMLTags::GetStringValue(nsHTMLTag aEnum)
   return kTagUnicodeTable[aEnum - 1];
 }
 
+// static
+nsIAtom *
+nsHTMLTags::GetAtom(nsHTMLTag aEnum)
+{
+  if (aEnum <= eHTMLTag_unknown || aEnum > NS_HTML_TAG_MAX) {
+    return nsnull;
+  }
+
+  return kTagAtomTable[aEnum - 1];
+}
+
 
 #ifdef NS_DEBUG
 
@@ -487,7 +499,7 @@ public:
 
      nsHTMLTags::AddRefTable();
      // Make sure we can find everything we are supposed to
-     for (int i = 0; i < NS_HTML_TAG_MAX; i++) {
+     for (int i = 0; i < NS_HTML_TAG_MAX; ++i) {
        tag = kTagUnicodeTable[i];
        id = nsHTMLTags::LookupTag(nsDependentString(tag));
        NS_ASSERTION(id != eHTMLTag_userdefined, "can't find tag id");
@@ -512,6 +524,6 @@ public:
    }
 };
 
-nsTestTagTable validateTagTable;
+static const nsTestTagTable validateTagTable;
 
 #endif
