@@ -37,6 +37,7 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
+#include "nsIAtom.h"
 #include "nsString.h"
 #include "nspr.h"
 #include "nsCOMPtr.h"
@@ -46,13 +47,16 @@
 #include "nsParser.h"
 #include "CNavDTD.h"
 #include "COtherDTD.h"
-#include "nsViewSourceHTML.h"
 #include "nsHTMLEntities.h"
 #include "nsHTMLTokenizer.h"
 //#include "nsTextTokenizer.h"
 #include "nsElementTable.h"
 #include "nsParserService.h"
 
+#ifdef MOZ_VIEW_SOURCE
+#include "nsViewSourceHTML.h"
+#endif
+
 #ifdef NS_DEBUG
 #include "nsLoggingSink.h"
 #include "nsExpatDriver.h"
@@ -68,9 +72,12 @@ NS_GENERIC_FACTORY_CONSTRUCTOR(nsExpatDriver)
 NS_GENERIC_FACTORY_CONSTRUCTOR(nsParser)
 NS_GENERIC_FACTORY_CONSTRUCTOR(CNavDTD)
 NS_GENERIC_FACTORY_CONSTRUCTOR(CTransitionalDTD)
-NS_GENERIC_FACTORY_CONSTRUCTOR(CViewSourceHTML)
 NS_GENERIC_FACTORY_CONSTRUCTOR(nsParserService)
 
+#ifdef MOZ_VIEW_SOURCE
+NS_GENERIC_FACTORY_CONSTRUCTOR(CViewSourceHTML)
+#endif
+
 static const nsModuleComponentInfo gComponents[] = {
 
 #ifdef NS_DEBUG
@@ -82,8 +89,14 @@ static const nsModuleComponentInfo gComponents[] = {
   { "Navigator HTML DTD", NS_CNAVDTD_CID, NULL, CNavDTDConstructor },
   { "Transitional DTD", NS_CTRANSITIONAL_DTD_CID, NULL,
     CTransitionalDTDConstructor },
+#ifdef MOZ_VIEW_SOURCE
   { "ViewSource DTD", NS_VIEWSOURCE_DTD_CID, NULL, CViewSourceHTMLConstructor },
-  { "ParserService", NS_PARSERSERVICE_CID, NULL, nsParserServiceConstructor }
+#endif
+  { "ParserService",
+    NS_PARSERSERVICE_CID,
+    NS_PARSER_CONTRACTID_PREFIX "/parser-service;1",
+    nsParserServiceConstructor
+  }
 };
 
 static PRBool gInitialized = PR_FALSE;
@@ -95,12 +108,17 @@ Initialize(nsIModule* aSelf)
     nsresult rv = nsHTMLTags::AddRefTable();
     NS_ENSURE_SUCCESS(rv, rv);
 
-    nsHTMLEntities::AddRefTable();
+    rv = nsHTMLEntities::AddRefTable();
+    if (NS_FAILED(rv)) {
+      nsHTMLTags::ReleaseTable();
+      return rv;
+    }
     InitializeElementTable();
     CNewlineToken::AllocNewline();
     gInitialized = PR_TRUE;
   }
-  return NS_OK;
+
+  return nsParser::Init();
 }
 
 PR_STATIC_CALLBACK(void)
@@ -110,7 +128,7 @@ Shutdown(nsIModule* aSelf)
     nsHTMLTags::ReleaseTable();
     nsHTMLEntities::ReleaseTable();
     nsDTDContext::ReleaseGlobalObjects();
-    nsParser::FreeSharedObjects();
+    nsParser::Shutdown();
     DeleteElementTable();
     CNewlineToken::FreeNewline();
     gInitialized = PR_FALSE;
