@@ -42,7 +42,6 @@
   {0x80, 0x1b, 0x0, 0x60, 0x8, 0xbf, 0xc4, 0x89 }}
 
 
-class nsIDTDDebug;
 class nsIParserNode;
 class nsParser;
 class nsITokenizer;
@@ -76,6 +75,11 @@ private:
     nsresult WriteAttributes(PRInt32 attrCount);
     nsresult GenerateSummary();
     void StartNewPreBlock(void);
+    // Utility method for adding attributes to the nodes we generate
+    void AddAttrToNode(nsCParserStartNode& aNode,
+                       nsTokenAllocator* aAllocator,
+                       const nsAString& aAttrName,
+                       const nsAString& aAttrValue);
 
 protected:
 
@@ -97,8 +101,10 @@ protected:
     PRInt32             mValue;
     PRInt32             mPopupTag;
     PRInt32             mSummaryTag;
-    PRBool              mSyntaxHighlight;
-    PRBool              mWrapLongLines;
+    PRPackedBool        mSyntaxHighlight;
+    PRPackedBool        mWrapLongLines;
+    PRPackedBool        mHasOpenRoot;
+    PRPackedBool        mHasOpenBody;
 
     nsDTDMode           mDTDMode;
     eParserCommands     mParserCommand;   //tells us to viewcontent/viewsource/viewerrors...
@@ -110,8 +116,6 @@ protected:
     nsString            mFilename;
     nsString            mTags;
     nsString            mErrors;
-    PRPackedBool        mHasOpenRoot;
-    PRPackedBool        mHasOpenBody;
 
     PRUint32            mTokenCount;
 };
