@@ -23,31 +23,14 @@
 #ifndef nsUnicodeToGEOSTD8_h___
 #define nsUnicodeToGEOSTD8_h___
 
-#include "nsUCvLatinSupport.h"
-
-//----------------------------------------------------------------------
-// Class nsUnicodeToGEOSTD8 [declaration]
+#include "nsISupports.h"
 
 /**
  * A character set converter from Unicode to GEOSTD8.
  *
  */
-class nsUnicodeToGEOSTD8 : public nsTableEncoderSupport
-{
-public:
-
-  /**
-   * Class constructor.
-   */
-  nsUnicodeToGEOSTD8();
-
-protected:
-
-  //--------------------------------------------------------------------
-  // Subclassing of nsEncoderSupport class [declaration]
-
-  NS_IMETHOD GetMaxLength(const PRUnichar * aSrc, PRInt32 aSrcLength, 
-      PRInt32 * aDestLength);
-};
+NS_METHOD
+nsUnicodeToGEOSTD8Constructor(nsISupports *aOuter, REFNSIID aIID,
+                              void **aResult);
 
 #endif /* nsUnicodeToGEOSTD8_h___ */
