@@ -1,36 +1,41 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 4 -*-
  *
- * The contents of this file are subject to the Netscape Public
- * License Version 1.1 (the "License"); you may not use this file
- * except in compliance with the License. You may obtain a copy of
- * the License at http://www.mozilla.org/NPL/
+ * ***** BEGIN LICENSE BLOCK *****
+ * Version: MPL 1.1/GPL 2.0/LGPL 2.1
  *
- * Software distributed under the License is distributed on an "AS
- * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express oqr
- * implied. See the License for the specific language governing
- * rights and limitations under the License.
+ * The contents of this file are subject to the Mozilla Public License Version
+ * 1.1 (the "License"); you may not use this file except in compliance with
+ * the License. You may obtain a copy of the License at
+ * http://www.mozilla.org/MPL/
+ *
+ * Software distributed under the License is distributed on an "AS IS" basis,
+ * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
+ * for the specific language governing rights and limitations under the
+ * License.
  *
  * The Original Code is Mozilla Communicator client code, released
  * March 31, 1998.
  *
- * The Initial Developer of the Original Code is Netscape
- * Communications Corporation.  Portions created by Netscape are
- * Copyright (C) 1998 Netscape Communications Corporation. All
- * Rights Reserved.
+ * The Initial Developer of the Original Code is
+ * Netscape Communications Corporation.
+ * Portions created by the Initial Developer are Copyright (C) 1998
+ * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): 
+ * Contributor(s):
  *
- * Alternatively, the contents of this file may be used under the
- * terms of the GNU Public License (the "GPL"), in which case the
- * provisions of the GPL are applicable instead of those above.
- * If you wish to allow use of your version of this file only
- * under the terms of the GPL and not to allow others to use your
- * version of this file under the NPL, indicate your decision by
- * deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL.  If you do not delete
- * the provisions above, a recipient may use your version of this
- * file under either the NPL or the GPL.
- */
+ * Alternatively, the contents of this file may be used under the terms of
+ * either of the GNU General Public License Version 2 or later (the "GPL"),
+ * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
+ * in which case the provisions of the GPL or the LGPL are applicable instead
+ * of those above. If you wish to allow use of your version of this file only
+ * under the terms of either the GPL or the LGPL, and not to allow others to
+ * use your version of this file under the terms of the MPL, indicate your
+ * decision by deleting the provisions above and replace them with the notice
+ * and other provisions required by the GPL or the LGPL. If you do not delete
+ * the provisions above, a recipient may use your version of this file under
+ * the terms of any one of the MPL, the GPL or the LGPL.
+ *
+ * ***** END LICENSE BLOCK ***** */
 
 /*
  * JS debugging API.
@@ -99,8 +104,8 @@ JS_SetTrap(JSContext *cx, JSScript *script, jsbytecode *pc,
     rt = cx->runtime;
     trap = FindTrap(rt, script, pc);
     if (trap) {
-        /* Restore opcode at pc so it can be saved again. */
-        *pc = (jsbytecode)trap->op;
+        JS_ASSERT(trap->script == script && trap->pc == pc);
+        JS_ASSERT(*pc == JSOP_TRAP);
     } else {
         trap = (JSTrap *) JS_malloc(cx, sizeof *trap);
         if (!trap || !js_AddRoot(cx, &trap->closure, "trap->closure")) {
@@ -108,14 +113,14 @@ JS_SetTrap(JSContext *cx, JSScript *script, jsbytecode *pc,
                 JS_free(cx, trap);
             return JS_FALSE;
         }
+        JS_APPEND_LINK(&trap->links, &rt->trapList);
+        trap->script = script;
+        trap->pc = pc;
+        trap->op = (JSOp)*pc;
+        *pc = JSOP_TRAP;
     }
-    JS_APPEND_LINK(&trap->links, &rt->trapList);
-    trap->script = script;
-    trap->pc = pc;
-    trap->op = (JSOp)*pc;
     trap->handler = handler;
     trap->closure = closure;
-    *pc = JSOP_TRAP;
     return JS_TRUE;
 }
 
@@ -352,7 +357,7 @@ js_watch_set(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
                              vp, wp->closure);
             if (ok) {
                 /*
-                 * Create pseudo-frame for call to setter so that any 
+                 * Create pseudo-frame for call to setter so that any
                  * stack-walking security code in the setter will correctly
                  * identify the guilty party.
                  */
@@ -394,7 +399,7 @@ js_watch_set_wrapper(JSContext *cx, JSObject *obj, uintN argc, jsval *argv,
     return js_watch_set(cx, obj, userid, rval);
 }
 
-extern JSPropertyOp
+JSPropertyOp
 js_WrapWatchedSetter(JSContext *cx, jsid id, uintN attrs, JSPropertyOp setter)
 {
     JSAtom *atom;
@@ -588,7 +593,7 @@ JS_ClearAllWatchPoints(JSContext *cx)
 JS_PUBLIC_API(uintN)
 JS_PCToLineNumber(JSContext *cx, JSScript *script, jsbytecode *pc)
 {
-    return js_PCToLineNumber(script, pc);
+    return js_PCToLineNumber(cx, script, pc);
 }
 
 JS_PUBLIC_API(jsbytecode *)
@@ -633,21 +638,53 @@ JS_GetFramePC(JSContext *cx, JSStackFrame *fp)
     return fp->pc;
 }
 
+JS_PUBLIC_API(JSStackFrame *)
+JS_GetScriptedCaller(JSContext *cx, JSStackFrame *fp)
+{
+    if (!fp)
+        fp = cx->fp;
+    while ((fp = fp->down) != NULL) {
+        if (fp->script)
+            return fp;
+    }
+    return NULL;
+}
+
+JS_PUBLIC_API(JSPrincipals *)
+JS_StackFramePrincipals(JSContext *cx, JSStackFrame *fp)
+{
+    if (fp->fun && cx->findObjectPrincipals) {
+        JSObject *callee = JSVAL_TO_OBJECT(fp->argv[-2]);
+
+        if (fp->fun->object != callee)
+            return cx->findObjectPrincipals(cx, callee);
+        /* FALL THROUGH */
+    }
+    if (fp->script)
+        return fp->script->principals;
+    return NULL;
+}
+
+JS_PUBLIC_API(JSPrincipals *)
+JS_EvalFramePrincipals(JSContext *cx, JSStackFrame *fp, JSStackFrame *caller)
+{
+    if (cx->findObjectPrincipals)
+        return cx->findObjectPrincipals(cx, JSVAL_TO_OBJECT(fp->argv[-2]));
+    if (!caller)
+        return NULL;
+    return JS_StackFramePrincipals(cx, caller);
+}
+
 JS_PUBLIC_API(void *)
 JS_GetFrameAnnotation(JSContext *cx, JSStackFrame *fp)
 {
-    if (fp->annotation) {
-        JSPrincipals *principals = fp->script
-            ? fp->script->principals
-            : NULL;
+    if (fp->annotation && fp->script) {
+        JSPrincipals *principals = JS_StackFramePrincipals(cx, fp);
 
-        if (principals == NULL)
-            return NULL;
-
-        if (principals->globalPrivilegesEnabled(cx, principals)) {
+        if (principals && principals->globalPrivilegesEnabled(cx, principals)) {
             /*
-             * Only give out an annotation if privileges have not
-             * been revoked globally.
+             * Give out an annotation only if privileges have not been revoked
+             * or disabled globally.
              */
             return fp->annotation;
         }
@@ -665,13 +702,12 @@ JS_SetFrameAnnotation(JSContext *cx, JSStackFrame *fp, void *annotation)
 JS_PUBLIC_API(void *)
 JS_GetFramePrincipalArray(JSContext *cx, JSStackFrame *fp)
 {
-    JSPrincipals *principals = fp->script
-        ? fp->script->principals
-        : NULL;
+    JSPrincipals *principals;
 
-    return principals
-        ? principals->getPrincipalArray(cx, principals)
-        : NULL;
+    principals = JS_StackFramePrincipals(cx, fp);
+    if (!principals)
+        return NULL;
+    return principals->getPrincipalArray(cx, principals);
 }
 
 JS_PUBLIC_API(JSBool)
@@ -809,7 +845,7 @@ JS_EvaluateUCInStackFrame(JSContext *cx, JSStackFrame *fp,
                           const char *filename, uintN lineno,
                           jsval *rval)
 {
-    uint32 flags;
+    uint32 flags, options;
     JSScript *script;
     JSBool ok;
 
@@ -819,12 +855,13 @@ JS_EvaluateUCInStackFrame(JSContext *cx, JSStackFrame *fp,
      */
     flags = fp->flags;
     fp->flags |= JSFRAME_DEBUGGER | JSFRAME_EVAL;
+    options = cx->options;
+    cx->options = options | JSOPTION_COMPILE_N_GO;
     script = JS_CompileUCScriptForPrincipals(cx, fp->scopeChain,
-                                             fp->script
-                                             ? fp->script->principals
-                                             : NULL,
+                                             JS_StackFramePrincipals(cx, fp),
                                              bytes, length, filename, lineno);
     fp->flags = flags;
+    cx->options = options;
     if (!script)
         return JS_FALSE;
 
@@ -888,12 +925,42 @@ JS_GetPropertyDesc(JSContext *cx, JSObject *obj, JSScopeProperty *sprop,
     JSPropertyOp getter;
     JSScope *scope;
     JSScopeProperty *aprop;
+    jsval lastException;
+    JSBool wasThrowing;
 
     pd->id = ID_TO_VALUE(sprop->id);
-    if (!js_GetProperty(cx, obj, sprop->id, &pd->value))
-        return JS_FALSE;
+
+    wasThrowing = cx->throwing;
+    if (wasThrowing) {
+        lastException = cx->exception;
+        if (JSVAL_IS_GCTHING(lastException) &&
+            !js_AddRoot(cx, &lastException, "lastException")) {
+                return JS_FALSE;
+        }
+        cx->throwing = JS_FALSE;
+    }
+
+    if (!js_GetProperty(cx, obj, sprop->id, &pd->value)) {
+        if (!cx->throwing) {
+            pd->flags = JSPD_ERROR;
+            pd->value = JSVAL_VOID;
+        } else {
+            pd->flags = JSPD_EXCEPTION;
+            pd->value = cx->exception;
+        }
+    } else {
+        pd->flags = 0;
+    }
+
+    cx->throwing = wasThrowing;
+    if (wasThrowing) {
+        cx->exception = lastException;
+        if (JSVAL_IS_GCTHING(lastException))
+            js_RemoveRoot(cx->runtime, &lastException);
+    }
+
     getter = sprop->getter;
-    pd->flags = ((sprop->attrs & JSPROP_ENUMERATE) ? JSPD_ENUMERATE : 0)
+    pd->flags |= ((sprop->attrs & JSPROP_ENUMERATE) ? JSPD_ENUMERATE : 0)
               | ((sprop->attrs & JSPROP_READONLY)  ? JSPD_READONLY  : 0)
               | ((sprop->attrs & JSPROP_PERMANENT) ? JSPD_PERMANENT : 0)
 #if JS_HAS_CALL_OBJECT
@@ -1079,7 +1146,7 @@ JS_GetObjectTotalSize(JSContext *cx, JSObject *obj)
         scope = OBJ_SCOPE(obj);
         if (scope->object == obj) {
             nbytes += sizeof *scope;
-            nbytes += JS_BIT(scope->sizeLog2) * sizeof(JSScopeProperty *);
+            nbytes += SCOPE_CAPACITY(scope) * sizeof(JSScopeProperty *);
         }
     }
     return nbytes;
@@ -1115,7 +1182,7 @@ JS_GetFunctionTotalSize(JSContext *cx, JSFunction *fun)
     if (obj) {
         obytes = JS_GetObjectTotalSize(cx, obj);
         if (fun->nrefs > 1)
-            obytes = (obytes + fun->nrefs - 1) / fun->nrefs;
+            obytes = JS_HOWMANY(obytes, fun->nrefs);
         nbytes += obytes;
     }
     if (fun->script)
@@ -1151,12 +1218,10 @@ JS_GetScriptTotalSize(JSContext *cx, JSScript *script)
     if (script->filename)
         nbytes += strlen(script->filename) + 1;
 
-    notes = script->notes;
-    if (notes) {
-        for (sn = notes; !SN_IS_TERMINATOR(sn); sn += SN_LENGTH(sn))
-            continue;
-        nbytes += (sn - notes + 1) * sizeof *sn;
-    }
+    notes = SCRIPT_NOTES(script);
+    for (sn = notes; !SN_IS_TERMINATOR(sn); sn = SN_NEXT(sn))
+        continue;
+    nbytes += (sn - notes + 1) * sizeof *sn;
 
     tnotes = script->trynotes;
     if (tnotes) {
@@ -1170,7 +1235,7 @@ JS_GetScriptTotalSize(JSContext *cx, JSScript *script)
         JS_ASSERT(principals->refcount);
         pbytes = sizeof *principals;
         if (principals->refcount > 1)
-            pbytes = (pbytes + principals->refcount - 1) / principals->refcount;
+            pbytes = JS_HOWMANY(pbytes, principals->refcount);
         nbytes += pbytes;
     }
 
