@@ -1,44 +1,46 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 4 -*-
  *
- * The contents of this file are subject to the Netscape Public
- * License Version 1.1 (the "License"); you may not use this file
- * except in compliance with the License. You may obtain a copy of
- * the License at http://www.mozilla.org/NPL/
+ * ***** BEGIN LICENSE BLOCK *****
+ * Version: MPL 1.1/GPL 2.0/LGPL 2.1
  *
- * Software distributed under the License is distributed on an "AS
- * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express oqr
- * implied. See the License for the specific language governing
- * rights and limitations under the License.
+ * The contents of this file are subject to the Mozilla Public License Version
+ * 1.1 (the "License"); you may not use this file except in compliance with
+ * the License. You may obtain a copy of the License at
+ * http://www.mozilla.org/MPL/
+ *
+ * Software distributed under the License is distributed on an "AS IS" basis,
+ * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
+ * for the specific language governing rights and limitations under the
+ * License.
  *
  * The Original Code is Mozilla Communicator client code, released
  * March 31, 1998.
  *
- * The Initial Developer of the Original Code is Netscape
- * Communications Corporation.  Portions created by Netscape are
- * Copyright (C) 1998 Netscape Communications Corporation. All
- * Rights Reserved.
+ * The Initial Developer of the Original Code is
+ * Netscape Communications Corporation.
+ * Portions created by the Initial Developer are Copyright (C) 1998
+ * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): 
+ * Contributor(s):
  *
- * Alternatively, the contents of this file may be used under the
- * terms of the GNU Public License (the "GPL"), in which case the
- * provisions of the GPL are applicable instead of those above.
- * If you wish to allow use of your version of this file only
- * under the terms of the GPL and not to allow others to use your
- * version of this file under the NPL, indicate your decision by
- * deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL.  If you do not delete
- * the provisions above, a recipient may use your version of this
- * file under either the NPL or the GPL.
- */
+ * Alternatively, the contents of this file may be used under the terms of
+ * either of the GNU General Public License Version 2 or later (the "GPL"),
+ * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
+ * in which case the provisions of the GPL or the LGPL are applicable instead
+ * of those above. If you wish to allow use of your version of this file only
+ * under the terms of either the GPL or the LGPL, and not to allow others to
+ * use your version of this file under the terms of the MPL, indicate your
+ * decision by deleting the provisions above and replace them with the notice
+ * and other provisions required by the GPL or the LGPL. If you do not delete
+ * the provisions above, a recipient may use your version of this file under
+ * the terms of any one of the MPL, the GPL or the LGPL.
+ *
+ * ***** END LICENSE BLOCK ***** */
 
 /*
  * PR time code.
  */
 #include "jsstddef.h"
-#ifdef MOZILLA_CLIENT
-#include "platform.h"
-#endif
 #ifdef SOLARIS
 #define _REENTRANT 1
 #endif
@@ -52,13 +54,12 @@
 
 #define PRMJ_DO_MILLISECONDS 1
 
-#ifdef XP_PC
-#ifndef __MWERKS__
+#ifdef XP_OS2
 #include <sys/timeb.h>
-#else
-#include <WINDEF.H>
-#include <WINBASE.H>
 #endif
+#ifdef XP_WIN
+#include <windef.h>
+#include <winbase.h>
 #endif
 
 #ifdef XP_MAC
@@ -69,6 +70,9 @@
 #include <UTCUtils.h>
 #include <Power.h>
 #include <CodeFragments.h>
+#if !TARGET_CARBON
+#include <Traps.h>
+#endif
 #endif
 
 #if defined(XP_UNIX) || defined(XP_BEOS)
@@ -157,9 +161,15 @@ static void MyReadLocation(MachineLocation * loc)
         MacintoshInitializeTime();
         ReadLocation(&storedLoc);
         /* install a sleep queue routine, so that when the machine wakes up, time can be recomputed. */
-        if ((gSleepQEntry.sleepQProc = NewSleepQUPP(MySleepQProc)) != NULL) {
-            SleepQInstall(&gSleepQEntry);
-            gSleepQEntryInstalled = JS_TRUE;
+        if (&SleepQInstall != (void*)kUnresolvedCFragSymbolAddress
+#if !TARGET_CARBON
+            && NGetTrapAddress(0xA28A, OSTrap) != NGetTrapAddress(_Unimplemented, ToolTrap)
+#endif
+           ) {
+            if ((gSleepQEntry.sleepQProc = NewSleepQUPP(MySleepQProc)) != NULL) {
+                SleepQInstall(&gSleepQEntry);
+                gSleepQEntryInstalled = JS_TRUE;
+            }
         }
         didReadLocation = JS_TRUE;
      }
@@ -187,7 +197,7 @@ pascal OSErr __JSInitialize(const CFragInitBlock* initBlock)
 
 pascal void __JSTerminate()
 {
-  // clean up the sleepQ entry
+  /* clean up the sleepQ entry */
   if (gSleepQEntryInstalled)
     SleepQRemove(&gSleepQEntry);
   
@@ -214,7 +224,7 @@ static void PRMJ_basetime(JSInt64 tsecs, PRMJTime *prtm);
 JSInt32
 PRMJ_LocalGMTDifference()
 {
-#if defined(XP_UNIX) || defined(XP_PC) || defined(XP_BEOS)
+#if defined(XP_UNIX) || defined(XP_WIN) || defined(XP_OS2) || defined(XP_BEOS)
     struct tm ltime;
 
     /* get the difference between this time zone and GMT */
@@ -299,14 +309,16 @@ PRMJ_ToExtendedTime(JSInt32 base_time)
 JSInt64
 PRMJ_Now(void)
 {
-#ifdef XP_PC
+#ifdef XP_OS2
     JSInt64 s, us, ms2us, s2us;
-#ifndef __MWERKS__
     struct timeb b;
-#else
-    SYSTEMTIME time;
-#endif /* __MWERKS__ */
-#endif /* XP_PC */
+#endif
+#ifdef XP_WIN
+    JSInt64 s, us,
+    win2un = JSLL_INIT(0x19DB1DE, 0xD53E8000),
+    ten = JSLL_INIT(0, 10);
+    FILETIME time, midnight;
+#endif
 #if defined(XP_UNIX) || defined(XP_BEOS)
     struct timeval tv;
     JSInt64 s, us, s2us;
@@ -320,8 +332,7 @@ PRMJ_Now(void)
     JSInt64	 s2us;
 #endif /* XP_MAC */
 
-#ifdef XP_PC
-#ifndef __MWERKS__
+#ifdef XP_OS2
     ftime(&b);
     JSLL_UI2L(ms2us, PRMJ_USEC_PER_MSEC);
     JSLL_UI2L(s2us, PRMJ_USEC_PER_SEC);
@@ -331,17 +342,27 @@ PRMJ_Now(void)
     JSLL_MUL(s, s, s2us);
     JSLL_ADD(s, s, us);
     return s;
-#else
-    GetLocalTime(&time);
-    JSLL_UI2L(ms2us, PRMJ_USEC_PER_MSEC);
-    JSLL_UI2L(s2us, PRMJ_USEC_PER_SEC);
-    JSLL_UI2L(s, time.wSecond);
-    JSLL_UI2L(us, time.wMilliseconds);
-    JSLL_MUL(us, us, ms2us);
-    JSLL_MUL(s, s, s2us);
+#endif
+#ifdef XP_WIN
+    /* The windows epoch is around 1600. The unix epoch is around 1970.
+       win2un is the difference (in windows time units which are 10 times
+       more precise than the JS time unit) */
+    GetSystemTimeAsFileTime(&time);
+    /* Win9x gets confused at midnight
+       http://support.microsoft.com/default.aspx?scid=KB;en-us;q224423
+       So if the low part (precision <8mins) is 0 then we get the time
+       again. */
+    if (!time.dwLowDateTime) {
+        GetSystemTimeAsFileTime(&midnight);
+        time.dwHighDateTime = midnight.dwHighDateTime;
+    }
+    JSLL_UI2L(s, time.dwHighDateTime);
+    JSLL_UI2L(us, time.dwLowDateTime);
+    JSLL_SHL(s, s, 32);
     JSLL_ADD(s, s, us);
+    JSLL_SUB(s, s, win2un);
+    JSLL_DIV(s, s, ten);
     return s;
-#endif /* __MWERKS__ */
 #endif
 
 #if defined(XP_UNIX) || defined(XP_BEOS)
@@ -453,7 +474,7 @@ PRMJ_DSTOffset(JSInt64 local_time)
 size_t
 PRMJ_FormatTime(char *buf, int buflen, char *fmt, PRMJTime *prtm)
 {
-#if defined(XP_UNIX) || defined(XP_PC) || defined(XP_MAC) || defined(XP_BEOS)
+#if defined(XP_UNIX) || defined(XP_WIN) || defined(XP_OS2) || defined(XP_MAC) || defined(XP_BEOS)
     struct tm a;
 
     /* Zero out the tm struct.  Linux, SunOS 4 struct tm has extra members int
