@@ -47,7 +47,7 @@
 #include "nsISupports.h"
 #include "nsEvent.h"
 #include "nsStyleStruct.h"
-#include "nsIStyleContext.h"
+#include "nsStyleContext.h"
 #include "nsIContent.h"
 
 /**
@@ -75,12 +75,10 @@ class nsIAtom;
 class nsIPresContext;
 class nsIPresShell;
 class nsIRenderingContext;
-class nsISizeOfHandler;
 class nsIView;
 class nsIWidget;
 class nsIDOMRange;
 class nsISelectionController;
-class nsIPercentHeightObserver;
 #ifdef ACCESSIBILITY
 class nsIAccessible;
 #endif
@@ -91,6 +89,13 @@ struct nsRect;
 struct nsSize;
 struct nsMargin;
 
+// Calback function used to destroy the value associated with a property.
+typedef void 
+(*NSFramePropertyDtorFunc)(nsIPresContext* aPresContext,
+                           nsIFrame*       aFrame,
+                           nsIAtom*        aPropertyName,
+                           void*           aPropertyValue);
+
 // IID for the nsIFrame interface 
 // a6cf9050-15b3-11d2-932e-00805f8add32
 #define NS_IFRAME_IID \
@@ -98,7 +103,7 @@ struct nsMargin;
 
 /**
  * Indication of how the frame can be split. This is used when doing runaround
- * of floaters, and when pulling up child frames from a next-in-flow.
+ * of floats, and when pulling up child frames from a next-in-flow.
  *
  * The choices are splittable, not splittable at all, and splittable in
  * a non-rectangular fashion. This last type only applies to block-level
@@ -144,7 +149,7 @@ typedef PRUint32 nsFrameState;
 
 // If this bit is set, then there is a child frame in the frame that
 // extends outside this frame's bounding box. The implication is that
-// the frames rect does not completely cover its children and
+// the frame's rect does not completely cover its children and
 // therefore operations like rendering and hit testing (for example)
 // must operate differently.
 #define NS_FRAME_OUTSIDE_CHILDREN                     0x00000008
@@ -161,12 +166,7 @@ typedef PRUint32 nsFrameState;
 // If this bit is set, then the frame corresponds to generated content
 #define NS_FRAME_GENERATED_CONTENT                    0x00000040
 
-// If this bit is set, then the frame has requested one or more image
-// loads via the nsIPresContext.StartLoadImage API at some time during
-// its lifetime.
-#define NS_FRAME_HAS_LOADED_IMAGES                    0x00000080
-
-// If this bit is set, then the frame is has been moved out of the flow,
+// If this bit is set, then the frame has been moved out of the flow,
 // e.g., it is absolutely positioned or floated
 #define NS_FRAME_OUT_OF_FLOW                          0x00000100
 
@@ -210,6 +210,10 @@ typedef PRUint32 nsFrameState;
 // If this bit is set the frame has descendant with a view
 #define NS_FRAME_HAS_CHILD_WITH_VIEW                  0x00040000
 
+// If this bit is set, then reflow may be dispatched from the current
+// frame instead of the root frame.
+#define NS_FRAME_REFLOW_ROOT                          0x00080000
+
 // The lower 20 bits of the frame state word are reserved by this API.
 #define NS_FRAME_RESERVED                             0x000FFFFF
 
@@ -247,11 +251,6 @@ enum nsSpread {
   eSpreadDown   = 2
 };
 
-enum nsContextProviderRelationship {
-  eContextProvider_Ancestor = 0,
-  eContextProvider_Descendant = 1
-};
-
 // Carried out margin flags
 #define NS_CARRIED_TOP_MARGIN_IS_AUTO    0x1
 #define NS_CARRIED_BOTTOM_MARGIN_IS_AUTO 0x2
@@ -261,7 +260,7 @@ enum nsContextProviderRelationship {
 // For HTML reflow we rename with the different paint layers are
 // actually used for.
 #define NS_FRAME_PAINT_LAYER_BACKGROUND eFramePaintLayer_Underlay
-#define NS_FRAME_PAINT_LAYER_FLOATERS   eFramePaintLayer_Content
+#define NS_FRAME_PAINT_LAYER_FLOATS   eFramePaintLayer_Content
 #define NS_FRAME_PAINT_LAYER_FOREGROUND eFramePaintLayer_Overlay
 #define NS_FRAME_PAINT_LAYER_DEBUG      eFramePaintLayer_Overlay
 
@@ -349,6 +348,19 @@ typedef PRUint32 nsReflowStatus;
   ((_completionStatus) | NS_INLINE_BREAK | NS_INLINE_BREAK_AFTER |      \
    NS_INLINE_MAKE_BREAK_TYPE(NS_STYLE_CLEAR_LINE))
 
+// The frame (not counting a continuation) did not fit in the available height and 
+// wasn't at the top of a page. If it was at the top of a page, then it is not 
+// possible to reflow it again with more height, so we don't set it in that case.
+#define NS_FRAME_TRUNCATED  0x0010
+#define NS_FRAME_IS_TRUNCATED(status) \
+  (0 != ((status) & NS_FRAME_TRUNCATED))
+#define NS_FRAME_SET_TRUNCATION(status, aReflowState, aMetrics) \
+  if (!aReflowState.mFlags.mIsTopOfPage &&                      \
+      aReflowState.availableHeight < aMetrics.height)           \
+    status |= NS_FRAME_TRUNCATED;                               \
+  else                                                          \
+    status &= ~NS_FRAME_TRUNCATED;
+
 //----------------------------------------------------------------------
 
 /**
@@ -374,11 +386,23 @@ typedef PRBool nsDidReflowStatus;
  * Frames are NOT reference counted. Use the Destroy() member function
  * to destroy a frame. The lifetime of the frame hierarchy is bounded by the
  * lifetime of the presentation shell which owns the frames.
+ *
+ * nsIFrame is a private Gecko interface. If you are not Gecko then you
+ * should not use it. If you're not in layout, then you won't be able to
+ * link to many of the functions defined here. Too bad.
+ *
+ * If you're not in layout but you must call functions in here, at least
+ * restrict yourself to calling virtual methods, which won't hurt you as badly.
  */
 class nsIFrame : public nsISupports
 {
 public:
   NS_DEFINE_STATIC_IID_ACCESSOR(NS_IFRAME_IID)
+
+  nsIPresContext* GetPresContext() const {
+    return GetStyleContext()->GetRuleNode()->GetPresContext();
+  }
+
   /**
    * Called to initialize the frame. This is called immediately after creating
    * the frame.
@@ -400,7 +424,7 @@ public:
   NS_IMETHOD  Init(nsIPresContext*  aPresContext,
                    nsIContent*      aContent,
                    nsIFrame*        aParent,
-                   nsIStyleContext* aContext,
+                   nsStyleContext*  aContext,
                    nsIFrame*        aPrevInFlow) = 0;
 
   /**
@@ -409,6 +433,11 @@ public:
    */
   NS_IMETHOD  Destroy(nsIPresContext* aPresContext) = 0;
 
+  /*
+   * Notify the frame that it has been removed as the primary frame for its content
+   */
+  virtual void RemovedAsPrimaryFrame(nsIPresContext* aPresContext) {}
+
   /**
    * Called to set the initial list of frames. This happens after the frame
    * has been initialized.
@@ -516,12 +545,9 @@ public:
                           nsIFrame*       aNewFrame) = 0;
 
   /**
-   * Get the content object associated with this frame. Adds a reference to
-   * the content object so the caller must do a release.
-   *
-   * @see nsISupports#Release()
+   * Get the content object associated with this frame. Does not add a reference.
    */
-  nsresult GetContent(nsIContent** aContent) const {  *aContent = mContent; NS_IF_ADDREF(*aContent); return NS_OK; }
+  nsIContent* GetContent() const { return mContent; }
 
   /**
    * Get the offsets of the frame. most will be 0,0
@@ -530,47 +556,69 @@ public:
   NS_IMETHOD GetOffsets(PRInt32 &start, PRInt32 &end) const = 0;
 
   /**
-   * Get the style context associated with this frame. Note that GetStyleContext()
-   * adds a reference to the style context so the caller must do a release.
+   * Reset the offsets when splitting frames during Bidi reordering
    *
-   * @see nsISupports#Release()
    */
-  nsresult GetStyleContext(nsIStyleContext** aStyleContext) const { 
-    *aStyleContext = mStyleContext; NS_IF_ADDREF(*aStyleContext); return NS_OK;
-  }
-  nsresult SetStyleContext(nsIPresContext*  aPresContext, nsIStyleContext* aContext) { 
+  virtual void AdjustOffsetsForBidi(PRInt32 aStart, PRInt32 aEnd) {}
+
+  /**
+   * Get the style context associated with this frame.
+   *
+   */
+  nsStyleContext* GetStyleContext() const { return mStyleContext; }
+  void SetStyleContext(nsIPresContext* aPresContext, nsStyleContext* aContext)
+  { 
     if (aContext != mStyleContext) {
-      NS_IF_RELEASE(mStyleContext);
-      if (nsnull != aContext) {
-        mStyleContext = aContext;
-        NS_ADDREF(aContext);
+      if (mStyleContext)
+        mStyleContext->Release();
+      mStyleContext = aContext;
+      if (aContext) {
+        aContext->AddRef();
         DidSetStyleContext(aPresContext);
       }
     }
-    return NS_OK;
   }
 
   // Style post processing hook
   NS_IMETHOD DidSetStyleContext(nsIPresContext* aPresContext) = 0;
 
   /**
-   * Get the style data associated with this frame.
+   * Get the style data associated with this frame.  This returns a
+   * const style struct pointer that should never be modified.  See
+   * |nsIStyleContext::GetStyleData| for more information.
+   *
+   * The use of the typesafe functions below is preferred to direct use
+   * of this function.
    */
-  nsresult GetStyleData(nsStyleStructID       aSID,
-                        const nsStyleStruct*& aStyleStruct) const {
+  virtual const nsStyleStruct* GetStyleDataExternal(nsStyleStructID aSID) const = 0;
+
+  const nsStyleStruct* GetStyleData(nsStyleStructID aSID) const {
+#ifdef _IMPL_NS_LAYOUT
     NS_ASSERTION(mStyleContext, "No style context found!");
-    aStyleStruct = mStyleContext->GetStyleData(aSID);
-    return NS_OK;
+    return mStyleContext->GetStyleData(aSID);
+#else
+    return GetStyleDataExternal(aSID);
+#endif
   }
 
-  // Fill a style struct with data
-  nsresult GetStyle(nsStyleStructID aSID, const nsStyleStruct** aStruct) const {
-    NS_ASSERTION(mStyleContext, "No style context found!");
-    mStyleContext->GetStyle(aSID, aStruct);
-    return NS_OK;
-  }
+  /**
+   * Define typesafe getter functions for each style struct by
+   * preprocessing the list of style structs.  These functions are the
+   * preferred way to get style data.  The macro creates functions like:
+   *   const nsStyleBorder* GetStyleBorder();
+   *   const nsStyleColor* GetStyleColor();
+   */
+
+  #define STYLE_STRUCT(name_, checkdata_cb_, ctor_args_)                      \
+    const nsStyle##name_ * GetStyle##name_() const {                          \
+      return NS_STATIC_CAST(const nsStyle##name_*,                            \
+                            GetStyleData(eStyleStruct_##name_));              \
+    }
+  #include "nsStyleStructList.h"
+  #undef STYLE_STRUCT
 
   // Utility function: more convenient than 2 calls to GetStyleData to get border and padding
+  
   NS_IMETHOD  CalcBorderPadding(nsMargin& aBorderPadding) const = 0;
 
   /**
@@ -583,15 +631,15 @@ public:
    * The indicies must be consecutive and implementations MUST return an 
    * NS_ERROR_INVALID_ARG if asked for an index that is out of range.
    */
-  NS_IMETHOD  GetAdditionalStyleContext(PRInt32 aIndex, 
-                                        nsIStyleContext** aStyleContext) const = 0;
-  NS_IMETHOD  SetAdditionalStyleContext(PRInt32 aIndex, 
-                                        nsIStyleContext* aStyleContext) = 0;
+  virtual nsStyleContext* GetAdditionalStyleContext(PRInt32 aIndex) const = 0;
+
+  virtual void SetAdditionalStyleContext(PRInt32 aIndex,
+                                         nsStyleContext* aStyleContext) = 0;
 
   /**
    * Accessor functions for geometric parent
    */
-  nsresult GetParent(nsIFrame** aParent) const { *aParent = mParent; return NS_OK; }
+  nsIFrame* GetParent() const { return mParent; }
   NS_IMETHOD SetParent(const nsIFrame* aParent) { mParent = (nsIFrame*)aParent; return NS_OK; }
 
   /**
@@ -602,45 +650,12 @@ public:
    * Note: moving or sizing the frame does not affect the view's size or
    * position.
    */
-  nsresult GetRect(nsRect& aRect) const {
-    aRect = mRect;
-    return NS_OK;
-  }
-
-  nsresult GetOrigin(nsPoint& aPoint) const {
-    aPoint.x = mRect.x;
-    aPoint.y = mRect.y;
-    return NS_OK;
-  }
-
-  nsresult GetSize(nsSize& aSize) const {
-    aSize.width = mRect.width;
-    aSize.height = mRect.height;
-    return NS_OK;
-  }
-
-  nsresult SetRect(nsIPresContext* aPresContext,
-               const nsRect&   aRect) {
-    MoveTo(aPresContext, aRect.x, aRect.y);
-    SizeTo(aPresContext, aRect.width, aRect.height);
-    return NS_OK;
-  }
-
-  nsresult MoveTo(nsIPresContext* aPresContext,
-                  nscoord         aX,
-                  nscoord         aY) {
-    mRect.x = aX;
-    mRect.y = aY;
-    return NS_OK;
-  }
-
-  nsresult SizeTo(nsIPresContext* aPresContext,
-                  nscoord         aWidth,
-                  nscoord         aHeight) {
-    mRect.width = aWidth;
-    mRect.height = aHeight;
-    return NS_OK;
-  }
+  nsRect GetRect() const { return mRect; }
+  nsPoint GetPosition() const { return nsPoint(mRect.x, mRect.y); }
+  nsSize GetSize() const { return nsSize(mRect.width, mRect.height); }
+  void SetRect(const nsRect& aRect) { mRect = aRect; }
+  void SetPosition(const nsPoint& aPt) { mRect.MoveTo(aPt); }
+  void SetSize(const nsSize& aSize) { mRect.SizeTo(aSize); }
 
   /**
    * Used to iterate the list of additional child list names. Returns the atom
@@ -649,39 +664,27 @@ public:
    *
    * Note that the list is only the additional named child lists and does not
    * include the unnamed principal child list.
-   *
-   * @return NS_ERROR_INVALID_ARG if the index is < 0 and NS_OK otherwise
    */
-  NS_IMETHOD  GetAdditionalChildListName(PRInt32   aIndex,
-                                         nsIAtom** aListName) const = 0;
+  virtual nsIAtom* GetAdditionalChildListName(PRInt32 aIndex) const = 0;
 
   /**
    * Get the first child frame from the specified child list.
    *
    * @param   aListName the name of the child list. A NULL pointer for the atom
    *            name means the unnamed principal child list
-   * @return  NS_ERROR_INVALID_ARG if there is no child list with the specified name
+   * @return  the child frame, or NULL if there is no such child
    * @see     #GetAdditionalListName()
    */
-  NS_IMETHOD  FirstChild(nsIPresContext* aPresContext,
-                         nsIAtom*        aListName,
-                         nsIFrame**      aFirstChild) const = 0;
+  virtual nsIFrame* GetFirstChild(nsIAtom* aListName) const = 0;
 
   /**
-   * Child frames are linked together in a singly-linked
+   * Child frames are linked together in a singly-linked list
    */
-  nsresult GetNextSibling(nsIFrame** aNextSibling) const {
-    *aNextSibling = mNextSibling;
-    return NS_OK;
-  }
-
-  nsresult SetNextSibling(nsIFrame* aNextSibling) {
-    mNextSibling = aNextSibling;
-    return NS_OK;
-  }
+  nsIFrame* GetNextSibling() const { return mNextSibling; }
+  void SetNextSibling(nsIFrame* aNextSibling) { mNextSibling = aNextSibling; }
 
   /**
-   * Paint is responsible for painting the a frame. The aWhichLayer
+   * Paint is responsible for painting the frame. The aWhichLayer
    * argument indicates which layer of painting should be done during
    * the call.
    */
@@ -691,6 +694,12 @@ public:
                     nsFramePaintLayer    aWhichLayer,
                     PRUint32             aFlags = 0) = 0;
 
+  /* 
+   * Does the frame paint its background? If not, then all or part of it will be
+   * painted by ancestors.
+   */
+  virtual PRBool CanPaintBackground() { return PR_TRUE; }
+
   /**
    * Event handling of GUI events.
    *
@@ -769,18 +778,13 @@ public:
    * Get the current frame-state value for this frame. aResult is
    * filled in with the state bits. 
    */
-  nsresult GetFrameState(nsFrameState* aResult) {
-    *aResult = mState;
-    return NS_OK;
-  }
+  nsFrameState GetStateBits() const { return mState; }
 
   /**
-   * Set the current frame-state value for this frame. 
+   * Update the current frame-state value for this frame. 
    */
-  nsresult SetFrameState(nsFrameState aNewState) {
-    mState = aNewState;
-    return NS_OK;
-  }
+  void AddStateBits(nsFrameState aBits) { mState |= aBits; }
+  void RemoveStateBits(nsFrameState aBits) { mState &= ~aBits; }
 
   /**
    * This call is invoked when content is changed in the content tree.
@@ -789,11 +793,11 @@ public:
    *
    * @param aPresContext the presentation context
    * @param aContent     the content node that was changed
-   * @param aSubContent  a hint to the frame about the change
+   * @param aAppend      a hint to the frame about the change
    */
-  NS_IMETHOD  ContentChanged(nsIPresContext* aPresContext,
-                             nsIContent*     aChild,
-                             nsISupports*    aSubContent) = 0;
+  NS_IMETHOD  CharacterDataChanged(nsIPresContext* aPresContext,
+                                   nsIContent*     aChild,
+                                   PRBool          aAppend) = 0;
 
   /**
    * This call is invoked when the value of a content objects's attribute
@@ -809,21 +813,7 @@ public:
                                nsIContent*     aChild,
                                PRInt32         aNameSpaceID,
                                nsIAtom*        aAttribute,
-                               PRInt32         aModType, 
-                               PRInt32         aHint) = 0;
-
-  /**
-   * This call is invoked when the value of a content object's state
-   * is changed. 
-   * The first frame that maps that content is asked to deal
-   * with the change by doing whatever is appropriate.
-   *
-   * @param aChild the content object
-   * @param aHint the level of change that has already been dealt with
-   */
-  NS_IMETHOD  ContentStateChanged(nsIPresContext* aPresContext,
-                                  nsIContent*     aChild,
-                                  PRInt32         aHint) = 0;
+                               PRInt32         aModType) = 0;
 
   /**
    * Return how your frame can be split.
@@ -838,6 +828,20 @@ public:
   NS_IMETHOD  GetNextInFlow(nsIFrame** aNextInFlow) const = 0;
   NS_IMETHOD  SetNextInFlow(nsIFrame*) = 0;
 
+  /**
+   * Return the first frame in our current flow. 
+   */
+  virtual nsIFrame* GetFirstInFlow() const {
+    return NS_CONST_CAST(nsIFrame*, this);
+  }
+
+  /**
+   * Return the last frame in our current flow.
+   */
+  virtual nsIFrame* GetLastInFlow() const {
+    return NS_CONST_CAST(nsIFrame*, this);
+  }
+
   /**
    * Pre-reflow hook. Before a frame is reflowed this method will be called.
    * This call will always be invoked at least once before a subsequent Reflow
@@ -941,17 +945,24 @@ public:
 
   /**
    * Accessor functions to get/set the associated view object
+   *
+   * GetView returns non-null if and only if |HasView| returns true.
    */
-  NS_IMETHOD  GetView(nsIPresContext* aPresContext,
-                      nsIView**       aView) const = 0;  // may be null
-  NS_IMETHOD  SetView(nsIPresContext* aPresContext,
-                      nsIView*        aView) = 0;
+  PRBool HasView() const { return mState & NS_FRAME_HAS_VIEW; }
+  nsIView* GetView() const;
+  virtual nsIView* GetViewExternal() const;
+  nsresult SetView(nsIView* aView);
 
   /**
-   * Find the first geometric parent that has a view
+   * Find the closest view (on |this| or an ancestor).
    */
-  NS_IMETHOD  GetParentWithView(nsIPresContext* aPresContext,
-                                nsIFrame**      aParent) const = 0;
+  nsIView* GetClosestView() const;
+
+  /**
+   * Find the closest ancestor (excluding |this| !) that has a view
+   */
+  nsIFrame* GetAncestorWithView() const;
+  virtual nsIFrame* GetAncestorWithViewExternal() const;
 
   /**
    * Returns the offset from this frame to the closest geometric parent that
@@ -973,32 +984,52 @@ public:
                                     nsPoint&        aOffset,
                                     nsIView**       aView) const = 0;
 
+  /**
+   * Returns true if and only if all views, from |GetClosestView| up to
+   * the top of the view hierarchy are visible.
+   */
+  virtual PRBool AreAncestorViewsVisible() const;
+
   /**
    * Returns the window that contains this frame. If this frame has a
    * view and the view has a window, then this frames window is
    * returned, otherwise this frame's geometric parent is checked
    * recursively upwards.
    */
-  NS_IMETHOD  GetWindow(nsIPresContext* aPresContext,
-                        nsIWidget**     aWidget) const = 0;
+  virtual nsIWidget* GetWindow() const;
 
   /**
    * Get the "type" of the frame. May return a NULL atom pointer
    *
    * @see nsLayoutAtoms
    */
-  NS_IMETHOD  GetFrameType(nsIAtom** aType) const = 0;
+  virtual nsIAtom* GetType() const = 0;
   
   /**
-   * Is this frame a "containing block"?
+   * Is this frame a containing block for non-positioned elements?
    */
-  NS_IMETHOD  IsPercentageBase(PRBool& aBase) const = 0;
+  virtual PRBool IsContainingBlock() const = 0;
 
   /**
-   * called when the frame has been scrolled to a new
-   * position. only called for frames with views.
+   * Invalidate part of the frame by asking the view manager to repaint.
+   * aDamageRect is allowed to extend outside the frame's bounds. We'll do the right
+   * thing. But it must be within the bounds of the view enclosing this frame.
+   * We deliberately don't have an Invalidate() method that defaults to the frame's bounds.
+   * We want all callers to *think* about what has changed in the frame and what area might
+   * need to be repainted.
+   *
+   * @param aDamageRect is in the frame's local coordinate space
    */
-  NS_IMETHOD  Scrolled(nsIView *aView) = 0;
+  void Invalidate(const nsRect& aDamageRect, PRBool aImmediate = PR_FALSE) const;
+
+  /**
+   * Computes a rect that includes this frame's outline. The returned rect is
+   * relative to this frame's origin.
+   *
+   * @param if nonnull, we record whether this rect is bigger than the frame's bounds
+   * @return the rect relative to this frame's origin
+   */
+  nsRect GetOutlineRect(PRBool* aAnyOutline = nsnull) const;
 
   /** Selection related calls
    */
@@ -1007,8 +1038,8 @@ public:
    *  to redraw event if aSelected == the frame selection with the last parameter.
    *  data in struct may be changed when passed in.
    *  @param aRange is the range that will dictate if the frames need to be redrawn null means the whole content needs to be redrawn
-   *  @param aSelected is it selected
-   *  @param aSpread should is spread selection to flow elements around it? or go down to its children?
+   *  @param aSelected is it selected?
+   *  @param aSpread should it spread the selection to flow elements around it? or go down to its children?
    */
   NS_IMETHOD  SetSelected(nsIPresContext* aPresContext,
                           nsIDOMRange*    aRange,
@@ -1088,38 +1119,29 @@ public:
 #endif
 
   /**
-   *  Called during appending or cancelling a reflow command to give frames notice
-   *  of reflow commands that will be targeted below them. 
-   */
-  NS_IMETHOD ReflowCommandNotify(nsIPresShell*        aShell,
-                                 nsHTMLReflowCommand* aRC,
-                                 PRBool               aCommandAdded) = 0;
-
-  /**
-   * Called in style ReResolution to get the frame that contains the style context that is the
-   * parent style context for this frame. 
-   * @param aProviderFrame: set to the frame who's style context is to be used as the parent context
-   *                        in reresolving style for this frame. Note that this may or may not be the
-   *                        parent frame of this frame.
-   * @param aRelatioship : indicates how the provider frame relates to this frame: 
-   *                       i.e. if the parent context provider frame is a child of this frame, 
-   *                       then aRelationship must be set to eContextProvider_Descendant, 
-   *                       if a parent, grandparent etc. set to eContextProvider_Ancestor 
-   *                       (this is used to control recursion in style reresolution)
+   * Get the frame whose style context should be the parent of this
+   * frame's style context (i.e., provide the parent style context).
+   * This frame must either be an ancestor of this frame or a child.  If
+   * this frame returns a child frame, then the child frame must be sure
+   * to return a grandparent or higher!
    *
-   * if return value is NS_OK then aProvdierFrame must be set, otherwise it is ignored.
-   *
-   * PRECONDITIONS: aProviderFrame cannot be null
-   * POSTCONDITIONS: *aProfiderFrame will be a valid frame if this routine succeeds
-   *                 aRelationship will be set to a legal enum value indicating the correct relationship
+   * @param aPresContext:   PresContext
+   * @param aProviderFrame: The frame whose style context should be the
+   *                        parent of this frame's style context.  Null
+   *                        is permitted, and means that this frame's
+   *                        style context should be the root of the
+   *                        style context tree.
+   * @param aIsChild:       True if |aProviderFrame| is set to a child
+   *                        of this frame; false if it is an ancestor or
+   *                        null.
    */
-  NS_IMETHOD GetParentStyleContextProvider(nsIPresContext* aPresContext,
-                                           nsIFrame** aProviderFrame, 
-                                           nsContextProviderRelationship& aRelationship) = 0;
+  NS_IMETHOD GetParentStyleContextFrame(nsIPresContext* aPresContext,
+                                        nsIFrame**      aProviderFrame,
+                                        PRBool*         aIsChild) = 0;
 
   /**
    * Determines whether a frame is visible for painting
-   * this takes into acount whether it is painting selection for printin
+   * this takes into account whether it is painting a selection or printing.
    * @param aPresContext PresContext
    * @param aRenderingContext PresContext
    * @param aCheckVis indicates whether it should check for CSS visibility, 
@@ -1132,19 +1154,48 @@ public:
                                   PRBool*              aIsVisible) = 0;
 
   /**
-   * Determine whether the frame is logically empty, i.e., whether the
-   * layout would be the same whether or not the frame is present.
-   * Placeholder frames should return true.  Block frames should be
-   * considered empty whenever margins collapse through them, even
-   * though those margins are relevant.
-   *
-   * aIsPre should be ignored by frames to which the 'white-space'
-   * property applies.
+   * Determine whether the frame is logically empty, which is roughly
+   * whether the layout would be the same whether or not the frame is
+   * present.  Placeholder frames should return true.  Block frames
+   * should be considered empty whenever margins collapse through them,
+   * even though those margins are relevant.  Text frames containing
+   * only whitespace that does not contribute to the height of the line
+   * should return true.
    */
-  NS_IMETHOD IsEmpty(PRBool aIsQuirkMode,
-                     PRBool aIsPre,
-                     PRBool* aResult) = 0;
+  virtual PRBool IsEmpty() = 0;
 
+  /**
+   * IsGeneratedContentFrame returns whether a frame corresponds to
+   * generated content
+   *
+   * @return whether the frame correspods to generated content
+   */
+  PRBool IsGeneratedContentFrame() {
+    return (mState & NS_FRAME_GENERATED_CONTENT) != 0;
+  }
+
+  /**
+   * IsPseudoFrame returns whether a frame is a pseudo frame (eg an
+   * anonymous table-row frame created for a CSS table-cell without an
+   * enclosing table-row.
+   *
+   * @param aParentContent the content node corresponding to the parent frame
+   * @return whether the frame is a pseudo frame
+   */   
+  PRBool IsPseudoFrame(nsIContent* aParentContent) {
+    return mContent == aParentContent;
+  }
+
+
+  
+  virtual void* GetProperty(nsIPresContext* aPresContext,
+                            nsIAtom*        aPropertyName,
+                            PRBool          aRemoveProperty) const = 0;
+
+  virtual nsresult SetProperty(nsIPresContext*         aPresContext,
+                               nsIAtom*                aPropertyName,
+                               void*                   aPropertyValue,
+                               NSFramePropertyDtorFunc aPropDtorFunc) = 0;
 #ifdef IBMBIDI
   /**
    *  retrieve and set Bidi property of this frame
@@ -1159,13 +1210,29 @@ public:
                              void*           aPropertyValue) = 0;
 #endif // IBMBIDI
 
+  /** Create or retrieve the previously stored overflow area, if the frame does 
+    * not overflow and no creation is required return nsnull.
+    * @param aPresContext PresContext
+    * @param aCreateIfNecessary  create a new nsRect for the overflow area
+    * @return pointer to the overflow area rectangle 
+    */
+  virtual nsRect* GetOverflowAreaProperty(nsIPresContext* aPresContext,
+                                          PRBool          aCreateIfNecessary = PR_FALSE) = 0;
+
+  /**
+   * Return PR_TRUE if and only if this frame obeys visibility:hidden.
+   * if it does not, then nsContainerFrame will hide its view even though
+   * this means children can't be made visible again.
+   */
+  virtual PRBool SupportsVisibilityHidden() { return PR_TRUE; }
+
 protected:
   // Members
   nsRect           mRect;
   nsIContent*      mContent;
-  nsIStyleContext* mStyleContext;
+  nsStyleContext*  mStyleContext;
   nsIFrame*        mParent;
-  nsIFrame*        mNextSibling;  // singly linked list of frames
+  nsIFrame*        mNextSibling;  // singly-linked list of frames
   nsFrameState     mState;
 
 private:
@@ -1173,14 +1240,4 @@ private:
   NS_IMETHOD_(nsrefcnt) Release(void) = 0;
 };
 
-// typesafe way to access style data.  See comment in nsStyleStruct.h
-// and also overloaded function in nsIStyleContext.h
-template <class T>
-inline void
-GetStyleData(nsIFrame* aFrame, const T** aStyleStruct)
-{
-    aFrame->GetStyleData(NS_GET_STYLESTRUCTID(T),
-                    *NS_REINTERPRET_CAST(const nsStyleStruct**, aStyleStruct));
-}
-
 #endif /* nsIFrame_h___ */
