@@ -20,8 +20,8 @@
  * the Initial Developer. All Rights Reserved.
  *
  * Contributor(s):
- *   Jan Varga (varga@utcru.sk)
- *   Brian Ryner <bryner@netscape.com>
+ *   Jan Varga (varga@nixcorp.com)
+ *   Brian Ryner <bryner@brianryner.com>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or 
@@ -41,7 +41,6 @@
 #include "nsINameSpaceManager.h"
 #include "nsHTMLAtoms.h"
 #include "nsXULAtoms.h"
-#include "nsCSSAtoms.h"
 #include "nsLayoutAtoms.h"
 #include "nsIDOMDocument.h"
 #include "nsIBoxObject.h"
@@ -49,6 +48,7 @@
 #include "nsTreeContentView.h"
 #include "nsChildIterator.h"
 #include "nsIDOMHTMLOptionElement.h"
+#include "nsIDOMHTMLOptGroupElement.h"
 #include "nsIDOMClassInfo.h"
 #include "nsCRT.h"
 #include "nsIEventStateManager.h"
@@ -62,20 +62,6 @@
 #define ROW_FLAG_EMPTY          0x04
 #define ROW_FLAG_SEPARATOR      0x08
 
-class Property
-{
-  public:
-    Property(nsIAtom* aAtom)
-      : mAtom(aAtom), mNext(nsnull) {
-    }
-    ~Property() {
-      delete mNext;
-    }
-
-    nsCOMPtr<nsIAtom>   mAtom;
-    Property*           mNext;
-};
-
 class Row
 {
   public:
@@ -94,11 +80,10 @@ class Row
 
     Row(nsIContent* aContent, PRInt32 aParentIndex)
       : mContent(aContent), mParentIndex(aParentIndex),
-        mSubtreeSize(0), mProperty(nsnull), mFlags(0) {
+        mSubtreeSize(0), mFlags(0) {
     }
 
     ~Row() {
-      delete mProperty;
     }
 
     void SetContainer(PRBool aContainer) {
@@ -121,11 +106,6 @@ class Row
     }
     PRBool IsSeparator() { return mFlags & ROW_FLAG_SEPARATOR; }
 
-    void SetProperty(Property* aProperty) {
-      delete mProperty;
-      mProperty = aProperty;
-    }
-
     // Weak reference to a content item.
     nsIContent*         mContent;
 
@@ -135,13 +115,10 @@ class Row
     // Subtree size for this item.
     PRInt32             mSubtreeSize;
 
-    // List of parsed properties
-    Property*           mProperty;
-
   private:
     // Hide so that only Create() and Destroy() can be used to
     // allocate and deallocate from the heap
-    static void* operator new(size_t) { return 0; } 
+    static void* operator new(size_t) CPP_THROW_NEW { return 0; } 
     static void operator delete(void*, size_t) {}
 
     // State flags
@@ -156,11 +133,12 @@ class Row
 // document's observer list.
 
 nsTreeContentView::nsTreeContentView(void) :
-  mBoxObject(nsnull), mSelection(nsnull), mRoot(nsnull), mDocument(nsnull),
+  mBoxObject(nsnull),
+  mSelection(nsnull),
+  mRoot(nsnull),
+  mDocument(nsnull),
   mUpdateSelection(PR_FALSE)
 {
-  NS_INIT_ISUPPORTS();
-
   static const size_t kBucketSizes[] = {
     sizeof(Row)
   };
@@ -172,7 +150,7 @@ nsTreeContentView::nsTreeContentView(void) :
 
 nsTreeContentView::~nsTreeContentView(void)
 {
-  // Remove ourselfs from document's observers.
+  // Remove ourselves from mDocument's observers.
   if (mDocument)
     mDocument->RemoveObserver(this);
 }
@@ -246,10 +224,17 @@ nsTreeContentView::GetRowProperties(PRInt32 aIndex, nsISupportsArray* aPropertie
     return NS_ERROR_INVALID_ARG;   
 
   Row* row = (Row*)mRows[aIndex];
-  Property* property = row->mProperty;
-  while (property) {
-    aProperties->AppendElement(property->mAtom);
-    property = property->mNext;
+  nsCOMPtr<nsIContent> realRow;
+  if (row->IsSeparator())
+    realRow = row->mContent;
+  else
+    nsTreeUtils::GetImmediateChild(row->mContent, nsXULAtoms::treerow, getter_AddRefs(realRow));
+
+  if (realRow) {
+    nsAutoString properties;
+    realRow->GetAttr(kNameSpaceID_None, nsXULAtoms::properties, properties);
+    if (!properties.IsEmpty())
+      nsTreeUtils::TokenizeProperties(properties, aProperties);
   }
 
   return NS_OK;
@@ -271,7 +256,7 @@ nsTreeContentView::GetCellProperties(PRInt32 aRow, const PRUnichar* aColID, nsIS
     if (cell) {
       nsAutoString properties;
       cell->GetAttr(kNameSpaceID_None, nsXULAtoms::properties, properties);
-      if (properties.Length())
+      if (!properties.IsEmpty())
         nsTreeUtils::TokenizeProperties(properties, aProperties);
     }
   }
@@ -284,7 +269,7 @@ nsTreeContentView::GetColumnProperties(const PRUnichar* aColID, nsIDOMElement* a
 {
   nsAutoString properties;
   aColElt->GetAttribute(NS_LITERAL_STRING("properties"), properties);
-  if (properties.Length())
+  if (!properties.IsEmpty())
     nsTreeUtils::TokenizeProperties(properties, aProperties);
 
   return NS_OK;
@@ -528,16 +513,21 @@ nsTreeContentView::GetCellText(PRInt32 aRow, const PRUnichar* aColID, nsAString&
       && !_retval.IsEmpty())
     return NS_OK;
 
-  nsCOMPtr<nsIAtom> rowTag;
-  row->mContent->GetTag(*getter_AddRefs(rowTag));
-  if (rowTag == nsHTMLAtoms::option) {
+  nsIAtom *rowTag = row->mContent->Tag();
+  if (rowTag == nsHTMLAtoms::option &&
+      row->mContent->IsContentOfType(nsIContent::eHTML)) {
     // Use the text node child as the label
     nsCOMPtr<nsIDOMHTMLOptionElement> elem = do_QueryInterface(row->mContent);
     elem->GetText(_retval);
-    return NS_OK;
-  } else if (rowTag == nsXULAtoms::treeitem) {
+  } else if (rowTag == nsHTMLAtoms::optgroup &&
+             row->mContent->IsContentOfType(nsIContent::eHTML)) {
+    nsCOMPtr<nsIDOMHTMLOptGroupElement> elem = do_QueryInterface(row->mContent);
+    elem->GetLabel(_retval);
+  } else if (rowTag == nsXULAtoms::treeitem &&
+             row->mContent->IsContentOfType(nsIContent::eXUL)) {
     nsCOMPtr<nsIContent> realRow;
-    nsTreeUtils::GetImmediateChild(row->mContent, nsXULAtoms::treerow, getter_AddRefs(realRow));
+    nsTreeUtils::GetImmediateChild(row->mContent, nsXULAtoms::treerow,
+                                   getter_AddRefs(realRow));
     if (realRow) {
       nsCOMPtr<nsIContent> cell;
       GetNamedCell(realRow, aColID, getter_AddRefs(cell));
@@ -562,9 +552,8 @@ nsTreeContentView::SetTree(nsITreeBoxObject* aTree)
 
     mRoot = do_QueryInterface(element);
 
-    // Add ourselfs to document's observers.
-    nsCOMPtr<nsIDocument> document;
-    mRoot->GetDocument(*getter_AddRefs(document));
+    // Add ourselves to document's observers.
+    nsIDocument* document = mRoot->GetDocument();
     if (document) {
       document->AddObserver(this);
       mDocument = document;
@@ -592,10 +581,21 @@ nsTreeContentView::ToggleOpenState(PRInt32 aIndex)
   // We don't serialize content right here, since content might be generated
   // lazily.
   Row* row = (Row*)mRows[aIndex];
-  if (row->IsOpen())
-    row->mContent->UnsetAttr(kNameSpaceID_None, nsXULAtoms::open, PR_TRUE);
-  else
-    row->mContent->SetAttr(kNameSpaceID_None, nsXULAtoms::open, NS_LITERAL_STRING("true"), PR_TRUE);
+
+  if (row->mContent->Tag() == nsHTMLAtoms::optgroup &&
+      row->mContent->IsContentOfType(nsIContent::eHTML)) {
+    // we don't use an attribute for optgroup's open state
+    PRBool wasOpen = row->IsOpen();
+    if (wasOpen)
+      CloseContainer(aIndex);
+    else
+      OpenContainer(aIndex);
+  } else {
+    if (row->IsOpen())
+      row->mContent->SetAttr(kNameSpaceID_None, nsXULAtoms::open, NS_LITERAL_STRING("false"), PR_TRUE);
+    else
+      row->mContent->SetAttr(kNameSpaceID_None, nsXULAtoms::open, NS_LITERAL_STRING("true"), PR_TRUE);
+  }
 
   return NS_OK;
 }
@@ -684,51 +684,7 @@ nsTreeContentView::GetIndexOfItem(nsIDOMElement* aItem, PRInt32* _retval)
   return NS_OK;
 }
 
-NS_IMETHODIMP
-nsTreeContentView::BeginUpdate(nsIDocument *aDocument)
-{
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsTreeContentView::EndUpdate(nsIDocument *aDocument)
-{
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsTreeContentView::BeginLoad(nsIDocument *aDocument)
-{
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsTreeContentView::EndLoad(nsIDocument *aDocument)
-{
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsTreeContentView::BeginReflow(nsIDocument *aDocument, nsIPresShell* aShell)
-{
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsTreeContentView::EndReflow(nsIDocument *aDocument, nsIPresShell* aShell)
-{
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsTreeContentView::ContentChanged(nsIDocument *aDocument,
-                                     nsIContent* aContent,
-                                     nsISupports* aSubContent)
-{
-  return NS_OK;
-}
-
-NS_IMETHODIMP
+void
 nsTreeContentView::ContentStatesChanged(nsIDocument* aDocument,
                                            nsIContent* aContent1,
                                            nsIContent* aContent2,
@@ -737,57 +693,50 @@ nsTreeContentView::ContentStatesChanged(nsIDocument* aDocument,
   if (!aContent1 || !mSelection ||
       !aContent1->IsContentOfType(nsIContent::eHTML) ||
       !(aStateMask & NS_EVENT_STATE_CHECKED))
-    return NS_OK;
-        
-  nsCOMPtr<nsIAtom> contentTag;
-  aContent1->GetTag(*getter_AddRefs(contentTag));   
-  if (contentTag == nsHTMLAtoms::option) {
+    return;
+
+  if (aContent1->Tag() == nsHTMLAtoms::option) {
     // update the selected state for this node
     PRInt32 index = FindContent(aContent1);
     if (index >= 0)
       mSelection->ToggleSelect(index);
   }
-  
-  return NS_OK;
 }
 
-NS_IMETHODIMP
+void
 nsTreeContentView::AttributeChanged(nsIDocument *aDocument,
-                                       nsIContent*  aContent,
-                                       PRInt32      aNameSpaceID,
-                                       nsIAtom*     aAttribute,
-                                       PRInt32      aModType, 
-                                       PRInt32      aHint)
+                                    nsIContent*  aContent,
+                                    PRInt32      aNameSpaceID,
+                                    nsIAtom*     aAttribute,
+                                    PRInt32      aModType)
 {
   // Make sure this notification concerns us.
   // First check the tag to see if it's one that we care about.
-  nsCOMPtr<nsIAtom> tag;
-  aContent->GetTag(*getter_AddRefs(tag));
+  nsIAtom *tag = aContent->Tag();
 
-  if (aContent->IsContentOfType(nsIContent::eXUL))
-  {
+  if (aContent->IsContentOfType(nsIContent::eXUL)) {
     if (tag != nsXULAtoms::treecol &&
         tag != nsXULAtoms::treeitem &&
         tag != nsXULAtoms::treeseparator &&
         tag != nsXULAtoms::treerow &&
         tag != nsXULAtoms::treecell)
-      return NS_OK;
+      return;
   }
   else
-    return NS_OK;
+    return;
 
   // If we have a legal tag, go up to the tree and make sure that it's ours.
   nsCOMPtr<nsIContent> parent = aContent;
-  nsCOMPtr<nsIAtom> parentTag;
+  nsINodeInfo *ni = nsnull;
   do {
-    nsCOMPtr<nsIContent> temp = parent;
-    temp->GetParent(*getter_AddRefs(parent));
+    parent = parent->GetParent();
     if (parent)
-      parent->GetTag(*getter_AddRefs(parentTag));
-  } while (parent && parentTag != nsXULAtoms::tree);
+      ni = parent->GetNodeInfo();
+  } while (parent && !ni->Equals(nsXULAtoms::tree, kNameSpaceID_XUL));
+
   if (parent != mRoot) {
     // This is not for us, we can bail out.
-    return NS_OK;
+    return;
   }
 
   // Handle changes of the hidden attribute.
@@ -802,28 +751,28 @@ nsTreeContentView::AttributeChanged(nsIDocument *aDocument,
       // Hide this row along with its children.
       PRInt32 count;
       RemoveRow(index, &count);
-      mBoxObject->RowCountChanged(index, -count);
+      if (mBoxObject)
+        mBoxObject->RowCountChanged(index, -count);
     }
     else if (!hidden && index < 0) {
       // Show this row along with its children.
-      nsCOMPtr<nsIContent> container;
-      aContent->GetParent(*getter_AddRefs(container));
+      nsCOMPtr<nsIContent> container = aContent->GetParent();
       if (container) {
-        nsCOMPtr<nsIContent> parent;
-        container->GetParent(*getter_AddRefs(parent));
+        nsCOMPtr<nsIContent> parent = container->GetParent();
         if (parent)
           InsertRowFor(parent, container, aContent);
       }
     }
 
-    return NS_OK;
+    return;
   }
 
   if (tag == nsXULAtoms::treecol) {
     if (aAttribute == nsXULAtoms::properties) {
       nsAutoString id;
       aContent->GetAttr(kNameSpaceID_None, nsHTMLAtoms::id, id);
-      mBoxObject->InvalidateColumn(id.get());
+      if (mBoxObject)
+        mBoxObject->InvalidateColumn(id.get());
     }
   }
   else if (tag == nsXULAtoms::treeitem) {
@@ -835,7 +784,8 @@ nsTreeContentView::AttributeChanged(nsIDocument *aDocument,
         aContent->GetAttr(kNameSpaceID_None, nsXULAtoms::container, container);
         PRBool isContainer = container.Equals(NS_LITERAL_STRING("true"));
         row->SetContainer(isContainer);
-        mBoxObject->InvalidateRow(index);
+        if (mBoxObject)
+          mBoxObject->InvalidateRow(index);
       }
       else if (aAttribute == nsXULAtoms::open) {
         nsAutoString open;
@@ -852,31 +802,25 @@ nsTreeContentView::AttributeChanged(nsIDocument *aDocument,
         aContent->GetAttr(kNameSpaceID_None, nsXULAtoms::empty, empty);
         PRBool isEmpty = empty.Equals(NS_LITERAL_STRING("true"));
         row->SetEmpty(isEmpty);
-        mBoxObject->InvalidateRow(index);
+        if (mBoxObject)
+          mBoxObject->InvalidateRow(index);
       }
     }
   }
   else if (tag == nsXULAtoms::treeseparator) {
     PRInt32 index = FindContent(aContent);
     if (index >= 0) {
-      if (aAttribute == nsXULAtoms::properties) {
-        Row* row = (Row*)mRows[index];
-        row->SetProperty(nsnull);
-        ParseProperties(aContent, &row->mProperty);
+      if (aAttribute == nsXULAtoms::properties && mBoxObject) {
         mBoxObject->InvalidateRow(index);
       }
     }
   }
   else if (tag == nsXULAtoms::treerow) {
     if (aAttribute == nsXULAtoms::properties) {
-      nsCOMPtr<nsIContent> parent;
-      aContent->GetParent(*getter_AddRefs(parent));
+      nsCOMPtr<nsIContent> parent = aContent->GetParent();
       if (parent) {
         PRInt32 index = FindContent(parent);
-        if (index >= 0) {
-          Row* row = (Row*)mRows[index];
-          row->SetProperty(nsnull);
-          ParseProperties(aContent, &row->mProperty);
+        if (index >= 0 && mBoxObject) {
           mBoxObject->InvalidateRow(index);
         }
       }
@@ -888,14 +832,12 @@ nsTreeContentView::AttributeChanged(nsIDocument *aDocument,
         aAttribute == nsXULAtoms::mode ||
         aAttribute == nsHTMLAtoms::value ||
         aAttribute == nsHTMLAtoms::label) {
-      nsCOMPtr<nsIContent> parent;
-      aContent->GetParent(*getter_AddRefs(parent));
+      nsIContent* parent = aContent->GetParent();
       if (parent) {
-        nsCOMPtr<nsIContent> grandParent;
-        parent->GetParent(*getter_AddRefs(grandParent));
+        nsCOMPtr<nsIContent> grandParent = parent->GetParent();
         if (grandParent) {
           PRInt32 index = FindContent(grandParent);
-          if (index >= 0) {
+          if (index >= 0 && mBoxObject) {
             // XXX Should we make an effort to invalidate only cell ?
             mBoxObject->InvalidateRow(index);
           }
@@ -903,108 +845,103 @@ nsTreeContentView::AttributeChanged(nsIDocument *aDocument,
       }
     }
   }
-
-  return NS_OK;
 }
 
-NS_IMETHODIMP
+void
 nsTreeContentView::ContentAppended(nsIDocument *aDocument,
-                                      nsIContent* aContainer,
-                                      PRInt32     aNewIndexInContainer)
+                                   nsIContent* aContainer,
+                                   PRInt32     aNewIndexInContainer)
 {
-  nsCOMPtr<nsIContent> child;
-  aContainer->ChildAt(aNewIndexInContainer, *getter_AddRefs(child));
+  nsIContent *child = aContainer->GetChildAt(aNewIndexInContainer);
   ContentInserted(aDocument, aContainer, child, aNewIndexInContainer);
-
-  return NS_OK;
 }
 
-NS_IMETHODIMP
+void
 nsTreeContentView::ContentInserted(nsIDocument *aDocument,
                                       nsIContent* aContainer,
                                       nsIContent* aChild,
                                       PRInt32 aIndexInContainer)
 {
+  NS_ASSERTION(aChild, "null ptr");
+
   // Make sure this notification concerns us.
   // First check the tag to see if it's one that we care about.
-  nsCOMPtr<nsIAtom> childTag;
-  aChild->GetTag(*getter_AddRefs(childTag));
+  nsIAtom *childTag = aChild->Tag();
 
   if (aChild->IsContentOfType(nsIContent::eHTML)) {
-    if (childTag != nsHTMLAtoms::option)
-      return NS_OK;
+    if (childTag != nsHTMLAtoms::option &&
+        childTag != nsHTMLAtoms::optgroup)
+      return;
   } else if (aChild->IsContentOfType(nsIContent::eXUL)) {
     if (childTag != nsXULAtoms::treeitem &&
         childTag != nsXULAtoms::treeseparator &&
         childTag != nsXULAtoms::treechildren &&
         childTag != nsXULAtoms::treerow &&
         childTag != nsXULAtoms::treecell)
-      return NS_OK;
+      return;
   } else
-    return NS_OK;
+    return;
 
   // If we have a legal tag, go up to the tree/select and make sure
   // that it's ours.
-  nsCOMPtr<nsIContent> element = aContainer;
-  nsCOMPtr<nsIAtom> parentTag;
-  
-  while (element) {
-    element->GetTag(*getter_AddRefs(parentTag));
+
+  for (nsIContent* element = aContainer; element; element = element->GetParent()) {
+    nsIAtom *parentTag = element->Tag();
     if ((element->IsContentOfType(nsIContent::eXUL) && parentTag == nsXULAtoms::tree) ||
         (element->IsContentOfType(nsIContent::eHTML) && parentTag == nsHTMLAtoms::select))
       if (element == mRoot) // this is for us, stop looking
         break;
       else // this is not for us, we can bail out
-        return NS_OK;
-    nsCOMPtr<nsIContent> temp = element;
-    temp->GetParent(*getter_AddRefs(element));
+        return;
   }
 
   if (childTag == nsXULAtoms::treeitem ||
       childTag == nsXULAtoms::treeseparator) {
-    nsCOMPtr<nsIContent> parent;
-    aContainer->GetParent(*getter_AddRefs(parent));
+    nsCOMPtr<nsIContent> parent = aContainer->GetParent();
     if (parent)
       InsertRowFor(parent, aContainer, aChild);
   }
+  else if (childTag == nsHTMLAtoms::optgroup) {
+    InsertRowFor(aContainer, aContainer, aChild);
+  }
   else if (childTag == nsHTMLAtoms::option) {
     PRInt32 count;
     PRInt32 parentIndex = FindContent(aContainer);
     InsertRow(parentIndex, aIndexInContainer, aChild, &count);
-    mBoxObject->RowCountChanged(parentIndex + aIndexInContainer + 1, count);
+    if (mBoxObject)
+      mBoxObject->RowCountChanged(parentIndex + aIndexInContainer + 1, count);
   }
   else if (childTag == nsXULAtoms::treechildren) {
     PRInt32 index = FindContent(aContainer);
     if (index >= 0) {
       Row* row = (Row*)mRows[index];
       row->SetEmpty(PR_FALSE);
-      mBoxObject->InvalidateRow(index);
+      if (mBoxObject)
+        mBoxObject->InvalidateRow(index);
       if (row->IsContainer() && row->IsOpen()) {
         PRInt32 count;
         EnsureSubtree(index, &count);
-        mBoxObject->RowCountChanged(index + 1, count);
+        if (mBoxObject)
+          mBoxObject->RowCountChanged(index + 1, count);
       }
     }
   }
   else if (childTag == nsXULAtoms::treerow) {
     PRInt32 index = FindContent(aContainer);
-    if (index >= 0)
+    if (index >= 0 && mBoxObject)
       mBoxObject->InvalidateRow(index);
   }
   else if (childTag == nsXULAtoms::treecell) {
-    nsCOMPtr<nsIContent> parent;
-    aContainer->GetParent(*getter_AddRefs(parent));
+    nsCOMPtr<nsIContent> parent = aContainer->GetParent();
     if (parent) {
       PRInt32 index = FindContent(parent);
-      if (index >= 0)
+      if (index >= 0 && mBoxObject)
         mBoxObject->InvalidateRow(index);
     }
   }
-
-  return NS_OK;
 }
 
-NS_IMETHODIMP
+void
 nsTreeContentView::ContentReplaced(nsIDocument *aDocument,
                                       nsIContent* aContainer,
                                       nsIContent* aOldChild,
@@ -1013,59 +950,56 @@ nsTreeContentView::ContentReplaced(nsIDocument *aDocument,
 {
   ContentRemoved(aDocument, aContainer, aOldChild, aIndexInContainer);
   ContentInserted(aDocument, aContainer, aNewChild, aIndexInContainer);
-
-  return NS_OK;
 }
 
-NS_IMETHODIMP
+void
 nsTreeContentView::ContentRemoved(nsIDocument *aDocument,
                                      nsIContent* aContainer,
                                      nsIContent* aChild,
                                      PRInt32 aIndexInContainer)
 {
+  NS_ASSERTION(aChild, "null ptr");
+
   // Make sure this notification concerns us.
   // First check the tag to see if it's one that we care about.
-  nsCOMPtr<nsIAtom> tag;
-  aChild->GetTag(*getter_AddRefs(tag));
+  nsIAtom *tag = aChild->Tag();
 
   if (aChild->IsContentOfType(nsIContent::eHTML)) {
-    if (tag != nsHTMLAtoms::option)
-      return NS_OK;
+    if (tag != nsHTMLAtoms::option &&
+        tag != nsHTMLAtoms::optgroup)
+      return;
   } else if (aChild->IsContentOfType(nsIContent::eXUL)) {
     if (tag != nsXULAtoms::treeitem &&
         tag != nsXULAtoms::treeseparator &&
         tag != nsXULAtoms::treechildren &&
         tag != nsXULAtoms::treerow &&
         tag != nsXULAtoms::treecell)
-      return NS_OK;
+      return;
   } else
-    return NS_OK;
+    return;
 
   // If we have a legal tag, go up to the tree/select and make sure
   // that it's ours.
-  nsCOMPtr<nsIContent> element = aContainer;
-  nsCOMPtr<nsIAtom> parentTag;
-  
-  while (element) {
-    element->GetTag(*getter_AddRefs(parentTag));
+  for (nsIContent* element = aContainer; element; element = element->GetParent()) {
+    nsIAtom *parentTag = element->Tag();
     if ((element->IsContentOfType(nsIContent::eXUL) && parentTag == nsXULAtoms::tree) || 
         (element->IsContentOfType(nsIContent::eHTML) && parentTag == nsHTMLAtoms::select))
       if (element == mRoot) // this is for us, stop looking
         break;
       else // this is not for us, we can bail out
-        return NS_OK;
-    nsCOMPtr<nsIContent> temp = element;
-    temp->GetParent(*getter_AddRefs(element));
+        return;
   }
 
   if (tag == nsXULAtoms::treeitem ||
       tag == nsXULAtoms::treeseparator ||
-      tag == nsHTMLAtoms::option) {
+      tag == nsHTMLAtoms::option ||
+      tag == nsHTMLAtoms::optgroup) {
     PRInt32 index = FindContent(aChild);
     if (index >= 0) {
       PRInt32 count;
       RemoveRow(index, &count);
-      mBoxObject->RowCountChanged(index, -count);
+      if (mBoxObject)
+        mBoxObject->RowCountChanged(index, -count);
     }
   }
   else if (tag == nsXULAtoms::treechildren) {
@@ -1076,94 +1010,44 @@ nsTreeContentView::ContentRemoved(nsIDocument *aDocument,
       PRInt32 count;
       RemoveSubtree(index, &count);
       // Invalidate also the row to update twisty.
-      mBoxObject->InvalidateRow(index);
-      mBoxObject->RowCountChanged(index + 1, -count);
+      if (mBoxObject) {
+        mBoxObject->InvalidateRow(index);
+        mBoxObject->RowCountChanged(index + 1, -count);
+      }
     } else {
-      nsCOMPtr<nsIAtom> containerTag;
-      aContainer->GetTag(*getter_AddRefs(containerTag));
-      if (containerTag == nsXULAtoms::tree ) {
+      if (aContainer->Tag() == nsXULAtoms::tree) {
+        PRInt32 count = mRows.Count();
         ClearRows();
-        mBoxObject->Invalidate();
+        if (count && mBoxObject)
+          mBoxObject->RowCountChanged(0, -count);
       }
     }
   }
   else if (tag == nsXULAtoms::treerow) {
     PRInt32 index = FindContent(aContainer);
-    if (index >= 0)
+    if (index >= 0 && mBoxObject)
       mBoxObject->InvalidateRow(index);
   }
   else if (tag == nsXULAtoms::treecell) {
-    nsCOMPtr<nsIContent> parent;
-    aContainer->GetParent(*getter_AddRefs(parent));
+    nsCOMPtr<nsIContent> parent = aContainer->GetParent();
     if (parent) {
       PRInt32 index = FindContent(parent);
-      if (index >= 0)
+      if (index >= 0 && mBoxObject)
         mBoxObject->InvalidateRow(index);
     }
   }
-
-  return NS_OK;
 }
 
-NS_IMETHODIMP
-nsTreeContentView::StyleSheetAdded(nsIDocument *aDocument,
-                                      nsIStyleSheet* aStyleSheet)
-{
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsTreeContentView::StyleSheetRemoved(nsIDocument *aDocument,
-                                        nsIStyleSheet* aStyleSheet)
-{
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsTreeContentView::StyleSheetDisabledStateChanged(nsIDocument *aDocument,
-                                                     nsIStyleSheet* aStyleSheet,
-                                                     PRBool aDisabled)
-{
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsTreeContentView::StyleRuleChanged(nsIDocument *aDocument,
-                                       nsIStyleSheet* aStyleSheet,
-                                       nsIStyleRule* aStyleRule,
-                                       PRInt32 aHint)
-{
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsTreeContentView::StyleRuleAdded(nsIDocument *aDocument,
-                                     nsIStyleSheet* aStyleSheet,
-                                     nsIStyleRule* aStyleRule)
-{
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsTreeContentView::StyleRuleRemoved(nsIDocument *aDocument,
-                                       nsIStyleSheet* aStyleSheet,
-                                       nsIStyleRule* aStyleRule)
-{
-  return NS_OK;
-}
-
-NS_IMETHODIMP
+void
 nsTreeContentView::DocumentWillBeDestroyed(nsIDocument *aDocument)
 {
-  // Remove ourselfs from document's observers.
+  // Remove ourselves from mDocument's observers.
   if (mDocument) {
     mDocument->RemoveObserver(this);
     mDocument = nsnull;
   }
 
   ClearRows();
-
-  return NS_OK;
 }
 
 
@@ -1174,9 +1058,9 @@ nsTreeContentView::Serialize(nsIContent* aContent, PRInt32 aParentIndex, PRInt32
   ChildIterator iter, last;
   for (ChildIterator::Init(aContent, &iter, &last); iter != last; ++iter) {
     nsCOMPtr<nsIContent> content = *iter;
-    nsCOMPtr<nsIAtom> tag;
-    content->GetTag(*getter_AddRefs(tag));
+    nsIAtom *tag = content->Tag();
     PRInt32 count = aRows.Count();
+
     if (content->IsContentOfType(nsIContent::eXUL)) {
       if (tag == nsXULAtoms::treeitem)
         SerializeItem(content, aParentIndex, aIndex, aRows);
@@ -1185,6 +1069,8 @@ nsTreeContentView::Serialize(nsIContent* aContent, PRInt32 aParentIndex, PRInt32
     } else if (content->IsContentOfType(nsIContent::eHTML)) {
       if (tag == nsHTMLAtoms::option)
         SerializeOption(content, aParentIndex, aIndex, aRows);
+      else if (tag == nsHTMLAtoms::optgroup)
+        SerializeOptGroup(content, aParentIndex, aIndex, aRows);
     }
     *aIndex += aRows.Count() - count;
   }
@@ -1201,11 +1087,6 @@ nsTreeContentView::SerializeItem(nsIContent* aContent, PRInt32 aParentIndex, PRI
   Row* row = Row::Create(mAllocator, aContent, aParentIndex);
   aRows.AppendElement(row);
 
-  nsCOMPtr<nsIContent> realRow;
-  nsTreeUtils::GetImmediateChild(aContent, nsXULAtoms::treerow, getter_AddRefs(realRow));
-  if (realRow)
-    ParseProperties(realRow, &row->mProperty);
-
   nsAutoString container;
   aContent->GetAttr(kNameSpaceID_None, nsXULAtoms::container, container);
   if (container.Equals(NS_LITERAL_STRING("true"))) {
@@ -1245,13 +1126,11 @@ nsTreeContentView::SerializeSeparator(nsIContent* aContent, PRInt32 aParentIndex
   Row* row = Row::Create(mAllocator, aContent, aParentIndex);
   row->SetSeparator(PR_TRUE);
   aRows.AppendElement(row);
-
-  ParseProperties(aContent, &row->mProperty);
 }
 
 void
 nsTreeContentView::SerializeOption(nsIContent* aContent, PRInt32 aParentIndex,
-                                       PRInt32* aIndex, nsVoidArray& aRows)
+                                   PRInt32* aIndex, nsVoidArray& aRows)
 {
   Row* row = Row::Create(mAllocator, aContent, aParentIndex);
   aRows.AppendElement(row);
@@ -1267,43 +1146,75 @@ nsTreeContentView::SerializeOption(nsIContent* aContent, PRInt32 aParentIndex,
 }
 
 void
-nsTreeContentView::GetIndexInSubtree(nsIContent* aContainer, nsIContent* aContent, PRInt32* aIndex)
+nsTreeContentView::SerializeOptGroup(nsIContent* aContent, PRInt32 aParentIndex,
+                                     PRInt32* aIndex, nsVoidArray& aRows)
 {
-  ChildIterator iter, last;
-  for (ChildIterator::Init(aContainer, &iter, &last); iter != last; ++iter) {
-    nsCOMPtr<nsIContent> content = *iter;
+  Row* row = Row::Create(mAllocator, aContent, aParentIndex);
+  aRows.AppendElement(row);
+  row->SetContainer(PR_TRUE);
+  row->SetOpen(PR_TRUE);
+
+  nsCOMPtr<nsIContent> child;
+  nsTreeUtils::GetImmediateChild(aContent, nsHTMLAtoms::option, getter_AddRefs(child));
+  if (child) {
+    // Now, recursively serialize our child.
+    PRInt32 count = aRows.Count();
+    PRInt32 index = 0;
+    Serialize(aContent, aParentIndex + *aIndex + 1, &index, aRows);
+    row->mSubtreeSize += aRows.Count() - count;
+  }
+  else
+    row->SetEmpty(PR_TRUE);
+}
+
+void
+nsTreeContentView::GetIndexInSubtree(nsIContent* aContainer,
+                                     nsIContent* aContent, PRInt32* aIndex)
+{
+  PRUint32 childCount = aContainer->GetChildCount();
+  for (PRUint32 i = 0; i < childCount; i++) {
+    nsIContent *content = aContainer->GetChildAt(i);
+
     if (content == aContent)
       break;
 
-    nsCOMPtr<nsIAtom> tag;
-    content->GetTag(*getter_AddRefs(tag));
-    if (tag == nsXULAtoms::treeitem) {
-      nsAutoString hidden;
-      content->GetAttr(kNameSpaceID_None, nsHTMLAtoms::hidden, hidden);
-      if (! hidden.Equals(NS_LITERAL_STRING("true"))) {
-        (*aIndex)++;
-        nsAutoString container;
-        content->GetAttr(kNameSpaceID_None, nsXULAtoms::container, container);
-        if (container.Equals(NS_LITERAL_STRING("true"))) {
-          nsAutoString open;
-          content->GetAttr(kNameSpaceID_None, nsXULAtoms::open, open);
-          if (open.Equals(NS_LITERAL_STRING("true"))) {
-            nsCOMPtr<nsIContent> child;
-            nsTreeUtils::GetImmediateChild(content, nsXULAtoms::treechildren, getter_AddRefs(child));
-            if (child)
-              GetIndexInSubtree(child, aContent, aIndex);
+    nsIAtom *tag = content->Tag();
+
+    if (content->IsContentOfType(nsIContent::eXUL)) {
+      if (tag == nsXULAtoms::treeitem) {
+        nsAutoString hidden;
+        content->GetAttr(kNameSpaceID_None, nsHTMLAtoms::hidden, hidden);
+        if (! hidden.Equals(NS_LITERAL_STRING("true"))) {
+          (*aIndex)++;
+          nsAutoString container;
+          content->GetAttr(kNameSpaceID_None, nsXULAtoms::container, container);
+          if (container.Equals(NS_LITERAL_STRING("true"))) {
+            nsAutoString open;
+            content->GetAttr(kNameSpaceID_None, nsXULAtoms::open, open);
+            if (open.Equals(NS_LITERAL_STRING("true"))) {
+              nsCOMPtr<nsIContent> child;
+              nsTreeUtils::GetImmediateChild(content, nsXULAtoms::treechildren, getter_AddRefs(child));
+              if (child)
+                GetIndexInSubtree(child, aContent, aIndex);
+            }
           }
         }
       }
+      else if (tag == nsXULAtoms::treeseparator) {
+        nsAutoString hidden;
+        content->GetAttr(kNameSpaceID_None, nsHTMLAtoms::hidden, hidden);
+        if (! hidden.Equals(NS_LITERAL_STRING("true")))
+          (*aIndex)++;
+      }
     }
-    else if (tag == nsXULAtoms::treeseparator) {
-      nsAutoString hidden;
-      content->GetAttr(kNameSpaceID_None, nsHTMLAtoms::hidden, hidden);
-      if (! hidden.Equals(NS_LITERAL_STRING("true")))
+    else if (content->IsContentOfType(nsIContent::eHTML)) {
+      if (tag == nsHTMLAtoms::optgroup) {
+        (*aIndex)++;
+        GetIndexInSubtree(content, aContent, aIndex);
+      }
+      else if (tag == nsHTMLAtoms::option)
         (*aIndex)++;
     }
-    else if (tag == nsHTMLAtoms::option)
-      (*aIndex)++;
   }
 }
 
@@ -1311,11 +1222,16 @@ void
 nsTreeContentView::EnsureSubtree(PRInt32 aIndex, PRInt32* aCount)
 {
   Row* row = (Row*)mRows[aIndex];
+
   nsCOMPtr<nsIContent> child;
-  nsTreeUtils::GetImmediateChild(row->mContent, nsXULAtoms::treechildren, getter_AddRefs(child));
-  if (! child) {
-    *aCount = 0;
-    return;
+  if (row->mContent->Tag() == nsHTMLAtoms::optgroup)
+    child = row->mContent;
+  else {
+    nsTreeUtils::GetImmediateChild(row->mContent, nsXULAtoms::treechildren, getter_AddRefs(child));
+    if (! child) {
+      *aCount = 0;
+      return;
+    }
   }
 
   nsAutoVoidArray rows;
@@ -1334,14 +1250,34 @@ nsTreeContentView::EnsureSubtree(PRInt32 aIndex, PRInt32* aCount)
   *aCount = count;
 }
 
+void
+nsTreeContentView::RemoveSubtree(PRInt32 aIndex, PRInt32* aCount)
+{
+  Row* row = (Row*)mRows[aIndex];
+  PRInt32 count = row->mSubtreeSize;
+
+  for(PRInt32 i = 0; i < count; i++) {
+    Row* nextRow = (Row*)mRows[aIndex + i + 1];
+    Row::Destroy(mAllocator, nextRow);
+  }
+  mRows.RemoveElementsAt(aIndex + 1, count);
+
+  row->mSubtreeSize -= count;
+  UpdateSubtreeSizes(row->mParentIndex, -count);
+
+  UpdateParentIndexes(aIndex, 0, -count);
+
+  *aCount = count;
+}
+
 void
 nsTreeContentView::InsertRowFor(nsIContent* aParent, nsIContent* aContainer, nsIContent* aChild)
 {
   PRInt32 parentIndex = -1;
   PRBool insertRow = PR_FALSE;
 
-  nsCOMPtr<nsIAtom> parentTag;
-  aParent->GetTag(*getter_AddRefs(parentTag));
+  nsIAtom *parentTag = aParent->Tag();
+
   if ((aParent->IsContentOfType(nsIContent::eXUL) && parentTag == nsXULAtoms::tree) ||
       (aParent->IsContentOfType(nsIContent::eHTML) && parentTag == nsHTMLAtoms::select)) {
     // Allow insertion to the outermost container.
@@ -1366,42 +1302,28 @@ nsTreeContentView::InsertRowFor(nsIContent* aParent, nsIContent* aContainer, nsI
 
     PRInt32 count;
     InsertRow(parentIndex, index, aChild, &count);
-    mBoxObject->RowCountChanged(parentIndex + index + 1, count);
+    if (mBoxObject)
+      mBoxObject->RowCountChanged(parentIndex + index + 1, count);
   }
 }
 
-void
-nsTreeContentView::RemoveSubtree(PRInt32 aIndex, PRInt32* aCount)
-{
-  Row* row = (Row*)mRows[aIndex];
-  PRInt32 count = row->mSubtreeSize;
-
-  for(PRInt32 i = 0; i < count; i++) {
-    Row* nextRow = (Row*)mRows[aIndex + i + 1];
-    Row::Destroy(mAllocator, nextRow);
-  }
-  mRows.RemoveElementsAt(aIndex + 1, count);
-
-  row->mSubtreeSize -= count;
-  UpdateSubtreeSizes(row->mParentIndex, -count);
-
-  UpdateParentIndexes(aIndex, 0, -count);
-
-  *aCount = count;
-}
-
 void
 nsTreeContentView::InsertRow(PRInt32 aParentIndex, PRInt32 aIndex, nsIContent* aContent, PRInt32* aCount)
 {
   nsAutoVoidArray rows;
-  nsCOMPtr<nsIAtom> tag;
-  aContent->GetTag(*getter_AddRefs(tag));
-  if (tag == nsXULAtoms::treeitem)
-    SerializeItem(aContent, aParentIndex, &aIndex, rows);
-  else if (tag == nsXULAtoms::treeseparator)
-    SerializeSeparator(aContent, aParentIndex, &aIndex, rows);
-  else if (tag == nsHTMLAtoms::option)
-    SerializeOption(aContent, aParentIndex, &aIndex, rows);
+  nsIAtom *tag = aContent->Tag();
+  if (aContent->IsContentOfType(nsIContent::eXUL)) {
+    if (tag == nsXULAtoms::treeitem)
+      SerializeItem(aContent, aParentIndex, &aIndex, rows);
+    else if (tag == nsXULAtoms::treeseparator)
+      SerializeSeparator(aContent, aParentIndex, &aIndex, rows);
+  } else if (aContent->IsContentOfType(nsIContent::eHTML)) {
+    if (tag == nsHTMLAtoms::option)
+      SerializeOption(aContent, aParentIndex, &aIndex, rows);
+    else if (tag == nsHTMLAtoms::optgroup)
+      SerializeOptGroup(aContent, aParentIndex, &aIndex, rows);
+  }
+
   mRows.InsertElementsAt(rows, aParentIndex + aIndex + 1);
   PRInt32 count = rows.Count();
 
@@ -1452,8 +1374,10 @@ nsTreeContentView::OpenContainer(PRInt32 aIndex)
 
   PRInt32 count;
   EnsureSubtree(aIndex, &count);
-  mBoxObject->InvalidateRow(aIndex);
-  mBoxObject->RowCountChanged(aIndex + 1, count);
+  if (mBoxObject) {
+    mBoxObject->InvalidateRow(aIndex);
+    mBoxObject->RowCountChanged(aIndex + 1, count);
+  }
 }
 
 
@@ -1465,8 +1389,10 @@ nsTreeContentView::CloseContainer(PRInt32 aIndex)
 
   PRInt32 count;
   RemoveSubtree(aIndex, &count);
-  mBoxObject->InvalidateRow(aIndex);
-  mBoxObject->RowCountChanged(aIndex + 1, -count);
+  if (mBoxObject) {
+    mBoxObject->InvalidateRow(aIndex);
+    mBoxObject->RowCountChanged(aIndex + 1, -count);
+  }
 }
 
 PRInt32
@@ -1503,8 +1429,9 @@ nsTreeContentView::UpdateParentIndexes(PRInt32 aIndex, PRInt32 aSkip, PRInt32 aC
 nsresult
 nsTreeContentView::GetNamedCell(nsIContent* aContainer, const PRUnichar* aColID, nsIContent** aResult)
 {
-  PRInt32 colIndex;
-  mBoxObject->GetColumnIndex(aColID, &colIndex);
+  PRInt32 colIndex = -1;
+  if (mBoxObject)
+    mBoxObject->GetColumnIndex(aColID, &colIndex);
 
   // Traverse through cells, try to find the cell by "ref" attribute or by cell
   // index in a row. "ref" attribute has higher priority.
@@ -1513,9 +1440,8 @@ nsTreeContentView::GetNamedCell(nsIContent* aContainer, const PRUnichar* aColID,
   ChildIterator iter, last;
   for (ChildIterator::Init(aContainer, &iter, &last); iter != last; ++iter) {
     nsCOMPtr<nsIContent> cell = *iter;
-    nsCOMPtr<nsIAtom> tag;
-    cell->GetTag(*getter_AddRefs(tag));
-    if (tag == nsXULAtoms::treecell) {
+
+    if (cell->Tag() == nsXULAtoms::treecell) {
       nsAutoString ref;
       cell->GetAttr(kNameSpaceID_None, nsXULAtoms::ref, ref);
       if (!ref.IsEmpty() && ref.Equals(aColID)) {
@@ -1531,47 +1457,3 @@ nsTreeContentView::GetNamedCell(nsIContent* aContainer, const PRUnichar* aColID,
 
   return NS_OK;
 }
-
-nsresult
-nsTreeContentView::ParseProperties(nsIContent* aContent, Property** aProperty)
-{
-  nsAutoString properties;
-  aContent->GetAttr(kNameSpaceID_None, nsXULAtoms::properties, properties);
-  if (properties.Length()) {
-    Property* lastProperty = *aProperty;
-
-    nsAString::const_iterator end;
-    properties.EndReading(end);
-
-    nsAString::const_iterator iter;
-    properties.BeginReading(iter);
-
-    do {
-      // Skip whitespace
-      while (iter != end && nsCRT::IsAsciiSpace(*iter))
-        ++iter;
-
-      // If only whitespace, we're done
-      if (iter == end)
-        break;
-
-      // Note the first non-whitespace character
-      nsAString::const_iterator first = iter;
-
-      // Advance to the next whitespace character
-      while (iter != end && ! nsCRT::IsAsciiSpace(*iter))
-        ++iter;
-
-      nsCOMPtr<nsIAtom> atom = dont_AddRef(NS_NewAtom(Substring(first, iter)));
-      Property* newProperty = new Property(atom);
-      if (lastProperty)
-        lastProperty->mNext = newProperty;
-      else
-        *aProperty = newProperty;
-      lastProperty = newProperty;
-
-    } while (iter != end);
-  }
-
-  return NS_OK;
-}
