@@ -271,7 +271,7 @@ void nsMapiEntryArray::CleanUp(void)
 
 MOZ_DECL_CTOR_COUNTER(nsAbWinHelper)
 
-ULONG nsAbWinHelper::mEntryCounter = 0 ;
+PRUint32 nsAbWinHelper::mEntryCounter = 0 ;
 // There seems to be a deadlock/auto-destruction issue
 // in MAPI when multiple threads perform init/release 
 // operations at the same time. So I've put a mutex
@@ -431,7 +431,7 @@ BOOL nsAbWinHelper::GetPropertiesUString(const nsMapiEntry& aObject, const ULONG
         ULONG i = 0 ;
 
         for (i = 0 ; i < valueCount ; ++ i) {
-            if (values [i].ulPropTag == aPropertyTags [i]) {
+            if (PROP_ID(values [i].ulPropTag) == PROP_ID(aPropertyTags [i])) {
                 if (PROP_TYPE(values [i].ulPropTag) == PT_STRING8) {
                     nsAutoString temp ;
 
@@ -581,6 +581,9 @@ BOOL nsAbWinHelper::SetPropertiesUString(const nsMapiEntry& aObject, const ULONG
                                          ULONG aNbProperties, nsXPIDLString *aValues) 
 {
     LPSPropValue values = new SPropValue [aNbProperties] ;
+    if (!values)
+        return FALSE ;
+
     ULONG i = 0 ;
     ULONG currentValue = 0 ;
     nsCAutoString alternativeValue ;
@@ -593,10 +596,16 @@ BOOL nsAbWinHelper::SetPropertiesUString(const nsMapiEntry& aObject, const ULONG
         }
         else if (PROP_TYPE(aPropertiesTag [i]) == PT_STRING8) {
             alternativeValue.AssignWithConversion(aValues [i].get()) ;
-            values [currentValue ++].Value.lpszA = nsCRT::strdup(alternativeValue.get()) ;
+            char *av = nsCRT::strdup(alternativeValue.get()) ;
+            if (!av) {
+                retCode = FALSE ;
+                break ;
+            }
+            values [currentValue ++].Value.lpszA = av ;
         }
     }
-    retCode = SetMAPIProperties(aObject, currentValue, values) ;
+    if (retCode)
+        retCode = SetMAPIProperties(aObject, currentValue, values) ;
     for (i = 0 ; i < currentValue ; ++ i) {
         if (PROP_TYPE(aPropertiesTag [i]) == PT_STRING8) {
             nsCRT::free(values [i].Value.lpszA) ;
