@@ -18,19 +18,21 @@
  * Rights Reserved.
  *
  * Contributor(s):
- *   Jan Varga (varga@utcru.sk)
+ *   Jan Varga (varga@nixcorp.com)
  *   Hkan Waara (hwaara@chello.se)
+ *   Neil Rashbrook (neil@parkwaycc.co.uk)
+ *   Seth Spitzer <sspitzer@netscape.com>
  */
 
 /* This is where functions related to the 3 pane window are kept */
 
 // from MailNewsTypes.h
+const nsMsgKey_None = 0xFFFFFFFF;
 const nsMsgViewIndex_None = 0xFFFFFFFF;
-
+const kMailCheckOncePrefName = "mail.startup.enabledMailCheckOnce";
 
 var gFolderTree; 
 var gMessagePane;
-var gMessagePaneFrame;
 var gThreadTree;
 var gSearchInput;
 
@@ -51,12 +53,14 @@ var gNextMessageAfterLoad = null;
 var gNextMessageViewIndexAfterDelete = -2;
 var gCurrentlyDisplayedMessage=nsMsgViewIndex_None;
 var gStartFolderUri = null;
-var gStartMsgKey = -1;
-
+var gStartMsgKey = nsMsgKey_None;
+var gSearchEmailAddress = null;
+var gRightMouseButtonDown = false;
 // Global var to keep track of which row in the thread pane has been selected
 // This is used to make sure that the row with the currentIndex has the selection
 // after a Delete or Move of a message that has a row index less than currentIndex.
 var gThreadPaneCurrentSelectedIndex = -1;
+var gLoadStartFolder = true;
 
 // Global var to keep track of if the 'Delete Message' or 'Move To' thread pane
 // context menu item was triggered.  This helps prevent the tree view from
@@ -68,6 +72,25 @@ var gHaveLoadedMessage;
 
 var gDisplayStartupPage = false;
 
+var gNotifyDefaultInboxLoadedOnStartup = false;
+
+function SelectAndScrollToKey(aMsgKey)
+{
+  // select the desired message
+  // if the key isn't found, we won't select anything
+  gDBView.selectMsgByKey(aMsgKey);
+  
+  // is there a selection?
+  // if not, bail out.
+  var indicies = GetSelectedIndices(gDBView);
+  if (!indicies || !indicies.length)
+    return false;
+
+  // now scroll to it
+  EnsureRowInThreadTreeIsVisible(indicies[0]);
+  return true;
+}
+
 // the folderListener object
 var folderListener = {
     OnItemAdded: function(parentItem, item, view) { },
@@ -83,7 +106,7 @@ var folderListener = {
 
       //if we don't have a folder loaded, don't bother.
       if(currentURI) {
-        if(property.GetUnicode() == "TotalMessages" || property.GetUnicode() == "TotalUnreadMessages") {
+        if(property.toString() == "TotalMessages" || property.toString() == "TotalUnreadMessages") {
           var folder = item.QueryInterface(Components.interfaces.nsIMsgFolder);
           if(folder) {
             var folderResource = folder.QueryInterface(Components.interfaces.nsIRDFResource); 
@@ -104,86 +127,155 @@ var folderListener = {
     OnItemPropertyFlagChanged: function(item, property, oldFlag, newFlag) { },
 
     OnItemEvent: function(folder, event) {
-       var eventType = event.GetUnicode();
+       var eventType = event.toString();
        if (eventType == "FolderLoaded") {
          if (folder) {
-           var resource = folder.QueryInterface(Components.interfaces.nsIRDFResource);
-           if(resource) {
-             var uri = resource.Value;
-             if(uri == gCurrentFolderToReroot) {
-               gCurrentFolderToReroot="";
-               var msgFolder = folder.QueryInterface(Components.interfaces.nsIMsgFolder);
-               if(msgFolder) {
-                 msgFolder.endFolderLoading();
-                 // suppress command updating when rerooting the folder
-                 // when rerooting, we'll be clearing the selection
-                 // which will cause us to update commands.
-                 if (gDBView) {
-                   gDBView.suppressCommandUpdating = true;
-                   // if the db's view isn't set, something went wrong and we should reroot
-                   // the folder, which will re-open the view.
-                   if (!gDBView.db)
-                    gRerootOnFolderLoad = true;
-                 }
-                 if (gRerootOnFolderLoad)
-                   RerootFolder(uri, msgFolder, gCurrentLoadingFolderViewType, gCurrentLoadingFolderViewFlags, gCurrentLoadingFolderSortType, gCurrentLoadingFolderSortOrder);
+           var uri = folder.URI;
+           if (uri == gCurrentFolderToReroot) {
+             gQSViewIsDirty = true;
+             gCurrentFolderToReroot = null;
+             var msgFolder = folder.QueryInterface(Components.interfaces.nsIMsgFolder);
+             if(msgFolder) {
+               msgFolder.endFolderLoading();
+               // suppress command updating when rerooting the folder
+               // when rerooting, we'll be clearing the selection
+               // which will cause us to update commands.
+               if (gDBView) {
+                 gDBView.suppressCommandUpdating = true;
+                 // if the db's view isn't set, something went wrong and we should reroot
+                 // the folder, which will re-open the view.
+                 if (!gDBView.db)
+                   gRerootOnFolderLoad = true;
+               }
+               if (gRerootOnFolderLoad)
+                 RerootFolder(uri, msgFolder, gCurrentLoadingFolderViewType, gCurrentLoadingFolderViewFlags, gCurrentLoadingFolderSortType, gCurrentLoadingFolderSortOrder);
 
-                 var db = msgFolder.getMsgDatabase(msgWindow);
-                 if (db) 
-                   db.resetHdrCacheSize(100);
+               if (gDBView) 
+                 gDBView.suppressCommandUpdating = false;
+
+               gIsEditableMsgFolder = IsSpecialFolder(msgFolder, MSG_FOLDER_FLAG_DRAFTS);
+
+               gCurrentLoadingFolderSortType = 0;
+               gCurrentLoadingFolderSortOrder = 0;
+               gCurrentLoadingFolderViewType = 0;
+               gCurrentLoadingFolderViewFlags = 0;
+
+               var scrolled = false;
+
+               LoadCurrentlyDisplayedMessage();  //used for rename folder msg loading after folder is loaded.
+
+               if (gStartMsgKey != nsMsgKey_None) {
+                 scrolled = SelectAndScrollToKey(gStartMsgKey);
+                 gStartMsgKey = nsMsgKey_None;
+               }
+
+               if (gNextMessageAfterLoad) {
+                 var type = gNextMessageAfterLoad;
+                 gNextMessageAfterLoad = null;
+
+                 // scroll to and select the proper message
+                 scrolled = ScrollToMessage(type, true, true /* selectMessage */);
+               }
+             }
+           }
+           if (uri == gCurrentLoadingFolderURI) {
+             // NOTE,
+             // if you change the scrolling code below,
+             // double check the scrolling logic in
+             // searchBar.js, restorePreSearchView()
+
+             gCurrentLoadingFolderURI = "";
+
+             // if we didn't just scroll, 
+             // scroll to the first new message
+             // but don't select it
+             if (!scrolled)
+               scrolled = ScrollToMessage(nsMsgNavigationType.firstNew, true, false /* selectMessage */);
+
+             if (!scrolled && pref.getBoolPref("mailnews.remember_selected_message")) {
+               // if we failed to scroll to a new message,
+               // reselect the last selected message
+               var lastMessageLoaded = msgFolder.lastMessageLoaded;
                  
-                 if (gDBView) {
-                   gDBView.suppressCommandUpdating = false;
-                 }
-
-                 gIsEditableMsgFolder = IsSpecialFolder(msgFolder, MSG_FOLDER_FLAG_DRAFTS);
-
-                 gCurrentLoadingFolderSortType = 0;
-                 gCurrentLoadingFolderSortOrder = 0;
-                 gCurrentLoadingFolderViewType = 0;
-                 gCurrentLoadingFolderViewFlags = 0;
-
-                 var scrolled = false;
-
-                 LoadCurrentlyDisplayedMessage();  //used for rename folder msg loading after folder is loaded.
-
-                 if (gStartMsgKey != -1) { 
-                   // select the desired message
-                   gDBView.selectMsgByKey(gStartMsgKey);
-                   gStartMsgKey = -1;
-
-                   // now scroll to it
-	           var indicies = GetSelectedIndices(gDBView);
-                   EnsureRowInThreadTreeIsVisible(indicies[0]);
-                   scrolled = true;
-                 }
-                 if (gNextMessageAfterLoad) {
-                   var type = gNextMessageAfterLoad;
-                   gNextMessageAfterLoad = null;
-
-                   // scroll to and select the proper message
-                   scrolled = ScrollToMessage(type, true, true /* selectMessage */);
-                 }
+               if (lastMessageLoaded != nsMsgKey_None) {
+                 scrolled = SelectAndScrollToKey(lastMessageLoaded);
                }
              }
-             if(uri == gCurrentLoadingFolderURI) {
-               gCurrentLoadingFolderURI = "";
-               //Now let's select the first new message if there is one
-               if (!scrolled) {
-                 // if we didn't just scroll, scroll to the first new message
-                 // don't select it though
-                 scrolled = ScrollToMessage(nsMsgNavigationType.firstNew, true, false /* selectMessage */);
-                    
-                 // if we failed to find a new message, scroll to the top
-                 if (!scrolled) {
-                   EnsureRowInThreadTreeIsVisible(0);
+
+             if (!scrolled) {
+               // if we still haven't scrolled,
+               // scroll to the newest, which might be the top or the bottom
+               // depending on our sort order and sort type
+               if (gDBView.sortOrder == nsMsgViewSortOrder.ascending) {
+                 switch (gDBView.sortType) {
+                   case nsMsgViewSortType.byDate: 
+                   case nsMsgViewSortType.byId: 
+                   case nsMsgViewSortType.byThread: 
+                     scrolled = ScrollToMessage(nsMsgNavigationType.lastMessage, true, false /* selectMessage */);
+                     break;
                  }
                }
-               SetBusyCursor(window, false);
-             }
+
+               // if still we haven't scrolled,
+               // scroll to the top.
+               if (!scrolled)
+                 EnsureRowInThreadTreeIsVisible(0);
+             }            
+
+             // NOTE,
+             // if you change the scrolling code above,
+             // double check the scrolling logic in
+             // searchBar.js, restorePreSearchView()
+
+             SetBusyCursor(window, false);
+           }
+           if (gNotifyDefaultInboxLoadedOnStartup && (folder.flags & 0x1000))
+           {
+              var defaultAccount = accountManager.defaultAccount;
+              defaultServer = defaultAccount.incomingServer;
+              var inboxFolder = GetInboxFolder(defaultServer);
+              if (inboxFolder && inboxFolder.URI == folder.URI)
+              {
+                NotifyObservers(null,"defaultInboxLoadedOnStartup",null);
+                gNotifyDefaultInboxLoadedOnStartup = false;
+              }
+           }
+           //folder loading is over, now issue quick search if there is an email address
+           if (gSearchEmailAddress)
+           {
+             Search(gSearchEmailAddress);
+             gSearchEmailAddress = null;
+           } 
+           else if (gDefaultSearchViewTerms)
+           {
+             Search("");
+           }
+           else {
+             ViewChangeByValue(pref.getIntPref("mailnews.view.last"));
            }
          }
        } 
+       else if (eventType == "ImapHdrDownloaded") {
+         if (folder) {
+           var imapFolder = folder.QueryInterface(Components.interfaces.nsIMsgImapMailFolder);
+           if (imapFolder) {
+            var hdrParser = imapFolder.hdrParser;
+            if (hdrParser) {
+              var msgHdr = hdrParser.GetNewMsgHdr();
+              if (msgHdr)
+              {
+                var hdrs = hdrParser.headers;
+                if (hdrs && hdrs.indexOf("X-attachment-size:") > 0) {
+                  msgHdr.OrFlags(0x10000000);  // 0x10000000 is MSG_FLAG_ATTACHMENT
+                }
+                if (hdrs && hdrs.indexOf("X-image-size:") > 0) {
+                  msgHdr.setStringProperty("imageSize", "1");
+                }
+              }
+            }
+           }
+         }
+       }
        else if (eventType == "DeleteOrMoveMsgCompleted") {
          HandleDeleteOrMoveMsgCompleted(folder);
        }     
@@ -193,9 +285,12 @@ var folderListener = {
        else if (eventType == "CompactCompleted") {
          HandleCompactCompleted(folder);
        }
-       else if(eventType == "RenameCompleted") {
+       else if (eventType == "RenameCompleted") {
          SelectFolder(folder.URI);
        }
+       else if (eventType == "JunkStatusChanged") {
+         HandleJunkStatusChanged(folder);
+       }
     }
 }
 
@@ -267,76 +362,142 @@ function HandleDeleteOrMoveMsgFailed(folder)
   // ThreadPaneSelectionChange(true);
 }
 
+// WARNING
+// this is a fragile and complicated function.
+// be careful when hacking on it.
+// don't forget about things like different imap 
+// delete models, multiple views (from multiple thread panes, 
+// search windows, stand alone message windows)
 function HandleDeleteOrMoveMsgCompleted(folder)
 {
+  // you might not have a db view.  this can happen if
+  // biff fires when the 3 pane is set to account central.
+  if (!gDBView)
+    return;
+
   gDBView.onDeleteCompleted(true);
-  if (gNextMessageViewIndexAfterDelete != -2) 
-  {
-    if (IsCurrentLoadedFolder(folder)) 
+
+  if (!IsCurrentLoadedFolder(folder)) {
+    // default value after delete/move/copy is over
+    gNextMessageViewIndexAfterDelete = -2;
+    return;
+  }
+
+  var treeView = gDBView.QueryInterface(Components.interfaces.nsITreeView);
+  var treeSelection = treeView.selection;
+          
+  if (gNextMessageViewIndexAfterDelete == -2) {
+    // a move or delete can cause our selection can change underneath us.
+    // this can happen when the user
+    // deletes message from the stand alone msg window
+    // or the search view, or another 3 pane
+    if (treeSelection.count == 0) {
+      // this can happen if you double clicked a message
+      // in the thread pane, and deleted it from the stand alone msg window
+      // see bug #172392
+      treeSelection.clearSelection();
+      setTitleFromFolder(folder, null);
+      ClearMessagePane();
+      UpdateMailToolbar("delete from another view, 0 rows now selected");
+    }
+    else if (treeSelection.count == 1) {
+      // this can happen if you had two messages selected
+      // in the thread pane, and you deleted one of them from another view
+      // (like the view in the stand alone msg window)
+      // since one item is selected, we should load it.
+      var startIndex = {};
+      var endIndex = {};
+      treeSelection.getRangeAt(0, startIndex, endIndex);
+        
+      // select the selected item, so we'll load it
+      treeSelection.select(startIndex.value); 
+      treeView.selectionChanged();
+
+      EnsureRowInThreadTreeIsVisible(startIndex.value); 
+
+      UpdateMailToolbar("delete from another view, 1 row now selected");
+    }
+    else {
+      // this can happen if you have more than 2 messages selected
+      // in the thread pane, and you deleted one of them from another view
+      // (like the view in the stand alone msg window)
+      // since multiple messages are still selected, do nothing.
+    }
+  }
+  else {
+    if (gNextMessageViewIndexAfterDelete != nsMsgViewIndex_None) 
     {
-      var treeView = gDBView.QueryInterface(Components.interfaces.nsITreeView);
-      var treeSelection = treeView.selection;
-      if (gNextMessageViewIndexAfterDelete != nsMsgViewIndex_None) 
+      var viewSize = treeView.rowCount;
+      if (gNextMessageViewIndexAfterDelete >= viewSize) 
       {
-        viewSize = treeView.rowCount;
-        if (gNextMessageViewIndexAfterDelete >= viewSize) 
-        {
-          if (viewSize > 0)
-            gNextMessageViewIndexAfterDelete = viewSize - 1;
-          else
-          {           
-            gNextMessageViewIndexAfterDelete = nsMsgViewIndex_None;
+        if (viewSize > 0)
+          gNextMessageViewIndexAfterDelete = viewSize - 1;
+        else
+        {           
+          gNextMessageViewIndexAfterDelete = nsMsgViewIndex_None;
 
-            //there is nothing to select viewSize is 0
-
-            treeSelection.clearSelection();
-            setTitleFromFolder(folder,null);
-            ClearMessagePane();
-          }
+          // there is nothing to select since viewSize is 0
+          treeSelection.clearSelection();
+          setTitleFromFolder(folder, null);
+          ClearMessagePane();
+          UpdateMailToolbar("delete from current view, 0 rows left");
         }
       }
-
-      // if we are about to set the selection with a new element then DON'T clear
-      // the selection then add the next message to select. This just generates
-      // an extra round of command updating notifications that we are trying to
-      // optimize away.
-      if (gNextMessageViewIndexAfterDelete != nsMsgViewIndex_None) 
-      {
-        // when deleting a message we don't update the commands when the selection goes to 0
-        // (we have a hack in nsMsgDBView which prevents that update) so there is no need to
-        // update commands when we select the next message after the delete; the commands already
-        // have the right update state...
-        gDBView.suppressCommandUpdating = true;
-
-        // This check makes sure that the tree does not perform a
-        // selection on a non selected row (row < 0), else assertions will
-        // be thrown.
-        if (gNextMessageViewIndexAfterDelete >= 0)
-          treeSelection.select(gNextMessageViewIndexAfterDelete);
-        
-        // if gNextMessageViewIndexAfterDelete has the same value 
-        // as the last index we had selected, the tree won't generate a
-        // selectionChanged notification for the tree view. So force a manual
-        // selection changed call. (don't worry it's cheap if we end up calling it twice).
-        if (treeView)
-          treeView.selectionChanged();
-
-
-        EnsureRowInThreadTreeIsVisible(gNextMessageViewIndexAfterDelete); 
-        gDBView.suppressCommandUpdating = false;
-      }
     }
-      gNextMessageViewIndexAfterDelete = -2;  
-     //default value after delete/move/copy is over
+
+    // if we are about to set the selection with a new element then DON'T clear
+    // the selection then add the next message to select. This just generates
+    // an extra round of command updating notifications that we are trying to
+    // optimize away.
+    if (gNextMessageViewIndexAfterDelete != nsMsgViewIndex_None) 
+    {
+      // when deleting a message we don't update the commands 
+      // when the selection goes to 0
+      // (we have a hack in nsMsgDBView which prevents that update) 
+      // so there is no need to
+      // update commands when we select the next message after the delete; 
+      // the commands already
+      // have the right update state...
+      gDBView.suppressCommandUpdating = true;
+
+      // This check makes sure that the tree does not perform a
+      // selection on a non selected row (row < 0), else assertions will
+      // be thrown.
+      if (gNextMessageViewIndexAfterDelete >= 0)
+        treeSelection.select(gNextMessageViewIndexAfterDelete);
+        
+      // if gNextMessageViewIndexAfterDelete has the same value 
+      // as the last index we had selected, the tree won't generate a
+      // selectionChanged notification for the tree view. So force a manual
+      // selection changed call. 
+      // (don't worry it's cheap if we end up calling it twice).
+      if (treeView)
+        treeView.selectionChanged();
+
+      EnsureRowInThreadTreeIsVisible(gNextMessageViewIndexAfterDelete); 
+      gDBView.suppressCommandUpdating = false;
+
+      // hook for extra toolbar items
+      // XXX TODO
+      // I think there is a bug in the suppression code above.
+      // what if I have two rows selected, and I hit delete, 
+      // and so we load the next row.
+      // what if I have commands that only enable where 
+      // exactly one row is selected?
+      UpdateMailToolbar("delete from current view, at least one row selected");
+    }
   }
+
+  // default value after delete/move/copy is over
+  gNextMessageViewIndexAfterDelete = -2;
 }
 
-function HandleCompactCompleted (folder)
+function HandleCompactCompleted(folder)
 {
-  if(folder)
+  if (folder)
   {
     var resource = folder.QueryInterface(Components.interfaces.nsIRDFResource);
-    if(resource)
+    if (resource)
     {
       var uri = resource.Value;
       var msgFolder = msgWindow.openFolder;
@@ -457,18 +618,18 @@ function OnLoadMessenger()
   AddMailOfflineObserver();
   CreateMailWindowGlobals();
   Create3PaneGlobals();
+  AddToolBarPrefListener();
+  ShowHideToolBarButtons();
   verifyAccounts(null);
     
   HideAccountCentral();
-  loadStartPage();
   InitMsgWindow();
-
   messenger.SetWindow(window, msgWindow);
 
   InitializeDataSources();
   InitPanes();
 
-  accountManager.SetSpecialFoldersForIdentities();
+  accountManager.setSpecialFolders();
   accountManager.addIncomingServerListener(gThreePaneIncomingServerListener);
 
   AddToSession();
@@ -476,16 +637,15 @@ function OnLoadMessenger()
   //set up correctly.
   // argument[0] --> folder uri
   // argument[1] --> optional message key
-
-  if ("arguments" in window && window.arguments[0])
+  // argument[2] --> optional email address; //will come from aim; needs to show msgs from buddy's email address  
+  if ("arguments" in window)
   {
-    gStartFolderUri = window.arguments[0];
-    gStartMsgKey = window.arguments[1];
-  }
-  else
-  {
-    gStartFolderUri = null;
-    gStartMsgKey = -1;
+    gStartFolderUri = (window.arguments.length > 0) ? window.arguments[0]
+                                                    : null;
+    gStartMsgKey = (window.arguments.length > 1) ? window.arguments[1]
+                                                 : nsMsgKey_None;
+    gSearchEmailAddress = (window.arguments.length > 2) ? window.arguments[2]
+                                                        : null;
   }
 
   setTimeout("loadStartFolder(gStartFolderUri);", 0);
@@ -495,6 +655,16 @@ function OnLoadMessenger()
 
   gHaveLoadedMessage = false;
 
+  gNotifyDefaultInboxLoadedOnStartup = true;
+
+  // fix for #168937.  now that we don't have a sidebar
+  // users who haven't moved the splitter will
+  // see it jump around
+  var messengerBox = document.getElementById("messengerBox");
+  if (!messengerBox.getAttribute("width")) {
+    messengerBox.setAttribute("width","500px");
+  }
+
   //Set focus to the Thread Pane the first time the window is opened.
   SetFocusThreadPane();
 }
@@ -502,9 +672,20 @@ function OnLoadMessenger()
 function OnUnloadMessenger()
 {
   accountManager.removeIncomingServerListener(gThreePaneIncomingServerListener);
+  RemoveToolBarPrefListener();
+  // FIX ME - later we will be able to use onload from the overlay
+  OnUnloadMsgHeaderPane();
+
   OnMailWindowUnload();
 }
 
+function NotifyObservers(aSubject, aTopic, aData)
+{
+  var observerService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
+  observerService.notifyObservers(aSubject, aTopic, aData);
+}
+
+
 function Create3PaneGlobals()
 {
 }
@@ -542,8 +723,6 @@ function loadStartFolder(initialUri)
     var defaultServer = null;
     var startFolderResource = null;
     var isLoginAtStartUpEnabled = false;
-    var enabledNewMailCheckOnce = false;
-    var mailCheckOncePref = "mail.startup.enabledMailCheckOnce";
 
     //First get default account
     try
@@ -559,14 +738,12 @@ function loadStartFolder(initialUri)
 
             startFolderResource = rootMsgFolder.QueryInterface(Components.interfaces.nsIRDFResource);
 
-            enabledNewMailCheckOnce = pref.getBoolPref(mailCheckOncePref);
-
             // Enable checknew mail once by turning checkmail pref 'on' to bring 
             // all users to one plane. This allows all users to go to Inbox. User can 
             // always go to server settings panel and turn off "Check for new mail at startup"
-            if (!enabledNewMailCheckOnce)
+            if (!pref.getBoolPref(kMailCheckOncePrefName))
             {
-                pref.setBoolPref(mailCheckOncePref, true);
+                pref.setBoolPref(kMailCheckOncePrefName, true);
                 defaultServer.loginAtStartUp = true;
             }
 
@@ -591,16 +768,16 @@ function loadStartFolder(initialUri)
             }
         }
 
-        var startFolder = startFolderResource.QueryInterface(Components.interfaces.nsIFolder);
-        SelectFolder(startFolder.URI);
-                
+        var startFolder = startFolderResource.QueryInterface(Components.interfaces.nsIMsgFolder);
+
         // only do this on startup, when we pass in null
-        if (!initialUri && isLoginAtStartUpEnabled)
+        if (!initialUri && isLoginAtStartUpEnabled && gLoadStartFolder)
         {
-            // Perform biff on the server to check for new mail, except for imap
-            if (defaultServer.type != "imap")
-              defaultServer.PerformBiff();         
-        } 
+            // Perform biff on the server to check for new mail
+              defaultServer.PerformBiff(msgWindow);        
+        }
+
+        SelectFolder(startFolder.URI);
 
         // because the "open" state persists, we'll call
         // PerformExpand() for all servers that are open at startup.
@@ -616,9 +793,30 @@ function loadStartFolder(initialUri)
         dump('Exception in LoadStartFolder caused by no default account.  We know about this\n');
     }
 
-    if (!initialUri) 
+    MsgGetMessagesForAllServers(defaultServer);
+
+    if (CheckForUnsentMessages())
     {
-        MsgGetMessagesForAllServers(defaultServer);
+        var ioService = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
+        if (!ioService.offline) 
+        {
+            InitPrompts();
+            InitServices();
+
+            if (gPromptService) 
+            {
+                var buttonPressed = gPromptService.confirmEx(window, 
+                                    gOfflinePromptsBundle.getString('sendMessagesOfflineWindowTitle'), 
+                                    gOfflinePromptsBundle.getString('sendMessagesLabel'),
+                                    gPromptService.BUTTON_TITLE_IS_STRING * (gPromptService.BUTTON_POS_0 + 
+                                        gPromptService.BUTTON_POS_1),
+                                    gOfflinePromptsBundle.getString('sendMessagesSendButtonLabel'),
+                                    gOfflinePromptsBundle.getString('sendMessagesNoSendButtonLabel'),
+                                    null, null, {value:0});
+                if (buttonPressed == 0) 
+                    SendUnsentMessages();
+            }
+        }
     }
 }
 
@@ -688,6 +886,11 @@ function UpgradeFolderPaneUI()
     var folderTotalCol = document.getElementById("folderTotalCol");
     folderTotalCol.setAttribute("hidden", "true");
     pref.setIntPref("mail.ui.folderpane.version", 2);
+  } // we fall through to the == 2 case so we'll upgrade v 1 profiles correctly
+  if (folderPaneUIVersion <= 2) {
+    var folderSizeCol = document.getElementById("folderSizeCol");
+    folderSizeCol.setAttribute("hidden", "true");
+    pref.setIntPref("mail.ui.folderpane.version", 3);
   }
 }
 
@@ -697,15 +900,13 @@ function OnLoadFolderPane()
 
     var folderUnreadCol = document.getElementById("folderUnreadCol");
     var hidden = folderUnreadCol.getAttribute("hidden");
-    if (!hidden)
+    if (hidden != "true")
     {
         var folderNameCell = document.getElementById("folderNameCell");
         folderNameCell.setAttribute("label", "?folderTreeSimpleName");
     }
     folderUnreadCol.addEventListener("DOMAttrModified", OnFolderUnreadColAttrModified, false);
 
-    SortFolderPane("folderNameCol");
-
     //Add folderDataSource and accountManagerDataSource to folderPane
     accountManagerDataSource = accountManagerDataSource.QueryInterface(Components.interfaces.nsIRDFDataSource);
     folderDataSource = folderDataSource.QueryInterface(Components.interfaces.nsIRDFDataSource);
@@ -735,10 +936,37 @@ function UpgradeThreadPaneUI()
 
   try {
     threadPaneUIVersion = pref.getIntPref("mailnews.ui.threadpane.version");
-    if (threadPaneUIVersion == 1) {
-      labelCol = document.getElementById("labelCol");
-      labelCol.setAttribute("hidden", "true");
-      pref.setIntPref("mailnews.ui.threadpane.version", 2);
+    if (threadPaneUIVersion < 4) {
+      var threadTree = document.getElementById("threadTree");
+      var junkCol = document.getElementById("junkStatusCol");
+      var beforeCol;
+
+      if (threadPaneUIVersion < 3) {
+        var subjectCol = document.getElementById("subjectCol");
+      
+        beforeCol = subjectCol.boxObject.nextSibling.boxObject.nextSibling;
+        if (beforeCol)
+          threadTree._reorderColumn(junkCol, beforeCol, true);
+        else // subjectCol was the last column, put it after
+          threadTree._reorderColumn(junkCol, subjectCol, false);
+
+        if (threadPaneUIVersion == 1) {
+          labelCol = document.getElementById("labelCol");
+          labelCol.setAttribute("hidden", "true");
+        }
+      }
+
+      var senderCol = document.getElementById("senderCol");
+      var recipientCol = document.getElementById("recipientCol");
+    
+      beforeCol = junkCol.boxObject.nextSibling.boxObject.nextSibling;
+      if (beforeCol)
+        threadTree._reorderColumn(recipientCol, beforeCol, true);
+      else // junkCol was the last column, put it after
+        threadTree._reorderColumn(recipientCol, junkCol, false);
+      threadTree._reorderColumn(senderCol, recipientCol, true);
+
+      pref.setIntPref("mailnews.ui.threadpane.version", 4);
     }
 	}
   catch (ex) {
@@ -781,9 +1009,7 @@ function GetMessagePane()
 
 function GetMessagePaneFrame()
 {
-    if (gMessagePaneFrame) return gMessagePaneFrame;
-    gMessagePaneFrame = top.frames['messagepane'];
-    return gMessagePaneFrame;
+    return window.content;
 }
 
 function FindInSidebar(currentWindow, id)
@@ -830,9 +1056,9 @@ function IsThreadAndMessagePaneSplitterCollapsed()
   var messagePane = GetMessagePane();
   try {
     return (messagePane.getAttribute("collapsed") == "true");
-	}
+  }
   catch (ex) {
-		return false;
+    return false;
   }
 }
 
@@ -865,17 +1091,19 @@ function ClearThreadPaneSelection()
 
 function ClearMessagePane()
 {
-	if(gHaveLoadedMessage)
-	{	
+  if(gHaveLoadedMessage)
+  {
     gHaveLoadedMessage = false;
-		gCurrentDisplayedMessage = null;
-		if (window.frames["messagepane"].location != "about:blank")
-		    window.frames["messagepane"].location = "about:blank";
-		// hide the message header view AND the message pane...
-		HideMessageHeaderPane();
-	}
-}
+    gCurrentDisplayedMessage = null;
+    if (GetMessagePaneFrame().location != "about:blank")
+        GetMessagePaneFrame().location = "about:blank";
+    // hide the message header view AND the message pane...
+    HideMessageHeaderPane();
 
+    // hide the junk bar
+    SetUpJunkBar(null);
+  }
+}
 
 function GetSelectedFolderIndex()
 {
@@ -922,7 +1150,12 @@ function TreeOnMouseDown(event)
     // where the click happened without loading the message headers in
     // the Folder or Thread Pane.
     if (event.button == 2)
+    {
+      gRightMouseButtonDown = true;
       ChangeSelectionWithoutContentLoad(event, event.target.parentNode);
+    }
+    else
+      gRightMouseButtonDown = false;
 }
 
 function FolderPaneOnClick(event)
@@ -936,8 +1169,12 @@ function FolderPaneOnClick(event)
     var col = {};
     var elt = {};
     folderTree.treeBoxObject.getCellAt(event.clientX, event.clientY, row, col, elt);
-    if (row.value == -1)
+    if (row.value == -1) {
+      if (event.originalTarget.localName == "treecol")
+        // clicking on the name column in the folder pane should not sort
+        event.preventBubble();
       return;
+    }
 
     if (elt.value == "twisty")
     {
@@ -963,16 +1200,14 @@ function FolderPaneOnClick(event)
             }
         }
     }
-    else if ((event.originalTarget.localName == "treecol") ||
-             (event.originalTarget.localName == "slider") ||
+    else if ((event.originalTarget.localName == "slider") ||
              (event.originalTarget.localName == "scrollbarbutton")) {
-      // clicking on the name column in the folder pane should not sort
       event.preventBubble();
     }
     else if (event.detail == 2) {
       FolderPaneDoubleClick(row.value, event);
     }
-    else if (gDBView && gDBView.isSearchView)
+    else if (gDBView && gDBView.viewType == nsMsgViewType.eShowQuickSearchResults)
     {
       onClearSearch();
     }
@@ -1057,7 +1292,7 @@ function GetSelectedMsgFolders()
             var msgFolder = folderResource.QueryInterface(Components.interfaces.nsIMsgFolder);
 
             if(msgFolder)
-                folderArray[k++] = msgFolder;
+              folderArray[k++] = msgFolder;
           }
         }
     }
@@ -1068,10 +1303,7 @@ function GetSelectedMsgFolders()
 function GetFirstSelectedMessage()
 {
     try {
-        // Use this instead of gDBView.URIForFirstSelectedMessage, else it
-        // will return the currentIndex message instead of the highlighted
-        // message.
-        return GetSelectedMessages()[0];
+        return gDBView.URIForFirstSelectedMessage;
     }
     catch (ex) {
         return null;
@@ -1097,8 +1329,12 @@ function GetSelectedMessages()
   try {
     var messageArray = {}; 
     var length = {};
-    gDBView.getURIsForSelection(messageArray,length);
-    return messageArray.value;
+    var view = GetDBView();
+    view.getURIsForSelection(messageArray,length);
+    if (length.value)
+      return messageArray.value;
+    else 
+      return null;
   }
   catch (ex) {
     dump("ex = " + ex + "\n");
@@ -1125,7 +1361,7 @@ function GetLoadedMessage()
 //Clear everything related to the current message. called after load start page.
 function ClearMessageSelection()
 {
-	ClearThreadPaneSelection();
+  ClearThreadPaneSelection();
 }
 
 function GetCompositeDataSource(command)
@@ -1136,62 +1372,70 @@ function GetCompositeDataSource(command)
 	return null;
 }
 
+// Figures out how many messages are selected (hilighted - does not necessarily
+// have the dotted outline) above a given index row value in the thread pane.
+function NumberOfSelectedMessagesAboveCurrentIndex(index)
+{
+  var numberOfMessages = 0;
+  var indicies = GetSelectedIndices(gDBView);
+
+  if (indicies && indicies.length)
+  {
+    for (var i = 0; i < indicies.length; i++)
+    {
+      if (indicies[i] < index)
+        ++numberOfMessages;
+      else
+        break;
+    }
+  }
+  return numberOfMessages;
+}
+
 function SetNextMessageAfterDelete()
 {
   var treeSelection = GetThreadTree().treeBoxObject.selection;
 
-  gThreadPaneDeleteOrMoveOccurred = true;
   if (treeSelection.isSelected(treeSelection.currentIndex))
     gNextMessageViewIndexAfterDelete = gDBView.msgToSelectAfterDelete;
-  else if (treeSelection.currentIndex > gThreadPaneCurrentSelectedIndex)
-    // Since the currentIndex (the row with the outline/dotted line) is greater
-    // than the currently selected row (the row that is highlighted), we need to
-    // make sure that upon a Delete or Move of the selected row, the highlight
-    // returns to the currentIndex'ed row.  It is necessary to subtract 1
-    // because the row being deleted is above the row with the currentIndex.
-    // If the subtraction is not done, then the highlight will end up on the
-    // row listed after the currentIndex'ed row.
-    gNextMessageViewIndexAfterDelete = treeSelection.currentIndex - 1;
+  else if(gDBView.removeRowOnMoveOrDelete)
+  {
+    // Only set gThreadPaneDeleteOrMoveOccurred to true if the message was
+    // truly moved to the trash or deleted, as opposed to an IMAP delete
+    // (where it is only "marked as deleted".  This will prevent bug 142065.
+    //
+    // If it's an IMAP delete, then just set gNextMessageViewIndexAfterDelete
+    // to treeSelection.currentIndex (where the outline is at) because nothing
+    // was moved or deleted from the folder.
+    gThreadPaneDeleteOrMoveOccurred = true;
+    gNextMessageViewIndexAfterDelete = treeSelection.currentIndex - NumberOfSelectedMessagesAboveCurrentIndex(treeSelection.currentIndex);
+  }
   else
     gNextMessageViewIndexAfterDelete = treeSelection.currentIndex;
 }
 
-function EnsureAllAncestorsAreExpanded(tree, resource)
+function EnsureFolderIndex(builder, msgFolder)
 {
-    // get the parent of the desired folder, and then try to get
-    // the index of the parent in the tree
-    var folder = resource.QueryInterface(Components.interfaces.nsIFolder);
-    
-    // if this is a server, there are no ancestors, so stop.
-    var msgFolder = folder.QueryInterface(Components.interfaces.nsIMsgFolder);
-    if (msgFolder.isServer)
-      return;
-
-    var parentFolderResource = RDF.GetResource(folder.parent.URI);
-    var folderIndex = GetFolderIndex(tree, parentFolderResource);
-
-    if (folderIndex == -1) {
-      // if we couldn't find the parent, recurse
-      EnsureAllAncestorsAreExpanded(tree, parentFolderResource);
-      // ok, now we should be able to find the parent
-      folderIndex = GetFolderIndex(tree, parentFolderResource);
-    }
-
-    // if the parent isn't open, open it
-    if (!(tree.treeBoxObject.view.isContainerOpen(folderIndex)))
-      tree.treeBoxObject.view.toggleOpenState(folderIndex);
+  // try to get the index of the folder in the tree
+  var index = builder.getIndexOfResource(msgFolder);
+  if (index == -1) {
+    // if we couldn't find the folder, open the parent
+    builder.toggleOpenState(EnsureFolderIndex(builder, msgFolder.parent));
+    index = builder.getIndexOfResource(msgFolder);
+  }
+  return index;
 }
 
 function SelectFolder(folderUri)
 {
     var folderTree = GetFolderTree();
     var folderResource = RDF.GetResource(folderUri);
+    var msgFolder = folderResource.QueryInterface(Components.interfaces.nsIMsgFolder);
 
     // before we can select a folder, we need to make sure it is "visible"
     // in the tree.  to do that, we need to ensure that all its
     // ancestors are expanded
-    EnsureAllAncestorsAreExpanded(folderTree, folderResource);
-    var folderIndex = GetFolderIndex(folderTree, folderResource);
+    var folderIndex = EnsureFolderIndex(folderTree.builderView, msgFolder);
     ChangeSelection(folderTree, folderIndex);
 }
 
@@ -1202,6 +1446,11 @@ function SelectMessage(messageUri)
     gDBView.selectMsgByKey(msgHdr.messageKey);
 }
 
+function ReloadWithAllParts()
+{
+  gDBView.reloadMessageWithAllParts();
+}
+
 function ReloadMessage()
 {
   gDBView.reloadMessage();
@@ -1248,3 +1497,4 @@ function GetFolderAttribute(tree, source, attribute)
         target = target.QueryInterface(Components.interfaces.nsIRDFLiteral).Value;
     return target;
 }
+
