@@ -40,7 +40,7 @@
 #include "nsIComponentManager.h"
 #include "nsIServiceManager.h"
 #include "nsISupportsArray.h"
-#include "nsIIOService.h"
+#include "nsNetUtil.h"
 
 #include "prmem.h"
 #include "plstr.h"
@@ -84,6 +84,9 @@
 #include "nsIAddressBook.h"
 #include "nsAbBaseCID.h"
 
+#if defined(MOZ_LDAP_XPCOM)
+#include "nsILDAPPrefsService.h"
+#endif
 #include "nsIMsgFilterService.h"
 #include "nsIMsgFilterList.h"
 
@@ -96,11 +99,6 @@
 #endif
 
 static NS_DEFINE_CID(kPrefServiceCID, NS_PREF_CID);
-static NS_DEFINE_CID(kSmtpServiceCID, NS_SMTPSERVICE_CID);   
-static NS_DEFINE_CID(kMsgAccountManagerCID, NS_MSGACCOUNTMANAGER_CID);
-static NS_DEFINE_CID(kAB4xUpgraderServiceCID, NS_AB4xUPGRADER_CID);
-static NS_DEFINE_CID(kAddressBookCID, NS_ADDRESSBOOK_CID);
-static NS_DEFINE_CID(kStringBundleServiceCID, NS_STRINGBUNDLESERVICE_CID);
 
 #define IMAP_SCHEMA "imap:/"
 #define IMAP_SCHEMA_LENGTH 6
@@ -132,7 +130,7 @@ static NS_DEFINE_CID(kStringBundleServiceCID, NS_STRINGBUNDLESERVICE_CID);
 #define DEFAULT_4X_TEMPLATES_FOLDER_NAME "Templates"
 #define UNSENT_MESSAGES_FOLDER_NAME "Unsent%20Messages"
 
-#define FILTER_FILE_NAME	"rules.dat"		/* this is XP in 5.x */
+#define FILTER_FILE_NAME	"msgFilterRules.dat"		/* this is XP in 5.x */
 
 /* we are going to clear these after migration */
 #define PREF_4X_MAIL_IDENTITY_USEREMAIL "mail.identity.useremail"
@@ -175,6 +173,8 @@ static NS_DEFINE_CID(kStringBundleServiceCID, NS_STRINGBUNDLESERVICE_CID);
 #define PREF_4X_NEWS_MAX_ARTICLES "news.max_articles"
 #define PREF_4X_NEWS_NOTIFY_ON "news.notify.on"
 #define PREF_4X_NEWS_MARK_OLD_READ "news.mark_old_read"
+#define PREF_4X_MAIL_ATTACH_VCARD "mail.attach_vcard"
+#define PREF_4X_MAIL_IDENTITY_VCARD_ROOT "mail.identity.vcard"
 
 #define PREF_4X_AUTOCOMPLETE_ON_LOCAL_AB "ldap_2.autoComplete.useAddressBooks"
 #define PREF_MOZILLA_AUTOCOMPLETE_ON_LOCAL_AB "mail.enable_autocomplete"
@@ -183,6 +183,28 @@ static NS_DEFINE_CID(kStringBundleServiceCID, NS_STRINGBUNDLESERVICE_CID);
 #define DEFAULT_DRAFT_FOLDER_PREF_NAME  "mail.identity.default.draft_folder"
 #define DEFAULT_STATIONERY_FOLDER_PREF_NAME "mail.identity.default.stationery_folder"
 
+#define DEFAULT_PAB_FILENAME_PREF_NAME "ldap_2.servers.pab.filename"
+
+// this is for the hidden preference setting in mozilla/modules/libpref/src/init/mailnews.js
+// pref("mail.migration.copyMailFiles", true);
+//
+// see bugzilla bug 80035 (http://bugzilla.mozilla.org/show_bug.cgi?id=80035)
+//
+// the default value for this setting is true which means when migrating from
+// Netscape 4.x, mozilla will copy all the contents of Local Folders and Imap
+// Folder to the newly created subfolders of migrated mozilla profile
+// when this value is set to false, mozilla will not copy these contents and
+// still share them with Netscape 4.x
+//
+// Advantages of forbidding copy operation:
+//     reduce the disk usage
+//     quick migration
+// Disadvantage of forbidding copy operation:
+//     without perfect lock mechamism, there is possibility of data corruption
+//     when Netscape 4.x and mozilla run at the same time and access the same
+//     mail file at the same time
+#define PREF_MIGRATION_MODE_FOR_MAIL "mail.migration.copyMailFiles"
+
 #define ESCAPE_USER_NAME(outName,inName) \
     *((char **)getter_Copies(outName)) = nsEscape((const char *)inName, url_XAlphas);
 
@@ -195,8 +217,7 @@ static NS_DEFINE_CID(kStringBundleServiceCID, NS_STRINGBUNDLESERVICE_CID);
   nsXPIDLCString macro_oldStr; \
   nsresult macro_rv; \
   macro_rv = IDENTITY->MACRO_GETTER(getter_Copies(macro_oldStr));	\
-  if (NS_FAILED(macro_rv)) return macro_rv;	\
-  if (!macro_oldStr) { \
+  if (NS_FAILED(macro_rv) || !macro_oldStr) { \
     IDENTITY->MACRO_SETTER("");	\
   }\
   else {	\
@@ -354,8 +375,6 @@ NS_IMPL_ISUPPORTS2(nsMessengerMigrator, nsIMessengerMigrator, nsIObserver)
 nsMessengerMigrator::nsMessengerMigrator() :
   m_haveShutdown(PR_FALSE)
 {
-
-  NS_INIT_REFCNT();
 }
 
 nsMessengerMigrator::~nsMessengerMigrator()
@@ -399,9 +418,7 @@ nsresult nsMessengerMigrator::Init()
 nsresult 
 nsMessengerMigrator::Shutdown()
 {
-  if (m_prefs) {
 	m_prefs = nsnull;
-  }
 
   m_haveShutdown = PR_TRUE;
   return NS_OK;
@@ -428,7 +445,7 @@ nsMessengerMigrator::initializeStrings()
 {
   nsresult rv;
   nsCOMPtr<nsIStringBundleService> bundleService =
-    do_GetService(kStringBundleServiceCID, &rv);
+    do_GetService(NS_STRINGBUNDLE_CONTRACTID, &rv);
   NS_ENSURE_SUCCESS(rv, rv);
   
   nsCOMPtr<nsIStringBundle> bundle;
@@ -476,7 +493,7 @@ nsMessengerMigrator::ProceedWithMigration()
     // otherwise, they don't really have anything to migrate
     rv = m_prefs->CopyCharPref(PREF_4X_MAIL_POP_NAME, &prefvalue);
     if (NS_SUCCEEDED(rv)) {
-	    if (!prefvalue || (PL_strlen(prefvalue) == 0)) {
+	    if (!prefvalue || !*prefvalue) {
 	      rv = NS_ERROR_FAILURE;
 	    }
     }
@@ -486,7 +503,7 @@ nsMessengerMigrator::ProceedWithMigration()
     // otherwise, they don't really have anything to migrate
     rv = m_prefs->CopyCharPref(PREF_4X_NETWORK_HOSTS_IMAP_SERVER, &prefvalue);
     if (NS_SUCCEEDED(rv)) {
-	    if (!prefvalue || (PL_strlen(prefvalue) == 0)) {
+	    if (!prefvalue || !*prefvalue) {
 	      rv = NS_ERROR_FAILURE;
 	    }
     }
@@ -507,7 +524,7 @@ nsMessengerMigrator::CreateLocalMailAccount(PRBool migrating)
 {
   nsresult rv;
   nsCOMPtr<nsIMsgAccountManager> accountManager = 
-           do_GetService(kMsgAccountManagerCID, &rv);
+           do_GetService(NS_MSGACCOUNTMANAGER_CONTRACTID, &rv);
   if (NS_FAILED(rv)) return rv;
 
   // create the server
@@ -649,7 +666,7 @@ nsMessengerMigrator::UpgradePrefs()
 #endif 
 
     nsCOMPtr<nsIMsgAccountManager> accountManager = 
-             do_GetService(kMsgAccountManagerCID, &rv);
+             do_GetService(NS_MSGACCOUNTMANAGER_CONTRACTID, &rv);
     if (NS_FAILED(rv)) return rv;
 
     // create a dummy identity, for migration only
@@ -661,11 +678,11 @@ nsMessengerMigrator::UpgradePrefs()
     rv = MigrateIdentity(identity);
     if (NS_FAILED(rv)) return rv;    
     
-    nsCOMPtr<nsISmtpServer> smtpServer;
     nsCOMPtr<nsISmtpService> smtpService = 
-             do_GetService(kSmtpServiceCID, &rv);
+             do_GetService(NS_SMTPSERVICE_CONTRACTID, &rv);
     if (NS_FAILED(rv)) return rv;    
 
+    nsCOMPtr<nsISmtpServer> smtpServer;
     rv = smtpService->CreateSmtpServer(getter_AddRefs(smtpServer));
     if (NS_FAILED(rv)) return rv;    
 
@@ -715,12 +732,16 @@ nsMessengerMigrator::UpgradePrefs()
     rv = MigrateNewsAccounts(identity);
     if (NS_FAILED(rv)) return rv;
 
+    // this will upgrade the ldap prefs
+#if defined(MOZ_LDAP_XPCOM)
+    nsCOMPtr <nsILDAPPrefsService> ldapPrefsService = do_GetService("@mozilla.org/ldapprefs-service;1", &rv);
+#endif    
     rv = MigrateAddressBookPrefs();
     NS_ENSURE_SUCCESS(rv,rv);
 
     rv = MigrateAddressBooks();
     if (NS_FAILED(rv)) return rv;
-    
+
     // we're done migrating, let's save the prefs
     rv = m_prefs->SavePrefFile(nsnull);
     if (NS_FAILED(rv)) return rv;
@@ -728,6 +749,7 @@ nsMessengerMigrator::UpgradePrefs()
 	// remove the temporary identity we used for migration purposes
     identity->ClearAllValues();
     rv = accountManager->RemoveIdentity(identity);
+
     return rv;
 }
 
@@ -774,9 +796,23 @@ nsMessengerMigrator::MigrateIdentity(nsIMsgIdentity *identity)
   MIGRATE_SIMPLE_STR_PREF(PREF_4X_MAIL_IDENTITY_REPLY_TO,identity,SetReplyTo)
   MIGRATE_SIMPLE_WSTR_PREF(PREF_4X_MAIL_IDENTITY_ORGANIZATION,identity,SetOrganization)
   MIGRATE_SIMPLE_BOOL_PREF(PREF_4X_MAIL_COMPOSE_HTML,identity,SetComposeHtml)
-  MIGRATE_SIMPLE_FILE_PREF_TO_FILE_PREF(PREF_4X_MAIL_SIGNATURE_FILE,identity,SetSignature);
-  MIGRATE_SIMPLE_FILE_PREF_TO_BOOL_PREF(PREF_4X_MAIL_SIGNATURE_FILE,identity,SetAttachSignature);
-  MIGRATE_SIMPLE_INT_PREF(PREF_4X_MAIL_SIGNATURE_DATE,identity,SetSignatureDate);
+  MIGRATE_SIMPLE_FILE_PREF_TO_FILE_PREF(PREF_4X_MAIL_SIGNATURE_FILE,identity,SetSignature)
+  MIGRATE_SIMPLE_FILE_PREF_TO_BOOL_PREF(PREF_4X_MAIL_SIGNATURE_FILE,identity,SetAttachSignature)
+  MIGRATE_SIMPLE_INT_PREF(PREF_4X_MAIL_SIGNATURE_DATE,identity,SetSignatureDate)
+
+  MIGRATE_SIMPLE_BOOL_PREF(PREF_4X_MAIL_ATTACH_VCARD, identity, SetAttachVCard)
+  nsCOMPtr <nsIAddressBook> ab = do_CreateInstance(NS_ADDRESSBOOK_CONTRACTID);
+  if (ab) 
+  {
+      nsXPIDLCString escapedVCardStr;
+      rv = ab->Convert4xVCardPrefs(PREF_4X_MAIL_IDENTITY_VCARD_ROOT, getter_Copies(escapedVCardStr));
+      if (NS_SUCCEEDED(rv) && !escapedVCardStr.IsEmpty()) 
+      {
+          rv = identity->SetEscapedVCard(escapedVCardStr.get());
+          NS_ASSERTION(NS_SUCCEEDED(rv), "failed to set escaped vCard string");
+      }
+  }    
+
   /* NOTE:  if you add prefs here, make sure you update nsMsgIdentity::Copy() */
   return NS_OK;
 }
@@ -945,13 +981,14 @@ nsMessengerMigrator::Convert4XUri(const char *old_uri, PRBool for_news, const ch
 	nsXPIDLCString hostname;
 	nsXPIDLCString username;
 
-    nsCOMPtr<nsIIOService> ioService = do_GetService(NS_IOSERVICE_CONTRACTID);
-    if (!ioService) return NS_ERROR_FAILURE;
-    
-    rv = ioService->ExtractUrlPart(nsDependentCString(old_uri), nsIIOService::url_Host, hostname);
+    nsCOMPtr<nsIURI> uri;
+    rv = NS_NewURI(getter_AddRefs(uri), nsDependentCString(old_uri));
     if (NS_FAILED(rv)) return rv;
 
-    rv = ioService->ExtractUrlPart(nsDependentCString(old_uri), nsIIOService::url_Username, username);
+    rv = uri->GetHost(hostname);
+    if (NS_FAILED(rv)) return rv;
+
+    rv = uri->GetUsername(username);
     if (NS_FAILED(rv)) return rv;
 
     // in 4.x, mac and windows stored the URI as IMAP://<hostname> 
@@ -1004,7 +1041,7 @@ nsMessengerMigrator::Convert4XUri(const char *old_uri, PRBool for_news, const ch
   if (NS_SUCCEEDED(rv)) {
 	rv = mail_dir->GetUnixStyleFilePath(&mail_directory_value);
   }
-  if (NS_FAILED(rv) || !mail_directory_value || (PL_strlen(mail_directory_value) == 0)) {
+  if (NS_FAILED(rv) || !mail_directory_value || !*mail_directory_value) {
 #ifdef DEBUG_MIGRATOR
     printf("%s was not set, attempting to use %s instead.\n",PREF_PREMIGRATION_MAIL_DIRECTORY,PREF_MAIL_DIRECTORY);
 #endif
@@ -1015,7 +1052,7 @@ nsMessengerMigrator::Convert4XUri(const char *old_uri, PRBool for_news, const ch
 	rv = mail_dir->GetUnixStyleFilePath(&mail_directory_value);
     } 
 
-    if (NS_FAILED(rv) || !mail_directory_value || (PL_strlen(mail_directory_value) == 0)) {
+    if (NS_FAILED(rv) || !mail_directory_value || !*mail_directory_value) {
       NS_ASSERTION(0,"failed to get a base value for the mail.directory");
       return NS_ERROR_UNEXPECTED;
     }
@@ -1089,7 +1126,7 @@ nsMessengerMigrator::Convert4XUri(const char *old_uri, PRBool for_news, const ch
   // this meant it was reall <foobar>/<default folder name>
   // this insanity only happened on mac and windows.
   // Need to escape spaces in folder name (ie, "A Folder" --> "A%20Folder").
-  if (!folderPath || (PL_strlen(folderPath) == 0)) {
+  if (!folderPath || !*folderPath) {
     nsXPIDLCString escaped_default_folder;
     ESCAPE_FOLDER_NAME(escaped_default_folder, default_folder_name);
     *new_uri = PR_smprintf("%s/%s/%s",MAILBOX_SCHEMA,usernameAtHostname, escaped_default_folder.get());
@@ -1129,7 +1166,7 @@ nsMessengerMigrator::MigrateLocalMailAccount()
 {
   nsresult rv;
   nsCOMPtr<nsIMsgAccountManager> accountManager = 
-           do_GetService(kMsgAccountManagerCID, &rv);
+           do_GetService(NS_MSGACCOUNTMANAGER_CONTRACTID, &rv);
   if (NS_FAILED(rv)) return rv;
 
   // create the server
@@ -1146,10 +1183,6 @@ nsMessengerMigrator::MigrateLocalMailAccount()
   noServer = do_QueryInterface(server, &rv);
   if (NS_FAILED(rv)) return rv;
 
-  // we don't want "nobody at Local Folders" to show up in the
-  // folder pane, so we set the pretty name to "Local Folders"
-  server->SetPrettyName(mLocalFoldersName.get());
-  
   // create the directory structure for old 4.x "Local Mail"
   // under <profile dir>/Mail/Local Folders or
   // <"mail.directory" pref>/Local Folders 
@@ -1188,13 +1221,21 @@ nsMessengerMigrator::MigrateLocalMailAccount()
   if (NS_FAILED(rv)) return rv;
 
   // set the local path for this "none" server
-  //
-  // we need to set this to <profile>/Mail/Local Folders, because that's where
-  // the 4.x "Local Mail" (when using imap) got copied.
-  // it would be great to use the server key, but we don't know it
-  // when we are copying of the mail.
-  rv = mailDirSpec->AppendRelativeUnixPath(mLocalFoldersHostname.get());
-  if (NS_FAILED(rv)) return rv; 
+
+  // get the migration mode for mail
+  PRBool copyMailFileInMigration = PR_TRUE;
+  rv = m_prefs->GetBoolPref(PREF_MIGRATION_MODE_FOR_MAIL, &copyMailFileInMigration);
+  if (NS_FAILED(rv))
+    return rv;
+  if (copyMailFileInMigration)
+  {
+    // we need to set this to <profile>/Mail/Local Folders, because that's where
+    // the 4.x "Local Mail" (when using imap) got copied.
+    // it would be great to use the server key, but we don't know it
+    // when we are copying of the mail.
+    rv = mailDirSpec->AppendRelativeUnixPath(mLocalFoldersHostname.get());
+    if (NS_FAILED(rv)) return rv; 
+  }
   rv = server->SetLocalPath(mailDirSpec);
   if (NS_FAILED(rv)) return rv;
     
@@ -1203,6 +1244,10 @@ nsMessengerMigrator::MigrateLocalMailAccount()
     mailDirSpec->CreateDir();
   }
   
+  // we don't want "nobody at Local Folders" to show up in the
+  // folder pane, so we set the pretty name to "Local Folders"
+  server->SetPrettyName(mLocalFoldersName.get());
+  
   // pass the "Local Folders" server so the send later uri pref 
   // will be "mailbox://nobody@Local Folders/Unsent Messages"
   rv = SetSendLaterUriPref(server);
@@ -1245,7 +1290,7 @@ nsMessengerMigrator::MigrateMovemailAccount(nsIMsgIdentity *identity)
   nsCOMPtr<nsIMsgIncomingServer> server;
   
   nsCOMPtr<nsIMsgAccountManager> accountManager = 
-           do_GetService(kMsgAccountManagerCID, &rv);
+           do_GetService(NS_MSGACCOUNTMANAGER_CONTRACTID, &rv);
   if (NS_FAILED(rv)) return rv;
 
   // get the pop username
@@ -1354,7 +1399,7 @@ nsMessengerMigrator::MigratePopAccount(nsIMsgIdentity *identity)
 {
   nsresult rv;
   nsCOMPtr<nsIMsgAccountManager> accountManager = 
-           do_GetService(kMsgAccountManagerCID, &rv);
+           do_GetService(NS_MSGACCOUNTMANAGER_CONTRACTID, &rv);
   if (NS_FAILED(rv)) return rv;
 
   nsCOMPtr<nsIMsgIncomingServer> server;
@@ -1584,7 +1629,7 @@ nsMessengerMigrator::MigrateImapAccounts(nsIMsgIdentity *identity)
   if (!hostList || !*hostList) return NS_OK;  // NS_ERROR_FAILURE?
   
   char *token = nsnull;
-  char *rest = NS_CONST_CAST(char*,(const char*)hostList);
+  char *rest = hostList;
   nsCAutoString str;
 
   PRBool isDefaultAccount = PR_TRUE;
@@ -1618,7 +1663,7 @@ nsMessengerMigrator::MigrateImapAccount(nsIMsgIdentity *identity, const char *ho
   nsresult rv;  
 
   nsCOMPtr<nsIMsgAccountManager> accountManager = 
-           do_GetService(kMsgAccountManagerCID, &rv);
+           do_GetService(NS_MSGACCOUNTMANAGER_CONTRACTID, &rv);
   if (NS_FAILED(rv)) return rv;
 
   if (!hostAndPort) return NS_ERROR_NULL_POINTER;
@@ -1878,10 +1923,35 @@ nsMessengerMigrator::migrateAddressBookPrefEnum(const char *aPref, void *aClosur
 #ifdef DEBUG_AB_MIGRATION
   printf("investigate pref: %s\n",aPref);
 #endif
+  nsXPIDLCString abFileName;
   // we only care about ldap_2.servers.*.filename" prefs
   if (!charEndsWith(aPref, ADDRESSBOOK_PREF_NAME_SUFFIX)) return;
       
-  nsXPIDLCString abFileName;
+  // check for pab without a filename and if so, set it to pab.na2.
+  // This is an ugly hack for installations that haven't set 
+  // pab.filename.
+      rv = prefs->CopyCharPref(DEFAULT_PAB_FILENAME_PREF_NAME, getter_Copies(abFileName));
+      if (NS_FAILED(rv))
+      {
+        // pab.filename not set - set it to pab.na2 and pretend aPref is it.
+        prefs->SetCharPref(DEFAULT_PAB_FILENAME_PREF_NAME, "pab.na2");
+        aPref = "ldap_2.servers.pab.filename";
+      }
+
+  // check if this really is an ldap server, in which case, we're not going to migrate
+  // the na2 file for now, since it confuses RDF when the import code adds this
+  // server as a personal address book. I'd like to fix that but it's tricky
+  // and the .na2 file for ldap servers should only be for offline data.
+
+  nsCAutoString serverPrefName(aPref);
+  PRInt32 fileNamePos = serverPrefName.Find(".filename");
+  serverPrefName.Truncate(fileNamePos + 1);
+  serverPrefName.Append("serverName");
+  nsXPIDLCString serverName;
+  rv = prefs->CopyCharPref(serverPrefName.get(), getter_Copies(serverName));
+  if (NS_SUCCEEDED(rv) && !serverName.IsEmpty())
+    return; // skip this - it's an ldap server
+      
   rv = prefs->CopyCharPref(aPref,getter_Copies(abFileName));
   NS_ASSERTION(NS_SUCCEEDED(rv),"ab migration failed: failed to get ab filename");
   if (NS_FAILED(rv)) return;
@@ -1920,14 +1990,14 @@ nsMessengerMigrator::migrateAddressBookPrefEnum(const char *aPref, void *aClosur
   rv = NS_GetSpecialDirectory(NS_APP_USER_PROFILE_50_DIR, getter_AddRefs(ab4xFile));
   NS_ASSERTION(NS_SUCCEEDED(rv) && ab4xFile,"ab migration failed: failed to get profile dir");
   if (NS_FAILED(rv) || !ab4xFile) return;
+  
+  rv = ab4xFile->AppendNative(nsDependentCString(abFileName));
+  NS_ASSERTION(NS_SUCCEEDED(rv),"ab migration failed:  failed to append filename");
+  if (NS_FAILED(rv)) return;
 
   // TODO: Change users of nsIFileSpec to nsIFile and avoid this.
   rv = NS_NewFileSpecFromIFile(ab4xFile, getter_AddRefs(ab4xFileSpec));
   if (NS_FAILED(rv)) return;
-  
-  rv = ab4xFileSpec->AppendRelativeUnixPath((const char *)abFileName);
-  NS_ASSERTION(NS_SUCCEEDED(rv),"ab migration failed:  failed to append filename");
-  if (NS_FAILED(rv)) return;
 
   rv = NS_GetSpecialDirectory(NS_OS_TEMP_DIR, getter_AddRefs(tmpLDIFFile));
   if (NS_FAILED(rv) || !tmpLDIFFile) return;
@@ -1977,7 +2047,7 @@ nsMessengerMigrator::migrateAddressBookPrefEnum(const char *aPref, void *aClosur
   NS_ASSERTION(NS_SUCCEEDED(rv),"ab migration failed: failed to append filename");
   if (NS_FAILED(rv)) return;
      
-  nsCOMPtr <nsIAddressBook> ab = do_CreateInstance(kAddressBookCID, &rv);
+  nsCOMPtr <nsIAddressBook> ab = do_CreateInstance(NS_ADDRESSBOOK_CONTRACTID, &rv);
   NS_ASSERTION(NS_SUCCEEDED(rv) && ab, "failed to get address book");
   if (NS_FAILED(rv) || !ab) return;
 
@@ -2005,9 +2075,11 @@ nsMessengerMigrator::migrateAddressBookPrefEnum(const char *aPref, void *aClosur
 #ifdef DEBUG_AB_MIGRATION
   printf("remove the tmp file\n");
 #endif /* DEBUG_AB_MIGRATION */
+  rv = ab4xFile->Remove(PR_FALSE);
+  NS_ASSERTION(NS_SUCCEEDED(rv),"failed to delete the na2 file");
+  if (NS_FAILED(rv)) return;
   rv = tmpLDIFFile->Remove(PR_TRUE);
   NS_ASSERTION(NS_SUCCEEDED(rv),"failed to delete the temp ldif file");
-  if (NS_FAILED(rv)) return;
   
   return;
 }
@@ -2274,7 +2346,7 @@ nsMessengerMigrator::MigrateNewsAccount(nsIMsgIdentity *identity, const char *ho
 {  
 	nsresult rv;
     nsCOMPtr<nsIMsgAccountManager> accountManager = 
-             do_GetService(kMsgAccountManagerCID, &rv);
+             do_GetService(NS_MSGACCOUNTMANAGER_CONTRACTID, &rv);
     if (NS_FAILED(rv)) return rv;
 
 	nsFileSpec thisNewsHostsDir = newsHostsDir;
@@ -2296,7 +2368,8 @@ nsMessengerMigrator::MigrateNewsAccount(nsIMsgIdentity *identity, const char *ho
 
     // create the server
 	nsCOMPtr<nsIMsgIncomingServer> server;
-    rv = accountManager->CreateIncomingServer(nsnull, hostname.get(), "nntp",
+    // for news, username is always null
+    rv = accountManager->CreateIncomingServer(nsnull /* username */, hostname.get(), "nntp",
                               getter_AddRefs(server));
     NS_ENSURE_SUCCESS(rv,rv);
  
