@@ -40,23 +40,25 @@
 
 #include "nsIBaseWindow.h"
 #include "nsISupportsArray.h"
+#include "nsXPCOM.h"
 #include "nsISupportsPrimitives.h"
 #include "nsIComponentManager.h"
+#include "nsNetError.h"
 
-NS_IMPL_THREADSAFE_ADDREF(nsMsgProgress);
-NS_IMPL_THREADSAFE_RELEASE(nsMsgProgress);
+NS_IMPL_THREADSAFE_ADDREF(nsMsgProgress)
+NS_IMPL_THREADSAFE_RELEASE(nsMsgProgress)
 
 NS_INTERFACE_MAP_BEGIN(nsMsgProgress)
    NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsISupports, nsIMsgStatusFeedback)
    NS_INTERFACE_MAP_ENTRY(nsIMsgProgress)
    NS_INTERFACE_MAP_ENTRY(nsIMsgStatusFeedback)
    NS_INTERFACE_MAP_ENTRY(nsIWebProgressListener)
+   NS_INTERFACE_MAP_ENTRY(nsIProgressEventSink)
 NS_INTERFACE_MAP_END_THREADSAFE
 
 
 nsMsgProgress::nsMsgProgress()
 {
-  NS_INIT_ISUPPORTS();
   m_closeProgress = PR_FALSE;
   m_processCanceled = PR_FALSE;
   m_pendingStateFlags = -1;
@@ -68,13 +70,18 @@ nsMsgProgress::~nsMsgProgress()
   (void)ReleaseListeners();
 }
 
-/* void openProgressDialog (in nsIDOMWindowInternal parent, in string dialogURL, in nsISupports parameters); */
-NS_IMETHODIMP nsMsgProgress::OpenProgressDialog(nsIDOMWindowInternal *parent,
+/* void openProgressDialog (in nsIDOMWindowInternal parent, in nsIMsgWindow aMsgWindow,
+  in string dialogURL, in nsISupports parameters); 
+*/
+NS_IMETHODIMP nsMsgProgress::OpenProgressDialog(nsIDOMWindowInternal *parent, nsIMsgWindow *aMsgWindow,
                                                 const char *dialogURL,
                                                 nsISupports *parameters)
 {
   nsresult rv = NS_ERROR_FAILURE;
   
+  m_msgWindow = aMsgWindow;
+  if (m_msgWindow)
+    m_msgWindow->SetStatusFeedback(this);
   if (m_dialog)
     return NS_ERROR_ALREADY_INITIALIZED;
   
@@ -114,7 +121,7 @@ NS_IMETHODIMP nsMsgProgress::OpenProgressDialog(nsIDOMWindowInternal *parent,
 NS_IMETHODIMP nsMsgProgress::CloseProgressDialog(PRBool forceClose)
 {
   m_closeProgress = PR_TRUE;
-  return OnStateChange(nsnull, nsnull, nsIWebProgressListener::STATE_STOP, forceClose);
+  return OnStateChange(nsnull, nsnull, nsIWebProgressListener::STATE_STOP, forceClose ? NS_ERROR_FAILURE : NS_OK);
 }
 
 /* nsIPrompt GetPrompter (); */
@@ -139,7 +146,7 @@ NS_IMETHODIMP nsMsgProgress::GetProcessCanceledByUser(PRBool *aProcessCanceledBy
 NS_IMETHODIMP nsMsgProgress::SetProcessCanceledByUser(PRBool aProcessCanceledByUser)
 {
   m_processCanceled = aProcessCanceledByUser;
-  OnStateChange(nsnull, nsnull, nsIWebProgressListener::STATE_STOP, PR_FALSE);
+  OnStateChange(nsnull, nsnull, nsIWebProgressListener::STATE_STOP, NS_BINDING_ABORTED);
   return NS_OK;
 }
 
@@ -206,6 +213,9 @@ NS_IMETHODIMP nsMsgProgress::OnStateChange(nsIWebProgress *aWebProgress, nsIRequ
     }
   }
   
+  if (aStateFlags == nsIWebProgressListener::STATE_STOP && m_msgWindow && NS_FAILED(aStatus))
+    m_msgWindow->StopUrls();
+
   return NS_OK;
 }
 
@@ -320,7 +330,7 @@ NS_IMETHODIMP nsMsgProgress::ShowProgress(PRInt32 percent)
 }
 
 /* [noscript] void setDocShell (in nsIDocShell shell, in nsIDOMWindowInternal window); */
-NS_IMETHODIMP nsMsgProgress::SetDocShell(nsIDocShell *shell, nsIDOMWindowInternal *window)
+NS_IMETHODIMP nsMsgProgress::SetDocShell(nsIDocShell *shell, nsIDOMWindow *window)
 {
   return NS_ERROR_NOT_IMPLEMENTED;
 }
@@ -331,3 +341,37 @@ NS_IMETHODIMP nsMsgProgress::CloseWindow()
   return NS_ERROR_NOT_IMPLEMENTED;
 }
 
+NS_IMETHODIMP nsMsgProgress::SetMsgWindow(nsIMsgWindow *aMsgWindow)
+{
+  m_msgWindow = aMsgWindow;
+  return NS_OK;
+}
+
+NS_IMETHODIMP nsMsgProgress::GetMsgWindow(nsIMsgWindow **aMsgWindow)
+{
+  NS_ENSURE_ARG_POINTER(aMsgWindow);
+  NS_IF_ADDREF(*aMsgWindow = m_msgWindow);
+  return NS_OK;
+}
+
+
+NS_IMETHODIMP nsMsgProgress::OnProgress(nsIRequest *request, nsISupports* ctxt, 
+                                          PRUint32 aProgress, PRUint32 aProgressMax)
+{
+  // XXX: What should the nsIWebProgress be?
+  return OnProgressChange(nsnull, request, aProgress, aProgressMax, 
+                          aProgress /* current total progress */, aProgressMax /* max total progress */);
+}
+
+NS_IMETHODIMP nsMsgProgress::OnStatus(nsIRequest *request, nsISupports* ctxt, 
+                                            nsresult aStatus, const PRUnichar* aStatusArg)
+{
+  nsresult rv;
+  nsCOMPtr<nsIStringBundleService> sbs = do_GetService(NS_STRINGBUNDLE_CONTRACTID, &rv);
+  if (NS_FAILED(rv)) return rv;
+  nsXPIDLString str;
+  rv = sbs->FormatStatusMessage(aStatus, aStatusArg, getter_Copies(str));
+  if (NS_FAILED(rv)) return rv;
+  nsAutoString msg(NS_STATIC_CAST(const PRUnichar*, str));
+  return ShowStatusString(msg.get());
+}
