@@ -53,35 +53,35 @@
 #include "nsIIOService.h"
 #include "nsNetCID.h"
 #include "nsEscape.h"
+#include "nsIStreamListener.h"
+#include "nsIOutputStream.h"
+#include "nsIInputStream.h"
+#include "nsIFileSpec.h"
+#include <time.h>
 
-static NS_DEFINE_CID(kUrlListenerManagerCID, NS_URLLISTENERMANAGER_CID);
-static NS_DEFINE_CID(kStandardUrlCID, NS_STANDARDURL_CID);
-static NS_DEFINE_CID(kMsgMailSessionCID, NS_MSGMAILSESSION_CID);
 static NS_DEFINE_CID(kIOServiceCID, NS_IOSERVICE_CID);
 
 nsMsgMailNewsUrl::nsMsgMailNewsUrl()
 {
-    NS_INIT_REFCNT();
- 
-	// nsIURI specific state
-	m_errorMessage = nsnull;
-	m_runningUrl = PR_FALSE;
-	m_updatingFolder = PR_FALSE;
+  // nsIURI specific state
+  m_errorMessage = nsnull;
+  m_runningUrl = PR_FALSE;
+  m_updatingFolder = PR_FALSE;
   m_addContentToCache = PR_FALSE;
   m_msgIsInLocalCache = PR_FALSE;
   m_suppressErrorMsgs = PR_FALSE;
-
-	nsComponentManager::CreateInstance(kUrlListenerManagerCID, nsnull, NS_GET_IID(nsIUrlListenerManager), (void **) getter_AddRefs(m_urlListeners));
-	nsComponentManager::CreateInstance(kStandardUrlCID, nsnull, NS_GET_IID(nsIURL), (void **) getter_AddRefs(m_baseURL));
+  
+  m_urlListeners = do_CreateInstance(NS_URLLISTENERMANAGER_CONTRACTID);
+  m_baseURL = do_CreateInstance(NS_STANDARDURL_CONTRACTID);
 }
- 
+
 nsMsgMailNewsUrl::~nsMsgMailNewsUrl()
 {
 	PR_FREEIF(m_errorMessage);
 }
   
-NS_IMPL_THREADSAFE_ADDREF(nsMsgMailNewsUrl);
-NS_IMPL_THREADSAFE_RELEASE(nsMsgMailNewsUrl);
+NS_IMPL_THREADSAFE_ADDREF(nsMsgMailNewsUrl)
+NS_IMPL_THREADSAFE_RELEASE(nsMsgMailNewsUrl)
 
 NS_INTERFACE_MAP_BEGIN(nsMsgMailNewsUrl)
    NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsISupports, nsIMsgMailNewsUrl)
@@ -231,7 +231,7 @@ NS_IMETHODIMP nsMsgMailNewsUrl::GetMsgWindow(nsIMsgWindow **aMsgWindow)
 	if (!m_msgWindow)
 	{
 //		nsCOMPtr<nsIMsgMailSession> mailSession = 
-//		         do_GetService(kMsgMailSessionCID, &rv); 
+//		         do_GetService(NS_MSGMAILSESSION_CONTRACTID, &rv); 
 
 //		if(NS_SUCCEEDED(rv))
 //		mailSession->GetTemporaryMsgStatusFeedback(getter_AddRefs(m_statusFeedback));
@@ -428,10 +428,10 @@ NS_IMETHODIMP nsMsgMailNewsUrl::GetSpec(nsACString &aSpec)
 
 NS_IMETHODIMP nsMsgMailNewsUrl::SetSpec(const nsACString &aSpec)
 {
-  const nsPromiseFlatCString &spec = PromiseFlatCString(aSpec);
+  nsCAutoString spec(aSpec);
   // Parse out "filename" attribute if present.
   char *start, *end;
-  start = PL_strcasestr(spec.get(),FILENAME_PART);
+  start = PL_strcasestr(spec.BeginWriting(),FILENAME_PART);
   if (start)
   {	// Make sure we only get our own value.
     end = PL_strcasestr((char*)(start+FILENAME_PART_LEN),"&");
@@ -550,9 +550,25 @@ NS_IMETHODIMP nsMsgMailNewsUrl::GetOriginCharset(nsACString &aOriginCharset)
     return m_baseURL->GetOriginCharset(aOriginCharset);
 }
 
+NS_IMETHODIMP nsMsgMailNewsUrl::GetBaseURI(nsIURI **aBaseURI)
+{
+  NS_ENSURE_ARG_POINTER(aBaseURI);
+  return m_baseURL->QueryInterface(NS_GET_IID(nsIURI), (void**) aBaseURI);
+}
+
 NS_IMETHODIMP nsMsgMailNewsUrl::Equals(nsIURI *other, PRBool *_retval)
 {
-	return m_baseURL->Equals(other, _retval);
+  nsCOMPtr <nsIMsgMailNewsUrl> mailUrl = do_QueryInterface(other);
+  // we really want to compare the base uris to each other, not our base URI
+  // with the other's real URI.
+  if (mailUrl)
+  {
+    nsCOMPtr <nsIURI> baseURI;
+    mailUrl->GetBaseURI(getter_AddRefs(baseURI));
+    if (baseURI)
+      return m_baseURL->Equals(baseURI, _retval);
+  }
+  return m_baseURL->Equals(other, _retval);
 }
 
 NS_IMETHODIMP nsMsgMailNewsUrl::SchemeIs(const char *aScheme, PRBool *_retval)
@@ -606,6 +622,7 @@ NS_IMETHODIMP nsMsgMailNewsUrl::Resolve(const nsACString &relativePath, nsACStri
     if (NS_SUCCEEDED(rv) && !scheme.IsEmpty())
     {
       result = relativePath;
+      rv = NS_OK;
     }
     else
     {
@@ -614,7 +631,7 @@ NS_IMETHODIMP nsMsgMailNewsUrl::Resolve(const nsACString &relativePath, nsACStri
     }
   }
 
-  return NS_OK;
+  return rv;
 }
 
 NS_IMETHODIMP nsMsgMailNewsUrl::GetDirectory(nsACString &aDirectory)
@@ -794,7 +811,7 @@ NS_IMETHODIMP nsMsgMailNewsUrl::GetMimeHeaders(nsIMimeHeaders * *mimeHeaders)
 {
     NS_ENSURE_ARG_POINTER(mimeHeaders);
     NS_IF_ADDREF(*mimeHeaders = mMimeHeaders);
-    return NS_OK;
+    return (mMimeHeaders) ? NS_OK : NS_ERROR_NULL_POINTER;
 }
 
 NS_IMETHODIMP nsMsgMailNewsUrl::SetMimeHeaders(nsIMimeHeaders *mimeHeaders)
@@ -802,3 +819,201 @@ NS_IMETHODIMP nsMsgMailNewsUrl::SetMimeHeaders(nsIMimeHeaders *mimeHeaders)
     mMimeHeaders = mimeHeaders;
     return NS_OK;
 }
+
+#define SAVE_BUF_SIZE 8192
+class nsMsgSaveAsListener : public nsIStreamListener
+{
+public:
+  NS_DECL_ISUPPORTS
+  NS_DECL_NSIREQUESTOBSERVER
+  NS_DECL_NSISTREAMLISTENER
+
+  nsMsgSaveAsListener(nsIFileSpec *aFileSpec, PRBool addDummyEnvelope);
+  virtual ~nsMsgSaveAsListener();
+  nsresult SetupMsgWriteStream(nsIFileSpec *aFileSpec, PRBool addDummyEnvelope);
+protected:
+  nsCOMPtr<nsIOutputStream> m_outputStream;
+  nsCOMPtr<nsIFileSpec> m_outputFile;
+  PRBool m_addDummyEnvelope;
+  PRBool m_writtenData;
+  PRUint32 m_leftOver;
+  char m_dataBuffer[SAVE_BUF_SIZE+1]; // temporary buffer for this save operation
+
+};
+
+NS_IMPL_ISUPPORTS1(nsMsgSaveAsListener, nsIStreamListener)
+
+nsMsgSaveAsListener::nsMsgSaveAsListener(nsIFileSpec *aFileSpec, PRBool addDummyEnvelope)
+{
+  m_outputFile = aFileSpec;
+  m_writtenData = PR_FALSE;
+  m_addDummyEnvelope = addDummyEnvelope;
+  m_leftOver = 0;
+}
+
+nsMsgSaveAsListener::~nsMsgSaveAsListener()
+{
+}
+
+NS_IMETHODIMP nsMsgSaveAsListener::OnStartRequest(nsIRequest *request, nsISupports *ctxt)
+{
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsMsgSaveAsListener::OnStopRequest(nsIRequest *request, nsISupports * aCtxt, nsresult aStatus)
+{
+  if (m_outputStream)
+  {
+    m_outputStream->Flush();
+    m_outputStream->Close();
+  }
+  if (m_outputFile)
+    m_outputFile->CloseStream();
+  return NS_OK;
+} 
+
+NS_IMETHODIMP nsMsgSaveAsListener::OnDataAvailable(nsIRequest* request, 
+                                  nsISupports* aSupport,
+                                  nsIInputStream* inStream, 
+                                  PRUint32 srcOffset,
+                                  PRUint32 count)
+{
+  nsresult rv;
+  PRUint32 available;
+  rv = inStream->Available(&available);
+  if (!m_writtenData)
+  {
+    m_writtenData = PR_TRUE;
+    rv = SetupMsgWriteStream(m_outputFile, m_addDummyEnvelope);
+    NS_ENSURE_SUCCESS(rv, rv);
+  }
+
+
+  PRUint32 readCount, maxReadCount = SAVE_BUF_SIZE - m_leftOver;
+  PRUint32 writeCount;
+  char *start, *end;
+  PRUint32 linebreak_len = 0;
+
+  while (count > 0)
+  {
+      if (count < maxReadCount)
+          maxReadCount = count;
+      rv = inStream->Read(m_dataBuffer + m_leftOver,
+                          maxReadCount,
+                          &readCount);
+      if (NS_FAILED(rv)) return rv;
+
+      m_leftOver += readCount;
+      m_dataBuffer[m_leftOver] = '\0';
+
+      start = m_dataBuffer;
+      end = PL_strchr(start, '\r');
+      if (!end)
+          end = PL_strchr(start, '\n');
+      else if (*(end+1) == nsCRT::LF && linebreak_len == 0)
+          linebreak_len = 2;
+
+      if (linebreak_len == 0) // not initialize yet
+          linebreak_len = 1;
+
+      count -= readCount;
+      maxReadCount = SAVE_BUF_SIZE - m_leftOver;
+
+      if (!end && count > maxReadCount)
+          // must be a very very long line; sorry cannot handle it
+          return NS_ERROR_FAILURE;
+
+      while (start && end)
+      {
+          if (PL_strncasecmp(start, "X-Mozilla-Status:", 17) &&
+              PL_strncasecmp(start, "X-Mozilla-Status2:", 18) &&
+              PL_strncmp(start, "From - ", 7))
+          {
+              rv = m_outputStream->Write(start, end-start, &writeCount);
+              rv = m_outputStream->Write(MSG_LINEBREAK,sizeof(MSG_LINEBREAK) - 1, &writeCount);
+          }
+          start = end+linebreak_len;
+          if (start >= m_dataBuffer + m_leftOver)
+          {
+              maxReadCount = SAVE_BUF_SIZE;
+              m_leftOver = 0;
+              break;
+          }
+          end = PL_strchr(start, '\r');
+          if (!end)
+              end = PL_strchr(start, '\n');
+          if (start && !end)
+          {
+              m_leftOver -= (start - m_dataBuffer);
+              memcpy(m_dataBuffer, start,
+                            m_leftOver+1); // including null
+              maxReadCount = SAVE_BUF_SIZE - m_leftOver;
+          }
+      }
+      if (NS_FAILED(rv)) return rv;
+  }
+  return rv;
+  
+  //  rv = m_outputStream->WriteFrom(inStream, PR_MIN(available, count), &bytesWritten);
+}
+
+nsresult nsMsgSaveAsListener::SetupMsgWriteStream(nsIFileSpec *aFileSpec, PRBool addDummyEnvelope)
+{
+  nsresult rv = NS_ERROR_FAILURE;
+
+  // if the file already exists, delete it.
+  // do this before we get the outputstream
+  nsFileSpec fileSpec;
+  aFileSpec->GetFileSpec(&fileSpec);
+  fileSpec.Delete(PR_FALSE);
+
+  rv = aFileSpec->GetOutputStream(getter_AddRefs(m_outputStream));
+  NS_ENSURE_SUCCESS(rv,rv);
+
+  if (m_outputStream && addDummyEnvelope)
+  {
+    nsCAutoString result;
+    char *ct;
+    PRUint32 writeCount;
+    time_t now = time ((time_t*) 0);
+    ct = ctime(&now);
+    ct[24] = 0;
+    result = "From - ";
+    result += ct;
+    result += MSG_LINEBREAK;
+    
+    m_outputStream->Write(result.get(), result.Length(),
+                               &writeCount);
+    result = "X-Mozilla-Status: 0001";
+    result += MSG_LINEBREAK;
+    m_outputStream->Write(result.get(), result.Length(),
+                               &writeCount);
+    result =  "X-Mozilla-Status2: 00000000";
+    result += MSG_LINEBREAK;
+    m_outputStream->Write(result.get(), result.Length(),
+                               &writeCount);
+  }
+  return rv;
+}
+
+
+NS_IMETHODIMP nsMsgMailNewsUrl::GetSaveAsListener(PRBool addDummyEnvelope, 
+                                                  nsIFileSpec *aFileSpec, nsIStreamListener **aSaveListener)
+{
+  NS_ENSURE_ARG_POINTER(aSaveListener);
+  nsMsgSaveAsListener *saveAsListener = new nsMsgSaveAsListener(aFileSpec, addDummyEnvelope);
+  return saveAsListener->QueryInterface(NS_GET_IID(nsIStreamListener), (void **) aSaveListener);
+}
+
+
+NS_IMETHODIMP nsMsgMailNewsUrl::SetFolder(nsIMsgFolder * /* aFolder */)
+{
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+NS_IMETHODIMP nsMsgMailNewsUrl::GetFolder(nsIMsgFolder ** /* aFolder */)
+{
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
+
