@@ -25,8 +25,8 @@
 #include "nsMsgComposeSecure.h"
 #include "nspr.h"
 #include "nsCOMPtr.h"
-#include "nsICMS.h"
 #include "nsIX509Cert.h"
+#include "nsISMimeCert.h"
 #include "nsIMimeConverter.h"
 #include "nsMimeStringResources.h"
 #include "nsMimeTypes.h"
@@ -38,6 +38,9 @@
 #include "nsIServiceManager.h"
 #include "nsIMsgIdentity.h"
 #include "nsIMsgCompFields.h"
+#include "nsCRT.h"
+#include "nsReadableUtils.h"
+#include "nsArray.h"
 
 // String bundle for smime. Class static.
 nsCOMPtr<nsIStringBundle> nsMsgComposeSecure::mSMIMEBundle = nsnull;
@@ -68,15 +71,12 @@ static char *mime_make_separator(const char *prefix);
 #include "nsMsgMimeCID.h"
 #include "nsIMimeConverter.h"
 
-/* This is the next generation string retrieval call */
-static NS_DEFINE_CID(kCMimeConverterCID, NS_MIME_CONVERTER_CID);
-
 MimeEncoderData *
 MIME_B64EncoderInit(nsresult (*PR_CALLBACK output_fn) (const char *buf, PRInt32 size, void *closure), void *closure) 
 {
   MimeEncoderData *returnEncoderData = nsnull;
   nsIMimeConverter *converter;
-  nsresult res = nsComponentManager::CreateInstance(kCMimeConverterCID, nsnull, 
+  nsresult res = nsComponentManager::CreateInstance(NS_MIME_CONVERTER_CONTRACTID, nsnull, 
                                            NS_GET_IID(nsIMimeConverter), (void **)&converter);
   if (NS_SUCCEEDED(res) && nsnull != converter) 
   {
@@ -91,7 +91,7 @@ MIME_QPEncoderInit(nsresult (*PR_CALLBACK output_fn) (const char *buf, PRInt32 s
 {
   MimeEncoderData *returnEncoderData = nsnull;
   nsIMimeConverter *converter;
-  nsresult res = nsComponentManager::CreateInstance(kCMimeConverterCID, nsnull, 
+  nsresult res = nsComponentManager::CreateInstance(NS_MIME_CONVERTER_CONTRACTID, nsnull, 
                                            NS_GET_IID(nsIMimeConverter), (void **)&converter);
   if (NS_SUCCEEDED(res) && nsnull != converter) 
   {
@@ -106,7 +106,7 @@ MIME_UUEncoderInit(char *filename, nsresult (*PR_CALLBACK output_fn) (const char
 {
   MimeEncoderData *returnEncoderData = nsnull;
   nsIMimeConverter *converter;
-  nsresult res = nsComponentManager::CreateInstance(kCMimeConverterCID, nsnull, 
+  nsresult res = nsComponentManager::CreateInstance(NS_MIME_CONVERTER_CONTRACTID, nsnull, 
                                            NS_GET_IID(nsIMimeConverter), (void **)&converter);
   if (NS_SUCCEEDED(res) && nsnull != converter) 
   {
@@ -121,7 +121,7 @@ MIME_EncoderDestroy(MimeEncoderData *data, PRBool abort_p)
 {
   //MimeEncoderData *returnEncoderData = nsnull;
   nsIMimeConverter *converter;
-  nsresult res = nsComponentManager::CreateInstance(kCMimeConverterCID, nsnull, 
+  nsresult res = nsComponentManager::CreateInstance(NS_MIME_CONVERTER_CONTRACTID, nsnull, 
                                            NS_GET_IID(nsIMimeConverter), (void **)&converter);
   if (NS_SUCCEEDED(res) && nsnull != converter) 
   {
@@ -138,7 +138,7 @@ MIME_EncoderWrite(MimeEncoderData *data, const char *buffer, PRInt32 size)
   //  MimeEncoderData *returnEncoderData = nsnull;
   nsIMimeConverter *converter;
   PRInt32 written = 0;
-  nsresult res = nsComponentManager::CreateInstance(kCMimeConverterCID, nsnull, 
+  nsresult res = nsComponentManager::CreateInstance(NS_MIME_CONVERTER_CONTRACTID, nsnull, 
     NS_GET_IID(nsIMimeConverter), (void **)&converter);
   if (NS_SUCCEEDED(res) && nsnull != converter) {
     res = converter->EncoderWrite(data, buffer, size, &written);
@@ -191,8 +191,8 @@ char
 NS_IMPL_ISUPPORTS1(nsMsgSMIMEComposeFields, nsIMsgSMIMECompFields)
 
 nsMsgSMIMEComposeFields::nsMsgSMIMEComposeFields()
+:mSignMessage(PR_FALSE), mAlwaysEncryptMessage(PR_FALSE)
 {
-  NS_INIT_ISUPPORTS();
 }
 
 nsMsgSMIMEComposeFields::~nsMsgSMIMEComposeFields()
@@ -211,13 +211,13 @@ NS_IMETHODIMP nsMsgSMIMEComposeFields::GetSignMessage(PRBool *_retval)
   return NS_OK;
 }
 
-NS_IMETHODIMP nsMsgSMIMEComposeFields::SetAlwaysEncryptMessage(PRBool value)
+NS_IMETHODIMP nsMsgSMIMEComposeFields::SetRequireEncryptMessage(PRBool value)
 {
   mAlwaysEncryptMessage = value;
   return NS_OK;
 }
 
-NS_IMETHODIMP nsMsgSMIMEComposeFields::GetAlwaysEncryptMessage(PRBool *_retval)
+NS_IMETHODIMP nsMsgSMIMEComposeFields::GetRequireEncryptMessage(PRBool *_retval)
 {
   *_retval = mAlwaysEncryptMessage;
   return NS_OK;
@@ -231,7 +231,6 @@ NS_IMPL_ISUPPORTS1(nsMsgComposeSecure, nsIMsgComposeSecure)
 
 nsMsgComposeSecure::nsMsgComposeSecure()
 {
-  NS_INIT_ISUPPORTS();
   /* member initializers and constructor code */
   mStream = 0;
   mDataHash = 0;
@@ -243,12 +242,18 @@ nsMsgComposeSecure::nsMsgComposeSecure()
   mEncryptionCinfo = 0;
   mEncryptionContext = 0;
   mCryptoEncoderData = 0;
+  mBuffer = 0;
+  mBufferedBytes = 0;
 }
 
 nsMsgComposeSecure::~nsMsgComposeSecure()
 {
   /* destructor code */
   if (mEncryptionContext) {
+    if (mBufferedBytes) {
+      mEncryptionContext->Update(mBuffer, mBufferedBytes);
+      mBufferedBytes = 0;
+    }
     mEncryptionContext->Finish();
   }
 
@@ -259,6 +264,8 @@ nsMsgComposeSecure::~nsMsgComposeSecure()
 	  MIME_EncoderDestroy (mCryptoEncoderData, PR_TRUE);
   }
 
+  delete [] mBuffer;
+
   PR_FREEIF(mMultipartSignedBoundary);
 }
 
@@ -343,10 +350,67 @@ void nsMsgComposeSecure::InitializeSMIMEBundle()
                               getter_AddRefs(mSMIMEBundle));
 }
 
+void nsMsgComposeSecure::SetError(nsIMsgSendReport *sendReport, const PRUnichar *bundle_string)
+{
+  if (!sendReport || !bundle_string)
+    return;
+
+  if (mErrorAlreadyReported)
+    return;
+
+  mErrorAlreadyReported = PR_TRUE;
+  
+  nsXPIDLString errorString;
+  nsresult res;
+
+  res = GetSMIMEBundleString(bundle_string,
+                             getter_Copies(errorString));
+
+  if (NS_SUCCEEDED(res) && !errorString.IsEmpty())
+  {
+    sendReport->SetMessage(nsIMsgSendReport::process_Current,
+                           errorString.get(),
+                           PR_TRUE);
+  }
+}
+
+void nsMsgComposeSecure::SetErrorWithParam(nsIMsgSendReport *sendReport, const PRUnichar *bundle_string, const char *param)
+{
+  if (!sendReport || !bundle_string || !param)
+    return;
+
+  if (mErrorAlreadyReported)
+    return;
+
+  mErrorAlreadyReported = PR_TRUE;
+  
+  nsXPIDLString errorString;
+  nsresult res;
+  const PRUnichar *params[1];
+
+  NS_ConvertASCIItoUCS2 ucs2(param);
+  params[0]= ucs2.get();
+
+  res = SMIMEBundleFormatStringFromName(bundle_string,
+                                        params,
+                                        1,
+                                        getter_Copies(errorString));
+
+  if (NS_SUCCEEDED(res) && !errorString.IsEmpty())
+  {
+    sendReport->SetMessage(nsIMsgSendReport::process_Current,
+                           errorString.get(),
+                           PR_TRUE);
+  }
+}
+
 nsresult nsMsgComposeSecure::ExtractEncryptionState(nsIMsgIdentity * aIdentity, nsIMsgCompFields * aComposeFields, PRBool * aSignMessage, PRBool * aEncrypt)
 {
   if (!aComposeFields && !aIdentity)
-    return NS_OK; // kick out...invalid args....
+    return NS_ERROR_FAILURE; // kick out...invalid args....
+
+  NS_ENSURE_ARG(aSignMessage);
+  NS_ENSURE_ARG(aEncrypt);
 
   nsCOMPtr<nsISupports> securityInfo;
   if (aComposeFields)
@@ -358,15 +422,26 @@ nsresult nsMsgComposeSecure::ExtractEncryptionState(nsIMsgIdentity * aIdentity,
     if (smimeCompFields)
     {
       smimeCompFields->GetSignMessage(aSignMessage);
-      smimeCompFields->GetAlwaysEncryptMessage(aEncrypt);
+      smimeCompFields->GetRequireEncryptMessage(aEncrypt);
+      return NS_OK;
     }
   }
-  else if (aIdentity)  // get the default info from the identity....
-  {
-    aIdentity->GetBoolAttribute("encrypt_mail_always", aEncrypt);
-    aIdentity->GetBoolAttribute("sign_mail", aSignMessage);
+
+  // get the default info from the identity....
+  PRInt32 ep = 0;
+  nsresult testrv = aIdentity->GetIntAttribute("encryptionpolicy", &ep);
+  if (NS_FAILED(testrv)) {
+    *aEncrypt = PR_FALSE;
+  }
+  else {
+    *aEncrypt = (ep > 0);
   }
 
+  testrv = aIdentity->GetBoolAttribute("sign_mail", aSignMessage);
+  if (NS_FAILED(testrv))
+  {
+    *aSignMessage = PR_FALSE;
+  }
   return NS_OK;
 }
 
@@ -378,6 +453,7 @@ NS_IMETHODIMP nsMsgComposeSecure::BeginCryptoEncapsulation(nsOutputFileStream *
                                                            nsIMsgSendReport *sendReport,
                                                            PRBool aIsDraft)
 {
+  mErrorAlreadyReported = PR_FALSE;
   nsresult rv = NS_OK;
 
   PRBool encryptMessages = PR_FALSE;
@@ -409,17 +485,17 @@ NS_IMETHODIMP nsMsgComposeSecure::BeginCryptoEncapsulation(nsOutputFileStream *
   switch (mCryptoState)
 	{
 	case mime_crypto_clear_signed:
-    rv = MimeInitMultipartSigned(PR_TRUE);
+    rv = MimeInitMultipartSigned(PR_TRUE, sendReport);
     break;
 	case mime_crypto_opaque_signed:
     PR_ASSERT(0);    /* #### no api for this yet */
-    rv = -1;
+    rv = NS_ERROR_NOT_IMPLEMENTED;
 	  break;
 	case mime_crypto_signed_encrypted:
-    rv = MimeInitEncryption(PR_TRUE);
+    rv = MimeInitEncryption(PR_TRUE, sendReport);
 	  break;
 	case mime_crypto_encrypted:
-    rv = MimeInitEncryption(PR_FALSE);
+    rv = MimeInitEncryption(PR_FALSE, sendReport);
 	  break;
 	case mime_crypto_none:
 	  /* This can happen if mime_crypto_hack_certs() decided to turn off
@@ -436,24 +512,24 @@ FAIL:
 }
 
 /* void finishCryptoEncapsulation (in boolean aAbort); */
-NS_IMETHODIMP nsMsgComposeSecure::FinishCryptoEncapsulation(PRBool aAbort)
+NS_IMETHODIMP nsMsgComposeSecure::FinishCryptoEncapsulation(PRBool aAbort, nsIMsgSendReport *sendReport)
 {
-  nsresult rv;
+  nsresult rv = NS_OK;
 
   if (!aAbort) {
 	  switch (mCryptoState) {
 		case mime_crypto_clear_signed:
-      rv = MimeFinishMultipartSigned (PR_TRUE);
+      rv = MimeFinishMultipartSigned (PR_TRUE, sendReport);
       break;
     case mime_crypto_opaque_signed:
       PR_ASSERT(0);    /* #### no api for this yet */
       rv = NS_ERROR_FAILURE;
       break;
     case mime_crypto_signed_encrypted:
-      rv = MimeFinishEncryption (PR_TRUE);
+      rv = MimeFinishEncryption (PR_TRUE, sendReport);
 		  break;
 		case mime_crypto_encrypted:
-		  rv = MimeFinishEncryption (PR_FALSE);
+		  rv = MimeFinishEncryption (PR_FALSE, sendReport);
 		  break;
 		default:
 		  PR_ASSERT(0);
@@ -464,7 +540,7 @@ NS_IMETHODIMP nsMsgComposeSecure::FinishCryptoEncapsulation(PRBool aAbort)
   return rv;
 }
 
-nsresult nsMsgComposeSecure::MimeInitMultipartSigned(PRBool aOuter)
+nsresult nsMsgComposeSecure::MimeInitMultipartSigned(PRBool aOuter, nsIMsgSendReport *sendReport)
 {
   /* First, construct and write out the multipart/signed MIME header data.
    */
@@ -476,7 +552,7 @@ nsresult nsMsgComposeSecure::MimeInitMultipartSigned(PRBool aOuter)
 										&mMultipartSignedBoundary);
   if (NS_FAILED(rv)) goto FAIL;
 
-  L = nsCRT::strlen(header);
+  L = strlen(header);
 
   if (aOuter){
 	  /* If this is the outer block, write it to the file. */
@@ -512,7 +588,7 @@ nsresult nsMsgComposeSecure::MimeInitMultipartSigned(PRBool aOuter)
   return rv;
 }
 
-nsresult nsMsgComposeSecure::MimeInitEncryption(PRBool aSign)
+nsresult nsMsgComposeSecure::MimeInitEncryption(PRBool aSign, nsIMsgSendReport *sendReport)
 {
   nsresult rv;
 
@@ -530,7 +606,7 @@ nsresult nsMsgComposeSecure::MimeInitEncryption(PRBool aSign)
 				MIME_SMIME_ENCRYPTED_CONTENT_DESCRIPTION);
   PRInt32 L;
   if (!s) return NS_ERROR_OUT_OF_MEMORY;
-  L = nsCRT::strlen(s);
+  L = strlen(s);
   if (PRInt32(mStream->write(s, L)) < L) {
 	  return NS_ERROR_FAILURE;
   }
@@ -543,7 +619,7 @@ nsresult nsMsgComposeSecure::MimeInitEncryption(PRBool aSign)
 
   if (!mIsDraft) {
     PRUint32 numCerts;
-    mCerts->Count(&numCerts);
+    mCerts->GetLength(&numCerts);
     PR_ASSERT(numCerts > 0);
 	  if (numCerts == 0) return NS_ERROR_FAILURE;
   }
@@ -560,17 +636,27 @@ nsresult nsMsgComposeSecure::MimeInitEncryption(PRBool aSign)
   PR_ASSERT(mSelfEncryptionCert);
   PR_SetError(0,0);
   mEncryptionCinfo = do_CreateInstance(NS_CMSMESSAGE_CONTRACTID, &rv);
-  if (NS_FAILED(rv)) return 0;
-  rv = mEncryptionCinfo->CreateEncrypted(mCerts); // XXX Fix this later XXX //
+  if (NS_FAILED(rv)) return rv;
+  rv = mEncryptionCinfo->CreateEncrypted(mCerts);
   if (NS_FAILED(rv)) {
+    SetError(sendReport, NS_LITERAL_STRING("ErrorCanNotEncrypt").get());
 	  goto FAIL;
 	}
 
   mEncryptionContext = do_CreateInstance(NS_CMSENCODER_CONTRACTID, &rv);
-  if (NS_FAILED(rv)) return 0;
+  if (NS_FAILED(rv)) return rv;
 
-  rv = mEncryptionContext->Start(mEncryptionCinfo, mime_crypto_write_base64, mCryptoEncoderData); // XXX Fix this later XXX //
+  if (!mBuffer) {
+    mBuffer = new char[eBufferSize];
+    if (!mBuffer)
+      return NS_ERROR_OUT_OF_MEMORY;
+  }
+
+  mBufferedBytes = 0;
+
+  rv = mEncryptionContext->Start(mEncryptionCinfo, mime_crypto_write_base64, mCryptoEncoderData);
   if (NS_FAILED(rv)) {
+    SetError(sendReport, NS_LITERAL_STRING("ErrorCanNotEncrypt").get());
 	  goto FAIL;
 	}
 
@@ -578,7 +664,7 @@ nsresult nsMsgComposeSecure::MimeInitEncryption(PRBool aSign)
 	 the data to be encrypted, and initialize the sign-hashing code too.
    */
   if (aSign) {
-	  rv = MimeInitMultipartSigned(PR_FALSE);
+	  rv = MimeInitMultipartSigned(PR_FALSE, sendReport);
 	  if (NS_FAILED(rv)) goto FAIL;
 	}
 
@@ -586,7 +672,7 @@ nsresult nsMsgComposeSecure::MimeInitEncryption(PRBool aSign)
   return rv;
 }
 
-nsresult nsMsgComposeSecure::MimeFinishMultipartSigned (PRBool aOuter)
+nsresult nsMsgComposeSecure::MimeFinishMultipartSigned (PRBool aOuter, nsIMsgSendReport *sendReport)
 {
   int status;
   nsresult rv;
@@ -594,7 +680,6 @@ nsresult nsMsgComposeSecure::MimeFinishMultipartSigned (PRBool aOuter)
   unsigned char * sec_item_data;
   nsCOMPtr<nsICMSMessage> cinfo = do_CreateInstance(NS_CMSMESSAGE_CONTRACTID, &rv);
   nsCOMPtr<nsICMSEncoder> encoder = do_CreateInstance(NS_CMSENCODER_CONTRACTID, &rv);
-  PRStatus ds_status = PR_FAILURE;
   char * header = nsnull;
 
   /* Compute the hash...
@@ -640,7 +725,7 @@ nsresult nsMsgComposeSecure::MimeFinishMultipartSigned (PRBool aOuter)
 		goto FAIL;
 	}
 
-	L = nsCRT::strlen(header);
+	L = strlen(header);
 	if (aOuter) {
 		/* If this is the outer block, write it to the file. */
     if (PRInt32(mStream->write(header, L)) < L) {
@@ -652,7 +737,6 @@ nsresult nsMsgComposeSecure::MimeFinishMultipartSigned (PRBool aOuter)
   }
 
 	PR_Free(header);
-	if (status < 0) goto FAIL;
 
   /* Create the signature...
    */
@@ -663,6 +747,7 @@ nsresult nsMsgComposeSecure::MimeFinishMultipartSigned (PRBool aOuter)
   PR_SetError(0,0);
   rv = cinfo->CreateSigned(mSelfSigningCert, mSelfEncryptionCert, sec_item_data, sec_item_len);
   if (NS_FAILED(rv))	{
+    SetError(sendReport, NS_LITERAL_STRING("ErrorCanNotSign").get());
 	  goto FAIL;
 	}
 
@@ -684,12 +769,14 @@ nsresult nsMsgComposeSecure::MimeFinishMultipartSigned (PRBool aOuter)
   PR_SetError(0,0);
   rv = encoder->Start(cinfo, mime_crypto_write_base64, mSigEncoderData);
   if (NS_FAILED(rv)) {
+    SetError(sendReport, NS_LITERAL_STRING("ErrorCanNotSign").get());
 	  goto FAIL;
 	}
 
   // We're not passing in any data, so no update needed.
   rv = encoder->Finish();
   if (NS_FAILED(rv)) {
+    SetError(sendReport, NS_LITERAL_STRING("ErrorCanNotSign").get());
 	  goto FAIL;
 	}
 
@@ -714,7 +801,7 @@ nsresult nsMsgComposeSecure::MimeFinishMultipartSigned (PRBool aOuter)
 		rv = NS_ERROR_OUT_OF_MEMORY;
 		goto FAIL;
 	}
-	L = nsCRT::strlen(header);
+	L = strlen(header);
 	if (aOuter) {
 		/* If this is the outer block, write it to the file. */
 		if (PRInt32(mStream->write(header, L)) < L)
@@ -734,14 +821,14 @@ FAIL:
 /* Helper function for mime_finish_crypto_encapsulation() to close off
    an opaque crypto object (for encrypted or signed-and-encrypted messages.)
  */
-nsresult nsMsgComposeSecure::MimeFinishEncryption (PRBool aSign)
+nsresult nsMsgComposeSecure::MimeFinishEncryption (PRBool aSign, nsIMsgSendReport *sendReport)
 {
   nsresult rv;
 
   /* If this object is both encrypted and signed, close off the
 	 signature first (since it's inside.) */
   if (aSign) {
-	  rv = MimeFinishMultipartSigned (PR_FALSE);
+	  rv = MimeFinishMultipartSigned (PR_FALSE, sendReport);
     if (NS_FAILED(rv)) {
       goto FAIL;
     }
@@ -751,8 +838,18 @@ nsresult nsMsgComposeSecure::MimeFinishEncryption (PRBool aSign)
    */
   PR_ASSERT(mEncryptionContext);
 
+  if (mBufferedBytes) {
+    rv = mEncryptionContext->Update(mBuffer, mBufferedBytes);
+    mBufferedBytes = 0;
+    if (NS_FAILED(rv)) {
+      PR_ASSERT(PR_GetError() < 0);
+      goto FAIL;
+    }
+  }
+  
   rv = mEncryptionContext->Finish();
   if (NS_FAILED(rv)) {
+    SetError(sendReport, NS_LITERAL_STRING("ErrorCanNotEncrypt").get());
     goto FAIL;
 	}
 
@@ -770,6 +867,9 @@ nsresult nsMsgComposeSecure::MimeFinishEncryption (PRBool aSign)
   rv = MIME_EncoderDestroy(mCryptoEncoderData, PR_FALSE);
   mCryptoEncoderData = 0;
 
+  if (PRInt32(mStream->write(CRLF, 2)) < 2)
+    rv = NS_ERROR_FAILURE;
+
  FAIL:
   return rv;
 }
@@ -781,7 +881,6 @@ nsresult nsMsgComposeSecure::MimeCryptoHackCerts(const char *aRecipients,
                                                  PRBool aEncrypt,
                                                  PRBool aSign)
 {
-  int status = 0;
   char *all_mailboxes = 0, *mailboxes = 0, *mailbox_list = 0;
   const char *mailbox = 0;
   PRUint32 count = 0;
@@ -790,79 +889,82 @@ nsresult nsMsgComposeSecure::MimeCryptoHackCerts(const char *aRecipients,
   nsCOMPtr<nsIMsgHeaderParser> pHeader = do_GetService(NS_MAILNEWS_MIME_HEADER_PARSER_CONTRACTID, &res);
   NS_ENSURE_SUCCESS(res,res);
 
-  res = NS_NewISupportsArray(getter_AddRefs(mCerts));
+  res = NS_NewArray(getter_AddRefs(mCerts));
   if (NS_FAILED(res)) {
     return res;
   }
 
-  nsXPIDLString errorString;
-  PRBool no_clearsigning_p = PR_FALSE;
-
   PR_ASSERT(aEncrypt || aSign);
-  certdb->GetEmailEncryptionCert(mEncryptionCertName, getter_AddRefs(mSelfEncryptionCert));
-  certdb->GetEmailSigningCert(mSigningCertName, getter_AddRefs(mSelfSigningCert));
+  certdb->FindEmailEncryptionCert(mEncryptionCertName, getter_AddRefs(mSelfEncryptionCert));
+  certdb->FindEmailSigningCert(mSigningCertName, getter_AddRefs(mSelfSigningCert));
 
   // must have both the signing and encryption certs to sign
 	if ((mSelfSigningCert == nsnull) && aSign) {
-    res = GetSMIMEBundleString(NS_LITERAL_STRING("NoSenderSigningCert").get(),
-                               getter_Copies(errorString));
-    if ( NS_SUCCEEDED(res) ) {
-      res = NS_ERROR_FAILURE;
-    }
+    SetError(sendReport, NS_LITERAL_STRING("NoSenderSigningCert").get());
+    res = NS_ERROR_FAILURE;
 		goto FAIL;
 	}
 
-	if ((mSelfEncryptionCert == nsnull) && aSign) {
-    res = GetSMIMEBundleString(NS_LITERAL_STRING("SignNoSenderEncryptionCert").get(),
-                               getter_Copies(errorString));
-    if ( NS_SUCCEEDED(res) ) {
-      res = NS_ERROR_FAILURE;
-    }
-    goto FAIL;
-  }
-
 	if ((mSelfEncryptionCert == nsnull) && aEncrypt) {
-    res = GetSMIMEBundleString(NS_LITERAL_STRING("NoSenderEncryptionCert").get(),
-                               getter_Copies(errorString));
-    if ( NS_SUCCEEDED(res) ) {
-      res = NS_ERROR_FAILURE;
-    }
+    SetError(sendReport, NS_LITERAL_STRING("NoSenderEncryptionCert").get());
+    res = NS_ERROR_FAILURE;
     goto FAIL;
   }
 
   pHeader->ExtractHeaderAddressMailboxes(nsnull,aRecipients, &all_mailboxes);
   pHeader->RemoveDuplicateAddresses(nsnull, all_mailboxes, 0, PR_FALSE /*removeAliasesToMe*/, &mailboxes);
-  PR_FREEIF(all_mailboxes);
+  if (all_mailboxes) {
+    nsMemory::Free(all_mailboxes);
+    all_mailboxes = nsnull;
+  }
 
   if (mailboxes) {
 	  pHeader->ParseHeaderAddresses (nsnull, mailboxes, 0, &mailbox_list, &count);
+    nsMemory::Free(mailboxes);
+    mailboxes = nsnull;
   }
-  PR_FREEIF(mailboxes);
   if (count < 0) return count;
 
+  if (aEncrypt && mSelfEncryptionCert) {
+    // Make sure self's configured cert is prepared for being used
+    // as an email recipient cert.
+    
+    nsCOMPtr<nsISMimeCert> sc = do_QueryInterface(mSelfEncryptionCert);
+    if (sc) {
+      sc->SaveSMimeProfile();
+    }
+  }
+
   /* If the message is to be encrypted, then get the recipient certs */
   if (aEncrypt) {
 	  mailbox = mailbox_list;
-	  for (; count > 0; count--) {
-      nsCOMPtr<nsIX509Cert> cert;
-		  certdb->GetCertByEmailAddress(nsnull, mailbox, getter_AddRefs(cert));
-		  if (!cert) {
-        // failure to find an encryption cert is
-        // fatal for now. We won't be able to encrypt anyway
-        // ssaux 12/03/2001.
-        const PRUnichar *params[1];
-        // here I assume that mailbox contains ascii rather than utf8.
-        NS_ConvertASCIItoUCS2 wstr_mailbox(mailbox);
-        params[0]= wstr_mailbox.get();
-        res = 
-          SMIMEBundleFormatStringFromName(NS_LITERAL_STRING("MissingRecipientEncryptionCert").get(),
-                                          params,
-                                          1,
-                                          getter_Copies(errorString));
 
-        if ( NS_SUCCEEDED(res) ) {
-          res = NS_ERROR_FAILURE;
+    PRBool already_added_self_cert = PR_FALSE;
+
+	  for (; count > 0; count--) {
+	    nsCString mailbox_lowercase;
+	    ToLowerCase(nsDependentCString(mailbox), mailbox_lowercase);
+      nsCOMPtr<nsIX509Cert> cert;
+      certdb->FindCertByEmailAddress(nsnull, mailbox_lowercase.get(), getter_AddRefs(cert));
+      PRBool foundValidCert = PR_FALSE;
+
+		  if (cert) {
+        PRUint32 verification_result;
+
+        if (NS_SUCCEEDED(
+            cert->VerifyForUsage(nsIX509Cert::CERT_USAGE_EmailRecipient, &verification_result))
+            &&
+            nsIX509Cert::VERIFIED_OK == verification_result)
+        {
+          foundValidCert = PR_TRUE;
         }
+      }
+      
+      if (!foundValidCert) {
+        // Failure to find a valid encryption cert is fatal.
+        // here I assume that mailbox contains ascii rather than utf8.
+        SetErrorWithParam(sendReport, NS_LITERAL_STRING("MissingRecipientEncryptionCert").get(), mailbox);
+        res = NS_ERROR_FAILURE;
         goto FAIL;
 		  }
 
@@ -872,19 +974,27 @@ nsresult nsMsgComposeSecure::MimeCryptoHackCerts(const char *aRecipients,
 		 of the recipients if we're sending a signed-but-not-encrypted
 		 message.)
 	   */
-      mCerts->AppendElement(cert);
-		  mailbox += nsCRT::strlen(mailbox) + 1;
+
+      PRBool isSame;
+      if (NS_SUCCEEDED(cert->Equals(mSelfEncryptionCert, &isSame))
+          && isSame) {
+        already_added_self_cert = PR_TRUE;
+      }
+
+      mCerts->AppendElement(cert, PR_FALSE);
+      // To understand this loop, especially the "+= strlen +1", look at the documentation
+      // of ParseHeaderAddresses. Basically, it returns a list of zero terminated strings.
+		  mailbox += strlen(mailbox) + 1;
 	  }
+    
+    if (!already_added_self_cert) {
+      mCerts->AppendElement(mSelfEncryptionCert, PR_FALSE);
+    }
 	}
 FAIL:
-  PR_FREEIF(mailbox_list);
-
-  if (NS_FAILED(res) && errorString.get() && sendReport) {
-    sendReport->SetMessage(nsIMsgSendReport::process_Current,
-                           errorString.get(),
-                           PR_TRUE);
+  if (mailbox_list) {
+    nsMemory::Free(mailbox_list);
   }
-
   return res;
 }
 
@@ -922,13 +1032,31 @@ NS_IMETHODIMP nsMsgComposeSecure::MimeCryptoWriteBlock (const char *buf, PRInt32
 	  /* If we're encrypting, or signing-and-encrypting, write this data
 		 by filtering it through the crypto library. */
 
-	  rv = mEncryptionContext->Update(buf, size);
-    if (NS_FAILED(rv)) {
-		  status = PR_GetError();
-		  PR_ASSERT(status < 0);
-		  if (status >= 0) status = -1;
-		  goto FAIL;
-		}
+    /* We want to create equally sized encryption strings */
+    const char *inputBytesIterator = buf;
+    PRUint32 inputBytesLeft = size;
+
+    while (inputBytesLeft) {
+      const PRUint32 spaceLeftInBuffer = eBufferSize - mBufferedBytes;
+      const PRUint32 bytesToAppend = NS_MIN(inputBytesLeft, spaceLeftInBuffer);
+
+      memcpy(mBuffer+mBufferedBytes, inputBytesIterator, bytesToAppend);
+      mBufferedBytes += bytesToAppend;
+      
+      inputBytesIterator += bytesToAppend;
+      inputBytesLeft -= bytesToAppend;
+
+      if (eBufferSize == mBufferedBytes) {
+        rv = mEncryptionContext->Update(mBuffer, mBufferedBytes);
+        mBufferedBytes = 0;
+        if (NS_FAILED(rv)) {
+          status = PR_GetError();
+          PR_ASSERT(status < 0);
+          if (status >= 0) status = -1;
+          goto FAIL;
+        }
+      }
+    }
   } else {
 	  /* If we're not encrypting (presumably just signing) then write this
 		 data directly to the file. */
