@@ -36,6 +36,7 @@
  * ***** END LICENSE BLOCK ***** */
 
 #include "msgCore.h"
+#include "netCore.h"
 #include "nsImapOfflineSync.h"
 #include "nsImapMailFolder.h"
 #include "nsMsgFolderFlags.h"
@@ -48,14 +49,15 @@
 #include "nsIRequestObserver.h"
 #include "nsSpecialSystemDirectory.h"
 #include "nsIFileStream.h"
-static NS_DEFINE_CID(kMsgAccountManagerCID, NS_MSGACCOUNTMANAGER_CID);
+#include "nsIMsgCopyService.h"
+#include "nsImapProtocol.h"
+
 static NS_DEFINE_CID(kRDFServiceCID, NS_RDFSERVICE_CID);
 
 NS_IMPL_ISUPPORTS2(nsImapOfflineSync, nsIUrlListener, nsIMsgCopyServiceListener)
 
 nsImapOfflineSync::nsImapOfflineSync(nsIMsgWindow *window, nsIUrlListener *listener, nsIMsgFolder *singleFolderOnly)
 {
-  NS_INIT_REFCNT();
   m_singleFolderToUpdate = singleFolderOnly;
   m_window = window;
   // not the perfect place for this, but I think it will work.
@@ -95,21 +97,35 @@ nsImapOfflineSync::OnStopRunningUrl(nsIURI* url, nsresult exitCode)
   PRBool stopped = PR_FALSE;
   if (m_window)
     m_window->GetStopped(&stopped);
-  if (stopped)
-    exitCode = NS_BINDING_ABORTED;
 
   if (m_curTempFile)
   {
     m_curTempFile->Delete(PR_FALSE);
     m_curTempFile = nsnull;
   }
+  if (stopped)
+  {
+    if (m_listener)
+      m_listener->OnStopRunningUrl(url, NS_BINDING_ABORTED);
+    return NS_OK;
+  }
+  nsCOMPtr<nsIImapUrl> imapUrl = do_QueryInterface(url);
+
+  if (imapUrl)
+    nsImapProtocol::LogImapUrl(NS_SUCCEEDED(rv) ? "offline imap url succeeded:" : "offline imap url failed:", imapUrl);
   // NS_BINDING_ABORTED is used for the user pressing stop, which
   // should cause us to abort the offline process. Other errors
   // should allow us to continue.
-  if (exitCode != NS_BINDING_ABORTED)
+  if (NS_SUCCEEDED(exitCode))
     rv = ProcessNextOperation();
-  else if (m_listener)  // notify main observer.
-    m_listener->OnStopRunningUrl(url, exitCode);
+  // else if it's a non-stop error, and we're doing multiple folders,
+  // go to the next folder.
+  else if (!m_singleFolderToUpdate)
+  {
+    rv = AdvanceToNextFolder();
+    if (NS_SUCCEEDED(rv))
+      rv = ProcessNextOperation();
+  }
 
   return rv;
 }
@@ -136,7 +152,7 @@ nsresult nsImapOfflineSync::AdvanceToNextServer()
   m_currentServer = nsnull;
   PRUint32 numServers; 
   m_allServers->Count(&numServers);
-  nsCOMPtr <nsIFolder> rootFolder;
+  nsCOMPtr <nsIMsgFolder> rootFolder;
 
   while (serverIndex < numServers)
   {
@@ -175,7 +191,11 @@ nsresult nsImapOfflineSync::AdvanceToNextFolder()
 	// we always start by changing flags
   mCurrentPlaybackOpType = nsIMsgOfflineImapOperation::kFlagsChanged;
 	
-  m_currentFolder = nsnull;
+  if (m_currentFolder)
+  {
+    m_currentFolder->SetMsgDatabase(nsnull);
+    m_currentFolder = nsnull;
+  }
 
   if (!m_currentServer)
      rv = AdvanceToNextServer();
@@ -344,11 +364,13 @@ nsImapOfflineSync::ProcessAppendMsgOperation(nsIMsgOfflineImapOperation *current
                   if (NS_SUCCEEDED(rv))
                   {
                     m_curTempFile = tempFileSpec;
-                    rv = destFolder->CopyFileMessage(tempFileSpec,
+                    nsCOMPtr<nsIMsgCopyService> copyService = do_GetService(NS_MSGCOPYSERVICE_CONTRACTID);
+                    if (copyService)
+                      rv = copyService->CopyFileMessage(tempFileSpec, destFolder,
                       /* nsIMsgDBHdr* msgToReplace */ nsnull,
                       PR_TRUE /* isDraftOrTemplate */,
-                      m_window,
-                      this);
+                        this,
+                        m_window);
                   }
                   else
                     m_curTempFile->Delete(PR_FALSE);
@@ -439,7 +461,9 @@ void nsImapOfflineSync::ProcessMoveOperation(nsIMsgOfflineImapOperation *current
               messages->AppendElement(iSupports);
             }
           }
-          destFolder->CopyMessages(m_currentFolder, messages, PR_TRUE, m_window, this, PR_FALSE, PR_FALSE);
+          nsCOMPtr<nsIMsgCopyService> copyService = do_GetService(NS_MSGCOPYSERVICE_CONTRACTID, &rv);
+          if (copyService)
+            copyService->CopyMessages(m_currentFolder, messages, destFolder, PR_TRUE, this, m_window, PR_FALSE);
         }
       }
     }
@@ -534,7 +558,9 @@ void nsImapOfflineSync::ProcessCopyOperation(nsIMsgOfflineImapOperation *current
               messages->AppendElement(iSupports);
             }
           }
-          destFolder->CopyMessages(m_currentFolder, messages, PR_FALSE, m_window, this, PR_FALSE, PR_FALSE);
+          nsCOMPtr<nsIMsgCopyService> copyService = do_GetService(NS_MSGCOPYSERVICE_CONTRACTID, &rv);
+          if (copyService)
+            copyService->CopyMessages(m_currentFolder, messages, destFolder, PR_FALSE, this, m_window, PR_FALSE);
         }
       }
     }
@@ -569,7 +595,7 @@ PRBool nsImapOfflineSync::CreateOfflineFolders()
 
 PRBool nsImapOfflineSync::CreateOfflineFolder(nsIMsgFolder *folder)
 {
-  nsCOMPtr<nsIFolder> parent;
+  nsCOMPtr<nsIMsgFolder> parent;
   folder->GetParent(getter_AddRefs(parent));
 
   nsCOMPtr <nsIMsgImapMailFolder> imapFolder = do_QueryInterface(parent);
@@ -578,8 +604,7 @@ PRBool nsImapOfflineSync::CreateOfflineFolder(nsIMsgFolder *folder)
   imapFolder->GetOnlineName(getter_Copies(onlineName));
 
   NS_ConvertASCIItoUCS2 folderName(onlineName);
-//  folderName.AssignWithConversion(onlineName);
-	nsresult rv = imapFolder->PlaybackOfflineFolderCreate(folderName.get(), nsnull,  getter_AddRefs(createFolderURI));
+  nsresult rv = imapFolder->PlaybackOfflineFolderCreate(folderName.get(), nsnull,  getter_AddRefs(createFolderURI));
   if (createFolderURI && NS_SUCCEEDED(rv))
   {
     nsCOMPtr <nsIMsgMailNewsUrl> mailnewsUrl = do_QueryInterface(createFolderURI);
@@ -591,17 +616,11 @@ PRBool nsImapOfflineSync::CreateOfflineFolder(nsIMsgFolder *folder)
 
 PRInt32 nsImapOfflineSync::GetCurrentUIDValidity()
 {
-   uid_validity_info uidStruct;
-
   if (m_currentFolder)
   {
-    nsCOMPtr <nsIImapMiscellaneousSink> miscellaneousSink = do_QueryInterface(m_currentFolder);
-    if (miscellaneousSink)
-    {
-      uidStruct.returnValidity = kUidUnknown;
-      miscellaneousSink->GetStoredUIDValidity(nsnull, &uidStruct);
-      mCurrentUIDValidity = uidStruct.returnValidity;    
-    }
+    nsCOMPtr <nsIImapMailFolderSink> imapFolderSink = do_QueryInterface(m_currentFolder);
+    if (imapFolderSink)
+      imapFolderSink->GetUidValidity(&mCurrentUIDValidity);
   }
   return mCurrentUIDValidity; 
 }
@@ -646,11 +665,12 @@ nsresult nsImapOfflineSync::ProcessNextOperation()
   while (m_currentFolder && !m_currentDB)
   {
     m_currentFolder->GetFlags(&folderFlags);
-    // need to check if folder has offline events, or is configured for offline
-    if (folderFlags & (MSG_FOLDER_FLAG_OFFLINEEVENTS | MSG_FOLDER_FLAG_OFFLINE))
-    {
+    // need to check if folder has offline events, /* or is configured for offline */
+    // shouldn't need to check if configured for offline use, since any folder with
+    // events should have MSG_FOLDER_FLAG_OFFLINEEVENTS set.
+    if (folderFlags & (MSG_FOLDER_FLAG_OFFLINEEVENTS /* | MSG_FOLDER_FLAG_OFFLINE */))
       m_currentFolder->GetDBFolderInfoAndDB(getter_AddRefs(folderInfo), getter_AddRefs(m_currentDB));
-    }
+
     if (m_currentDB)
     {
       m_CurrentKeys.RemoveAll();
@@ -658,6 +678,7 @@ nsresult nsImapOfflineSync::ProcessNextOperation()
       if ((m_currentDB->ListAllOfflineOpIds(&m_CurrentKeys) != 0) || !m_CurrentKeys.GetSize())
       {
         m_currentDB = nsnull;
+        folderInfo = nsnull; // can't hold onto folderInfo longer than db
         m_currentFolder->ClearFlag(MSG_FOLDER_FLAG_OFFLINEEVENTS);
       }
       else
@@ -699,7 +720,7 @@ nsresult nsImapOfflineSync::ProcessNextOperation()
         else if (folderFlags & MSG_FOLDER_FLAG_IMAPBOX)
         {
           //					if (imapFolder->GetHasOfflineEvents())
-          //						XP_ASSERT(PR_FALSE);
+          //						NS_ASSERTION(PR_FALSE, "Hardcoded assertion");
           
           if (!m_pseudoOffline)	// if pseudo offline, falls through to playing ops back.
           {
@@ -918,7 +939,7 @@ void nsImapOfflineSync::DeleteAllOfflineOpsForCurrentDB()
     if (++m_KeyIndex < m_CurrentKeys.GetSize())
       m_currentDB->GetOfflineOpForKey(m_CurrentKeys[m_KeyIndex], PR_FALSE, getter_AddRefs(currentOp));
   }
-  // turn off MSG_FOLDER_PREF_OFFLINEEVENTS
+  // turn off MSG_FOLDER_FLAG_OFFLINEEVENTS
   if (m_currentFolder)
     m_currentFolder->ClearFlag(MSG_FOLDER_FLAG_OFFLINEEVENTS);
 }
@@ -940,7 +961,7 @@ nsresult nsImapOfflineDownloader::ProcessNextOperation()
     // Update the INBOX first so the updates on the remaining
     // folders pickup the results of any filter moves.
     nsCOMPtr<nsIMsgAccountManager> accountManager = 
-             do_GetService(kMsgAccountManagerCID, &rv);
+             do_GetService(NS_MSGACCOUNTMANAGER_CONTRACTID, &rv);
     if (NS_FAILED(rv)) return rv;
     nsCOMPtr<nsISupportsArray> servers;
   
@@ -952,14 +973,11 @@ nsresult nsImapOfflineDownloader::ProcessNextOperation()
     AdvanceToNextServer();
     if (m_currentServer)
     {
-      nsCOMPtr <nsIFolder> rootFolder;
-      m_currentServer->GetRootFolder(getter_AddRefs(rootFolder));
+      nsCOMPtr <nsIMsgFolder> rootMsgFolder;
+      m_currentServer->GetRootFolder(getter_AddRefs(rootMsgFolder));
       nsCOMPtr<nsIMsgFolder> inbox;
-      if (rootFolder)
+      if (rootMsgFolder)
       {
-        nsCOMPtr<nsIMsgFolder> rootMsgFolder = do_QueryInterface(rootFolder, &rv);
-        if (rootMsgFolder)
-        {
           PRUint32 numFolders;
           rootMsgFolder->GetFoldersWithFlag(MSG_FOLDER_FLAG_INBOX, 1, &numFolders, getter_AddRefs(inbox));
           if (inbox)
@@ -988,12 +1006,16 @@ nsresult nsImapOfflineDownloader::ProcessNextOperation()
             // so just advance to the next server.
             if (!imapInbox || offlineImapFolder)
             {
+              // here we should check if this a pop3 server/inbox, and the user doesn't want
+              // to download pop3 mail for offline use.
+              if (!imapInbox)
+              {
+              }
               rv = inbox->GetNewMessages(m_window, this);
               if (NS_SUCCEEDED(rv))
                 return rv; // otherwise, fall through.
             }
           }
-        }
       }
       return ProcessNextOperation(); // recurse and do next server.
     }
@@ -1014,11 +1036,15 @@ nsresult nsImapOfflineDownloader::ProcessNextOperation()
     if (m_currentFolder)
       imapFolder = do_QueryInterface(m_currentFolder);
     m_currentFolder->GetFlags(&folderFlags);
-		// need to check if folder has offline events, or is configured for offline
-		if (imapFolder && folderFlags & MSG_FOLDER_FLAG_OFFLINE)
-      return m_currentFolder->DownloadAllForOffline(this, m_window);
-    else
-      AdvanceToNextFolder();
+    // need to check if folder has offline events, or is configured for offline
+    if (imapFolder && folderFlags & MSG_FOLDER_FLAG_OFFLINE)
+    {
+      rv = m_currentFolder->DownloadAllForOffline(this, m_window);
+      if (NS_SUCCEEDED(rv) || rv == NS_BINDING_ABORTED)
+        return rv;
+      // if this fails and the user didn't cancel/stop, fall through to code that advances to next folder
+    }
+    AdvanceToNextFolder();
   }
   if (m_listener)
     m_listener->OnStopRunningUrl(nsnull, NS_OK);
