@@ -39,14 +39,20 @@
 /*
 */
 
+#ifdef MOZ_LOGGING
+// sorry, this has to be before the pre-compiled header
+#define FORCE_PR_LOG /* Allow logging in the release build */
+#endif
 
 #include "prthread.h"
 #include "prprf.h"
 #include "nscore.h"
 #include "nsCOMPtr.h"
 #include "nsISupportsArray.h"
+
 #include "nsIImportMail.h"
 #include "nsIImportGeneric.h"
+#include "nsXPCOM.h"
 #include "nsISupportsPrimitives.h"
 #include "nsIImportMailboxDescriptor.h"
 #include "nsCRT.h"
@@ -58,6 +64,7 @@
 #include "nsIFileSpec.h"
 
 #include "nsIMsgAccountManager.h"
+#include "nsIMessengerMigrator.h"
 #include "nsIMsgMailSession.h"
 #include "nsMsgBaseCID.h"
 #include "nsIMsgFolder.h"
@@ -69,11 +76,10 @@
 
 #include "ImportDebug.h"
 
-static NS_DEFINE_CID(kMsgAccountCID, NS_MSGACCOUNT_CID);
-static NS_DEFINE_CID(kMsgIdentityCID, NS_MSGIDENTITY_CID);
-static NS_DEFINE_CID(kMsgBiffManagerCID, NS_MSGBIFFMANAGER_CID);
+#define IMPORT_MSGS_URL       "chrome://messenger/locale/importMsgs.properties"
+
 static NS_DEFINE_CID(kProxyObjectManagerCID, NS_PROXYEVENT_MANAGER_CID);
-static NS_DEFINE_CID(kSupportsWStringCID, NS_SUPPORTS_WSTRING_CID);
+static NS_DEFINE_CID(kSupportsWStringCID, NS_SUPPORTS_STRING_CID);
 
 ////////////////////////////////////////////////////////////////////////
 
@@ -81,43 +87,6 @@ PR_STATIC_CALLBACK( void) ImportMailThread( void *stuff);
 
 static nsCOMPtr<nsIImportService>	gService;
 
-static void ConvertToUnicode(const char *pStr, nsString &dist)
-{
-	nsresult rv;
-
-	if (!gService)
-		gService = do_GetService(NS_IMPORTSERVICE_CONTRACTID, &rv);
-
-	if (gService)
-		rv = gService->SystemStringToUnicode(pStr, dist);
-
-	if (NS_FAILED(rv))
-	{
-		// XXX bad cast
-		NS_ASSERTION(NS_SUCCEEDED(rv), "decoding method may occur bad cast\n");
-		dist.AssignWithConversion(pStr);
-	}
-}
-
-static void ConvertFromUnicode(const PRUnichar *pStr, nsCString &dist)
-{
-	nsresult rv;
-
-	if (!gService)
-		gService = do_GetService(NS_IMPORTSERVICE_CONTRACTID, &rv);
-
-	if (gService)
-		rv = gService->SystemStringFromUnicode(pStr, dist);
-
-	if (NS_FAILED(rv))
-	{
-		// XXX bad cast
-		NS_ASSERTION(NS_SUCCEEDED(rv), "encodeing method may occur bad cast\n");
-		dist.AssignWithConversion(pStr);
-	}
-}
-
-
 class ImportThreadData;
 
 class nsImportGenericMail : public nsIImportGeneric
@@ -139,8 +108,8 @@ public:
 	/* boolean WantsProgress (); */
 	NS_IMETHOD WantsProgress(PRBool *_retval);
 
-    /* boolean BeginImport (in nsISupportsWString successLog, in nsISupportsWString errorLog, in boolean isAddrLocHome); */
-    NS_IMETHODIMP BeginImport(nsISupportsWString *successLog, nsISupportsWString *errorLog, PRBool isAddrLocHome, PRBool *_retval) ;
+    /* boolean BeginImport (in nsISupportsString successLog, in nsISupportsString errorLog, in boolean isAddrLocHome); */
+    NS_IMETHODIMP BeginImport(nsISupportsString *successLog, nsISupportsString *errorLog, PRBool isAddrLocHome, PRBool *_retval) ;
 
 	/* boolean ContinueImport (); */
 	NS_IMETHOD ContinueImport(PRBool *_retval);
@@ -152,31 +121,29 @@ public:
 	NS_IMETHOD CancelImport(void);
 
 private:
-	PRBool	GetAccount( nsIMsgFolder **ppFolder);
-	PRBool	FindAccount( nsIMsgFolder **ppFolder);
+	PRBool	CreateFolder( nsIMsgFolder **ppFolder);
 	void	GetDefaultMailboxes( void);
 	void	GetDefaultLocation( void);
 	void	GetDefaultDestination( void);
-	void	GetUniquePrettyName( nsIMsgAccountManager *pMgr, nsString& name);
-	void	GetMailboxName( PRUint32 index, nsISupportsWString *pStr);
+	void	GetMailboxName( PRUint32 index, nsISupportsString *pStr);
 
 public:
-	static void	SetLogs( nsString& success, nsString& error, nsISupportsWString *pSuccess, nsISupportsWString *pError);
+	static void	SetLogs( nsString& success, nsString& error, nsISupportsString *pSuccess, nsISupportsString *pError);
 	static void ReportError( PRInt32 id, const PRUnichar *pName, nsString *pStream);
 
 private:
-	PRUnichar *			m_pName;	// module name that created this interface
+	nsString			m_pName;	// module name that created this interface
 	nsIMsgFolder *		m_pDestFolder;
 	PRBool				m_deleteDestFolder;
-	PRBool				m_createdAccount;
-	nsIFileSpec *		m_pSrcLocation;
+	PRBool				m_createdFolder;
+	nsCOMPtr <nsIFileSpec> m_pSrcLocation;
 	PRBool				m_gotLocation;
 	PRBool				m_found;
 	PRBool				m_userVerify;
 	nsIImportMail *		m_pInterface;
 	nsISupportsArray *	m_pMailboxes;
-	nsISupportsWString *m_pSuccessLog;
-	nsISupportsWString *m_pErrorLog;
+	nsISupportsString *m_pSuccessLog;
+	nsISupportsString *m_pErrorLog;
 	PRUint32			m_totalSize;
 	PRBool				m_doImport;
 	ImportThreadData *	m_pThreadData;
@@ -192,11 +159,10 @@ public:
 	PRUint32				currentSize;
 	nsIMsgFolder *			destRoot;
 	PRBool					ownsDestRoot;
-	PRBool					ownsAccount;
 	nsISupportsArray *		boxes;
 	nsIImportMail *			mailImport;
-	nsISupportsWString *	successLog;
-	nsISupportsWString *	errorLog;
+	nsISupportsString *	successLog;
+	nsISupportsString *	errorLog;
 	PRUint32				currentMailbox;
 
 	ImportThreadData();
@@ -227,8 +193,6 @@ nsresult NS_NewGenericMail(nsIImportGeneric** aImportGeneric)
 
 nsImportGenericMail::nsImportGenericMail()
 {
-    NS_INIT_REFCNT();
-	m_pSrcLocation = nsnull;
 	m_found = PR_FALSE;
 	m_userVerify = PR_FALSE;
 	m_gotLocation = PR_FALSE;
@@ -242,9 +206,11 @@ nsImportGenericMail::nsImportGenericMail()
 
 	m_pDestFolder = nsnull;
 	m_deleteDestFolder = PR_FALSE;
-	m_createdAccount = PR_FALSE;
-	m_pName = nsnull;
+	m_createdFolder = PR_FALSE;
 
+  // Init logging module.
+  if (!IMPORTLOGMODULE)
+    IMPORTLOGMODULE = PR_NewLogModule("IMPORT");
 }
 
 
@@ -255,11 +221,7 @@ nsImportGenericMail::~nsImportGenericMail()
 		m_pThreadData = nsnull;
 	}
 	
-	if (m_pName)
-		nsCRT::free( m_pName);
-
 	NS_IF_RELEASE( m_pDestFolder);
-	NS_IF_RELEASE( m_pSrcLocation);
 	NS_IF_RELEASE( m_pInterface);
 	NS_IF_RELEASE( m_pMailboxes);
 	NS_IF_RELEASE( m_pSuccessLog);
@@ -295,8 +257,7 @@ NS_IMETHODIMP nsImportGenericMail::GetData(const char *dataId, nsISupports **_re
 	if (!nsCRT::strcasecmp( dataId, "mailLocation")) {
 		if (!m_pSrcLocation)
 			GetDefaultLocation();
-		*_retval = m_pSrcLocation;
-		NS_IF_ADDREF( m_pSrcLocation);
+		NS_IF_ADDREF(*_retval = m_pSrcLocation);
 	}
 	
 	if (!nsCRT::strcasecmp( dataId, "mailDestination")) {
@@ -307,10 +268,10 @@ NS_IMETHODIMP nsImportGenericMail::GetData(const char *dataId, nsISupports **_re
 	}
 	
 	if (!nsCRT::strcasecmp( dataId, "currentMailbox")) {
-		// create an nsISupportsWString, get the current mailbox
+		// create an nsISupportsString, get the current mailbox
 		// name being imported and put it in the string
-		nsCOMPtr<nsISupportsWString>	data;
-		rv = nsComponentManager::CreateInstance( kSupportsWStringCID, nsnull, NS_GET_IID(nsISupportsWString), getter_AddRefs( data));
+		nsCOMPtr<nsISupportsString>	data;
+		rv = nsComponentManager::CreateInstance( kSupportsWStringCID, nsnull, NS_GET_IID(nsISupportsString), getter_AddRefs( data));
 		if (NS_FAILED( rv))
 			return( rv);
 		if (m_pThreadData) {
@@ -325,6 +286,7 @@ NS_IMETHODIMP nsImportGenericMail::GetData(const char *dataId, nsISupports **_re
 
 NS_IMETHODIMP nsImportGenericMail::SetData( const char *dataId, nsISupports *item)
 {
+    nsresult rv = NS_OK;
 	NS_PRECONDITION(dataId != nsnull, "null ptr");
     if (!dataId)
         return NS_ERROR_NULL_POINTER;
@@ -342,9 +304,15 @@ NS_IMETHODIMP nsImportGenericMail::SetData( const char *dataId, nsISupports *ite
 	
 	if (!nsCRT::strcasecmp( dataId, "mailLocation")) {
 		NS_IF_RELEASE( m_pMailboxes);
-		NS_IF_RELEASE( m_pSrcLocation);
-		if (item)
-			item->QueryInterface( NS_GET_IID(nsIFileSpec), (void **) &m_pSrcLocation);
+    m_pSrcLocation = nsnull;
+    if (item) {
+      nsresult rv;
+      nsCOMPtr <nsILocalFile> location = do_QueryInterface(item, &rv);
+      NS_ENSURE_SUCCESS(rv,rv);
+      
+      rv = NS_NewFileSpecFromIFile(location, getter_AddRefs(m_pSrcLocation));
+      NS_ENSURE_SUCCESS(rv,rv);
+    }
 	}
 	
 	if (!nsCRT::strcasecmp( dataId, "mailDestination")) {
@@ -355,17 +323,14 @@ NS_IMETHODIMP nsImportGenericMail::SetData( const char *dataId, nsISupports *ite
 	}
 	
 	if (!nsCRT::strcasecmp( dataId, "name")) {
-		// BIG CHEAT, is this OK to do?
-		PRUnichar *pName = (PRUnichar *)item;
-		if (m_pName)
-			nsCRT::free( m_pName);
-		if (pName)
-			m_pName = nsCRT::strdup( pName);
-		else
-			m_pName = nsnull;
+		nsCOMPtr<nsISupportsString> nameString;
+		if (item) {
+			item->QueryInterface( NS_GET_IID(nsISupportsString), getter_AddRefs(nameString));
+			rv = nameString->GetData(m_pName);
+		}
 	}
 
-	return( NS_OK);
+	return rv;
 }
 
 NS_IMETHODIMP nsImportGenericMail::GetStatus( const char *statusKind, PRInt32 *_retval)
@@ -427,40 +392,14 @@ void nsImportGenericMail::GetDefaultDestination( void)
 
 	nsIMsgFolder *	rootFolder;
 	m_deleteDestFolder = PR_FALSE;
-	m_createdAccount = PR_FALSE;
-	if (GetAccount( &rootFolder)) {
+	m_createdFolder = PR_FALSE;
+	if (CreateFolder( &rootFolder)) {
 		m_pDestFolder = rootFolder;
 		m_deleteDestFolder = PR_TRUE;
-		m_createdAccount = PR_TRUE;
+		m_createdFolder = PR_TRUE;
 		return;
 	}
-
-	if (FindAccount( &rootFolder)) {
-		// create the sub folder for our import.
-		char *pName = nsnull;
-		rootFolder->GenerateUniqueSubfolderName( "Imported Mail", nsnull, &pName);
-		if (pName) {
-			IMPORT_LOG1( "* Creating folder for importing mail: %s\n", pName);
-
-			// need convert to Unicode
-			nsAutoString childName;
-			ConvertToUnicode(pName, childName);
-
-			rootFolder->CreateSubfolder( childName.get(), nsnull);
-			nsCOMPtr<nsISupports> subFolder;
-			rootFolder->GetChildNamed( pName, getter_AddRefs( subFolder));
-			nsMemory::Free(pName);
-			if (subFolder) {
-				subFolder->QueryInterface( NS_GET_IID(nsIMsgFolder), (void **) &m_pDestFolder);
-				if (m_pDestFolder)
-					m_deleteDestFolder = PR_TRUE;
-			}
-		}
-		rootFolder->Release();
-	}
-	if (!m_pDestFolder) {
-		IMPORT_LOG0( "*** FAILED to create a default import destination\n");
-	}
+  IMPORT_LOG0("*** GetDefaultDestination: Failed to create a default import destination folder.");
 }
 
 NS_IMETHODIMP nsImportGenericMail::WantsProgress(PRBool *_retval)
@@ -487,7 +426,6 @@ NS_IMETHODIMP nsImportGenericMail::WantsProgress(PRBool *_retval)
 	PRBool			result = PR_FALSE;
 
 	if (m_pMailboxes) {
-		nsISupports *	pSupports;
 		PRUint32		i;
 		PRBool			import;
 		PRUint32		count = 0;
@@ -497,20 +435,17 @@ NS_IMETHODIMP nsImportGenericMail::WantsProgress(PRBool *_retval)
 		rv = m_pMailboxes->Count( &count);
 		
 		for (i = 0; i < count; i++) {
-			pSupports = m_pMailboxes->ElementAt( i);
-			if (pSupports) {
-				nsCOMPtr<nsISupports> interface( dont_AddRef( pSupports));
-				nsCOMPtr<nsIImportMailboxDescriptor> box( do_QueryInterface( pSupports));
-				if (box) {
-					import = PR_FALSE;
-					size = 0;
-					rv = box->GetImport( &import);
-					if (import) {
-						rv = box->GetSize( &size);
-						result = PR_TRUE;
-					}
-					totalSize += size;
+			nsCOMPtr<nsIImportMailboxDescriptor> box =
+				do_QueryElementAt(m_pMailboxes, i);
+			if (box) {
+				import = PR_FALSE;
+				size = 0;
+				rv = box->GetImport( &import);
+				if (import) {
+					rv = box->GetSize( &size);
+					result = PR_TRUE;
 				}
+				totalSize += size;
 			}
 		}
 
@@ -524,26 +459,21 @@ NS_IMETHODIMP nsImportGenericMail::WantsProgress(PRBool *_retval)
 	return( NS_OK);
 }
 
-void nsImportGenericMail::GetMailboxName( PRUint32 index, nsISupportsWString *pStr)
+void nsImportGenericMail::GetMailboxName( PRUint32 index, nsISupportsString *pStr)
 {
 	if (m_pMailboxes) {
-		nsISupports *pSupports = m_pMailboxes->ElementAt( index);
-		if (pSupports) {
-			nsCOMPtr<nsISupports> iFace( dont_AddRef( pSupports));
-			nsCOMPtr<nsIImportMailboxDescriptor> box( do_QueryInterface( pSupports));
-			if (box) {
-				PRUnichar *pName = nsnull;
-				box->GetDisplayName( &pName);
-				if (pName) {
-					pStr->SetData( pName);
-					nsCRT::free( pName);
-				}
+		nsCOMPtr<nsIImportMailboxDescriptor> box(do_QueryElementAt(m_pMailboxes, index));
+		if (box) {
+			nsXPIDLString name;
+			box->GetDisplayName(getter_Copies(name));
+			if (!name.IsEmpty()) {
+				pStr->SetData(name);
 			}
 		}
-	}		
+	}
 }
 
-NS_IMETHODIMP nsImportGenericMail::BeginImport(nsISupportsWString *successLog, nsISupportsWString *errorLog, PRBool isAddrLocHome, PRBool *_retval)
+NS_IMETHODIMP nsImportGenericMail::BeginImport(nsISupportsString *successLog, nsISupportsString *errorLog, PRBool isAddrLocHome, PRBool *_retval)
 {
 	NS_PRECONDITION(_retval != nsnull, "null ptr");
     if (!_retval)
@@ -560,7 +490,7 @@ NS_IMETHODIMP nsImportGenericMail::BeginImport(nsISupportsWString *successLog, n
 	}
 	
 	if (!m_pInterface || !m_pMailboxes) {
-		IMPORT_LOG0( "*** Something is not set properly, interface, mailboxes, or destination.\n");
+    IMPORT_LOG0( "*** BeginImport: Either the interface or source mailbox is not set properly.");
 		nsImportStringBundle::GetStringByID( IMPORT_ERROR_MB_NOTINITIALIZED, error);
 		SetLogs( success, error, successLog, errorLog);
 		*_retval = PR_FALSE;
@@ -568,7 +498,7 @@ NS_IMETHODIMP nsImportGenericMail::BeginImport(nsISupportsWString *successLog, n
 	}
 	
 	if (!m_pDestFolder) {
-		IMPORT_LOG0( "*** No import destination.\n");
+    IMPORT_LOG0( "*** BeginImport: The destination mailbox is not set properly.");
 		nsImportStringBundle::GetStringByID( IMPORT_ERROR_MB_NODESTFOLDER, error);
 		SetLogs( success, error, successLog, errorLog);
 		*_retval = PR_FALSE;
@@ -700,35 +630,19 @@ void nsImportGenericMail::ReportError( PRInt32 id, const PRUnichar *pName, nsStr
 }
 
 
-void nsImportGenericMail::SetLogs( nsString& success, nsString& error, nsISupportsWString *pSuccess, nsISupportsWString *pError)
+void nsImportGenericMail::SetLogs( nsString& success, nsString& error, nsISupportsString *pSuccess, nsISupportsString *pError)
 {
-	nsString	str;
-	PRUnichar *	pStr = nsnull;
-	if (pSuccess) {
-		pSuccess->GetData( &pStr);
-		if (pStr) {
-			str = pStr;
-			nsCRT::free( pStr);
-			pStr = nsnull;
-			str.Append( success);
-			pSuccess->SetData( str.get());
-		}
-		else {
-			pSuccess->SetData( success.get());			
-		}
-	}
-	if (pError) {
-		pError->GetData( &pStr);
-		if (pStr) {
-			str = pStr;
-			nsCRT::free( pStr);
-			str.Append( error);
-			pError->SetData( str.get());
-		}
-		else {
-			pError->SetData( error.get());			
-		}
-	}	
+    nsAutoString str;
+    if (pSuccess) {
+        pSuccess->GetData(str);
+        str.Append(success);
+        pSuccess->SetData(str);
+    }
+    if (pError) {
+        pError->GetData(str);
+        str.Append(error);
+        pError->SetData(str);
+    }
 }
 
 NS_IMETHODIMP nsImportGenericMail::CancelImport(void)
@@ -753,7 +667,6 @@ ImportThreadData::ImportThreadData()
 	currentSize = 0;
 	destRoot = nsnull;
 	ownsDestRoot = PR_FALSE;
-	ownsAccount = PR_FALSE;
 	boxes = nsnull;
 	mailImport = nsnull;
 	successLog = nsnull;
@@ -803,11 +716,9 @@ void ImportThreadData::DriverAbort()
 PR_STATIC_CALLBACK( void)
 ImportMailThread( void *stuff)
 {
-	IMPORT_LOG0( "In Begin ImporMailThread\n");
-
 	ImportThreadData *pData = (ImportThreadData *)stuff;
 	
-	IMPORT_LOG0( "In ImportMailThread!!!!!!!\n");
+  IMPORT_LOG0("ImportMailThread: Starting...");
 		
 	nsresult rv = NS_OK;
 
@@ -816,7 +727,6 @@ ImportMailThread( void *stuff)
 	PRUint32	count = 0;
 	rv = pData->boxes->Count( &count);
 	
-	nsISupports *	pSupports;
 	PRUint32		i;
 	PRBool			import;
 	PRUint32		size;	
@@ -824,7 +734,6 @@ ImportMailThread( void *stuff)
 	PRUint32		newDepth;
 	nsString		lastName;
 	PRUnichar *		pName;
-	nsCString		strName;
 	
 	nsCOMPtr<nsIMsgFolder>  	curFolder( destRoot);
 	nsCOMPtr<nsIMsgFolder>		curProxy;
@@ -832,6 +741,7 @@ ImportMailThread( void *stuff)
 	nsCOMPtr<nsIMsgFolder>		newFolder;
 	nsCOMPtr<nsIFileSpec>  		outBox;
 	nsCOMPtr<nsISupports>		subFolder;
+	nsCOMPtr<nsIEnumerator>     enumerator;
 
 	PRBool						exists;
 	
@@ -847,154 +757,157 @@ ImportMailThread( void *stuff)
 		rv = proxyMgr->GetProxyForObject( NS_UI_THREAD_EVENTQ, NS_GET_IID(nsIMsgFolder),
 										curFolder, PROXY_SYNC | PROXY_ALWAYS, getter_AddRefs( curProxy));
 
-		IMPORT_LOG1( "Proxy result for curFolder: 0x%lx\n", (long) rv);
+		if (NS_SUCCEEDED(rv))
+			// GetSubfolders() will initialize folders if they are not already initialized.
+			curProxy->GetSubFolders(getter_AddRefs(enumerator));
+    else
+      IMPORT_LOG1( "*** ImportMailThread: Can't get the destination root folder proxy. rv=(0x%lx)", (long) rv);
 	}
 	else {
-		IMPORT_LOG0( "Unable to obtain proxy service to do import\n");
+    IMPORT_LOG0("*** ImportMailThread: Unable to obtain proxy service to do the import.");
 		nsImportStringBundle::GetStringByID( IMPORT_ERROR_MB_NOPROXY, error, bundle);
 		pData->abort = PR_TRUE;
 	}
 
-	IMPORT_LOG1( "Total number of mailboxes: %d\n", (int) count);
+
+  IMPORT_LOG1("ImportMailThread: Total number of folders to import = %d.", count);
 
   // Note that the front-end js script only displays one import result string so 
   // we combine both good and bad import status into one string (in var 'success').
 
 	for (i = 0; (i < count) && !(pData->abort); i++) {
-		pSupports = pData->boxes->ElementAt( i);
-		if (pSupports) {
-			nsCOMPtr<nsISupports> iFace( dont_AddRef( pSupports));
-			nsCOMPtr<nsIImportMailboxDescriptor> box( do_QueryInterface( pSupports));
-			if (box) {
-				pData->currentMailbox = i;
+		nsCOMPtr<nsIImportMailboxDescriptor> box =
+			do_QueryElementAt(pData->boxes, i);
+		if (box) {
+			pData->currentMailbox = i;
 
-				import = PR_FALSE;
-				size = 0;
-				rv = box->GetImport( &import);
-				if (import)
-					rv = box->GetSize( &size);
-				rv = box->GetDepth( &newDepth);
-				if (newDepth > depth) {
+			import = PR_FALSE;
+			size = 0;
+			rv = box->GetImport( &import);
+			if (import)
+				rv = box->GetSize( &size);
+			rv = box->GetDepth( &newDepth);
+			if (newDepth > depth) {
           // OK, we are going to add a subfolder under the last/previous folder we processed, so
           // find this folder (stored in 'lastName') who is going to be the new parent folder.
-					ConvertFromUnicode(lastName.get(), strName);
-					IMPORT_LOG1( "* Finding folder for child named: %s\n", strName.get());
-					rv = curProxy->GetChildNamed( strName.get(), getter_AddRefs( subFolder));
-					if (NS_FAILED( rv)) {
-						nsImportGenericMail::ReportError( IMPORT_ERROR_MB_FINDCHILD, lastName.get(), &success);
-						pData->fatalError = PR_TRUE;
-						break;
-					}
-
-					rv = proxyMgr->GetProxyForObject( NS_UI_THREAD_EVENTQ, NS_GET_IID(nsIMsgFolder), 
-													subFolder, PROXY_SYNC | PROXY_ALWAYS, getter_AddRefs( curProxy));
-					if (NS_FAILED( rv)) {
-						nsImportStringBundle::GetStringByID( IMPORT_ERROR_MB_NOPROXY, error, bundle);
-						pData->fatalError = PR_TRUE;
-						break;
-					}
-
-          // Make sure this new parent folder obj has the correct subfolder list so far.
-          nsCOMPtr<nsIEnumerator> enumerator;
-          rv = curProxy->GetSubFolders(getter_AddRefs(enumerator));
-
-					IMPORT_LOG1( "Created proxy for new subFolder: 0x%lx\n", (long) rv);
-				}
-				else if (newDepth < depth) {
-					rv = NS_OK;
-					while ((newDepth < depth) && NS_SUCCEEDED( rv)) {
-						nsCOMPtr<nsIFolder> parFolder;
-						curProxy->GetParent( getter_AddRefs( parFolder));
-						rv = proxyMgr->GetProxyForObject( NS_UI_THREAD_EVENTQ, NS_GET_IID(nsIMsgFolder),
-														parFolder, PROXY_SYNC | PROXY_ALWAYS, getter_AddRefs( curProxy));
-						depth--;
-					}
-					if (NS_FAILED( rv)) {
-						nsImportStringBundle::GetStringByID( IMPORT_ERROR_MB_NOPROXY, error, bundle);
-						pData->fatalError = PR_TRUE;
-						break;
-					}
-				}
-				depth = newDepth;
-				pName = nsnull;
-				box->GetDisplayName( &pName);
-				if (pName) {
-					lastName = pName;
-					nsCRT::free( pName);
-				}
-				else
-					lastName.Assign(NS_LITERAL_STRING("Unknown!"));
-				
-				ConvertFromUnicode(lastName.get(), strName);
-				exists = PR_FALSE;
-				rv = curProxy->ContainsChildNamed( strName.get(), &exists);
-				if (exists) {
-					char *pSubName = nsnull;
-					curProxy->GenerateUniqueSubfolderName( strName.get(), nsnull, &pSubName);
-					if (pSubName) {
-						strName.Assign(pSubName);
-						nsMemory::Free(pSubName);
-					}
-				}
-				ConvertToUnicode(strName.get(), lastName);
-				
-				IMPORT_LOG1( "* Creating new import folder: %s\n", strName.get());
-				nsAutoString newName;
-				ConvertToUnicode(strName.get(), newName);
-
-				rv = curProxy->CreateSubfolder( newName.get(),nsnull);
-				
-				IMPORT_LOG1( "New folder created, rv: 0x%lx\n", (long) rv);
-				if (NS_SUCCEEDED( rv)) {
-					rv = curProxy->GetChildNamed( strName.get(), getter_AddRefs( subFolder));
-					IMPORT_LOG1( "GetChildNamed for new folder returned rv: 0x%lx\n", (long) rv);
-					if (NS_SUCCEEDED( rv)) {
-						newFolder = do_QueryInterface( subFolder);
-						if (newFolder) {
-							newFolder->GetPath( getter_AddRefs( outBox));
-							IMPORT_LOG0( "Got path for newly created folder\n");
-						}
-						else {
-							IMPORT_LOG0( "Newly created folder not found\n");
-						}
-					}
-				}
-				
+        IMPORT_LOG1("ImportMailThread: Processing child folder '%s'.", NS_ConvertUCS2toUTF8(lastName).get());
+				rv = curProxy->GetChildNamed( lastName.get(), getter_AddRefs( subFolder));
 				if (NS_FAILED( rv)) {
-					nsImportGenericMail::ReportError( IMPORT_ERROR_MB_CREATE, lastName.get(), &success);
+          IMPORT_LOG1("*** ImportMailThread: Failed to get the interface for child folder '%s'.", NS_ConvertUCS2toUTF8(lastName).get());
+					nsImportGenericMail::ReportError( IMPORT_ERROR_MB_FINDCHILD, lastName.get(), &error);
+					pData->fatalError = PR_TRUE;
+					break;
 				}
 
-				if (size && import && newFolder && outBox && NS_SUCCEEDED( rv)) {
-					PRBool fatalError = PR_FALSE;
-					pData->currentSize = size;
-					PRUnichar *pSuccess = nsnull;
-					PRUnichar *pError = nsnull;
-					rv = pData->mailImport->ImportMailbox( box, outBox, &pError, &pSuccess, &fatalError);
-					if (pError) {
-						error.Append( pError);
-						nsCRT::free( pError);
-					}
-					if (pSuccess) {
-						success.Append( pSuccess);
-						nsCRT::free( pSuccess);
-					}
+				rv = proxyMgr->GetProxyForObject( NS_UI_THREAD_EVENTQ, NS_GET_IID(nsIMsgFolder), 
+												subFolder, PROXY_SYNC | PROXY_ALWAYS, getter_AddRefs( curProxy));
+				if (NS_FAILED( rv)) {
+          IMPORT_LOG1("*** ImportMailThread: Failed to get the proxy interface for child folder '%s'.", NS_ConvertUCS2toUTF8(lastName).get());
+					nsImportStringBundle::GetStringByID( IMPORT_ERROR_MB_NOPROXY, error, bundle);
+					pData->fatalError = PR_TRUE;
+					break;
+				}
 
-					pData->currentSize = 0;
-					pData->currentTotal += size;
+				// Make sure this new parent folder obj has the correct subfolder list so far.
+				rv = curProxy->GetSubFolders(getter_AddRefs(enumerator));
+			}
+			else if (newDepth < depth) {
+				rv = NS_OK;
+				while ((newDepth < depth) && NS_SUCCEEDED( rv)) {
+					nsCOMPtr<nsIMsgFolder> parFolder;
+					rv = curProxy->GetParent( getter_AddRefs( parFolder));
+          if (NS_FAILED( rv)) {
+            IMPORT_LOG1("*** ImportMailThread: Failed to get the interface for parent folder '%s'.", lastName.get());
+					  nsImportGenericMail::ReportError( IMPORT_ERROR_MB_FINDCHILD, lastName.get(), &error);
+					  pData->fatalError = PR_TRUE;
+					  break;
+				  }
+
+					rv = proxyMgr->GetProxyForObject( NS_UI_THREAD_EVENTQ, NS_GET_IID(nsIMsgFolder),
+													parFolder, PROXY_SYNC | PROXY_ALWAYS, getter_AddRefs( curProxy));
+					depth--;
+				}
+				if (NS_FAILED( rv)) {
+          IMPORT_LOG1("*** ImportMailThread: Failed to get the proxy interface for parent folder '%s'.", lastName.get());
+					nsImportStringBundle::GetStringByID( IMPORT_ERROR_MB_NOPROXY, error, bundle);
+					pData->fatalError = PR_TRUE;
+					break;
+				}
+			}
+			depth = newDepth;
+			pName = nsnull;
+			box->GetDisplayName( &pName);
+			if (pName) {
+				lastName = pName;
+				nsCRT::free( pName);
+			}
+			else
+				lastName.Assign(NS_LITERAL_STRING("Unknown!"));
+				
+			exists = PR_FALSE;
+			rv = curProxy->ContainsChildNamed( lastName.get(), &exists);
+			if (exists) {
+				nsXPIDLString subName;
+				curProxy->GenerateUniqueSubfolderName( lastName.get(), nsnull, getter_Copies(subName));
+				if (!subName.IsEmpty()) 
+					lastName.Assign(subName);
+			}
+				
+      IMPORT_LOG1("ImportMailThread: Creating new import folder '%s'.", NS_ConvertUCS2toUTF8(lastName).get());
+
+			rv = curProxy->CreateSubfolder( lastName.get(),nsnull);
+				
+			if (NS_SUCCEEDED( rv)) {
+				rv = curProxy->GetChildNamed( lastName.get(), getter_AddRefs( subFolder));
+				if (NS_SUCCEEDED( rv)) {
+					newFolder = do_QueryInterface( subFolder);
+					if (newFolder) {
+						newFolder->GetPath( getter_AddRefs( outBox));
+					}
+					else {
+            IMPORT_LOG1("*** ImportMailThread: Failed to locate subfolder interface '%s'.", lastName.get());
+					}
+				}
+        else
+          IMPORT_LOG1("*** ImportMailThread: Failed to locate subfolder '%s' after it's been created.", lastName.get());
+			}
+      else
+        IMPORT_LOG1("*** ImportMailThread: Failed to create subfolder '%s'.", lastName.get());
+				
+			if (NS_FAILED( rv)) {
+				nsImportGenericMail::ReportError( IMPORT_ERROR_MB_CREATE, lastName.get(), &error);
+			}
+
+			if (size && import && newFolder && outBox && NS_SUCCEEDED( rv)) {
+				PRBool fatalError = PR_FALSE;
+				pData->currentSize = size;
+				PRUnichar *pSuccess = nsnull;
+				PRUnichar *pError = nsnull;
+				rv = pData->mailImport->ImportMailbox( box, outBox, &pError, &pSuccess, &fatalError);
+				if (pError) {
+					error.Append( pError);
+					nsCRT::free( pError);
+				}
+				if (pSuccess) {
+					success.Append( pSuccess);
+					nsCRT::free( pSuccess);
+				}
+
+				pData->currentSize = 0;
+				pData->currentTotal += size;
 					
-					outBox->CloseStream();
+				outBox->CloseStream();
 
           // OK, we've copied the actual folder/file over if the folder size is not 0
           // (ie, the msg summary is no longer valid) so close the msg database so that
           // when the folder is reopened the folder db can be reconstructed (which
           // validates msg summary and forces folder to be reparsed).
-          newFolder->ForceDBClosed();
+				newFolder->ForceDBClosed();
 
-					if (fatalError) {
-						IMPORT_LOG1( "*** ImportMailbox returned fatalError, mailbox #%d\n", (int) i);
-						pData->fatalError = PR_TRUE;
-						break;
-					}
+				if (fatalError) {
+					IMPORT_LOG1( "*** ImportMailThread: ImportMailbox returned fatalError, mailbox #%d\n", (int) i);
+					pData->fatalError = PR_TRUE;
+					break;
 				}
 			}
 		}
@@ -1010,7 +923,7 @@ ImportMailThread( void *stuff)
 	nsImportGenericMail::SetLogs( success, error, pData->successLog, pData->errorLog);
 
 	if (pData->abort || pData->fatalError) {
-		IMPORT_LOG0( "Abort or fatalError set\n");
+		IMPORT_LOG0( "*** ImportMailThread: Abort or fatalError flag was set\n");
 		if (pData->ownsDestRoot) {
 			IMPORT_LOG0( "Calling destRoot->RecursiveDelete\n");
 
@@ -1027,237 +940,97 @@ ImportMailThread( void *stuff)
 
 }
 
-
-PRBool nsImportGenericMail::GetAccount( nsIMsgFolder **ppFolder)
+// Creates a folder in Local Folders with the module name + mail
+// for e.g: Outlook Mail
+PRBool nsImportGenericMail::CreateFolder( nsIMsgFolder **ppFolder)
 {
-	nsresult	rv;
-	
-	*ppFolder = nsnull;
-		
-	nsCOMPtr <nsIMsgAccountManager> accMgr = do_GetService(NS_MSGACCOUNTMANAGER_CONTRACTID, &rv);
+  nsresult rv;
+  *ppFolder = nsnull;
+
+  nsCOMPtr<nsIStringBundle> bundle;
+  nsCOMPtr<nsIStringBundleService> bundleService(do_GetService(
+                                     NS_STRINGBUNDLE_CONTRACTID, &rv));
+  if (NS_FAILED(rv) || !bundleService) 
+      return PR_FALSE;
+  rv = bundleService->CreateBundle(IMPORT_MSGS_URL, getter_AddRefs(bundle));
+  if (NS_FAILED(rv)) 
+      return PR_FALSE;
+  nsXPIDLString folderName;
+  if (!m_pName.IsEmpty()) {
+    const PRUnichar *moduleName[] = { m_pName.get() };
+    rv = bundle->FormatStringFromName(NS_LITERAL_STRING("ModuleFolderName").get(),
+                                      moduleName, 1,
+                                      getter_Copies(folderName));
+  }
+  else {
+    rv = bundle->GetStringFromName(NS_LITERAL_STRING("DefaultFolderName").get(),
+                                   getter_Copies(folderName));
+  }
+  if (NS_FAILED(rv)) {
+      IMPORT_LOG0( "*** Failed to get Folder Name!\n");
+      return PR_FALSE;
+  }
+  nsCOMPtr <nsIMsgAccountManager> accMgr = do_GetService(NS_MSGACCOUNTMANAGER_CONTRACTID, &rv);
+  if (NS_FAILED(rv)) {
+    IMPORT_LOG0( "*** Failed to create account manager!\n");
+    return PR_FALSE;
+  }
+
+  nsCOMPtr <nsIMsgIncomingServer> server;
+  rv = accMgr->GetLocalFoldersServer(getter_AddRefs(server)); 
+  // if Local Folders does not exist already, create it
+  if (NS_FAILED(rv) || !server)
+  {
+    nsCOMPtr <nsIMessengerMigrator> messengerMigrator = do_GetService(NS_MESSENGERMIGRATOR_CONTRACTID, &rv);
     if (NS_FAILED(rv)) {
-		IMPORT_LOG0( "*** Failed to create a account manager!\n");
-		return( PR_FALSE);
-	}
-
-	// Create a new account for the import
-	// TODO: Ensure that the named used for this server is unique.
-	// Get the default name from the import module.
-	nsString	prettyName;
-	if (m_pName)
-		prettyName = m_pName;
-	else
-		prettyName.Assign(NS_LITERAL_STRING("Imported Mail"));
-	
-	nsCOMPtr<nsIMsgIncomingServer> server;
-
-	// Let's find a host name we're not using yet for a new "none" server
-	int		count = 1;
-	char	hostName[30];
-	PR_snprintf( hostName, 30, "%s", "imported.mail");
-	while (count < 1000) {
-		rv = accMgr->FindServer( "import", hostName, "none", getter_AddRefs( server));
-		if (NS_SUCCEEDED( rv)) {
-			PR_snprintf( hostName, 30, "imported%d.mail", count);
-			count++;
-		}
-		else
-			break;
-	}
-	
-	if (NS_SUCCEEDED( rv)) {
-		IMPORT_LOG0( "*** Failed to find a unique 'none' server to create for importing\n");
-		return( PR_FALSE);
-	}
-	
-	rv = accMgr->CreateIncomingServer( "import", hostName, "none", getter_AddRefs( server));
-	if (NS_FAILED( rv)) {
-		IMPORT_LOG0( "*** Failed to create a 'none' incoming server\n");
-		return( PR_FALSE);
-	}
-	server->SetType( "none");
-
-    // usually, servers of type none can't be deleted; this must be
-    // overridden
-    server->SetCanDelete(PR_TRUE);
-
-	// Let's get a reasonable "pretty name" that doesn't exist yet?
-	GetUniquePrettyName( accMgr, prettyName);
-	
-	server->SetPrettyName( (PRUnichar *) prettyName.get());
-	// server->SetHostName( hostName);
-	// server->SetUsername( "import");  
-
-	
-	// create a new account with the server and identity.
-	nsCOMPtr<nsIMsgAccount>	account;
-
-	rv = accMgr->CreateAccount( getter_AddRefs( account));
-	if (NS_FAILED( rv)) {
-		IMPORT_LOG0( "*** Error creating new account\n");
-		return( PR_FALSE);
-	}
-	
-	rv = account->SetIncomingServer( server);	
-	if (NS_FAILED( rv)) {
-		IMPORT_LOG0( "*** Error setting incoming server on account\n");
-	}
-
-	nsCOMPtr<nsIFolder>	rootFolder;
-	rv = server->GetRootFolder( getter_AddRefs( rootFolder));
-	if (NS_SUCCEEDED( rv) && (rootFolder != nsnull)) {
-		rv = rootFolder->QueryInterface( NS_GET_IID(nsIMsgFolder), (void **)ppFolder);
-		if (NS_SUCCEEDED( rv)) {
-			IMPORT_LOG0( "****** CREATED NEW ACCOUNT FOR IMPORT\n");
-			return( PR_TRUE);
-		}
-	}
-			
-	IMPORT_LOG0( "****** FAILED TO CREATE NEW ACCOUNT FOR IMPORT\n");
-
-	return( PR_FALSE);
-}
-
-void nsImportGenericMail::GetUniquePrettyName( nsIMsgAccountManager *pMgr, nsString& name)
-{
-	nsString	newName = name;
-	nsString	num;
-	int			count = 1;
-	
-	nsCOMPtr<nsISupportsArray>	array;
-
-	nsresult rv = pMgr->GetAllServers( getter_AddRefs( array));
-	if (NS_FAILED( rv))
-		return;
-
-	PRBool		found;
-	PRUint32	sz = 0;
-	rv = array->Count( &sz);
-	if (!sz)
-		return;
-
-	nsCOMPtr<nsISupports>			sup;
-	nsCOMPtr<nsIMsgIncomingServer>	server;
-
-	do {
-		found = PR_FALSE;
-		for (int i = 0; (i < (int) sz) && !found; i++) {
-			rv = array->GetElementAt( i, getter_AddRefs( sup));
-			if (NS_SUCCEEDED( rv) && sup) {
-				server = do_QueryInterface( sup);
-				if (server) {
-					nsXPIDLString	prettyName;
-					rv = server->GetPrettyName( getter_Copies( prettyName));
-					if (NS_SUCCEEDED( rv)) {
-						if (newName.Equals( prettyName, nsCaseInsensitiveStringComparator()))
-							found = PR_TRUE;
-					}	
-				}
-			}
-		}
-		if (found) {
-			newName = name;
-			newName.Append(NS_LITERAL_STRING(" "));
-			newName.AppendInt( count);
-			count++;
-		}
-
-	} while (found == PR_TRUE);
-
-	name = newName;
-}
-
-PRBool nsImportGenericMail::FindAccount( nsIMsgFolder **ppFolder)
-{
-	nsresult	rv;
-	
-	*ppFolder = nsnull;
-
-	nsCOMPtr <nsIMsgAccountManager> accMgr = do_GetService(NS_MSGACCOUNTMANAGER_CONTRACTID, &rv);
+      IMPORT_LOG0( "*** Failed to create messenger migrator!\n");
+      return PR_FALSE;
+    }
+    rv = messengerMigrator->CreateLocalMailAccount(PR_FALSE);
     if (NS_FAILED(rv)) {
-		IMPORT_LOG0( "*** Failed to create a account manager!\n");
-		return( PR_FALSE);
-	}	
-	/*
-		First check the default account to see if it has a local mail
-		store.  If it does then use this account to import the mail.
-	*/
-	char *							pServerType = nsnull;
-	nsCOMPtr<nsIMsgAccount>			defAcc;
-	nsCOMPtr<nsIMsgIncomingServer>	localServer;
-	nsCOMPtr<nsIFolder>				rootFolder;
-	rv = accMgr->GetDefaultAccount( getter_AddRefs( defAcc));
-	if (NS_SUCCEEDED( rv) && (defAcc != nsnull)) {
-		rv = defAcc->GetIncomingServer( getter_AddRefs( localServer));
-		if (NS_SUCCEEDED( rv) && (localServer != nsnull)) {
-			rv = localServer->GetType( &pServerType);
-			if (NS_SUCCEEDED( rv) && pServerType &&
-				(!nsCRT::strcasecmp( pServerType, "none") ||
-				 !nsCRT::strcasecmp( pServerType, "POP3"))) {
-				nsCRT::free( pServerType);
-				rv = localServer->GetRootFolder( getter_AddRefs( rootFolder));
-				if (NS_SUCCEEDED( rv) && (rootFolder != nsnull)) {
-					rv = rootFolder->QueryInterface( NS_GET_IID(nsIMsgFolder), (void **)ppFolder);
-					if (NS_SUCCEEDED( rv))
-						return( PR_TRUE);
-				}
-			}
-			if (pServerType) {
-				nsCRT::free( pServerType);
-				pServerType = nsnull;
-			}
-		}
-	}
-
-	
-	/*
-		Either an error occurred or the default account does not exist or
-		the default account is not of type "none" or "POP3".  Look for the
-		first account that we can use.
-	*/
-
-	nsCOMPtr<nsISupportsArray> accounts;
-	rv = accMgr->GetAccounts( getter_AddRefs( accounts));
-	if (NS_FAILED( rv)) {
-		IMPORT_LOG0( "*** Failed to get the accounts from the account manager\n");
-		return( PR_FALSE);
-	}
-	
-	PRUint32	cnt = 0;
-	accounts->Count( &cnt);
-	nsISupports *	pSupports;
-	for (PRUint32 i = 0; i < cnt; i++) {
-		rv = accounts->GetElementAt( i, &pSupports);
-		if (NS_SUCCEEDED( rv) && pSupports) {
-			nsCOMPtr<nsISupports>		iFace( dont_AddRef( pSupports));
-			nsCOMPtr<nsIMsgAccount>		acc( do_QueryInterface( pSupports));
-			if (acc) {
-				nsCOMPtr<nsIMsgIncomingServer> server;
-				rv = acc->GetIncomingServer( getter_AddRefs( server));
-				if (NS_SUCCEEDED( rv) && (server != nsnull)) {
-					char *pType = nsnull;
-					rv = server->GetType( &pType);
-					if (pType) {
-						if (!nsCRT::strcasecmp( pType, "none") ||
-							!nsCRT::strcasecmp( pType, "POP3")) {
-							nsCRT::free( pType);
-							rv = server->GetRootFolder( getter_AddRefs( rootFolder));
-							if (NS_SUCCEEDED( rv) && (rootFolder != nsnull)) {   
-								rv = rootFolder->QueryInterface( NS_GET_IID(nsIMsgFolder), (void **) ppFolder);
-								if (NS_SUCCEEDED( rv))
-									return( PR_TRUE);
-							}
-						}
-						else {
-							nsCRT::free( pType);
-						}
-					}
-				}
-			}
-		}
-	}
-	
-	return( PR_FALSE);
-		
+      IMPORT_LOG0( "*** Failed to create Local Folders!\n");
+      return PR_FALSE;
+    }
+    rv = accMgr->GetLocalFoldersServer(getter_AddRefs(server)); 
+  }
+  if (NS_SUCCEEDED(rv) && server) {
+    nsCOMPtr <nsIMsgFolder> localRootFolder;
+    rv = server->GetRootMsgFolder(getter_AddRefs(localRootFolder)); 
+    if (localRootFolder) {
+      // we need to call GetSubFolders() so that the folders get initialized 
+      // if they are not initialized yet. 
+      nsCOMPtr <nsIEnumerator> aEnumerator;
+      rv = localRootFolder->GetSubFolders(getter_AddRefs(aEnumerator));
+      if (NS_SUCCEEDED(rv)) {
+        // check if the folder name we picked already exists.
+        PRBool exists = PR_FALSE;
+        rv = localRootFolder->ContainsChildNamed(folderName.get(), &exists);
+        if (exists) {
+          nsXPIDLString name;
+          localRootFolder->GenerateUniqueSubfolderName(folderName.get(), nsnull, getter_Copies(name));
+          if (!name.IsEmpty())
+            folderName.Assign(name);
+          else {
+            IMPORT_LOG0( "*** Failed to find a unique folder name!\n");
+            return PR_FALSE;
+          }
+        }
+        IMPORT_LOG1( "Creating folder for importing mail: '%s'\n", NS_ConvertUCS2toUTF8(folderName).get());
+        rv = localRootFolder->CreateSubfolder(folderName.get(), nsnull);
+        if (NS_SUCCEEDED(rv)) {
+          nsCOMPtr<nsISupports> subFolder;
+          rv = localRootFolder->GetChildNamed(folderName.get(), getter_AddRefs(subFolder));
+          if (subFolder) {
+            subFolder->QueryInterface(NS_GET_IID(nsIMsgFolder), (void **) ppFolder);
+            if (*ppFolder) {
+              IMPORT_LOG1("Folder '%s' created successfully\n", NS_ConvertUCS2toUTF8(folderName).get());
+              return PR_TRUE;
+            }
+          } // if subFolder
+        }
+      }
+    } // if localRootFolder
+  } // if server
+  IMPORT_LOG0("****** FAILED TO CREATE FOLDER FOR IMPORT\n");
+  return PR_FALSE;
 }
-
