@@ -86,10 +86,26 @@
 #include "nsITransport.h"
 #include "mimeebod.h"
 #include "mimeeobj.h"
+// <for functions="HTML2Plaintext,HTMLSantinize">
+#include "nsXPCOM.h"
+#include "nsParserCIID.h"
+#include "nsIParser.h"
+#include "nsIHTMLContentSink.h"
+#include "nsIContentSerializer.h"
+#include "nsLayoutCID.h"
+#include "nsIComponentManager.h"
+#include "nsReadableUtils.h"
+#include "nsIHTMLToTextSink.h"
+#include "mozISanitizingSerializer.h"
+// </for>
 
 
 static NS_DEFINE_CID(kPrefCID, NS_PREF_CID);
 static NS_DEFINE_CID(kIOServiceCID, NS_IOSERVICE_CID);
+// <for functions="HTML2Plaintext,HTMLSantinize">
+static NS_DEFINE_CID(kParserCID, NS_PARSER_CID);
+static NS_DEFINE_CID(kNavDTDCID, NS_CNAVDTD_CID);
+// </for>
 
 #ifdef HAVE_MIME_DATA_SLOT
 #define LOCK_LAST_CACHED_MESSAGE
@@ -140,7 +156,7 @@ ProcessBodyAsAttachment(MimeObject *obj, nsMsgAttachmentData **data)
   {
     char *fname = NULL;
     fname = mime_decode_filename(tmp->real_name, charset, obj->options);
-    PR_FREEIF(charset);
+    nsMemory::Free(charset);
     if (fname && fname != tmp->real_name)
     {
       PR_Free(tmp->real_name);
@@ -258,26 +274,21 @@ ValidateRealName(nsMsgAttachmentData *aAttach, MimeHeaders *aHdrs)
     nsString  newAttachName(NS_LITERAL_STRING("attachment"));
     nsresult  rv = NS_OK;
     nsCAutoString contentType (aAttach->real_type);
-    PRInt32 pos = contentType.FindCharInSet(";");
+    PRInt32 pos = contentType.FindChar(';');
     if (pos > 0)
       contentType.Truncate(pos);
 
     nsCOMPtr<nsIMIMEService> mimeFinder (do_GetService(NS_MIMESERVICE_CONTRACTID, &rv));
-    if (NS_SUCCEEDED(rv) && mimeFinder) 
+    if (NS_SUCCEEDED(rv)) 
     {
-      nsIMIMEInfo *mimeInfo = nsnull;
-      rv = mimeFinder->GetFromMIMEType(contentType.get(), &mimeInfo);
-      if (NS_SUCCEEDED(rv) && mimeInfo) 
-      {
-        char *aFileExtension = nsnull;
+      nsXPIDLCString fileExtension;
+      rv = mimeFinder->GetPrimaryExtension(contentType.get(), nsnull, getter_Copies(fileExtension));
 
-        if ( (NS_SUCCEEDED(mimeInfo->GetPrimaryExtension(&aFileExtension))) && aFileExtension)
-        {
-          newAttachName.Append(NS_LITERAL_STRING("."));
-          newAttachName.AppendWithConversion(aFileExtension);
-          PR_FREEIF(aFileExtension);
-        }
-      }        
+      if (NS_SUCCEEDED(rv) && !fileExtension.IsEmpty())
+      {
+        newAttachName.Append(PRUnichar('.'));
+        newAttachName.AppendWithConversion(fileExtension);
+      }
     }
 
     aAttach->real_name = ToNewCString(newAttachName);
@@ -314,12 +325,22 @@ GenerateAttachmentData(MimeObject *object, const char *aMessageURL, MimeDisplayO
   else
   {
     isIMAPPart = PR_FALSE;
-    urlSpec = mime_set_url_part(aMessageURL, part.get(), PR_TRUE);
+    char *no_part_url = nsnull;
+    if (options->part_to_load && options->format_out == nsMimeOutput::nsMimeMessageBodyDisplay)
+      no_part_url = mime_get_base_url(aMessageURL);
+    if (no_part_url) {
+      urlSpec = mime_set_url_part(no_part_url, part.get(), PR_TRUE);
+      PR_Free(no_part_url);
+    }
+    else
+      urlSpec = mime_set_url_part(aMessageURL, part.get(), PR_TRUE);
   }
 
   if (!urlSpec)
     return NS_ERROR_OUT_OF_MEMORY;
 
+  if ((options->format_out == nsMimeOutput::nsMimeMessageBodyDisplay) && (nsCRT::strncasecmp(aMessageURL, urlSpec, strlen(urlSpec)) == 0))
+    return NS_OK;
   nsMsgAttachmentData *tmp = &(aAttachData[attIndex++]);
   nsresult rv = nsMimeNewURI(&(tmp->url), urlSpec, nsnull);
 
@@ -341,7 +362,7 @@ GenerateAttachmentData(MimeObject *object, const char *aMessageURL, MimeDisplayO
       for (i = 0; i < 2 && !tmp->real_name; i ++)
       {
         PR_FREEIF(disp);
-        PR_FREEIF(charset);
+        nsMemory::Free(charset);
         disp = MimeHeaders_get(((MimeContainer *)object)->children[i]->headers, HEADER_CONTENT_DISPOSITION, PR_FALSE, PR_FALSE);
         tmp->real_name = MimeHeaders_get_parameter(disp, "filename", &charset, nsnull);
       }
@@ -356,7 +377,7 @@ GenerateAttachmentData(MimeObject *object, const char *aMessageURL, MimeDisplayO
 
       char *fname = nsnull;
       fname = mime_decode_filename(tmp->real_name, charset, options);
-      PR_FREEIF(charset);
+      nsMemory::Free(charset);
 
       if (fname && fname != tmp->real_name)
       {
@@ -379,10 +400,11 @@ GenerateAttachmentData(MimeObject *object, const char *aMessageURL, MimeDisplayO
       PR_FREEIF(tmp->real_name);
       tmp->real_name = MimeHeaders_get_parameter(disp, "name", &charset, nsnull);
       if (isAnAppleDoublePart)
-        for (i = 0; i < 2 && !tmp->real_name; i ++)
+        // the data fork is the 2nd part, and we should ALWAYS look there first for the file name
+        for (i = 1; i >= 0 && !tmp->real_name; i --) 
         {
           PR_FREEIF(disp);
-          PR_FREEIF(charset);
+          nsMemory::Free(charset);
           disp = MimeHeaders_get(((MimeContainer *)object)->children[i]->headers, HEADER_CONTENT_TYPE, PR_FALSE, PR_FALSE);
           tmp->real_name = MimeHeaders_get_parameter(disp, "name", &charset, nsnull);
         }
@@ -397,7 +419,7 @@ GenerateAttachmentData(MimeObject *object, const char *aMessageURL, MimeDisplayO
 
         char *fname = nsnull;
         fname = mime_decode_filename(tmp->real_name, charset, options);
-        PR_FREEIF(charset);
+        nsMemory::Free(charset);
 
         if (fname && fname != tmp->real_name)
         {
@@ -461,9 +483,7 @@ BuildAttachmentList(MimeObject *anObject, nsMsgAttachmentData *aAttachData, cons
       if (child->content_type)                          // and it's content-type is one of folowing...
         if (!nsCRT::strcasecmp (child->content_type, TEXT_PLAIN) ||
             !nsCRT::strcasecmp (child->content_type, TEXT_HTML) ||
-            !nsCRT::strcasecmp (child->content_type, TEXT_MDL) ||
-            !nsCRT::strcasecmp (child->content_type, MULTIPART_ALTERNATIVE) ||
-            !nsCRT::strcasecmp (child->content_type, MULTIPART_RELATED))
+            !nsCRT::strcasecmp (child->content_type, TEXT_MDL))
         {
           if (child->headers) // and finally, be sure it doesn't have a content-disposition: attachment 
           {
@@ -512,7 +532,6 @@ MimeGetAttachmentList(MimeObject *tobj, const char *aMessageURL, nsMsgAttachment
   MimeObject            *obj;
   MimeContainer         *cobj;
   PRInt32               n;
-  PRBool                isAlternativeOrRelated = PR_FALSE;
   PRBool                isAnInlineMessage;
 
   if (!data) 
@@ -520,7 +539,7 @@ MimeGetAttachmentList(MimeObject *tobj, const char *aMessageURL, nsMsgAttachment
   *data = nsnull;
 
   obj = mime_get_main_object(tobj);
-  if (nsnull == obj)
+  if (!obj)
     return 0;
 
   if (!mime_subclass_p(obj->clazz, (MimeObjectClass*) &mimeContainerClass))
@@ -531,9 +550,6 @@ MimeGetAttachmentList(MimeObject *tobj, const char *aMessageURL, nsMsgAttachment
       return ProcessBodyAsAttachment(obj, data);
   }
 
-  if (mime_subclass_p(obj->clazz, (MimeObjectClass*) &mimeMultipartAlternativeClass))
-    return 0;
-  
   isAnInlineMessage = mime_typep(obj, (MimeObjectClass *) &mimeMessageClass);
 
   cobj = (MimeContainer*) obj;
@@ -662,7 +678,7 @@ nsMimeNewURI(nsIURI** aInstancePtrResult, const char *aSpec, nsIURI *aBase)
 }
 
 extern "C" nsresult 
-SetMailCharacterSetToMsgWindow(MimeObject *obj, const PRUnichar *aCharacterSet)
+SetMailCharacterSetToMsgWindow(MimeObject *obj, const char *aCharacterSet)
 {
   nsresult rv = NS_OK;
 
@@ -684,7 +700,9 @@ SetMailCharacterSetToMsgWindow(MimeObject *obj, const PRUnichar *aCharacterSet)
             nsCOMPtr<nsIMsgWindow> msgWindow;
             msgurl->GetMsgWindow(getter_AddRefs(msgWindow));
             if (msgWindow)
-              rv = msgWindow->SetMailCharacterSet(aCharacterSet);
+              rv = msgWindow->SetMailCharacterSet(!nsCRT::strcasecmp(aCharacterSet, "us-ascii") ?
+                                                  "ISO-8859-1" :
+                                                  aCharacterSet);
           }
         }
       }
@@ -694,13 +712,6 @@ SetMailCharacterSetToMsgWindow(MimeObject *obj, const PRUnichar *aCharacterSet)
   return rv;
 }
 
-static char *
-mime_reformat_date(const char *date, void *stream_closure)
-{
-  /*  struct mime_stream_data *msd = (struct mime_stream_data *) stream_closure; */
-  return nsCRT::strdup(date);
-}
-
 static char *
 mime_file_type (const char *filename, void *stream_closure)
 {
@@ -751,31 +762,68 @@ int ConvertUsingEncoderAndDecoder(const char *stringToUse, PRInt32 inLength,
     rv = NS_ERROR_OUT_OF_MEMORY;
   }
   else {
-    // convert to unicode
-    rv = decoder->Convert(stringToUse, &srcLen, unichars, &unicharLength);
-    if (NS_SUCCEEDED(rv)) {
-      rv = encoder->GetMaxLength(unichars, unicharLength, &dstLength);
-      // allocale an output buffer
-      dstPtr = (char *) PR_Malloc(dstLength + 1);
-      if (dstPtr == nsnull) {
-        rv = NS_ERROR_OUT_OF_MEMORY;
-      }
-      else {
-        PRInt32 buffLength = dstLength;
-        // convert from unicode
-        rv = encoder->SetOutputErrorBehavior(nsIUnicodeEncoder::kOnError_Replace, nsnull, '?');
+    // convert to unicode, replacing failed chars with 0xFFFD as in
+    // the methode used in nsXMLHttpRequest::ConvertBodyToText and nsScanner::Append
+    // 
+    // We will need several pass to convert the whole string if it has invalid characters
+    // 'totalChars' is where the sum of the number of converted characters will be done
+    // 'dataLen' is the number of character left to convert
+    // 'outLen' is the number of characters still available in the output buffer as input of decoder->Convert
+    // and the number of characters written in it as output.
+    PRInt32 totalChars = 0,
+            inBufferIndex = 0,
+            outBufferIndex = 0;
+    PRInt32 dataLen = srcLen,
+            outLen = unicharLength;
+
+    do {
+      PRInt32 inBufferLength = dataLen;
+      rv = decoder->Convert(&stringToUse[inBufferIndex],
+                           &inBufferLength,
+                           &unichars[outBufferIndex],
+                           &outLen);
+      totalChars += outLen;
+      // Done if conversion successful
+      if (NS_SUCCEEDED(rv))
+          break;
+
+      // We consume one byte, replace it with U+FFFD
+      // and try the conversion again.
+      outBufferIndex += outLen;
+      unichars[outBufferIndex++] = PRUnichar(0xFFFD);
+      // totalChars is updated here
+      outLen = unicharLength - (++totalChars);
+
+      inBufferIndex += inBufferLength + 1;
+      dataLen -= inBufferLength + 1;
+
+      decoder->Reset();
+
+      // If there is not at least one byte available after the one we
+      // consumed, we're done
+    } while ( dataLen > 0 );
+
+    rv = encoder->GetMaxLength(unichars, totalChars, &dstLength);
+    // allocale an output buffer
+    dstPtr = (char *) PR_Malloc(dstLength + 1);
+    if (dstPtr == nsnull) {
+      rv = NS_ERROR_OUT_OF_MEMORY;
+    }
+    else {
+      PRInt32 buffLength = dstLength;
+      // convert from unicode
+      rv = encoder->SetOutputErrorBehavior(nsIUnicodeEncoder::kOnError_Replace, nsnull, '?');
+      if (NS_SUCCEEDED(rv)) {
+        rv = encoder->Convert(unichars, &totalChars, dstPtr, &dstLength);
         if (NS_SUCCEEDED(rv)) {
-          rv = encoder->Convert(unichars, &unicharLength, dstPtr, &dstLength);
+          PRInt32 finLen = buffLength - dstLength;
+          rv = encoder->Finish((char *)(dstPtr+dstLength), &finLen);
           if (NS_SUCCEEDED(rv)) {
-            PRInt32 finLen = buffLength - dstLength;
-            rv = encoder->Finish((char *)(dstPtr+dstLength), &finLen);
-            if (NS_SUCCEEDED(rv)) {
-              dstLength += finLen;
-            }
-            dstPtr[dstLength] = '\0';
-            *pConvertedString = dstPtr;       // set the result string
-            *outLength = dstLength;
+            dstLength += finLen;
           }
+          dstPtr[dstLength] = '\0';
+          *pConvertedString = dstPtr;       // set the result string
+          *outLength = dstLength;
         }
       }
     }
@@ -815,7 +863,7 @@ mime_convert_charset (const char *input_line, PRInt32 input_length,
 }
 
 static int
-mime_output_fn(char *buf, PRInt32 size, void *stream_closure)
+mime_output_fn(const char *buf, PRInt32 size, void *stream_closure)
 {
   PRUint32  written = 0;
   struct mime_stream_data *msd = (struct mime_stream_data *) stream_closure;
@@ -828,7 +876,7 @@ mime_output_fn(char *buf, PRInt32 size, void *stream_closure)
   
   // Now, write to the WriteBody method if this is a message body and not
   // a part retrevial
-  if (!msd->options->part_to_load)
+  if (!msd->options->part_to_load || msd->options->format_out == nsMimeOutput::nsMimeMessageBodyDisplay)
   {
     if (msd->output_emitter)
     {
@@ -847,7 +895,7 @@ mime_output_fn(char *buf, PRInt32 size, void *stream_closure)
 
 #ifdef XP_MAC
 static int
-compose_only_output_fn(char *buf, PRInt32 size, void *stream_closure)
+compose_only_output_fn(const char *buf, PRInt32 size, void *stream_closure)
 {
     return 0;
 }
@@ -917,7 +965,7 @@ mime_display_stream_complete (nsMIMESession *stream)
     // Ok, now we are going to process the attachment data by getting all
     // of the attachment info and then driving the emitter with this data.
     //
-    if (!msd->options->part_to_load)
+    if (!msd->options->part_to_load || msd->options->format_out == nsMimeOutput::nsMimeMessageBodyDisplay)
     {
       nsMsgAttachmentData *attachments;
       nsresult rv = MimeGetAttachmentList(obj, msd->url_name, &attachments);
@@ -1041,7 +1089,7 @@ static void   *mime_image_begin(const char *image_url, const char *content_type,
                               void *stream_closure);
 static void   mime_image_end(void *image_closure, int status);
 static char   *mime_image_make_image_html(void *image_data);
-static int    mime_image_write_buffer(char *buf, PRInt32 size, void *image_closure);
+static int    mime_image_write_buffer(const char *buf, PRInt32 size, void *image_closure);
 
 /* Interface between libmime and inline display of images: the abomination
    that is known as "internal-external-reconnect".
@@ -1116,15 +1164,12 @@ mime_image_begin(const char *image_url, const char *content_type,
             {
               mailUrl->CacheCacheEntry(entry);
               entry->MarkValid();
-              nsCOMPtr<nsITransport> transport;
-              rv = entry->GetTransport(getter_AddRefs(transport));
-              if (NS_FAILED(rv)) return nsnull;
 
               // remember the content type as meta data so we can pull it out in the imap code
               // to feed the cache entry directly to imglib w/o going through mime.
               entry->SetMetaDataElement("contentType", content_type);
-              nsCOMPtr<nsIOutputStream> out;
-              rv = transport->OpenOutputStream(0, PRUint32(-1), 0, getter_AddRefs(mid->memCacheOutputStream));
+
+              rv = entry->OpenOutputStream(0, getter_AddRefs(mid->memCacheOutputStream));
               if (NS_FAILED(rv)) return nsnull;
             }
           }
@@ -1169,7 +1214,7 @@ mime_image_make_image_html(void *image_closure)
 
   /* Internal-external-reconnect only works when going to the screen. */
   if (!mid->istream)
-    return nsCRT::strdup("<P><CENTER><IMG SRC=\"resource:/res/network/gopher-image.gif\" ALT=\"[Image]\"></CENTER><P>");
+    return nsCRT::strdup("<P><CENTER><IMG SRC=\"resource://gre/res/network/gopher-image.gif\" ALT=\"[Image]\"></CENTER><P>");
 
   if ( (!mid->url) || (!(*mid->url)) )
     url = "";
@@ -1189,7 +1234,7 @@ mime_image_make_image_html(void *image_closure)
 }
 
 static int
-mime_image_write_buffer(char *buf, PRInt32 size, void *image_closure)
+mime_image_write_buffer(const char *buf, PRInt32 size, void *image_closure)
 {
   mime_image_stream_data *mid =
                 (mime_image_stream_data *) image_closure;
@@ -1212,27 +1257,6 @@ mime_image_write_buffer(char *buf, PRInt32 size, void *image_closure)
   return size;
 }
 
-// 
-// Utility for finding HTML part.
-//
-static MimeObject*
-mime_find_text_html_part_1(MimeObject* obj)
-{
-  if (mime_subclass_p(obj->clazz,
-                      (MimeObjectClass*) &mimeInlineTextHTMLClass)) {
-    return obj;
-  }
-  if (mime_subclass_p(obj->clazz, (MimeObjectClass*) &mimeContainerClass)) {
-    MimeContainer* cobj = (MimeContainer*) obj;
-    PRInt32 i;
-    for (i=0 ; i<cobj->nchildren ; i++) {
-      MimeObject* result = mime_find_text_html_part_1(cobj->children[i]);
-      if (result) return result;
-    }
-  }
-  return NULL;
-}
-
 MimeObject*
 mime_get_main_object(MimeObject* obj)
 {
@@ -1255,14 +1279,24 @@ mime_get_main_object(MimeObject* obj)
     }
     else
     {
-      // We don't care about a signed/smime object; Go inside to the 
-      // thing that we signed or smime'ed
-      //
-      cobj = (MimeContainer*) obj;
-      if (cobj->nchildren > 0)
-        obj = cobj->children[0];
+      if (mime_subclass_p(obj->clazz, (MimeObjectClass*)&mimeContainerClass))
+      {
+        // We don't care about a signed/smime object; Go inside to the 
+        // thing that we signed or smime'ed
+        //
+        cobj = (MimeContainer*) obj;
+        if (cobj->nchildren > 0)
+          obj = cobj->children[0];
+        else
+          obj = nsnull;
+      }
       else
-        obj = nsnull;
+      {
+        // we received a message with a child object that looks like a signed
+        // object, but it is not a subclass of mimeContainer, so let's
+        // return the given child object.
+        return obj;
+      }
     }
   }
   return nsnull;
@@ -1429,7 +1463,7 @@ MimeDisplayOptions::MimeDisplayOptions()
   attachment_icon_layer_id = 0;
 
   missing_parts = PR_FALSE;
-
+  show_attachment_inline_p = PR_FALSE;
 }
 
 MimeDisplayOptions::~MimeDisplayOptions()
@@ -1522,6 +1556,7 @@ mime_bridge_create_display_stream(
   // Set the defaults, based on the context, and the output-type.
   //
   MIME_HeaderType = MimeHeadersAll;
+  msd->options->write_html_p = PR_TRUE;
   switch (format_out) 
   {
 		case nsMimeOutput::nsMimeMessageSplitDisplay:   // the wrapper HTML output to produce the split header/body display
@@ -1546,6 +1581,12 @@ mime_bridge_create_display_stream(
     case nsMimeOutput::nsMimeMessageRaw:              // the raw RFC822 data (view source) and attachments
 		case nsMimeOutput::nsMimeMessageDraftOrTemplate:  // Loading drafts & templates
 		case nsMimeOutput::nsMimeMessageEditorTemplate:   // Loading templates into editor
+    case nsMimeOutput::nsMimeMessageFilterSniffer:    // generating an output that can be scan by a message filter
+      break;
+
+    case nsMimeOutput::nsMimeMessageDecrypt:
+      msd->options->decrypt_p = PR_TRUE;
+      msd->options->write_html_p = PR_FALSE;
       break;
   }
 
@@ -1579,12 +1620,11 @@ mime_bridge_create_display_stream(
   }
  
   if (msd->options->headers == MimeHeadersMicro &&
-     (msd->url_name == NULL || (nsCRT::strncmp(msd->url_name, "news:", 5) != 0 &&
-              nsCRT::strncmp(msd->url_name, "snews:", 6) != 0)) )
+     (msd->url_name == NULL || (strncmp(msd->url_name, "news:", 5) != 0 &&
+              strncmp(msd->url_name, "snews:", 6) != 0)) )
     msd->options->headers = MimeHeadersMicroPlus;
 
   msd->options->url = msd->url_name;
-  msd->options->write_html_p          = PR_TRUE;
   msd->options->output_init_fn        = mime_output_init_fn;
   
   msd->options->output_fn             = mime_output_fn;
@@ -1618,9 +1658,12 @@ mime_bridge_create_display_stream(
 
   // If this is a part, then we should emit the HTML to render the data
   // (i.e. embedded images)
-  if (msd->options->part_to_load)
+  if (msd->options->part_to_load && msd->options->format_out != nsMimeOutput::nsMimeMessageBodyDisplay)
     msd->options->write_html_p = PR_FALSE;
 
+  if (msd->options->prefs)
+    msd->options->prefs->GetBoolPref("mail.inline_attachments", &(msd->options->show_attachment_inline_p));
+
   obj = mime_new ((MimeObjectClass *)&mimeMessageClass, (MimeHeaders *) NULL, MESSAGE_RFC822);
   if (!obj)
   {
@@ -1947,9 +1990,6 @@ mimeSetNewURL(nsMIMESession *stream, char *url)
   return NS_OK;
 }
 
-/* This is the next generation string retrieval call */
-static NS_DEFINE_CID(kStringBundleServiceCID, NS_STRINGBUNDLESERVICE_CID);
-
 #define     MIME_URL "chrome://messenger/locale/mime.properties"
 
 extern "C" 
@@ -1967,7 +2007,7 @@ MimeGetStringByID(PRInt32 stringID)
     propertyURL = MIME_URL;
 
     nsCOMPtr<nsIStringBundleService> sBundleService = 
-             do_GetService(kStringBundleServiceCID, &res); 
+             do_GetService(NS_STRINGBUNDLE_CONTRACTID, &res); 
     if (NS_SUCCEEDED(res) && (nsnull != sBundleService)) 
     {
       res = sBundleService->CreateBundle(propertyURL, getter_AddRefs(stringBundle));
@@ -2062,10 +2102,8 @@ nsresult GetMailNewsFont(MimeObject *obj, PRBool styleFixed,  PRInt32 *fontPixel
     else
       charset.Assign(text->charset);
 
-    nsCOMPtr<nsICharsetConverterManager2> charSetConverterManager2;
-    nsCOMPtr <nsIAtom> charsetAtom;
+    nsCOMPtr<nsICharsetConverterManager> charSetConverterManager2;
     nsCOMPtr<nsIAtom> langGroupAtom;
-    const PRUnichar* langGroup = nsnull;
     nsCAutoString prefStr;
 
     ToLowerCase(charset);
@@ -2075,21 +2113,16 @@ nsresult GetMailNewsFont(MimeObject *obj, PRBool styleFixed,  PRInt32 *fontPixel
       return rv;
 
     // get a language, e.g. x-western, ja
-    rv = charSetConverterManager2->GetCharsetAtom2(charset.get(),getter_AddRefs(charsetAtom));
+    rv = charSetConverterManager2->GetCharsetLangGroup(charset.get(), getter_AddRefs(langGroupAtom));
     if (NS_FAILED(rv))
       return rv;
-    rv = charSetConverterManager2->GetCharsetLangGroup(charsetAtom,getter_AddRefs(langGroupAtom));
+    rv = langGroupAtom->ToUTF8String(fontLang);
     if (NS_FAILED(rv))
       return rv;
-    rv = langGroupAtom->GetUnicode(&langGroup);
-    if (NS_FAILED(rv))
-      return rv;
-
-    fontLang.AssignWithConversion(langGroup);
 
     // get a font size from pref
     prefStr.Assign(!styleFixed ? "font.size.variable." : "font.size.fixed.");
-    prefStr.AppendWithConversion(langGroup);
+    prefStr.Append(fontLang);
     rv = prefs->GetIntPref(prefStr.get(), fontPixelSize);
     if (NS_FAILED(rv))
       return rv;
@@ -2108,3 +2141,131 @@ nsresult GetMailNewsFont(MimeObject *obj, PRBool styleFixed,  PRInt32 *fontPixel
 
   return NS_OK;
 }
+
+/* This function syncronously converts an HTML document (as string)
+   to plaintext (as string) using the Gecko converter.
+
+   flags: see nsIDocumentEncoder.h
+*/
+// TODO: |printf|s?
+/* <copy from="mozilla/htmlparser/test/outsinks/Convert.cpp"
+         author="akk"
+         adapted-by="Ben Bucksch"
+         comment=" 'This code would not have been possible without akk.' ;-P.
+                   No, really. "
+   > */
+nsresult
+HTML2Plaintext(const nsString& inString, nsString& outString,
+               PRUint32 flags, PRUint32 wrapCol)
+{
+  nsresult rv = NS_OK;
+
+#if DEBUG_BenB
+  printf("Converting HTML to plaintext\n");
+  char* charstar = ToNewUTF8String(inString);
+  printf("HTML source is:\n--------------------\n%s--------------------\n",
+         charstar);
+  delete[] charstar;
+#endif
+
+  // Create a parser
+  nsCOMPtr<nsIParser> parser = do_CreateInstance(kParserCID);
+  NS_ENSURE_TRUE(parser, NS_ERROR_FAILURE);
+
+  // Create the appropriate output sink
+  nsCOMPtr<nsIContentSink> sink =
+                               do_CreateInstance(NS_PLAINTEXTSINK_CONTRACTID);
+  NS_ENSURE_TRUE(sink, NS_ERROR_FAILURE);
+
+  nsCOMPtr<nsIHTMLToTextSink> textSink(do_QueryInterface(sink));
+  NS_ENSURE_TRUE(textSink, NS_ERROR_FAILURE);
+
+  textSink->Initialize(&outString, flags, wrapCol);
+
+  parser->SetContentSink(sink);
+  nsCOMPtr<nsIDTD> dtd = do_CreateInstance(kNavDTDCID);
+  NS_ENSURE_TRUE(dtd, NS_ERROR_FAILURE);
+
+  parser->RegisterDTD(dtd);
+
+  rv = parser->Parse(inString, 0, NS_LITERAL_CSTRING("text/html"),
+                     PR_FALSE, PR_TRUE);
+
+  // Aah! How can NS_ERROR and NS_ABORT_IF_FALSE be no-ops in release builds???
+  if (NS_FAILED(rv))
+  {
+    NS_ERROR("Parse() failed!");
+    return rv;
+  }
+
+#if DEBUG_BenB
+  charstar = ToNewUTF8String(outString);
+  printf("Plaintext is:\n--------------------\n%s--------------------\n",
+         charstar);
+  delete[] charstar;
+#endif
+
+  return rv;
+}
+// </copy>
+
+
+
+/* This function syncronously sanitizes an HTML document (string->string)
+   using the Gecko ContentSink mozISanitizingHTMLSerializer.
+
+   flags: currently unused
+   allowedTags: see mozSanitizingHTMLSerializer::ParsePrefs()
+*/
+// copied from HTML2Plaintext above
+nsresult
+HTMLSanitize(const nsString& inString, nsString& outString,
+             PRUint32 flags, const nsAString& allowedTags)
+{
+  nsresult rv = NS_OK;
+
+#if DEBUG_BenB
+  printf("Sanitizing HTML\n");
+  char* charstar = ToNewUTF8String(inString);
+  printf("Original HTML is:\n--------------------\n%s--------------------\n",
+         charstar);
+  delete[] charstar;
+#endif
+
+  // Create a parser
+  nsCOMPtr<nsIParser> parser = do_CreateInstance(kParserCID);
+  NS_ENSURE_TRUE(parser, NS_ERROR_FAILURE);
+
+  // Create the appropriate output sink
+  nsCOMPtr<nsIContentSink> sink =
+                    do_CreateInstance(MOZ_SANITIZINGHTMLSERIALIZER_CONTRACTID);
+  NS_ENSURE_TRUE(sink, NS_ERROR_FAILURE);
+
+  nsCOMPtr<mozISanitizingHTMLSerializer> sanSink(do_QueryInterface(sink));
+  NS_ENSURE_TRUE(sanSink, NS_ERROR_FAILURE);
+
+  sanSink->Initialize(&outString, flags, allowedTags);
+
+  parser->SetContentSink(sink);
+  nsCOMPtr<nsIDTD> dtd = do_CreateInstance(kNavDTDCID);
+  NS_ENSURE_TRUE(dtd, NS_ERROR_FAILURE);
+
+  parser->RegisterDTD(dtd);
+
+  rv = parser->Parse(inString, 0, NS_LITERAL_CSTRING("text/html"),
+                     PR_FALSE, PR_TRUE);
+  if (NS_FAILED(rv))
+  {
+    NS_ERROR("Parse() failed!");
+    return rv;
+  }
+
+#if DEBUG_BenB
+  charstar = ToNewUTF8String(outString);
+  printf("Sanitized HTML is:\n--------------------\n%s--------------------\n",
+         charstar);
+  delete[] charstar;
+#endif
+
+  return rv;
+}
