@@ -155,21 +155,6 @@ public:
   char        *m_url;
 };
 
-
-/* Stupid utility function.  Really ought to be part of the standard string
-   package if you ask me...*/
-
-static char* mime_strnchr(char* str, char c, int length)
-{
-  int i;
-  for (i=0 ; i<length ; i++) {
-    if (*str == c) return str;
-    str++;
-  }
-  return NULL;
-}
-
-
 static int
 MimeMultipartRelated_initialize(MimeObject* obj)
 {
@@ -454,14 +439,6 @@ MimeMultipartRelated_output_child_p(MimeObject *obj, MimeObject* child)
     /* This is a child part.  Just remember the mapping between the URL
        it represents and the part-URL to get it back. */
 
-    /* but first before acceptig it as a valid related part, make sure we
-       are able to display it inline as an embedded object. Else just ignore
-       it, that will prevent any bad surprise. */
-    MimeObjectClass *clazz = mime_find_class (child->content_type, child->headers, child->options, PR_FALSE);
-    PRBool canDisplay = (clazz
-          ? clazz->displayable_inline_p(clazz, child->headers)
-          : PR_FALSE);
-    if (canDisplay) {
     char* location = MimeHeaders_get(child->headers, HEADER_CONTENT_LOCATION,
                      PR_FALSE, PR_FALSE);
     if (!location) {
@@ -486,21 +463,22 @@ MimeMultipartRelated_output_child_p(MimeObject *obj, MimeObject* child)
       char *absolute;
       char *base_url = MimeHeaders_get(child->headers, HEADER_CONTENT_BASE,
                        PR_FALSE, PR_FALSE);
-      /* rhp: need this for supporting Content-Location */
-      if (!base_url)
-      {
-        base_url = MimeHeaders_get(child->headers, HEADER_CONTENT_LOCATION, PR_FALSE, PR_FALSE);
-      }
-      /* rhp: need this for supporting Content-Location */
-
       absolute = MakeAbsoluteURL(base_url ? base_url : relobj->base_url, location);
 
       PR_FREEIF(base_url);
       PR_Free(location);
       if (absolute) {
         nsCAutoString partnum;
+        nsCAutoString imappartnum;
         partnum.Adopt(mime_part_address(child));
         if (!partnum.IsEmpty()) {
+          if (obj->options->missing_parts)
+          {
+            char * imappart = mime_imap_part_address(child);
+            if (imappart)
+              imappartnum.Adopt(imappart);
+          }
+
           /*
             AppleDouble part need special care: we need to output only the data fork part of it.
             The problem at this point is that we haven't yet decoded the children of the AppleDouble
@@ -510,9 +488,23 @@ MimeMultipartRelated_output_child_p(MimeObject *obj, MimeObject* child)
             partnum.Append(".2");
 
           char* part;
-          part = mime_set_url_part(obj->options->url, partnum.get(),
-                       PR_FALSE);
-          if (part) {
+          if (!imappartnum.IsEmpty()) 
+            part = mime_set_url_imap_part(obj->options->url, imappartnum.get(), partnum.get());
+          else
+          {
+            char *no_part_url = nsnull;
+            if (obj->options->part_to_load && obj->options->format_out == nsMimeOutput::nsMimeMessageBodyDisplay)
+              no_part_url = mime_get_base_url(obj->options->url);
+            if (no_part_url)
+            {
+              part = mime_set_url_part(no_part_url, partnum.get(), PR_FALSE);
+              PR_Free(no_part_url);
+            }
+            else
+              part = mime_set_url_part(obj->options->url, partnum.get(), PR_FALSE);
+          }
+          if (part)
+          {
             char *name = MimeHeaders_get_name(child->headers, child->options);
             // let's stick the filename in the part so save as will work.
             if (name)
@@ -570,7 +562,6 @@ MimeMultipartRelated_output_child_p(MimeObject *obj, MimeObject* child)
           }
         }
       }
-    }
   } else {
     /* Ah-hah!  We're the head object.  */
     char* base_url;
@@ -729,7 +720,7 @@ MimeMultipartRelated_parse_child_line (MimeObject *obj,
 
 
 static int
-real_write(MimeMultipartRelated* relobj, char* buf, PRInt32 size)
+real_write(MimeMultipartRelated* relobj, const char* buf, PRInt32 size)
 {
   MimeObject* obj = (MimeObject*) relobj;
   void* closure = relobj->real_output_closure;
@@ -773,6 +764,23 @@ push_tag(MimeMultipartRelated* relobj, const char* buf, PRInt32 size)
   return 0;
 }
 
+static PRBool accept_related_part(MimeMultipartRelated* relobj, MimeObject* part_obj)
+{
+  if (!relobj || !part_obj)
+    return PR_FALSE;
+    
+  /* before accepting it as a valid related part, make sure we
+     are able to display it inline as an embedded object. Else just ignore
+     it, that will prevent any bad surprise... */
+  MimeObjectClass *clazz = mime_find_class (part_obj->content_type, part_obj->headers, part_obj->options, PR_FALSE);
+  if (clazz ? clazz->displayable_inline_p(clazz, part_obj->headers) : PR_FALSE)
+    return PR_TRUE;
+
+  /* ...but we always accept it if it's referenced by an anchor */
+  return (relobj->curtag && relobj->curtag_length >= 3 &&
+    (relobj->curtag[1] == 'A' || relobj->curtag[1] == 'a') && nsCRT::IsAsciiSpace(relobj->curtag[2]));
+}
+
 static int
 flush_tag(MimeMultipartRelated* relobj)
 {
@@ -809,7 +817,7 @@ flush_tag(MimeMultipartRelated* relobj)
     if (!*buf) break;
     if (isquote) 
     {
-      ptr = mime_strnchr(buf, '"', length - (buf - relobj->curtag));
+      ptr = PL_strnchr(buf, '"', length - (buf - relobj->curtag));
     } else {
       for (ptr = buf; *ptr ; ptr++) {
         if (*ptr == '>' || nsCRT::IsAsciiSpace(*ptr)) break;
@@ -829,8 +837,14 @@ flush_tag(MimeMultipartRelated* relobj)
         ptr2++;
       /* Compare the beginning of the word with "cid:". Yuck. */
       if (((ptr2 - buf) > 4) && 
-        (buf[0]=='c' && buf[1]=='i' && buf[2]=='d' && buf[3]==':'))
+        ((buf[0]=='c' || buf[0]=='C') && 
+         (buf[1]=='i' || buf[1]=='I') && 
+         (buf[2]=='d' || buf[2]=='D') && 
+          buf[3]==':'))
       {
+        // Make sure it's lowercase, otherwise it won't be found in the hash table
+        buf[0] = 'c'; buf[1] = 'i'; buf[2] = 'd';
+
         /* Null terminate the word so we can... */
         c = *ptr2;
         *ptr2 = '\0';
@@ -850,7 +864,7 @@ flush_tag(MimeMultipartRelated* relobj)
         }
         
         /*If we found a mailbox part URL, write that out instead.*/
-        if (part_url)
+        if (part_url && accept_related_part(relobj, value->m_obj))
         {
           status = real_write(relobj, part_url, strlen(part_url));
           if (status < 0) return status;
@@ -887,7 +901,7 @@ flush_tag(MimeMultipartRelated* relobj)
         *ptr2 = holder;
         PR_FREEIF(absolute);
 
-        if (realout)
+        if (realout && accept_related_part(relobj, value->m_obj))
         {
           status = real_write(relobj, realout, strlen(realout));
           if (status < 0) return status;
@@ -923,7 +937,7 @@ flush_tag(MimeMultipartRelated* relobj)
 
 
 static int
-mime_multipart_related_output_fn(char* buf, PRInt32 size, void *stream_closure)
+mime_multipart_related_output_fn(const char* buf, PRInt32 size, void *stream_closure)
 {
   MimeMultipartRelated *relobj = (MimeMultipartRelated *) stream_closure;
   char* ptr;
@@ -931,7 +945,7 @@ mime_multipart_related_output_fn(char* buf, PRInt32 size, void *stream_closure)
   int status;
   while (size > 0) {
     if (relobj->curtag_length > 0) {
-      ptr = mime_strnchr(buf, '>', size);
+      ptr = PL_strnchr(buf, '>', size);
       if (!ptr) {
         return push_tag(relobj, buf, size);
       }
@@ -943,7 +957,7 @@ mime_multipart_related_output_fn(char* buf, PRInt32 size, void *stream_closure)
       buf += delta;
       size -= delta;
     }
-    ptr = mime_strnchr(buf, '<', size);
+    ptr = PL_strnchr(buf, '<', size);
     if (ptr && ptr - buf >= size) ptr = 0;
     if (!ptr) {
       return real_write(relobj, buf, size);
