@@ -39,13 +39,15 @@
 #include "nsFTPChannel.h"
 #include "nsIStreamListener.h"
 #include "nsIServiceManager.h"
-#include "nsCExternalHandlerService.h"
 #include "nsIMIMEService.h"
 #include "nsNetUtil.h"
 #include "nsMimeTypes.h"
 #include "nsIProxyObjectManager.h"
 #include "nsReadableUtils.h"
-#include "nsIPref.h"
+#include "nsIPrefService.h"
+#include "nsIPrefBranch.h"
+#include "nsIStreamConverterService.h"
+#include "nsISocketTransport.h"
 
 #if defined(PR_LOGGING)
 extern PRLogModuleInfo* gFTPLog;
@@ -87,7 +89,6 @@ nsFTPChannel::nsFTPChannel()
       mStatus(NS_OK),
       mCanceled(PR_FALSE)
 {
-    NS_INIT_REFCNT();
 }
 
 nsFTPChannel::~nsFTPChannel()
@@ -116,6 +117,15 @@ NS_INTERFACE_MAP_BEGIN(nsFTPChannel)
     NS_INTERFACE_MAP_ENTRY(nsIRequestObserver)
     NS_INTERFACE_MAP_ENTRY(nsICacheListener)
     NS_INTERFACE_MAP_ENTRY(nsIDirectoryListing)
+    if (aIID.Equals(NS_GET_IID(nsIProperties))) {
+        if (!mProperties) {
+            mProperties =
+                do_CreateInstance(NS_PROPERTIES_CONTRACTID, (nsIChannel *) this);
+            NS_ENSURE_STATE(mProperties);
+        }
+        return mProperties->QueryInterface(aIID, aInstancePtr);
+    }
+    else
     NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsISupports, nsIChannel)
 NS_INTERFACE_MAP_END
 
@@ -145,18 +155,6 @@ nsFTPChannel::Init(nsIURI* uri, nsIProxyInfo* proxyInfo, nsICacheSession* sessio
 }
 
 
-NS_METHOD
-nsFTPChannel::Create(nsISupports* aOuter, const nsIID& aIID, void* *aResult)
-{
-    nsFTPChannel* fc;
-    NS_NEWXPCOM(fc, nsFTPChannel);
-    if (!fc) return NS_ERROR_OUT_OF_MEMORY;
-    NS_ADDREF(fc);
-    nsresult rv = fc->QueryInterface(aIID, aResult);
-    NS_RELEASE(fc);
-    return rv;
-}
-
 ////////////////////////////////////////////////////////////////////////////////
 // nsIRequest methods:
 
@@ -167,14 +165,9 @@ nsFTPChannel::Create(nsISupports* aOuter, const nsIID& aIID, void* *aResult)
 // cross thread call.
 
 NS_IMETHODIMP
-nsFTPChannel::GetName(PRUnichar* *result)
+nsFTPChannel::GetName(nsACString &result)
 {
-    nsresult rv;
-    nsCAutoString urlStr;
-    rv = mURL->GetSpec(urlStr);
-    if (NS_FAILED(rv)) return rv;
-    *result = ToNewUnicode(NS_ConvertUTF8toUCS2(urlStr));
-    return *result ? NS_OK : NS_ERROR_OUT_OF_MEMORY;
+    return mURL->GetSpec(result);
 }
 
 NS_IMETHODIMP
@@ -269,8 +262,7 @@ nsFTPChannel::GetURI(nsIURI* *aURL)
 NS_IMETHODIMP
 nsFTPChannel::Open(nsIInputStream **result)
 {
-    NS_NOTREACHED("nsFTPChannel::Open");
-    return NS_ERROR_NOT_IMPLEMENTED;
+    return NS_ImplementChannelOpen(this, result);
 }
 
 nsresult
@@ -411,40 +403,39 @@ nsFTPChannel::SetLoadFlags(PRUint32 aLoadFlags)
 // extension mapping.
 
 NS_IMETHODIMP
-nsFTPChannel::GetContentType(char* *aContentType) {
-    nsresult rv = NS_OK;
-
-    if (!aContentType) return NS_ERROR_NULL_POINTER;
-
+nsFTPChannel::GetContentType(nsACString &aContentType)
+{
     nsAutoLock lock(mLock);
-    *aContentType = nsnull;
+
     if (mContentType.IsEmpty()) {
-
-        nsCOMPtr<nsIMIMEService> MIMEService (do_GetService(NS_MIMESERVICE_CONTRACTID, &rv));
-        if (NS_FAILED(rv)) return rv;
-        rv = MIMEService->GetTypeFromURI(mURL, aContentType);
-        if (NS_SUCCEEDED(rv)) {
-            mContentType = *aContentType;
-        } else {
-            mContentType = UNKNOWN_CONTENT_TYPE;
-            rv = NS_OK;
-        }
+        aContentType = NS_LITERAL_CSTRING(UNKNOWN_CONTENT_TYPE);
+    } else {
+        aContentType = mContentType;
     }
 
-    if (!*aContentType) {
-        *aContentType = ToNewCString(mContentType);
-    }
-
-    if (!*aContentType) return NS_ERROR_OUT_OF_MEMORY;
-    PR_LOG(gFTPLog, PR_LOG_DEBUG, ("nsFTPChannel::GetContentType() returned %s\n", *aContentType));
-    return rv;
+    PR_LOG(gFTPLog, PR_LOG_DEBUG, ("nsFTPChannel::GetContentType() returned %s\n", PromiseFlatCString(aContentType).get()));
+    return NS_OK;
 }
 
 NS_IMETHODIMP
-nsFTPChannel::SetContentType(const char *aContentType)
+nsFTPChannel::SetContentType(const nsACString &aContentType)
 {
     nsAutoLock lock(mLock);
-    mContentType = aContentType;
+    NS_ParseContentType(aContentType, mContentType, mContentCharset);
+    return NS_OK;
+}
+
+NS_IMETHODIMP
+nsFTPChannel::GetContentCharset(nsACString &aContentCharset)
+{
+    aContentCharset = mContentCharset;
+    return NS_OK;
+}
+
+NS_IMETHODIMP
+nsFTPChannel::SetContentCharset(const nsACString &aContentCharset)
+{
+    mContentCharset = aContentCharset;
     return NS_OK;
 }
 
@@ -593,11 +584,13 @@ nsFTPChannel::OnStatus(nsIRequest *request, nsISupports *aContext,
         // The state machine needs to know that the data connection
         // was successfully started so that it can issue data commands
         // securely.
-        NS_ASSERTION(mFTPState, "ftp state is null.");
-        (void) mFTPState->DataConnectionEstablished();
+        if (mFTPState)
+            mFTPState->DataConnectionEstablished();
+        else
+            NS_ERROR("ftp state is null.");
     }
 
-    if (!mEventSink)
+    if (!mEventSink || (mLoadFlags & LOAD_BACKGROUND) || !mIsPending || NS_FAILED(mStatus))
         return NS_OK;
 
     return mEventSink->OnStatus(this, mUserContext, aStatus,
@@ -606,12 +599,13 @@ nsFTPChannel::OnStatus(nsIRequest *request, nsISupports *aContext,
 
 NS_IMETHODIMP
 nsFTPChannel::OnProgress(nsIRequest *request, nsISupports* aContext,
-                                  PRUint32 aProgress, PRUint32 aProgressMax) {
-    if (!mEventSink)
+                         PRUint32 aProgress, PRUint32 aProgressMax)
+{
+    if (!mEventSink || (mLoadFlags & LOAD_BACKGROUND) || !mIsPending)
         return NS_OK;
 
     return mEventSink->OnProgress(this, mUserContext, 
-                                  aProgress, (PRUint32) mContentLength);
+                                  aProgress, aProgressMax);
 }
 
 
@@ -627,17 +621,18 @@ nsFTPChannel::OnStopRequest(nsIRequest *request, nsISupports* aContext,
 
     nsresult rv = NS_OK;
     
-    mStatus = aStatus;
-    
+    if (NS_SUCCEEDED(mStatus))
+        mStatus = aStatus;
+
     if (mListener) {
-        (void) mListener->OnStopRequest(this, mUserContext, aStatus);
+        (void) mListener->OnStopRequest(this, mUserContext, mStatus);
     }
     if (mLoadGroup) {
-        (void) mLoadGroup->RemoveRequest(this, nsnull, aStatus);
+        (void) mLoadGroup->RemoveRequest(this, nsnull, mStatus);
     }
     
     if (mCacheEntry) {
-        if (NS_SUCCEEDED(aStatus)) {
+        if (NS_SUCCEEDED(mStatus)) {
             (void) mCacheEntry->SetExpirationTime( NowInSeconds() + 900 ); // valid for 15 minutes.
             (void) mCacheEntry->MarkValid();
 	}
@@ -651,6 +646,10 @@ nsFTPChannel::OnStopRequest(nsIRequest *request, nsISupports* aContext,
     if (mUploadStream)
         mUploadStream->Close();
 
+    if (mFTPState) {
+        mFTPState->DataConnectionComplete();
+        NS_RELEASE(mFTPState);
+    }
     mIsPending = PR_FALSE;
     return rv;
 }
@@ -663,13 +662,33 @@ nsFTPChannel::OnStartRequest(nsIRequest *request, nsISupports *aContext)
            ("nsFTPChannel::OnStartRequest() called [this=%x, request=%x]\n", 
             this, request));
    
-    nsresult rv = NS_OK;
-    request->GetStatus(&mStatus);
+    if (NS_SUCCEEDED(mStatus))
+        request->GetStatus(&mStatus);
+    
     nsCOMPtr<nsIResumableChannel> resumable = do_QueryInterface(request);
     if (resumable)
         resumable->GetEntityID(getter_AddRefs(mEntityID));
     
+    nsresult rv = NS_OK;
     if (mListener) {
+        if (mContentType.IsEmpty()) {
+            // Time to sniff!
+            nsCOMPtr<nsIStreamConverterService> serv =
+                do_GetService("@mozilla.org/streamConverters;1", &rv);
+            if (NS_SUCCEEDED(rv)) {
+                NS_ConvertASCIItoUCS2 from(UNKNOWN_CONTENT_TYPE);
+                nsCOMPtr<nsIStreamListener> converter;
+                rv = serv->AsyncConvertData(from.get(),
+                                            NS_LITERAL_STRING("*/*").get(),
+                                            mListener,
+                                            mUserContext,
+                                            getter_AddRefs(converter));
+                if (NS_SUCCEEDED(rv)) {
+                    mListener = converter;
+                }
+            }
+        }
+        
         rv = mListener->OnStartRequest(this, mUserContext);
         if (NS_FAILED(rv)) return rv;
     }
@@ -716,61 +735,47 @@ nsFTPChannel::OnCacheEntryAvailable(nsICacheEntryDescriptor *entry,
 }
 
 NS_IMETHODIMP
-nsFTPChannel::SetUploadStream(nsIInputStream *stream, const char *contentType, PRInt32 contentLength)
+nsFTPChannel::SetUploadStream(nsIInputStream *stream, const nsACString &contentType, PRInt32 contentLength)
 {
     mUploadStream = stream;
     return NS_OK;
 }
 
 NS_IMETHODIMP
-nsFTPChannel::SetUploadFile(nsIFile *file, const char *contentType, PRInt32 contentLength)
+nsFTPChannel::GetUploadStream(nsIInputStream **stream)
 {
-    if (!file) return NS_ERROR_NULL_POINTER;
-
-    nsresult rv;
-    // Grab a file input stream
-    nsCOMPtr<nsIInputStream> stream;
-    rv = NS_NewLocalFileInputStream(getter_AddRefs(stream), file);    
-    if (NS_FAILED(rv))
-        return rv;
-
-    // set the stream on ourselves
-    return SetUploadStream(stream, nsnull, -1); 
+    NS_ENSURE_ARG_POINTER(stream);
+    *stream = mUploadStream;
+    NS_IF_ADDREF(*stream);
+    return NS_OK;
 }
 
 NS_IMETHODIMP
-nsFTPChannel::SetListFormat(PRUint32 format) {
-    if (format != FORMAT_PREF &&
-        format != FORMAT_RAW &&
-        format != FORMAT_HTML &&
-        format != FORMAT_HTTP_INDEX) {
-        return NS_ERROR_FAILURE;
-    }
-
+nsFTPChannel::SetListFormat(PRUint32 format)
+{
     // Convert the pref value
     if (format == FORMAT_PREF) {
-        nsresult rv;
-        nsCOMPtr<nsIPref> prefs = do_GetService(NS_PREF_CONTRACTID, &rv);
-        if (NS_FAILED(rv)) return rv;
-        PRInt32 sFormat;
-        rv = prefs->GetIntPref("network.dir.format", &sFormat);
-        if (NS_FAILED(rv))
-            format = FORMAT_HTML; // default
-        else
-            format = sFormat;
-
-        if (format == FORMAT_PREF) {
-            NS_WARNING("Who set the directory format pref to 'read from prefs'??");
-            return NS_ERROR_FAILURE;
+        format = FORMAT_HTML; // default
+        nsCOMPtr<nsIPrefBranch> prefs = do_GetService(NS_PREFSERVICE_CONTRACTID);
+        if (prefs) {
+            PRInt32 sFormat;
+            if (NS_SUCCEEDED(prefs->GetIntPref("network.dir.format", &sFormat)))
+                format = sFormat;
         }
     }
-
+    if (format != FORMAT_RAW &&
+        format != FORMAT_HTML &&
+        format != FORMAT_HTTP_INDEX) {
+        NS_WARNING("invalid directory format");
+        return NS_ERROR_FAILURE;
+    }
     mListFormat = format;
     return NS_OK;
 }
 
 NS_IMETHODIMP
-nsFTPChannel::GetListFormat(PRUint32 *format) {
+nsFTPChannel::GetListFormat(PRUint32 *format)
+{
     *format = mListFormat;
     return NS_OK;
 }
