@@ -29,18 +29,18 @@
 
 #include "nsIEnumerator.h"
 #include "prprf.h"
-#include "nsSpecialSystemDirectory.h"
 #include "nsCOMPtr.h"
 #include "nsIComponentManager.h"
-#include "nsFileStream.h"
 #include "nsEscape.h"
 #include "nsDirectoryServiceDefs.h"
 #include "nsAppDirectoryServiceDefs.h"
 #include "nsILocalFile.h"
 #include "nsReadableUtils.h"
 
-#ifdef XP_MAC
+#if defined(XP_MAC) || defined(XP_MACOSX)
 #include <Processes.h>
+#include <CFBundle.h>
+#include "nsILocalFileMac.h"
 #endif
 
 #ifdef XP_UNIX
@@ -52,6 +52,10 @@
 #include "prsystem.h"
 #endif
 
+#ifdef VMS
+#include <rmsdef.h>
+#endif
+
 #include "nsICharsetConverterManager.h"
 #include "nsIPlatformCharset.h"
 
@@ -63,11 +67,13 @@
 #define PROFILE_NAME_ENVIRONMENT_VARIABLE "PROFILE_NAME"
 #define PROFILE_HOME_ENVIRONMENT_VARIABLE "PROFILE_HOME"
 #define DEFAULT_UNIX_PROFILE_NAME "default"
+#ifndef XP_MACOSX   /* Don't use symlink-based locking on OS X */
+#define USE_SYMLINK_LOCKING
+#endif
 #elif defined (XP_BEOS)
 #endif
 
 // IID and CIDs of all the services needed
-static NS_DEFINE_CID(kRegistryCID, NS_REGISTRY_CID);
 static NS_DEFINE_CID(kCharsetConverterManagerCID, NS_ICHARSETCONVERTERMANAGER_CID);
 
 // Registry Keys
@@ -92,6 +98,7 @@ static NS_DEFINE_CID(kCharsetConverterManagerCID, NS_ICHARSETCONVERTERMANAGER_CI
 #define kRegistryVersionString (NS_LITERAL_STRING("Version"))
 #define kRegistryVersion_1_0 (NS_LITERAL_STRING("1.0"))
 #define kRegistryCurrentVersion (NS_LITERAL_STRING("1.0"))
+#define kRegistryStartWithLastString (NS_LITERAL_CSTRING("AutoStartWithLast"))
 
 // **********************************************************************
 // class nsProfileAccess
@@ -106,6 +113,7 @@ nsProfileAccess::nsProfileAccess()
     mProfileDataChanged  =  PR_FALSE;
     mForgetProfileCalled =  PR_FALSE;
     m4xProfilesAdded     =  PR_FALSE;
+    mStartWithLastProfile = PR_FALSE;
     mProfiles            =  new nsVoidArray();
 
     // Get the profile registry path
@@ -126,7 +134,7 @@ nsProfileAccess::~nsProfileAccess()
 
 // A wrapper function to call the interface to get a platform file charset.
 nsresult
-GetPlatformCharset(nsString& aCharset)
+GetPlatformCharset(nsCString& aCharset)
 {
     nsresult rv;
 
@@ -136,14 +144,14 @@ GetPlatformCharset(nsString& aCharset)
         rv = platformCharset->GetCharset(kPlatformCharsetSel_FileName, aCharset);
     }
     if (NS_FAILED(rv)) {
-        aCharset.Assign(NS_LITERAL_STRING("ISO-8859-1"));  // use ISO-8859-1 in case of any error
+        aCharset.Assign(NS_LITERAL_CSTRING("ISO-8859-1"));  // use ISO-8859-1 in case of any error
     }
     return rv;
 }
 
 // Apply a charset conversion from the given charset to Unicode for input C string.
 nsresult
-ConvertStringToUnicode(nsString& aCharset, const char* inString, nsAString& outString)
+ConvertStringToUnicode(nsCString& aCharset, const char* inString, nsAString& outString)
 {
     nsresult rv;
     // convert result to unicode
@@ -152,7 +160,7 @@ ConvertStringToUnicode(nsString& aCharset, const char* inString, nsAString& outS
 
     if(NS_SUCCEEDED(rv)) {
         nsCOMPtr <nsIUnicodeDecoder> decoder; // this may be cached
-        rv = ccm->GetUnicodeDecoder(&aCharset, getter_AddRefs(decoder));
+        rv = ccm->GetUnicodeDecoderRaw(aCharset.get(), getter_AddRefs(decoder));
 
         if(NS_SUCCEEDED(rv) && decoder) {
             PRInt32 uniLength = 0;
@@ -273,7 +281,7 @@ nsProfileAccess::SetValue(ProfileStruct* aProfile)
 // profiles array. All the udates will be done to this
 // data structure to reflect the latest status.
 // Data will be flushed at the end.
-nsresult 
+nsresult
 nsProfileAccess::FillProfileInfo(nsIFile* regName)
 {
     nsresult rv = NS_OK;
@@ -281,7 +289,7 @@ nsProfileAccess::FillProfileInfo(nsIFile* regName)
     nsCOMPtr<nsIRegistry> registry(do_CreateInstance(NS_REGISTRY_CONTRACTID, &rv));
     if (NS_FAILED(rv)) return rv;
     rv = registry->Open(regName);
-    if (NS_FAILED(rv)) return rv;   
+    if (NS_FAILED(rv)) return rv;
 
     // Enumerate all subkeys (immediately) under the given node.
     nsCOMPtr<nsIEnumerator> enumKeys;
@@ -344,6 +352,14 @@ nsProfileAccess::FillProfileInfo(nsIFile* regName)
         mHavePREGInfo = kRegistryNoString;
         mProfileDataChanged = PR_TRUE;
     }
+    
+    // Get the StartWithLastProfile flag
+    PRInt32 tempLong;
+    rv = registry->GetInt(profilesTreeKey,
+                           kRegistryStartWithLastString.get(),
+                           &tempLong);
+    if (NS_SUCCEEDED(rv))
+        mStartWithLastProfile = tempLong;
 
     rv = registry->EnumerateSubtrees( profilesTreeKey, getter_AddRefs(enumKeys));
     if (NS_FAILED(rv)) return rv;
@@ -646,7 +662,7 @@ nsProfileAccess::UpdateRegistry(nsIFile* regName)
     nsCOMPtr<nsIRegistry> registry(do_CreateInstance(NS_REGISTRY_CONTRACTID, &rv));
     if (NS_FAILED(rv)) return rv;
     rv = registry->Open(regName);
-    if (NS_FAILED(rv)) return rv;   
+    if (NS_FAILED(rv)) return rv;
 
     // Enumerate all subkeys (immediately) under the given node.
     nsCOMPtr<nsIEnumerator> enumKeys;
@@ -685,6 +701,12 @@ nsProfileAccess::UpdateRegistry(nsIFile* regName)
                              mHavePREGInfo.get());
     if (NS_FAILED(rv)) return rv;
 
+    // Set the StartWithLastProfile flag
+    rv = registry->SetInt(profilesTreeKey,
+                          kRegistryStartWithLastString.get(),
+                          mStartWithLastProfile);
+    if (NS_FAILED(rv)) return rv;
+
     rv = registry->EnumerateSubtrees(profilesTreeKey, getter_AddRefs(enumKeys));
     if (NS_FAILED(rv)) return rv;
 
@@ -910,6 +932,22 @@ nsProfileAccess::SetProfileLastModTime(const PRUnichar *profileName, PRInt64 las
     return NS_ERROR_FAILURE;
 }
 
+nsresult
+nsProfileAccess::GetStartWithLastUsedProfile(PRBool *aStartWithLastUsedProfile)
+{
+    NS_ENSURE_ARG_POINTER(aStartWithLastUsedProfile);
+    *aStartWithLastUsedProfile = mStartWithLastProfile;
+    return NS_OK;
+}
+
+nsresult
+nsProfileAccess::SetStartWithLastUsedProfile(PRBool aStartWithLastUsedProfile)
+{
+    mStartWithLastProfile = aStartWithLastUsedProfile;
+    mProfileDataChanged = PR_TRUE;
+    return NS_OK;
+}
+
 // Return the list of profiles, 4x, 5x, or both.
 nsresult
 nsProfileAccess::GetProfileList(PRInt32 whichKind, PRUint32 *length, PRUnichar ***result)
@@ -1012,22 +1050,18 @@ nsProfileAccess::ProfileExists(const PRUnichar *profileName)
 
 // Capture the 4x profile information from the old registry (4x)
 nsresult
-nsProfileAccess::Get4xProfileInfo(const char *registryName, PRBool fromImport)
+nsProfileAccess::Get4xProfileInfo(nsIFile *registryFile, PRBool fromImport)
 {
     nsresult rv = NS_OK;
     if (fromImport && m4xProfilesAdded)
         return rv;
 
-    nsAutoString charSet;
+    nsCAutoString charSet;
     rv = GetPlatformCharset(charSet);
     if (NS_FAILED(rv)) return rv;
 
-#if defined(XP_PC) || defined(XP_MAC)
-    NS_ASSERTION(registryName, "Invalid registryName");
-
-    nsCOMPtr<nsILocalFile> registryFile;
-    rv = NS_NewNativeLocalFile(nsDependentCString(registryName), PR_TRUE, getter_AddRefs(registryFile));
-    if (NS_FAILED(rv)) return rv;
+#if defined(XP_WIN) || defined(XP_OS2) || defined(XP_MAC) || defined(XP_MACOSX)
+    NS_ENSURE_ARG(registryFile);
 
     nsCOMPtr<nsIRegistry> oldReg(do_CreateInstance(NS_REGISTRY_CONTRACTID, &rv));
     if (NS_FAILED(rv)) return rv;
@@ -1069,16 +1103,16 @@ nsProfileAccess::Get4xProfileInfo(const char *registryName, PRBool fromImport)
         // There is some problem I guess in sending a space as itself
         // NOTE: This needs to be done BEFORE the test for existence.
 
-#if defined(XP_MAC)
+#if defined(XP_MAC) || defined(XP_MACOSX)
         // 4.x profiles coming from japanese machine are already in unicode.
         // So, there is no need to decode into unicode further.
         NS_ConvertUCS2toUTF8 temp(profile);
-        nsCAutoString profileName(nsUnescape( NS_CONST_CAST(char*, temp.get())));
+        nsCAutoString profileName(nsUnescape(temp.BeginWriting()));
         nsAutoString convertedProfName(NS_ConvertUTF8toUCS2(profileName).get());
 #else
         nsCAutoString temp; temp.AssignWithConversion(profile);
 
-        nsCAutoString profileName(nsUnescape( NS_CONST_CAST(char*, temp.get())));
+        nsCAutoString profileName(nsUnescape(temp.BeginWriting()));
         nsAutoString convertedProfName;
         ConvertStringToUnicode(charSet, profileName.get(), convertedProfName);
 #endif
@@ -1145,28 +1179,25 @@ nsProfileAccess::Get4xProfileInfo(const char *registryName, PRBool fromImport)
         if ( ! unixProfileName.IsEmpty() && ! unixProfileDirectory.IsEmpty() ) {
             nsCAutoString profileLocation(unixProfileDirectory);
             profileLocation += "/.netscape";
-            nsCOMPtr<nsIFileSpec> users4xDotNetscapeDirectory;
-
-            rv = NS_NewFileSpec(getter_AddRefs(users4xDotNetscapeDirectory));
-            if (NS_FAILED(rv)) return rv;
-
-            rv = users4xDotNetscapeDirectory->SetNativePath(profileLocation.get());
-            if (NS_FAILED(rv)) return rv;
-            rv = users4xDotNetscapeDirectory->Exists(&exists);
-
-            if (NS_FAILED(rv)) return rv;
 
+            nsCOMPtr<nsILocalFile> fileInNSDir;
+            rv = NS_NewNativeLocalFile(profileLocation + NS_LITERAL_CSTRING("/preferences.js"),
+                                       PR_TRUE,
+                                       getter_AddRefs(fileInNSDir));
+            if (NS_FAILED(rv))
+              return rv;
+            rv = fileInNSDir->Exists(&exists);
 #ifdef DEBUG
-            printf("%s exists:  %d\n",profileLocation.get(), exists);
+            printf("%s/preferences.js exists:  %d\n",profileLocation.get(), NS_SUCCEEDED(rv) && exists);
 #endif
-            if (exists) {
+            if (NS_SUCCEEDED(rv) && exists) {
                 ProfileStruct*  profileItem     = new ProfileStruct();
                 if (!profileItem)
                     return NS_ERROR_OUT_OF_MEMORY;
 
                 profileItem->updateProfileEntry = PR_TRUE;
 
-                profileItem->profileName = NS_ConvertASCIItoUCS2(unixProfileName).get();
+                CopyASCIItoUTF16(unixProfileName, profileItem->profileName);
 
                 nsCOMPtr<nsILocalFile> localFile;
                 rv = NS_NewNativeLocalFile(profileLocation, PR_TRUE, getter_AddRefs(localFile));
@@ -1361,53 +1392,70 @@ nsresult ProfileStruct::InternalizeLocation(nsIRegistry *aRegistry, nsRegistryKe
         if (NS_FAILED(rv)) return rv;
         regLocationData = profLoc;
 
-#if defined(XP_MAC)
+#if defined(XP_MAC) || defined(XP_MACOSX)
         // 4.x profiles coming from japanese machine are already in unicode.
         // So, there is no need to decode into unicode further.
 
         // Unescape profile location
         NS_ConvertUCS2toUTF8 tempLoc(profLoc);
-        nsCAutoString profileLocation(nsUnescape( NS_CONST_CAST(char*, tempLoc.get())));
+        nsCAutoString profileLocation(nsUnescape(tempLoc.BeginWriting()));
         nsAutoString convertedProfLoc(NS_ConvertUTF8toUCS2(profileLocation).get());
 #else
-        nsAutoString charSet;
+        nsCAutoString charSet;
         rv = GetPlatformCharset(charSet);
         if (NS_FAILED(rv)) return rv;
 
         // Unescape profile location and convert it to the right format
         nsCAutoString tempLoc; tempLoc.AssignWithConversion(profLoc);
 
-        nsCAutoString profileLocation(nsUnescape( NS_CONST_CAST(char*, tempLoc.get())));
+        nsCAutoString profileLocation(nsUnescape(tempLoc.BeginWriting()));
         nsAutoString convertedProfLoc;
         ConvertStringToUnicode(charSet, profileLocation.get(), convertedProfLoc);
 #endif
 
         // Now we have a unicode path - make it into a file
+#if defined(XP_MACOSX)
+        // This is an HFS style path, which can't be used with nsIFile, so convert it.
+        rv = NS_ERROR_FAILURE;
+        CFStringRef pathStrRef = ::CFStringCreateWithCharacters(NULL,
+                                      convertedProfLoc.get(), convertedProfLoc.Length());
+        if (pathStrRef)
+        {
+            CFURLRef pathURLRef = ::CFURLCreateWithFileSystemPath(NULL, pathStrRef, kCFURLHFSPathStyle, true);
+            if (pathURLRef)
+            {
+                rv = NS_NewNativeLocalFile(nsCString(), PR_TRUE, getter_AddRefs(tempLocal));
+                if (NS_SUCCEEDED(rv))
+                {
+                    nsCOMPtr<nsILocalFileMac> tempLocalMac(do_QueryInterface(tempLocal));
+                    rv = tempLocalMac->InitWithCFURL(pathURLRef);
+                }
+                ::CFRelease(pathURLRef);
+            }
+            ::CFRelease(pathStrRef);
+        }
+#else        
         rv = NS_NewLocalFile(convertedProfLoc, PR_TRUE, getter_AddRefs(tempLocal));
+#endif
     }
     else
     {
         nsXPIDLString regData;
 
-            rv = aRegistry->GetString(profKey,
-                                      kRegistryDirectoryString.get(),
-                                      getter_Copies(regData));
-            if (NS_FAILED(rv)) return rv;
-            regLocationData = regData;
+        rv = aRegistry->GetString(profKey,
+                                  kRegistryDirectoryString.get(),
+                                  getter_Copies(regData));
+        if (NS_FAILED(rv)) return rv;
+        regLocationData = regData;
 
-#ifdef XP_MAC
-            // For a brief time, this was a unicode path
-            PRInt32 firstColon = regLocationData.FindChar(PRUnichar(':'));
-            if (firstColon == -1)
-            {
-                rv = NS_NewLocalFile(nsString(), PR_TRUE, getter_AddRefs(tempLocal));
-                if (NS_SUCCEEDED(rv)) // XXX this only works on XP_MAC because regLocationData is ASCII
-                    rv = tempLocal->SetPersistentDescriptor(NS_ConvertUCS2toUTF8(regLocationData));
-            }
-            else
+#if defined(XP_MAC) || defined(XP_MACOSX)
+            rv = NS_NewNativeLocalFile(nsCString(), PR_TRUE, getter_AddRefs(tempLocal));
+        if (NS_SUCCEEDED(rv)) // regLocationData is ASCII so no loss
+            rv = tempLocal->SetPersistentDescriptor(NS_LossyConvertUCS2toASCII(regLocationData));
+#else
+        rv = NS_NewLocalFile(regLocationData, PR_TRUE, getter_AddRefs(tempLocal));
 #endif
-            rv = NS_NewLocalFile(regLocationData, PR_TRUE, getter_AddRefs(tempLocal));
-        }
+    }
 
     if (NS_SUCCEEDED(rv) && tempLocal)
     {
@@ -1427,7 +1475,7 @@ nsresult ProfileStruct::ExternalizeLocation(nsIRegistry *aRegistry, nsRegistryKe
     {
         nsAutoString regData;
 
-#if XP_MAC
+#if defined(XP_MAC) || defined(XP_MACOSX)
         PRBool leafCreated;
         nsCAutoString descBuf;
 
@@ -1449,7 +1497,7 @@ nsresult ProfileStruct::ExternalizeLocation(nsIRegistry *aRegistry, nsRegistryKe
                                  regData.get());
 
     }
-    else if (regLocationData.Length() != 0)
+    else if (!regLocationData.IsEmpty())
     {
         // Write the original data back out - maybe it can be resolved later.
         rv = aRegistry->SetString(profKey,
@@ -1476,7 +1524,7 @@ nsresult ProfileStruct::InternalizeMigratedFromLocation(nsIRegistry *aRegistry,
                                   getter_Copies(regData));
     if (NS_SUCCEEDED(rv))
     {
-#ifdef XP_MAC
+#if defined(XP_MAC) || defined(XP_MACOSX)
         rv = NS_NewLocalFile(nsString(), PR_TRUE, getter_AddRefs(tempLocal));
         if (NS_SUCCEEDED(rv))
         {
@@ -1498,14 +1546,14 @@ nsresult ProfileStruct::ExternalizeMigratedFromLocation(nsIRegistry *aRegistry,
 {
     nsresult rv = NS_OK;
     nsCAutoString regData;
-    
+
     if (migratedFrom)
     {
-#if XP_MAC
+#if defined(XP_MAC) || defined(XP_MACOSX)
         rv = migratedFrom->GetPersistentDescriptor(regData);
 #else
         nsAutoString path;
-        rv = resolvedLocation->GetPath(path);
+        rv = migratedFrom->GetPath(path);
         regData = NS_ConvertUCS2toUTF8(path);
 #endif
 
@@ -1528,513 +1576,9 @@ nsresult ProfileStruct::EnsureDirPathExists(nsILocalFile *aDir, PRBool *wasCreat
     rv = aDir->Exists(&exists);
     if (NS_SUCCEEDED(rv) && !exists)
     {
-        rv = aDir->Create(nsIFile::DIRECTORY_TYPE, 0775);
+        rv = aDir->Create(nsIFile::DIRECTORY_TYPE, 0700);
         *wasCreated = NS_SUCCEEDED(rv);
     }
     return rv;
 }
 
-
-// **********************************************************************
-// class nsProfileLock
-// **********************************************************************
-
-nsProfileLock::nsProfileLock() :
-    mHaveLock(PR_FALSE)
-#if defined (XP_WIN)
-    ,mLockFileHandle(INVALID_HANDLE_VALUE)
-#elif defined (XP_OS2)
-    ,mLockFileHandle(-1)
-#elif defined (XP_UNIX)
-    ,mPidLockFileName(nsnull)
-    ,mLockFileDesc(-1)
-#endif
-{
-#if defined (XP_UNIX)
-    next = prev = this;
-#endif
-}
-
-
-nsProfileLock::nsProfileLock(nsProfileLock& src)
-{
-    *this = src;
-}
-
-
-nsProfileLock& nsProfileLock::operator=(nsProfileLock& rhs)
-{
-    Unlock();
-
-    mHaveLock = rhs.mHaveLock;
-    rhs.mHaveLock = PR_FALSE;
-
-#if defined (XP_MAC)
-    mLockFile = rhs.mLockFile;
-    rhs.mLockFile = nsnull;
-#elif defined (XP_WIN)
-    mLockFileHandle = rhs.mLockFileHandle;
-    rhs.mLockFileHandle = INVALID_HANDLE_VALUE;
-#elif defined (XP_OS2)
-    mLockFileHandle = rhs.mLockFileHandle;
-    rhs.mLockFileHandle = -1;
-#elif defined (XP_UNIX)
-    mLockFileDesc = rhs.mLockFileDesc;
-    rhs.mLockFileDesc = -1;
-    mPidLockFileName = rhs.mPidLockFileName;
-    rhs.mPidLockFileName = nsnull;
-    if (mPidLockFileName)
-    {
-        // rhs had a symlink lock, therefore it was on the list.
-        PR_REMOVE_LINK(&rhs);
-        PR_APPEND_LINK(this, &mPidLockList);
-    }
-#endif
-
-    return *this;
-}
-
-
-nsProfileLock::~nsProfileLock()
-{
-    Unlock();
-}
-
-
-#if defined (XP_UNIX)
-
-static int setupPidLockCleanup;
-
-PRCList nsProfileLock::mPidLockList =
-    PR_INIT_STATIC_CLIST(&nsProfileLock::mPidLockList);
-
-void nsProfileLock::RemovePidLockFiles()
-{
-    while (!PR_CLIST_IS_EMPTY(&mPidLockList))
-    {
-        nsProfileLock *lock = NS_STATIC_CAST(nsProfileLock*, mPidLockList.next);
-        lock->Unlock();
-    }
-}
-
-static struct sigaction SIGHUP_oldact;
-static struct sigaction SIGINT_oldact;
-static struct sigaction SIGQUIT_oldact;
-static struct sigaction SIGILL_oldact;
-static struct sigaction SIGABRT_oldact;
-static struct sigaction SIGSEGV_oldact;
-static struct sigaction SIGTERM_oldact;
-
-void nsProfileLock::FatalSignalHandler(int signo)
-{
-    // Remove any locks still held.
-    RemovePidLockFiles();
-
-    // Chain to the old handler, which may exit.
-    struct sigaction *oldact = nsnull;
-
-    switch (signo) {
-      case SIGHUP:
-        oldact = &SIGHUP_oldact;
-        break;
-      case SIGINT:
-        oldact = &SIGINT_oldact;
-        break;
-      case SIGQUIT:
-        oldact = &SIGQUIT_oldact;
-        break;
-      case SIGILL:
-        oldact = &SIGILL_oldact;
-        break;
-      case SIGABRT:
-        oldact = &SIGABRT_oldact;
-        break;
-      case SIGSEGV:
-        oldact = &SIGSEGV_oldact;
-        break;
-      case SIGTERM:
-        oldact = &SIGTERM_oldact;
-        break;
-      default:
-        NS_NOTREACHED("bad signo");
-        break;
-    }
-
-    if (oldact &&
-        oldact->sa_handler &&
-        oldact->sa_handler != SIG_DFL &&
-        oldact->sa_handler != SIG_IGN)
-    {
-        oldact->sa_handler(signo);
-    }
-
-    // Backstop exit call, just in case.
-    _exit(signo);
-}
-
-#endif /* XP_UNIX */
-
-
-nsresult nsProfileLock::Lock(nsILocalFile* aFile)
-{
-#if defined (XP_UNIX)
-    NS_NAMED_LITERAL_STRING(OLD_LOCKFILE_NAME, "lock");
-    NS_NAMED_LITERAL_STRING(LOCKFILE_NAME, ".parentlock");
-#else
-    NS_NAMED_LITERAL_STRING(LOCKFILE_NAME, "parent.lock");
-#endif
-
-    nsresult rv;
-    NS_ENSURE_STATE(!mHaveLock);
-
-    PRBool isDir;
-    rv = aFile->IsDirectory(&isDir);
-    if (NS_FAILED(rv))
-        return rv;
-    if (!isDir)
-        return NS_ERROR_FILE_NOT_DIRECTORY;
-
-    nsCOMPtr<nsILocalFile> lockFile;
-    rv = aFile->Clone((nsIFile **)((void **)getter_AddRefs(lockFile)));
-    if (NS_FAILED(rv))
-        return rv;
-
-    rv = lockFile->Append(LOCKFILE_NAME);
-    if (NS_FAILED(rv))
-        return rv;
-
-#if defined(XP_MAC)
-    struct LockProcessInfo
-    {
-        ProcessSerialNumber psn;
-        unsigned long launchDate;
-    };
-
-    PRFileDesc *fd = nsnull;
-    PRInt32 ioBytes;
-    ProcessInfoRec processInfo;
-    LockProcessInfo lockProcessInfo;
-
-    rv = lockFile->OpenNSPRFileDesc(PR_RDONLY, 0, &fd);
-    if (NS_SUCCEEDED(rv))
-    {
-        ioBytes = PR_Read(fd, &lockProcessInfo, sizeof(LockProcessInfo));
-        PR_Close(fd);
-
-        if (ioBytes == sizeof(LockProcessInfo))
-        {
-            processInfo.processAppSpec = nsnull;
-            processInfo.processName = nsnull;
-            processInfo.processInfoLength = sizeof(ProcessInfoRec);
-            if (::GetProcessInformation(&lockProcessInfo.psn, &processInfo) == noErr &&
-                processInfo.processLaunchDate == lockProcessInfo.launchDate)
-            {
-                return NS_ERROR_FILE_ACCESS_DENIED;
-            }
-        }
-        else
-        {
-            NS_WARNING("Could not read lock file - ignoring lock");
-        }
-    }
-
-    rv = lockFile->OpenNSPRFileDesc(PR_WRONLY | PR_CREATE_FILE | PR_TRUNCATE, 0, &fd);
-    if (NS_FAILED(rv))
-        return rv;
-
-    static LockProcessInfo sSelfInfo;
-    static PRBool sSelfInfoInited;
-
-    if (!sSelfInfoInited)
-    {
-        ProcessSerialNumber psn;
-        if (::GetCurrentProcess(&psn) == noErr)
-        {
-            processInfo.processAppSpec = nsnull;
-            processInfo.processName = nsnull;
-            processInfo.processInfoLength = sizeof(ProcessInfoRec);
-            if (::GetProcessInformation(&psn, &processInfo) == noErr)
-            {
-                sSelfInfo.psn = processInfo.processNumber;
-                sSelfInfo.launchDate = processInfo.processLaunchDate;
-                sSelfInfoInited = PR_TRUE;
-            }
-        }
-    }
-    if (!sSelfInfoInited)
-    {
-        PR_Close(fd);
-        return NS_ERROR_FAILURE;
-    }
-
-    ioBytes = PR_Write(fd, &sSelfInfo, sizeof(LockProcessInfo));
-    PR_Close(fd);
-    if (ioBytes != sizeof(LockProcessInfo))
-        return NS_ERROR_FAILURE;
-    mLockFile = lockFile;
-
-#elif defined(XP_WIN)
-    nsCAutoString filePath;
-    rv = lockFile->GetNativePath(filePath);
-    if (NS_FAILED(rv))
-        return rv;
-    mLockFileHandle = CreateFile(filePath.get(),
-                                 GENERIC_READ | GENERIC_WRITE,
-                                 0, // no sharing - of course
-                                 nsnull,
-                                 OPEN_ALWAYS,
-                                 FILE_FLAG_DELETE_ON_CLOSE,
-                                 nsnull);
-    if (mLockFileHandle == INVALID_HANDLE_VALUE)
-        return NS_ERROR_FILE_ACCESS_DENIED;
-#elif defined(XP_OS2)
-    nsCAutoString filePath;
-    rv = lockFile->GetNativePath(filePath);
-    if (NS_FAILED(rv))
-        return rv;
-
-    ULONG   ulAction = 0;
-    APIRET  rc;
-    rc = DosOpen(filePath.get(),
-                  &mLockFileHandle,
-                  &ulAction,
-                  0,
-                  FILE_NORMAL,
-                  OPEN_ACTION_CREATE_IF_NEW | OPEN_ACTION_OPEN_IF_EXISTS,
-                  OPEN_ACCESS_READWRITE | OPEN_SHARE_DENYREADWRITE | OPEN_FLAGS_NOINHERIT,
-                  0 );
-    if (rc != NO_ERROR)
-    {
-        mLockFileHandle = -1;
-        return NS_ERROR_FILE_ACCESS_DENIED;
-    }
-#elif defined(VMS)
-    nsCAutoString filePath;
-    rv = lockFile->GetNativePath(filePath);
-    if (NS_FAILED(rv))
-        return rv;
-
-    remove(filePath.get());
-    mLockFileDesc = open(filePath.get(), O_WRONLY | O_CREAT | O_EXCL, 0666);
-    if (mLockFileDesc == -1)
-    {
-        NS_ERROR("Failed to open lock file.");
-        return NS_ERROR_FAILURE;
-    }
-#elif defined(XP_UNIX)
-    nsCOMPtr<nsILocalFile> oldLockFile;
-    rv = aFile->Clone((nsIFile **)((void **)getter_AddRefs(oldLockFile)));
-    if (NS_FAILED(rv))
-        return rv;
-
-    rv = oldLockFile->Append(OLD_LOCKFILE_NAME);
-    if (NS_FAILED(rv))
-        return rv;
-    nsCAutoString oldFilePath;
-    rv = oldLockFile->GetNativePath(oldFilePath);
-    if (NS_FAILED(rv))
-        return rv;
-
-    // First, try the 4.x-compatible symlink technique, which works with NFS
-    // without depending on (broken or missing, too often) lockd.
-    struct in_addr inaddr;
-    inaddr.s_addr = INADDR_LOOPBACK;
-
-    char hostname[256];
-    PRStatus status = PR_GetSystemInfo(PR_SI_HOSTNAME, hostname, sizeof hostname);
-    if (status == PR_SUCCESS)
-    {
-        char netdbbuf[PR_NETDB_BUF_SIZE];
-        PRHostEnt hostent;
-        status = PR_GetHostByName(hostname, netdbbuf, sizeof netdbbuf, &hostent);
-        if (status == PR_SUCCESS)
-            memcpy(&inaddr, hostent.h_addr, sizeof inaddr);
-    }
-
-    char *signature =
-        PR_smprintf("%s:%lu", inet_ntoa(inaddr), (unsigned long)getpid());
-    const char *oldFileName = oldFilePath.get();
-    int symlink_rv, symlink_errno, tries = 0;
-
-    // use ns4.x-compatible symlinks if the FS supports them
-    while ((symlink_rv = symlink(signature, oldFileName)) < 0)
-    {
-        symlink_errno = errno;
-        if (symlink_errno != EEXIST)
-            break;
-
-        // the link exists; see if it's from this machine, and if
-        // so if the process is still active
-        char buf[1024];
-        int len = readlink(oldFileName, buf, sizeof buf - 1);
-        if (len > 0)
-        {
-            buf[len] = '\0';
-            char *colon = strchr(buf, ':');
-            if (colon)
-            {
-                *colon++ = '\0';
-                unsigned long addr = inet_addr(buf);
-                if (addr != (unsigned long) -1)
-                {
-                    char *after = nsnull;
-                    pid_t pid = strtol(colon, &after, 0);
-                    if (pid != 0 && *after == '\0')
-                    {
-                        if (addr != inaddr.s_addr)
-                        {
-                            // Remote lock: give up even if stuck.
-                            break;
-                        }
-
-                        // kill(pid,0) is a neat trick to check if a
-                        // process exists
-                        if (kill(pid, 0) == 0 || errno != ESRCH)
-                        {
-                            // Local process appears to be alive, ass-u-me it
-                            // is another Mozilla instance, or a compatible
-                            // derivative, that's currently using the profile.
-                            // XXX need an "are you Mozilla?" protocol
-                            break;
-                        }
-                    }
-                }
-            }
-        }
-
-        // Lock seems to be bogus: try to claim it.  Give up after a large
-        // number of attempts (100 comes from the 4.x codebase).
-        (void) unlink(oldFileName);
-        if (++tries > 100)
-            break;
-    }
-
-    PR_smprintf_free(signature);
-    signature = nsnull;
-
-    if (symlink_rv == 0)
-    {
-        // We exclusively created the symlink: record its name for eventual
-        // unlock-via-unlink.
-        mPidLockFileName = strdup(oldFileName);
-        if (mPidLockFileName)
-        {
-            PR_APPEND_LINK(this, &mPidLockList);
-            if (!setupPidLockCleanup++)
-            {
-                // Clean up on normal termination.
-                atexit(RemovePidLockFiles);
-
-                // Clean up on abnormal termination, using POSIX sigaction.
-                // Don't arm a handler if the signal is being ignored, e.g.,
-                // because mozilla is run via nohup.
-                struct sigaction act, oldact;
-                act.sa_handler = FatalSignalHandler;
-                act.sa_flags = 0;
-                sigfillset(&act.sa_mask);
-
-#define CATCH_SIGNAL(signame)                                                 \
-    PR_BEGIN_MACRO                                                            \
-        if (sigaction(signame, NULL, &oldact) == 0 &&                         \
-            oldact.sa_handler != SIG_IGN)                                     \
-        {                                                                     \
-            sigaction(signame, &act, &signame##_oldact);                      \
-        }                                                                     \
-    PR_END_MACRO
-
-                CATCH_SIGNAL(SIGHUP);
-                CATCH_SIGNAL(SIGINT);
-                CATCH_SIGNAL(SIGQUIT);
-                CATCH_SIGNAL(SIGILL);
-                CATCH_SIGNAL(SIGABRT);
-                CATCH_SIGNAL(SIGSEGV);
-                CATCH_SIGNAL(SIGTERM);
-
-#undef CATCH_SIGNAL
-            }
-        }
-    }
-    else if (symlink_errno != EEXIST)
-    {
-        // Symlinks aren't supported (for example, on Win32 SAMBA servers).
-        // F_SETLK is not well supported on all NFS servers, which is why we
-        // try symlinks first.
-        nsCAutoString filePath;
-        rv = lockFile->GetNativePath(filePath);
-        if (NS_FAILED(rv))
-            return rv;
-
-        mLockFileDesc = open(filePath.get(), O_WRONLY | O_CREAT | O_TRUNC, 0666);
-        if (mLockFileDesc == -1)
-        {
-            NS_ERROR("Failed to open lock file.");
-            return NS_ERROR_FAILURE;
-        }
-        struct flock lock;
-        lock.l_start = 0;
-        lock.l_len = 0; // len = 0 means entire file
-        lock.l_type = F_WRLCK;
-        lock.l_whence = SEEK_SET;
-        if (fcntl(mLockFileDesc, F_SETLK, &lock) == -1)
-        {
-            if (errno == EAGAIN || errno == EACCES)
-                return NS_ERROR_FILE_ACCESS_DENIED;
-            return NS_ERROR_FAILURE;
-        }
-    }
-    else
-    {
-        // Couldn't create the symlink (but symlink(2) is supported).
-        // This error code will cause the right dialog to be displayed.
-        return NS_ERROR_FILE_ACCESS_DENIED;
-    }
-#endif /* XP_UNIX */
-
-    mHaveLock = PR_TRUE;
-
-    return rv;
-}
-
-
-nsresult nsProfileLock::Unlock()
-{
-    nsresult rv = NS_OK;
-
-    if (mHaveLock)
-    {
-#if defined (XP_MAC)
-        if (mLockFile)
-            rv = mLockFile->Remove(PR_FALSE);
-#elif defined (XP_WIN)
-        if (mLockFileHandle != INVALID_HANDLE_VALUE)
-        {
-            CloseHandle(mLockFileHandle);
-            mLockFileHandle = INVALID_HANDLE_VALUE;
-        }
-#elif defined (XP_OS2)
-        if (mLockFileHandle != -1)
-        {
-            DosClose(mLockFileHandle);
-            mLockFileHandle = -1;
-        }
-#elif defined (XP_UNIX)
-        if (mPidLockFileName)
-        {
-            PR_REMOVE_LINK(this);
-            (void) unlink(mPidLockFileName);
-            free(mPidLockFileName);
-            mPidLockFileName = nsnull;
-        }
-        else if (mLockFileDesc != -1)
-        {
-            close(mLockFileDesc);
-            mLockFileDesc = -1;
-            // Don't remove it
-        }
-#endif
-
-        mHaveLock = PR_FALSE;
-    }
-
-    return rv;
-}
