@@ -63,8 +63,10 @@
 #include "nsIServiceManager.h"
 #include "nsIStandardURL.h"
 #include "nsIStreamListener.h"
+#ifdef MOZ_XUL
 #include "nsIXULPrototypeCache.h"
 #include "nsIXULPrototypeDocument.h"
+#endif
 #include "nsNetCID.h"
 #include "nsXPIDLString.h"
 #include "nsString.h"
@@ -72,11 +74,15 @@
 
 //----------------------------------------------------------------------
 
-static NS_DEFINE_CID(kChromeRegistryCID,         NS_CHROMEREGISTRY_CID);
 static NS_DEFINE_CID(kEventQueueServiceCID,      NS_EVENTQUEUESERVICE_CID);
 static NS_DEFINE_CID(kIOServiceCID,              NS_IOSERVICE_CID);
 static NS_DEFINE_CID(kStandardURLCID,            NS_STANDARDURL_CID);
+#ifdef MOZ_XUL
 static NS_DEFINE_CID(kXULPrototypeCacheCID,      NS_XULPROTOTYPECACHE_CID);
+#endif
+
+// This comes from nsChromeRegistry.cpp
+extern nsIChromeRegistry* gChromeRegistry;
 
 //----------------------------------------------------------------------
 //
@@ -159,7 +165,7 @@ PRLogModuleInfo* nsCachedChromeChannel::gLog;
 
 NS_IMPL_ISUPPORTS2(nsCachedChromeChannel,
                    nsIChannel,
-                   nsIRequest);
+                   nsIRequest)
 
 nsresult
 nsCachedChromeChannel::Create(nsIURI* aURI, nsIChannel** aResult)
@@ -181,8 +187,6 @@ nsCachedChromeChannel::Create(nsIURI* aURI, nsIChannel** aResult)
 nsCachedChromeChannel::nsCachedChromeChannel(nsIURI* aURI)
     : mURI(aURI), mLoadGroup(nsnull), mLoadFlags (nsIRequest::LOAD_NORMAL), mStatus(NS_OK)
 {
-    NS_INIT_REFCNT();
-
 #ifdef PR_LOGGING
     if (! gLog)
         gLog = PR_NewLogModule("nsCachedChromeChannel");
@@ -496,7 +500,6 @@ nsCachedChromeChannel::DestroyLoadEvent(PLEvent* aEvent)
 
 nsChromeProtocolHandler::nsChromeProtocolHandler()
 {
-    NS_INIT_REFCNT();
 }
 
 nsresult
@@ -509,7 +512,7 @@ nsChromeProtocolHandler::~nsChromeProtocolHandler()
 {
 }
 
-NS_IMPL_THREADSAFE_ISUPPORTS2(nsChromeProtocolHandler, nsIProtocolHandler, nsISupportsWeakReference);
+NS_IMPL_THREADSAFE_ISUPPORTS2(nsChromeProtocolHandler, nsIProtocolHandler, nsISupportsWeakReference)
 
 NS_METHOD
 nsChromeProtocolHandler::Create(nsISupports *aOuter, REFNSIID aIID, void **aResult)
@@ -567,8 +570,12 @@ nsChromeProtocolHandler::NewURI(const nsACString &aSpec,
                                 nsIURI *aBaseURI,
                                 nsIURI **result)
 {
+    NS_PRECONDITION(result, "Null out param");
+    
     nsresult rv;
 
+    *result = nsnull;
+
     // Chrome: URLs (currently) have no additional structure beyond that provided
     // by standard URLs, so there is no "outer" given to CreateInstance
 
@@ -580,25 +587,68 @@ nsChromeProtocolHandler::NewURI(const nsACString &aSpec,
     if (NS_FAILED(rv))
         return rv;
 
-    return CallQueryInterface(url, result);
+    nsCOMPtr<nsIURI> uri(do_QueryInterface(url, &rv));
+    if (NS_FAILED(rv))
+        return rv;
+    
+    // Canonify the "chrome:" URL; e.g., so that we collapse
+    // "chrome://navigator/content/" and "chrome://navigator/content"
+    // and "chrome://navigator/content/navigator.xul".
+
+    // Try the global cache first.
+    nsCOMPtr<nsIChromeRegistry> reg = gChromeRegistry;
+
+    // If that fails, the service has not been instantiated yet; let's
+    // do that now.
+    if (!reg) {
+        reg = do_GetService(NS_CHROMEREGISTRY_CONTRACTID, &rv);
+        if (NS_FAILED(rv))
+            return rv;
+    }
+
+    NS_ASSERTION(reg, "Must have a chrome registry by now");
+    
+    rv = reg->Canonify(uri);
+    if (NS_FAILED(rv))
+        return rv;
+
+    *result = uri;
+    NS_ADDREF(*result);
+    return NS_OK;
 }
 
 NS_IMETHODIMP
 nsChromeProtocolHandler::NewChannel(nsIURI* aURI,
                                     nsIChannel* *aResult)
 {
+    NS_ENSURE_ARG_POINTER(aURI);
+    NS_PRECONDITION(aResult, "Null out param");
+    
+#ifdef DEBUG
+    // Check that the uri we got is already canonified
+    nsresult debug_rv;
+    nsCOMPtr<nsIChromeRegistry> debugReg(do_GetService(NS_CHROMEREGISTRY_CONTRACTID, &debug_rv));
+    if (NS_SUCCEEDED(debug_rv)) {
+        nsCOMPtr<nsIURI> debugClone;
+        debug_rv = aURI->Clone(getter_AddRefs(debugClone));
+        if (NS_SUCCEEDED(debug_rv)) {
+            debug_rv = debugReg->Canonify(debugClone);
+            if (NS_SUCCEEDED(debug_rv)) {
+                PRBool same;
+                debug_rv = aURI->Equals(debugClone, &same);
+                if (NS_SUCCEEDED(debug_rv)) {
+                    NS_ASSERTION(same, "Non-canonified chrome uri passed to nsChromeProtocolHandler::NewChannel!");
+                }
+            }
+                
+        }
+    }
+#endif
+
     nsresult rv;
     nsCOMPtr<nsIChannel> result;
 
-    // Canonify the "chrome:" URL; e.g., so that we collapse
-    // "chrome://navigator/content/navigator.xul" and "chrome://navigator/content"
-    // and "chrome://navigator/content/navigator.xul".
-    nsCOMPtr<nsIChromeRegistry> reg(do_GetService(kChromeRegistryCID, &rv));
-    if (NS_FAILED(rv)) return rv;
-
-    rv = reg->Canonify(aURI);
-    if (NS_FAILED(rv)) return rv;
-
+#ifdef MOZ_XUL
     // Check the prototype cache to see if we've already got the
     // document in the cache.
     nsCOMPtr<nsIXULPrototypeCache> cache =
@@ -606,8 +656,23 @@ nsChromeProtocolHandler::NewChannel(nsIURI* aURI,
     if (NS_FAILED(rv)) return rv;
 
     nsCOMPtr<nsIXULPrototypeDocument> proto;
-    rv = cache->GetPrototype(aURI, getter_AddRefs(proto));
-    if (NS_FAILED(rv)) return rv;
+    cache->GetPrototype(aURI, getter_AddRefs(proto));
+
+    // Same comment as nsXULDocument::StartDocumentLoad and
+    // nsXULDocument::ResumeWalk
+    // - Ben Goodger
+    //
+    // We don't abort on failure here because there are too many valid
+    // cases that can return failure, and the null-ness of |proto| is enough
+    // to trigger the fail-safe parse-from-disk solution. Example failure cases
+    // (for reference) include:
+    //
+    // NS_ERROR_NOT_AVAILABLE: the URI cannot be found in the FastLoad cache, 
+    //                         parse from disk
+    // other: the FastLoad cache file, XUL.mfl, could not be found, probably
+    //        due to being accessed before a profile has been selected (e.g.
+    //        loading chrome for the profile manager itself). This must be 
+    //        parsed from disk. 
 
     if (proto) {
         // ...in which case, we'll create a dummy stream that'll just
@@ -615,15 +680,23 @@ nsChromeProtocolHandler::NewChannel(nsIURI* aURI,
         rv = nsCachedChromeChannel::Create(aURI, getter_AddRefs(result));
         if (NS_FAILED(rv)) return rv;
     }
-    else {
+    else
+#endif
+        {
         // Miss. Resolve the chrome URL using the registry and do a
         // normal necko load.
         //nsXPIDLCString oldSpec;
         //aURI->GetSpec(getter_Copies(oldSpec));
         //printf("*************************** %s\n", (const char*)oldSpec);
 
-        nsXPIDLCString spec;
-        rv = reg->ConvertChromeURL(aURI, getter_Copies(spec));
+        nsCOMPtr<nsIChromeRegistry> reg = gChromeRegistry;
+        if (!reg) {
+            reg = do_GetService(NS_CHROMEREGISTRY_CONTRACTID, &rv);
+            if (NS_FAILED(rv)) return rv;
+        }
+
+        nsCAutoString spec;
+        rv = reg->ConvertChromeURL(aURI, spec);
         if (NS_FAILED(rv)) return rv;
 
         nsCOMPtr<nsIIOService> ioServ(do_GetService(kIOServiceCID, &rv));
@@ -689,42 +762,32 @@ nsChromeProtocolHandler::NewChannel(nsIURI* aURI,
             fastLoadServ->GetOutputStream(getter_AddRefs(objectOutput));
             if (objectOutput) {
                 nsCOMPtr<nsIFile> file;
-                nsCOMPtr<nsIChannel> chan = result;
-                while (!fileChan) {
-                    nsCOMPtr<nsIURI> uri;
-                    chan->GetURI(getter_AddRefs(uri));
 
-                    // Loop, jar: URIs can nest (e.g. jar:jar:A.jar!B.jar!C.xml).
+                if (fileChan) {
+                    fileChan->GetFile(getter_AddRefs(file));
+                } else {
+                    nsCOMPtr<nsIURI> uri;
+                    result->GetURI(getter_AddRefs(uri));
+
+                    // Loop, jar URIs can nest (e.g. jar:jar:A.jar!B.jar!C.xml).
                     // Often, however, we have jar:resource:/chrome/A.jar!C.xml.
                     nsCOMPtr<nsIJARURI> jarURI;
                     while ((jarURI = do_QueryInterface(uri)) != nsnull)
                         jarURI->GetJARFile(getter_AddRefs(uri));
 
-                    // Here we must have a URL of the form resource:/chrome/A.jar
-                    // or file:/some/path/to/A.jar.  Let's hope for the latter.
+                    // Here we have a URL of the form resource:/chrome/A.jar
+                    // or file:/some/path/to/A.jar.
                     nsCOMPtr<nsIFileURL> fileURL(do_QueryInterface(uri));
-                    if (fileURL) {
+                    if (fileURL)
                         fileURL->GetFile(getter_AddRefs(file));
-                        if (file)
-                            break;
-                    }
-
-                    // Thanks to the way that the resource: URL implementation
-                    // hides its substitution code from itself and the rest of
-                    // the world, we must make a new channel simply to get the
-                    // substituted URI.
-                    ioServ->NewChannelFromURI(uri, getter_AddRefs(chan));
-                    if (!chan)
-                        break;
-                    fileChan = do_QueryInterface(chan);
                 }
 
-                if (!file && fileChan)
-                    fileChan->GetFile(getter_AddRefs(file));
                 if (file) {
                     rv = fastLoadServ->AddDependency(file);
+#ifdef MOZ_XUL
                     if (NS_FAILED(rv))
                         cache->AbortFastLoads();
+#endif
                 }
             }
         }
