@@ -41,24 +41,46 @@
 #include "nsNSSCertificate.h"
 #include "smime.h"
 #include "cms.h"
+#include "nsICMSMessageErrors.h"
+#include "nsArray.h"
 
 #include "prlog.h"
 #ifdef PR_LOGGING
 extern PRLogModuleInfo* gPIPNSSLog;
 #endif
 
+#include "nsNSSCleaner.h"
+NSSCleanupAutoPtrClass(CERTCertificate, CERT_DestroyCertificate)
+
 NS_IMPL_THREADSAFE_ISUPPORTS1(nsHash, nsIHash)
 
 nsHash::nsHash() : m_ctxt(nsnull)
 {
-  NS_INIT_ISUPPORTS();
 }
 
-
 nsHash::~nsHash()
 {
+  nsNSSShutDownPreventionLock locker;
+  if (isAlreadyShutDown())
+    return;
+
+  destructorSafeDestroyNSSReference();
+  shutdown(calledFromObject);
+}
+
+void nsHash::virtualDestroyNSSReference()
+{
+  destructorSafeDestroyNSSReference();
+}
+
+void nsHash::destructorSafeDestroyNSSReference()
+{
+  if (isAlreadyShutDown())
+    return;
+
   if (m_ctxt) {
     HASH_Destroy(m_ctxt);
+    m_ctxt = nsnull;
   }
 }
 
@@ -70,6 +92,10 @@ NS_IMETHODIMP nsHash::ResultLen(PRInt16 aAlg, PRUint32 * aLen)
 
 NS_IMETHODIMP nsHash::Create(PRInt16 aAlg)
 {
+  nsNSSShutDownPreventionLock locker;
+  if (isAlreadyShutDown())
+    return NS_ERROR_NOT_AVAILABLE;
+
   m_ctxt = HASH_Create((HASH_HashType)aAlg);
   if (m_ctxt == nsnull) {
     return NS_ERROR_FAILURE;
@@ -79,18 +105,30 @@ NS_IMETHODIMP nsHash::Create(PRInt16 aAlg)
 
 NS_IMETHODIMP nsHash::Begin()
 {
+  nsNSSShutDownPreventionLock locker;
+  if (isAlreadyShutDown())
+    return NS_ERROR_NOT_AVAILABLE;
+
   HASH_Begin(m_ctxt);
   return NS_OK;
 }
 
 NS_IMETHODIMP nsHash::Update(unsigned char* aBuf, PRUint32 aLen)
 {
+  nsNSSShutDownPreventionLock locker;
+  if (isAlreadyShutDown())
+    return NS_ERROR_NOT_AVAILABLE;
+
   HASH_Update(m_ctxt, (const unsigned char*)aBuf, aLen);
   return NS_OK;
 }
 
 NS_IMETHODIMP nsHash::End(unsigned char* aBuf, PRUint32* aResultLen, PRUint32 aMaxResultLen)
 {
+  nsNSSShutDownPreventionLock locker;
+  if (isAlreadyShutDown())
+    return NS_ERROR_NOT_AVAILABLE;
+
   HASH_End(m_ctxt, aBuf, aResultLen, aMaxResultLen);
   return NS_OK;
 }
@@ -99,17 +137,33 @@ NS_IMPL_THREADSAFE_ISUPPORTS1(nsCMSMessage, nsICMSMessage)
 
 nsCMSMessage::nsCMSMessage()
 {
-  NS_INIT_ISUPPORTS();
   m_cmsMsg = nsnull;
 }
 nsCMSMessage::nsCMSMessage(NSSCMSMessage *aCMSMsg)
 {
-  NS_INIT_ISUPPORTS();
   m_cmsMsg = aCMSMsg;
 }
 
 nsCMSMessage::~nsCMSMessage()
 {
+  nsNSSShutDownPreventionLock locker;
+  if (isAlreadyShutDown())
+    return;
+
+  destructorSafeDestroyNSSReference();
+  shutdown(calledFromObject);
+}
+
+void nsCMSMessage::virtualDestroyNSSReference()
+{
+  destructorSafeDestroyNSSReference();
+}
+
+void nsCMSMessage::destructorSafeDestroyNSSReference()
+{
+  if (isAlreadyShutDown())
+    return;
+
   if (m_cmsMsg) {
     NSS_CMSMessage_Destroy(m_cmsMsg);
   }
@@ -117,62 +171,123 @@ nsCMSMessage::~nsCMSMessage()
 
 NS_IMETHODIMP nsCMSMessage::VerifySignature()
 {
-  PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("nsCMSMessage::VerifySignature\n"));
-  return  NS_ERROR_NOT_IMPLEMENTED;
+  return CommonVerifySignature(nsnull, 0);
+}
+
+NSSCMSSignerInfo* nsCMSMessage::GetTopLevelSignerInfo()
+{
+  nsNSSShutDownPreventionLock locker;
+  if (isAlreadyShutDown())
+    return nsnull;
+
+  if (!m_cmsMsg)
+    return nsnull;
+
+  if (!NSS_CMSMessage_IsSigned(m_cmsMsg))
+    return nsnull;
+
+  NSSCMSContentInfo *cinfo = NSS_CMSMessage_ContentLevel(m_cmsMsg, 0);
+  if (!cinfo)
+    return nsnull;
+
+  NSSCMSSignedData *sigd = (NSSCMSSignedData*)NSS_CMSContentInfo_GetContent(cinfo);
+  if (!sigd)
+    return nsnull;
+
+  PR_ASSERT(NSS_CMSSignedData_SignerInfoCount(sigd) > 0);
+  return NSS_CMSSignedData_GetSignerInfo(sigd, 0);
 }
 
 NS_IMETHODIMP nsCMSMessage::GetSignerEmailAddress(char * * aEmail)
 {
+  nsNSSShutDownPreventionLock locker;
+  if (isAlreadyShutDown())
+    return NS_ERROR_NOT_AVAILABLE;
+
   PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("nsCMSMessage::GetSignerEmailAddress\n"));
-  return NS_ERROR_NOT_IMPLEMENTED;
+  NS_ENSURE_ARG(aEmail);
+
+  NSSCMSSignerInfo *si = GetTopLevelSignerInfo();
+  if (!si)
+    return NS_ERROR_FAILURE;
+
+  *aEmail = NSS_CMSSignerInfo_GetSignerEmailAddress(si);
+  return NS_OK;
 }
 
 NS_IMETHODIMP nsCMSMessage::GetSignerCommonName(char ** aName)
 {
+  nsNSSShutDownPreventionLock locker;
+  if (isAlreadyShutDown())
+    return NS_ERROR_NOT_AVAILABLE;
+
   PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("nsCMSMessage::GetSignerCommonName\n"));
-  return NS_ERROR_NOT_IMPLEMENTED;
+  NS_ENSURE_ARG(aName);
+
+  NSSCMSSignerInfo *si = GetTopLevelSignerInfo();
+  if (!si)
+    return NS_ERROR_FAILURE;
+
+  *aName = NSS_CMSSignerInfo_GetSignerCommonName(si);
+  return NS_OK;
 }
 
-NS_IMETHODIMP nsCMSMessage::ContentIsEncrypted(int *)
+NS_IMETHODIMP nsCMSMessage::ContentIsEncrypted(PRBool *isEncrypted)
 {
+  nsNSSShutDownPreventionLock locker;
+  if (isAlreadyShutDown())
+    return NS_ERROR_NOT_AVAILABLE;
+
   PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("nsCMSMessage::ContentIsEncrypted\n"));
-  return NS_ERROR_NOT_IMPLEMENTED;
+  NS_ENSURE_ARG(isEncrypted);
+
+  if (!m_cmsMsg)
+    return NS_ERROR_FAILURE;
+
+  *isEncrypted = NSS_CMSMessage_IsEncrypted(m_cmsMsg);
+
+  return NS_OK;
 }
 
-NS_IMETHODIMP nsCMSMessage::ContentIsSigned(int *)
+NS_IMETHODIMP nsCMSMessage::ContentIsSigned(PRBool *isSigned)
 {
+  nsNSSShutDownPreventionLock locker;
+  if (isAlreadyShutDown())
+    return NS_ERROR_NOT_AVAILABLE;
+
   PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("nsCMSMessage::ContentIsSigned\n"));
-  return NS_ERROR_NOT_IMPLEMENTED;
+  NS_ENSURE_ARG(isSigned);
+
+  if (!m_cmsMsg)
+    return NS_ERROR_FAILURE;
+
+  *isSigned = NSS_CMSMessage_IsSigned(m_cmsMsg);
+
+  return NS_OK;
 }
 
 NS_IMETHODIMP nsCMSMessage::GetSignerCert(nsIX509Cert **scert)
 {
-  if (!m_cmsMsg)
-    return NS_ERROR_FAILURE;
+  nsNSSShutDownPreventionLock locker;
+  if (isAlreadyShutDown())
+    return NS_ERROR_NOT_AVAILABLE;
 
-  if (!NSS_CMSMessage_IsSigned(m_cmsMsg))
-    return NS_ERROR_FAILURE;
-
-  NSSCMSContentInfo *cinfo = NSS_CMSMessage_ContentLevel(m_cmsMsg, 0);
-  if (!cinfo)
-    return NS_ERROR_FAILURE;
-
-  NSSCMSSignedData *sigd = (NSSCMSSignedData*)NSS_CMSContentInfo_GetContent(cinfo);
-  if (!sigd)
-    return NS_ERROR_FAILURE;
-
-  PR_ASSERT(NSS_CMSSignedData_SignerInfoCount(sigd) > 0);
-  NSSCMSSignerInfo *si = NSS_CMSSignedData_GetSignerInfo(sigd, 0);
+  NSSCMSSignerInfo *si = GetTopLevelSignerInfo();
   if (!si)
     return NS_ERROR_FAILURE;
 
   if (si->cert) {
+    PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("nsCMSMessage::GetSignerCert got signer cert\n"));
+
     *scert = new nsNSSCertificate(si->cert);
     if (*scert) {
       (*scert)->AddRef();
     }
   }
   else {
+    PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("nsCMSMessage::GetSignerCert no signer cert, do we have a cert list? %s\n",
+      (si->certList != nsnull ? "yes" : "no") ));
+
     *scert = nsnull;
   }
   
@@ -181,47 +296,71 @@ NS_IMETHODIMP nsCMSMessage::GetSignerCert(nsIX509Cert **scert)
 
 NS_IMETHODIMP nsCMSMessage::GetEncryptionCert(nsIX509Cert **ecert)
 {
+  nsNSSShutDownPreventionLock locker;
+  if (isAlreadyShutDown())
+    return NS_ERROR_NOT_AVAILABLE;
+
     return NS_ERROR_NOT_IMPLEMENTED;
 }
 
 NS_IMETHODIMP nsCMSMessage::VerifyDetachedSignature(unsigned char* aDigestData, PRUint32 aDigestDataLen)
 {
-  PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("nsCMSMessage::VerifyDetachedSignature\n"));
+  nsNSSShutDownPreventionLock locker;
+  if (isAlreadyShutDown())
+    return NS_ERROR_NOT_AVAILABLE;
+
+  if (!aDigestData || !aDigestDataLen)
+    return NS_ERROR_FAILURE;
+
+  return CommonVerifySignature(aDigestData, aDigestDataLen);
+}
+
+nsresult nsCMSMessage::CommonVerifySignature(unsigned char* aDigestData, PRUint32 aDigestDataLen)
+{
+  nsNSSShutDownPreventionLock locker;
+  if (isAlreadyShutDown())
+    return NS_ERROR_NOT_AVAILABLE;
+
+  PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("nsCMSMessage::CommonVerifySignature, content level count %d\n", NSS_CMSMessage_ContentLevelCount(m_cmsMsg)));
   NSSCMSContentInfo *cinfo = nsnull;
   NSSCMSSignedData *sigd = nsnull;
   NSSCMSSignerInfo *si;
-  SECItem digest;
   PRInt32 nsigners;
   nsresult rv = NS_ERROR_FAILURE;
 
-  digest.data = aDigestData;
-  digest.len = aDigestDataLen;
-
   if (NSS_CMSMessage_IsSigned(m_cmsMsg) == PR_FALSE) {
-    PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("nsCMSMessage::VerifyDetachedSignature - not signed\n"));
+    PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("nsCMSMessage::CommonVerifySignature - not signed\n"));
     return NS_ERROR_CMS_VERIFY_NOT_SIGNED;
   } 
 
   cinfo = NSS_CMSMessage_ContentLevel(m_cmsMsg, 0);
   if (cinfo) {
+    // I don't like this hard cast. We should check in some way, that we really have this type.
     sigd = (NSSCMSSignedData*)NSS_CMSContentInfo_GetContent(cinfo);
   }
   
   if (!sigd) {
-    PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("nsCMSMessage::VerifyDetachedSignature - no content info\n"));
+    PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("nsCMSMessage::CommonVerifySignature - no content info\n"));
     rv = NS_ERROR_CMS_VERIFY_NO_CONTENT_INFO;
     goto loser;
   }
 
-  if (NSS_CMSSignedData_SetDigestValue(sigd, SEC_OID_SHA1, &digest)) {
-    PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("nsCMSMessage::VerifyDetachedSignature - bad digest\n"));
-    rv = NS_ERROR_CMS_VERIFY_BAD_DIGEST;
-    goto loser;
+  if (aDigestData && aDigestDataLen)
+  {
+    SECItem digest;
+    digest.data = aDigestData;
+    digest.len = aDigestDataLen;
+
+    if (NSS_CMSSignedData_SetDigestValue(sigd, SEC_OID_SHA1, &digest)) {
+      PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("nsCMSMessage::CommonVerifySignature - bad digest\n"));
+      rv = NS_ERROR_CMS_VERIFY_BAD_DIGEST;
+      goto loser;
+    }
   }
 
   // Import certs. Note that import failure is not a signature verification failure. //
-  if (NSS_CMSSignedData_ImportCerts(sigd, CERT_GetDefaultCertDB(), certUsageEmailSigner, PR_TRUE) != SECSuccess) {
-    PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("nsCMSMessage::VerifyDetachedSignature - can not import certs\n"));
+  if (NSS_CMSSignedData_ImportCerts(sigd, CERT_GetDefaultCertDB(), certUsageEmailRecipient, PR_TRUE) != SECSuccess) {
+    PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("nsCMSMessage::CommonVerifySignature - can not import certs\n"));
   }
 
   nsigners = NSS_CMSSignedData_SignerInfoCount(sigd);
@@ -230,42 +369,42 @@ NS_IMETHODIMP nsCMSMessage::VerifyDetachedSignature(unsigned char* aDigestData,
 
   // We verify the first signer info,  only //
   if (NSS_CMSSignedData_VerifySignerInfo(sigd, 0, CERT_GetDefaultCertDB(), certUsageEmailSigner) != SECSuccess) {
-    PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("nsCMSMessage::VerifyDetachedSignature - unable to verify signature\n"));
+    PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("nsCMSMessage::CommonVerifySignature - unable to verify signature\n"));
 
     if (NSSCMSVS_SigningCertNotFound == si->verificationStatus) {
-      PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("nsCMSMessage::VerifyDetachedSignature - signing cert not found\n"));
+      PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("nsCMSMessage::CommonVerifySignature - signing cert not found\n"));
       rv = NS_ERROR_CMS_VERIFY_NOCERT;
     }
     else if(NSSCMSVS_SigningCertNotTrusted == si->verificationStatus) {
-      PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("nsCMSMessage::VerifyDetachedSignature - signing cert not trusted\n"));
+      PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("nsCMSMessage::CommonVerifySignature - signing cert not trusted\n"));
       rv = NS_ERROR_CMS_VERIFY_UNTRUSTED;
     }
     else if(NSSCMSVS_Unverified == si->verificationStatus) {
-      PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("nsCMSMessage::VerifyDetachedSignature - can not verify\n"));
+      PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("nsCMSMessage::CommonVerifySignature - can not verify\n"));
       rv = NS_ERROR_CMS_VERIFY_ERROR_UNVERIFIED;
     }
     else if(NSSCMSVS_ProcessingError == si->verificationStatus) {
-      PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("nsCMSMessage::VerifyDetachedSignature - processing error\n"));
+      PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("nsCMSMessage::CommonVerifySignature - processing error\n"));
       rv = NS_ERROR_CMS_VERIFY_ERROR_PROCESSING;
     }
     else if(NSSCMSVS_BadSignature == si->verificationStatus) {
-      PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("nsCMSMessage::VerifyDetachedSignature - bad signature\n"));
+      PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("nsCMSMessage::CommonVerifySignature - bad signature\n"));
       rv = NS_ERROR_CMS_VERIFY_BAD_SIGNATURE;
     }
     else if(NSSCMSVS_DigestMismatch == si->verificationStatus) {
-      PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("nsCMSMessage::VerifyDetachedSignature - digest mismatch\n"));
+      PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("nsCMSMessage::CommonVerifySignature - digest mismatch\n"));
       rv = NS_ERROR_CMS_VERIFY_DIGEST_MISMATCH;
     }
     else if(NSSCMSVS_SignatureAlgorithmUnknown == si->verificationStatus) {
-      PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("nsCMSMessage::VerifyDetachedSignature - algo unknown\n"));
+      PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("nsCMSMessage::CommonVerifySignature - algo unknown\n"));
       rv = NS_ERROR_CMS_VERIFY_UNKNOWN_ALGO;
     }
     else if(NSSCMSVS_SignatureAlgorithmUnsupported == si->verificationStatus) {
-      PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("nsCMSMessage::VerifyDetachedSignature - algo not supported\n"));
+      PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("nsCMSMessage::CommonVerifySignature - algo not supported\n"));
       rv = NS_ERROR_CMS_VERIFY_UNSUPPORTED_ALGO;
     }
     else if(NSSCMSVS_MalformedSignature == si->verificationStatus) {
-      PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("nsCMSMessage::VerifyDetachedSignature - malformed signature\n"));
+      PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("nsCMSMessage::CommonVerifySignature - malformed signature\n"));
       rv = NS_ERROR_CMS_VERIFY_MALFORMED_SIGNATURE;
     }
 
@@ -274,7 +413,7 @@ NS_IMETHODIMP nsCMSMessage::VerifyDetachedSignature(unsigned char* aDigestData,
 
   // Save the profile. Note that save import failure is not a signature verification failure. //
   if (NSS_SMIMESignerInfo_SaveSMIMEProfile(si) != SECSuccess) {
-    PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("nsCMSMessage::VerifyDetachedSignature - unable to save smime profile\n"));
+    PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("nsCMSMessage::CommonVerifySignature - unable to save smime profile\n"));
   }
 
   rv = NS_OK;
@@ -282,42 +421,162 @@ loser:
   return rv;
 }
 
-NS_IMETHODIMP nsCMSMessage::CreateEncrypted(nsISupportsArray * aRecipientCerts)
+class nsZeroTerminatedCertArray : public nsNSSShutDownObject
 {
+public:
+  nsZeroTerminatedCertArray()
+  :mCerts(nsnull), mPoolp(nsnull), mSize(0)
+  {
+  }
+  
+  ~nsZeroTerminatedCertArray()
+  {
+    nsNSSShutDownPreventionLock locker;
+    if (isAlreadyShutDown())
+      return;
+
+    destructorSafeDestroyNSSReference();
+    shutdown(calledFromObject);
+  }
+
+  void virtualDestroyNSSReference()
+  {
+    destructorSafeDestroyNSSReference();
+  }
+
+  void destructorSafeDestroyNSSReference()
+  {
+    if (isAlreadyShutDown())
+      return;
+
+    if (mCerts)
+    {
+      for (PRUint32 i=0; i < mSize; i++) {
+        if (mCerts[i]) {
+          CERT_DestroyCertificate(mCerts[i]);
+        }
+      }
+    }
+
+    if (mPoolp)
+      PORT_FreeArena(mPoolp, PR_FALSE);
+  }
+
+  PRBool allocate(PRUint32 count)
+  {
+    // only allow allocation once
+    if (mPoolp)
+      return PR_FALSE;
+  
+    mSize = count;
+
+    if (!mSize)
+      return PR_FALSE;
+  
+    mPoolp = PORT_NewArena(1024);
+    if (!mPoolp)
+      return PR_FALSE;
+
+    mCerts = (CERTCertificate**)PORT_ArenaZAlloc(
+      mPoolp, (count+1)*sizeof(CERTCertificate*));
+
+    if (!mCerts)
+      return PR_FALSE;
+
+    // null array, including zero termination
+    for (PRUint32 i = 0; i < count+1; i++) {
+      mCerts[i] = nsnull;
+    }
+
+    return PR_TRUE;
+  }
+  
+  void set(PRUint32 i, CERTCertificate *c)
+  {
+    nsNSSShutDownPreventionLock locker;
+    if (isAlreadyShutDown())
+      return;
+
+    if (i >= mSize)
+      return;
+    
+    if (mCerts[i]) {
+      CERT_DestroyCertificate(mCerts[i]);
+    }
+    
+    mCerts[i] = CERT_DupCertificate(c);
+  }
+  
+  CERTCertificate *get(PRUint32 i)
+  {
+    nsNSSShutDownPreventionLock locker;
+    if (isAlreadyShutDown())
+      return nsnull;
+
+    if (i >= mSize)
+      return nsnull;
+    
+    return CERT_DupCertificate(mCerts[i]);
+  }
+
+  CERTCertificate **getRawArray()
+  {
+    nsNSSShutDownPreventionLock locker;
+    if (isAlreadyShutDown())
+      return nsnull;
+
+    return mCerts;
+  }
+
+private:
+  CERTCertificate **mCerts;
+  PLArenaPool *mPoolp;
+  PRUint32 mSize;
+};
+
+NS_IMETHODIMP nsCMSMessage::CreateEncrypted(nsIArray * aRecipientCerts)
+{
+  nsNSSShutDownPreventionLock locker;
+  if (isAlreadyShutDown())
+    return NS_ERROR_NOT_AVAILABLE;
+
   PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("nsCMSMessage::CreateEncrypted\n"));
   NSSCMSContentInfo *cinfo;
   NSSCMSEnvelopedData *envd;
   NSSCMSRecipientInfo *recipientInfo;
-  CERTCertificate **recipientCerts;
-  PLArenaPool *tmpPoolp = nsnull;
+  nsZeroTerminatedCertArray recipientCerts;
   SECOidTag bulkAlgTag;
-  int keySize, i;
+  int keySize;
+  PRUint32 i;
   nsNSSCertificate *nssRecipientCert;
   nsresult rv = NS_ERROR_FAILURE;
 
   // Check the recipient certificates //
   PRUint32 recipientCertCount;
-  aRecipientCerts->Count(&recipientCertCount);
+  aRecipientCerts->GetLength(&recipientCertCount);
   PR_ASSERT(recipientCertCount > 0);
 
-  if ((tmpPoolp = PORT_NewArena(1024)) == nsnull) {
-    goto loser;
-  }
-
-  if ((recipientCerts = (CERTCertificate**)PORT_ArenaZAlloc(tmpPoolp,
-                                           (recipientCertCount+1)*sizeof(CERTCertificate*)))
-                                           == nsnull) {
+  if (!recipientCerts.allocate(recipientCertCount)) {
     goto loser;
   }
 
   for (i=0; i<recipientCertCount; i++) {
-    nssRecipientCert = NS_STATIC_CAST(nsNSSCertificate*, aRecipientCerts->ElementAt(i));
-    recipientCerts[i] = nssRecipientCert->GetCert();
-  }
-  recipientCerts[i] = nsnull;
+    nsCOMPtr<nsIX509Cert> x509cert = do_QueryElementAt(aRecipientCerts, i);
 
+    nssRecipientCert = 
+      NS_STATIC_CAST(nsNSSCertificate*, 
+                     NS_STATIC_CAST(nsIX509Cert*, x509cert));
+
+    if (!nssRecipientCert)
+      return NS_ERROR_FAILURE;
+
+    CERTCertificate *c = nssRecipientCert->GetCert();
+    CERTCertificateCleaner rcCleaner(c);
+    recipientCerts.set(i, c);
+  }
+  
   // Find a bulk key algorithm //
-  if (NSS_SMIMEUtil_FindBulkAlgForRecipients(recipientCerts, &bulkAlgTag,
+  if (NSS_SMIMEUtil_FindBulkAlgForRecipients(recipientCerts.getRawArray(), &bulkAlgTag,
                                             &keySize) != SECSuccess) {
     PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("nsCMSMessage::CreateEncrypted - can't find bulk alg for recipients\n"));
     rv = NS_ERROR_CMS_ENCRYPT_NO_BULK_ALG;
@@ -349,8 +608,10 @@ NS_IMETHODIMP nsCMSMessage::CreateEncrypted(nsISupportsArray * aRecipientCerts)
   }
 
   // Create and attach recipient information //
-  for (i=0; recipientCerts[i] != nsnull; i++) {
-    if ((recipientInfo = NSS_CMSRecipientInfo_Create(m_cmsMsg, recipientCerts[i])) == nsnull) {
+  for (i=0; i < recipientCertCount; i++) {
+    CERTCertificate *rc = recipientCerts.get(i);
+    CERTCertificateCleaner rcCleaner(rc);
+    if ((recipientInfo = NSS_CMSRecipientInfo_Create(m_cmsMsg, rc)) == nsnull) {
       PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("nsCMSMessage::CreateEncrypted - can't create recipient info\n"));
       goto loser;
     }
@@ -360,35 +621,41 @@ NS_IMETHODIMP nsCMSMessage::CreateEncrypted(nsISupportsArray * aRecipientCerts)
     }
   }
 
-  if (tmpPoolp) {
-    PORT_FreeArena(tmpPoolp, PR_FALSE);
-  }
-
   return NS_OK;
 loser:
   if (m_cmsMsg) {
     NSS_CMSMessage_Destroy(m_cmsMsg);
     m_cmsMsg = nsnull;
   }
-  if (tmpPoolp) {
-    PORT_FreeArena(tmpPoolp, PR_FALSE);
-  }
 
   return rv;
 }
 
 NS_IMETHODIMP nsCMSMessage::CreateSigned(nsIX509Cert* aSigningCert, nsIX509Cert* aEncryptCert, unsigned char* aDigestData, PRUint32 aDigestDataLen)
 {
+  nsNSSShutDownPreventionLock locker;
+  if (isAlreadyShutDown())
+    return NS_ERROR_NOT_AVAILABLE;
+
   PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("nsCMSMessage::CreateSigned\n"));
   NSSCMSContentInfo *cinfo;
   NSSCMSSignedData *sigd;
   NSSCMSSignerInfo *signerinfo;
-  CERTCertificate *scert, *ecert;
+  CERTCertificate *scert = nsnull, *ecert = nsnull;
   nsresult rv = NS_ERROR_FAILURE;
 
   /* Get the certs */
   scert = NS_STATIC_CAST(nsNSSCertificate*, aSigningCert)->GetCert();
-  ecert = NS_STATIC_CAST(nsNSSCertificate*, aEncryptCert)->GetCert();
+  if (!scert) {
+    return NS_ERROR_FAILURE;
+  }
+
+  if (aEncryptCert) {
+    ecert = NS_STATIC_CAST(nsNSSCertificate*, aEncryptCert)->GetCert();
+  }
+
+  CERTCertificateCleaner ecertCleaner(ecert);
+  CERTCertificateCleaner scertCleaner(scert);
 
   /*
    * create the message object
@@ -451,15 +718,25 @@ NS_IMETHODIMP nsCMSMessage::CreateSigned(nsIX509Cert* aSigningCert, nsIX509Cert*
     goto loser;
   }
 
-  if (NSS_CMSSignerInfo_AddSMIMEEncKeyPrefs(signerinfo, ecert, 
-	                                  CERT_GetDefaultCertDB())
-	!= SECSuccess) {
-    PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("nsCMSMessage::CreateSigned - can't add smime enc key prefs\n"));
-    goto loser;
-  }
-  if (NSS_CMSSignedData_AddCertificate(sigd, ecert) != SECSuccess) {
-    PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("nsCMSMessage::CreateSigned - can't add own encryption certificate\n"));
-    goto loser;
+  if (ecert) {
+    if (NSS_CMSSignerInfo_AddSMIMEEncKeyPrefs(signerinfo, ecert, 
+	                                    CERT_GetDefaultCertDB())
+	  != SECSuccess) {
+      PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("nsCMSMessage::CreateSigned - can't add smime enc key prefs\n"));
+      goto loser;
+    }
+
+    if (NSS_CMSSignerInfo_AddMSSMIMEEncKeyPrefs(signerinfo, ecert, 
+	                                    CERT_GetDefaultCertDB())
+	  != SECSuccess) {
+      PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("nsCMSMessage::CreateSigned - can't add MS smime enc key prefs\n"));
+      goto loser;
+    }
+
+    if (NSS_CMSSignedData_AddCertificate(sigd, ecert) != SECSuccess) {
+      PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("nsCMSMessage::CreateSigned - can't add own encryption certificate\n"));
+      goto loser;
+    }
   }
 
   if (NSS_CMSSignedData_AddSignerInfo(sigd, signerinfo) != SECSuccess) {
@@ -492,17 +769,43 @@ loser:
 NS_IMPL_THREADSAFE_ISUPPORTS1(nsCMSDecoder, nsICMSDecoder)
 
 nsCMSDecoder::nsCMSDecoder()
+: m_dcx(nsnull)
 {
-  NS_INIT_ISUPPORTS();
 }
 
 nsCMSDecoder::~nsCMSDecoder()
 {
+  nsNSSShutDownPreventionLock locker;
+  if (isAlreadyShutDown())
+    return;
+
+  destructorSafeDestroyNSSReference();
+  shutdown(calledFromObject);
+}
+
+void nsCMSDecoder::virtualDestroyNSSReference()
+{
+  destructorSafeDestroyNSSReference();
+}
+
+void nsCMSDecoder::destructorSafeDestroyNSSReference()
+{
+  if (isAlreadyShutDown())
+    return;
+
+  if (m_dcx) {
+    NSS_CMSDecoder_Cancel(m_dcx);
+    m_dcx = nsnull;
+  }
 }
 
 /* void start (in NSSCMSContentCallback cb, in voidPtr arg); */
 NS_IMETHODIMP nsCMSDecoder::Start(NSSCMSContentCallback cb, void * arg)
 {
+  nsNSSShutDownPreventionLock locker;
+  if (isAlreadyShutDown())
+    return NS_ERROR_NOT_AVAILABLE;
+
   PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("nsCMSDecoder::Start\n"));
   m_ctx = new PipUIContext();
 
@@ -517,6 +820,10 @@ NS_IMETHODIMP nsCMSDecoder::Start(NSSCMSContentCallback cb, void * arg)
 /* void update (in string bug, in long len); */
 NS_IMETHODIMP nsCMSDecoder::Update(const char *buf, PRInt32 len)
 {
+  nsNSSShutDownPreventionLock locker;
+  if (isAlreadyShutDown())
+    return NS_ERROR_NOT_AVAILABLE;
+
   PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("nsCMSDecoder::Update\n"));
   NSS_CMSDecoder_Update(m_dcx, (char *)buf, len);
   return NS_OK;
@@ -525,12 +832,21 @@ NS_IMETHODIMP nsCMSDecoder::Update(const char *buf, PRInt32 len)
 /* void finish (); */
 NS_IMETHODIMP nsCMSDecoder::Finish(nsICMSMessage ** aCMSMsg)
 {
+  nsNSSShutDownPreventionLock locker;
+  if (isAlreadyShutDown())
+    return NS_ERROR_NOT_AVAILABLE;
+
   PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("nsCMSDecoder::Finish\n"));
   NSSCMSMessage *cmsMsg;
   cmsMsg = NSS_CMSDecoder_Finish(m_dcx);
+  m_dcx = nsnull;
   if (cmsMsg) {
-    nsCOMPtr<nsICMSMessage> msg = new nsCMSMessage(cmsMsg);
-    *aCMSMsg = msg;
+    nsCMSMessage *obj = new nsCMSMessage(cmsMsg);
+    // The NSS object cmsMsg still carries a reference to the context
+    // we gave it on construction.
+    // Make sure the context will live long enough.
+    obj->referenceContext(m_ctx);
+    *aCMSMsg = obj;
     NS_ADDREF(*aCMSMsg);
   }
   return NS_OK;
@@ -539,17 +855,42 @@ NS_IMETHODIMP nsCMSDecoder::Finish(nsICMSMessage ** aCMSMsg)
 NS_IMPL_THREADSAFE_ISUPPORTS1(nsCMSEncoder, nsICMSEncoder)
 
 nsCMSEncoder::nsCMSEncoder()
+: m_ecx(nsnull)
 {
-  NS_INIT_ISUPPORTS();
 }
 
 nsCMSEncoder::~nsCMSEncoder()
 {
+  nsNSSShutDownPreventionLock locker;
+  if (isAlreadyShutDown())
+    return;
+
+  destructorSafeDestroyNSSReference();
+  shutdown(calledFromObject);
+}
+
+void nsCMSEncoder::virtualDestroyNSSReference()
+{
+  destructorSafeDestroyNSSReference();
+}
+
+void nsCMSEncoder::destructorSafeDestroyNSSReference()
+{
+  nsNSSShutDownPreventionLock locker;
+  if (isAlreadyShutDown())
+    return;
+
+  if (m_ecx)
+    NSS_CMSEncoder_Cancel(m_ecx);
 }
 
 /* void start (); */
 NS_IMETHODIMP nsCMSEncoder::Start(nsICMSMessage *aMsg, NSSCMSContentCallback cb, void * arg)
 {
+  nsNSSShutDownPreventionLock locker;
+  if (isAlreadyShutDown())
+    return NS_ERROR_NOT_AVAILABLE;
+
   PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("nsCMSEncoder::Start\n"));
   nsCMSMessage *cmsMsg = NS_STATIC_CAST(nsCMSMessage*, aMsg);
   m_ctx = new PipUIContext();
@@ -565,8 +906,12 @@ NS_IMETHODIMP nsCMSEncoder::Start(nsICMSMessage *aMsg, NSSCMSContentCallback cb,
 /* void update (in string aBuf, in long aLen); */
 NS_IMETHODIMP nsCMSEncoder::Update(const char *aBuf, PRInt32 aLen)
 {
+  nsNSSShutDownPreventionLock locker;
+  if (isAlreadyShutDown())
+    return NS_ERROR_NOT_AVAILABLE;
+
   PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("nsCMSEncoder::Update\n"));
-  if (NSS_CMSEncoder_Update(m_ecx, aBuf, aLen) != SECSuccess) {
+  if (!m_ecx || NSS_CMSEncoder_Update(m_ecx, aBuf, aLen) != SECSuccess) {
     PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("nsCMSEncoder::Update - can't update encoder\n"));
     return NS_ERROR_FAILURE;
   }
@@ -576,17 +921,27 @@ NS_IMETHODIMP nsCMSEncoder::Update(const char *aBuf, PRInt32 aLen)
 /* void finish (); */
 NS_IMETHODIMP nsCMSEncoder::Finish()
 {
+  nsNSSShutDownPreventionLock locker;
+  if (isAlreadyShutDown())
+    return NS_ERROR_NOT_AVAILABLE;
+
+  nsresult rv = NS_OK;
   PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("nsCMSEncoder::Finish\n"));
-  if (NSS_CMSEncoder_Finish(m_ecx) != SECSuccess) {
+  if (!m_ecx || NSS_CMSEncoder_Finish(m_ecx) != SECSuccess) {
     PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("nsCMSEncoder::Finish - can't finish encoder\n"));
-    return NS_ERROR_FAILURE;
+    rv = NS_ERROR_FAILURE;
   }
-  return NS_OK;
+  m_ecx = nsnull;
+  return rv;
 }
 
 /* void encode (in nsICMSMessage aMsg); */
 NS_IMETHODIMP nsCMSEncoder::Encode(nsICMSMessage *aMsg)
 {
+  nsNSSShutDownPreventionLock locker;
+  if (isAlreadyShutDown())
+    return NS_ERROR_NOT_AVAILABLE;
+
   PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("nsCMSEncoder::Encode\n"));
   return NS_ERROR_NOT_IMPLEMENTED;
 }
