@@ -1,4 +1,5 @@
-/*
+/* alg1485.c - implementation of RFCs 1485, 1779 and 2253.
+ *
  * The contents of this file are subject to the Mozilla Public
  * License Version 1.1 (the "License"); you may not use this file
  * except in compliance with the License. You may obtain a copy of
@@ -31,30 +32,59 @@
  * GPL.
  */
 
+#include "prprf.h"
 #include "cert.h"
 #include "xconst.h"
 #include "genname.h"
 #include "secitem.h"
 #include "secerr.h"
 
+/* for better RFC 2253 compliance. */
+#define NSS_STRICT_RFC_2253_VALUES_ONLY 1
+
 struct NameToKind {
-    char *name;
-    SECOidTag kind;
+    const char * name;
+    unsigned int maxLen; /* max bytes in UTF8 encoded string value */
+    SECOidTag    kind;
 };
 
-static struct NameToKind name2kinds[] = {
-    { "CN",   		SEC_OID_AVA_COMMON_NAME, 		},
-    { "ST",   		SEC_OID_AVA_STATE_OR_PROVINCE, 		},
-    { "OU",   		SEC_OID_AVA_ORGANIZATIONAL_UNIT_NAME,	},
-    { "DC",   		SEC_OID_AVA_DC,	},
-    { "C",    		SEC_OID_AVA_COUNTRY_NAME, 		},
-    { "O",    		SEC_OID_AVA_ORGANIZATION_NAME, 		},
-    { "L",    		SEC_OID_AVA_LOCALITY, 			},
-    { "dnQualifier",	SEC_OID_AVA_DN_QUALIFIER, 		},
-    { "E",    		SEC_OID_PKCS9_EMAIL_ADDRESS, 		},
-    { "UID",  		SEC_OID_RFC1274_UID, 			},
-    { "MAIL", 		SEC_OID_RFC1274_MAIL, 			},
-    { 0,      		SEC_OID_UNKNOWN 			},
+/* Add new entries to this table, and maybe to function CERT_ParseRFC1485AVA */
+static const struct NameToKind name2kinds[] = {
+/* keywords given in RFC 2253 */
+    { "CN",                      64, SEC_OID_AVA_COMMON_NAME              },
+    { "L",                      128, SEC_OID_AVA_LOCALITY                 },
+    { "ST",                     128, SEC_OID_AVA_STATE_OR_PROVINCE        },
+    { "O",                       64, SEC_OID_AVA_ORGANIZATION_NAME        },
+    { "OU",                      64, SEC_OID_AVA_ORGANIZATIONAL_UNIT_NAME },
+    { "C",                        2, SEC_OID_AVA_COUNTRY_NAME             },
+    { "STREET",                 128, SEC_OID_AVA_STREET_ADDRESS           },
+    { "DC",                     128, SEC_OID_AVA_DC                       },
+    { "UID",                    256, SEC_OID_RFC1274_UID                  },
+
+#ifndef NSS_STRICT_RFC_2253_KEYWORDS_ONLY
+/* NSS legacy keywords */
+    { "dnQualifier",          32767, SEC_OID_AVA_DN_QUALIFIER             },
+    { "E",                      128, SEC_OID_PKCS9_EMAIL_ADDRESS          },
+    { "MAIL",                   256, SEC_OID_RFC1274_MAIL                 },
+
+#ifndef NSS_LEGACY_KEYWORDS_ONLY
+/* values from draft-ietf-ldapbis-user-schema-05 */
+    { "SN",                      64, SEC_OID_AVA_SURNAME                  },
+    { "serialNumber",            64, SEC_OID_AVA_SERIAL_NUMBER            },
+    { "title",                   64, SEC_OID_AVA_TITLE                    },
+    { "postalAddress",          128, SEC_OID_AVA_POSTAL_ADDRESS           },
+    { "postalCode",              40, SEC_OID_AVA_POSTAL_CODE              },
+    { "postOfficeBox",           40, SEC_OID_AVA_POST_OFFICE_BOX          },
+    { "givenName",               64, SEC_OID_AVA_GIVEN_NAME               },
+    { "initials",                64, SEC_OID_AVA_INITIALS                 },
+    { "generationQualifier",     64, SEC_OID_AVA_GENERATION_QUALIFIER     },
+    { "houseIdentifier",         64, SEC_OID_AVA_HOUSE_IDENTIFIER         },
+#if 0 /* removed.  Not yet in any IETF draft or RFC. */
+    { "pseudonym",               64, SEC_OID_AVA_PSEUDONYM                },
+#endif
+#endif
+#endif
+    { 0,                        256, SEC_OID_UNKNOWN                      }
 };
 
 #define C_DOUBLE_QUOTE '\042'
@@ -73,126 +103,6 @@ static struct NameToKind name2kinds[] = {
      ((c) == ';') || ((c) == C_BACKSLASH))
 
 
-
-
-
-
-
-#if 0
-/*
-** Find the start and end of a <string>. Strings can be wrapped in double
-** quotes to protect special characters.
-*/
-static int BracketThing(char **startp, char *end, char *result)
-{
-    char *start = *startp;
-    char c;
-
-    /* Skip leading white space */
-    while (start < end) {
-	c = *start++;
-	if (!OPTIONAL_SPACE(c)) {
-	    start--;
-	    break;
-	}
-    }
-    if (start == end) return 0;
-
-    switch (*start) {
-      case '#':
-	/* Process hex thing */
-	start++;
-	*startp = start;
-	while (start < end) {
-	    c = *start++;
-	    if (((c >= '0') && (c <= '9')) ||
-		((c >= 'a') && (c <= 'f')) ||
-		((c >= 'A') && (c <= 'F'))) {
-		continue;
-	    }
-	    break;
-	}
-	rv = IS_HEX;
-	break;
-
-      case C_DOUBLE_QUOTE:
-	start++;
-	*startp = start;
-	while (start < end) {
-	    c = *start++;
-	    if (c == C_DOUBLE_QUOTE) {
-		break;
-	    }
-	    *result++ = c;
-	}
-	rv = IS_STRING;
-	break;
-
-      default:
-	while (start < end) {
-	    c = *start++;
-	    if (SPECIAL_CHAR(c)) {
-		start--;
-		break;
-	    }
-	    *result++ = c;
-	}
-	rv = IS_STRING;
-	break;
-    }
-
-    /* Terminate result string */
-    *result = 0;
-    return start;
-}
-
-static char *BracketSomething(char **startp, char* end, int spacesOK)
-{
-    char *start = *startp;
-    char c;
-    int stopAtDQ;
-
-    /* Skip leading white space */
-    while (start < end) {
-	c = *start;
-	if (!OPTIONAL_SPACE(c)) {
-	    break;
-	}
-	start++;
-    }
-    if (start == end) return 0;
-    stopAtDQ = 0;
-    if (*start == C_DOUBLE_QUOTE) {
-	stopAtDQ = 1;
-    }
-
-    /*
-    ** Find the end of the something. The something is terminated most of
-    ** the time by a space. However, if spacesOK is true then it is
-    ** terminated by a special character only.
-    */
-    *startp = start;
-    while (start < end) {
-	c = *start;
-	if (stopAtDQ) {
-	    if (c == C_DOUBLE_QUOTE) {
-		*start = ' ';
-		break;
-	    }
-	} else {
-	if (SPECIAL_CHAR(c)) {
-	    break;
-	}
-	if (!spacesOK && OPTIONAL_SPACE(c)) {
-	    break;
-	}
-	}
-	start++;
-    }
-    return start;
-}
-#endif
-
 #define IS_PRINTABLE(c)						\
     ((((c) >= 'a') && ((c) <= 'z')) ||				\
      (((c) >= 'A') && ((c) <= 'Z')) ||				\
@@ -206,6 +116,17 @@ static char *BracketSomething(char **startp, char* end, int spacesOK)
      ((c) == '=') ||						\
      ((c) == '?'))
 
+int
+cert_AVAOidTagToMaxLen(SECOidTag tag)
+{
+    const struct NameToKind *n2k = name2kinds;
+
+    while (n2k->kind != tag && n2k->kind != SEC_OID_UNKNOWN) {
+	++n2k;
+    }
+    return (n2k->kind != SEC_OID_UNKNOWN) ? n2k->maxLen : -1;
+}
+
 static PRBool
 IsPrintable(unsigned char *data, unsigned len)
 {
@@ -387,7 +308,7 @@ CERT_ParseRFC1485AVA(PRArenaPool *arena, char **pbp, char *endptr,
 		    PRBool singleAVA) 
 {
     CERTAVA *a;
-    struct NameToKind *n2k;
+    const struct NameToKind *n2k;
     int vt;
     int valLen;
     char *bp;
@@ -437,7 +358,8 @@ CERT_ParseRFC1485AVA(PRArenaPool *arena, char **pbp, char *endptr,
                 } else if (Is7Bit((unsigned char *)valBuf, valLen)) {
                     vt = SEC_ASN1_T61_STRING;
 		} else {
-		    vt = SEC_ASN1_UNIVERSAL_STRING;
+		    /* according to RFC3280, UTF8String is preferred encoding */
+		    vt = SEC_ASN1_UTF8_STRING;
 		}
 	    }
 	    a = CERT_CreateAVA(arena, n2k->kind, vt, (char *) valBuf);
@@ -520,29 +442,42 @@ CERT_AsciiToName(char *string)
 
 /************************************************************************/
 
+typedef struct stringBufStr {
+    char *buffer;
+    unsigned offset;
+    unsigned size;
+} stringBuf;
+
+#define DEFAULT_BUFFER_SIZE 200
+
 static SECStatus
-AppendStr(char **bufp, unsigned *buflenp, char *str)
+AppendStr(stringBuf *bufp, char *str)
 {
     char *buf;
     unsigned bufLen, bufSize, len;
+    int size = 0;
 
     /* Figure out how much to grow buf by (add in the '\0') */
-    buf = *bufp;
-    bufLen = *buflenp;
+    buf = bufp->buffer;
+    bufLen = bufp->offset;
     len = PORT_Strlen(str);
     bufSize = bufLen + len;
-    if (buf) {
-	buf = (char*) PORT_Realloc(buf, bufSize);
-    } else {
+    if (!buf) {
 	bufSize++;
-	buf = (char*) PORT_Alloc(bufSize);
+	size = PR_MAX(DEFAULT_BUFFER_SIZE,bufSize*2);
+	buf = (char *) PORT_Alloc(size);
+	bufp->size = size;
+    } else if (bufp->size < bufSize) {
+	size = bufSize*2;
+	buf =(char *) PORT_Realloc(buf,size);
+	bufp->size = size;
     }
     if (!buf) {
 	PORT_SetError(SEC_ERROR_NO_MEMORY);
 	return SECFailure;
     }
-    *bufp = buf;
-    *buflenp = bufSize;
+    bufp->buffer = buf;
+    bufp->offset = bufSize;
 
     /* Concatenate str onto buf */
     buf = buf + bufLen;
@@ -557,12 +492,14 @@ CERT_RFC1485_EscapeAndQuote(char *dst, int dstlen, char *src, int srclen)
     int i, reqLen=0;
     char *d = dst;
     PRBool needsQuoting = PR_FALSE;
+    char lastC = 0;
     
     /* need to make an initial pass to determine if quoting is needed */
     for (i = 0; i < srclen; i++) {
 	char c = src[i];
 	reqLen++;
-	if (SPECIAL_CHAR(c)) {
+	if (!needsQuoting && (SPECIAL_CHAR(c) ||
+	    (OPTIONAL_SPACE(c) && OPTIONAL_SPACE(lastC)))) {
 	    /* entirety will need quoting */
 	    needsQuoting = PR_TRUE;
 	}
@@ -570,9 +507,10 @@ CERT_RFC1485_EscapeAndQuote(char *dst, int dstlen, char *src, int srclen)
 	    /* this char will need escaping */
 	    reqLen++;
 	}
+	lastC = c;
     }
     /* if it begins or ends in optional space it needs quoting */
-    if (srclen > 0 &&
+    if (!needsQuoting && srclen > 0 && 
 	(OPTIONAL_SPACE(src[srclen-1]) || OPTIONAL_SPACE(src[0]))) {
 	needsQuoting = PR_TRUE;
     }
@@ -602,109 +540,216 @@ CERT_RFC1485_EscapeAndQuote(char *dst, int dstlen, char *src, int srclen)
     return SECSuccess;
 }
 
-static SECStatus
-AppendAVA(char **bufp, unsigned *buflenp, CERTAVA *ava)
+/* convert an OID to dotted-decimal representation */
+/* Returns a string that must be freed with PR_smprintf_free(), */
+char *
+CERT_GetOidString(const SECItem *oid)
 {
-    char *tagName;
-    char tmpBuf[384];
+    PRUint8 *end;
+    PRUint8 *d;
+    PRUint8 *e;
+    char *a         = NULL;
+    char *b;
+
+#define MAX_OID_LEN 1024 /* bytes */
+
+    if (oid->len > MAX_OID_LEN) {
+    	PORT_SetError(SEC_ERROR_INPUT_LEN);
+	return NULL;
+    }
+
+    /* d will point to the next sequence of bytes to decode */
+    d = (PRUint8 *)oid->data;
+    /* end points to one past the legitimate data */
+    end = &d[ oid->len ];
+
+    /*
+     * Check for our pseudo-encoded single-digit OIDs
+     */
+    if( (*d == 0x80) && (2 == oid->len) ) {
+	/* Funky encoding.  The second byte is the number */
+	a = PR_smprintf("%lu", (PRUint32)d[1]);
+	if( (char *)NULL == a ) {
+	    PORT_SetError(SEC_ERROR_NO_MEMORY);
+	    return (char *)NULL;
+	}
+	return a;
+    }
+
+    for( ; d < end; d = &e[1] ) {
+    
+	for( e = d; e < end; e++ ) {
+	    if( 0 == (*e & 0x80) ) {
+		break;
+	    }
+	}
+    
+	if( ((e-d) > 4) || (((e-d) == 4) && (*d & 0x70)) ) {
+	    /* More than a 32-bit number */
+	} else {
+	    PRUint32 n = 0;
+      
+	    switch( e-d ) {
+	    case 4:
+		n |= ((PRUint32)(e[-4] & 0x0f)) << 28;
+	    case 3:
+		n |= ((PRUint32)(e[-3] & 0x7f)) << 21;
+	    case 2:
+		n |= ((PRUint32)(e[-2] & 0x7f)) << 14;
+	    case 1:
+		n |= ((PRUint32)(e[-1] & 0x7f)) <<  7;
+	    case 0:
+		n |= ((PRUint32)(e[-0] & 0x7f))      ;
+	    }
+      
+	    if( (char *)NULL == a ) {
+		/* This is the first number.. decompose it */
+		PRUint32 one = PR_MIN(n/40, 2); /* never > 2 */
+		PRUint32 two = n - one * 40;
+        
+		a = PR_smprintf("OID.%lu.%lu", one, two);
+		if( (char *)NULL == a ) {
+		    PORT_SetError(SEC_ERROR_NO_MEMORY);
+		    return (char *)NULL;
+		}
+	    } else {
+		b = PR_smprintf("%s.%lu", a, n);
+		if( (char *)NULL == b ) {
+		    PR_smprintf_free(a);
+		    PORT_SetError(SEC_ERROR_NO_MEMORY);
+		    return (char *)NULL;
+		}
+        
+		PR_smprintf_free(a);
+		a = b;
+	    }
+	}
+    }
+
+    return a;
+}
+
+/* convert DER-encoded hex to a string */
+static SECItem *
+get_hex_string(SECItem *data)
+{
+    SECItem *rv;
+    unsigned int i, j;
+    static const char hex[] = { "0123456789ABCDEF" };
+
+    /* '#' + 2 chars per octet + terminator */
+    rv = SECITEM_AllocItem(NULL, NULL, data->len*2 + 2);
+    if (!rv) {
+	return NULL;
+    }
+    rv->data[0] = '#';
+    rv->len = 1 + 2 * data->len;
+    for (i=0; i<data->len; i++) {
+	j = data->data[i];
+	rv->data[2*i+1] = hex[j >> 4];
+	rv->data[2*i+2] = hex[j & 15];
+    }
+    rv->data[rv->len] = 0;
+    return rv;
+}
+
+static SECStatus
+AppendAVA(stringBuf *bufp, CERTAVA *ava)
+{
+    const struct NameToKind *n2k = name2kinds;
+    const char *tagName;
     unsigned len, maxLen;
     int tag;
     SECStatus rv;
     SECItem *avaValue = NULL;
+    char *unknownTag = NULL;
+    PRBool hexValue = PR_FALSE;
+    char tmpBuf[384];
 
     tag = CERT_GetAVATag(ava);
-    switch (tag) {
-      case SEC_OID_AVA_COUNTRY_NAME:
-	tagName = "C";
-	maxLen = 2;
-	break;
-      case SEC_OID_AVA_ORGANIZATION_NAME:
-	tagName = "O";
-	maxLen = 64;
-	break;
-      case SEC_OID_AVA_COMMON_NAME:
-	tagName = "CN";
-	maxLen = 64;
-	break;
-      case SEC_OID_AVA_LOCALITY:
-	tagName = "L";
-	maxLen = 128;
-	break;
-      case SEC_OID_AVA_STATE_OR_PROVINCE:
-	tagName = "ST";
-	maxLen = 128;
-	break;
-      case SEC_OID_AVA_ORGANIZATIONAL_UNIT_NAME:
-	tagName = "OU";
-	maxLen = 64;
-	break;
-      case SEC_OID_AVA_DC:
-	tagName = "DC";
-	maxLen = 128;
-	break;
-      case SEC_OID_AVA_DN_QUALIFIER:
-	tagName = "dnQualifier";
-	maxLen = 0x7fff;
-	break;
-      case SEC_OID_PKCS9_EMAIL_ADDRESS:
-	tagName = "E";
-	maxLen = 128;
-	break;
-      case SEC_OID_RFC1274_UID:
-	tagName = "UID";
-	maxLen = 256;
-	break;
-      case SEC_OID_RFC1274_MAIL:
-	tagName = "MAIL";
-	maxLen = 256;
-	break;
-      default:
-#if 0
-	PORT_SetError(SEC_ERROR_INVALID_AVA);
-	return SECFailure;
-#else
-	rv = AppendStr(bufp, buflenp, "ERR=Unknown AVA");
-	return rv;
-#endif
+    while (n2k->kind != tag && n2k->kind != SEC_OID_UNKNOWN) {
+        ++n2k;
     }
+    if (n2k->kind != SEC_OID_UNKNOWN) {
+        tagName = n2k->name;
+    } else {
+	/* handle unknown attribute types per RFC 2253 */
+	tagName = unknownTag = CERT_GetOidString(&ava->type);
+	if (!tagName)
+	    return SECFailure;
+    }
+    maxLen = n2k->maxLen;
 
+#ifdef NSS_STRICT_RFC_2253_VALUES_ONLY
+    if (!unknownTag)
+#endif
     avaValue = CERT_DecodeAVAValue(&ava->value);
     if(!avaValue) {
-	return SECFailure;
+	/* the attribute value is not recognized, get the hex value */
+	avaValue = get_hex_string(&ava->value);
+	if(!avaValue) {
+	    if (unknownTag) PR_smprintf_free(unknownTag);
+	    return SECFailure;
+	}
+	hexValue = PR_TRUE;
     }
 
     /* Check value length */
-    if (avaValue->len > maxLen) {
-	PORT_SetError(SEC_ERROR_INVALID_AVA);
-	return SECFailure;
+    if (avaValue->len > maxLen + 3) {  /* must be room for "..." */
+	/* avaValue is a UTF8 string, freshly allocated and returned to us 
+	** by CERT_DecodeAVAValue just above, so we can modify it here.
+	** See if we're in the middle of a multi-byte UTF8 character.
+	*/
+	while (((avaValue->data[maxLen] & 0xc0) == 0x80) && maxLen > 0) {
+	   maxLen--;
+	}
+	/* add elipsis to signify truncation. */
+	avaValue->data[maxLen++] = '.'; 
+	avaValue->data[maxLen++] = '.';
+	avaValue->data[maxLen++] = '.';
+	avaValue->data[maxLen]   = 0;
+	avaValue->len = maxLen;
     }
 
     len = PORT_Strlen(tagName);
+    if (len+1 > sizeof(tmpBuf)) {
+	if (unknownTag) PR_smprintf_free(unknownTag);
+	SECITEM_FreeItem(avaValue, PR_TRUE);
+	PORT_SetError(SEC_ERROR_OUTPUT_LEN);
+	return SECFailure;
+    }
     PORT_Memcpy(tmpBuf, tagName, len);
+    if (unknownTag) PR_smprintf_free(unknownTag);
     tmpBuf[len++] = '=';
     
-    /* escape and quote as necessary */
-    rv = CERT_RFC1485_EscapeAndQuote(tmpBuf+len, sizeof(tmpBuf)-len, 
+    /* escape and quote as necessary - don't quote hex strings */
+    if (hexValue) {
+        /* appent avaValue to tmpBuf */
+	if (avaValue->len + len + 1 > sizeof tmpBuf) {
+	    PORT_SetError(SEC_ERROR_LIBRARY_FAILURE);
+	    rv = SECFailure;
+    	} else {
+	    PORT_Strncpy(tmpBuf+len, (char *)avaValue->data, avaValue->len + 1);
+	    rv = SECSuccess;
+	}
+    } else 
+	rv = CERT_RFC1485_EscapeAndQuote(tmpBuf+len, sizeof(tmpBuf)-len, 
 		    		     (char *)avaValue->data, avaValue->len);
     SECITEM_FreeItem(avaValue, PR_TRUE);
     if (rv) return SECFailure;
     
-    rv = AppendStr(bufp, buflenp, tmpBuf);
+    rv = AppendStr(bufp, tmpBuf);
     return rv;
 }
 
 char *
 CERT_NameToAscii(CERTName *name)
 {
-    SECStatus rv;
     CERTRDN** rdns;
     CERTRDN** lastRdn;
     CERTRDN** rdn;
-    CERTAVA** avas;
-    CERTAVA* ava;
     PRBool first = PR_TRUE;
-    char *buf = NULL;
-    unsigned buflen = 0;
+    stringBuf strBuf = { NULL, 0, 0 };
     
     rdns = name->rdns;
     if (rdns == NULL) {
@@ -720,24 +765,35 @@ CERT_NameToAscii(CERTName *name)
      * Loop over name contents in _reverse_ RDN order appending to string
      */
     for (rdn = lastRdn; rdn >= rdns; rdn--) {
-	avas = (*rdn)->avas;
-	while ((ava = *avas++) != NULL) {
-	    /* Put in comma separator */
+	CERTAVA** avas = (*rdn)->avas;
+	CERTAVA* ava;
+	PRBool newRDN = PR_TRUE;
+
+	/* 
+	 * XXX Do we need to traverse the AVAs in reverse order, too?
+	 */
+	while (avas && (ava = *avas++) != NULL) {
+	    SECStatus rv;
+	    /* Put in comma or plus separator */
 	    if (!first) {
-		rv = AppendStr(&buf, &buflen, ", ");
+		/* Use of spaces is deprecated in RFC 2253. */
+		rv = AppendStr(&strBuf, newRDN ? "," : "+");
 		if (rv) goto loser;
 	    } else {
 		first = PR_FALSE;
 	    }
 	    
 	    /* Add in tag type plus value into buf */
-	    rv = AppendAVA(&buf, &buflen, ava);
+	    rv = AppendAVA(&strBuf, ava);
 	    if (rv) goto loser;
+	    newRDN = PR_FALSE;
 	}
     }
-    return buf;
-  loser:
-    PORT_Free(buf);
+    return strBuf.buffer;
+loser:
+    if (strBuf.buffer) {
+	PORT_Free(strBuf.buffer);
+    }
     return NULL;
 }
 
@@ -759,7 +815,7 @@ CERT_DerNameToAscii(SECItem *dername)
 	goto loser;
     }
     
-    rv = SEC_ASN1DecodeItem(arena, &name, CERT_NameTemplate, dername);
+    rv = SEC_QuickDERDecodeItem(arena, &name, CERT_NameTemplate, dername);
     
     if ( rv != SECSuccess ) {
 	goto loser;
@@ -776,27 +832,28 @@ loser:
 }
 
 static char *
-CERT_GetNameElement(CERTName *name, int wantedTag)
+CERT_GetNameElement(PRArenaPool *arena, CERTName *name, int wantedTag)
 {
     CERTRDN** rdns;
     CERTRDN *rdn;
-    CERTAVA** avas;
-    CERTAVA* ava;
     char *buf = 0;
-    int tag;
-    SECItem *decodeItem = NULL;
     
     rdns = name->rdns;
-    while ((rdn = *rdns++) != 0) {
-	avas = rdn->avas;
-	while ((ava = *avas++) != 0) {
-	    tag = CERT_GetAVATag(ava);
+    while (rdns && (rdn = *rdns++) != 0) {
+	CERTAVA** avas = rdn->avas;
+	CERTAVA*  ava;
+	while (avas && (ava = *avas++) != 0) {
+	    int tag = CERT_GetAVATag(ava);
 	    if ( tag == wantedTag ) {
-		decodeItem = CERT_DecodeAVAValue(&ava->value);
+		SECItem *decodeItem = CERT_DecodeAVAValue(&ava->value);
 		if(!decodeItem) {
 		    return NULL;
 		}
-		buf = (char *)PORT_ZAlloc(decodeItem->len + 1);
+		if (arena) {
+		    buf = (char *)PORT_ArenaZAlloc(arena,decodeItem->len + 1);
+		} else {
+		    buf = (char *)PORT_ZAlloc(decodeItem->len + 1);
+		}
 		if ( buf ) {
 		    PORT_Memcpy(buf, decodeItem->data, decodeItem->len);
 		    buf[decodeItem->len] = 0;
@@ -815,7 +872,6 @@ char *
 CERT_GetCertificateEmailAddress(CERTCertificate *cert)
 {
     char *rawEmailAddr = NULL;
-    char *emailAddr = NULL;
     SECItem subAltName;
     SECStatus rv;
     CERTGeneralName *nameList = NULL;
@@ -825,13 +881,16 @@ CERT_GetCertificateEmailAddress(CERTCertificate *cert)
     
     subAltName.data = NULL;
 
-    rawEmailAddr = CERT_GetNameElement(&(cert->subject), SEC_OID_PKCS9_EMAIL_ADDRESS);
+    rawEmailAddr = CERT_GetNameElement(cert->arena, &(cert->subject),
+						 SEC_OID_PKCS9_EMAIL_ADDRESS);
     if ( rawEmailAddr == NULL ) {
-	rawEmailAddr = CERT_GetNameElement(&(cert->subject), SEC_OID_RFC1274_MAIL);
+	rawEmailAddr = CERT_GetNameElement(cert->arena, &(cert->subject), 
+							SEC_OID_RFC1274_MAIL);
     }
     if ( rawEmailAddr == NULL) {
 
-	rv = CERT_FindCertExtension(cert,  SEC_OID_X509_SUBJECT_ALT_NAME, &subAltName);
+	rv = CERT_FindCertExtension(cert,  SEC_OID_X509_SUBJECT_ALT_NAME, 
+								&subAltName);
 	if (rv != SECSuccess) {
 	    goto finish;
 	}
@@ -846,14 +905,16 @@ CERT_GetCertificateEmailAddress(CERTCertificate *cert)
 	if (nameList != NULL) {
 	    do {
 		if (current->type == certDirectoryName) {
-		    rawEmailAddr = CERT_GetNameElement(&(current->name.directoryName), 
-						       SEC_OID_PKCS9_EMAIL_ADDRESS);
+		    rawEmailAddr = CERT_GetNameElement(cert->arena,
+			&(current->name.directoryName), 
+					       SEC_OID_PKCS9_EMAIL_ADDRESS);
 		    if ( rawEmailAddr == NULL ) {
-			rawEmailAddr = CERT_GetNameElement(&(current->name.directoryName), 
-							   SEC_OID_RFC1274_MAIL);
+			rawEmailAddr = CERT_GetNameElement(cert->arena,
+			  &(current->name.directoryName), SEC_OID_RFC1274_MAIL);
 		    }
 		} else if (current->type == certRFC822Name) {
-		    rawEmailAddr = (char*)PORT_ZAlloc(current->name.other.len + 1);
+		    rawEmailAddr = (char*)PORT_ArenaZAlloc(cert->arena,
+						current->name.other.len + 1);
 		    if (!rawEmailAddr) {
 			goto finish;
 		    }
@@ -869,18 +930,12 @@ CERT_GetCertificateEmailAddress(CERTCertificate *cert)
 	}
     }
     if (rawEmailAddr) {
-	emailAddr = (char*)PORT_ArenaZAlloc(cert->arena, PORT_Strlen(rawEmailAddr) + 1);
-	for (i = 0; i <= PORT_Strlen(rawEmailAddr); i++) {
-	    emailAddr[i] = tolower(rawEmailAddr[i]);
+	for (i = 0; i <= (int) PORT_Strlen(rawEmailAddr); i++) {
+	    rawEmailAddr[i] = tolower(rawEmailAddr[i]);
 	}
-    } else {
-	emailAddr = NULL;
-    }
+    } 
 
 finish:
-    if ( rawEmailAddr ) {
-	PORT_Free(rawEmailAddr);
-    }
 
     /* Don't free nameList, it's part of the arena. */
 
@@ -892,9 +947,152 @@ finish:
 	SECITEM_FreeItem(&subAltName, PR_FALSE);
     }
 
-    return(emailAddr);
+    return(rawEmailAddr);
 }
 
+static char *
+appendStringToBuf(char *dest, char *src, PRUint32 *pRemaining)
+{
+    PRUint32 len;
+    if (dest && src && src[0] && *pRemaining > (len = PL_strlen(src))) {
+	PRUint32 i;
+	for (i = 0; i < len; ++i)
+	    dest[i] = tolower(src[i]);
+	dest[len] = 0;
+	dest        += len + 1;
+	*pRemaining -= len + 1;
+    }
+    return dest;
+}
+
+static char *
+appendItemToBuf(char *dest, SECItem *src, PRUint32 *pRemaining)
+{
+    if (dest && src && src->data && src->len && src->data[0] && 
+        *pRemaining > src->len + 1 ) {
+	PRUint32 len = src->len;
+	PRUint32 i;
+	for (i = 0; i < len && src->data[i] ; ++i)
+	    dest[i] = tolower(src->data[i]);
+	dest[len] = 0;
+	dest        += len + 1;
+	*pRemaining -= len + 1;
+    }
+    return dest;
+}
+
+/* Returns a pointer to an environment-like string, a series of 
+** null-terminated strings, terminated by a zero-length string.
+** This function is intended to be internal to NSS.
+*/
+char *
+cert_GetCertificateEmailAddresses(CERTCertificate *cert)
+{
+    char *           rawEmailAddr = NULL;
+    char *           addrBuf      = NULL;
+    char *           pBuf         = NULL;
+    PRArenaPool *    tmpArena     = PORT_NewArena(DER_DEFAULT_CHUNKSIZE);
+    PRUint32         maxLen       = 0;
+    PRInt32          finalLen     = 0;
+    SECStatus        rv;
+    SECItem          subAltName;
+    
+    if (!tmpArena) 
+    	return addrBuf;
+
+    subAltName.data = NULL;
+    maxLen = cert->derCert.len;
+    PORT_Assert(maxLen);
+    if (!maxLen) 
+	maxLen = 2000;  /* a guess, should never happen */
+
+    pBuf = addrBuf = (char *)PORT_ArenaZAlloc(tmpArena, maxLen + 1);
+    if (!addrBuf) 
+    	goto loser;
+
+    rawEmailAddr = CERT_GetNameElement(tmpArena, &cert->subject,
+				       SEC_OID_PKCS9_EMAIL_ADDRESS);
+    pBuf = appendStringToBuf(pBuf, rawEmailAddr, &maxLen);
+
+    rawEmailAddr = CERT_GetNameElement(tmpArena, &cert->subject, 
+				       SEC_OID_RFC1274_MAIL);
+    pBuf = appendStringToBuf(pBuf, rawEmailAddr, &maxLen);
+
+    rv = CERT_FindCertExtension(cert,  SEC_OID_X509_SUBJECT_ALT_NAME, 
+				&subAltName);
+    if (rv == SECSuccess && subAltName.data) {
+	CERTGeneralName *nameList     = NULL;
+
+	if (!!(nameList = CERT_DecodeAltNameExtension(tmpArena, &subAltName))) {
+	    CERTGeneralName *current = nameList;
+	    do {
+		if (current->type == certDirectoryName) {
+		    rawEmailAddr = CERT_GetNameElement(tmpArena,
+			                       &current->name.directoryName, 
+					       SEC_OID_PKCS9_EMAIL_ADDRESS);
+		    pBuf = appendStringToBuf(pBuf, rawEmailAddr, &maxLen);
+
+		    rawEmailAddr = CERT_GetNameElement(tmpArena,
+					      &current->name.directoryName, 
+					      SEC_OID_RFC1274_MAIL);
+		    pBuf = appendStringToBuf(pBuf, rawEmailAddr, &maxLen);
+		} else if (current->type == certRFC822Name) {
+		    pBuf = appendItemToBuf(pBuf, &current->name.other, &maxLen);
+		}
+		current = cert_get_next_general_name(current);
+	    } while (current != nameList);
+	}
+	SECITEM_FreeItem(&subAltName, PR_FALSE);
+	/* Don't free nameList, it's part of the tmpArena. */
+    }
+    /* now copy superstring to cert's arena */
+    finalLen = (pBuf - addrBuf) + 1;
+    pBuf = NULL;
+    if (finalLen > 1) {
+	pBuf = PORT_ArenaAlloc(cert->arena, finalLen);
+	if (pBuf) {
+	    PORT_Memcpy(pBuf, addrBuf, finalLen);
+	}
+    }
+loser:
+    if (tmpArena)
+	PORT_FreeArena(tmpArena, PR_FALSE);
+
+    return pBuf;
+}
+
+/* returns pointer to storage in cert's arena.  Storage remains valid
+** as long as cert's reference count doesn't go to zero.
+** Caller should strdup or otherwise copy.
+*/
+const char *	/* const so caller won't muck with it. */
+CERT_GetFirstEmailAddress(CERTCertificate * cert)
+{
+    if (cert && cert->emailAddr && cert->emailAddr[0])
+    	return (const char *)cert->emailAddr;
+    return NULL;
+}
+
+/* returns pointer to storage in cert's arena.  Storage remains valid
+** as long as cert's reference count doesn't go to zero.
+** Caller should strdup or otherwise copy.
+*/
+const char *	/* const so caller won't muck with it. */
+CERT_GetNextEmailAddress(CERTCertificate * cert, const char * prev)
+{
+    if (cert && prev && prev[0]) {
+    	PRUint32 len = PL_strlen(prev);
+	prev += len + 1;
+	if (prev && prev[0])
+	    return prev;
+    }
+    return NULL;
+}
+
+/* This is seriously bogus, now that certs store their email addresses in
+** subject Alternative Name extensions. 
+** Returns a string allocated by PORT_StrDup, which the caller must free.
+*/
 char *
 CERT_GetCertEmailAddress(CERTName *name)
 {
@@ -902,9 +1100,9 @@ CERT_GetCertEmailAddress(CERTName *name)
     char *emailAddr;
 
     
-    rawEmailAddr = CERT_GetNameElement(name, SEC_OID_PKCS9_EMAIL_ADDRESS);
+    rawEmailAddr = CERT_GetNameElement(NULL, name, SEC_OID_PKCS9_EMAIL_ADDRESS);
     if ( rawEmailAddr == NULL ) {
-	rawEmailAddr = CERT_GetNameElement(name, SEC_OID_RFC1274_MAIL);
+	rawEmailAddr = CERT_GetNameElement(NULL, name, SEC_OID_RFC1274_MAIL);
     }
     emailAddr = CERT_FixupEmailAddr(rawEmailAddr);
     if ( rawEmailAddr ) {
@@ -913,57 +1111,58 @@ CERT_GetCertEmailAddress(CERTName *name)
     return(emailAddr);
 }
 
+/* The return value must be freed with PORT_Free. */
 char *
 CERT_GetCommonName(CERTName *name)
 {
-    return(CERT_GetNameElement(name, SEC_OID_AVA_COMMON_NAME));
+    return(CERT_GetNameElement(NULL, name, SEC_OID_AVA_COMMON_NAME));
 }
 
 char *
 CERT_GetCountryName(CERTName *name)
 {
-    return(CERT_GetNameElement(name, SEC_OID_AVA_COUNTRY_NAME));
+    return(CERT_GetNameElement(NULL, name, SEC_OID_AVA_COUNTRY_NAME));
 }
 
 char *
 CERT_GetLocalityName(CERTName *name)
 {
-    return(CERT_GetNameElement(name, SEC_OID_AVA_LOCALITY));
+    return(CERT_GetNameElement(NULL, name, SEC_OID_AVA_LOCALITY));
 }
 
 char *
 CERT_GetStateName(CERTName *name)
 {
-    return(CERT_GetNameElement(name, SEC_OID_AVA_STATE_OR_PROVINCE));
+    return(CERT_GetNameElement(NULL, name, SEC_OID_AVA_STATE_OR_PROVINCE));
 }
 
 char *
 CERT_GetOrgName(CERTName *name)
 {
-    return(CERT_GetNameElement(name, SEC_OID_AVA_ORGANIZATION_NAME));
+    return(CERT_GetNameElement(NULL, name, SEC_OID_AVA_ORGANIZATION_NAME));
 }
 
 char *
 CERT_GetDomainComponentName(CERTName *name)
 {
-    return(CERT_GetNameElement(name, SEC_OID_AVA_DC));
+    return(CERT_GetNameElement(NULL, name, SEC_OID_AVA_DC));
 }
 
 char *
 CERT_GetOrgUnitName(CERTName *name)
 {
-    return(CERT_GetNameElement(name, SEC_OID_AVA_ORGANIZATIONAL_UNIT_NAME));
+    return(CERT_GetNameElement(NULL, name, SEC_OID_AVA_ORGANIZATIONAL_UNIT_NAME));
 }
 
 char *
 CERT_GetDnQualifier(CERTName *name)
 {
-    return(CERT_GetNameElement(name, SEC_OID_AVA_DN_QUALIFIER));
+    return(CERT_GetNameElement(NULL, name, SEC_OID_AVA_DN_QUALIFIER));
 }
 
 char *
 CERT_GetCertUid(CERTName *name)
 {
-    return(CERT_GetNameElement(name, SEC_OID_RFC1274_UID));
+    return(CERT_GetNameElement(NULL, name, SEC_OID_RFC1274_UID));
 }
 
