@@ -86,8 +86,17 @@ SECStatus
 SEC_DeletePermCertificate(CERTCertificate *cert)
 {
     PRStatus nssrv;
+    NSSTrustDomain *td = STAN_GetDefaultTrustDomain();
     NSSCertificate *c = STAN_GetNSSCertificate(cert);
+
+    /* get rid of the token instances */
     nssrv = NSSCertificate_DeleteStoredObject(c, NULL);
+
+    /* get rid of the cache entry */
+    nssTrustDomain_LockCertCache(td);
+    nssTrustDomain_RemoveCertFromCacheLOCKED(td, c);
+    nssTrustDomain_UnlockCertCache(td);
+
     return (nssrv == PR_SUCCESS) ? SECSuccess : SECFailure;
 }
 
@@ -131,47 +140,70 @@ CERT_ChangeCertTrust(CERTCertDBHandle *handle, CERTCertificate *cert,
     return rv;
 }
 
+extern const NSSError NSS_ERROR_INVALID_CERTIFICATE;
+
 SECStatus
 __CERT_AddTempCertToPerm(CERTCertificate *cert, char *nickname,
 		       CERTCertTrust *trust)
 {
-    PRStatus nssrv;
     NSSUTF8 *stanNick;
     PK11SlotInfo *slot;
     NSSToken *internal;
     NSSCryptoContext *context;
+    nssCryptokiObject *permInstance;
     NSSCertificate *c = STAN_GetNSSCertificate(cert);
     context = c->object.cryptoContext;
     if (!context) {
-	return PR_FAILURE; /* wasn't a temp cert */
+	PORT_SetError(SEC_ERROR_ADDING_CERT); 
+	return SECFailure; /* wasn't a temp cert */
     }
-    stanNick = NSSCertificate_GetNickname(c, NULL);
+    stanNick = nssCertificate_GetNickname(c, NULL);
     if (stanNick && nickname && strcmp(nickname, stanNick) != 0) {
 	/* take the new nickname */
-	PORT_Free(cert->nickname);
+	cert->nickname = NULL;
 	stanNick = NULL;
     }
     if (!stanNick && nickname) {
 	stanNick = nssUTF8_Duplicate((NSSUTF8 *)nickname, c->object.arena);
-	cert->nickname = PORT_Strdup(nickname);
     }
     /* Delete the temp instance */
-    nssCertificateStore_Remove(context->certStore, c);
+    nssCertificateStore_Lock(context->certStore);
+    nssCertificateStore_RemoveCertLOCKED(context->certStore, c);
+    nssCertificateStore_Unlock(context->certStore);
     c->object.cryptoContext = NULL;
-    /* the perm instance will assume the reference */
-    nssList_Clear(c->object.instanceList, NULL);
     /* Import the perm instance onto the internal token */
     slot = PK11_GetInternalKeySlot();
     internal = PK11Slot_GetNSSToken(slot);
-    nssrv = nssToken_ImportCertificate(internal, NULL, c, stanNick, PR_TRUE);
-    if (nssrv != PR_SUCCESS) {
+    permInstance = nssToken_ImportCertificate(internal, NULL,
+                                              NSSCertificateType_PKIX,
+                                              &c->id,
+                                              stanNick,
+                                              &c->encoding,
+                                              &c->issuer,
+                                              &c->subject,
+                                              &c->serial,
+					      cert->emailAddr,
+                                              PR_TRUE);
+    PK11_FreeSlot(slot);
+    if (!permInstance) {
+	if (NSS_GetError() == NSS_ERROR_INVALID_CERTIFICATE) {
+	    PORT_SetError(SEC_ERROR_REUSED_ISSUER_AND_SERIAL);
+	}
 	return SECFailure;
     }
+    nssPKIObject_AddInstance(&c->object, permInstance);
+    nssTrustDomain_AddCertsToCache(STAN_GetDefaultTrustDomain(), &c, 1);
     /* reset the CERTCertificate fields */
     cert->nssCertificate = NULL;
-    cert = STAN_GetCERTCertificate(c); /* will return same pointer */
+    cert = STAN_GetCERTCertificateOrRelease(c); /* should return same pointer */
+    if (!cert) {
+        return SECFailure;
+    }
     cert->istemp = PR_FALSE;
     cert->isperm = PR_TRUE;
+    if (!trust) {
+	return SECSuccess;
+    }
     return (STAN_ChangeCertTrust(cert, trust) == PR_SUCCESS) ? 
 							SECSuccess: SECFailure;
 }
@@ -189,11 +221,11 @@ __CERT_NewTempCertificate(CERTCertDBHandle *handle, SECItem *derCert,
 {
     PRStatus nssrv;
     NSSCertificate *c;
-    NSSCryptoContext *context;
-    NSSArena *arena;
     CERTCertificate *cc;
     NSSCertificate *tempCert;
+    nssPKIObject *pkio;
     NSSCryptoContext *gCC = STAN_GetDefaultCryptoContext();
+    NSSTrustDomain *gTD = STAN_GetDefaultTrustDomain();
     if (!isperm) {
 	NSSDER encoding;
 	NSSITEM_FROM_SECITEM(&encoding, derCert);
@@ -206,30 +238,49 @@ __CERT_NewTempCertificate(CERTCertDBHandle *handle, SECItem *derCert,
 	                                                           &encoding);
 	}
 	if (c) {
-	    return STAN_GetCERTCertificate(c);
+	    /* actually, that search ends up going by issuer/serial,
+	     * so it is still possible to return a cert with the same
+	     * issuer/serial but a different encoding, and we're
+	     * going to reject that
+	     */
+	    if (!nssItem_Equal(&c->encoding, &encoding, NULL)) {
+		nssCertificate_Destroy(c);
+		PORT_SetError(SEC_ERROR_REUSED_ISSUER_AND_SERIAL);
+		cc = NULL;
+	    } else {
+    		cc = STAN_GetCERTCertificateOrRelease(c);
+	    }
+	    return cc;
 	}
     }
-    arena = NSSArena_Create();
-    if (!arena) {
+    pkio = nssPKIObject_Create(NULL, NULL, gTD, gCC);
+    if (!pkio) {
 	return NULL;
     }
-    c = nss_ZNEW(arena, NSSCertificate);
+    c = nss_ZNEW(pkio->arena, NSSCertificate);
     if (!c) {
-	nssArena_Destroy(arena);
+	nssPKIObject_Destroy(pkio);
 	return NULL;
     }
-    NSSITEM_FROM_SECITEM(&c->encoding, derCert);
-    nssrv = nssPKIObject_Initialize(&c->object, arena, NULL, NULL);
-    if (nssrv != PR_SUCCESS) {
-	goto loser;
+    c->object = *pkio;
+    if (copyDER) {
+	nssItem_Create(c->object.arena, &c->encoding, 
+	               derCert->len, derCert->data);
+    } else {
+	NSSITEM_FROM_SECITEM(&c->encoding, derCert);
     }
     /* Forces a decoding of the cert in order to obtain the parts used
      * below
      */
+    /* 'c' is not adopted here, if we fail loser frees what has been
+     * allocated so far for 'c' */
     cc = STAN_GetCERTCertificate(c);
-    nssItem_Create(arena, 
+    if (!cc) {
+        goto loser;
+    }
+    nssItem_Create(c->object.arena, 
                    &c->issuer, cc->derIssuer.len, cc->derIssuer.data);
-    nssItem_Create(arena, 
+    nssItem_Create(c->object.arena, 
                    &c->subject, cc->derSubject.len, cc->derSubject.data);
     if (PR_TRUE) {
 	/* CERTCertificate stores serial numbers decoded.  I need the DER
@@ -238,31 +289,30 @@ __CERT_NewTempCertificate(CERTCertDBHandle *handle, SECItem *derCert,
 	SECItem derSerial = { 0 };
 	CERT_SerialNumberFromDERCert(&cc->derCert, &derSerial);
 	if (!derSerial.data) goto loser;
-	nssItem_Create(arena, &c->serial, derSerial.len, derSerial.data);
+	nssItem_Create(c->object.arena, &c->serial, derSerial.len, derSerial.data);
 	PORT_Free(derSerial.data);
     }
     if (nickname) {
-	c->object.tempName = nssUTF8_Create(arena, 
+	c->object.tempName = nssUTF8_Create(c->object.arena, 
                                             nssStringType_UTF8String, 
                                             (NSSUTF8 *)nickname, 
                                             PORT_Strlen(nickname));
     }
-    if (cc->emailAddr) {
-	c->email = nssUTF8_Create(arena, 
+    if (cc->emailAddr && cc->emailAddr[0]) {
+	c->email = nssUTF8_Create(c->object.arena, 
 	                          nssStringType_PrintableString, 
 	                          (NSSUTF8 *)cc->emailAddr, 
 	                          PORT_Strlen(cc->emailAddr));
     }
-    context = STAN_GetDefaultCryptoContext();
     /* this function cannot detect if the cert exists as a temp cert now, but
      * didn't when CERT_NewTemp was first called.
      */
-    nssrv = NSSCryptoContext_ImportCertificate(context, c);
+    nssrv = NSSCryptoContext_ImportCertificate(gCC, c);
     if (nssrv != PR_SUCCESS) {
 	goto loser;
     }
     /* so find the entry in the temp store */
-    tempCert = NSSCryptoContext_FindCertificateByIssuerAndSerialNumber(context,
+    tempCert = NSSCryptoContext_FindCertificateByIssuerAndSerialNumber(gCC,
                                                                    &c->issuer,
                                                                    &c->serial);
     /* destroy the copy */
@@ -270,11 +320,13 @@ __CERT_NewTempCertificate(CERTCertDBHandle *handle, SECItem *derCert,
     if (tempCert) {
 	/* and use the "official" entry */
 	c = tempCert;
-	cc = STAN_GetCERTCertificate(c);
+    	cc = STAN_GetCERTCertificateOrRelease(c);
+        if (!cc) {
+            return NULL;
+        }
     } else {
 	return NULL;
     }
-    c->object.trustDomain = STAN_GetDefaultTrustDomain();
     cc->istemp = PR_TRUE;
     cc->isperm = PR_FALSE;
     return cc;
@@ -309,17 +361,18 @@ CERT_FindCertByIssuerAndSN(CERTCertDBHandle *handle, CERTIssuerAndSN *issuerAndS
 static NSSCertificate *
 get_best_temp_or_perm(NSSCertificate *ct, NSSCertificate *cp)
 {
-    nssBestCertificateCB best;
     NSSUsage usage;
+    NSSCertificate *arr[3];
+    if (!ct) {
+	return nssCertificate_AddRef(cp);
+    } else if (!cp) {
+	return nssCertificate_AddRef(ct);
+    }
+    arr[0] = ct;
+    arr[1] = cp;
+    arr[2] = NULL;
     usage.anyUsage = PR_TRUE;
-    nssBestCertificate_SetArgs(&best, NULL, &usage, NULL);
-    if (ct) {
-	nssBestCertificate_Callback(ct, (void *)&best);
-    }
-    if (cp) {
-	nssBestCertificate_Callback(cp, (void *)&best);
-    }
-    return best.cert;
+    return nssCertificateArray_FindBestCertificate(arr, NULL, &usage, NULL);
 }
 
 CERTCertificate *
@@ -338,18 +391,12 @@ CERT_FindCertByName(CERTCertDBHandle *handle, SECItem *name)
                                                      NULL, &usage, NULL);
     c = get_best_temp_or_perm(ct, cp);
     if (ct) {
-	CERTCertificate *cert = STAN_GetCERTCertificate(ct);
-	CERT_DestroyCertificate(cert);
+	CERT_DestroyCertificate(STAN_GetCERTCertificateOrRelease(ct));
     }
     if (cp) {
-	CERTCertificate *cert = STAN_GetCERTCertificate(cp);
-	CERT_DestroyCertificate(cert);
-    }
-    if (c) {
-	return STAN_GetCERTCertificate(c);
-    } else {
-	return NULL;
+	CERT_DestroyCertificate(STAN_GetCERTCertificateOrRelease(cp));
     }
+    return c ? STAN_GetCERTCertificateOrRelease(c) : NULL;
 }
 
 CERTCertificate *
@@ -388,17 +435,12 @@ CERT_FindCertByNickname(CERTCertDBHandle *handle, char *nickname)
 	c = get_best_temp_or_perm(ct, STAN_GetNSSCertificate(cert));
 	CERT_DestroyCertificate(cert);
 	if (ct) {
-	    CERTCertificate *cert2 = STAN_GetCERTCertificate(ct);
-	    CERT_DestroyCertificate(cert2);
+	    CERT_DestroyCertificate(STAN_GetCERTCertificateOrRelease(ct));
 	}
     } else {
 	c = ct;
     }
-    if (c) {
-	return STAN_GetCERTCertificate(c);
-    } else {
-	return NULL;
-    }
+    return c ? STAN_GetCERTCertificateOrRelease(c) : NULL;
 }
 
 CERTCertificate *
@@ -415,7 +457,7 @@ CERT_FindCertByDERCert(CERTCertDBHandle *handle, SECItem *derCert)
 	                                                       &encoding);
 	if (!c) return NULL;
     }
-    return STAN_GetCERTCertificate(c);
+    return STAN_GetCERTCertificateOrRelease(c);
 }
 
 CERTCertificate *
@@ -425,29 +467,34 @@ CERT_FindCertByNicknameOrEmailAddr(CERTCertDBHandle *handle, char *name)
     NSSCertificate *c, *ct;
     CERTCertificate *cert;
     NSSUsage usage;
+
+    if (NULL == name) {
+        PORT_SetError(SEC_ERROR_INVALID_ARGS);
+	return NULL;
+    }
     usage.anyUsage = PR_TRUE;
     cc = STAN_GetDefaultCryptoContext();
     ct = NSSCryptoContext_FindBestCertificateByNickname(cc, name, 
                                                        NULL, &usage, NULL);
-    if (!ct) {
-	ct = NSSCryptoContext_FindBestCertificateByEmail(cc, name, 
-	                                                NULL, &usage, NULL);
+    if (!ct && PORT_Strchr(name, '@') != NULL) {
+        char* lowercaseName = CERT_FixupEmailAddr(name);
+        if (lowercaseName) {
+	    ct = NSSCryptoContext_FindBestCertificateByEmail(cc, lowercaseName, 
+							    NULL, &usage, NULL);
+            PORT_Free(lowercaseName);
+        }
     }
     cert = PK11_FindCertFromNickname(name, NULL);
     if (cert) {
 	c = get_best_temp_or_perm(ct, STAN_GetNSSCertificate(cert));
 	CERT_DestroyCertificate(cert);
 	if (ct) {
-	    CERTCertificate *cert2 = STAN_GetCERTCertificate(ct);
-	    CERT_DestroyCertificate(cert2);
+	    CERT_DestroyCertificate(STAN_GetCERTCertificateOrRelease(ct));
 	}
     } else {
 	c = ct;
     }
-    if (c) {
-	return STAN_GetCERTCertificate(c);
-    }
-    return NULL;
+    return c ? STAN_GetCERTCertificateOrRelease(c) : NULL;
 }
 
 static void 
@@ -503,23 +550,34 @@ CERT_CreateSubjectCertList(CERTCertList *certList, CERTCertDBHandle *handle,
     /* Iterate over the matching temp certs.  Add them to the list */
     ci = tSubjectCerts;
     while (ci && *ci) {
-	cert = STAN_GetCERTCertificate(*ci);
-	add_to_subject_list(certList, cert, validOnly, sorttime);
+	cert = STAN_GetCERTCertificateOrRelease(*ci);
+	/* *ci may be invalid at this point, don't reference it again */
+        if (cert) {
+	    /* NOTE: add_to_subject_list adopts the incoming cert. */
+	    add_to_subject_list(certList, cert, validOnly, sorttime);
+        }
 	ci++;
     }
     /* Iterate over the matching perm certs.  Add them to the list */
     ci = pSubjectCerts;
     while (ci && *ci) {
-	cert = STAN_GetCERTCertificate(*ci);
-	add_to_subject_list(certList, cert, validOnly, sorttime);
+	cert = STAN_GetCERTCertificateOrRelease(*ci);
+	/* *ci may be invalid at this point, don't reference it again */
+        if (cert) {
+	    /* NOTE: add_to_subject_list adopts the incoming cert. */
+	    add_to_subject_list(certList, cert, validOnly, sorttime);
+        }
 	ci++;
     }
+    /* all the references have been adopted or freed at this point, just
+     * free the arrays now */
     nss_ZFreeIf(tSubjectCerts);
     nss_ZFreeIf(pSubjectCerts);
     return certList;
 loser:
-    nss_ZFreeIf(tSubjectCerts);
-    nss_ZFreeIf(pSubjectCerts);
+    /* need to free the references in tSubjectCerts and pSubjectCerts! */
+    nssCertificateArray_Destroy(tSubjectCerts);
+    nssCertificateArray_Destroy(pSubjectCerts);
     if (myList && certList != NULL) {
 	CERT_DestroyCertList(certList);
     }
@@ -529,53 +587,19 @@ loser:
 void
 CERT_DestroyCertificate(CERTCertificate *cert)
 {
-    int refCount;
-    CERTCertDBHandle *handle;
     if ( cert ) {
-	NSSCertificate *tmp = STAN_GetNSSCertificate(cert);
-	handle = cert->dbhandle;
-#ifdef NSS_CLASSIC
-        CERT_LockCertRefCount(cert);
-	PORT_Assert(cert->referenceCount > 0);
-	refCount = --cert->referenceCount;
-        CERT_UnlockCertRefCount(cert);
-	if ( ( refCount == 0 ) && !cert->keepSession ) {
-	    PRArenaPool *arena  = cert->arena;
-	    /* zero cert before freeing. Any stale references to this cert
-	     * after this point will probably cause an exception.  */
-	    PORT_Memset(cert, 0, sizeof *cert);
-	    cert = NULL;
-	    /* free the arena that contains the cert. */
-	    PORT_FreeArena(arena, PR_FALSE);
-        }
-#else
+	/* don't use STAN_GetNSSCertificate because we don't want to
+	 * go to the trouble of translating the CERTCertificate into
+	 * an NSSCertificate just to destroy it.  If it hasn't been done
+	 * yet, don't do it at all.
+	 */
+	NSSCertificate *tmp = cert->nssCertificate;
 	if (tmp) {
-	    NSSTrustDomain *td = STAN_GetDefaultTrustDomain();
-	    refCount = (int)tmp->object.refCount;
-	    /* This is a hack.  For 3.4, there are persistent references
-	     * to 4.0 certificates during the lifetime of a cert.  In the
-	     * case of a temp cert, the persistent reference is in the
-	     * cert store of the global crypto context.  For a perm cert,
-	     * the persistent reference is in the cache.  Thus, the last
-	     * external reference is really the penultimate NSS reference.
-	     * When the count drops to two, it is really one, but the
-	     * persistent reference must be explicitly deleted.  In 4.0,
-	     * this ugliness will not appear.  Crypto contexts will remove
-	     * their own cert references, and the cache will have its
-	     * own management code also.
-	     */
-	    if (refCount == 2) {
-		NSSCryptoContext *cc = tmp->object.cryptoContext;
-		if (cc != NULL) {
-		    nssCertificateStore_Remove(cc->certStore, tmp);
-		} else {
-		    nssTrustDomain_RemoveCertFromCache(td, tmp);
-		}
-	    }
 	    /* delete the NSSCertificate */
 	    NSSCertificate_Destroy(tmp);
-	} 
-#endif
+	} else {
+	    PORT_FreeArena(cert->arena, PR_FALSE);
+	}
     }
     return;
 }
@@ -682,21 +706,14 @@ CERT_GetDBContentVersion(CERTCertDBHandle *handle)
     return 0;
 }
 
-/*
- *
- * Manage S/MIME profiles
- *
- */
-
 SECStatus
-CERT_SaveSMimeProfile(CERTCertificate *cert, SECItem *emailProfile,
-		      SECItem *profileTime)
+certdb_SaveSingleProfile(CERTCertificate *cert, const char *emailAddr, 
+				SECItem *emailProfile, SECItem *profileTime)
 {
     int64 oldtime;
     int64 newtime;
     SECStatus rv = SECFailure;
     PRBool saveit;
-    char *emailAddr;
     SECItem oldprof, oldproftime;
     SECItem *oldProfile = NULL;
     SECItem *oldProfileTime = NULL;
@@ -705,12 +722,6 @@ CERT_SaveSMimeProfile(CERTCertificate *cert, SECItem *emailProfile,
     NSSCryptoContext *cc;
     nssSMIMEProfile *stanProfile = NULL;
     PRBool freeOldProfile = PR_FALSE;
-    
-    emailAddr = cert->emailAddr;
-    
-    if ( emailAddr == NULL ) {
-	goto loser;
-    }
 
     c = STAN_GetNSSCertificate(cert);
     if (!c) return SECFailure;
@@ -725,8 +736,8 @@ CERT_SaveSMimeProfile(CERTCertificate *cert, SECItem *emailProfile,
 	    oldProfileTime = &oldproftime;
 	}
     } else {
-	oldProfile = PK11_FindSMimeProfile(&slot, emailAddr, &cert->derSubject, 
-							&oldProfileTime); 
+	oldProfile = PK11_FindSMimeProfile(&slot, (char *)emailAddr, 
+					&cert->derSubject, &oldProfileTime); 
 	freeOldProfile = PR_TRUE;
     }
 
@@ -809,8 +820,8 @@ CERT_SaveSMimeProfile(CERTCertificate *cert, SECItem *emailProfile,
 		rv = (nssrv == PR_SUCCESS) ? SECSuccess : SECFailure;
 	    }
 	} else {
-	    rv = PK11_SaveSMimeProfile(slot, emailAddr, &cert->derSubject, 
-						emailProfile, profileTime);
+	    rv = PK11_SaveSMimeProfile(slot, (char *)emailAddr, 
+				&cert->derSubject, emailProfile, profileTime);
 	}
     } else {
 	rv = SECSuccess;
@@ -826,21 +837,71 @@ loser:
     if (stanProfile) {
 	nssSMIMEProfile_Destroy(stanProfile);
     }
+    if (slot) {
+	PK11_FreeSlot(slot);
+    }
     
     return(rv);
 }
 
+/*
+ *
+ * Manage S/MIME profiles
+ *
+ */
+
+SECStatus
+CERT_SaveSMimeProfile(CERTCertificate *cert, SECItem *emailProfile,
+		      SECItem *profileTime)
+{
+    const char *emailAddr;
+    SECStatus rv;
+
+    if (!cert) {
+        return SECFailure;
+    }
+
+    if (cert->slot &&  !PK11_IsInternal(cert->slot)) {
+        /* this cert comes from an external source, we need to add it
+        to the cert db before creating an S/MIME profile */
+        PK11SlotInfo* internalslot = PK11_GetInternalKeySlot();
+        if (!internalslot) {
+            return SECFailure;
+        }
+        rv = PK11_ImportCert(internalslot, cert,
+            CK_INVALID_HANDLE, NULL, PR_FALSE);
+
+        PK11_FreeSlot(internalslot);
+        if (rv != SECSuccess ) {
+            return SECFailure;
+        }
+    }
+
+    
+    for (emailAddr = CERT_GetFirstEmailAddress(cert); emailAddr != NULL;
+		emailAddr = CERT_GetNextEmailAddress(cert,emailAddr)) {
+	rv = certdb_SaveSingleProfile(cert,emailAddr,emailProfile,profileTime);
+	if (rv != SECSuccess) {
+	   return SECFailure;
+	}
+    }
+    return SECSuccess;
+
+}
+
+
 SECItem *
 CERT_FindSMimeProfile(CERTCertificate *cert)
 {
     PK11SlotInfo *slot = NULL;
     NSSCertificate *c;
     NSSCryptoContext *cc;
+    SECItem *rvItem = NULL;
+
     c = STAN_GetNSSCertificate(cert);
     if (!c) return NULL;
     cc = c->object.cryptoContext;
     if (cc != NULL) {
-	SECItem *rvItem = NULL;
 	nssSMIMEProfile *stanProfile;
 	stanProfile = nssCryptoContext_FindSMIMEProfileForCertificate(cc, c);
 	if (stanProfile) {
@@ -853,8 +914,12 @@ CERT_FindSMimeProfile(CERTCertificate *cert)
 	}
 	return rvItem;
     }
-    return 
+    rvItem =
 	PK11_FindSMimeProfile(&slot, cert->emailAddr, &cert->derSubject, NULL);
+    if (slot) {
+    	PK11_FreeSlot(slot);
+    }
+    return rvItem;
 }
 
 /*
