@@ -32,52 +32,76 @@
  */
 
 #ifdef DEBUG
-static const char CVS_ID[] = "@(#) $RCSfile: cryptocontext.c,v $ $Revision: 1.9.16.1 $ $Date: 2002/04/10 03:28:10 $ $Name: MOZILLA_1_0_RELEASE $";
+static const char CVS_ID[] = "@(#) $RCSfile: cryptocontext.c,v $ $Revision: 1.12 $ $Date: 2003/01/08 21:58:29 $ $Name: MOZILLA_1_7_8_RELEASE $";
 #endif /* DEBUG */
 
-#ifndef NSSPKI_H
-#include "nsspki.h"
-#endif /* NSSPKI_H */
-
-#ifndef PKIM_H
-#include "pkim.h"
-#endif /* PKIM_H */
-
-#ifndef PKIT_H
-#include "pkit.h"
-#endif /* PKIT_H */
-
 #ifndef DEV_H
 #include "dev.h"
 #endif /* DEV_H */
 
+#ifndef PKIM_H
+#include "pkim.h"
+#endif /* PKIM_H */
+
 #ifndef PKISTORE_H
 #include "pkistore.h"
 #endif /* PKISTORE_H */
 
-#ifdef NSS_3_4_CODE
-#include "pk11func.h"
-#include "dev3hack.h"
+#include "pki1t.h"
+
+#ifdef PURE_STAN_BUILD
+struct NSSCryptoContextStr
+{
+    PRInt32 refCount;
+    NSSArena *arena;
+    NSSTrustDomain *td;
+    NSSToken *token;
+    nssSession *session;
+    nssCertificateStore *certStore;
+};
 #endif
 
 extern const NSSError NSS_ERROR_NOT_FOUND;
 
-NSS_IMPLEMENT PRStatus
-NSSCryptoContext_Destroy
-(
-  NSSCryptoContext *cc
+NSS_IMPLEMENT NSSCryptoContext *
+nssCryptoContext_Create (
+  NSSTrustDomain *td,
+  NSSCallback *uhhOpt
 )
 {
-    if (cc->certStore) {
-	nssCertificateStore_Destroy(cc->certStore);
+    NSSArena *arena;
+    NSSCryptoContext *rvCC;
+    arena = NSSArena_Create();
+    if (!arena) {
+	return NULL;
     }
-    nssArena_Destroy(cc->arena);
-    return PR_SUCCESS;
+    rvCC = nss_ZNEW(arena, NSSCryptoContext);
+    if (!rvCC) {
+	return NULL;
+    }
+    rvCC->td = td;
+    rvCC->arena = arena;
+    return rvCC;
 }
 
 NSS_IMPLEMENT PRStatus
-NSSCryptoContext_SetDefaultCallback
-(
+NSSCryptoContext_Destroy (
+  NSSCryptoContext *cc
+)
+{
+    PRStatus status = PR_SUCCESS;
+    if (cc->certStore) {
+	status = nssCertificateStore_Destroy(cc->certStore);
+	if (status == PR_FAILURE) {
+	    return status;
+	}
+    }
+    nssArena_Destroy(cc->arena);
+    return status;
+}
+
+NSS_IMPLEMENT PRStatus
+NSSCryptoContext_SetDefaultCallback (
   NSSCryptoContext *td,
   NSSCallback *newCallback,
   NSSCallback **oldCallbackOpt
@@ -88,8 +112,7 @@ NSSCryptoContext_SetDefaultCallback
 }
 
 NSS_IMPLEMENT NSSCallback *
-NSSCryptoContext_GetDefaultCallback
-(
+NSSCryptoContext_GetDefaultCallback (
   NSSCryptoContext *td,
   PRStatus *statusOpt
 )
@@ -99,8 +122,7 @@ NSSCryptoContext_GetDefaultCallback
 }
 
 NSS_IMPLEMENT NSSTrustDomain *
-NSSCryptoContext_GetTrustDomain
-(
+NSSCryptoContext_GetTrustDomain (
   NSSCryptoContext *td
 )
 {
@@ -109,8 +131,7 @@ NSSCryptoContext_GetTrustDomain
 }
 
 NSS_IMPLEMENT PRStatus
-NSSCryptoContext_ImportCertificate
-(
+NSSCryptoContext_ImportCertificate (
   NSSCryptoContext *cc,
   NSSCertificate *c
 )
@@ -130,8 +151,7 @@ NSSCryptoContext_ImportCertificate
 }
 
 NSS_IMPLEMENT NSSCertificate *
-NSSCryptoContext_ImportPKIXCertificate
-(
+NSSCryptoContext_ImportPKIXCertificate (
   NSSCryptoContext *cc,
   struct NSSPKIXCertificateStr *pc
 )
@@ -141,8 +161,7 @@ NSSCryptoContext_ImportPKIXCertificate
 }
 
 NSS_IMPLEMENT NSSCertificate *
-NSSCryptoContext_ImportEncodedCertificate
-(
+NSSCryptoContext_ImportEncodedCertificate (
   NSSCryptoContext *cc,
   NSSBER *ber
 )
@@ -152,8 +171,7 @@ NSSCryptoContext_ImportEncodedCertificate
 }
 
 NSS_IMPLEMENT PRStatus
-NSSCryptoContext_ImportEncodedPKIXCertificateChain
-(
+NSSCryptoContext_ImportEncodedPKIXCertificateChain (
   NSSCryptoContext *cc,
   NSSBER *ber
 )
@@ -163,8 +181,7 @@ NSSCryptoContext_ImportEncodedPKIXCertificateChain
 }
 
 NSS_IMPLEMENT PRStatus
-nssCryptoContext_ImportTrust
-(
+nssCryptoContext_ImportTrust (
   NSSCryptoContext *cc,
   NSSTrust *trust
 )
@@ -177,15 +194,16 @@ nssCryptoContext_ImportTrust
 	}
     }
     nssrv = nssCertificateStore_AddTrust(cc->certStore, trust);
+#if 0
     if (nssrv == PR_SUCCESS) {
 	trust->object.cryptoContext = cc;
     }
+#endif
     return nssrv;
 }
 
 NSS_IMPLEMENT PRStatus
-nssCryptoContext_ImportSMIMEProfile
-(
+nssCryptoContext_ImportSMIMEProfile (
   NSSCryptoContext *cc,
   nssSMIMEProfile *profile
 )
@@ -198,15 +216,16 @@ nssCryptoContext_ImportSMIMEProfile
 	}
     }
     nssrv = nssCertificateStore_AddSMIMEProfile(cc->certStore, profile);
+#if 0
     if (nssrv == PR_SUCCESS) {
 	profile->object.cryptoContext = cc;
     }
+#endif
     return nssrv;
 }
 
 NSS_IMPLEMENT NSSCertificate *
-NSSCryptoContext_FindBestCertificateByNickname
-(
+NSSCryptoContext_FindBestCertificateByNickname (
   NSSCryptoContext *cc,
   NSSUTF8 *name,
   NSSTime *timeOpt, /* NULL for "now" */
@@ -214,41 +233,26 @@ NSSCryptoContext_FindBestCertificateByNickname
   NSSPolicies *policiesOpt /* NULL for none */
 )
 {
-    PRIntn i;
-    NSSCertificate *c;
-    NSSCertificate **nickCerts;
-    nssBestCertificateCB best;
+    NSSCertificate **certs;
+    NSSCertificate *rvCert = NULL;
     if (!cc->certStore) {
 	return NULL;
     }
-    nssBestCertificate_SetArgs(&best, timeOpt, usage, policiesOpt);
-    /* This could be improved by querying the store with a callback */
-    nickCerts = nssCertificateStore_FindCertificatesByNickname(cc->certStore,
-                                                               name,
-                                                               NULL,
-                                                               0,
-                                                               NULL);
-    if (nickCerts) {
-	PRStatus nssrv;
-	for (i=0, c = *nickCerts; c != NULL; c = nickCerts[++i]) {
-	    nssrv = nssBestCertificate_Callback(c, &best);
-	    NSSCertificate_Destroy(c);
-	    if (nssrv != PR_SUCCESS) {
-		if (best.cert) {
-		    NSSCertificate_Destroy(best.cert);
-		    best.cert = NULL;
-		}
-		break;
-	    }
-	}
-	nss_ZFreeIf(nickCerts);
+    certs = nssCertificateStore_FindCertificatesByNickname(cc->certStore,
+                                                           name,
+                                                           NULL, 0, NULL);
+    if (certs) {
+	rvCert = nssCertificateArray_FindBestCertificate(certs,
+	                                                 timeOpt,
+	                                                 usage,
+	                                                 policiesOpt);
+	nssCertificateArray_Destroy(certs);
     }
-    return best.cert;
+    return rvCert;
 }
 
 NSS_IMPLEMENT NSSCertificate **
-NSSCryptoContext_FindCertificatesByNickname
-(
+NSSCryptoContext_FindCertificatesByNickname (
   NSSCryptoContext *cc,
   NSSUTF8 *name,
   NSSCertificate *rvOpt[],
@@ -269,8 +273,7 @@ NSSCryptoContext_FindCertificatesByNickname
 }
 
 NSS_IMPLEMENT NSSCertificate *
-NSSCryptoContext_FindCertificateByIssuerAndSerialNumber
-(
+NSSCryptoContext_FindCertificateByIssuerAndSerialNumber (
   NSSCryptoContext *cc,
   NSSDER *issuer,
   NSSDER *serialNumber
@@ -286,8 +289,7 @@ NSSCryptoContext_FindCertificateByIssuerAndSerialNumber
 }
 
 NSS_IMPLEMENT NSSCertificate *
-NSSCryptoContext_FindBestCertificateBySubject
-(
+NSSCryptoContext_FindBestCertificateBySubject (
   NSSCryptoContext *cc,
   NSSDER *subject,
   NSSTime *timeOpt,
@@ -295,40 +297,26 @@ NSSCryptoContext_FindBestCertificateBySubject
   NSSPolicies *policiesOpt
 )
 {
-    PRIntn i;
-    NSSCertificate *c;
-    NSSCertificate **subjectCerts;
-    nssBestCertificateCB best;
+    NSSCertificate **certs;
+    NSSCertificate *rvCert = NULL;
     if (!cc->certStore) {
 	return NULL;
     }
-    nssBestCertificate_SetArgs(&best, timeOpt, usage, policiesOpt);
-    subjectCerts = nssCertificateStore_FindCertificatesBySubject(cc->certStore,
-                                                                 subject,
-                                                                 NULL,
-                                                                 0,
-                                                                 NULL);
-    if (subjectCerts) {
-	PRStatus nssrv;
-	for (i=0, c = *subjectCerts; c != NULL; c = subjectCerts[++i]) {
-	    nssrv = nssBestCertificate_Callback(c, &best);
-	    NSSCertificate_Destroy(c);
-	    if (nssrv != PR_SUCCESS) {
-		if (best.cert) {
-		    NSSCertificate_Destroy(best.cert);
-		    best.cert = NULL;
-		}
-		break;
-	    }
-	}
-	nss_ZFreeIf(subjectCerts);
+    certs = nssCertificateStore_FindCertificatesBySubject(cc->certStore,
+                                                          subject,
+                                                          NULL, 0, NULL);
+    if (certs) {
+	rvCert = nssCertificateArray_FindBestCertificate(certs,
+	                                                 timeOpt,
+	                                                 usage,
+	                                                 policiesOpt);
+	nssCertificateArray_Destroy(certs);
     }
-    return best.cert;
+    return rvCert;
 }
 
 NSS_IMPLEMENT NSSCertificate **
-NSSCryptoContext_FindCertificatesBySubject
-(
+nssCryptoContext_FindCertificatesBySubject (
   NSSCryptoContext *cc,
   NSSDER *subject,
   NSSCertificate *rvOpt[],
@@ -348,9 +336,22 @@ NSSCryptoContext_FindCertificatesBySubject
     return rvCerts;
 }
 
+NSS_IMPLEMENT NSSCertificate **
+NSSCryptoContext_FindCertificatesBySubject (
+  NSSCryptoContext *cc,
+  NSSDER *subject,
+  NSSCertificate *rvOpt[],
+  PRUint32 maximumOpt, /* 0 for no max */
+  NSSArena *arenaOpt
+)
+{
+    return nssCryptoContext_FindCertificatesBySubject(cc, subject,
+                                                      rvOpt, maximumOpt,
+                                                      arenaOpt);
+}
+
 NSS_IMPLEMENT NSSCertificate *
-NSSCryptoContext_FindBestCertificateByNameComponents
-(
+NSSCryptoContext_FindBestCertificateByNameComponents (
   NSSCryptoContext *cc,
   NSSUTF8 *nameComponents,
   NSSTime *timeOpt,
@@ -363,8 +364,7 @@ NSSCryptoContext_FindBestCertificateByNameComponents
 }
 
 NSS_IMPLEMENT NSSCertificate **
-NSSCryptoContext_FindCertificatesByNameComponents
-(
+NSSCryptoContext_FindCertificatesByNameComponents (
   NSSCryptoContext *cc,
   NSSUTF8 *nameComponents,
   NSSCertificate *rvOpt[],
@@ -377,8 +377,7 @@ NSSCryptoContext_FindCertificatesByNameComponents
 }
 
 NSS_IMPLEMENT NSSCertificate *
-NSSCryptoContext_FindCertificateByEncodedCertificate
-(
+NSSCryptoContext_FindCertificateByEncodedCertificate (
   NSSCryptoContext *cc,
   NSSBER *encodedCertificate
 )
@@ -392,8 +391,7 @@ NSSCryptoContext_FindCertificateByEncodedCertificate
 }
 
 NSS_IMPLEMENT NSSCertificate *
-NSSCryptoContext_FindBestCertificateByEmail
-(
+NSSCryptoContext_FindBestCertificateByEmail (
   NSSCryptoContext *cc,
   NSSASCII7 *email,
   NSSTime *timeOpt,
@@ -401,40 +399,26 @@ NSSCryptoContext_FindBestCertificateByEmail
   NSSPolicies *policiesOpt
 )
 {
-    PRIntn i;
-    NSSCertificate *c;
-    NSSCertificate **emailCerts;
-    nssBestCertificateCB best;
+    NSSCertificate **certs;
+    NSSCertificate *rvCert = NULL;
     if (!cc->certStore) {
 	return NULL;
     }
-    nssBestCertificate_SetArgs(&best, timeOpt, usage, policiesOpt);
-    emailCerts = nssCertificateStore_FindCertificatesByEmail(cc->certStore,
-                                                             email,
-                                                             NULL,
-                                                             0,
-                                                             NULL);
-    if (emailCerts) {
-	PRStatus nssrv;
-	for (i=0, c = *emailCerts; c != NULL; c = emailCerts[++i]) {
-	    nssrv = nssBestCertificate_Callback(c, &best);
-	    NSSCertificate_Destroy(c);
-	    if (nssrv != PR_SUCCESS) {
-		if (best.cert) {
-		    NSSCertificate_Destroy(best.cert);
-		    best.cert = NULL;
-		}
-		break;
-	    }
-	}
-	nss_ZFreeIf(emailCerts);
+    certs = nssCertificateStore_FindCertificatesByEmail(cc->certStore,
+                                                        email,
+                                                        NULL, 0, NULL);
+    if (certs) {
+	rvCert = nssCertificateArray_FindBestCertificate(certs,
+	                                                 timeOpt,
+	                                                 usage,
+	                                                 policiesOpt);
+	nssCertificateArray_Destroy(certs);
     }
-    return best.cert;
+    return rvCert;
 }
 
 NSS_IMPLEMENT NSSCertificate **
-NSSCryptoContext_FindCertificatesByEmail
-(
+NSSCryptoContext_FindCertificatesByEmail (
   NSSCryptoContext *cc,
   NSSASCII7 *email,
   NSSCertificate *rvOpt[],
@@ -455,8 +439,7 @@ NSSCryptoContext_FindCertificatesByEmail
 }
 
 NSS_IMPLEMENT NSSCertificate *
-NSSCryptoContext_FindCertificateByOCSPHash
-(
+NSSCryptoContext_FindCertificateByOCSPHash (
   NSSCryptoContext *cc,
   NSSItem *hash
 )
@@ -466,8 +449,7 @@ NSSCryptoContext_FindCertificateByOCSPHash
 }
 
 NSS_IMPLEMENT NSSCertificate *
-NSSCryptoContext_FindBestUserCertificate
-(
+NSSCryptoContext_FindBestUserCertificate (
   NSSCryptoContext *cc,
   NSSTime *timeOpt,
   NSSUsage *usage,
@@ -479,8 +461,7 @@ NSSCryptoContext_FindBestUserCertificate
 }
 
 NSS_IMPLEMENT NSSCertificate **
-NSSCryptoContext_FindUserCertificates
-(
+NSSCryptoContext_FindUserCertificates (
   NSSCryptoContext *cc,
   NSSTime *timeOpt,
   NSSUsage *usageOpt,
@@ -495,8 +476,7 @@ NSSCryptoContext_FindUserCertificates
 }
 
 NSS_IMPLEMENT NSSCertificate *
-NSSCryptoContext_FindBestUserCertificateForSSLClientAuth
-(
+NSSCryptoContext_FindBestUserCertificateForSSLClientAuth (
   NSSCryptoContext *cc,
   NSSUTF8 *sslHostOpt,
   NSSDER *rootCAsOpt[], /* null pointer for none */
@@ -510,8 +490,7 @@ NSSCryptoContext_FindBestUserCertificateForSSLClientAuth
 }
 
 NSS_IMPLEMENT NSSCertificate **
-NSSCryptoContext_FindUserCertificatesForSSLClientAuth
-(
+NSSCryptoContext_FindUserCertificatesForSSLClientAuth (
   NSSCryptoContext *cc,
   NSSUTF8 *sslHostOpt,
   NSSDER *rootCAsOpt[], /* null pointer for none */
@@ -528,8 +507,7 @@ NSSCryptoContext_FindUserCertificatesForSSLClientAuth
 }
 
 NSS_IMPLEMENT NSSCertificate *
-NSSCryptoContext_FindBestUserCertificateForEmailSigning
-(
+NSSCryptoContext_FindBestUserCertificateForEmailSigning (
   NSSCryptoContext *cc,
   NSSASCII7 *signerOpt,
   NSSASCII7 *recipientOpt,
@@ -543,8 +521,7 @@ NSSCryptoContext_FindBestUserCertificateForEmailSigning
 }
 
 NSS_IMPLEMENT NSSCertificate *
-NSSCryptoContext_FindUserCertificatesForEmailSigning
-(
+NSSCryptoContext_FindUserCertificatesForEmailSigning (
   NSSCryptoContext *cc,
   NSSASCII7 *signerOpt, /* fgmr or a more general name? */
   NSSASCII7 *recipientOpt,
@@ -561,8 +538,7 @@ NSSCryptoContext_FindUserCertificatesForEmailSigning
 }
 
 NSS_IMPLEMENT NSSTrust *
-nssCryptoContext_FindTrustForCertificate
-(
+nssCryptoContext_FindTrustForCertificate (
   NSSCryptoContext *cc,
   NSSCertificate *cert
 )
@@ -574,8 +550,7 @@ nssCryptoContext_FindTrustForCertificate
 }
 
 NSS_IMPLEMENT nssSMIMEProfile *
-nssCryptoContext_FindSMIMEProfileForCertificate
-(
+nssCryptoContext_FindSMIMEProfileForCertificate (
   NSSCryptoContext *cc,
   NSSCertificate *cert
 )
@@ -588,8 +563,7 @@ nssCryptoContext_FindSMIMEProfileForCertificate
 }
 
 NSS_IMPLEMENT PRStatus
-NSSCryptoContext_GenerateKeyPair
-(
+NSSCryptoContext_GenerateKeyPair (
   NSSCryptoContext *cc,
   NSSAlgorithmAndParameters *ap,
   NSSPrivateKey **pvkOpt,
@@ -604,8 +578,7 @@ NSSCryptoContext_GenerateKeyPair
 }
 
 NSS_IMPLEMENT NSSSymmetricKey *
-NSSCryptoContext_GenerateSymmetricKey
-(
+NSSCryptoContext_GenerateSymmetricKey (
   NSSCryptoContext *cc,
   NSSAlgorithmAndParameters *ap,
   PRUint32 keysize,
@@ -618,8 +591,7 @@ NSSCryptoContext_GenerateSymmetricKey
 }
 
 NSS_IMPLEMENT NSSSymmetricKey *
-NSSCryptoContext_GenerateSymmetricKeyFromPassword
-(
+NSSCryptoContext_GenerateSymmetricKeyFromPassword (
   NSSCryptoContext *cc,
   NSSAlgorithmAndParameters *ap,
   NSSUTF8 *passwordOpt, /* if null, prompt */
@@ -632,8 +604,7 @@ NSSCryptoContext_GenerateSymmetricKeyFromPassword
 }
 
 NSS_IMPLEMENT NSSSymmetricKey *
-NSSCryptoContext_FindSymmetricKeyByAlgorithmAndKeyID
-(
+NSSCryptoContext_FindSymmetricKeyByAlgorithmAndKeyID (
   NSSCryptoContext *cc,
   NSSOID *algorithm,
   NSSItem *keyID,
@@ -649,34 +620,8 @@ struct token_session_str {
     nssSession *session;
 };
 
-#ifdef nodef
-static nssSession *
-get_token_session(NSSCryptoContext *cc, NSSToken *tok)
-{
-    struct token_session_str *ts;
-    for (ts  = (struct token_session_str *)nssListIterator_Start(cc->sessions);
-         ts != (struct token_session_str *)NULL;
-         ts  = (struct token_session_str *)nssListIterator_Next(cc->sessions))
-    {
-	if (ts->token == tok) { /* will this need to be more general? */
-	    break;
-	}
-    }
-    nssListIterator_Finish(cc->sessions);
-    if (!ts) {
-	/* need to create a session for this token. */
-	ts = nss_ZNEW(NULL, struct token_session_str);
-	ts->token = nssToken_AddRef(tok);
-	ts->session = nssSlot_CreateSession(tok->slot, cc->arena, PR_FALSE);
-	nssList_AddElement(cc->sessionList, (void *)ts);
-    }
-    return ts->session;
-}
-#endif
-
 NSS_IMPLEMENT NSSItem *
-NSSCryptoContext_Decrypt
-(
+NSSCryptoContext_Decrypt (
   NSSCryptoContext *cc,
   NSSAlgorithmAndParameters *apOpt,
   NSSItem *encryptedData,
@@ -685,64 +630,12 @@ NSSCryptoContext_Decrypt
   NSSArena *arenaOpt
 )
 {
-#if 0
-    NSSToken *tok;
-    nssSession *session;
-    NSSItem *rvData;
-    PRUint32 dataLen;
-    NSSAlgorithmAndParameters *ap;
-    CK_RV ckrv;
-    ap = (apOpt) ? apOpt : cc->defaultAlgorithm;
-    /* Get the token for this operation */
-    tok = nssTrustDomain_GetCryptoToken(cc->trustDomain, ap);
-    if (!tok) {
-	return (NSSItem *)NULL;
-    }
-    /* Get the local session for this token */
-    session = get_token_session(cc, tok);
-    /* Get the key needed to decrypt */
-    keyHandle = get_decrypt_key(cc, ap);
-    /* Set up the decrypt operation */
-    ckrv = CKAPI(tok)->C_DecryptInit(session->handle,
-                                     &ap->mechanism, keyHandle);
-    if (ckrv != CKR_OK) {
-	/* handle PKCS#11 error */
-	return (NSSItem *)NULL;
-    }
-    /* Get the length of the output buffer */
-    ckrv = CKAPI(tok)->C_Decrypt(session->handle,
-                                 (CK_BYTE_PTR)encryptedData->data,
-                                 (CK_ULONG)encryptedData->size,
-                                 (CK_BYTE_PTR)NULL,
-                                 (CK_ULONG_PTR)&dataLen);
-    if (ckrv != CKR_OK) {
-	/* handle PKCS#11 error */
-	return (NSSItem *)NULL;
-    }
-    /* Alloc return value memory */
-    rvData = nssItem_Create(NULL, NULL, dataLen, NULL);
-    if (!rvItem) {
-	return (NSSItem *)NULL;
-    }
-    /* Do the decryption */
-    ckrv = CKAPI(tok)->C_Decrypt(cc->session->handle,
-                                 (CK_BYTE_PTR)encryptedData->data,
-                                 (CK_ULONG)encryptedData->size,
-                                 (CK_BYTE_PTR)rvData->data,
-                                 (CK_ULONG_PTR)&dataLen);
-    if (ckrv != CKR_OK) {
-	/* handle PKCS#11 error */
-	nssItem_ZFreeIf(rvData);
-	return (NSSItem *)NULL;
-    }
-    return rvData;
-#endif
+    nss_SetError(NSS_ERROR_NOT_FOUND);
     return NULL;
 }
 
 NSS_IMPLEMENT PRStatus
-NSSCryptoContext_BeginDecrypt
-(
+NSSCryptoContext_BeginDecrypt (
   NSSCryptoContext *cc,
   NSSAlgorithmAndParameters *apOpt,
   NSSCallback *uhhOpt
@@ -753,8 +646,7 @@ NSSCryptoContext_BeginDecrypt
 }
 
 NSS_IMPLEMENT NSSItem *
-NSSCryptoContext_ContinueDecrypt
-(
+NSSCryptoContext_ContinueDecrypt (
   NSSCryptoContext *cc,
   NSSItem *data,
   NSSItem *rvOpt,
@@ -766,8 +658,7 @@ NSSCryptoContext_ContinueDecrypt
 }
 
 NSS_IMPLEMENT NSSItem *
-NSSCryptoContext_FinishDecrypt
-(
+NSSCryptoContext_FinishDecrypt (
   NSSCryptoContext *cc,
   NSSItem *rvOpt,
   NSSArena *arenaOpt
@@ -778,8 +669,7 @@ NSSCryptoContext_FinishDecrypt
 }
 
 NSS_IMPLEMENT NSSItem *
-NSSCryptoContext_Sign
-(
+NSSCryptoContext_Sign (
   NSSCryptoContext *cc,
   NSSAlgorithmAndParameters *apOpt,
   NSSItem *data,
@@ -793,8 +683,7 @@ NSSCryptoContext_Sign
 }
 
 NSS_IMPLEMENT PRStatus
-NSSCryptoContext_BeginSign
-(
+NSSCryptoContext_BeginSign (
   NSSCryptoContext *cc,
   NSSAlgorithmAndParameters *apOpt,
   NSSCallback *uhhOpt
@@ -805,8 +694,7 @@ NSSCryptoContext_BeginSign
 }
 
 NSS_IMPLEMENT PRStatus
-NSSCryptoContext_ContinueSign
-(
+NSSCryptoContext_ContinueSign (
   NSSCryptoContext *cc,
   NSSItem *data
 )
@@ -816,8 +704,7 @@ NSSCryptoContext_ContinueSign
 }
 
 NSS_IMPLEMENT NSSItem *
-NSSCryptoContext_FinishSign
-(
+NSSCryptoContext_FinishSign (
   NSSCryptoContext *cc,
   NSSItem *rvOpt,
   NSSArena *arenaOpt
@@ -828,8 +715,7 @@ NSSCryptoContext_FinishSign
 }
 
 NSS_IMPLEMENT NSSItem *
-NSSCryptoContext_SignRecover
-(
+NSSCryptoContext_SignRecover (
   NSSCryptoContext *cc,
   NSSAlgorithmAndParameters *apOpt,
   NSSItem *data,
@@ -843,8 +729,7 @@ NSSCryptoContext_SignRecover
 }
 
 NSS_IMPLEMENT PRStatus
-NSSCryptoContext_BeginSignRecover
-(
+NSSCryptoContext_BeginSignRecover (
   NSSCryptoContext *cc,
   NSSAlgorithmAndParameters *apOpt,
   NSSCallback *uhhOpt
@@ -855,8 +740,7 @@ NSSCryptoContext_BeginSignRecover
 }
 
 NSS_IMPLEMENT NSSItem *
-NSSCryptoContext_ContinueSignRecover
-(
+NSSCryptoContext_ContinueSignRecover (
   NSSCryptoContext *cc,
   NSSItem *data,
   NSSItem *rvOpt,
@@ -868,8 +752,7 @@ NSSCryptoContext_ContinueSignRecover
 }
 
 NSS_IMPLEMENT NSSItem *
-NSSCryptoContext_FinishSignRecover
-(
+NSSCryptoContext_FinishSignRecover (
   NSSCryptoContext *cc,
   NSSItem *rvOpt,
   NSSArena *arenaOpt
@@ -880,8 +763,7 @@ NSSCryptoContext_FinishSignRecover
 }
 
 NSS_IMPLEMENT NSSSymmetricKey *
-NSSCryptoContext_UnwrapSymmetricKey
-(
+NSSCryptoContext_UnwrapSymmetricKey (
   NSSCryptoContext *cc,
   NSSAlgorithmAndParameters *apOpt,
   NSSItem *wrappedKey,
@@ -893,8 +775,7 @@ NSSCryptoContext_UnwrapSymmetricKey
 }
 
 NSS_IMPLEMENT NSSSymmetricKey *
-NSSCryptoContext_DeriveSymmetricKey
-(
+NSSCryptoContext_DeriveSymmetricKey (
   NSSCryptoContext *cc,
   NSSPublicKey *bk,
   NSSAlgorithmAndParameters *apOpt,
@@ -909,8 +790,7 @@ NSSCryptoContext_DeriveSymmetricKey
 }
 
 NSS_IMPLEMENT NSSItem *
-NSSCryptoContext_Encrypt
-(
+NSSCryptoContext_Encrypt (
   NSSCryptoContext *cc,
   NSSAlgorithmAndParameters *apOpt,
   NSSItem *data,
@@ -924,8 +804,7 @@ NSSCryptoContext_Encrypt
 }
 
 NSS_IMPLEMENT PRStatus
-NSSCryptoContext_BeginEncrypt
-(
+NSSCryptoContext_BeginEncrypt (
   NSSCryptoContext *cc,
   NSSAlgorithmAndParameters *apOpt,
   NSSCallback *uhhOpt
@@ -936,8 +815,7 @@ NSSCryptoContext_BeginEncrypt
 }
 
 NSS_IMPLEMENT NSSItem *
-NSSCryptoContext_ContinueEncrypt
-(
+NSSCryptoContext_ContinueEncrypt (
   NSSCryptoContext *cc,
   NSSItem *data,
   NSSItem *rvOpt,
@@ -949,8 +827,7 @@ NSSCryptoContext_ContinueEncrypt
 }
 
 NSS_IMPLEMENT NSSItem *
-NSSCryptoContext_FinishEncrypt
-(
+NSSCryptoContext_FinishEncrypt (
   NSSCryptoContext *cc,
   NSSItem *rvOpt,
   NSSArena *arenaOpt
@@ -961,8 +838,7 @@ NSSCryptoContext_FinishEncrypt
 }
 
 NSS_IMPLEMENT PRStatus
-NSSCryptoContext_Verify
-(
+NSSCryptoContext_Verify (
   NSSCryptoContext *cc,
   NSSAlgorithmAndParameters *apOpt,
   NSSItem *data,
@@ -975,8 +851,7 @@ NSSCryptoContext_Verify
 }
 
 NSS_IMPLEMENT PRStatus
-NSSCryptoContext_BeginVerify
-(
+NSSCryptoContext_BeginVerify (
   NSSCryptoContext *cc,
   NSSAlgorithmAndParameters *apOpt,
   NSSItem *signature,
@@ -988,8 +863,7 @@ NSSCryptoContext_BeginVerify
 }
 
 NSS_IMPLEMENT PRStatus
-NSSCryptoContext_ContinueVerify
-(
+NSSCryptoContext_ContinueVerify (
   NSSCryptoContext *cc,
   NSSItem *data
 )
@@ -999,8 +873,7 @@ NSSCryptoContext_ContinueVerify
 }
 
 NSS_IMPLEMENT PRStatus
-NSSCryptoContext_FinishVerify
-(
+NSSCryptoContext_FinishVerify (
   NSSCryptoContext *cc
 )
 {
@@ -1009,8 +882,7 @@ NSSCryptoContext_FinishVerify
 }
 
 NSS_IMPLEMENT NSSItem *
-NSSCryptoContext_VerifyRecover
-(
+NSSCryptoContext_VerifyRecover (
   NSSCryptoContext *cc,
   NSSAlgorithmAndParameters *apOpt,
   NSSItem *signature,
@@ -1024,8 +896,7 @@ NSSCryptoContext_VerifyRecover
 }
 
 NSS_IMPLEMENT PRStatus
-NSSCryptoContext_BeginVerifyRecover
-(
+NSSCryptoContext_BeginVerifyRecover (
   NSSCryptoContext *cc,
   NSSAlgorithmAndParameters *apOpt,
   NSSCallback *uhhOpt
@@ -1036,8 +907,7 @@ NSSCryptoContext_BeginVerifyRecover
 }
 
 NSS_IMPLEMENT NSSItem *
-NSSCryptoContext_ContinueVerifyRecover
-(
+NSSCryptoContext_ContinueVerifyRecover (
   NSSCryptoContext *cc,
   NSSItem *data,
   NSSItem *rvOpt,
@@ -1049,8 +919,7 @@ NSSCryptoContext_ContinueVerifyRecover
 }
 
 NSS_IMPLEMENT NSSItem *
-NSSCryptoContext_FinishVerifyRecover
-(
+NSSCryptoContext_FinishVerifyRecover (
   NSSCryptoContext *cc,
   NSSItem *rvOpt,
   NSSArena *arenaOpt
@@ -1061,8 +930,7 @@ NSSCryptoContext_FinishVerifyRecover
 }
 
 NSS_IMPLEMENT NSSItem *
-NSSCryptoContext_WrapSymmetricKey
-(
+NSSCryptoContext_WrapSymmetricKey (
   NSSCryptoContext *cc,
   NSSAlgorithmAndParameters *apOpt,
   NSSSymmetricKey *keyToWrap,
@@ -1076,8 +944,7 @@ NSSCryptoContext_WrapSymmetricKey
 }
 
 NSS_IMPLEMENT NSSItem *
-NSSCryptoContext_Digest
-(
+NSSCryptoContext_Digest (
   NSSCryptoContext *cc,
   NSSAlgorithmAndParameters *apOpt,
   NSSItem *data,
@@ -1091,8 +958,7 @@ NSSCryptoContext_Digest
 }
 
 NSS_IMPLEMENT PRStatus
-NSSCryptoContext_BeginDigest
-(
+NSSCryptoContext_BeginDigest (
   NSSCryptoContext *cc,
   NSSAlgorithmAndParameters *apOpt,
   NSSCallback *uhhOpt
@@ -1102,8 +968,7 @@ NSSCryptoContext_BeginDigest
 }
 
 NSS_IMPLEMENT PRStatus
-NSSCryptoContext_ContinueDigest
-(
+NSSCryptoContext_ContinueDigest (
   NSSCryptoContext *cc,
   NSSAlgorithmAndParameters *apOpt,
   NSSItem *item
@@ -1118,8 +983,7 @@ NSSCryptoContext_ContinueDigest
 }
 
 NSS_IMPLEMENT NSSItem *
-NSSCryptoContext_FinishDigest
-(
+NSSCryptoContext_FinishDigest (
   NSSCryptoContext *cc,
   NSSItem *rvOpt,
   NSSArena *arenaOpt
@@ -1129,8 +993,7 @@ NSSCryptoContext_FinishDigest
 }
 
 NSS_IMPLEMENT NSSCryptoContext *
-NSSCryptoContext_Clone
-(
+NSSCryptoContext_Clone (
   NSSCryptoContext *cc
 )
 {
