@@ -32,7 +32,7 @@
  */
 
 #ifdef DEBUG
-static const char CVS_ID[] = "@(#) $RCSfile: pki3hack.c,v $ $Revision: 1.79 $ $Date: 2003/11/15 00:09:51 $ $Name: MOZILLA_1_7_3_RELEASE $";
+static const char CVS_ID[] = "@(#) $RCSfile: pki3hack.c,v $ $Revision: 1.79.16.1 $ $Date: 2004/10/15 23:52:34 $ $Name: MOZILLA_1_7_8_RELEASE $";
 #endif /* DEBUG */
 
 /*
@@ -90,13 +90,40 @@ STAN_GetDefaultCryptoContext()
 extern const NSSError NSS_ERROR_ALREADY_INITIALIZED;
 extern const NSSError NSS_ERROR_INTERNAL_ERROR;
 
+NSS_IMPLEMENT PRStatus
+STAN_InitTokenForSlotInfo(NSSTrustDomain *td, PK11SlotInfo *slot)
+{
+    NSSToken *token;
+    if (!td) {
+	td = g_default_trust_domain;
+    }
+    token = nssToken_CreateFromPK11SlotInfo(td, slot);
+    PK11Slot_SetNSSToken(slot, token);
+    NSSRWLock_LockWrite(td->tokensLock);
+    nssList_Add(td->tokenList, token);
+    NSSRWLock_UnlockWrite(td->tokensLock);
+    return PR_SUCCESS;
+}
+
+NSS_IMPLEMENT PRStatus
+STAN_ResetTokenInterator(NSSTrustDomain *td)
+{
+    if (!td) {
+	td = g_default_trust_domain;
+    }
+    NSSRWLock_LockWrite(td->tokensLock);
+    nssListIterator_Destroy(td->tokens);
+    td->tokens = nssList_CreateIterator(td->tokenList);
+    NSSRWLock_UnlockWrite(td->tokensLock);
+    return PR_SUCCESS;
+}
+
 NSS_IMPLEMENT PRStatus
 STAN_LoadDefaultNSS3TrustDomain (
   void
 )
 {
     NSSTrustDomain *td;
-    NSSToken *token;
     SECMODModuleList *mlp;
     SECMODListLock *moduleLock = SECMOD_GetDefaultModuleListLock();
     int i;
@@ -110,41 +137,48 @@ STAN_LoadDefaultNSS3TrustDomain (
     if (!td) {
 	return PR_FAILURE;
     }
-    td->tokenList = nssList_Create(td->arena, PR_TRUE);
+    /*
+     * Deadlock warning: we should never acquire the moduleLock while
+     * we hold the tokensLock. We can use the NSSRWLock Rank feature to
+     * guarrentee this. tokensLock have a higher rank than module lock.
+     */
     SECMOD_GetReadLock(moduleLock);
+    NSSRWLock_LockWrite(td->tokensLock);
+    td->tokenList = nssList_Create(td->arena, PR_TRUE);
     for (mlp = SECMOD_GetDefaultModuleList(); mlp != NULL; mlp=mlp->next) {
 	for (i=0; i < mlp->module->slotCount; i++) {
-	    token = nssToken_CreateFromPK11SlotInfo(td, mlp->module->slots[i]);
-	    PK11Slot_SetNSSToken(mlp->module->slots[i], token);
-	    nssList_Add(td->tokenList, token);
+	    STAN_InitTokenForSlotInfo(td, mlp->module->slots[i]);
 	}
     }
-    SECMOD_ReleaseReadLock(moduleLock);
     td->tokens = nssList_CreateIterator(td->tokenList);
+    NSSRWLock_UnlockWrite(td->tokensLock);
+    SECMOD_ReleaseReadLock(moduleLock);
     g_default_trust_domain = td;
     g_default_crypto_context = NSSTrustDomain_CreateCryptoContext(td, NULL);
     return PR_SUCCESS;
 }
 
+/*
+ * must be called holding the ModuleListLock (either read or write).
+ */
 NSS_IMPLEMENT SECStatus
 STAN_AddModuleToDefaultTrustDomain (
   SECMODModule *module
 )
 {
-    NSSToken *token;
     NSSTrustDomain *td;
     int i;
     td = STAN_GetDefaultTrustDomain();
     for (i=0; i<module->slotCount; i++) {
-	token = nssToken_CreateFromPK11SlotInfo(td, module->slots[i]);
-	PK11Slot_SetNSSToken(module->slots[i], token);
-	nssList_Add(td->tokenList, token);
+	STAN_InitTokenForSlotInfo(td, module->slots[i]);
     }
-    nssListIterator_Destroy(td->tokens);
-    td->tokens = nssList_CreateIterator(td->tokenList);
+    STAN_ResetTokenInterator(td);
     return SECSuccess;
 }
 
+/*
+ * must be called holding the ModuleListLock (either read or write).
+ */
 NSS_IMPLEMENT SECStatus
 STAN_RemoveModuleFromDefaultTrustDomain (
   SECMODModule *module
@@ -154,6 +188,7 @@ STAN_RemoveModuleFromDefaultTrustDomain (
     NSSTrustDomain *td;
     int i;
     td = STAN_GetDefaultTrustDomain();
+    NSSRWLock_LockWrite(td->tokensLock);
     for (i=0; i<module->slotCount; i++) {
 	token = PK11Slot_GetNSSToken(module->slots[i]);
 	if (token) {
@@ -165,6 +200,7 @@ STAN_RemoveModuleFromDefaultTrustDomain (
     }
     nssListIterator_Destroy(td->tokens);
     td->tokens = nssList_CreateIterator(td->tokenList);
+    NSSRWLock_UnlockWrite(td->tokensLock);
     return SECSuccess;
 }
 
@@ -792,6 +828,23 @@ STAN_GetCERTCertificate(NSSCertificate *c)
 {
     return stan_GetCERTCertificate(c, PR_FALSE);
 }
+/*
+ * Many callers of STAN_GetCERTCertificate() intend that
+ * the CERTCertificate returned inherits the reference to the 
+ * NSSCertificate. For these callers it's convenient to have 
+ * this function 'own' the reference and either return a valid 
+ * CERTCertificate structure which inherits the reference or 
+ * destroy the reference to NSSCertificate and returns NULL.
+ */
+NSS_IMPLEMENT CERTCertificate *
+STAN_GetCERTCertificateOrRelease(NSSCertificate *c)
+{
+    CERTCertificate *nss3cert = stan_GetCERTCertificate(c, PR_FALSE);
+    if (!nss3cert) {
+	nssCertificate_Destroy(c);
+    }
+    return nss3cert;
+}
 
 static nssTrustLevel
 get_stan_trust(unsigned int t, PRBool isClientAuth) 
@@ -989,9 +1042,11 @@ STAN_ChangeCertTrust(CERTCertificate *cc, CERTCertTrust *trust)
     tok = stan_GetTrustToken(c);
     moving_object = PR_FALSE;
     if (tok && PK11_IsReadOnly(tok->pk11slot))  {
+	NSSRWLock_LockRead(td->tokensLock);
 	tokens = nssList_CreateIterator(td->tokenList);
 	if (!tokens) {
 	    nssrv = PR_FAILURE;
+	    NSSRWLock_UnlockRead(td->tokensLock);
 	    goto done;
 	}
 	for (tok  = (NSSToken *)nssListIterator_Start(tokens);
@@ -1002,6 +1057,7 @@ STAN_ChangeCertTrust(CERTCertificate *cc, CERTCertTrust *trust)
 	}
 	nssListIterator_Finish(tokens);
 	nssListIterator_Destroy(tokens);
+	NSSRWLock_UnlockRead(td->tokensLock);
 	moving_object = PR_TRUE;
     } 
     if (tok) {
