@@ -76,8 +76,13 @@ fips_init()
   COPYDIR=${FIPSDIR}/copydir
 
   R_FIPSDIR=../fips
+  P_R_FIPSDIR=../fips
   R_COPYDIR=../fips/copydir
 
+  if [ -n "${MULTIACCESS_DBM}" ]; then
+     P_R_FIPSDIR="multiaccess:${D_FIPS}"
+  fi
+
   mkdir -p ${FIPSDIR}
   mkdir -p ${COPYDIR}
 
@@ -90,75 +95,105 @@ fips_init()
 ########################################################################
 fips_140_1()
 {
+  echo "$SCRIPTNAME: Verify this module is in FIPS mode  -----------------"
+  echo "modutil -dbdir ${P_R_FIPSDIR} -list"
+  modutil -dbdir ${P_R_FIPSDIR} -list 2>&1
+  modutil -dbdir ${P_R_FIPSDIR} -chkfips true 2>&1
+  html_msg $? 0 "Verify this module is in FIPS mode (modutil -chkfips true)"
+
   echo "$SCRIPTNAME: List the FIPS module certificates -----------------"
-  echo "certutil -d ${R_FIPSDIR} -L"
-  certutil -d ${R_FIPSDIR} -L 2>&1
+  echo "certutil -d ${P_R_FIPSDIR} -L"
+  certutil -d ${P_R_FIPSDIR} -L 2>&1
   html_msg $? 0 "List the FIPS module certificates (certutil -L)"
 
   echo "$SCRIPTNAME: List the FIPS module keys -------------------------"
-  echo "certutil -d ${R_FIPSDIR} -K -f ${R_FIPSPWFILE}"
-  certutil -d ${R_FIPSDIR} -K -f ${R_FIPSPWFILE} 2>&1
+  echo "certutil -d ${P_R_FIPSDIR} -K -f ${R_FIPSPWFILE}"
+  certutil -d ${P_R_FIPSDIR} -K -f ${R_FIPSPWFILE} 2>&1
   html_msg $? 0 "List the FIPS module keys (certutil -K)"
 
   echo "$SCRIPTNAME: Attempt to list FIPS module keys with incorrect password"
-  echo "certutil -d ${R_FIPSDIR} -K -f ${FIPSBADPWFILE}"
-  certutil -d ${R_FIPSDIR} -K -f ${FIPSBADPWFILE} 2>&1
+  echo "certutil -d ${P_R_FIPSDIR} -K -f ${FIPSBADPWFILE}"
+  certutil -d ${P_R_FIPSDIR} -K -f ${FIPSBADPWFILE} 2>&1
   RET=$?
   html_msg $RET 255 "Attempt to list FIPS module keys with incorrect password (certutil -K)"
   echo "certutil -K returned $RET"
 
   echo "$SCRIPTNAME: Validate the certificate --------------------------"
-  echo "certutil -d ${R_FIPSDIR} -V -n ${FIPSCERTNICK} -u SR -e -f ${R_FIPSPWFILE}"
-  certutil -d ${R_FIPSDIR} -V -n ${FIPSCERTNICK} -u SR -e -f ${R_FIPSPWFILE}
+  echo "certutil -d ${P_R_FIPSDIR} -V -n ${FIPSCERTNICK} -u SR -e -f ${R_FIPSPWFILE}"
+  certutil -d ${P_R_FIPSDIR} -V -n ${FIPSCERTNICK} -u SR -e -f ${R_FIPSPWFILE}
   html_msg $? 0 "Validate the certificate (certutil -V -e)"
 
   echo "$SCRIPTNAME: Export the certificate and key as a PKCS#12 file --"
-  echo "pk12util -d ${R_FIPSDIR} -o fips140.p12 -n ${FIPSCERTNICK} -w ${R_FIPSP12PWFILE} -k ${R_FIPSPWFILE}"
-  pk12util -d ${R_FIPSDIR} -o fips140.p12 -n ${FIPSCERTNICK} -w ${R_FIPSP12PWFILE} -k ${R_FIPSPWFILE} 2>&1
+  echo "pk12util -d ${P_R_FIPSDIR} -o fips140.p12 -n ${FIPSCERTNICK} -w ${R_FIPSP12PWFILE} -k ${R_FIPSPWFILE}"
+  pk12util -d ${P_R_FIPSDIR} -o fips140.p12 -n ${FIPSCERTNICK} -w ${R_FIPSP12PWFILE} -k ${R_FIPSPWFILE} 2>&1
   html_msg $? 0 "Export the certificate and key as a PKCS#12 file (pk12util -o)"
 
   echo "$SCRIPTNAME: Export the certificate as a DER-encoded file ------"
-  echo "certutil -d ${R_FIPSDIR} -L -n ${FIPSCERTNICK} -r -o fips140.crt"
-  certutil -d ${R_FIPSDIR} -L -n ${FIPSCERTNICK} -r -o fips140.crt 2>&1
+  echo "certutil -d ${P_R_FIPSDIR} -L -n ${FIPSCERTNICK} -r -o fips140.crt"
+  certutil -d ${P_R_FIPSDIR} -L -n ${FIPSCERTNICK} -r -o fips140.crt 2>&1
   html_msg $? 0 "Export the certificate as a DER (certutil -L -r)"
 
   echo "$SCRIPTNAME: List the FIPS module certificates -----------------"
-  echo "certutil -d ${R_FIPSDIR} -L"
-  certutil -d ${R_FIPSDIR} -L 2>&1
+  echo "certutil -d ${P_R_FIPSDIR} -L"
+  certutil -d ${P_R_FIPSDIR} -L 2>&1
   html_msg $? 0 "List the FIPS module certificates (certutil -L)"
 
   echo "$SCRIPTNAME: Delete the certificate and key from the FIPS module"
-  echo "certutil -d ${R_FIPSDIR} -F -n ${FIPSCERTNICK} -f ${R_FIPSPWFILE}"
-  certutil -d ${R_FIPSDIR} -F -n ${FIPSCERTNICK} -f ${R_FIPSPWFILE} 2>&1
+  echo "certutil -d ${P_R_FIPSDIR} -F -n ${FIPSCERTNICK} -f ${R_FIPSPWFILE}"
+  certutil -d ${P_R_FIPSDIR} -F -n ${FIPSCERTNICK} -f ${R_FIPSPWFILE} 2>&1
   html_msg $? 0 "Delete the certificate and key from the FIPS module (certutil -D)"
 
 
   echo "$SCRIPTNAME: List the FIPS module certificates -----------------"
-  echo "certutil -d ${R_FIPSDIR} -L"
-  certutil -d ${R_FIPSDIR} -L 2>&1
+  echo "certutil -d ${P_R_FIPSDIR} -L"
+  certutil -d ${P_R_FIPSDIR} -L 2>&1
   html_msg $? 0 "List the FIPS module certificates (certutil -L)"
 
   echo "$SCRIPTNAME: List the FIPS module keys."
-  echo "certutil -d ${R_FIPSDIR} -K -f ${R_FIPSPWFILE}"
-  certutil -d ${R_FIPSDIR} -K -f ${R_FIPSPWFILE} 2>&1
+  echo "certutil -d ${P_R_FIPSDIR} -K -f ${R_FIPSPWFILE}"
+  certutil -d ${P_R_FIPSDIR} -K -f ${R_FIPSPWFILE} 2>&1
   # certutil -K now returns a failure if no keys are found. This verifies that
   # our delete succeded.
   html_msg $? 255 "List the FIPS module keys (certutil -K)"
 
   echo "$SCRIPTNAME: Import the certificate and key from the PKCS#12 file"
-  echo "pk12util -d ${R_FIPSDIR} -i fips140.p12 -w ${R_FIPSP12PWFILE} -k ${R_FIPSPWFILE}"
-  pk12util -d ${R_FIPSDIR} -i fips140.p12 -w ${R_FIPSP12PWFILE} -k ${R_FIPSPWFILE} 2>&1
+  echo "pk12util -d ${P_R_FIPSDIR} -i fips140.p12 -w ${R_FIPSP12PWFILE} -k ${R_FIPSPWFILE}"
+  pk12util -d ${P_R_FIPSDIR} -i fips140.p12 -w ${R_FIPSP12PWFILE} -k ${R_FIPSPWFILE} 2>&1
   html_msg $? 0 "Import the certificate and key from the PKCS#12 file (pk12util -i)"
 
   echo "$SCRIPTNAME: List the FIPS module certificates -----------------"
-  echo "certutil -d ${R_FIPSDIR} -L"
-  certutil -d ${R_FIPSDIR} -L 2>&1
+  echo "certutil -d ${P_R_FIPSDIR} -L"
+  certutil -d ${P_R_FIPSDIR} -L 2>&1
   html_msg $? 0 "List the FIPS module certificates (certutil -L)"
 
   echo "$SCRIPTNAME: List the FIPS module keys --------------------------"
-  echo "certutil -d ${R_FIPSDIR} -K -f ${R_FIPSPWFILE}"
-  certutil -d ${R_FIPSDIR} -K -f ${R_FIPSPWFILE} 2>&1
+  echo "certutil -d ${P_R_FIPSDIR} -K -f ${R_FIPSPWFILE}"
+  certutil -d ${P_R_FIPSDIR} -K -f ${R_FIPSPWFILE} 2>&1
   html_msg $? 0 "List the FIPS module keys (certutil -K)"
+
+#
+# This test was interfering with QA running on more than one machine pointing
+# to the same binary file. Turn it off for now.
+#
+#  echo "$SCRIPTNAME: Detect mangled database --------------------------"
+#  SOFTOKEN=${DIST}/${OBJDIR}/lib/${DLL_PREFIX}softokn3.${DLL_SUFFIX}
+#  echo "cp ${SOFTOKEN} ${TMP}/softokn3.sav"
+#  cp ${SOFTOKEN} ${TMP}/softokn3.sav
+#  echo "mangling ${SOFTOKEN}"
+#  echo "mangle -i ${SOFTOKEN} -o 60000 -b 5"
+#  mangle -i ${SOFTOKEN} -o 60000 -b 5 2>&1
+#  if [ $? -eq 0 ]; then
+#    echo "dbtest -r  -d ${P_R_FIPSDIR} "
+# suppress the expected failure message
+#    dbtest -r  -d ${P_R_FIPSDIR}  > ${TMP}/dbtestoutput.txt 2>&1
+#    html_msg $? 46 "Init NSS with a corrupted library (dbtest -r)"
+#    echo "cp ${TMP}/softokn3.sav ${SOFTOKEN}"
+#    cp ${TMP}/softokn3.sav ${SOFTOKEN}
+#  else
+#    html_msg 0 0 "Skipping corruption test, can't open ${DLL_PREFIX}softokn3.${DLL_SUFFIX}"
+#  fi
+#  echo "rm ${TMP}/softokn3.sav"
+#  rm ${TMP}/softokn3.sav
 }
 
 ############################## fips_cleanup ############################
