@@ -18,7 +18,7 @@
  * 
  * Contributor(s):
  *   Travis Bogard <travis@netscape.com>
- *   Brian Ryner <bryner@netscape.com>
+ *   Brian Ryner <bryner@brianryner.com>
  */
 
 // Local Includes
@@ -52,8 +52,6 @@ static NS_DEFINE_CID(kAppShellCID, NS_APPSHELL_CID);
 nsWebBrowserChrome::nsWebBrowserChrome() : mBrowserWindow(nsnull), mTimerSet(PR_FALSE)
 
 {
-	NS_INIT_REFCNT();
-
    mActiveDocuments = 0;
    mChromeFlags = 0;
    mSizeSet = PR_FALSE;
@@ -159,38 +157,6 @@ NS_IMETHODIMP nsWebBrowserChrome::DestroyBrowserWindow()
    return mBrowserWindow->Destroy();
 }
 
-#if 0
-/* Just commenting out for now because it looks like somebody went to
-   a lot of work here. This method has been removed from nsIWebBrowserChrome
-   per the 5 Feb 01 API review, to be handled one level further down
-   in nsDocShellTreeOwner.
-*/
-NS_IMETHODIMP nsWebBrowserChrome::FindNamedBrowserItem(const PRUnichar* aName,
-   nsIDocShellTreeItem** aBrowserItem)
-{
-   NS_ENSURE_ARG_POINTER(aBrowserItem);
-   *aBrowserItem = nsnull;
-
-   PRInt32 i = 0;
-   PRInt32 n = mBrowserWindow->gBrowsers.Count();
-
-   nsString aNameStr(aName);
-
-   for (i = 0; i < n; i++)
-      {
-      nsBrowserWindow* bw = (nsBrowserWindow*)mBrowserWindow->gBrowsers.ElementAt(i);
-      nsCOMPtr<nsIDocShellTreeItem> docShellAsItem(do_QueryInterface(bw->mWebBrowser));
-      NS_ENSURE_TRUE(docShellAsItem, NS_ERROR_FAILURE);
-
-      docShellAsItem->FindItemWithName(aName, NS_STATIC_CAST(nsIWebBrowserChrome*, this), aBrowserItem);
-
-      if(!*aBrowserItem)
-         return NS_OK;
-      }
-   return NS_OK;
-}
-#endif
-
 NS_IMETHODIMP nsWebBrowserChrome::SizeBrowserTo(PRInt32 aCX, PRInt32 aCY)
 {
    mSizeSet = PR_TRUE;
@@ -322,8 +288,13 @@ NS_IMETHODIMP nsWebBrowserChrome::GetSiteWindow(void ** aParentNativeWindow)
 {
    NS_ENSURE_ARG_POINTER(aParentNativeWindow);
 
-   //XXX First Check In
-   NS_ASSERTION(PR_FALSE, "Not Yet Implemented");
+   if (mBrowserWindow) {
+     nsCOMPtr<nsIWidget> window;
+     mBrowserWindow->GetWindow(getter_AddRefs(window));
+     if (window) {
+       *aParentNativeWindow = window->GetNativeData(NS_NATIVE_WINDOW);
+     }
+   }
    return NS_OK;
 }
 
