@@ -48,7 +48,6 @@
 #include "nsCarbonHelpers.h"
 #include "nsGFXUtils.h"
 #include "nsMacResources.h"
-#include "nsRegionMac.h"
 #include "nsIRollupListener.h"
 #import "nsChildView.h"
 
@@ -84,7 +83,7 @@ static NS_DEFINE_CID(kCDragServiceCID,  NS_DRAGSERVICE_CID);
 extern nsIRollupListener * gRollupListener;
 extern nsIWidget         * gRollupWidget;
 
-PRBool OnMacOSX();
+static PRBool OnMacOSX();
 
 #define kWindowPositionSlop 20
 
@@ -281,7 +280,7 @@ nsCocoaWindow :: DragReceiveHandler (WindowPtr theWindow, void *handlerRefCon,
 #endif
 
 
-NS_IMPL_ISUPPORTS_INHERITED0(nsCocoaWindow, Inherited);
+NS_IMPL_ISUPPORTS_INHERITED0(nsCocoaWindow, Inherited)
 
 
 //-------------------------------------------------------------------------
@@ -324,8 +323,8 @@ nsCocoaWindow::nsCocoaWindow()
 nsCocoaWindow::~nsCocoaWindow()
 {
   if ( mWindow && mWindowMadeHere ) {
-    [mWindow release];
-    [mDelegate release];
+    [mWindow autorelease];
+    [mDelegate autorelease];
   }
   
 #if 0
@@ -370,11 +369,13 @@ nsresult nsCocoaWindow::StandardCreate(nsIWidget *aParent,
   Inherited::BaseCreate ( aParent, aRect, aHandleEventFunction, aContext, aAppShell,
                             aToolkit, aInitData );
                             
-  if ( !aNativeParent ) {
+  if (!aNativeParent || (aInitData && aInitData->mWindowType == eWindowType_popup))
+  {
     mOffsetParent = aParent;
 
     nsWindowType windowType = eWindowType_toplevel;
-    if (aInitData) {
+    if (aInitData)
+    {
       mWindowType = aInitData->mWindowType;
       // if a toplevel window was requested without a titlebar, use a dialog windowproc
       if (aInitData->mWindowType == eWindowType_toplevel &&
@@ -383,7 +384,9 @@ nsresult nsCocoaWindow::StandardCreate(nsIWidget *aParent,
         windowType = eWindowType_dialog;
     } 
     else
+    {
       mWindowType = (mIsDialog ? eWindowType_dialog : eWindowType_toplevel);
+    }
     
     // create the cocoa window
     NSRect rect;
@@ -393,16 +396,25 @@ nsresult nsCocoaWindow::StandardCreate(nsIWidget *aParent,
                               | NSResizableWindowMask;
     if ( mWindowType == eWindowType_popup || mWindowType == eWindowType_invisible )
       features = 0;
+
+    // XXXdwh Just don't make popup windows yet.  They mess up the world.
+    if (mWindowType == eWindowType_popup)
+      return NS_OK;
+
     mWindow = [[NSWindow alloc] initWithContentRect:rect styleMask:features 
                         backing:NSBackingStoreBuffered defer:NO];
     
+    // Popups will receive a "close" message when an app terminates
+    // that causes an extra release to occur.  Make sure popups
+    // are set not to release when closed.
+    if (features == 0)
+      [mWindow setReleasedWhenClosed: NO];
+
     // create a quickdraw view as the toplevel content view of the window
     NSQuickDrawView* content = [[[NSQuickDrawView alloc] init] autorelease];
     [content setFrame:[[mWindow contentView] frame]];
     [mWindow setContentView:content];
     
-    NSLog(@"New quickdraw view. %@", content);
-
     // register for mouse-moved events. The default is to ignore them for perf reasons.
     [mWindow setAcceptsMouseMovedEvents:YES];
     
@@ -873,7 +885,7 @@ NS_IMETHODIMP nsCocoaWindow::Show(PRBool bState)
     [mWindow orderFront:NULL];
   else
     [mWindow orderOut:NULL];
- 
+
   mVisible = bState;
 
 #if 0
@@ -1141,7 +1153,8 @@ NS_IMETHODIMP nsCocoaWindow::Move(PRInt32 aX, PRInt32 aY)
 // Position the window behind the given window
 //
 //-------------------------------------------------------------------------
-NS_METHOD nsCocoaWindow::PlaceBehind(nsIWidget *aWidget, PRBool aActivate)
+NS_METHOD nsCocoaWindow::PlaceBehind(nsTopLevelWidgetZPlacement aPlacement,
+                                     nsIWidget *aWidget, PRBool aActivate)
 {
 #if 0
   if (aWidget) {
@@ -1334,7 +1347,6 @@ NS_IMETHODIMP nsCocoaWindow::Resize(PRInt32 aX, PRInt32 aY, PRInt32 aWidth, PRIn
 //-------------------------------------------------------------------------
 NS_IMETHODIMP nsCocoaWindow::Resize(PRInt32 aWidth, PRInt32 aHeight, PRBool aRepaint)
 {
-printf("resizing to %d %d\n", aWidth, aHeight);
   if ( mWindow ) {
     NSRect newBounds = [mWindow frame];
     newBounds.size.width = aWidth;
@@ -1490,20 +1502,13 @@ PRBool nsCocoaWindow::DragEvent ( unsigned int aMessage, Point aMouseGlobal, UIn
 //-------------------------------------------------------------------------
 void nsCocoaWindow::ComeToFront() {
 #if 0
-  nsZLevelEvent  event;
+  nsZLevelEvent  event(NS_SETZLEVEL, this);
 
   event.point.x = mBounds.x;
   event.point.y = mBounds.y;
   event.time = PR_IntervalNow();
-  event.widget = this;
-  event.nativeMsg = nsnull;
-  event.eventStructType = NS_ZLEVEL_EVENT;
-  event.message = NS_SETZLEVEL;
 
-  event.mPlacement = nsWindowZTop;
-  event.mReqBelow = 0;
   event.mImmediate = PR_TRUE;
-  event.mAdjusted = PR_FALSE;
 
   DispatchWindowEvent(event);
 #endif
@@ -1530,7 +1535,7 @@ void nsCocoaWindow::IsActive(PRBool* aActive)
 //
 // Return true if we are on Mac OS X, caching the result after the first call
 //
-PRBool
+static PRBool
 OnMacOSX()
 {
 
@@ -1563,10 +1568,8 @@ nsCocoaWindow::DispatchEvent ( void* anEvent, void* aView, PRBool *_retval )
   
   ChildView* view = NS_REINTERPRET_CAST(ChildView*, aView);
   
-  nsMouseEvent geckoEvent;
-  geckoEvent.eventStructType = NS_MOUSE_EVENT;
+  nsMouseEvent geckoEvent(0, view ? [view widget] : this)
   
-  geckoEvent.widget = this;
   geckoEvent.nativeMsg = anEvent;
   geckoEvent.time = PR_IntervalNow();
   NSPoint mouseLoc = [event locationInWindow];
@@ -1574,8 +1577,6 @@ nsCocoaWindow::DispatchEvent ( void* anEvent, void* aView, PRBool *_retval )
   geckoEvent.refPoint.y = NS_STATIC_CAST(nscoord, mouseLoc.y);
 //printf("-- global mouse click at (%ld,%ld)\n", geckoEvent.refPoint.x, geckoEvent.refPoint.y );
   if ( view ) {
-    geckoEvent.widget = [view widget];
-    
     // convert point to view coordinate system
     NSPoint localPoint = [view convertPoint:mouseLoc fromView:nil];
     geckoEvent.point.x = NS_STATIC_CAST(nscoord, localPoint.x);
@@ -1663,17 +1664,9 @@ void
 nsCocoaWindow::ReportSizeEvent()
 {
   // nsEvent
-  nsSizeEvent sizeEvent;
-  sizeEvent.eventStructType = NS_SIZE_EVENT;
-  sizeEvent.message     = NS_SIZE;
-  sizeEvent.point.x     = 0;
-  sizeEvent.point.y     = 0;
+  nsSizeEvent sizeEvent(NS_SIZE, this);
   sizeEvent.time        = PR_IntervalNow();
 
-  // nsGUIEvent
-  sizeEvent.widget      = this;
-  sizeEvent.nativeMsg   = nsnull;
-
   // nsSizeEvent
   sizeEvent.windowSize  = &mBounds;
   sizeEvent.mWinWidth   = mBounds.width;
@@ -1719,7 +1712,6 @@ void StopResizing ( )
   return self;
 }
 
-
 - (void)windowDidResize:(NSNotification *)aNotification
 {
   if ( !mGeckoWindow->IsResizing() ) {
@@ -1735,31 +1727,30 @@ void StopResizing ( )
 
 - (void)windowDidBecomeMain:(NSNotification *)aNotification
 {
-  printf("got activation\n");
+  //printf("got activation\n");
 }
 
 
 - (void)windowDidResignMain:(NSNotification *)aNotification
 {
-  printf("got deactivate\n");
+  //printf("got deactivate\n");
 }
 
 
 - (void)windowDidBecomeKey:(NSNotification *)aNotification
 {
-  printf("we're key window\n");
+  //printf("we're key window\n");
 }
 
 
 - (void)windowDidResignKey:(NSNotification *)aNotification
 {
-  printf("we're not the key window\n");
+  //printf("we're not the key window\n");
 }
 
 
 - (void)windowDidMove:(NSNotification *)aNotification
 {
-  printf("did move\n");
 }
 
 @end
