@@ -47,17 +47,25 @@
 #include "nsVoidArray.h"
 
 #include "nsIServiceManager.h"
-#include "nsICharsetConverterManager.h"
 
 #include "prtime.h"
 #include "prthread.h"
 
+// For manipulation of the X event queue
+#include <sys/time.h>
+#include <sys/types.h>
+#include <unistd.h>
+#ifdef POLL_WITH_XCONNECTIONNUMBER
+#include <poll.h>
+#endif
+
 // unicode conversion
-#  include "nsIPlatformCharset.h"
+#include "nsIPlatformCharset.h"
+#include "nsICharsetConverterManager.h"
 
 static void ConvertHTMLtoUCS2(char* data, PRInt32 dataLength,
                               PRUnichar** unicodeData, PRInt32& outUnicodeLen);
-static void GetHTMLCharset   (char* data, PRInt32 dataLength, nsAString& str);
+static void GetHTMLCharset   (char* data, PRInt32 dataLength, nsACString& str);
 
 // The class statics:
 GtkWidget* nsClipboard::sWidget = 0;
@@ -70,7 +78,7 @@ GtkWidget* nsClipboard::sWidget = 0;
 
 static GdkAtom GDK_SELECTION_CLIPBOARD;
 
-NS_IMPL_ISUPPORTS1(nsClipboard, nsIClipboard);
+NS_IMPL_ISUPPORTS1(nsClipboard, nsIClipboard)
 
 //-------------------------------------------------------------------------
 //
@@ -303,7 +311,7 @@ NS_IMETHODIMP nsClipboard::SetNativeClipboardData(PRInt32 aWhichClipboard)
 
 
   GdkAtom selectionAtom = GetSelectionAtom(aWhichClipboard);
-  nsCOMPtr<nsITransferable> transferable(getter_AddRefs(GetTransferable(aWhichClipboard)));
+  nsCOMPtr<nsITransferable> transferable(GetTransferable(aWhichClipboard));
 
   // make sure we have a good transferable
   if (nsnull == transferable) {
@@ -604,7 +612,7 @@ nsClipboard::SelectionReceiver (GtkWidget *aWidget,
 
     nsresult rv;
     PRInt32 outUnicodeLen;
-    PRUnichar *unicodeData;
+    PRUnichar *unicodeData = nsnull;
 
 #ifdef DEBUG_CLIPBOARD
     g_print("        Converting from current locale to unicode\n");
@@ -612,17 +620,18 @@ nsClipboard::SelectionReceiver (GtkWidget *aWidget,
 
     nsCOMPtr<nsIUnicodeDecoder> decoder;
     // get the charset
-    nsAutoString platformCharset;
+    nsCAutoString platformCharset;
     nsCOMPtr <nsIPlatformCharset> platformCharsetService = do_GetService(NS_PLATFORMCHARSET_CONTRACTID, &rv);
     if (NS_SUCCEEDED(rv))
       rv = platformCharsetService->GetCharset(kPlatformCharsetSel_Menu, platformCharset);
     if (NS_FAILED(rv))
-      platformCharset.Assign(NS_LITERAL_STRING("ISO-8859-1"));
+      platformCharset.Assign(NS_LITERAL_CSTRING("ISO-8859-1"));
       
     // get the decoder
     nsCOMPtr<nsICharsetConverterManager> ccm = do_GetService(NS_CHARSETCONVERTERMANAGER_CONTRACTID, &rv);
-    rv = ccm->GetUnicodeDecoder(&platformCharset, getter_AddRefs(decoder));
-    NS_ASSERTION(NS_SUCCEEDED(rv), "GetUnicodeEncoder failed");
+    rv = ccm->GetUnicodeDecoderRaw(platformCharset.get(),
+                                   getter_AddRefs(decoder));
+    NS_ASSERTION(NS_SUCCEEDED(rv), "GetUnicodeEncoderRaw failed");
     if (NS_FAILED(rv)) {
       if (tmpData) XFreeStringList(tmpData);
       return;
@@ -655,7 +664,7 @@ nsClipboard::SelectionReceiver (GtkWidget *aWidget,
 
     nsresult rv;
     PRInt32 outUnicodeLen;
-    PRUnichar *unicodeData;
+    PRUnichar *unicodeData = nsnull;
 
     char *data = (char*)aSD->data;
     PRInt32 numberOfBytes = (PRInt32)aSD->length;
@@ -664,14 +673,10 @@ nsClipboard::SelectionReceiver (GtkWidget *aWidget,
     printf("UTF8_STRING is %s\nlength is %i\n", aSD->data, aSD->length);
 #endif
 
-    nsCOMPtr<nsIUnicodeDecoder> decoder;
-    // get the charset
-    nsAutoString platformCharset;
-    platformCharset.Assign(NS_LITERAL_STRING("UTF-8"));
-
     // get the decoder
+    nsCOMPtr<nsIUnicodeDecoder> decoder;
     nsCOMPtr<nsICharsetConverterManager> ccm = do_GetService(NS_CHARSETCONVERTERMANAGER_CONTRACTID, &rv);
-    rv = ccm->GetUnicodeDecoder(&platformCharset, getter_AddRefs(decoder));
+    rv = ccm->GetUnicodeDecoderRaw("UTF-8", getter_AddRefs(decoder));
 
     g_return_if_fail(NS_SUCCEEDED(rv));
 
@@ -747,7 +752,7 @@ NS_IMETHODIMP nsClipboard::ForceDataToClipboard(PRInt32 aWhichClipboard)
 
   // make sure we have a good transferable
 
-  nsCOMPtr<nsITransferable> transferable(getter_AddRefs(GetTransferable(aWhichClipboard)));
+  nsCOMPtr<nsITransferable> transferable(GetTransferable(aWhichClipboard));
 
   if (nsnull == transferable) {
     return NS_ERROR_FAILURE;
@@ -869,7 +874,7 @@ void nsClipboard::SelectionGetCB(GtkWidget        *widget,
 #ifdef DEBUG_CLIPBOARD
   g_print("  whichClipboard = %d\n", whichClipboard);
 #endif
-  nsCOMPtr<nsITransferable> transferable(getter_AddRefs(cb->GetTransferable(whichClipboard)));
+  nsCOMPtr<nsITransferable> transferable(cb->GetTransferable(whichClipboard));
 
   // Make sure we have a transferable:
   if (!transferable) {
@@ -971,17 +976,17 @@ void nsClipboard::SelectionGetCB(GtkWidget        *widget,
 
       nsCOMPtr<nsIUnicodeEncoder> encoder;
       // get the charset
-      nsAutoString platformCharset;
+      nsCAutoString platformCharset;
       nsCOMPtr <nsIPlatformCharset> platformCharsetService = do_GetService(NS_PLATFORMCHARSET_CONTRACTID, &rv);
       if (NS_SUCCEEDED(rv))
         rv = platformCharsetService->GetCharset(kPlatformCharsetSel_Menu, platformCharset);
       if (NS_FAILED(rv))
-        platformCharset.Assign(NS_LITERAL_STRING("ISO-8859-1"));
+        platformCharset.Assign(NS_LITERAL_CSTRING("ISO-8859-1"));
       
       // get the encoder
       nsCOMPtr<nsICharsetConverterManager> ccm = do_GetService(NS_CHARSETCONVERTERMANAGER_CONTRACTID, &rv);
-      rv = ccm->GetUnicodeEncoder(&platformCharset, getter_AddRefs(encoder));
-      NS_ASSERTION(NS_SUCCEEDED(rv), "GetUnicodeEncoder failed");
+      rv = ccm->GetUnicodeEncoderRaw(platformCharset.get(), getter_AddRefs(encoder));
+      NS_ASSERTION(NS_SUCCEEDED(rv), "GetUnicodeEncoderRaw failed");
       if (NS_FAILED(rv)) {
         nsMemory::Free(NS_REINTERPRET_CAST(char*, clipboardData));
         return;
@@ -1164,7 +1169,6 @@ nsITransferable *nsClipboard::GetTransferable(PRInt32 aWhichClipboard)
     transferable = mSelectionTransferable;
     break;
   }
-  NS_IF_ADDREF(transferable);
   return transferable;
 }
 
@@ -1272,7 +1276,7 @@ void nsClipboard::SetCutBuffer()
   PRUint32 dataLength;
   nsresult rv;
 
-  nsCOMPtr<nsITransferable> transferable(getter_AddRefs(GetTransferable(kGlobalClipboard)));
+  nsCOMPtr<nsITransferable> transferable(GetTransferable(kGlobalClipboard));
 
   // Make sure we have a transferable:
   if (!transferable) {
@@ -1303,76 +1307,119 @@ void nsClipboard::SetCutBuffer()
 #endif
 }
 
-void nsClipboard::SendClipPing()
-{
-#ifdef DEBUG_CLIPBOARD
-  printf("nsClipboard::SendClipPing()\n");
-#endif
-  XClientMessageEvent send_event;
-  send_event.type = ClientMessage;
-  send_event.message_type = XInternAtom(GDK_DISPLAY(), "_MOZ_CLIP_PING", PR_FALSE);
-  send_event.format = 32;
-  send_event.data.l[0] = CurrentTime;
-  send_event.window = GDK_WINDOW_XWINDOW(sWidget->window);
-
-  XSendEvent(GDK_DISPLAY(), GDK_WINDOW_XWINDOW(sWidget->window), PR_FALSE, 0, (XEvent*)&send_event);
-  XSync(GDK_DISPLAY(), False);
-}
-
-static Bool
-find_clipboard_event (Display  *display,
-                      XEvent   *xevent,
-                      XPointer  arg)
-{
-
-  GtkWidget *widget = (GtkWidget*) arg;
-
-  g_return_val_if_fail (widget != NULL, False);
-
-  if (xevent->xany.window == GDK_WINDOW_XWINDOW(widget->window) &&
-      (xevent->xany.type == SelectionNotify ||
-       xevent->xany.type == ClientMessage))
-    return True;
-  else
-    return False;
-}
-
-void send_selection_notify_to_widget(XEvent *xevent, GtkWidget *widget)
+static void
+DispatchSelectionNotifyEvent(GtkWidget *widget, XEvent *xevent)
 {
   GdkEvent event;
   event.selection.type = GDK_SELECTION_NOTIFY;
-  event.any.window = gdk_window_lookup(xevent->xany.window);
-  event.any.send_event = xevent->xany.send_event ? TRUE : FALSE;
-  event.selection.window = event.any.window;
+  event.selection.window = widget->window;
   event.selection.selection = xevent->xselection.selection;
   event.selection.target = xevent->xselection.target;
   event.selection.property = xevent->xselection.property;
   event.selection.time = xevent->xselection.time;
-  
+
   gtk_widget_event(widget, &event);
 }
 
-/* Maximum time to wait for a SelectionNotify event, in microseconds */
-#define NS_CLIPBOARD_TIMEOUT 500000
+static void
+DispatchPropertyNotifyEvent(GtkWidget *widget, XEvent *xevent)
+{
+  if (gdk_window_get_events(widget->window) & GDK_PROPERTY_CHANGE_MASK) {
+    GdkEvent event;
+    event.property.type = GDK_PROPERTY_NOTIFY;
+    event.property.window = widget->window;
+    event.property.atom = xevent->xproperty.atom;
+    event.property.time = xevent->xproperty.time;
+    event.property.state = xevent->xproperty.state;
+
+    gtk_widget_event(widget, &event);
+  }
+}
+
+struct checkEventContext
+{
+  GtkWidget *cbWidget;
+  Atom       selAtom;
+};
+
+static Bool
+checkEventProc(Display *display, XEvent *event, XPointer arg)
+{
+  checkEventContext *context = (checkEventContext *) arg;
+
+  if (event->xany.type == SelectionNotify ||
+      (event->xany.type == PropertyNotify &&
+       event->xproperty.atom == context->selAtom)) {
+
+    GdkWindow *cbWindow = gdk_window_lookup(event->xany.window);
+    if (cbWindow) {
+      GtkWidget *cbWidget = nsnull;
+      gdk_window_get_user_data(cbWindow, (gpointer *)&cbWidget);
+      if (cbWidget && GTK_IS_WIDGET(cbWidget)) {
+        context->cbWidget = cbWidget;
+        return True;
+      }
+    }
+  }
+
+  return False;
+}
+
+// Idle timeout for receiving selection and property notify events (microsec)
+static const int kClipboardTimeout = 500000;
 
 PRBool nsClipboard::FindSelectionNotifyEvent()
 {
-  PRTime entryTime=PR_Now();
-  XEvent xevent;
+  Display *xDisplay = GDK_DISPLAY();
+  checkEventContext context;
+  context.cbWidget = nsnull;
+  context.selAtom = gdk_atom_intern("GDK_SELECTION", FALSE);
 
-  while (1) {
-    if (XCheckTypedWindowEvent(GDK_DISPLAY(),
-                               GDK_WINDOW_XWINDOW(sWidget->window),
-                               SelectionNotify,
-                               &xevent)) {
-      send_selection_notify_to_widget(&xevent, sWidget);
-      return PR_TRUE;
+  // Send X events which are relevant to the ongoing selection retrieval
+  // to the clipboard widget.  Wait until either the operation completes, or
+  // we hit our timeout.  All other X events remain queued.
+
+  int select_result;
+
+#ifdef POLL_WITH_XCONNECTIONNUMBER
+  struct pollfd fds[1];
+  fds[0].fd = XConnectionNumber(xDisplay);
+  fds[0].events = POLLIN;
+#else
+  int cnumber = ConnectionNumber(xDisplay);
+  fd_set select_set;
+  FD_ZERO(&select_set);
+  FD_SET(cnumber, &select_set);
+  ++cnumber;
+  struct timeval tv;
+#endif
+
+  do {
+    XEvent xevent;
+
+    while (XCheckIfEvent(xDisplay, &xevent, checkEventProc,
+                         (XPointer) &context)) {
+
+      if (xevent.xany.type == SelectionNotify)
+        DispatchSelectionNotifyEvent(context.cbWidget, &xevent);
+      else
+        DispatchPropertyNotifyEvent(context.cbWidget, &xevent);
+
+      if (!mBlocking)
+        return PR_TRUE;
     }
-    PR_Sleep(20*PR_TicksPerSecond()/1000);  /* sleep for 20 ms/iteration */
-    if (PR_Now()-entryTime > NS_CLIPBOARD_TIMEOUT) break; 
-  } 
+
+#ifdef POLL_WITH_XCONNECTIONNUMBER
+    select_result = poll(fds, 1, kClipboardTimeout / 1000);
+#else
+    tv.tv_sec = 0;
+    tv.tv_usec = kClipboardTimeout;
+    select_result = select(cnumber, &select_set, nsnull, nsnull, &tv);
+#endif
+  } while (select_result == 1);
+
 #ifdef DEBUG_CLIPBOARD
-  printf("can't find a SelectionNotify event, giving up\n");
+  printf("exceeded clipboard timeout\n");
 #endif
   return PR_FALSE;
 }
@@ -1399,9 +1446,9 @@ PRBool nsClipboard::FindSelectionNotifyEvent()
 void ConvertHTMLtoUCS2(char* data, PRInt32 dataLength,
                        PRUnichar** unicodeData, PRInt32& outUnicodeLen)
 {
-  nsAutoString charset;
+  nsCAutoString charset;
   GetHTMLCharset(data, dataLength, charset);// get charset of HTML
-  if (charset.Equals(NS_LITERAL_STRING("UTF-16"))) {//current mozilla
+  if (charset.Equals(NS_LITERAL_CSTRING("UTF-16"))) {//current mozilla
     outUnicodeLen = dataLength / 2 - 1;
     *unicodeData = NS_REINTERPRET_CAST(PRUnichar*,
                    nsMemory::Alloc((outUnicodeLen + 1) * sizeof(PRUnichar)));
@@ -1411,7 +1458,7 @@ void ConvertHTMLtoUCS2(char* data, PRInt32 dataLength,
       (*unicodeData)[outUnicodeLen] = '\0';
     }
   }
-  else if (charset.Equals(NS_LITERAL_STRING("OLD-MOZILLA"))) {// old mozilla
+  else if (charset.Equals(NS_LITERAL_CSTRING("OLD-MOZILLA"))) {// old mozilla
     outUnicodeLen = dataLength / 2;
     *unicodeData = NS_REINTERPRET_CAST(PRUnichar*,
                    nsMemory::Alloc((outUnicodeLen + 1) * sizeof(PRUnichar)));
@@ -1433,7 +1480,7 @@ void ConvertHTMLtoUCS2(char* data, PRInt32 dataLength,
       outUnicodeLen = 0;
       return;
     }
-    rv = ccm->GetUnicodeDecoder(&charset, getter_AddRefs(decoder));
+    rv = ccm->GetUnicodeDecoder(charset.get(), getter_AddRefs(decoder));
     if (NS_FAILED(rv)) {
 #ifdef DEBUG_CLIPBOARD
       g_print("        get unicode decoder error\n");
@@ -1468,12 +1515,12 @@ void ConvertHTMLtoUCS2(char* data, PRInt32 dataLength,
  *  2. "OLD-MOZILLA": UCS2 encoded data
  *  3. other:         "text/html" with other charset than utf-16
  */
-void GetHTMLCharset(char* data, PRInt32 dataLength, nsAString& str)
+void GetHTMLCharset(char* data, PRInt32 dataLength, nsACString& str)
 {
   // if detect "FFFE" or "FEFF", assume utf-16
   PRUnichar* beginChar =  (PRUnichar*)data;
   if ((beginChar[0] == 0xFFFE) || (beginChar[0] == 0xFEFF)) {
-    str.Assign(NS_LITERAL_STRING("UTF-16"));
+    str.Assign(NS_LITERAL_CSTRING("UTF-16"));
     return;
   }
   // no "FFFE" and "FEFF", assume ASCII first to find "charset" info
@@ -1509,7 +1556,7 @@ void GetHTMLCharset(char* data, PRInt32 dataLength, nsAString& str)
 #ifdef DEBUG_CLIPBOARD
       printf("Charset of HTML = %s\n", charsetUpperStr.get());
 #endif
-      str.Assign(NS_ConvertUTF8toUCS2(charsetUpperStr));
+      str.Assign(charsetUpperStr);
       return;
     }
   }
@@ -1519,6 +1566,6 @@ void GetHTMLCharset(char* data, PRInt32 dataLength, nsAString& str)
   // TODO: it may also be "text/html" without "charset".
   // can't distinguish between them. Sochoose OLD-MOZILLA here to
   // make compitable with old-version mozilla
-  str.Assign(NS_LITERAL_STRING("OLD-MOZILLA"));
+  str.Assign(NS_LITERAL_CSTRING("OLD-MOZILLA"));
 }
 
