@@ -56,7 +56,7 @@
 #include "nsIDocumentViewer.h"
 #include "nsIDocumentObserver.h"
 
-#include "nsIDOMXULDocument.h"
+#include "nsIDOMDocument.h"
 #include "nsWidgetAtoms.h"
 
 #include <Menus.h>
@@ -70,11 +70,13 @@
 
 #include "nsGUIEvent.h"
 
+#if !XP_MACOSX
+#include "MenuSharing.h"
+#endif
+
 // CIDs
 #include "nsWidgetsCID.h"
-static NS_DEFINE_CID(kMenuBarCID, NS_MENUBAR_CID);
 static NS_DEFINE_CID(kMenuCID, NS_MENU_CID);
-static NS_DEFINE_CID(kMenuItemCID, NS_MENUITEM_CID);
 
 NS_IMPL_ISUPPORTS6(nsMenuBarX, nsIMenuBar, nsIMenuListener, nsIDocumentObserver, 
                     nsIChangeManager, nsIMenuCommandDispatcher, nsISupportsWeakReference)
@@ -89,8 +91,6 @@ EventHandlerUPP nsMenuBarX::sCommandEventHandler = nsnull;
 nsMenuBarX::nsMenuBarX()
   : mNumMenus(0), mParent(nsnull), mIsMenuBarAdded(PR_FALSE), mDocument(nsnull), mCurrentCommandID(1)
 {
-  NS_INIT_REFCNT();
-
   OSStatus status = ::CreateNewMenu(0, 0, &mRootMenu);
   NS_ASSERTION(status == noErr, "nsMenuBarX::nsMenuBarX:  creation of root menu failed.");
   
@@ -216,7 +216,7 @@ nsMenuBarX :: GetDocument ( nsIWebShell* inWebShell, nsIDocument** outDocument )
       nsCOMPtr<nsIDocumentViewer> docv(do_QueryInterface(cv));
       if (!docv)
         return;
-      docv->GetDocument(*outDocument);    // addrefs
+      docv->GetDocument(outDocument);    // addrefs
     }
   }
 }
@@ -254,9 +254,7 @@ nsMenuBarX :: RegisterAsDocumentObserver ( nsIWebShell* inWebShell )
 void
 nsMenuBarX :: AquifyMenuBar ( )
 {
-  nsCOMPtr<nsIDocument> containingDoc;
-  mMenuBarContent->GetDocument ( *getter_AddRefs(containingDoc) );
-  nsCOMPtr<nsIDOMDocument> domDoc ( do_QueryInterface(containingDoc) );
+  nsCOMPtr<nsIDOMDocument> domDoc(do_QueryInterface(mMenuBarContent->GetDocument()));
   if ( domDoc ) {
     // remove quit item and its separator
     HideItem ( domDoc, NS_LITERAL_STRING("menu_FileQuitSeparator"), nsnull );
@@ -337,13 +335,15 @@ nsMenuBarX :: CommandEventHandler ( EventHandlerCallRef inHandlerChain, EventRef
         
         case kHICommandAbout:
         {
-          // the 'about' command is special because we don't have a nsIMenu or nsIMenuItem
-          // for the apple menu. Grovel for the content node with an id of "aboutName" 
-          // and call it directly.
-          nsCOMPtr<nsIDOMXULDocument> xulDoc = do_QueryInterface(self->mDocument);
-	        if ( xulDoc ) {
+          // the 'about' command is special because we don't have a
+          // nsIMenu or nsIMenuItem for the apple menu. Grovel for the
+          // content node with an id of "aboutName" and call it
+          // directly.
+          nsCOMPtr<nsIDOMDocument> domDoc = do_QueryInterface(self->mDocument);
+	        if ( domDoc ) {
       	    nsCOMPtr<nsIDOMElement> domElement;
-      	    xulDoc->GetElementById(NS_LITERAL_STRING("aboutName"), getter_AddRefs(domElement));
+      	    domDoc->GetElementById(NS_LITERAL_STRING("aboutName"),
+                                   getter_AddRefs(domElement));
       	    nsCOMPtr<nsIContent> aboutContent ( do_QueryInterface(domElement) );
       	    self->ExecuteCommand(aboutContent);
           }
@@ -351,6 +351,20 @@ nsMenuBarX :: CommandEventHandler ( EventHandlerCallRef inHandlerChain, EventRef
           break;
         }
 
+#if !XP_MACOSX
+        case 'SHMN':
+        { // Shared menu support
+          MenuItemIndex index = command.menu.menuItemIndex;
+          if (index)
+          {
+            (void)SharedMenuHit(GetMenuID(command.menu.menuRef), command.menu.menuItemIndex);
+            ::HiliteMenu(0);
+            handled = noErr;
+          }
+          break;
+        }
+#endif
+
         default:
         {
           // given the commandID, look it up in our hashtable and dispatch to
@@ -434,7 +448,7 @@ nsEventStatus
 nsMenuBarX::MenuConstruct( const nsMenuEvent & aMenuEvent, nsIWidget* aParentWindow, 
                             void * menubarNode, void * aWebShell )
 {
-  mWebShellWeakRef = getter_AddRefs(NS_GetWeakReference(NS_STATIC_CAST(nsIWebShell*, aWebShell)));
+  mWebShellWeakRef = do_GetWeakReference(NS_STATIC_CAST(nsIWebShell*, aWebShell));
   nsIDOMNode* aDOMNode  = NS_STATIC_CAST(nsIDOMNode*, menubarNode);
   mMenuBarContent = do_QueryInterface(aDOMNode);           // strong ref
   NS_ASSERTION ( mMenuBarContent, "No content specified for this menubar" );
@@ -459,15 +473,12 @@ nsMenuBarX::MenuConstruct( const nsMenuEvent & aMenuEvent, nsIWidget* aParentWin
   // set this as a nsMenuListener on aParentWindow
   aParentWindow->AddMenuListener((nsIMenuListener *)this);
 
-  PRInt32 count;
-  mMenuBarContent->ChildCount(count);
-  for ( int i = 0; i < count; ++i ) { 
-    nsCOMPtr<nsIContent> menu;
-    mMenuBarContent->ChildAt ( i, *getter_AddRefs(menu) );
+  PRUint32 count = mMenuBarContent->GetChildCount();
+  for ( PRUint32 i = 0; i < count; ++i ) { 
+    nsIContent *menu = mMenuBarContent->GetChildAt(i);
     if ( menu ) {
-      nsCOMPtr<nsIAtom> tag;
-      menu->GetTag ( *getter_AddRefs(tag) );
-      if (tag == nsWidgetAtoms::menu) {
+      if (menu->Tag() == nsWidgetAtoms::menu &&
+          menu->IsContentOfType(nsIContent::eXUL)) {
         nsAutoString menuName;
         nsAutoString menuAccessKey(NS_LITERAL_STRING(" "));
         menu->GetAttr(kNameSpaceID_None, nsWidgetAtoms::label, menuName);
@@ -612,8 +623,7 @@ nsMenuBarX :: CreateAppleMenu ( nsIMenu* inMenu )
     nsCOMPtr<nsIContent> menu;
     inMenu->GetMenuContent(getter_AddRefs(menu));
     if (menu) {
-      nsCOMPtr<nsIDocument> doc;
-      menu->GetDocument(*getter_AddRefs(doc));
+      nsCOMPtr<nsIDocument> doc = menu->GetDocument();
       if (doc) {
         nsCOMPtr<nsIDOMDocument> domdoc ( do_QueryInterface(doc) );
         if ( domdoc ) {
@@ -716,57 +726,28 @@ NS_METHOD nsMenuBarX::Paint()
 // this is needed for menubar changes
 //
 
+NS_IMPL_NSIDOCUMENTOBSERVER_LOAD_STUB(nsMenuBarX)
+NS_IMPL_NSIDOCUMENTOBSERVER_REFLOW_STUB(nsMenuBarX)
+NS_IMPL_NSIDOCUMENTOBSERVER_STATE_STUB(nsMenuBarX)
+NS_IMPL_NSIDOCUMENTOBSERVER_STYLE_STUB(nsMenuBarX)
 
-NS_IMETHODIMP
-nsMenuBarX::BeginUpdate( nsIDocument * aDocument )
+void
+nsMenuBarX::BeginUpdate( nsIDocument * aDocument, nsUpdateType aUpdateType )
 {
-  return NS_OK;
 }
 
-NS_IMETHODIMP
-nsMenuBarX::EndUpdate( nsIDocument * aDocument )
+void
+nsMenuBarX::EndUpdate( nsIDocument * aDocument, nsUpdateType aUpdateType )
 {
-  return NS_OK;
 }
 
-NS_IMETHODIMP
-nsMenuBarX::BeginLoad( nsIDocument * aDocument )
+void
+nsMenuBarX::CharacterDataChanged( nsIDocument * aDocument,
+                                  nsIContent * aContent, PRBool aAppend)
 {
-  return NS_OK;
 }
 
-NS_IMETHODIMP
-nsMenuBarX::EndLoad( nsIDocument * aDocument )
-{
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsMenuBarX::BeginReflow(  nsIDocument * aDocument, nsIPresShell * aShell)
-{
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsMenuBarX::EndReflow( nsIDocument * aDocument, nsIPresShell * aShell)
-{
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsMenuBarX::ContentChanged( nsIDocument * aDocument, nsIContent * aContent, nsISupports * aSubContent)
-{
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsMenuBarX::ContentStatesChanged( nsIDocument * aDocument, nsIContent  * aContent1,
-                                  nsIContent  * aContent2, PRInt32 aStateMask)
-{
-  return NS_OK;
-}
-
-NS_IMETHODIMP
+void
 nsMenuBarX::ContentAppended( nsIDocument * aDocument, nsIContent  * aContainer,
                               PRInt32 aNewIndexInContainer)
 {
@@ -780,8 +761,7 @@ nsMenuBarX::ContentAppended( nsIDocument * aDocument, nsIContent  * aContainer,
     if ( obs )
       obs->ContentInserted ( aDocument, aContainer, aNewIndexInContainer );
     else {
-      nsCOMPtr<nsIContent> parent;
-      aContainer->GetParent(*getter_AddRefs(parent));
+      nsCOMPtr<nsIContent> parent = aContainer->GetParent();
       if(parent) {
         Lookup ( parent, getter_AddRefs(obs) );
         if ( obs )
@@ -789,78 +769,35 @@ nsMenuBarX::ContentAppended( nsIDocument * aDocument, nsIContent  * aContainer,
       }
     }
   }
-  return NS_OK;
 }
 
-NS_IMETHODIMP
-nsMenuBarX::ContentReplaced( nsIDocument * aDocument, nsIContent * aContainer, nsIContent * aOldChild,
-                          nsIContent * aNewChild, PRInt32 aIndexInContainer)
+void
+nsMenuBarX::ContentReplaced( nsIDocument * aDocument, nsIContent * aContainer,
+                             nsIContent * aOldChild, nsIContent * aNewChild,
+                             PRInt32 aIndexInContainer)
 {
-  return NS_OK;
 }
 
-NS_IMETHODIMP
-nsMenuBarX::StyleSheetAdded( nsIDocument * aDocument, nsIStyleSheet * aStyleSheet)
-{
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsMenuBarX::StyleSheetRemoved(nsIDocument * aDocument, nsIStyleSheet * aStyleSheet)
-{
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsMenuBarX::StyleSheetDisabledStateChanged(nsIDocument * aDocument, nsIStyleSheet * aStyleSheet,
-                                            PRBool aDisabled)
-{
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsMenuBarX::StyleRuleChanged( nsIDocument * aDocument, nsIStyleSheet * aStyleSheet,
-                              nsIStyleRule * aStyleRule, PRInt32 aHint)
-{
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsMenuBarX::StyleRuleAdded( nsIDocument * aDocument, nsIStyleSheet * aStyleSheet,
-                            nsIStyleRule * aStyleRule)
-{
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsMenuBarX::StyleRuleRemoved(nsIDocument * aDocument, nsIStyleSheet * aStyleSheet,
-                              nsIStyleRule  * aStyleRule)
-{
-  return NS_OK;
-}
-
-NS_IMETHODIMP
+void
 nsMenuBarX::DocumentWillBeDestroyed( nsIDocument * aDocument )
 {
   mDocument = nsnull;
-  return NS_OK;
 }
 
 
-NS_IMETHODIMP
-nsMenuBarX::AttributeChanged( nsIDocument * aDocument, nsIContent * aContent, PRInt32 aNameSpaceID,
-                              nsIAtom * aAttribute, PRInt32 aModType, PRInt32 aHint)
+void
+nsMenuBarX::AttributeChanged( nsIDocument * aDocument, nsIContent * aContent,
+                              PRInt32 aNameSpaceID, nsIAtom * aAttribute,
+                              PRInt32 aModType )
 {
   // lookup and dispatch to registered thang.
   nsCOMPtr<nsIChangeObserver> obs;
   Lookup ( aContent, getter_AddRefs(obs) );
   if ( obs )
-    obs->AttributeChanged ( aDocument, aNameSpaceID, aAttribute, aHint );
-
-  return NS_OK;
+    obs->AttributeChanged ( aDocument, aNameSpaceID, aAttribute );
 }
 
-NS_IMETHODIMP
+void
 nsMenuBarX::ContentRemoved( nsIDocument * aDocument, nsIContent * aContainer,
                             nsIContent * aChild, PRInt32 aIndexInContainer )
 {  
@@ -874,8 +811,7 @@ nsMenuBarX::ContentRemoved( nsIDocument * aDocument, nsIContent * aContainer,
     if ( obs )
       obs->ContentRemoved ( aDocument, aChild, aIndexInContainer );
     else {
-      nsCOMPtr<nsIContent> parent;
-      aContainer->GetParent(*getter_AddRefs(parent));
+      nsCOMPtr<nsIContent> parent = aContainer->GetParent();
       if(parent) {
         Lookup ( parent, getter_AddRefs(obs) );
         if ( obs )
@@ -883,12 +819,11 @@ nsMenuBarX::ContentRemoved( nsIDocument * aDocument, nsIContent * aContainer,
       }
     }
   }
-  return NS_OK;
 }
 
-NS_IMETHODIMP
+void
 nsMenuBarX::ContentInserted( nsIDocument * aDocument, nsIContent * aContainer,
-                            nsIContent * aChild, PRInt32 aIndexInContainer )
+                             nsIContent * aChild, PRInt32 aIndexInContainer )
 {  
   if ( aContainer == mMenuBarContent ) {
     //Register(aChild, );
@@ -900,8 +835,7 @@ nsMenuBarX::ContentInserted( nsIDocument * aDocument, nsIContent * aContainer,
     if ( obs )
       obs->ContentInserted ( aDocument, aChild, aIndexInContainer );
     else {
-      nsCOMPtr<nsIContent> parent;
-      aContainer->GetParent(*getter_AddRefs(parent));
+      nsCOMPtr<nsIContent> parent = aContainer->GetParent();
       if(parent) {
         Lookup ( parent, getter_AddRefs(obs) );
         if ( obs )
@@ -909,7 +843,6 @@ nsMenuBarX::ContentInserted( nsIDocument * aDocument, nsIContent * aContainer,
       }
     }
   }
-  return NS_OK;
 }
 
 #pragma mark - 
@@ -1031,7 +964,7 @@ MenuHelpersX::WebShellToPresContext (nsIWebShell* inWebShell, nsIPresContext** o
   if ( contentViewer ) {
     nsCOMPtr<nsIDocumentViewer> docViewer ( do_QueryInterface(contentViewer) );
     if ( docViewer )
-      docViewer->GetPresContext(*outContext);     // AddRefs for us
+      docViewer->GetPresContext(outContext);     // AddRefs for us
     else
       retval = NS_ERROR_FAILURE;
   }
@@ -1055,18 +988,17 @@ MenuHelpersX::DispatchCommandTo(nsIWeakReference* aWebShellWeakRef,
   MenuHelpersX::WebShellToPresContext(webShell, getter_AddRefs(presContext));
 
   nsEventStatus status = nsEventStatus_eConsumeNoDefault;
-  nsMouseEvent event;
-  event.eventStructType = NS_MOUSE_EVENT;
-  event.message = NS_XUL_COMMAND;
+  nsMouseEvent event(NS_XUL_COMMAND);
+
+  // FIXME: Should probably figure out how to init this with the actual
+  // pressed keys, but this is a big old edge case anyway. -dwh
 
   // See if we have a command element.  If so, we execute on the
   // command instead of on our content element.
   nsAutoString command;
   aTargetContent->GetAttr(kNameSpaceID_None, nsWidgetAtoms::command, command);
   if (!command.IsEmpty()) {
-    nsCOMPtr<nsIDocument> doc;
-    aTargetContent->GetDocument(*getter_AddRefs(doc));
-    nsCOMPtr<nsIDOMDocument> domDoc(do_QueryInterface(doc));
+    nsCOMPtr<nsIDOMDocument> domDoc(do_QueryInterface(aTargetContent->GetDocument()));
     nsCOMPtr<nsIDOMElement> commandElt;
     domDoc->GetElementById(command, getter_AddRefs(commandElt));
     nsCOMPtr<nsIContent> commandContent(do_QueryInterface(commandElt));
