@@ -50,6 +50,9 @@ NS_IMPL_THREADSAFE_ISUPPORTS2(xptiInterfaceInfoManager,
                               nsIInterfaceInfoSuperManager)
 
 static xptiInterfaceInfoManager* gInterfaceInfoManager = nsnull;
+#ifdef DEBUG
+static int gCallCount = 0;
+#endif
 
 // static
 xptiInterfaceInfoManager*
@@ -122,8 +125,6 @@ xptiInterfaceInfoManager::xptiInterfaceInfoManager(nsISupportsArray* aSearchPath
         mAdditionalManagersLock(PR_NewLock()),
         mSearchPath(aSearchPath)
 {
-    NS_INIT_ISUPPORTS();
-    
     const char* statsFilename = PR_GetEnv("MOZILLA_XPTI_STATS");
     if(statsFilename)
     {
@@ -171,24 +172,25 @@ xptiInterfaceInfoManager::~xptiInterfaceInfoManager()
     if(mAdditionalManagersLock)
         PR_DestroyLock(mAdditionalManagersLock);
 
+    gInterfaceInfoManager = nsnull;
 #ifdef DEBUG
     xptiInterfaceInfo::DEBUG_ShutdownNotification();
+    gCallCount = 0;
 #endif
 }
 
-static PRBool
+static nsresult
 GetDirectoryFromDirService(const char* codename, nsILocalFile** aDir)
 {
     NS_ASSERTION(codename,"loser!");
     NS_ASSERTION(aDir,"loser!");
     
-    nsCOMPtr<nsIProperties> dirService = 
-        do_GetService(NS_DIRECTORY_SERVICE_CONTRACTID);
-    if(!dirService)
-        return PR_FALSE;
+    nsresult rv;
+    nsCOMPtr<nsIProperties> dirService =
+        do_GetService(NS_DIRECTORY_SERVICE_CONTRACTID, &rv);
+    if (NS_FAILED(rv)) return rv;
 
-    return NS_SUCCEEDED(dirService->Get(codename, NS_GET_IID(nsILocalFile), 
-                                        (void**) aDir));
+    return dirService->Get(codename, NS_GET_IID(nsILocalFile), (void**) aDir);
 }
 
 static PRBool
@@ -224,8 +226,7 @@ AppendFromDirServiceList(const char* codename, nsISupportsArray* aPath)
 PRBool xptiInterfaceInfoManager::BuildFileSearchPath(nsISupportsArray** aPath)
 {
 #ifdef DEBUG
-    static int callCount = 0;
-    NS_ASSERTION(!callCount++, "Expected only one call!");
+    NS_ASSERTION(!gCallCount++, "Expected only one call!");
 #endif
 
     nsCOMPtr<nsISupportsArray> searchPath;
@@ -233,55 +234,67 @@ PRBool xptiInterfaceInfoManager::BuildFileSearchPath(nsISupportsArray** aPath)
     if(!searchPath)
         return PR_FALSE;
     
-    nsCOMPtr<nsILocalFile> dir;
+    nsCOMPtr<nsILocalFile> compDir;
 
     // Always put components directory first
 
     if(NS_FAILED(GetDirectoryFromDirService(NS_XPCOM_COMPONENT_DIR, 
-                                            getter_AddRefs(dir))) ||
-       !searchPath->AppendElement(dir))
+                                            getter_AddRefs(compDir))) ||
+       !searchPath->AppendElement(compDir))
     {
         return PR_FALSE;
     }
 
     // Add additional plugins dirs
     // No error checking here since this is optional in some embeddings
+    
+    // Add the GRE's component directory to searchPath if the 
+    // application is using an GRE.
+    // An application indicates that it's using an GRE by returning
+    // a valid nsIFile via it's directory service provider interface.
+    //
+    // Please see http://www.mozilla.org/projects/embedding/MRE.html
+    // for more info. on GREs
+    //
+    nsCOMPtr<nsILocalFile> greComponentDirectory;
+    nsresult rv = GetDirectoryFromDirService(NS_GRE_COMPONENT_DIR, 
+                                    getter_AddRefs(greComponentDirectory));
+    if(NS_SUCCEEDED(rv) && greComponentDirectory)
+    {
+        // make sure we only append a directory if its a different one
+        PRBool equalsCompDir = PR_FALSE;
+        greComponentDirectory->Equals(compDir, &equalsCompDir);
 
-    (void) AppendFromDirServiceList(NS_APP_PLUGINS_DIR_LIST, searchPath);
+        if(!equalsCompDir)
+            searchPath->AppendElement(greComponentDirectory);
+    }
+
+    (void)AppendFromDirServiceList(NS_APP_PLUGINS_DIR_LIST, searchPath);
 
     NS_ADDREF(*aPath = searchPath);
     return PR_TRUE;
 }
 
 PRBool 
-xptiInterfaceInfoManager::GetCloneOfManifestDir(nsILocalFile** aDir)
+xptiInterfaceInfoManager::GetCloneOfManifestLocation(nsILocalFile** aFile)
 {
-    // We cache the manifest directory to ensure that this call always returns 
-    // the same directory. Not doing so could be a problem if we try something 
-    // fancier in the future and that algorithm has different results when
-    // run at startup than when run later.
+    // We *trust* that this will not change!
+    nsCOMPtr<nsILocalFile> lf;
+    nsresult rv = GetDirectoryFromDirService(NS_XPCOM_XPTI_REGISTRY_FILE, 
+                                             getter_AddRefs(lf));
 
-    if(!mManifestDir)
-    {
-        // If we are going to do something fancy to get some other dir, then
-        // do it here...
+    if (NS_FAILED(rv)) return PR_FALSE;
 
-        if(!GetDirectoryFromDirService(NS_XPCOM_COMPONENT_DIR,
-                                       getter_AddRefs(mManifestDir)))
-            return PR_FALSE;
-    
-        // To be sure after the stuff above.
-        if(!mManifestDir)
-            return PR_FALSE;
-    }
-    return NS_SUCCEEDED(xptiCloneLocalFile(mManifestDir, aDir));
+    rv = xptiCloneLocalFile(lf, aFile);
+    if (NS_FAILED(rv)) return PR_FALSE;
+    return PR_TRUE;
 }
 
 PRBool 
 xptiInterfaceInfoManager::GetApplicationDir(nsILocalFile** aDir)
 {
     // We *trust* that this will not change!
-    return GetDirectoryFromDirService(NS_XPCOM_CURRENT_PROCESS_DIR, aDir);
+    return NS_SUCCEEDED(GetDirectoryFromDirService(NS_XPCOM_CURRENT_PROCESS_DIR, aDir));
 }
 
 PRBool 
@@ -321,7 +334,7 @@ xptiInterfaceInfoManager::BuildFileList(nsISupportsArray* aSearchPath,
             entries->GetNext(getter_AddRefs(sup));
             if(!sup)
                 return PR_FALSE;
-            nsCOMPtr<nsIFile> file = do_QueryInterface(sup);
+            nsCOMPtr<nsILocalFile> file = do_QueryInterface(sup);
             if(!file)
                 return PR_FALSE;
 
@@ -363,15 +376,21 @@ xptiInterfaceInfoManager::ReadXPTFile(nsILocalFile* aFile,
     XPTCursor cursor;
     PRInt32 flen;
     PRInt64 fileSize;
+    
+    PRBool saveFollowLinks;
+    aFile->GetFollowLinks(&saveFollowLinks);
+    aFile->SetFollowLinks(PR_TRUE);
 
     if(NS_FAILED(aFile->GetFileSize(&fileSize)) || !(flen = nsInt64(fileSize)))
     {
+        aFile->SetFollowLinks(saveFollowLinks);
         return nsnull;
     }
 
     whole = new char[flen];
     if (!whole)
     {
+        aFile->SetFollowLinks(saveFollowLinks);
         return nsnull;
     }
 
@@ -410,6 +429,7 @@ xptiInterfaceInfoManager::ReadXPTFile(nsILocalFile* aFile,
         XPT_DestroyXDRState(state);
     if(whole)
         delete [] whole;
+    aFile->SetFollowLinks(saveFollowLinks);
     return header;
 }
 
@@ -883,9 +903,7 @@ xptiInterfaceInfoManager::AddOnlyNewFilesFromFileList(nsISupportsArray* aSearchP
     if(NS_FAILED(aFileList->Count(&countOfFilesInFileList)))
         return PR_FALSE;
     NS_ASSERTION(countOfFilesInFileList, "loser!");
-    
-    PRUint32 countOfFilesInWorkingSet = aWorkingSet->GetFileCount();
-    NS_ASSERTION(countOfFilesInFileList > countOfFilesInWorkingSet,"loser!");
+    NS_ASSERTION(countOfFilesInFileList > aWorkingSet->GetFileCount(), "loser!");
 
     orderedFileArray = BuildOrderedFileArray(aSearchPath, aFileList, aWorkingSet);
 
@@ -1284,7 +1302,8 @@ xptiInterfaceInfoManager::VerifyAndAddEntryIfNew(xptiWorkingSet* aWorkingSet,
     
     // Build a new xptiInterfaceEntry object and hook it up. 
 
-    entry = xptiInterfaceEntry::NewEntry(iface->name, iface->iid,
+    entry = xptiInterfaceEntry::NewEntry(iface->name, strlen(iface->name),
+                                         iface->iid,
                                          typelibRecord, aWorkingSet);
     if(!entry)
     {
@@ -1939,12 +1958,13 @@ public:
     NS_DECL_NSISIMPLEENUMERATOR
 
     xptiAdditionalManagersEnumerator();
-    virtual ~xptiAdditionalManagersEnumerator() {}
 
     PRBool SizeTo(PRUint32 likelyCount) {return mArray.SizeTo(likelyCount);}
     PRBool AppendElement(nsIInterfaceInfoManager* element);
 
 private:
+    ~xptiAdditionalManagersEnumerator() {}
+
     nsSupportsArray mArray;
     PRUint32        mIndex;
     PRUint32        mCount;
@@ -1955,7 +1975,6 @@ NS_IMPL_ISUPPORTS1(xptiAdditionalManagersEnumerator, nsISimpleEnumerator)
 xptiAdditionalManagersEnumerator::xptiAdditionalManagersEnumerator()
     : mIndex(0), mCount(0)
 {
-    NS_INIT_ISUPPORTS();
 }
 
 PRBool xptiAdditionalManagersEnumerator::AppendElement(nsIInterfaceInfoManager* element)
