@@ -54,7 +54,6 @@
 #include "nsIPref.h"
 #include "nsReadableUtils.h"
 
-#include "nsINameSpaceManager.h"
 #include "nsEscape.h"
 #include "nsVoidArray.h"
 #include "nsIScriptGlobalObject.h"
@@ -65,9 +64,7 @@
 #include "nsIWebNavigation.h"
 #include "nsIWindowWatcher.h"
 
-#include "nsIXULPopupListener.h"
 #include "nsIDOMXULElement.h"
-#include "nsRDFCID.h"
 
 #include "nsGUIEvent.h"
 #include "nsWidgetsCID.h"
@@ -146,20 +143,12 @@
 /* Define Class IDs */
 static NS_DEFINE_CID(kWindowCID,           NS_WINDOW_CID);
 static NS_DEFINE_CID(kWebShellCID,         NS_WEB_SHELL_CID);
-static NS_DEFINE_CID(kAppShellServiceCID,  NS_APPSHELL_SERVICE_CID);
-static NS_DEFINE_CID(kAppShellCID,         NS_APPSHELL_CID);
 
 #include "nsWidgetsCID.h"
 static NS_DEFINE_CID(kMenuBarCID,          NS_MENUBAR_CID);
 static NS_DEFINE_CID(kMenuCID,             NS_MENU_CID);
 static NS_DEFINE_CID(kMenuItemCID,         NS_MENUITEM_CID);
 
-static NS_DEFINE_CID(kPrefCID,             NS_PREF_CID);
-static NS_DEFINE_CID(kEventQueueServiceCID, NS_EVENTQUEUESERVICE_CID);
-
-static NS_DEFINE_CID(kLayoutDocumentLoaderFactoryCID, NS_LAYOUT_DOCUMENT_LOADER_FACTORY_CID);
-static NS_DEFINE_CID(kXULPopupListenerCID, NS_XULPOPUPLISTENER_CID);
-
 
 #ifdef DEBUG_rods
 #define DEBUG_MENUSDEL 1
@@ -167,12 +156,8 @@ static NS_DEFINE_CID(kXULPopupListenerCID, NS_XULPOPUPLISTENER_CID);
 
 #include "nsIWebShell.h"
 
-static NS_DEFINE_CID(kStringBundleServiceCID,     NS_STRINGBUNDLESERVICE_CID);
-
 #define SIZE_PERSISTENCE_TIMEOUT 500 // msec
 
-const char * kPrimaryContentTypeValue  = "content-primary";
-
 struct ThreadedWindowEvent {
   PLEvent           event;
   nsWebShellWindow  *window;
@@ -201,8 +186,6 @@ struct nsWebShellInfo {
 
 nsWebShellWindow::nsWebShellWindow() : nsXULWindow()
 {
-  NS_INIT_REFCNT();
-
   mWebShell = nsnull;
   mWindow   = nsnull;
   mLockedUntilChromeLoad = PR_FALSE;
@@ -225,15 +208,17 @@ nsWebShellWindow::~nsWebShellWindow()
     mWindow->SetClientData(0);
   mWindow = nsnull; // Force release here.
 
-  PR_Lock(mSPTimerLock);
-  if (mSPTimer)
-    mSPTimer->Cancel();
-  PR_Unlock(mSPTimerLock);
-  PR_DestroyLock(mSPTimerLock);
+  if (mSPTimerLock) {
+    PR_Lock(mSPTimerLock);
+    if (mSPTimer)
+      mSPTimer->Cancel();
+    PR_Unlock(mSPTimerLock);
+    PR_DestroyLock(mSPTimerLock);
+  }
 }
 
-NS_IMPL_THREADSAFE_ADDREF(nsWebShellWindow);
-NS_IMPL_THREADSAFE_RELEASE(nsWebShellWindow);
+NS_IMPL_THREADSAFE_ADDREF(nsWebShellWindow)
+NS_IMPL_THREADSAFE_RELEASE(nsWebShellWindow)
 
 NS_INTERFACE_MAP_BEGIN(nsWebShellWindow)
    NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsISupports, nsIWebShellContainer)
@@ -250,7 +235,6 @@ nsresult nsWebShellWindow::Initialize(nsIXULWindow* aParent,
                                       nsIAppShell* aShell, nsIURI* aUrl, 
                                       PRBool aCreatedVisible,
                                       PRBool aLoadDefaultPage,
-                                      PRUint32 aZlevel,
                                       PRInt32 aInitialWidth, PRInt32 aInitialHeight,
                                       PRBool aIsHiddenWindow, nsWidgetInitData& widgetInitData)
 {
@@ -261,7 +245,6 @@ nsresult nsWebShellWindow::Initialize(nsIXULWindow* aParent,
   
   mShowAfterLoad = aCreatedVisible;
   mLoadDefaultPage = aLoadDefaultPage;
-  mZlevel = aZlevel;
   
   // XXX: need to get the default window size from prefs...
   // Doesn't come from prefs... will come from CSS/XUL/RDF
@@ -287,7 +270,7 @@ nsresult nsWebShellWindow::Initialize(nsIXULWindow* aParent,
   nsCOMPtr<nsIBaseWindow> parentAsWin(do_QueryInterface(aParent));
   if (parentAsWin) {
     parentAsWin->GetMainWidget(getter_AddRefs(parentWidget));
-    mParentWindow = getter_AddRefs(NS_GetWeakReference(aParent));
+    mParentWindow = do_GetWeakReference(aParent);
   }
 
   mWindow->SetClientData(this);
@@ -317,7 +300,7 @@ nsresult nsWebShellWindow::Initialize(nsIXULWindow* aParent,
   // Attach a WebProgress listener.during initialization...
   nsCOMPtr<nsIWebProgress> webProgress(do_GetInterface(mDocShell, &rv));
   if (webProgress) {
-    webProgress->AddProgressListener(this);
+    webProgress->AddProgressListener(this, nsIWebProgress::NOTIFY_STATE_NETWORK);
   }
 
   nsCOMPtr<nsIDocShellTreeItem> docShellAsItem(do_QueryInterface(mDocShell));
@@ -341,7 +324,7 @@ nsresult nsWebShellWindow::Initialize(nsIXULWindow* aParent,
                          nsnull,
                          nsnull,
                          nsnull);
-    NS_ENSURE_SUCCESS(rv, NS_ERROR_FAILURE);
+    NS_ENSURE_SUCCESS(rv, rv);
   }
                      
   return rv;
@@ -421,33 +404,68 @@ nsWebShellWindow::HandleEvent(nsGUIEvent *aEvent)
       case NS_MOVE: {
         // persist position, but not immediately, in case this OS is firing
         // repeated move events as the user drags the window
-        eventWindow->SetPersistenceTimer(PR_FALSE, PR_TRUE, PR_FALSE);
+        eventWindow->SetPersistenceTimer(PAD_POSITION);
         break;
       }
       case NS_SIZE: {
+        PRBool chromeLock = PR_FALSE;
         nsSizeEvent* sizeEvent = (nsSizeEvent*)aEvent;
         nsCOMPtr<nsIBaseWindow> shellAsWin(do_QueryInterface(webShell));
         shellAsWin->SetPositionAndSize(0, 0, sizeEvent->windowSize->width, 
           sizeEvent->windowSize->height, PR_FALSE);  
         // persist size, but not immediately, in case this OS is firing
         // repeated size events as the user drags the sizing handle
-        eventWindow->SetPersistenceTimer(PR_TRUE, PR_FALSE, PR_TRUE);
+        if (NS_FAILED(eventWindow->GetLockedState(chromeLock)) || !chromeLock)
+          eventWindow->SetPersistenceTimer(PAD_SIZE | PAD_MISC);
         result = nsEventStatus_eConsumeNoDefault;
         break;
       }
       case NS_SIZEMODE: {
         nsSizeModeEvent* modeEvent = (nsSizeModeEvent*)aEvent;
+
+        // an alwaysRaised (or higher) window will hide any newly opened
+        // normal browser windows. here we just drop a raised window
+        // to the normal zlevel if it's maximized. we make no provision
+        // for automatically re-raising it when restored.
+        if (modeEvent->mSizeMode == nsSizeMode_Maximized) {
+          PRUint32 zLevel;
+          eventWindow->GetZLevel(&zLevel);
+          if (zLevel > nsIXULWindow::normalZ)
+            eventWindow->SetZLevel(nsIXULWindow::normalZ);
+        }
+
         aEvent->widget->SetSizeMode(modeEvent->mSizeMode);
+
         // persist mode, but not immediately, because in many (all?)
         // cases this will merge with the similar call in NS_SIZE and
         // write the attribute values only once.
-        eventWindow->SetPersistenceTimer(PR_FALSE, PR_FALSE, PR_TRUE);
+        eventWindow->SetPersistenceTimer(PAD_MISC);
         result = nsEventStatus_eConsumeDoDefault;
+
         // Note the current implementation of SetSizeMode just stores
         // the new state; it doesn't actually resize. So here we store
         // the state and pass the event on to the OS. The day is coming
         // when we'll handle the event here, and the return result will
         // then need to be different.
+#ifdef XP_WIN
+        // This is a nasty hack to get around the fact that win32 sends the kill focus
+        // event in a different sequence than the deactivate depending on if you're
+        // minimizing the window vs. just clicking in a different window to cause
+        // the deactivation. Bug #82534
+        if(modeEvent->mSizeMode == nsSizeMode_Minimized) {
+          nsCOMPtr<nsIDOMWindowInternal> domWindow;
+          eventWindow->ConvertWebShellToDOMWindow(webShell, getter_AddRefs(domWindow));
+          if (domWindow) {
+            nsCOMPtr<nsPIDOMWindow> privateDOMWindow = do_QueryInterface(domWindow);
+            if(privateDOMWindow) {
+              nsCOMPtr<nsIFocusController> focusController;
+              privateDOMWindow->GetRootFocusController(getter_AddRefs(focusController));
+              if (focusController)
+                focusController->RewindFocusState();
+            }
+          }
+        }
+#endif
         break;
       }
       case NS_OS_TOOLBAR: {
@@ -589,8 +607,11 @@ nsWebShellWindow::HandleEvent(nsGUIEvent *aEvent)
 
             // since the window has been activated, replace persistent size data
             // with the newly activated window's
-            if (eventWindow->mChromeLoaded)
-              eventWindow->PersistPositionAndSize(PR_TRUE, PR_TRUE, PR_TRUE);
+            if (eventWindow->mChromeLoaded) {
+              eventWindow->PersistentAttributesDirty(
+                             PAD_POSITION | PAD_SIZE | PAD_MISC);
+              eventWindow->SavePersistentAttributes();
+            }
 
             break;
           }
@@ -716,7 +737,7 @@ NS_IMETHODIMP nsWebShellWindow::LoadMenuItem(
     // Try to find the key node.
     nsCOMPtr<nsIDocument> document;
     nsCOMPtr<nsIContent> content = do_QueryInterface(domElement);
-    if (NS_FAILED(rv = content->GetDocument(*getter_AddRefs(document)))) {
+    if (NS_FAILED(rv = content->GetDocument(getter_AddRefs(document)))) {
       NS_ERROR("Unable to retrieve the document.");
       return rv;
     }
@@ -951,17 +972,11 @@ void nsWebShellWindow::DynamicLoadMenus(nsIDOMDocument * aDOMDoc, nsIWidget * aP
       }
 
       nsCOMPtr<nsIPresContext> presContext;
-      if (NS_FAILED(rv = docViewer->GetPresContext(*getter_AddRefs(presContext)))) {
+      if (NS_FAILED(rv = docViewer->GetPresContext(getter_AddRefs(presContext)))) {
           NS_ERROR("Unable to retrieve the doc viewer's presentation context.");
           return;
       }
 
-      nsCOMPtr<nsIPresShell> presShell;
-      if (NS_FAILED(rv = presContext->GetShell(getter_AddRefs(presShell)))) {
-          NS_ERROR("Unable to retrieve the shell from the presentation context.");
-          return;
-      }
-
       nsRect rect;
 
       if (NS_FAILED(rv = mWindow->GetClientBounds(rect))) {
@@ -1151,23 +1166,23 @@ nsWebShellWindow::DestroyModalDialogEvent(PLEvent *aEvent)
 }
 
 void
-nsWebShellWindow::SetPersistenceTimer(PRBool aSize, PRBool aPosition, PRBool aMode)
+nsWebShellWindow::SetPersistenceTimer(PRUint32 aDirtyFlags)
 {
+  if (!mSPTimerLock)
+    return;
+
   PR_Lock(mSPTimerLock);
   if (mSPTimer) {
     mSPTimer->SetDelay(SIZE_PERSISTENCE_TIMEOUT);
-    mSPTimerSize |= aSize;
-    mSPTimerPosition |= aPosition;
-    mSPTimerMode |= aMode;
+    PersistentAttributesDirty(aDirtyFlags);
   } else {
     nsresult rv;
     mSPTimer = do_CreateInstance("@mozilla.org/timer;1", &rv);
     if (NS_SUCCEEDED(rv)) {
-      mSPTimer->Init(FirePersistenceTimer, this,
-                     SIZE_PERSISTENCE_TIMEOUT, NS_TYPE_ONE_SHOT);
-      mSPTimerSize = aSize;
-      mSPTimerPosition = aPosition;
-      mSPTimerMode = aMode;
+      NS_ADDREF_THIS(); // for the timer, which holds a reference to this window
+      mSPTimer->InitWithFuncCallback(FirePersistenceTimer, this,
+                                     SIZE_PERSISTENCE_TIMEOUT, nsITimer::TYPE_ONE_SHOT);
+      PersistentAttributesDirty(aDirtyFlags);
     }
   }
   PR_Unlock(mSPTimerLock);
@@ -1177,11 +1192,11 @@ void
 nsWebShellWindow::FirePersistenceTimer(nsITimer *aTimer, void *aClosure)
 {
   nsWebShellWindow *win = NS_STATIC_CAST(nsWebShellWindow *, aClosure);
+  if (!win->mSPTimerLock)
+    return;
   PR_Lock(win->mSPTimerLock);
-  win->mSPTimer = nsnull;
+  win->SavePersistentAttributes();
   PR_Unlock(win->mSPTimerLock);
-  win->PersistPositionAndSize(win->mSPTimerPosition, win->mSPTimerSize,
-                              win->mSPTimerMode);
 }
 
 
@@ -1196,13 +1211,14 @@ nsWebShellWindow::OnProgressChange(nsIWebProgress *aProgress,
                                    PRInt32 aCurTotalProgress,
                                    PRInt32 aMaxTotalProgress)
 {
+  NS_NOTREACHED("notification excluded in AddProgressListener(...)");
   return NS_OK;
 }
 
 NS_IMETHODIMP
 nsWebShellWindow::OnStateChange(nsIWebProgress *aProgress,
                                 nsIRequest *aRequest,
-                                PRInt32 aStateFlags,
+                                PRUint32 aStateFlags,
                                 nsresult aStatus)
 {
   // If the notification is not about a document finishing, then just
@@ -1247,7 +1263,7 @@ nsWebShellWindow::OnStateChange(nsIWebProgress *aProgress,
       return NS_OK;
 
     nsCOMPtr<nsIDocument> doc;
-    docv->GetDocument(*getter_AddRefs(doc));
+    docv->GetDocument(getter_AddRefs(doc));
     if (!doc)
       return NS_OK;
 
@@ -1289,6 +1305,7 @@ nsWebShellWindow::OnLocationChange(nsIWebProgress *aProgress,
                                    nsIRequest *aRequest,
                                    nsIURI *aURI)
 {
+  NS_NOTREACHED("notification excluded in AddProgressListener(...)");
   return NS_OK;
 }
 
@@ -1298,14 +1315,16 @@ nsWebShellWindow::OnStatusChange(nsIWebProgress* aWebProgress,
                                  nsresult aStatus,
                                  const PRUnichar* aMessage)
 {
+  NS_NOTREACHED("notification excluded in AddProgressListener(...)");
   return NS_OK;
 }
 
 NS_IMETHODIMP
 nsWebShellWindow::OnSecurityChange(nsIWebProgress *aWebProgress,
                                    nsIRequest *aRequest,
-                                   PRInt32 state)
+                                   PRUint32 state)
 {
+  NS_NOTREACHED("notification excluded in AddProgressListener(...)");
   return NS_OK;
 }
 
@@ -1353,9 +1372,11 @@ nsCOMPtr<nsIDOMDocument> nsWebShellWindow::GetNamedDOMDoc(const nsAString & aWeb
     childDocShell = mDocShell;
   } else {
     nsCOMPtr<nsIDocShellTreeItem> docShellAsItem;
-    nsCOMPtr<nsIDocShellTreeNode> docShellAsNode(do_QueryInterface(mDocShell));
-    docShellAsNode->FindChildWithName(PromiseFlatString(aWebShellName).get(), 
-      PR_TRUE, PR_FALSE, nsnull, getter_AddRefs(docShellAsItem));
+    nsCOMPtr<nsIDocShellTreeNodeTmp> docShellAsNode =
+      do_QueryInterface(mDocShell);
+    docShellAsNode->FindChildWithNameTmp(PromiseFlatString(aWebShellName).get(), 
+                                         PR_TRUE, PR_FALSE, nsnull, nsnull,
+                                         getter_AddRefs(docShellAsItem));
     childDocShell = do_QueryInterface(docShellAsItem);
     if (!childDocShell)
       return domDoc;
@@ -1371,7 +1392,7 @@ nsCOMPtr<nsIDOMDocument> nsWebShellWindow::GetNamedDOMDoc(const nsAString & aWeb
     return domDoc;
 
   nsCOMPtr<nsIDocument> doc;
-  docv->GetDocument(*getter_AddRefs(doc));
+  docv->GetDocument(getter_AddRefs(doc));
   if (doc)
     return nsCOMPtr<nsIDOMDocument>(do_QueryInterface(doc));
 
@@ -1395,22 +1416,22 @@ void nsWebShellWindow::LoadContentAreas() {
     nsCOMPtr<nsIDocumentViewer> docViewer = do_QueryInterface(contentViewer);
     if (docViewer) {
       nsCOMPtr<nsIDocument> doc;
-      docViewer->GetDocument(*getter_AddRefs(doc));
-      nsCOMPtr<nsIURI> mainURL;
-      doc->GetDocumentURL(getter_AddRefs(mainURL));
-      if (mainURL) {
-        nsCAutoString search;
-        nsCOMPtr<nsIURL> url = do_QueryInterface(mainURL);
-        if (url)
-          url->GetQuery(search);
-        searchSpec = NS_ConvertUTF8toUCS2(search);
+      docViewer->GetDocument(getter_AddRefs(doc));
+      nsIURI *mainURL = doc->GetDocumentURI();
+
+      nsCAutoString search;
+      nsCOMPtr<nsIURL> url = do_QueryInterface(mainURL);
+      if (url) {
+        url->GetQuery(search);
+
+        CopyUTF8toUTF16(search, searchSpec);
       }
     }
   }
 
   // content URLs are specified in the search part of the URL
   // as <contentareaID>=<escapedURL>[;(repeat)]
-  if (searchSpec.Length() > 0) {
+  if (!searchSpec.IsEmpty()) {
     PRInt32     begPos,
                 eqPos,
                 endPos;
@@ -1468,28 +1489,25 @@ PRBool nsWebShellWindow::ExecuteCloseHandler()
      than it otherwise would.) */
   nsCOMPtr<nsIWebShellWindow> kungFuDeathGrip(this);
 
-  nsresult rv;
-  nsCOMPtr<nsIScriptGlobalObjectOwner> globalObjectOwner(do_QueryInterface(mWebShell));
-  nsCOMPtr<nsIScriptGlobalObject> globalObject;
+  nsCOMPtr<nsIScriptGlobalObject> globalObject(do_GetInterface(mWebShell));
 
-  if (globalObjectOwner) {
-    if (NS_SUCCEEDED(globalObjectOwner->GetScriptGlobalObject(getter_AddRefs(globalObject))) && globalObject) {
-      nsCOMPtr<nsIContentViewer> contentViewer;
-      if (NS_SUCCEEDED(mDocShell->GetContentViewer(getter_AddRefs(contentViewer)))) {
-        nsCOMPtr<nsIDocumentViewer> docViewer;
-        nsCOMPtr<nsIPresContext> presContext;
-        docViewer = do_QueryInterface(contentViewer);
-        if (docViewer && NS_SUCCEEDED(docViewer->GetPresContext(*getter_AddRefs(presContext)))) {
-          nsEventStatus status = nsEventStatus_eIgnore;
-          nsMouseEvent event;
-          event.eventStructType = NS_EVENT;
-          event.message = NS_XUL_CLOSE;
-          rv = globalObject->HandleDOMEvent(presContext, &event, nsnull, NS_EVENT_FLAG_INIT, &status);
-          if (NS_FAILED(rv) || status == nsEventStatus_eConsumeNoDefault)
-            return PR_TRUE;
-          // else fall through and return PR_FALSE
-        }
-      }
+  if (globalObject) {
+    nsCOMPtr<nsIContentViewer> contentViewer;
+    mDocShell->GetContentViewer(getter_AddRefs(contentViewer));
+    nsCOMPtr<nsIDocumentViewer> docViewer(do_QueryInterface(contentViewer));
+
+    if (docViewer) {
+      nsCOMPtr<nsIPresContext> presContext;
+      docViewer->GetPresContext(getter_AddRefs(presContext));
+
+      nsEventStatus status = nsEventStatus_eIgnore;
+      nsMouseEvent event(NS_XUL_CLOSE);
+
+      nsresult rv = globalObject->HandleDOMEvent(presContext, &event, nsnull,
+                                                 NS_EVENT_FLAG_INIT, &status);
+      if (NS_SUCCEEDED(rv) && status == nsEventStatus_eConsumeNoDefault)
+        return PR_TRUE;
+      // else fall through and return PR_FALSE
     }
   }
 
@@ -1499,70 +1517,32 @@ PRBool nsWebShellWindow::ExecuteCloseHandler()
 //----------------------------------------------------------------
 //-- nsIDocumentObserver
 //----------------------------------------------------------------
-NS_IMETHODIMP
-nsWebShellWindow::BeginUpdate(nsIDocument *aDocument)
-{
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsWebShellWindow::EndUpdate(nsIDocument *aDocument)
-{
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsWebShellWindow::BeginLoad(nsIDocument *aDocument)
-{
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsWebShellWindow::EndLoad(nsIDocument *aDocument)
-{
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsWebShellWindow::BeginReflow(nsIDocument *aDocument, nsIPresShell* aShell)
-{
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsWebShellWindow::EndReflow(nsIDocument *aDocument, nsIPresShell* aShell)
-{
-  return NS_OK;
-}
+NS_IMPL_NSIDOCUMENTOBSERVER_CORE_STUB(nsWebShellWindow)
+NS_IMPL_NSIDOCUMENTOBSERVER_LOAD_STUB(nsWebShellWindow)
+NS_IMPL_NSIDOCUMENTOBSERVER_REFLOW_STUB(nsWebShellWindow)
+NS_IMPL_NSIDOCUMENTOBSERVER_STATE_STUB(nsWebShellWindow)
+NS_IMPL_NSIDOCUMENTOBSERVER_STYLE_STUB(nsWebShellWindow)
 
 ///////////////////////////////////////////////////////////////
 // nsIDocumentObserver
 // this is needed for menu changes
 ///////////////////////////////////////////////////////////////
-NS_IMETHODIMP
-nsWebShellWindow::ContentChanged(nsIDocument *aDocument,
-                                 nsIContent* aContent,
-                                 nsISupports* aSubContent)
+void
+nsWebShellWindow::CharacterDataChanged(nsIDocument *aDocument,
+                                       nsIContent* aContent,
+                                       PRBool aAppend)
 {
-  return NS_OK;
 }
 
-NS_IMETHODIMP
-nsWebShellWindow::ContentStatesChanged(nsIDocument *aDocument,
-                                       nsIContent* aContent1,
-                                       nsIContent* aContent2)
-{
-  return NS_OK;
-}
-
-NS_IMETHODIMP
+void
 nsWebShellWindow::AttributeChanged(nsIDocument *aDocument,
                                    nsIContent*  aContent,
                                    PRInt32      aNameSpaceID,
                                    nsIAtom*     aAttribute,
-                                   PRInt32      aModType, 
-                                   PRInt32      aHint)
+                                   PRInt32      aModType)
 {
+  // XXX: Uh, none of this nsIDocumentObserver stuff is needed if the
+  // blow code isn't needed.
 #if 0
   //printf("AttributeChanged\n");
   PRInt32 i;
@@ -1583,96 +1563,38 @@ nsWebShellWindow::AttributeChanged(nsIDocument *aDocument,
     }
   }
 #endif  
-  return NS_OK;
 }
 
-NS_IMETHODIMP
+void
 nsWebShellWindow::ContentAppended(nsIDocument *aDocument,
                             nsIContent* aContainer,
                             PRInt32     aNewIndexInContainer)
 {
-  return NS_OK;
 }
 
-NS_IMETHODIMP
+void
 nsWebShellWindow::ContentInserted(nsIDocument *aDocument,
                             nsIContent* aContainer,
                             nsIContent* aChild,
                             PRInt32 aIndexInContainer)
 {
-  return NS_OK;
 }
 
-NS_IMETHODIMP
+void
 nsWebShellWindow::ContentReplaced(nsIDocument *aDocument,
                             nsIContent* aContainer,
                             nsIContent* aOldChild,
                             nsIContent* aNewChild,
                             PRInt32 aIndexInContainer)
 {
-  return NS_OK;
 }
 
-NS_IMETHODIMP
+void
 nsWebShellWindow::ContentRemoved(nsIDocument *aDocument,
                            nsIContent* aContainer,
                            nsIContent* aChild,
                            PRInt32 aIndexInContainer)
 {
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsWebShellWindow::StyleSheetAdded(nsIDocument *aDocument,
-                            nsIStyleSheet* aStyleSheet)
-{
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsWebShellWindow::StyleSheetRemoved(nsIDocument *aDocument,
-                              nsIStyleSheet* aStyleSheet)
-{
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsWebShellWindow::StyleSheetDisabledStateChanged(nsIDocument *aDocument,
-                                           nsIStyleSheet* aStyleSheet,
-                                           PRBool aDisabled)
-{
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsWebShellWindow::StyleRuleChanged(nsIDocument *aDocument,
-                             nsIStyleSheet* aStyleSheet,
-                             nsIStyleRule* aStyleRule,
-                             PRInt32 aHint)
-{
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsWebShellWindow::StyleRuleAdded(nsIDocument *aDocument,
-                           nsIStyleSheet* aStyleSheet,
-                           nsIStyleRule* aStyleRule)
-{
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsWebShellWindow::StyleRuleRemoved(nsIDocument *aDocument,
-                             nsIStyleSheet* aStyleSheet,
-                             nsIStyleRule* aStyleRule)
-{
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsWebShellWindow::DocumentWillBeDestroyed(nsIDocument *aDocument)
-{
-  return NS_OK;
 }
 
 // This should rightfully be somebody's CONTRACTID?
@@ -1720,21 +1642,26 @@ NS_IMETHODIMP nsWebShellWindow::Destroy()
    if(docv)
       {
       nsCOMPtr<nsIDocument> doc;
-      docv->GetDocument(*getter_AddRefs(doc));
+      docv->GetDocument(getter_AddRefs(doc));
       if(doc)
          doc->RemoveObserver(NS_STATIC_CAST(nsIDocumentObserver*, this));
       }
    }
 #endif
 
+  nsCOMPtr<nsIWebShellWindow> kungFuDeathGrip(this);
+  if (mSPTimerLock) {
   PR_Lock(mSPTimerLock);
   if (mSPTimer) {
     mSPTimer->Cancel();
+    SavePersistentAttributes();
     mSPTimer = nsnull;
-    PersistPositionAndSize(mSPTimerPosition, mSPTimerSize, mSPTimerMode);
+    NS_RELEASE_THIS(); // the timer held a reference to us
   }
   PR_Unlock(mSPTimerLock);
-
+  PR_DestroyLock(mSPTimerLock);
+  mSPTimerLock = nsnull;
+  }
   return nsXULWindow::Destroy();
 }
 
