@@ -37,34 +37,34 @@
  * ***** END LICENSE BLOCK ***** */
 // The history window uses JavaScript in bookmarks.js too.
 
-var gHistoryOutliner;
+var gHistoryTree;
 var gLastHostname;
 var gLastDomain;
 var gGlobalHistory;
 var gPrefService;
+var gIOService;
 var gDeleteByHostname;
 var gDeleteByDomain;
 var gHistoryBundle;
 var gHistoryStatus;
 var gHistoryGrouping = "";
-var gWindowManager = null;
 
-function HistoryInit()
+function HistoryCommonInit()
 {
-    gHistoryOutliner =  document.getElementById("historyOutliner");
+    gHistoryTree =  document.getElementById("historyTree");
     gDeleteByHostname = document.getElementById("menu_deleteByHostname");
     gDeleteByDomain =   document.getElementById("menu_deleteByDomain");
     gHistoryBundle =    document.getElementById("historyBundle");
     gHistoryStatus =    document.getElementById("statusbar-display");
 
-    var outlinerController = new nsOutlinerController(gHistoryOutliner);
+    var treeController = new nsTreeController(gHistoryTree);
     var historyController = new nsHistoryController;
-    gHistoryOutliner.controllers.appendController(historyController);
+    gHistoryTree.controllers.appendController(historyController);
 
     if ("arguments" in window && window.arguments[0] && window.arguments.length >= 1) {
         // We have been supplied a resource URI to root the tree on
         var uri = window.arguments[0];
-        gHistoryOutliner.setAttribute("ref", uri);
+        gHistoryTree.setAttribute("ref", uri);
         if (uri.substring(0,5) == "find:" &&
             !(window.arguments.length > 1 && window.arguments[1] == "newWindow")) {
             // Update the windowtype so that future searches are directed 
@@ -85,12 +85,15 @@ function HistoryInit()
         catch(e) {
             gHistoryGrouping = "day";
         }
-        GroupBy(gHistoryGrouping);
+        UpdateTreeGrouping();
         if (gHistoryStatus) {  // must be the window
             switch(gHistoryGrouping) {
             case "none":
                 document.getElementById("groupByNone").setAttribute("checked", "true");
                 break;
+            case "site":
+                document.getElementById("groupBySite").setAttribute("checked", "true");
+                break;
             case "day":
             default:
                 document.getElementById("groupByDay").setAttribute("checked", "true");
@@ -102,8 +105,19 @@ function HistoryInit()
             pbi.addObserver("browser.history.grouping", groupObserver, false);
         }
     } 
-    gHistoryOutliner.focus();
-    gHistoryOutliner.outlinerBoxObject.view.selection.select(0);
+
+    SortInNewDirection(find_sort_direction(find_sort_column()));
+
+    if (gHistoryStatus)
+        gHistoryTree.focus();
+    gHistoryTree.treeBoxObject.view.selection.select(0);
+}
+
+function HistoryPanelUnload()
+{
+  var pb = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
+  var pbi = pb.QueryInterface(Components.interfaces.nsIPrefBranchInternal);
+  pbi.removeObserver("browser.history.grouping", groupObserver, false);
 }
 
 function updateHistoryCommands()
@@ -112,40 +126,51 @@ function updateHistoryCommands()
     goUpdateCommand("cmd_deleteByDomain");
 }
 
+function historyOnClick(aEvent)
+{
+  // This is kind of a hack but matches the currently implemented behaviour. 
+  // If a status bar is not present, assume we're in sidebar mode, and thus single clicks on containers
+  // will open the container. Single clicks on non-containers are handled below in historyOnSelect.
+  if (!gHistoryStatus && aEvent.button == 0) {
+    var row = { };
+    var col = { };
+    var elt = { };
+    gHistoryTree.treeBoxObject.getCellAt(aEvent.clientX, aEvent.clientY, row, col, elt);
+    if (row.value >= 0 && col.value) {
+      if (!isContainer(gHistoryTree, row.value))
+        OpenURL("current");
+      else if (elt.value != "twisty")
+        gHistoryTree.treeBoxObject.view.toggleOpenState(row.value);
+    }
+  }
+}
+
 function historyOnSelect()
 {
-    if (!gHistoryStatus) {
-        OpenURL(false);
-        return;
-    }
-
     // every time selection changes, save the last hostname
     gLastHostname = "";
     gLastDomain = "";
     var match;
-    var currentIndex = gHistoryOutliner.currentIndex;
-    var rowIsContainer = gHistoryGrouping == "day" ? isContainer(gHistoryOutliner, currentIndex) : false;
-    var url = gHistoryOutliner.outlinerBoxObject.view.getCellText(currentIndex, "URL");
+    var currentIndex = gHistoryTree.currentIndex;
+    var rowIsContainer = currentIndex < 0 || (gHistoryGrouping != "none" && isContainer(gHistoryTree, currentIndex));
+    var url = rowIsContainer ? "" : gHistoryTree.treeBoxObject.view.getCellText(currentIndex, "URL");
 
-    if (url && !rowIsContainer) {
-        // matches scheme://(hostname)...
-        match = url.match(/^.*?:\/\/(?:([^\/:]*)(?::([^\/:]*))?@)?([^\/:]*)(?::([^\/:]*))?(.*)$/);
+    if (url) {
+        if (!gIOService)
+            gIOService = Components.classes['@mozilla.org/network/io-service;1']
+                                   .getService(Components.interfaces.nsIIOService);
+        try {
+            gLastHostname = gIOService.newURI(url, null, null).host;
+            // matches the last foo.bar in foo.bar or baz.foo.bar
+            match = gLastHostname.match(/([^.]+\.[^.]+$)/);
+            if (match)
+                gLastDomain = match[1];
+        } catch (e) {}
+    }
 
-        if (match && match.length>1)
-            gLastHostname = match[3];
-      
+    if (gHistoryStatus)
         gHistoryStatus.label = url;
-    }
-    else {
-        gHistoryStatus.label = "";
-    }
 
-    if (gLastHostname) {
-        // matches the last foo.bar in foo.bar or baz.foo.bar
-        match = gLastHostname.match(/([^.]+\.[^.]+$)/);
-        if (match)
-            gLastDomain = match[1];
-    }
     document.commandDispatcher.updateCommands("select");
 }
 
@@ -200,16 +225,14 @@ nsHistoryController.prototype =
         switch(command) {
         case "cmd_deleteByHostname":
             if (!gGlobalHistory)
-                gGlobalHistory = Components.classes["@mozilla.org/browser/global-history;1"].getService(Components.interfaces.nsIBrowserHistory);
+                gGlobalHistory = Components.classes["@mozilla.org/browser/global-history;2"].getService(Components.interfaces.nsIBrowserHistory);
             gGlobalHistory.removePagesFromHost(gLastHostname, false)
-            gHistoryOutliner.builder.rebuild();
             return true;
 
         case "cmd_deleteByDomain":
             if (!gGlobalHistory)
-                gGlobalHistory = Components.classes["@mozilla.org/browser/global-history;1"].getService(Components.interfaces.nsIBrowserHistory);
+                gGlobalHistory = Components.classes["@mozilla.org/browser/global-history;2"].getService(Components.interfaces.nsIBrowserHistory);
             gGlobalHistory.removePagesFromHost(gLastDomain, true)
-            gHistoryOutliner.builder.rebuild();
             return true;
 
         default:
@@ -221,11 +244,11 @@ nsHistoryController.prototype =
 var historyDNDObserver = {
     onDragStart: function (aEvent, aXferData, aDragAction)
     {
-        var currentIndex = gHistoryOutliner.currentIndex;
-        if (isContainer(gHistoryOutliner, currentIndex))
+        var currentIndex = gHistoryTree.currentIndex;
+        if (isContainer(gHistoryTree, currentIndex))
             return false;
-        var url = gHistoryOutliner.outlinerBoxObject.view.getCellText(currentIndex, "URL");
-        var title = gHistoryOutliner.outlinerBoxObject.view.getCellText(currentIndex, "Name");
+        var url = gHistoryTree.treeBoxObject.view.getCellText(currentIndex, "URL");
+        var title = gHistoryTree.treeBoxObject.view.getCellText(currentIndex, "Name");
 
         var htmlString = "<A HREF='" + url + "'>" + title + "</A>";
         aXferData.data = new TransferData();
@@ -238,103 +261,161 @@ var historyDNDObserver = {
 
 function validClickConditions(event)
 {
-  var currentIndex = gHistoryOutliner.currentIndex;
+  var currentIndex = gHistoryTree.currentIndex;
   if (currentIndex == -1) return false;
-  var container = isContainer(gHistoryOutliner, currentIndex);
+  var container = isContainer(gHistoryTree, currentIndex);
   return (event.button == 0 &&
-          event.originalTarget.localName == 'outlinerchildren' &&
+          event.originalTarget.localName == 'treechildren' &&
           !container && gHistoryStatus);
 }
 
 function collapseExpand()
 {
-    var currentIndex = gHistoryOutliner.currentIndex;
-    gHistoryOutliner.outlinerBoxObject.view.toggleOpenState(currentIndex);
+    var currentIndex = gHistoryTree.currentIndex;
+    gHistoryTree.treeBoxObject.view.toggleOpenState(currentIndex);
 }
 
-function OpenURL(aInNewWindow)
+function OpenURL(aTarget)
 {
-    var currentIndex = gHistoryOutliner.currentIndex;     
-    var builder = gHistoryOutliner.builder.QueryInterface(Components.interfaces.nsIXULOutlinerBuilder);
-    var url = builder.getResourceAtIndex(currentIndex).Value;
+    var currentIndex = gHistoryTree.currentIndex;     
+    var builder = gHistoryTree.builder.QueryInterface(Components.interfaces.nsIXULTreeBuilder);
+    var url = builder.getResourceAtIndex(currentIndex).ValueUTF8;
+    var uri = Components.classes["@mozilla.org/network/standard-url;1"].
+                createInstance(Components.interfaces.nsIURI);
+    uri.spec = url;
+    if (uri.schemeIs("javascript") || uri.schemeIs("data")) {
+      var strBundleService = Components.classes["@mozilla.org/intl/stringbundle;1"]
+                                       .getService(Components.interfaces.nsIStringBundleService);
+      var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
+                                    .getService(Components.interfaces.nsIPromptService);
+      var historyBundle = strBundleService.createBundle("chrome://communicator/locale/history/history.properties");
+      var brandBundle = strBundleService.createBundle("chrome://global/locale/brand.properties");      
+      var brandStr = brandBundle.GetStringFromName("brandShortName");
+      var errorStr = historyBundle.GetStringFromName("load-js-data-url-error");
+      promptService.alert(window, brandStr, errorStr);
+      return false;
+    }
 
-    if (aInNewWindow) {
-      var count = gHistoryOutliner.outlinerBoxObject.view.selection.count;
+    if (aTarget != "current") {
+      var count = gHistoryTree.treeBoxObject.view.selection.count;
+      var URLArray = [];
       if (count == 1) {
-        if (isContainer(gHistoryOutliner, currentIndex))
+        if (isContainer(gHistoryTree, currentIndex))
           openDialog("chrome://communicator/content/history/history.xul", 
                      "", "chrome,all,dialog=no", url, "newWindow");
-        else      
-          openDialog( getBrowserURL(), "_blank", "chrome,all,dialog=no", url );
+        else
+          URLArray.push(url);
       }
       else {
         var min = new Object(); 
         var max = new Object();
-        var rangeCount = gHistoryOutliner.outlinerBoxObject.view.selection.getRangeCount();
+        var rangeCount = gHistoryTree.treeBoxObject.view.selection.getRangeCount();
         for (var i = 0; i < rangeCount; ++i) {
-          gHistoryOutliner.outlinerBoxObject.view.selection.getRangeAt(i, min, max);
+          gHistoryTree.treeBoxObject.view.selection.getRangeAt(i, min, max);
           for (var k = max.value; k >= min.value; --k) {
-            url = gHistoryOutliner.outlinerBoxObject.view.getCellText(k, "URL");
-            window.openDialog(getBrowserURL(), "_blank", "chrome,all,dialog=no", url);
+            url = gHistoryTree.treeBoxObject.view.getCellText(k, "URL");
+            URLArray.push(url);
           }
         }
       }
+      if (aTarget == "window") {
+        for (i = 0; i < URLArray.length; i++) {
+          openDialog(getBrowserURL(), "_blank", "chrome,all,dialog=no", URLArray[i]);
+        }
+      } else {
+        if (URLArray.length > 0)
+          OpenURLArrayInTabs(URLArray);
+      }
     }        
-    else
-        openTopWin(url);
+    else if (!isContainer(gHistoryTree, currentIndex))
+      openTopWin(url);
     return true;
 }
 
-function GroupBy(groupingType)
+// This opens the URLs contained in the given array in new tabs
+// of the most recent window, creates a new window if necessary.
+function OpenURLArrayInTabs(aURLArray)
 {
-    var outliner = document.getElementById("historyOutliner");
-    switch(groupingType) {
-    case "none":
-        outliner.setAttribute("ref", "NC:HistoryRoot");
-        break;
-    case "site":
-        // xxx for now
-        outliner.setAttribute("ref", "NC:HistoryByDate");
-        break;
-    case "day":
-    default:
-        outliner.setAttribute("ref", "NC:HistoryByDate");
-        break;
-    }
-    gPrefService.setCharPref("browser.history.grouping", groupingType);
+  var browserWin = getTopWin();
+  if (browserWin) {
+    var browser = browserWin.getBrowser();
+    browser.selectedTab = browser.addTab(aURLArray[0]);
+    for (var i = 1; i < aURLArray.length; ++i)
+      tab = browser.addTab(aURLArray[i]);
+  } else {
+    openTopWin(aURLArray.join("\n")); // Pretend that we're a home page group
+  }
+}
+
+/**
+ * Root the tree on a given URI (used for displaying search results)
+ */
+function setRoot(root)
+{
+  gHistoryTree.ref = root;
+}
+
+function GroupBy(aGroupingType)
+{
+  gHistoryGrouping = aGroupingType;
+  UpdateTreeGrouping();
+  gPrefService.setCharPref("browser.history.grouping", aGroupingType);
+}
+
+function UpdateTreeGrouping()
+{
+  var tree = document.getElementById("historyTree");
+  switch(gHistoryGrouping) {
+  case "none":
+    tree.setAttribute("ref", "NC:HistoryRoot");
+    break;
+  case "site":
+    tree.setAttribute("ref", "find:datasource=history&groupby=Hostname");
+    break;
+  case "day":
+  default:
+    tree.setAttribute("ref", "NC:HistoryByDate");
+    break;
+  }
 }
 
 var groupObserver = {
   observe: function(aPrefBranch, aTopic, aPrefName) {
     try {
-      GroupBy(aPrefBranch.QueryInterface(Components.interfaces.nsIPrefBranch).getCharPref(aPrefName));
+      gHistoryGrouping = gPrefService.getCharPref("browser.history.grouping");
     }
-    catch(ex) {
+    catch(e) {
+      gHistoryGrouping = "day";
     }
+    UpdateTreeGrouping();
   }
 }
 
 function historyAddBookmarks()
 {
-  var count = gHistoryOutliner.outlinerBoxObject.view.selection.count;
+  var count = gHistoryTree.treeBoxObject.view.selection.count;
   var url;
   var title;
   if (count == 1) {
-    var currentIndex = gHistoryOutliner.currentIndex;
-    url = gHistoryOutliner.outlinerBoxObject.view.getCellText(currentIndex, "URL");
-    title = gHistoryOutliner.outlinerBoxObject.view.getCellText(currentIndex, "Name");
-    BookmarksUtils.addBookmark(url, title, undefined, true);
+    var currentIndex = gHistoryTree.currentIndex;
+    url = gHistoryTree.treeBoxObject.view.getCellText(currentIndex, "URL");
+    title = gHistoryTree.treeBoxObject.view.getCellText(currentIndex, "Name");
+    BookmarksUtils.addBookmark(url, title, null, true);
   }
   else if (count > 1) {
     var min = new Object(); 
     var max = new Object();
-    var rangeCount = gHistoryOutliner.outlinerBoxObject.view.selection.getRangeCount();
+    var rangeCount = gHistoryTree.treeBoxObject.view.selection.getRangeCount();
+    if (!BMSVC) {
+      initServices();
+      initBMService();
+    }
     for (var i = 0; i < rangeCount; ++i) {
-      gHistoryOutliner.outlinerBoxObject.view.selection.getRangeAt(i, min, max);
+      gHistoryTree.treeBoxObject.view.selection.getRangeAt(i, min, max);
       for (var k = max.value; k >= min.value; --k) {
-        url = gHistoryOutliner.outlinerBoxObject.view.getCellText(k, "URL");
-        title = gHistoryOutliner.outlinerBoxObject.view.getCellText(k, "Name");
-        BookmarksUtils.addBookmark(url, title, undefined, false);
+        url = gHistoryTree.treeBoxObject.view.getCellText(k, "URL");
+        title = gHistoryTree.treeBoxObject.view.getCellText(k, "Name");
+        BookmarksUtils.addBookmark(url, title, null, false);
       }
     }
   }
@@ -343,81 +424,92 @@ function historyAddBookmarks()
 
 function updateItems()
 {
-  var count = gHistoryOutliner.outlinerBoxObject.view.selection.count;
+  var count = gHistoryTree.treeBoxObject.view.selection.count;
   var openItem = document.getElementById("miOpen");
   var bookmarkItem = document.getElementById("miAddBookmark");
   var copyLocationItem = document.getElementById("miCopyLinkLocation");
   var sep1 = document.getElementById("pre-bookmarks-separator");
+  var sep2 = document.getElementById("post-bookmarks-separator");
   var openItemInNewWindow = document.getElementById("miOpenInNewWindow");
+  var openItemInNewTab = document.getElementById("miOpenInNewTab");
   var collapseExpandItem = document.getElementById("miCollapseExpand");
   if (count > 1) {
     var hasContainer = false;
-    if (gHistoryGrouping == "day") {
+    if (gHistoryGrouping != "none") {
       var min = new Object(); 
       var max = new Object();
-      var rangeCount = gHistoryOutliner.outlinerBoxObject.view.selection.getRangeCount();
+      var rangeCount = gHistoryTree.treeBoxObject.view.selection.getRangeCount();
       for (var i = 0; i < rangeCount; ++i) {
-        gHistoryOutliner.outlinerBoxObject.view.selection.getRangeAt(i, min, max);
+        gHistoryTree.treeBoxObject.view.selection.getRangeAt(i, min, max);
         for (var k = max.value; k >= min.value; --k) {
-          if (isContainer(gHistoryOutliner, k)) {
+          if (isContainer(gHistoryTree, k)) {
             hasContainer = true;
             break;
           }
         }
       }
     }
-    if (hasContainer) {
-      bookmarkItem.setAttribute("hidden", "true");
-      copyLocationItem.setAttribute("hidden", "true");
-      sep1.setAttribute("hidden", "true");
-      document.getElementById("post-bookmarks-separator").setAttribute("hidden", "true");
-      openItem.setAttribute("hidden", "true");
-      openItemInNewWindow.setAttribute("hidden", "true");
-      collapseExpandItem.setAttribute("hidden", "true");
+    collapseExpandItem.hidden = true;
+    openItem.hidden = true;
+    if (hasContainer) {   // several items selected, folder(s) amongst them
+      bookmarkItem.hidden = true;
+      copyLocationItem.hidden = true;
+      sep1.hidden = true;
+      sep2.hidden = true;
+      openItemInNewWindow.hidden = true;
+      openItemInNewTab.hidden = true;
     }
-    else {
-      bookmarkItem.removeAttribute("hidden");
-      copyLocationItem.removeAttribute("hidden");
-      sep1.removeAttribute("hidden");
+    else {                // several items selected, but no folder
+      bookmarkItem.hidden = false;
+      copyLocationItem.hidden = false;
+      sep1.hidden = false;
+      sep2.hidden = false;
       bookmarkItem.setAttribute("label", document.getElementById('multipleBookmarks').getAttribute("label"));
-      openItem.setAttribute("hidden", "true");
+      bookmarkItem.setAttribute("accesskey", document.getElementById('multipleBookmarks').getAttribute("accesskey"));
       openItem.removeAttribute("default");
       openItemInNewWindow.setAttribute("default", "true");
+      openItemInNewWindow.hidden = false;
+      openItemInNewTab.hidden = false;
     }
   }
   else {
+    openItemInNewWindow.hidden = false;
     bookmarkItem.setAttribute("label", document.getElementById('oneBookmark').getAttribute("label"));
-    var currentIndex = gHistoryOutliner.currentIndex;
-    if (isContainer(gHistoryOutliner, currentIndex)) {
-        openItem.setAttribute("hidden", "true");
+    bookmarkItem.setAttribute("accesskey", document.getElementById('oneBookmark').getAttribute("accesskey"));
+    sep2.hidden = false;
+    var currentIndex = gHistoryTree.currentIndex;
+    if (isContainer(gHistoryTree, currentIndex)) {   // one folder selected
+        openItem.hidden = true;
         openItem.removeAttribute("default");
-        collapseExpandItem.removeAttribute("hidden");
+        openItemInNewWindow.removeAttribute("default");
+        openItemInNewTab.hidden = true;
+        collapseExpandItem.hidden = false;
         collapseExpandItem.setAttribute("default", "true");
-        bookmarkItem.setAttribute("hidden", "true");
-        copyLocationItem.setAttribute("hidden", "true");
-        sep1.setAttribute("hidden", "true");
-        if (isContainerOpen(gHistoryOutliner, currentIndex))
+        bookmarkItem.hidden = true;
+        copyLocationItem.hidden = true;
+        sep1.hidden = true;
+        if (isContainerOpen(gHistoryTree, currentIndex)) {
           collapseExpandItem.setAttribute("label", gHistoryBundle.getString("collapseLabel"));
-        else
+          collapseExpandItem.setAttribute("accesskey", gHistoryBundle.getString("collapseAccesskey"));
+        }
+        else {
           collapseExpandItem.setAttribute("label", gHistoryBundle.getString("expandLabel"));
+          collapseExpandItem.setAttribute("accesskey", gHistoryBundle.getString("expandAccesskey"));          
+        }
         return true;
-    }
-    collapseExpandItem.setAttribute("hidden", "true");
-    bookmarkItem.removeAttribute("hidden");
-    copyLocationItem.removeAttribute("hidden");
-    sep1.removeAttribute("hidden");
-    if (!gWindowManager) {
-      gWindowManager = Components.classes['@mozilla.org/rdf/datasource;1?name=window-mediator'].getService();
-      gWindowManager = gWindowManager.QueryInterface( Components.interfaces.nsIWindowMediator);
-    }
-    var topWindowOfType = gWindowManager.getMostRecentWindow("navigator:browser");
-    if (!topWindowOfType) {
-        openItem.setAttribute("hidden", "true");
+    }                                                // one entry selected (not a folder)
+    openItemInNewTab.hidden = false;
+    collapseExpandItem.hidden = true;
+    bookmarkItem.hidden = false;
+    copyLocationItem.hidden = false;
+    sep1.hidden = false;
+    if (!getTopWin()) {
+        openItem.hidden = true;
         openItem.removeAttribute("default");
         openItemInNewWindow.setAttribute("default", "true");
     }
     else {
-      openItem.removeAttribute("hidden");
+      openItem.hidden = false;
       if (!openItem.getAttribute("default"))
         openItem.setAttribute("default", "true");
       openItemInNewWindow.removeAttribute("default");
@@ -425,7 +517,3 @@ function updateItems()
   }
   return true;
 }
-
-
-
- 
\ No newline at end of file
