@@ -10,15 +10,15 @@
  * implied. See the License for the specific language governing
  * rights and limitations under the License.
  *
- * The Original Code is Mozilla Communicator client code,
- * released March 31, 1998.
+ * The Original Code is Mozilla Communicator client code, 
+ * released March 31, 1998. 
  *
- * The Initial Developer of the Original Code is Netscape Communications
+ * The Initial Developer of the Original Code is Netscape Communications 
  * Corporation.  Portions created by Netscape are
  * Copyright (C) 1998 Netscape Communications Corporation. All
  * Rights Reserved.
  *
- * Contributor(s):
+ * Contributor(s): 
  *     Daniel Veditz <dveditz@netscape.com>
  *     Douglas Turner <dougt@netscape.com>
  */
@@ -39,8 +39,10 @@ GetPersistentStringFromSpec(nsIFile* inSpec, nsACString &string)
 {
     nsresult rv;
 
+    nsCOMPtr<nsILocalFile> LocalFile = do_QueryInterface(inSpec, &rv);
+
     if (NS_SUCCEEDED(rv)) {
-        rv = inSpec->GetNativePath(string);
+        rv = LocalFile->GetNativePath(string);
     } 
     else {
         string.Truncate();
@@ -60,7 +62,7 @@ PRInt32 ReplaceWindowsSystemFile(nsIFile* currentSpec, nsIFile* finalSpec)
 {
     PRInt32 err = -1;
 
-    // Get OS version info
+    // Get OS version info 
     DWORD dwVersion = GetVersion();
 
     nsCAutoString final;
@@ -93,12 +95,12 @@ PRInt32 ReplaceWindowsSystemFile(nsIFile* currentSpec, nsIFile* finalSpec)
         {
             final = Dest;
         }
-
+        
         // NOTE: use OEM filenames! Even though it looks like a Windows
         //       .INI file, WININIT.INI is processed under DOS 
         
-        AnsiToOem( (char*)final.get(), (char*)final.get() );
-        AnsiToOem( (char*)current.get(), (char*)current.get() );
+        AnsiToOem( final.BeginWriting(), final.BeginWriting() );
+        AnsiToOem( current.BeginWriting(), current.BeginWriting() );
 
         if ( WritePrivateProfileString( "Rename", final.get(), current.get(), "WININIT.INI" ) )
             err = 0;
@@ -109,6 +111,7 @@ PRInt32 ReplaceWindowsSystemFile(nsIFile* currentSpec, nsIFile* finalSpec)
         if ( MoveFileEx(final.get(), current.get(), MOVEFILE_DELAY_UNTIL_REBOOT) )
           err = 0;
     }
+    
     return err;
 }
 #endif
@@ -118,7 +121,7 @@ nsresult GetRegFilePath(nsACString &regFilePath)
     nsresult rv;
     nsCOMPtr<nsILocalFile> iFileUtilityPath;
     //Get the program directory
-    nsCOMPtr<nsIProperties> directoryService =
+    nsCOMPtr<nsIProperties> directoryService = 
              do_GetService(NS_DIRECTORY_SERVICE_CONTRACTID, &rv);
     if (NS_FAILED(rv))
         return nsnull;
@@ -128,7 +131,7 @@ nsresult GetRegFilePath(nsACString &regFilePath)
         nsCOMPtr<nsIFile> tmp;
         rv = nsSoftwareUpdate::GetProgramDirectory()->Clone(getter_AddRefs(tmp));
 
-        if (NS_FAILED(rv) || !tmp)
+        if (NS_FAILED(rv) || !tmp) 
             return nsnull;
 
 #if defined (XP_MAC)
@@ -142,12 +145,12 @@ nsresult GetRegFilePath(nsACString &regFilePath)
                                   NS_GET_IID(nsIFile),
                                   getter_AddRefs(iFileUtilityPath));
     }
-    if (NS_FAILED(rv) || !iFileUtilityPath)
+    if (NS_FAILED(rv) || !iFileUtilityPath) 
         return nsnull;
 
     iFileUtilityPath->AppendNative(CLEANUP_REGISTRY);
 
-    //Yes, we know using GetNativePath is buggy on the Mac.
+    //Yes, we know using GetPath is buggy on the Mac.
     //When libreg is fixed to accept nsIFiles we'll change this to match.
     return iFileUtilityPath->GetNativePath(regFilePath);
 }
@@ -155,16 +158,16 @@ nsresult GetRegFilePath(nsACString &regFilePath)
 
 PRInt32 DeleteFileNowOrSchedule(nsIFile* filename)
 {
-    PRBool flagExists;
+    PRBool flagExists;  
     PRInt32 result = nsInstall::SUCCESS;
 
     filename->Remove(PR_FALSE);
     filename->Exists(&flagExists);
     if (flagExists)
         result = ScheduleFileForDeletion(filename);
-
+ 
     return result;
-}
+} 
 
 PRInt32 ScheduleFileForDeletion(nsIFile *filename)
 {
@@ -177,7 +180,7 @@ PRInt32 ScheduleFileForDeletion(nsIFile *filename)
 
     nsCAutoString path;
     GetRegFilePath(path);
-    err = NR_RegOpen((char*)path.get(), &reg);
+    err = NR_RegOpen(NS_CONST_CAST(char*, path.get()), &reg);
 
     if ( err == REGERR_OK )
     {
@@ -217,18 +220,22 @@ PRInt32 ScheduleFileForDeletion(nsIFile *filename)
 
 
 
-PRInt32 ReplaceFileNow(nsIFile* replacementFile, nsIFile* doomedFile )
+PRInt32 ReplaceFileNow(nsIFile* aReplacementFile, nsIFile* aDoomedFile )
 {
-    PRBool flagExists, flagIsEqual;
+    PRBool flagExists, flagRenamedDoomedFileExists, flagIsEqual;
+    nsCOMPtr<nsIFile> replacementFile;
     nsresult rv;
 
+    // make a clone of aReplacement file so we touch affect callers
+    aReplacementFile->Clone(getter_AddRefs(replacementFile));
+
     // replacement file must exist, doomed file doesn't have to
     replacementFile->Exists(&flagExists);
     if ( !flagExists )
         return nsInstall::DOES_NOT_EXIST;
 
     // don't have to do anything if the files are the same
-    replacementFile->Equals(doomedFile, &flagIsEqual);
+    replacementFile->Equals(aDoomedFile, &flagIsEqual);
     if ( flagIsEqual )
         return nsInstall::SUCCESS;
 
@@ -238,91 +245,129 @@ PRInt32 ReplaceFileNow(nsIFile* replacementFile, nsIFile* doomedFile )
     // first try to rename the doomed file out of the way (if it exists)
     nsCOMPtr<nsIFile>      renamedDoomedFile;
     nsCOMPtr<nsILocalFile> tmpLocalFile;
-    nsCOMPtr<nsIFile> parent;
-    nsCAutoString leafname;
     
-    doomedFile->Clone(getter_AddRefs(renamedDoomedFile));
-    renamedDoomedFile->Exists(&flagExists);
-    if ( flagExists )
+    aDoomedFile->Clone(getter_AddRefs(renamedDoomedFile));
+    renamedDoomedFile->Exists(&flagRenamedDoomedFileExists);
+    if ( flagRenamedDoomedFileExists )
     {
+#ifdef XP_MACOSX
+        // If we clone an nsIFile, and move the clone, the FSRef of the *original*
+        // file is not what you would expect - it points to the moved file. This
+        // is despite the fact that the two FSRefs are independent objects. Until
+        // the OS X file impl is changed to not use FSRefs, need to do this (see
+        // bug 200024).
+        nsCOMPtr<nsILocalFile> doomedFileLocal = do_QueryInterface(aDoomedFile);
+        nsCAutoString doomedFilePath;
+        rv = doomedFileLocal->GetNativePath(doomedFilePath);
+        if (NS_FAILED(rv))
+            return nsInstall::UNEXPECTED_ERROR;
+#endif
+
         tmpLocalFile = do_QueryInterface(renamedDoomedFile, &rv); // Convert to an nsILocalFile
 
-        //get the leafname so we can convert its extension to .old
+        //get the doomedLeafname so we can convert its extension to .old
+        nsAutoString doomedLeafname;
         nsCAutoString uniqueLeafName;
-        tmpLocalFile->GetNativeLeafName(leafname);
+        tmpLocalFile->GetLeafName(doomedLeafname);
 
-        PRInt32 extpos = leafname.RFindChar('.');
+        // do not RFind on the native charset! UTF8 or Unicode are OK
+        PRInt32 extpos = doomedLeafname.RFindChar('.');
         if (extpos != -1)
         {
             // We found the extension; 
-            leafname.Truncate(extpos + 1); //strip off the old extension
+            doomedLeafname.Truncate(extpos + 1); //strip off the old extension
         }
-        leafname.Append(NS_LITERAL_CSTRING("old"));
+        doomedLeafname.Append(NS_LITERAL_STRING("old"));
         
-        //Now reset the leafname
-        tmpLocalFile->SetNativeLeafName(leafname);
-        
-        MakeUnique(tmpLocalFile);                                 //  for the call to MakeUnique
+        //Now reset the doomedLeafname
+        tmpLocalFile->SetLeafName(doomedLeafname);
+        tmpLocalFile->CreateUnique(nsIFile::NORMAL_FILE_TYPE, 0644);
+        tmpLocalFile->GetNativeLeafName(uniqueLeafName);//this is the new "unique" doomedLeafname
 
-        tmpLocalFile->GetParent(getter_AddRefs(parent)); //get the parent for later use in MoveTo
-        tmpLocalFile->GetNativeLeafName(uniqueLeafName);//this is the new "unique" leafname
-
-        rv = doomedFile->Clone(getter_AddRefs(renamedDoomedFile));// Reset renamedDoomed file so doomedfile isn't
-                                                                  //   changed during the MoveTo call
-        if (NS_FAILED(rv)) result = nsInstall::UNEXPECTED_ERROR;
-        rv = renamedDoomedFile->MoveToNative(parent, uniqueLeafName);        
-
-        if (NS_SUCCEEDED(rv))
+        rv = aDoomedFile->Clone(getter_AddRefs(renamedDoomedFile));// Reset renamedDoomed file so aDoomedfile
+                                                                   // isn't changed during the MoveTo call
+        if (NS_FAILED(rv))
+            result = nsInstall::UNEXPECTED_ERROR;
+        else
         {
-            renamedDoomedFile = parent;                      //MoveTo on Mac doesn't reset the tmpFile object to 
-            renamedDoomedFile->AppendNative(uniqueLeafName); //the new name or location. That's why there's this 
-                                                             //explict assignment and Append call.
+            rv = renamedDoomedFile->MoveToNative(nsnull, uniqueLeafName);        
+            if (NS_FAILED(rv))
+            {
+                // MoveToNative() failing is OK.  It simply means that the file
+                // was locked in memory and needs to be replaced on browser
+                // shutdown or system reboot.
+                //
+                // Since renamedDoomedFile->MoveToNative() failed, it created a
+                // 0 byte '-old' file that needs to be cleaned up.
+                tmpLocalFile->Remove(PR_FALSE);
+            }
+            else
+            {
+                // The implementation of MoveToNative() on some platforms (osx and win32) resets
+                // the object to the 'moved to' filename.  This is incorrect behavior.  This
+                // implementation will be fixed in the future.  We need to take into account that
+                // fix by setting renamedDoomedFile to the filename that it was moved to above.
+                // See bug 200024.
+                //
+                // renamedDoomedFile needs to be reset because it's used later on in this
+                // function.
+                rv = renamedDoomedFile->SetNativeLeafName(uniqueLeafName);        
+                if (NS_FAILED(rv))
+                    result = nsInstall::UNEXPECTED_ERROR;
+            }
         }
 
+#ifdef XP_MACOSX
+        rv = doomedFileLocal->InitWithNativePath(doomedFilePath);
+        if (NS_FAILED(rv))
+            result = nsInstall::UNEXPECTED_ERROR;
+#endif
+
         if (result == nsInstall::UNEXPECTED_ERROR)
             return result;
     }
 
 
-    // if doomedFile is gone move new file into place
-    doomedFile->Exists(&flagExists);
-    if ( !flagExists )
+    // if aDoomedFile is still in the way, give up and return result.
+    aDoomedFile->Exists(&flagExists);
+    if ( flagExists )
+        return result;
+
+    nsCOMPtr<nsIFile> parentofDoomedFile;
+    nsCAutoString doomedLeafname;
+
+    rv = aDoomedFile->GetParent(getter_AddRefs(parentofDoomedFile));
+    if ( NS_SUCCEEDED(rv) )
+        rv = aDoomedFile->GetNativeLeafName(doomedLeafname);
+    if ( NS_SUCCEEDED(rv) )
     {
-        nsCOMPtr<nsIFile> parentofFinalFile;
-        nsCOMPtr<nsIFile> parentofReplacementFile;
-        nsCAutoString leafname;
+        rv = replacementFile->MoveToNative(parentofDoomedFile, doomedLeafname);
+        // The implementation of MoveToNative() on some platforms (osx and win32) resets
+        // the object to the 'moved to' filename.  This is incorrect behavior.  This
+        // implementation will be fixed in the future.  We need to take into account that
+        // fix by setting replacementFile to the filename that it was moved to above.
+        // See bug 200024.
+        //
+        // However, since replacementFile is a clone of aReplacementFile and is also
+        // not used beyond here, there's no need to set the path+leafname to what
+        // it was MoveToNative()'ed to.
+    }
 
-        doomedFile->GetParent(getter_AddRefs(parentofFinalFile));
-        replacementFile->GetParent(getter_AddRefs(parentofReplacementFile));
-
-        // XXX looks dangerous, the replacement file name may NOT be unique in the
-        // target directory if we have to move it! Either we should never move the
-        // files like this (i.e. error if not in the same dir) or we need to take
-        // a little more care in the move.
-        parentofReplacementFile->Equals(parentofFinalFile, &flagIsEqual);
-        if(!flagIsEqual)
-        {
-            NS_WARNING("File unpacked into a non-dest dir" );
-            replacementFile->GetNativeLeafName(leafname);
-            rv = replacementFile->MoveToNative(parentofFinalFile, leafname);
-        }
-        	
-        rv = doomedFile->GetNativeLeafName(leafname);
-        if ( NS_SUCCEEDED(rv))
-            rv = replacementFile->MoveToNative(parentofReplacementFile, leafname);
-
-        if ( NS_SUCCEEDED(rv) )
+    if (NS_SUCCEEDED(rv))
+    {
+        if (flagRenamedDoomedFileExists)
         {
             // we replaced the old file OK, now we have to
             // get rid of it if it was renamed out of the way
             result = DeleteFileNowOrSchedule( renamedDoomedFile );
         }
-        else
-        {
-            // couldn't rename file, try to put old file back
-            renamedDoomedFile->GetParent(getter_AddRefs(parent));
-            renamedDoomedFile->MoveToNative(parent, leafname);
-        }
+    }
+    else
+    {
+        // couldn't rename file, try to put old file back
+        renamedDoomedFile->MoveToNative(nsnull, doomedLeafname);
+        // No need to reset remanedDoomedFile after a MoveToNative() call
+        // because renamedDoomedFile is not used beyond here.
     }
 
     return result;
@@ -332,16 +377,16 @@ PRInt32 ReplaceFileNow(nsIFile* replacementFile, nsIFile* doomedFile )
 
 
 
-PRInt32 ReplaceFileNowOrSchedule(nsIFile* replacementFile, nsIFile* doomedFile, PRInt32 aMode)
+PRInt32 ReplaceFileNowOrSchedule(nsIFile* aReplacementFile, nsIFile* aDoomedFile, PRInt32 aMode)
 {
-    PRInt32 result = ReplaceFileNow( replacementFile, doomedFile );
+    PRInt32 result = ReplaceFileNow( aReplacementFile, aDoomedFile );
 
     if ( result == nsInstall::ACCESS_DENIED )
     {
         // if we couldn't replace the file schedule it for later
 #ifdef _WINDOWS
-        if ( (aMode & WIN_SYSTEM_FILE) &&
-             (ReplaceWindowsSystemFile(replacementFile, doomedFile) == 0) )
+        if ( (aMode & WIN_SYSTEM_FILE) && 
+             (ReplaceWindowsSystemFile(aReplacementFile, aDoomedFile) == 0) )
                 return nsInstall::REBOOT_NEEDED;
 #endif
 
@@ -352,10 +397,10 @@ PRInt32 ReplaceFileNowOrSchedule(nsIFile* replacementFile, nsIFile* doomedFile,
 
         nsCAutoString regFilePath;
         GetRegFilePath(regFilePath);
-        if ( REGERR_OK == NR_RegOpen((char*)regFilePath.get(), &reg) ) 
+        if ( REGERR_OK == NR_RegOpen(NS_CONST_CAST(char*, regFilePath.get()), &reg) ) 
         {
             err = NR_RegAddKey( reg, ROOTKEY_PRIVATE, REG_REPLACE_LIST_KEY, &listkey );
-            if ( err == REGERR_OK )
+            if ( err == REGERR_OK ) 
             {
                 char     valname[20];
                 REGERR   err2;
@@ -368,16 +413,16 @@ PRInt32 ReplaceFileNowOrSchedule(nsIFile* replacementFile, nsIFile* doomedFile,
                     {
                         nsCAutoString srcowner;
                         nsCAutoString destowner;
-                        nsresult rv = GetPersistentStringFromSpec(replacementFile, srcowner);
-                        nsresult rv2 = GetPersistentStringFromSpec(doomedFile, destowner);
+                        nsresult rv = GetPersistentStringFromSpec(aReplacementFile, srcowner);
+                        nsresult rv2 = GetPersistentStringFromSpec(aDoomedFile, destowner);
                         if ( NS_SUCCEEDED(rv) && NS_SUCCEEDED(rv2) )
                         {
                             const char *fsrc  = srcowner.get();
                             const char *fdest = destowner.get();
                             err = NR_RegSetEntry( reg, filekey, 
                                                   REG_REPLACE_SRCFILE,
-                                                  REGTYPE_ENTRY_BYTES,
-                                                  (void*)fsrc,
+                                                  REGTYPE_ENTRY_BYTES, 
+                                                  (void*)fsrc, 
                                                   strlen(fsrc)+sizeof('\0'));
 
                             err2 = NR_RegSetEntry(reg, filekey,
@@ -431,7 +476,7 @@ void DeleteScheduledFiles( HREG reg )
     REGENUM state = 0;
     nsresult rv = NS_OK;
 
-    // perform scheduled file deletions
+    // perform scheduled file deletions  
     if (REGERR_OK == NR_RegGetKey(reg,ROOTKEY_PRIVATE,REG_DELETE_LIST_KEY,&key))
     {
         // the delete key exists, so we loop through its children
@@ -452,12 +497,13 @@ void DeleteScheduledFiles( HREG reg )
                 err = NR_RegGetEntry( reg, key, namebuf, valbuf, &bufsize );
                 if ( err == REGERR_OK )
                 {
-                    // no need to check return value of
+                    // no need to check return value of 
                     // SetPersistentDescriptorString, it's always NS_OK
-                    //spec->SetPersistentDescriptorString(valbuf); //nsIFileXXX: Do we still need this instead of InitWithPath?
+                    //spec->SetPersistentDescriptorString(valbuf);
+                    //nsIFileXXX: Do we still need this instead of InitWithPath?
                     NS_NewNativeLocalFile(nsDependentCString(valbuf), PR_TRUE, getter_AddRefs(spec));
                     spec->Clone(getter_AddRefs(doomedFile));
-                    if (NS_SUCCEEDED(rv))
+                    if (NS_SUCCEEDED(rv)) 
                     {
                         PRBool flagExists;
                         doomedFile->Remove(PR_FALSE);
@@ -471,7 +517,7 @@ void DeleteScheduledFiles( HREG reg )
                 }
             }
 
-            // delete list node if empty
+            // delete list node if empty 
             state = 0;
             err = NR_RegEnumEntries(reg, key, &state, namebuf, sizeof(namebuf), 0);
             if ( err == REGERR_NOMORE )
@@ -488,7 +534,7 @@ void ReplaceScheduledFiles( HREG reg )
 {
     RKEY    key;
 
-    // replace files if any listed
+    // replace files if any listed 
     if (REGERR_OK == NR_RegGetKey(reg,ROOTKEY_PRIVATE,REG_REPLACE_LIST_KEY,&key))
     {
         char keyname[MAXREGNAMELEN];
@@ -503,7 +549,7 @@ void ReplaceScheduledFiles( HREG reg )
 
         uint32 bufsize;
         REGENUM state = 0;
-        while (REGERR_OK == NR_RegEnumSubkeys( reg, key, &state,
+        while (REGERR_OK == NR_RegEnumSubkeys( reg, key, &state, 
                                keyname, sizeof(keyname), REGENUM_CHILDREN))
         {
             bufsize = sizeof(srcFile);
@@ -538,7 +584,7 @@ void ReplaceScheduledFiles( HREG reg )
         }
 
 
-        // delete list node if empty
+        // delete list node if empty 
         state = 0;
         if (REGERR_NOMORE == NR_RegEnumSubkeys( reg, key, &state, keyname,
                                      sizeof(keyname), REGENUM_CHILDREN ))
