@@ -71,6 +71,23 @@ static void PR_CALLBACK WinRegCleanup(JSContext *cx, JSObject *obj)
     delete nativeThis;
 }
 
+/***********************************************************************/
+//
+// class for WinReg
+//
+JSClass WinRegClass = {
+  "WinReg",
+  JSCLASS_HAS_PRIVATE,
+  JS_PropertyStub,
+  JS_PropertyStub,
+  JS_PropertyStub,
+  JS_PropertyStub,
+  JS_EnumerateStub,
+  JS_ResolveStub,
+  JS_ConvertStub,
+  WinRegCleanup
+};
+
 /***********************************************************************************/
 // Native mothods for WinReg functions
 
@@ -80,17 +97,15 @@ static void PR_CALLBACK WinRegCleanup(JSContext *cx, JSObject *obj)
 PR_STATIC_CALLBACK(JSBool)
 WinRegSetRootKey(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
 {
-  nsWinReg *nativeThis  = (nsWinReg*)JS_GetPrivate(cx, obj);
+  nsWinReg *nativeThis =
+    (nsWinReg*)JS_GetInstancePrivate(cx, obj, &WinRegClass, argv);
+  if (!nativeThis)
+    return JS_FALSE;
+
   PRInt32  b0;
 
   *rval = JSVAL_NULL;
 
-  // If there's no private data, this must be the prototype, so ignore
-  if(nsnull == nativeThis)
-  {
-    return JS_TRUE;
-  }
-
   if(argc >= 1)
   {
     //  public void setRootKey(PRInt32 key);
@@ -118,18 +133,16 @@ WinRegSetRootKey(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *r
 PR_STATIC_CALLBACK(JSBool)
 WinRegKeyExists(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
 {
-  nsWinReg *nativeThis = (nsWinReg*)JS_GetPrivate(cx, obj);
+  nsWinReg *nativeThis =
+    (nsWinReg*)JS_GetInstancePrivate(cx, obj, &WinRegClass, argv);
+  if (!nativeThis)
+    return JS_FALSE;
+
   PRBool nativeRet;
   nsAutoString b0;
 
   *rval = JSVAL_FALSE;
 
-  // If there's no private data, this must be the prototype, so ignore
-  if(nsnull == nativeThis)
-  {
-    return JS_TRUE;
-  }
-
   if(argc >= 1)
   {
     //  public boolean keyExists ( String subKey );
@@ -159,19 +172,17 @@ WinRegKeyExists(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rv
 PR_STATIC_CALLBACK(JSBool)
 WinRegValueExists(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
 {
-  nsWinReg *nativeThis = (nsWinReg*)JS_GetPrivate(cx, obj);
+  nsWinReg *nativeThis =
+    (nsWinReg*)JS_GetInstancePrivate(cx, obj, &WinRegClass, argv);
+  if (!nativeThis)
+    return JS_FALSE;
+
   PRBool nativeRet;
   nsAutoString b0;
   nsAutoString b1;
 
   *rval = JSVAL_FALSE;
 
-  // If there's no private data, this must be the prototype, so ignore
-  if(nsnull == nativeThis)
-  {
-    return JS_TRUE;
-  }
-
   if(argc >= 2)
   {
     //  public boolean valueExists ( String subKey,
@@ -199,18 +210,16 @@ WinRegValueExists(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *
 PR_STATIC_CALLBACK(JSBool)
 WinRegIsKeyWritable(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
 {
-  nsWinReg *nativeThis = (nsWinReg*)JS_GetPrivate(cx, obj);
+  nsWinReg *nativeThis =
+    (nsWinReg*)JS_GetInstancePrivate(cx, obj, &WinRegClass, argv);
+  if (!nativeThis)
+    return JS_FALSE;
+
   PRInt32 nativeRet;
   nsAutoString b0;
 
   *rval = JSVAL_FALSE;
 
-  // If there's no private data, this must be the prototype, so ignore
-  if(nsnull == nativeThis)
-  {
-    return JS_TRUE;
-  }
-
   if(argc >= 1)
   {
     //  public boolean isKeyWritable ( String subKey );
@@ -236,19 +245,17 @@ WinRegIsKeyWritable(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval
 PR_STATIC_CALLBACK(JSBool)
 WinRegCreateKey(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
 {
-  nsWinReg *nativeThis = (nsWinReg*)JS_GetPrivate(cx, obj);
+  nsWinReg *nativeThis =
+    (nsWinReg*)JS_GetInstancePrivate(cx, obj, &WinRegClass, argv);
+  if (!nativeThis)
+    return JS_FALSE;
+
   PRInt32 nativeRet;
   nsAutoString b0;
   nsAutoString b1;
 
   *rval = INT_TO_JSVAL(nsInstall::UNEXPECTED_ERROR);
 
-  // If there's no private data, this must be the prototype, so ignore
-  if(nsnull == nativeThis)
-  {
-    return JS_TRUE;
-  }
-
   if(argc >= 2)                             
   {
     //  public int createKey ( String subKey,
@@ -276,18 +283,16 @@ WinRegCreateKey(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rv
 PR_STATIC_CALLBACK(JSBool)
 WinRegDeleteKey(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
 {
-  nsWinReg *nativeThis = (nsWinReg*)JS_GetPrivate(cx, obj);
+  nsWinReg *nativeThis =
+    (nsWinReg*)JS_GetInstancePrivate(cx, obj, &WinRegClass, argv);
+  if (!nativeThis)
+    return JS_FALSE;
+
   PRInt32 nativeRet;
   nsAutoString b0;
 
   *rval = INT_TO_JSVAL(nsInstall::UNEXPECTED_ERROR);
 
-  // If there's no private data, this must be the prototype, so ignore
-  if(nsnull == nativeThis)
-  {
-    return JS_TRUE;
-  }
-
   if(argc >= 1)                             
   {
     //  public int deleteKey ( String subKey);
@@ -314,19 +319,17 @@ WinRegDeleteKey(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rv
 PR_STATIC_CALLBACK(JSBool)
 WinRegDeleteValue(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
 {
-  nsWinReg *nativeThis = (nsWinReg*)JS_GetPrivate(cx, obj);
+  nsWinReg *nativeThis =
+    (nsWinReg*)JS_GetInstancePrivate(cx, obj, &WinRegClass, argv);
+  if (!nativeThis)
+    return JS_FALSE;
+
   PRInt32 nativeRet;
   nsString b0;
   nsString b1;
 
   *rval = INT_TO_JSVAL(nsInstall::UNEXPECTED_ERROR);
 
-  // If there's no private data, this must be the prototype, so ignore
-  if(nsnull == nativeThis)
-  {
-    return JS_TRUE;
-  }
-
   if(argc >= 2)                             
   {
     //  public int deleteValue ( String subKey,
@@ -354,7 +357,11 @@ WinRegDeleteValue(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *
 PR_STATIC_CALLBACK(JSBool)
 WinRegSetValueString(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
 {
-  nsWinReg *nativeThis = (nsWinReg*)JS_GetPrivate(cx, obj);
+  nsWinReg *nativeThis =
+    (nsWinReg*)JS_GetInstancePrivate(cx, obj, &WinRegClass, argv);
+  if (!nativeThis)
+    return JS_FALSE;
+
   PRInt32 nativeRet;
   nsAutoString b0;
   nsAutoString b1;
@@ -362,12 +369,6 @@ WinRegSetValueString(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsva
 
   *rval = INT_TO_JSVAL(nsInstall::UNEXPECTED_ERROR);
 
-  // If there's no private data, this must be the prototype, so ignore
-  if(nsnull == nativeThis)
-  {
-    return JS_TRUE;
-  }
-
   if(argc >= 3)
   {
     //  public int setValueString ( String subKey,
@@ -397,19 +398,17 @@ WinRegSetValueString(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsva
 PR_STATIC_CALLBACK(JSBool)
 WinRegGetValueString(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
 {
-  nsWinReg*    nativeThis = (nsWinReg*)JS_GetPrivate(cx, obj);
+  nsWinReg *nativeThis =
+    (nsWinReg*)JS_GetInstancePrivate(cx, obj, &WinRegClass, argv);
+  if (!nativeThis)
+    return JS_FALSE;
+
   nsString     nativeRet;
   nsAutoString b0;
   nsAutoString b1;
 
   *rval = JSVAL_NULL;
 
-  // If there's no private data, this must be the prototype, so ignore
-  if(nsnull == nativeThis)
-  {
-    return JS_TRUE;
-  }
-
   if(argc >= 2)                             
   {
     //  public string getValueString ( String subKey,
@@ -437,19 +436,17 @@ WinRegGetValueString(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsva
 PR_STATIC_CALLBACK(JSBool)
 WinRegEnumValueNames(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
 {
-  nsWinReg*      nativeThis = (nsWinReg*)JS_GetPrivate(cx, obj);
+  nsWinReg *nativeThis =
+    (nsWinReg*)JS_GetInstancePrivate(cx, obj, &WinRegClass, argv);
+  if (!nativeThis)
+    return JS_FALSE;
+
   nsAutoString   nativeRet;
   nsAutoString   b0;
   int32          b1;
 
   *rval = JSVAL_NULL;
 
-  // If there's no private data, this must be the prototype, so ignore
-  if(nsnull == nativeThis)
-  {
-    return JS_TRUE;
-  }
-
   if(argc >= 2)                             
   {
     //  public String enumValueNames ( String subkey,
@@ -484,19 +481,17 @@ WinRegEnumValueNames(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsva
 PR_STATIC_CALLBACK(JSBool)
 WinRegEnumKeys(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
 {
-  nsWinReg*      nativeThis = (nsWinReg*)JS_GetPrivate(cx, obj);
+  nsWinReg *nativeThis =
+    (nsWinReg*)JS_GetInstancePrivate(cx, obj, &WinRegClass, argv);
+  if (!nativeThis)
+    return JS_FALSE;
+
   nsAutoString   nativeRet;
   nsAutoString   b0;
   int32          b1;
 
   *rval = JSVAL_NULL;
 
-  // If there's no private data, this must be the prototype, so ignore
-  if(nsnull == nativeThis)
-  {
-    return JS_TRUE;
-  }
-
   if(argc >= 2)                             
   {
     //  public String enumKeys ( String subkey,
@@ -531,7 +526,11 @@ WinRegEnumKeys(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rva
 PR_STATIC_CALLBACK(JSBool)
 WinRegSetValueNumber(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
 {
-  nsWinReg *nativeThis = (nsWinReg*)JS_GetPrivate(cx, obj);
+  nsWinReg *nativeThis =
+    (nsWinReg*)JS_GetInstancePrivate(cx, obj, &WinRegClass, argv);
+  if (!nativeThis)
+    return JS_FALSE;
+
   PRInt32 nativeRet;
   nsAutoString b0;
   nsAutoString b1;
@@ -539,12 +538,6 @@ WinRegSetValueNumber(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsva
 
   *rval = INT_TO_JSVAL(nsInstall::UNEXPECTED_ERROR);
 
-  // If there's no private data, this must be the prototype, so ignore
-  if(nsnull == nativeThis)
-  {
-    return JS_TRUE;
-  }
-
   if(argc >= 3)
   {
     //  public int setValueNumber ( String subKey,
@@ -577,19 +570,17 @@ WinRegSetValueNumber(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsva
 PR_STATIC_CALLBACK(JSBool)
 WinRegGetValueNumber(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
 {
-  nsWinReg*    nativeThis = (nsWinReg*)JS_GetPrivate(cx, obj);
+  nsWinReg *nativeThis =
+    (nsWinReg*)JS_GetInstancePrivate(cx, obj, &WinRegClass, argv);
+  if (!nativeThis)
+    return JS_FALSE;
+
   PRInt32      nativeRet;
   nsAutoString b0;
   nsAutoString b1;
 
   *rval = JSVAL_NULL;
 
-  // If there's no private data, this must be the prototype, so ignore
-  if(nsnull == nativeThis)
-  {
-    return JS_TRUE;
-  }
-
   if(argc >= 2)                             
   {
     //  public int getValueNumber ( String subKey,
@@ -617,18 +608,16 @@ WinRegGetValueNumber(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsva
 PR_STATIC_CALLBACK(JSBool)
 WinRegSetValue(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
 {
-  nsWinReg *nativeThis = (nsWinReg*)JS_GetPrivate(cx, obj);
+  nsWinReg *nativeThis =
+    (nsWinReg*)JS_GetInstancePrivate(cx, obj, &WinRegClass, argv);
+  if (!nativeThis)
+    return JS_FALSE;
+
   nsAutoString b0;
   nsAutoString b1;
 
   *rval = INT_TO_JSVAL(nsInstall::UNEXPECTED_ERROR);
 
-  // If there's no private data, this must be the prototype, so ignore
-  if(nsnull == nativeThis)
-  {
-    return JS_TRUE;
-  }
-
   if(argc >= 3)
   {
     //  public int setValue ( String        subKey,
@@ -663,19 +652,17 @@ WinRegSetValue(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rva
 PR_STATIC_CALLBACK(JSBool)
 WinRegGetValue(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
 {
-  nsWinReg      *nativeThis = (nsWinReg*)JS_GetPrivate(cx, obj);
+  nsWinReg *nativeThis =
+    (nsWinReg*)JS_GetInstancePrivate(cx, obj, &WinRegClass, argv);
+  if (!nativeThis)
+    return JS_FALSE;
+
   nsWinRegValue *nativeRet;
   nsAutoString  b0;
   nsAutoString  b1;
 
   *rval = JSVAL_NULL;
 
-  // If there's no private data, this must be the prototype, so ignore
-  if(nsnull == nativeThis)
-  {
-    return JS_TRUE;
-  }
-
   if(argc >= 2)                             
   {
     //  public int getValue ( String subKey,
@@ -708,23 +695,6 @@ WinReg(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
 }
 
 
-/***********************************************************************/
-//
-// class for WinReg
-//
-JSClass WinRegClass = {
-  "WinReg",
-  JSCLASS_HAS_PRIVATE,
-  JS_PropertyStub,
-  JS_PropertyStub,
-  JS_PropertyStub,
-  JS_PropertyStub,
-  JS_EnumerateStub,
-  JS_ResolveStub,
-  JS_ConvertStub,
-  WinRegCleanup
-};
-
 static JSConstDoubleSpec winreg_constants[] = 
 {
     { nsWinReg::NS_HKEY_CLASSES_ROOT,        "HKEY_CLASSES_ROOT"            },
